#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

	Traitement des options de menus
	Point d'entrée depuis le menu principal		
	
'''
import os
import time

from PyQt5.QtGui import QCursor, QFont, QIcon
from PyQt5.QtCore import Qt, pyqtSlot, pyqtSignal
from PyQt5.QtWidgets import QDialog, QMessageBox, QFileDialog
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QApplication, QAction
				
import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
import lib.setting as AppSetting
from lib.util import zipUtil
from lib.htmlwindow import HtmlWindow
from lib.logviewver import LogViewver

from lib.project import Project
from lib.projectManagement import ProjectLoad, ProjectInfos
from lib.sqlitedb import Sqlitedb

from libapp.rapportexport import Rapportexport
from libapp.gedcomexport import Gedcomexport
from libapp.dataenter import Dataenter




class customApp() :
	
	log = None
	aideWindow  = False	
	aproposWindow = False	
	
	def __init__(self , parent ):
		self.parent = parent
	
	# Point d'entrée lors du lancement du programme
	def main( self ) :
				
		AppLog.log( LOG_INFO , "Appel du point d'entée principal (main())" )			
		self.onDataenter( )		
	
	def onRecentProjectMenuToShow( self, menuId ) :
		
		menuId.clear()
		recentProjectList = AppSetting.get('app/recentProjects', '')	
		for projectUrl in recentProjectList :
			recentAction = menuId.addAction( QIcon('img/menu_projicon.png'), os.path.basename(projectUrl))
			recentAction.setStatusTip(projectUrl)
			recentAction.setToolTip(projectUrl)			
			recentAction.triggered.connect( lambda check, pUrl = projectUrl : self.onRecentProjOpen( pUrl ) )	
			
		menuId.setToolTipsVisible(True)

	# Open project from filename (recents)
	def onRecentProjOpen( self, projectFileName ) :
		
		#print( "onRecentProjOpen , projectFileName=",projectFileName)
		self.loadProjectFile( projectFileName )				
		self.onDataenter( )
		
	# Open project from QFfileDialog
	def onProjOpen( self ) :	
				
		myProjectLoad = ProjectLoad()	# -> QFfileDialog	
		if myProjectLoad.projectFileName is not None :
			AppLog.log( LOG_INFO , "onProjOpen: ",  myProjectLoad.projectFileName )			
			self.loadProjectFile( myProjectLoad.projectFileName )				
			self.onDataenter(  )
				
	def loadProjectFile( self, projectFileName ) :
		
		self.parent.setWindowTitle( " " )
		
		AppLog.log( LOG_INFO , "loadProjectFile: ",  projectFileName )			
		
		if os.path.isfile(projectFileName) is False : 
			QMessageBox.critical(self.parent, 
						'Erreur', 
						"Le fichier projet n'existe pas:\n\n" + str(projectFileName),  
						QMessageBox.Cancel)
			return
		
		# Load Project
		currentProject = Project().load( projectFileName )	
		
		if os.path.isfile(currentProject.dbFile) is False :
			errorMessage = "Le ficher base de données n'existe pas : " + str(currentProject.dbFile )
			AppLog.log( LOG_ERROR , errorMessage )
			self.showCriticalMessage("Erreur", errorMessage)

		if os.path.isdir(currentProject.rootMedia) is False :
			errorMessage = "Le dossier des médias externes n'existe pas : " + str(currentProject.rootMedia )
			AppLog.log( LOG_ERROR , errorMessage )
			self.showCriticalMessage("Erreur", errorMessage)
		
		# Init main db handles	
		if os.access(currentProject.dbFile, os.W_OK) is True :	
			self.parent.db = Sqlitedb( currentProject.dbFile )		
		else :
			self.parent.db = None
			
		self.parent.rootMedia = currentProject.rootMedia		
		
		# Update window title
		#from main import myAppWindow		
		#if isinstance(self, myAppWindow):
		#	self.updateWindowTitle( projectFileName, isProjectname = True)		
		
		# Update app ini config
		AppSetting.set('app/lastProject',  projectFileName)			
		AppSetting.updatelist('app/recentProjects', projectFileName )		
		
		self.parent.setWindowTitle( projectFileName )
		
		return self.parent.db, self.parent.rootMedia

	def onEditDocumentDb( self ) :
		
		from modules.mediadbmanager.main import App #, iniParamFile # MyFilterModel, CustomDialog, PhotoViewer
		
		lastProjectFileName = AppSetting.get('app/lastProject', '')				
		currentProject = Project()					
		currentProject.load( lastProjectFileName )		
		
		docManagerObj = App( "modules/mediadbmanager/main.ini" , 
						currentProject.dbFile, 
						self.parent.rootMedia,
						"" 
						)
		docManagerObj.activeApp()
			
		lastProjectFileName = AppSetting.get('app/lastProject', '')					
		self.onRecentProjOpen( lastProjectFileName )
		
		print("END edit document")
					
	def onDataenter( self ) :
		
		QApplication.setOverrideCursor(Qt.WaitCursor)
		DataenterObj = Dataenter( self.parent, self.parent.db, self.parent.rootMedia )
		self.parent.signalAppCloseEvent.connect( DataenterObj.onCloseWidget )
		self.parent.setCentralWidget( DataenterObj )
		DataenterObj.main()		
		QApplication.restoreOverrideCursor()
				 
	def onProjNew( self ) :	
				
		myProjInfo = ProjectInfos()
		myProjInfo.newProjectData( )
		myProjInfo.exec()
				
		lastProjectFileName = AppSetting.get('app/lastProject', '')	
		AppLog.log( LOG_INFO , "open new lastProjectFileName : ", lastProjectFileName )			
		self.db = self.loadProjectFile( lastProjectFileName )		
		self.onDataenter( )
		self.parent.setWindowTitle( " " )
		
	def onProjInfo( self ):
		
		myProjInfo = ProjectInfos()
		self.currentProject = Project()					
		lastProjectFileName = AppSetting.get('app/lastProject', '')				
		self.currentProject.load( lastProjectFileName )
		myProjInfo.loadProjectData( lastProjectFileName, self.currentProject)		
		returnCode = myProjInfo.exec()
		
		if returnCode == QDialog.Accepted :				
			self.loadProjectFile( lastProjectFileName )				
			self.onDataenter( )
			
	def onRapportexportQuick ( self ) :
		self.RapportexportObj = Rapportexport( self.parent, self.parent.db )
		self.RapportexportObj.main( selectParams = False )
				
	def onRapportexport( self ) :
		self.RapportexportObj = Rapportexport(  self.parent, self.parent.db )
		self.RapportexportObj.main()								

	def onGedcomexport( self ) :
		GedcomexportObj = Gedcomexport( self.parent, self.parent.db )
		GedcomexportObj.main()

	def onAideOpen( self , dummySelf=None, dummyDb=None ) :	
				
		# Formulaire
		try :
			formId = QApplication.focusWidget().parentWidget().__class__.__name__ 
		except Exception as e:
			formId = ""		
		AppLog.log( LOG_INFO ,"F1 formId=", formId)

		# Focus sur le fond du formulaire ?
		try :
			formIdChild = QApplication.focusWidget().parentWidget().WidgetsForm.__class__.__name__ 
		except Exception as e:
			formIdChild=""			
		AppLog.log( LOG_INFO ,"F1 formIdChild=", formIdChild)
		if formIdChild != "" : formId = formIdChild			
		
		# Champ dans le formulaire		
		try :
			formFieldId = QApplication.focusWidget().objectName()
		except Exception as e:
			formFieldId = ""
		AppLog.log( LOG_INFO ,"F1 formFieldId=", formFieldId)
		
		AppLog.log( LOG_INFO ,"F1 context Help : htmlHelpWindow.load( 'index.html', '", formId, "', '", formFieldId, "' )")		
		
		# Si la fenêtre d'aide n'est pas déjà ouverte
		if self.aideWindow is False : 
			winId = "help"
			self.htmlHelpWindow = HtmlWindow()
			self.htmlHelpWindow.setType (winId, 'Aide' )			
			self.htmlHelpWindow.onWindowClosed.connect(self.onAideClosed)
			self.aideWindow = True

		self.htmlHelpWindow.load( 'index.html', formFieldId, formId )

	#@pyqtSlot( )		
	def onAideClosed( self ) :		
		AppLog.log( LOG_INFO , "Help window closed")		
		self.aideWindow = False
		
	def onContextHelpOpen( self ) :
		AppLog.log( LOG_INFO , "onContextHelpOpen")		
		self.onAideOpen( None, None )
				
	def onAproposOpen( self, dummyDb=None ) :		
		
		AppLog.log( LOG_INFO , "onAproposOpen")		
		if self.aproposWindow is False : 
			winId = "about"
			self.htmlAproposWindow = HtmlWindow()
			self.htmlAproposWindow.setType (winId, 'A propos' )			
			self.htmlAproposWindow.onWindowClosed.connect(self.onAproposClosed)
			self.aproposWindow = True

		appVersion = AppSetting.get('app/version', '')	
		appVersionNote = AppSetting.get('app/versionnote', '')	 
		
		import glob
		import os

		list_of_files = glob.glob('./**/*.py', recursive=True)
		latest_file = max(list_of_files, key=os.path.getctime)
		modTimesinceEpoc = os.path.getmtime(latest_file)
		# Convert seconds since epoch to readable timestamp
		modificationTime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(modTimesinceEpoc))			
			
		appDate = modificationTime # "31/010/2022 15:45"
		self.htmlAproposWindow.loadReplace( 'apropos.html', 
										metaReplace = { 
														"version": appVersion, 
														"versionnote": appVersionNote, 
														"appDate": appDate 
														} )				
	#@pyqtSlot( )		
	def onAproposClosed( self ) :		
		AppLog.log( LOG_INFO , "Apropos window closed")	
		self.aproposWindow = False
		
	def onArchiveZip( self ) :
		
		lastArchiveFileName = AppSetting.get('app/lastarchive', "temp/archive.zip")
		lastArchiveDir=""
		if lastArchiveFileName : 
			lastArchiveDir = os.path.dirname(lastArchiveFileName)
		
		timestr = time.strftime("%Y%m%d-%H%M%S")

		lastProjectFileName = os.path.splitext( os.path.basename( AppSetting.get('app/lastProject', '')))[0] 
		if lastProjectFileName == "" :
			lastProjectFileName = "basegenealogie"
		
		zipArchiveName =  os.path.join( lastArchiveDir, str(lastProjectFileName)+"-"+str(timestr)+".zip" )

		archiveFileName = QFileDialog.getSaveFileName(self.parent, 
												"Nom de l'archive", 
												zipArchiveName, 
												"Zip (*.zip);;Tous (*)")			
		if archiveFileName[0]:
			
			zipArchiveName = archiveFileName[0]
			
			lastProjectFileName = AppSetting.get('app/lastProject', '')				
			self.currentProject = Project()					
			self.currentProject.load( lastProjectFileName )
			
			# 'dossier/' : <url>
			zipList = { 
							'project/' : lastProjectFileName, 
							'data/' : self.currentProject.dbFile, 
							'medias/' : self.currentProject.rootMedia,
							'pageadd/' : os.path.dirname(self.currentProject.pageAdd)
						}
			
			AppLog.log( LOG_INFO , "Start zip database : ", zipArchiveName )
			AppLog.log( LOG_INFO , "add projectFile: ", lastProjectFileName )
			AppLog.log( LOG_INFO , "add dbFile: ", self.currentProject.dbFile )
			AppLog.log( LOG_INFO , "add dbMedia: ", self.currentProject.rootMedia  )
			AppLog.log( LOG_INFO , "add pageAdd: ", os.path.dirname(self.currentProject.pageAdd)  )
			
			try:
				
				QApplication.setOverrideCursor(Qt.WaitCursor)
				
				zipUtil.archive( self, zipArchiveName, zipList  )
				
				AppSetting.set('app/lastarchive', zipArchiveName)
				
				QApplication.restoreOverrideCursor()
				
			except Exception as e:
				
				QApplication.restoreOverrideCursor()
				AppLog.log( LOG_ERROR, str(e) )
				QMessageBox.critical(self.parent, 
									'Erreur', str(e), 
									QMessageBox.Cancel)
			
			AppLog.log( LOG_INFO , "End zip database" )
			
			QMessageBox.information(self.parent, 
								'Archive créée', 
								str(zipArchiveName), 
								QMessageBox.Ok)
			
 
	def onLogOpen( self) :
		
		self.logViewver = LogViewver()
		self.logViewver.exec()
		
		
	def showCriticalMessage( self, titleText, message ) :
		
		msg = QMessageBox()

		msg.setIcon(QMessageBox.Critical)
		
		msg.setText(message)
		#msg.setInformativeText('More information')
		msg.setWindowTitle(titleText)		

		msg.exec_()	
