#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''
	Ajouts dynamiques de d'options dans un menu de l'application
	(en dehors de main.py)
	
'''
from PyQt5.QtWidgets import QAction, QShortcut
from PyQt5.QtGui import QIcon, QKeySequence

from customapp.customApp import *

class MenuOptions( ) :

	menuOptions=[]
	
	def __init__(self , parent, customApp ):
		
		self.parent = parent # myApp		
		self.customApp = customApp

	# Point d'entrée depuis le menu principal		
	def AppendMenuOptions( self, menuId, menuIdLib, toolBarId, tooBarIdLib ) :		
		
		self.menuOptions = []

		if menuIdLib == "fichier" :
			
			toolBarId.setObjectName(tooBarIdLib)

			projnewAct = QAction(QIcon('img/menu_projnew.png'), 'Nouveau projet', self.parent)		
			projnewAct.setStatusTip('Créer un projet')
			projnewAct.triggered.connect( lambda: self.customApp.onProjNew( ) )		
			menuId.addAction(projnewAct)
			toolBarId.addAction( projnewAct )
			
			projopenAct = QAction(QIcon('img/menu_projopen.png'), 'Ouvrir un projet', self.parent)		
			projopenAct.setStatusTip('Ouvrir un projet existant')
			projopenAct.triggered.connect( lambda: self.customApp.onProjOpen( ) )		
			menuId.addAction(projopenAct)
			toolBarId.addAction( projopenAct )
				
			recentProjectMenu = menuId.addMenu( QIcon('img/menu_projrecent.png'), 'Projets récents ...')
			recentProjectMenu.aboutToShow.connect( lambda: self.customApp.onRecentProjectMenuToShow( recentProjectMenu ) )
					
			projinfoAct = QAction(QIcon('img/menu_projinfo.png'), 'Propriétés du projet', self.parent)		
			projinfoAct.setStatusTip('Afficher et éditer les propriétés du projets en cours')
			projinfoAct.triggered.connect( lambda: self.customApp.onProjInfo( ) )		
			menuId.addAction(projinfoAct)		
			toolBarId.addAction( projinfoAct )
						
			menuId.addSeparator()		
			
			dbArchiveAct = QAction(QIcon('img/menu_archivedb.png'), 'Archiver', self.parent)		
			dbArchiveAct.setStatusTip('Archiver les données dans un fichier Zip')
			dbArchiveAct.triggered.connect( lambda: self.customApp.onArchiveZip( ) )		
			menuId.addAction(dbArchiveAct)		
			toolBarId.addAction( dbArchiveAct )
						
			menuId.addSeparator()			
			
			rapportexportAct = QAction(QIcon('img/menu_rapportexport.png'), 'Generation de rapports', self.parent)
			rapportexportAct.setStatusTip('Exporter les données sous forme de rapport texte, html etc.')
			rapportexportAct.triggered.connect( lambda: self.customApp.onRapportexport( ) )  
			menuId.addAction( rapportexportAct )
			toolBarId.addAction( rapportexportAct )
			
			gedcomexportAct = QAction(QIcon('img/menu_gedcomexport.png'), 'Export gedcom', self.parent)
			gedcomexportAct.setStatusTip('Exporter les données au format Gedcom')
			gedcomexportAct.triggered.connect( lambda: self.customApp.onGedcomexport( ) ) 
			menuId.addAction( gedcomexportAct )	
			toolBarId.addAction( gedcomexportAct )

			menuId.addSeparator()

			exitAct = QAction(QIcon('img/menu_exit.png'), 'Quitter', self.parent)
			exitAct.setShortcut('Ctrl+Q')
			exitAct.setStatusTip('Fermer l''application')
			exitAct.triggered.connect(self.parent.close)
			
			menuId.addAction(exitAct)


		if menuIdLib == "edit" :
						
			toolBarId.setObjectName(tooBarIdLib)						
						
			editbaseAct = QAction(QIcon('img/menu_editbase.png'), 'Base généalogique', self.parent)
			editbaseAct.setStatusTip('Ajouter, editer les individus, évennements, sources ...')
			editbaseAct.triggered.connect( lambda: self.customApp.onDataenter() ) 			
			menuId.addAction( editbaseAct )
			toolBarId.addAction( editbaseAct )

			editDocDbAct = QAction(QIcon('img/menu_editbasedocument.png'), 'Base documents', self.parent)
			editDocDbAct.setStatusTip('Editer la base des documents ...')
			editDocDbAct.triggered.connect( lambda: self.customApp.onEditDocumentDb() ) 			
			menuId.addAction( editDocDbAct )
			toolBarId.addAction( editDocDbAct )
					
					
		if menuIdLib == "aide" :
			
			toolBarId.setObjectName(tooBarIdLib)

			aideAct = QAction(QIcon('img/menu_aide.png'), 'Aide', self.parent)
			aideAct.setStatusTip('Aide en ligne')
			aideAct.triggered.connect( lambda: self.customApp.onAideOpen( ) )  
			menuId.addAction( aideAct )
			toolBarId.addAction( aideAct )

			aproposAct = QAction(QIcon('img/menu_apropos.png'), 'A Propos ...', self.parent)
			aproposAct.setStatusTip('Informations de version')			
			aproposAct.triggered.connect( lambda: self.customApp.onAproposOpen( ) )  								
			menuId.addAction( aproposAct )
			toolBarId.addAction( aproposAct )

			menuId.addSeparator()
			logAct = QAction(QIcon('img/menu_logview.png'), 'Log', self.parent)
			logAct.setStatusTip('Visualiser le log')
			logAct.triggered.connect( lambda: self.customApp.onLogOpen( ) )  
			menuId.addAction( logAct )
			toolBarId.addAction( logAct )			

		# Ici, des entrées sont générées automatiquement: une entrée par table.
		if menuIdLib == "table" :
			toolBarId.setObjectName(tooBarIdLib)
			pass
			
			
		return self.menuOptions
	
