#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""

Usage :

from lib.logmanager import LogManager,LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

log = LogManager("main.log.txt")
log.logMessage( LOG_INFO , 'app start ...')

"""

import sys
import datetime
import locale
import os
import os.path
import inspect

from collections import deque

import lib.setting as AppSetting

Log = None

LOG_ARGVAL = 0
LOG_INFO = 1
LOG_INFOSETTING = 2
LOG_WARNING = 3
LOG_ERROR = 4
LOG_DEBUG = 5
LOG_DATA = 6

LOG_FATAL = 9


class LogManager() :

	trace_LOG_ARGVAL = True
	trace_LOG_INFO = True
	trace_LOG_INFOSETTING = True
	trace_LOG_WARNING = True
	trace_LOG_ERROR = True
	trace_LOG_DEBUG = True
	trace_LOG_DATA = True

	outToconsole = True

	def __init__(self, filelog, eraseContent = False, logConsole="1", maxLevelStr="LOG_DATA" ) :

		pathlog = filelog
		pathlog = pathlog.replace("\\", os.sep)

		self.filelog = pathlog
		self.filelogCopyName = None
		originalStdout = sys.stdout
		
		self.maxLevel = self.getLevelValue(maxLevelStr)
		
		self.outToconsole = AppSetting.get('log/logToConsole', "0")		
		if self.outToconsole == "1" : self.outToconsole = True
		else: self.outToconsole = False
		self.logMaxLine = AppSetting.get('log/logMaxLine', 0)		
		
		self.trace_LOG_DATA = AppSetting.get('log/logTraceData', "0")
		if self.trace_LOG_DATA == "1" : self.trace_LOG_DATA = True
		else: self.trace_LOG_DATA = False
		
		self.trace_LOG_DEBUG = AppSetting.get('log/logTraceDebug', "0")
		if self.trace_LOG_DEBUG == "1" : self.trace_LOG_DEBUG = True
		else: self.trace_LOG_DEBUG = False
		self.trace_LOG_ERROR = AppSetting.get('log/logTraceError', "0")
		if self.trace_LOG_ERROR == "1" : self.trace_LOG_ERROR = True
		else: self.trace_LOG_ERROR = False			
		self.trace_LOG_WARNING = AppSetting.get('log/logTraceWarning', "0")
		if self.trace_LOG_WARNING == "1" : self.trace_LOG_WARNING = True
		else: self.trace_LOG_WARNING = False	
		self.trace_LOG_INFO = AppSetting.get('log/logTraceInfo', "0")
		if self.trace_LOG_INFO == "1" : self.trace_LOG_INFO = True
		else: self.trace_LOG_INFO = False
					
		self.logMessage( LOG_INFO, "Log outToconsole = ", str(self.outToconsole))			
		self.logMessage( LOG_INFO, "Log logMaxLine = ", str(self.logMaxLine))
		self.logMessage( LOG_INFO, "Log trace_LOG_DATA = ", str(self.trace_LOG_DATA))
		self.logMessage( LOG_INFO, "Log trace_LOG_ERROR = ", str(self.trace_LOG_ERROR))
		self.logMessage( LOG_INFO, "Log trace_LOG_WARNING = ", str(self.trace_LOG_WARNING))
		self.logMessage( LOG_INFO, "Log trace_LOG_DEBUG = ", str(self.trace_LOG_DEBUG))
		self.logMessage( LOG_INFO, "Log trace_LOG_INFO = ", str(self.trace_LOG_INFO))
		
		try:
			callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
			frame = callerframerecord[0]
			context = inspect.getframeinfo(frame)

			if eraseContent is True :
				sys.stdout = open(self.filelog, 'w')
				self.outLineMessage( LOG_INFO, 'init log file (erase): ' + self.filelog , context )
			else :
				sys.stdout = open(self.filelog, 'a')
				#self.outLineMessage( LOG_INFO, 'init log file (append) : ' + self.filelog , context )

			sys.stdout = originalStdout

		except IOError:
			sys.exit( 'Unable to write to file ' + filelog )

		#self.logMessage( LOG_INFO, "self.filelog = " + self.filelog  )

	def getLevelValue( self, strLevel) :

		if  strLevel == "LOG_ARGVAL" :
			return 0
		elif strLevel == "LOG_INFO" :
			return 1
		elif  strLevel == "LOG_INFOSETTING" :
			return 2
		elif  strLevel == "LOG_WARNING" :
			return 3
		elif  strLevel == "LOG_ERROR" :
			return 4
		elif  strLevel == "LOG_DEBUG" :
			return 5
		elif  strLevel == "LOG_DATA" :
			return 6
		elif  strLevel == "LOG_FATAL" :
			return 9

		return 0


	def toggleTraceLevel( self, strLevel, toggle ) :

		if  strLevel == "LOG_ARGVAL" :
			self.trace_LOG_ARGVAL = toggle
		elif strLevel == "LOG_INFO" :
			self.trace_LOG_INFO = toggle
		elif  strLevel == "LOG_INFOSETTING" :
			self.trace_LOG_INFOSETTING = toggle
		elif  strLevel == "LOG_WARNING" :
			self.trace_LOG_WARNING = toggle
		elif  strLevel == "LOG_ERROR" :
			self.trace_LOG_ERROR = toggle
		elif  strLevel == "LOG_DEBUG" :
			self.trace_LOG_DEBUG = toggle
		elif  strLevel == "LOG_DATA" :
			self.trace_LOG_DATA = toggle

		return 0

	def toggleTraceToConsole( self, toggle ) :
		self.outToconsole = toggle

	def logMessage( self, level, *args ) :

		if self.trace_LOG_DATA is False and level == LOG_DATA : return
		if self.trace_LOG_INFO is False and level == LOG_INFO : return
		if self.trace_LOG_WARNING is False and level ==  LOG_WARNING : return
		if self.trace_LOG_ERROR is False and level == LOG_ERROR : return
		if self.trace_LOG_DEBUG is False and level == LOG_DEBUG : return
		if self.trace_LOG_INFOSETTING is False and level == LOG_INFOSETTING : return
		
		argN = 0
		message = ""
		for messageElem in args:
			message = message + str(messageElem)

		if int(level) > self.maxLevel and int(level) != LOG_FATAL :
			#print(message)
			return

		originalStdout = sys.stdout

		sys.stdout = open(self.filelog, 'a+')

		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		context = inspect.getframeinfo(frame)

		self.outLineMessage( level, message, context )

		sys.stdout = originalStdout

		if self.outToconsole is True :
			self.outLineMessage( level, message, context )

		if  level == LOG_FATAL :
			msg = QMessageBox()
			msg.setIcon(QMessageBox.Critical)
			msg.setText(message)
			msg.setInformativeText( os.path.basename(context.filename) + ':' + context.function + ':' + str(context.lineno) )
			msg.setWindowTitle("Erreur")
			msg.exec_()
			sys.exit(-1)

		self.logClean()

	def outLineMessage( self, level, message , context) :

		now = datetime.datetime.now()

		levelStr = ""
		if level == LOG_INFO :
			levelStr = "_INFO"
		elif  level == LOG_ARGVAL :
			levelStr = "_ARG"
		elif  level == LOG_DEBUG :
			levelStr = "_DEBUG"
		elif  level == LOG_DATA :
			levelStr = "_DATA"
		elif  level == LOG_FATAL :
			levelStr = "_FATAL"
		elif  level == LOG_WARNING :
			levelStr = "_WARNING"
		elif  level == LOG_ERROR :
			levelStr = "_ERREUR"
		elif  level == LOG_INFOSETTING :
			levelStr = "LOG_INFOSETTING"

		print(now.strftime("%d-%m-%Y %H:%M:%S.%f") + ';' + levelStr + ';' + os.path.basename(context.filename) + ':' + context.function + ':' + str(context.lineno) + ';' + message)


	def copy( self) :

		import time
		import os
		import shutil

		datetime_var = time.strftime("%Y_%m_%d_%H_%M_%S")

		src_dir = os.path.dirname(self.filelog)
		base=os.path.basename(self.filelog)
		src_base = os.path.splitext(base)[0]

		dst_file = src_dir + os.sep + src_base + "_" + str(datetime_var) + ".txt"

		file_path = dst_file.split(os.path.basename(dst_file))[0]
		if not os.path.exists(file_path):
			os.makedirs(file_path)

		dst_file = dst_file.replace('\\', os.sep)
		shutil.copy2(self.filelog, dst_file)

		self.filelogCopyName = dst_file


	def getFilelogCopyName( self) :
		return self.filelogCopyName

	def logClean( self ) :


		with open(self.filelog) as fin, open(self.filelog + "tmp", 'w') as fout:
			fout.writelines(deque(fin, int(self.logMaxLine)))
		os.replace( self.filelog + "tmp", self.filelog )
    
