#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''

		{{title}}

        Aide

        {{datetime}}

        {{author}}

        {{licence}}

'''
import re

from pathlib import Path

from PyQt5.QtWidgets import QApplication, QFileDialog, QLabel, QVBoxLayout, QDialog, QWidget
from PyQt5.QtCore import Qt, QUrl, QSettings, pyqtSignal, pyqtSlot
from PyQt5.QtWebEngineWidgets import QWebEngineView

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

from lib.util import Util
import lib.setting as AppSetting

from libdb.event_source import Event_source, Event_sourcesDb

class HtmlWindow(QDialog):

	onWindowClosed = pyqtSignal( )

	def __init__(self, *args, **kwargs):
		super(HtmlWindow, self).__init__(*args, **kwargs)

		self.initUI()

	def initUI(self):

		vbox = QVBoxLayout(self)

		self.webEngineView = QWebEngineView()
		vbox.addWidget(self.webEngineView)
		self.setLayout(vbox)

		self.setGeometry(300, 300, 600, 400)

		self.show()

	def setType (self, winId, windowTitle ) :

		self. resizeWin( winId )
		self.windowTitle = windowTitle
		self.setWindowTitle(self.windowTitle)

	def load(self, htmlPage, contextId=None, contextParentId=None ):

		metaReplaceTab = {
							"contextId": contextId,
							"contextParentId": contextParentId
						}

		try :
			fname = "aide/" + self.winId + "/" + contextId + ".html"
			fname = str(Path(fname).absolute())
			metaReplaceTab["fname"] = fname 
			f = open( fname, 'r')
			htmlData = f.read()
			htmlData = self.replaceHtmlMeta( htmlData, metaReplaceTab )			
			self.webEngineView.load(QUrl.fromLocalFile(fname))
			AppLog.log( LOG_INFO, "Load : ", fname )
			#self.webEngineView.setHtml(htmlData)
			#self.setWindowTitle(self.windowTitle + "[" + contextId + "]")
		except Exception as e:
			try :
				fname = "aide/" + self.winId + "/" + contextParentId + ".html"
				fname = str(Path(fname).absolute())
				metaReplaceTab["fname"] = fname 
				f= open( fname , 'r')
				htmlData = f.read()
				htmlData = self.replaceHtmlMeta( htmlData, metaReplaceTab )
				#self.webEngineView.setHtml(htmlData)
				#self.setWindowTitle(self.windowTitle + "[" + contextParentId + "]")
				self.webEngineView.load(QUrl.fromLocalFile(fname))
				AppLog.log( LOG_INFO, "Load : ", fname )
			except Exception as e:
				fname = "aide/index.html"
				fname = str(Path(fname).absolute())
				metaReplaceTab["fname"] = fname 
				f = open( fname, 'r')
				htmlData = f.read()
				htmlData = self.replaceHtmlMeta( htmlData, metaReplaceTab )
				#self.webEngineView.setHtml(htmlData)
				#self.setWindowTitle(self.windowTitle + "[" + contextParentId + "]")
				self.webEngineView.load(QUrl.fromLocalFile(fname))
				AppLog.log( LOG_INFO, "Load : ", fname )

	def replaceHtmlMeta( self, htmlData, metaReplace ) :
		for metaKey, metaVal in metaReplace.items() :
			htmlData = htmlData.replace( "{{" + metaKey + "}}" , metaVal)
		htmlData = re.sub('{{.*}}', '', htmlData)
		return htmlData

	def loadReplace( self, htmlPage, metaReplace ) :
			with open( "aide/" + htmlPage, 'r') as f:
				html = f.read()

				if metaReplace is not None :
					for metaKey, metaVal in metaReplace.items() :

						html = html.replace( "{{" + metaKey + "}}" , metaVal)

				html = re.sub('{{.*}}', '', html)

				self.webEngineView.setHtml(html)
				self.setWindowTitle(self.windowTitle )

	def resizeWin( self, winId ) :

		self.winId = winId

		try :
			settings=QSettings("config/app.ini", QSettings.IniFormat)
			WinPos = settings.value("app/"+self.winId+"_winpos")
			self.move(WinPos);
			winSize = settings.value("app/"+self.winId+"_winsize")
			self.resize(winSize)

		except Exception as e:
			AppLog.log( LOG_ERROR , str(e) )

	def closeEvent(self, event):

		settings=QSettings("config/app.ini", QSettings.IniFormat)
		settings.setValue("app/"+self.winId+"_winpos", self.pos())
		settings.setValue("app/"+self.winId+"_winsize", self.size())

		self.onWindowClosed.emit( )

	def resizeWindow( self, x, y, width, height ) :
		self.setGeometry(x, y, width, height )

	def resizeEvent( self, event) :
		# print("resizeEvent ", self.size())
		pass

	def moveEvent( self, event) :
		# print("moveEvent ", self.pos())
		pass

