#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
	Usage :
	
	import lib.log as AppLog
	from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL,LOG_DATA

	AppLog.log( LOG_INFO , 'message')
	

"""
import sys
import datetime
import locale
import os
import os.path
import inspect

import lib.setting as AppSetting

LOG_ARGVAL = 0
LOG_INFO = 1
LOG_INFOSETTING = 2
LOG_WARNING = 3
LOG_ERROR = 4
LOG_DEBUG = 5
LOG_DATA = 6

LOG_FATAL = 9

logFile = None
logFileFp = None
logContext = None

logToConsole = True

trace_LOG_ARGVAL = True
trace_LOG_INFO = True
trace_LOG_INFOSETTING = True
trace_LOG_WARNING = True
trace_LOG_ERROR = True
trace_LOG_DEBUG = True
trace_LOG_DATA = True

logFlushCountMax = 0
logFlushCount = 0

def init( logfile, eraseContent = False ):
	
	global logFile, logFileFp, logToConsole,logMaxLine,trace_LOG_ARGVAL,trace_LOG_INFO,trace_LOG_INFOSETTING,trace_LOG_WARNING,trace_LOG_ERROR,trace_LOG_DEBUG,trace_LOG_DATA
		
	logFile = logfile	
	
	if eraseContent is True :
		logFileFp = open(logFile, 'w')
		log(LOG_INFO, "Erase and open logFile: ", logFile)
	else :
		logFileFp = open(logFile, 'a')
		log(LOG_INFO, "Open logFile: ", logFile)
    
	logToConsole = AppSetting.get('log/logToConsole', "0")		
	if logToConsole == "1" : logToConsole = True
	else: logToConsole = False
	
	logMaxLine = AppSetting.get('log/logMaxLine', 10)		
	
	trace_LOG_DATA = AppSetting.get('log/logTraceData', "0")
	if trace_LOG_DATA == "1" : trace_LOG_DATA = True
	else: trace_LOG_DATA = False
	
	trace_LOG_DEBUG = AppSetting.get('log/logTraceDebug', "0")
	if trace_LOG_DEBUG == "1" : trace_LOG_DEBUG = True
	else: trace_LOG_DEBUG = False
	trace_LOG_ERROR = AppSetting.get('log/logTraceError', "0")
	if trace_LOG_ERROR == "1" : trace_LOG_ERROR = True
	else: trace_LOG_ERROR = False			
	trace_LOG_WARNING = AppSetting.get('log/logTraceWarning', "0")
	if trace_LOG_WARNING == "1" : trace_LOG_WARNING = True
	else: trace_LOG_WARNING = False	
	trace_LOG_INFO = AppSetting.get('log/logTraceInfo', "0")
	if trace_LOG_INFO == "1" : trace_LOG_INFO = True
	else: trace_LOG_INFO = False
				
	log( LOG_INFO, "Log outToconsole = ", str(logToConsole))			
	log( LOG_INFO, "Log logMaxLine = ", str(logMaxLine))
	log( LOG_INFO, "Log trace_LOG_DATA = ", str(trace_LOG_DATA))
	log( LOG_INFO, "Log trace_LOG_ERROR = ", str(trace_LOG_ERROR))
	log( LOG_INFO, "Log trace_LOG_WARNING = ", str(trace_LOG_WARNING))
	log( LOG_INFO, "Log trace_LOG_DEBUG = ", str(trace_LOG_DEBUG))
	log( LOG_INFO, "Log trace_LOG_INFO = ", str(trace_LOG_INFO))
	    
def log( level, *args ) :
	
	global logFile, logFileFp, logToConsole, logContext, logFlushCountMax, logFlushCount 

	# Full message
	message = ""
	for messageElem in args:
		message = message + str(messageElem)
	
	# Guess context
	if logContext is None : 
		try:
			callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
			frame = callerframerecord[0]
			context = inspect.getframeinfo(frame)
		except Exception as e:
			print("log error : ", message)
			print("log error : ", str(e))
	else :
		context = logContext
		
	# DateTime	
	now = datetime.datetime.now()

	# Error level
	levelStr = ""
	if level == LOG_INFO :
		levelStr = "_INFO"
	elif  level == LOG_ARGVAL :
		levelStr = "_ARG"
	elif  level == LOG_DEBUG :
		levelStr = "_DEBUG"
	elif  level == LOG_DATA :
		levelStr = "_DATA"
	elif  level == LOG_FATAL :
		levelStr = "_FATAL"
	elif  level == LOG_WARNING :
		levelStr = "_WARNING"
	elif  level == LOG_ERROR :
		levelStr = "_ERREUR"
	elif  level == LOG_INFOSETTING :
		levelStr = "LOG_INFOSETTING"				
		
	# Full log line
	lineLog = now.strftime("%d-%m-%Y %H:%M:%S.%f") + ';' + levelStr + ';' + os.path.basename(context.filename) + ':' + context.function + ':' + str(context.lineno) + ';' + message
	
	# Print message ...
	if logToConsole is True :
		print( lineLog )
	
	if logFileFp is not None :
		logFileFp.write(lineLog + "\n")
		logFlushCount = logFlushCount + 1
		if logFlushCountMax > 0 and logFlushCount > logFlushCountMax :
			logFlush()
			logFlushCount = 0
			
	else :
		print("ERR : logFileFp is None.")
		
	logContext = None

def logFlush() :
	
	global logFileFp
	
	logFileFp.flush()
	os.fsync(logFileFp)	

def toggleTraceLevel( strLevel, toggle ) :

	global trace_LOG_ARGVAL, trace_LOG_INFO,trace_LOG_INFOSETTING,trace_LOG_WARNING,trace_LOG_ERROR,trace_LOG_DEBUG,trace_LOG_DATA
	
	if  strLevel == "LOG_ARGVAL" :
		trace_LOG_ARGVAL = toggle
	elif strLevel == "LOG_INFO" :
		trace_LOG_INFO = toggle
	elif  strLevel == "LOG_INFOSETTING" :
		trace_LOG_INFOSETTING = toggle
	elif  strLevel == "LOG_WARNING" :
		trace_LOG_WARNING = toggle
	elif  strLevel == "LOG_ERROR" :
		trace_LOG_ERROR = toggle
	elif  strLevel == "LOG_DEBUG" :
		trace_LOG_DEBUG = toggle
	elif  strLevel == "LOG_DATA" :
		trace_LOG_DATA = toggle

	return 0

def toggleTraceToConsole( toggle ) :
	
	global logToConsole
	logToConsole = toggle

def copy() :

	import time
	import os
	import shutil

	datetime_var = time.strftime("%Y_%m_%d_%H_%M_%S")

	src_dir = os.path.dirname(self.filelog)
	base=os.path.basename(self.filelog)
	src_base = os.path.splitext(base)[0]

	dst_file = src_dir + os.sep + src_base + "_" + str(datetime_var) + ".txt"

	file_path = dst_file.split(os.path.basename(dst_file))[0]
	if not os.path.exists(file_path):
		os.makedirs(file_path)

	dst_file = dst_file.replace('\\', os.sep)
	shutil.copy2(self.filelog, dst_file)

	self.filelogCopyName = dst_file


def getFilelogCopyName( ) :
	return self.filelogCopyName

def logClean() :

	global logFile,logMaxLine
	
	with open(logFile) as fin, open(logFile + "tmp", 'w') as fout:
		fout.writelines(deque(fin, int(logMaxLine)))
	os.replace( logFile + "tmp", logFile )

def logErase() :
	
	global logFile
	
	try:
		
		os.remove(logFile)
		log( LOG_INFO, "Log effacé : ", str(logFile))	
		
	except Exception as e:	
		log( LOG_ERROR, "LogErase() error : ", str(e) )	
		print("LogErase() error : ", str(e))
