#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''

		{{title}}

		Aide

		{{datetime}}

		{{author}}

		{{licence}}

'''
import re, sys, shutil, io, csv

from PyQt5 import QtWidgets, QtCore, QtGui
from PyQt5.QtWidgets import QApplication, QFileDialog, QLabel, QVBoxLayout, QDialog, QWidget, QPushButton, QMessageBox
from PyQt5.QtWidgets import QLineEdit, QTableView, QDialog, QFileDialog, QListWidget, QGridLayout, QHBoxLayout, QDialogButtonBox, QListWidgetItem, QCheckBox
from PyQt5.QtCore import Qt, QSortFilterProxyModel, QSettings, pyqtSignal, pyqtSlot, QModelIndex
from PyQt5.QtGui import QCursor, QFont, QIcon, QColor, QPalette, QKeySequence

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
	
from lib.util import Util
import lib.setting as AppSetting

from libdb.event_source import Event_source, Event_sourcesDb

class LogViewver(QDialog):

	logFilterInfo = True
	logFilterWarning = True
	logFilterError = True
	logFilterData = False
	logFilterDebug = False

	log = None

	def __init__(self):

		QDialog.__init__(self)

		AppLog.log( LOG_INFO , "Logviewver init" )

		self.initUi()

		self.logFileName = AppSetting.get('app/logfile', "defaultlog.txt")
		self.setWindowTitle("Log : " + str(self.logFileName))

		self.refreshData()

	def initUi (self):

		layout = QGridLayout()
		self.setLayout(layout)

		self.filterQline = QLineEdit()
		self.filterQline.textChanged.connect(self.onFilterQlineChanged)

		layout.addWidget(self.filterQline,0,0)

		self.logTableView = QTableView()

		self.logTableView.setSortingEnabled(True)
		self.logTableView.sortByColumn(0, QtCore.Qt.AscendingOrder)
		self.logTableView.horizontalHeader().setSortIndicator(0, Qt.AscendingOrder)

		self.logTableView.horizontalHeader().setSectionsMovable(True)
		layout.addWidget(self.logTableView,1,0)

		boxButtonsLayout = QHBoxLayout()

		self.logSaveAsBtn = QPushButton("Enregistrer sous...", self)
		self.logSaveAsBtn.setToolTip( "Enregistrer le log sous ..." )
		self.logSaveAsBtn.setIcon(QIcon('img/log_saveas.png'))
		self.logSaveAsBtn.clicked.connect(self.onLogAsaveAs)
		boxButtonsLayout.addWidget(self.logSaveAsBtn, Qt.AlignLeft)

		self.logEraseBtn = QPushButton("Effacer", self)
		self.logEraseBtn.setToolTip( "Effacer le ficher de log" )
		self.logEraseBtn.setIcon(QIcon('img/log_erase.png'))
		self.logEraseBtn.clicked.connect(self.onLogErase)
		boxButtonsLayout.addWidget(self.logEraseBtn, Qt.AlignLeft)

		self.logFlushBtn = QPushButton("Purger le cache", self)
		self.logFlushBtn.setToolTip( "Forcer l'écriture des données du cache dans le ficher de log" )
		self.logFlushBtn.setIcon(QIcon('img/log_flush.png'))
		self.logFlushBtn.clicked.connect(self.onLogFlush)
		boxButtonsLayout.addWidget(self.logFlushBtn, Qt.AlignLeft)

		self.loginfoChk = QCheckBox("Info")
		self.loginfoChk.setChecked( self.logFilterInfo)
		self.loginfoChk.toggled.connect(self.onLoginfoChk)
		boxButtonsLayout.addWidget(self.loginfoChk, Qt.AlignRight)
		self.loginfoChk.setEnabled (AppLog.trace_LOG_INFO)
		
		self.logwarnChk = QCheckBox("Warning")
		self.logwarnChk.setChecked( self.logFilterWarning)
		self.logwarnChk.toggled.connect(self.onLogwarnChk)
		boxButtonsLayout.addWidget(self.logwarnChk, Qt.AlignRight)
		self.logwarnChk.setEnabled (AppLog.trace_LOG_WARNING)

		self.logerrChk = QCheckBox("Erreur")
		self.logerrChk.setChecked( self.logFilterError)
		self.logerrChk.toggled.connect(self.onLogerrChk)
		boxButtonsLayout.addWidget(self.logerrChk, Qt.AlignRight)
		self.logerrChk.setEnabled (AppLog.trace_LOG_ERROR)

		self.logdataChk = QCheckBox("Data")
		self.logdataChk.setChecked( self.logFilterData)
		self.logdataChk.toggled.connect(self.onLogdataChk)
		boxButtonsLayout.addWidget(self.logdataChk, Qt.AlignRight)
		self.logdataChk.setEnabled (AppLog.trace_LOG_DATA)

		self.logdebugChk = QCheckBox("Debug")
		self.logdebugChk.setChecked( self.logFilterDebug)
		self.logdebugChk.toggled.connect(self.onLogdebugChk)
		boxButtonsLayout.addWidget(self.logdebugChk, Qt.AlignRight)
		self.logdebugChk.setEnabled (AppLog.trace_LOG_DEBUG)

		# OK and Cancel buttons
		buttons = QDialogButtonBox(
			QDialogButtonBox.Ok,
			Qt.Horizontal, self)
		buttons.accepted.connect(self.onAccept)

		boxButtonsLayout.addWidget(buttons,Qt.AlignRight)

		layout.addLayout( boxButtonsLayout,2,0 )

		self.resize(1200, 600)

	def keyPressEvent(self, event):
			
		if event.matches(QKeySequence.Copy):
			self.copySelection()
			        
	def copySelection(self):
		selection = self.logTableView.selectedIndexes()
		if selection:
			rows = sorted(index.row() for index in selection)
			columns = sorted(index.column() for index in selection)
			rowcount = rows[-1] - rows[0] + 1
			colcount = columns[-1] - columns[0] + 1
			table = [[''] * colcount for _ in range(rowcount)]
			for index in selection:
				row = index.row() - rows[0]
				column = index.column() - columns[0]
				table[row][column] = index.data()
			stream = io.StringIO()
			csv.writer(stream, delimiter='\t').writerows(table)
			clipboard = QApplication.clipboard()            
			clipboard.setText(stream.getvalue())   

	def onLoginfoChk( self, state) :
		self.logFilterInfo = state
		self.refreshData()

	def onLogwarnChk( self, state) :
		self.logFilterWarning = state
		self.refreshData()

	def onLogerrChk( self, state) :
		self.logFilterError = state
		self.refreshData()

	def onLogdataChk( self, state) :
		self.logFilterData = state
		self.refreshData()

	def onLogdebugChk( self, state) :
		self.logFilterDebug = state
		self.refreshData()

	def refreshData( self ) :
		QApplication.setOverrideCursor(Qt.WaitCursor)
		self.loadLogData()
		self.displayLogData()
		self.logTableView.verticalScrollBar().setSliderPosition (self.logTableView.verticalScrollBar().maximum())
		QApplication.restoreOverrideCursor()

	def loadLogData( self ) :

		self.logLines = []

		AppLog.logFlush()

		with open(self.logFileName) as fp:
			line = fp.readline()
			cnt = 1
			while line:
				#print("Line {}: {}".format(cnt, line.strip()))

				if len(line) > 1 :
					line = line[0:len(line)-1]

				lineElem = line.split(";")

				addLine = True
				try :
					if self.logFilterInfo is False and lineElem[1] == "_INFO" : addLine = False
					if self.logFilterWarning is False and lineElem[1] == "_WARNING" : addLine = False
					if self.logFilterError is False and lineElem[1] == "_ERREUR" : addLine = False
					if self.logFilterData is False and lineElem[1] == "_DATA" : addLine = False
					if self.logFilterDebug is False and lineElem[1] == "_DEBUG" : addLine = False
				except Exception as e:			
					print("lineElem=", lineElem)	
					print("Log error : ", str(e))

				if addLine is True :
				   self.logLines.append( lineElem )
				   #print(lineElem)

				line = fp.readline()
				cnt += 1

	def displayLogData( self) :

		self.logModel = LogTableModel(self.logLines)

		self.logProxyModel = SortFilterProxyModel()

		self.logProxyModel.setFilterCaseSensitivity(Qt.CaseInsensitive)
		self.logProxyModel.setSourceModel(self.logModel)
		self.logTableView.setModel(self.logProxyModel)

		self.logTableView.horizontalHeader().setStretchLastSection(True)
		self.logTableView.resizeColumnsToContents()

	def onLogFlush(self) :
		AppLog.logFlush()
		self.refreshData()

	def onLogErase( self) :
		msg = QMessageBox()
		msg.setIcon(QMessageBox.Information)

		msg.setText("Effacer le log ?")
		#msg.setInformativeText("This is additional information")
		msg.setWindowTitle("Confirmez")
		#msg.setDetailedText("The details are as follows:")
		msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
		#msg.buttonClicked.connect(msgbtn)

		retval = msg.exec_()
		if retval == QMessageBox.Ok :
			AppLog.logErase()
			logfile = AppSetting.get('app/logfile', "defaultlog.txt")
			AppLog.init( logfile, True )						
			AppLog.log( LOG_INFO , "Logviewver init (erased)" )			
			self.refreshData()

	def onFilterQlineChanged( self, text ) :
		self.logProxyModel.setFilterValue( text )

	# Cross close
	def closeEvent ( self, event) :
		self.reject()

	def onAccept( self) :
		self.accept()

	def onLogAsaveAs( self) :
		
		fileName = QtWidgets.QFileDialog.getSaveFileName(self, "Sauver le log sous...", './log.txt', 'Texte (*.txt)|Tous (*.*)')[0]
		
		if fileName :
			try :
				shutil.copyfile ( self.logFileName ,  str(fileName) )
				AppLog.log( LOG_INFO , 'Log copié: ' +  str(fileName) )
			except Exception as e:
				msg = QMessageBox()
				msg.setIcon(QMessageBox.Error)
				msg.setText("Erreur lors de la copie du fichier de log")
				msg.setInformativeText(str(e))
				msg.setWindowTitle("Erreur")
				msg.setDetailedText("Copie: ",self.logFileName, " ---> ", str(fileName))
				msg.setStandardButtons(QMessageBox.Ok )
				retval = msg.exec_()


class SortFilterProxyModel(QSortFilterProxyModel):

	def __init__(self, *args, **kwargs):
		QSortFilterProxyModel.__init__(self, *args, **kwargs)
		self.filters = {}
		self.filterValue = None

	def setFilterByColumn(self, text, column):
		self.filters[column] = text
		self.invalidateFilter()

	def setFilterValue( self, filterValue) :
		#print("filterValue=", str(filterValue))
		self.filterValue = filterValue
		self.invalidateFilter()

	def filterAcceptsRow(self, source_row, source_parent):

		if self.filterValue is None :
			return True

		#print("call filterAcceptsRow")

		model = self.sourceModel()

		for columnkey in range (0,model.columnCount(self)) :

			ix = model.index(source_row, columnkey, source_parent)
			if ix.isValid():

				text = model.data(ix, Qt.UserRole)
				#print("text ", str(text))
				
				if text is None or str(self.filterValue).lower() in text.lower() :
					return True

		return False

class LogTableModel(QtCore.QAbstractTableModel):

	def __init__(self, data=[[]], parent=None):
		super().__init__(parent)
		self.datas = data

	def headerData(self, section: int, orientation: Qt.Orientation, role: int):
		if role == QtCore.Qt.DisplayRole:
			if orientation == Qt.Horizontal:
				if section == 0 :
					return ("DateHeure")
				if section == 1 :
					return ("Niveau")
				if section == 2 :
					return ("Source")
				if section == 3 :
					return ("Message")
			else:
				return str(section+1)

	def columnCount(self, parent=None):
		try :
			return len(self.datas[0])
		except Exception as e:
			return 0

	def rowCount(self, parent=None):
		try :
			return len(self.datas)
		except Exception as e:
			return 0

	def data(self, index: QModelIndex, role: int):
		row = index.row()
		col = index.column()

		if role == Qt.DisplayRole:
			#print("data : ", row, ",", col," -->", str(self.data[row][col]))
			return str(self.datas[row][col])

		if role == Qt.UserRole: # Raw
			return str(self.datas[row][col])


		if role == Qt.ToolTipRole:
			return str(self.datas[row][col])

		if role == Qt.BackgroundRole :
			if col == 1 :
				if str(self.datas[row][col]) == "_INFO" :
					return QtGui.QBrush(QColor(0, 153, 0) )
				if str(self.datas[row][col]) == "_WARNING" :
					return QtGui.QBrush( QColor(255, 153, 0))
				if str(self.datas[row][col]) == "_ERREUR" :
					return QtGui.QBrush( QColor(204, 0, 0) )
				if str(self.datas[row][col]) == "_DEBUG" :
					return QtGui.QBrush( QColor(102, 102, 255) )
				# DATA
				'''

		elif  level == LOG_ARGVAL :
			levelStr = "ARG"
		elif  level == LOG_DATA :
			levelStr = "DATA"
		elif  level == LOG_FATAL :
			levelStr = "FATAL"
		elif  level == INFOSETTING :
			levelStr = "LOG_SETTING"
				'''
		if role == Qt.TextColorRole :
			if col == 1 :
				if str(self.datas[row][col]) == "_INFO" :
					return QColor(Qt.white)
				if str(self.datas[row][col]) == "_WARNING" :
					return QColor(Qt.white)
				if str(self.datas[row][col]) == "_ERREUR" :
					return QColor(Qt.white)
				if str(self.datas[row][col]) == "_DEBUG" :
					return QColor(Qt.white)
