#!/usr/bin/env python3
# -*- coding: utf-8 -*-


'''

		{{title}}

        Application entry

        {{datetime}}

        {{author}}

        {{licence}}

'''

import os, shutil

from PyQt5 import QtCore
from PyQt5.QtWidgets import QMessageBox, QFileDialog, QMessageBox, QWidget, QLabel, QVBoxLayout, QPushButton
from PyQt5.QtWidgets import QDialog, QGridLayout, QCheckBox, QTextEdit, QLineEdit, QDialogButtonBox
from PyQt5.QtCore import Qt

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

from lib.util import Util
import lib.setting as AppSetting
                    
from lib.project import Project   
   
class ProjectLoad(QDialog):
		
	projectFileName = None
	
	def __init__(self):

		QDialog.__init__(self)

		fileSelected , check = QFileDialog.getOpenFileName(None, "Ouvrir un projet (*.pro)",
                                               "", "Projets (*.pro);;Tous (*)")
		if check:
			self.projectFileName = fileSelected
			
			  
class ProjectInfos(QDialog):
		
		
	editMode = None
	
	def __init__(self):

		QDialog.__init__(self)

		self.projname = QLabel('Projet')
		self.projname.setAlignment(Qt.AlignRight | Qt.AlignVCenter)		
		self.projname.setToolTip( "Chemin du fichier projet" )
		self.projnameEdit = QLineEdit()		
		self.projnameEdit.setEnabled( False )
		
		self.title = QLabel('Titre')
		self.title.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.title.setToolTip( "Titre du projet" )
		self.titleEdit = QLineEdit()		

		self.summary = QLabel('Description')
		self.summary.setAlignment(Qt.AlignRight )
		self.summaryEdit = QTextEdit()

		self.dbFilename = QLabel('Nom de la base')
		self.dbFilename.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.dbFilename.setToolTip( "Chemin d'accès complet vers la base de données Sqlite" )
		self.dbFilenameEdit = QLineEdit()
		self.selectDbDirButton = QPushButton("...")
		self.selectDbDirButton.clicked.connect(self.onSelectDbDir)

		self.rootMedia = QLabel('Dossier des médias')
		self.rootMedia.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.rootMedia.setToolTip( "Dossier racine des médias externes" )
		self.rootMediaEdit = QLineEdit()
		self.selectMediaDirButton = QPushButton("...")
		self.selectMediaDirButton.clicked.connect(self.onSelectMediaDir)

		self.pageAdd = QLabel('Page à inclure')
		self.pageAdd.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.pageAdd.setToolTip( "Page Html à inclure dans les rapports" )
		self.pageAddEdit = QLineEdit()
		self.selectpageAddButton = QPushButton("...")
		self.selectpageAddButton.clicked.connect(self.onSelectPageAdd)

		self.authorName = QLabel('Auteur (nom)')
		self.authorName.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.authorName.setToolTip( "Nom de l'auteur" )
		self.authorNameEdit = QLineEdit()

		self.authorMail = QLabel('Auteur (email)')
		self.authorMail.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.authorMail.setToolTip( "Email de l'auteur" )
		self.authorMailEdit = QLineEdit()

		self.decuJus = QLabel('DecuJus')
		self.decuJus.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.decuJus.setToolTip( "identifiant interne (id) de l'individu decuJus" )
		self.decuJusEdit = QLineEdit()

		# xxx
		self.rootExportPath = QLabel('Dossier d''export ')
		self.rootExportPath.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.rootExportPath.setToolTip( "Dossier d'export par défaut" )
		self.rootExportPathEdit = QLineEdit()
		self.selectExportPathDirButton = QPushButton("...")
		self.selectExportPathDirButton.clicked.connect(self.onSelectExportPathDir)


		self.saveButton = QPushButton("Sauver")
		self.saveButton.clicked.connect(self.onSave)
		self.cancelButton = QPushButton("Annuler")
		self.cancelButton.clicked.connect(self.onCancel)

		# Main Layout
		self.layout = QGridLayout()

		# Position of Widgets/Elements on Main Layout		
		self.layout.addWidget(self.projname, 1, 0)
		self.layout.addWidget(self.projnameEdit, 1, 1)		
		
		self.layout.addWidget(self.title, 2, 0)
		self.layout.addWidget(self.titleEdit, 2, 1)

		self.layout.addWidget(self.summary, 3, 0)
		self.layout.addWidget(self.summaryEdit, 3, 1)

		self.layout.addWidget(self.dbFilename, 4, 0)
		self.layout.addWidget(self.dbFilenameEdit, 4, 1)
		self.layout.addWidget(self.selectDbDirButton, 4, 2)
		self.dbFilenameEdit.textChanged.connect(self.onDbFileNameChanged)

		self.layout.addWidget(self.rootMedia, 5, 0)
		self.layout.addWidget(self.rootMediaEdit, 5, 1)		
		self.layout.addWidget(self.selectMediaDirButton, 5, 2)
		self.rootMediaEdit.textChanged.connect(self.onRootMediaEditChanged)		

		self.layout.addWidget(self.pageAdd, 6, 0)
		self.layout.addWidget(self.pageAddEdit, 6, 1)		
		self.layout.addWidget(self.selectpageAddButton, 6, 2)
		self.pageAddEdit.textChanged.connect(self.onPageAddEditChanged)		

		self.layout.addWidget(self.authorName, 7, 0)
		self.layout.addWidget(self.authorNameEdit, 7, 1)

		self.layout.addWidget(self.authorMail, 8, 0)
		self.layout.addWidget(self.authorMailEdit, 8, 1)
		
		self.layout.addWidget(self.decuJus, 9, 0)
		self.layout.addWidget(self.decuJusEdit, 9, 1)

		# xxx
		self.layout.addWidget(self.rootExportPath, 10, 0)
		self.layout.addWidget(self.rootExportPathEdit, 10, 1)		
		self.layout.addWidget(self.selectExportPathDirButton, 10, 2)
		self.rootExportPathEdit.textChanged.connect(self.onRootExportPathEditChanged)	

		buttonlayout = QGridLayout()

		self.cancelButton.setMaximumWidth(180)
		buttonlayout.addWidget(self.cancelButton,1,0) 
		
		self.saveButton.setMaximumWidth(180)
		buttonlayout.addWidget(self.saveButton,1,1)
		
		self.layout.addLayout(buttonlayout, 12,1)

		self.setLayout(self.layout)

		self.resize(900, 460)

	def onDbFileNameChanged( self ) :
		if os.access(self.dbFilenameEdit.text(), os.W_OK) is False :
			self.dbFilenameEdit.setToolTip("Le ficher base de données n'existe pas" )
			self.dbFilenameEdit.setStyleSheet("color: red;")
		else :
			self.dbFilenameEdit.setStyleSheet("color: black;")
			
	def onRootMediaEditChanged( self ) :
		if os.path.isdir(self.rootMediaEdit.text()) is False :
			self.rootMediaEdit.setToolTip( "Le dossier des médias externes n'existe pas" )
			self.rootMediaEdit.setStyleSheet("color: red;")
		else :
			self.rootMediaEdit.setStyleSheet("color: black;")
	
	# xxx
	def onRootExportPathEditChanged( self ) :
		if os.path.isdir(self.rootExportPathEdit.text()) is False :
			self.rootExportPathEdit.setToolTip( "Le dossier d'export n'existe pas" )
			self.rootExportPathEdit.setStyleSheet("color: red;")
		else :
			self.rootExportPathEdit.setStyleSheet("color: black;")
		
	def onPageAddEditChanged (self) :
		if os.access(self.pageAddEdit.text(), os.W_OK) is False :
			self.pageAddEdit.setToolTip( "Ce fichier n'existe pas" )
			self.pageAddEdit.setStyleSheet("color: red;")
		else :
			self.pageAddEdit.setStyleSheet("color: black;")
		
		
	def loadProjectData( self, projectFileName, projectObj  ) :
		
		self.projectObj = projectObj
		self.projectFileName = projectFileName
		
		self.setWindowTitle('Propriétés du projet en cours : '+str(os.path.basename(self.projectFileName)) )
		self.projnameEdit.setText(str(self.projectFileName))
		self.projnameEdit.setToolTip( str(self.projectFileName) )		
		self.titleEdit.setText(self.projectObj.title)
		self.summaryEdit.setText(self.projectObj.description)
		self.dbFilenameEdit.setText(self.projectObj.dbFile)
		self.rootMediaEdit.setText(self.projectObj.rootMedia)
		self.pageAddEdit.setText(self.projectObj.pageAdd)
		self.authorNameEdit.setText(self.projectObj.authorName)
		self.authorMailEdit.setText(self.projectObj.authorMail)	
		self.decuJusEdit.setText(self.projectObj.decuJus)	
		# xxx
		self.rootExportPathEdit.setText(self.projectObj.rootExportPath)			
		
		self.editMode = "edit"

	def newProjectData( self  ) :
		
		self.projectObj = Project()	
		self.projectFileName = None
		
		self.setWindowTitle('Nouveau projet' )
		self.projnameEdit.setText(str(self.projectFileName))
		self.projnameEdit.setToolTip( "Nom du fichier projet qui sera créé" )		
		self.titleEdit.setText(self.projectObj.title)
		self.summaryEdit.setText(self.projectObj.description)
		self.dbFilenameEdit.setText(self.projectObj.dbFile)
		self.rootMediaEdit.setText(self.projectObj.rootMedia)
		self.pageAddEdit.setText(self.projectObj.pageAdd)
		self.authorNameEdit.setText(self.projectObj.authorName)
		self.authorMailEdit.setText(self.projectObj.authorMail)		
		self.decuJusEdit.setText(self.projectObj.decuJus)	
		# xxx	
		self.rootExportPathEdit.setText(self.projectObj.rootExportPath)	
		
		self.saveButton.setText("Créer ce projet...")
		
		self.editMode = "new"

	def showEvent(self, event):
		super(ProjectInfos, self).showEvent(event)

	def onSelectPageAdd( self) :
		
		defaultDir = self.pageAddEdit.text()
		
		pageAdd , check = QFileDialog.getOpenFileName(None, "Sélectionner un fichier à inclure",
                                               defaultDir, "Html (*.html);;Tous (*)")											
		if not pageAdd:			
			AppLog.log( LOG_INFO , "Selection page à inclure : annulé par l'utilisateur")	
		else :
			self.pageAddEdit.setText( pageAdd )
			
	def onSelectMediaDir( self ) :
		
		defaultDir = self.projectObj.rootMedia
		
		self.mediaDir = str(QFileDialog.getExistingDirectory(
												self, 
												"Choisir le dossier des médias", 
												defaultDir, 
												QFileDialog.ShowDirsOnly))
		if not self.mediaDir:			
			AppLog.log( LOG_INFO , "Selection du dossier média : annulé par l'utilisateur")	
		else :
			self.rootMediaEdit.setText( self.mediaDir )

	# xxx	
	def onSelectExportPathDir( self ) :
		
		defaultDir = self.projectObj.rootExportPath
		
		self.mediaDir = str(QFileDialog.getExistingDirectory(
												self, 
												"Choisir le dossier d'export", 
												defaultDir, 
												QFileDialog.ShowDirsOnly))
		if not self.mediaDir:			
			AppLog.log( LOG_INFO , "Selection du dossier d'export : annulé par l'utilisateur")	
		else :
			self.rootExportPathEdit.setText( self.mediaDir )

	def onSelectDbDir( self ) :
		
		defaultDir = self.projectObj.dbFile
		
		self.dbDir = str(QFileDialog.getExistingDirectory(
												self, 
												"Choisir le dossier pour la base de données", 
												defaultDir, 
												QFileDialog.ShowDirsOnly))
		if not self.dbDir:			
			AppLog.log( LOG_INFO , "Selection du dossier db : annulé par l'utilisateur")	
		else :
			self.dbFilenameEdit.setText( self.dbDir + "/genea.sqlite" )
			if self.rootMediaEdit.text() == "" :
				self.rootMediaEdit.setText( self.dbDir )
				
			if os.path.isfile(self.dbFilenameEdit.text()) : 
				self.showCriticalMessage( "Erreur" , "Ce dossier contient déjà une base de donnée : " + str(self.dbFilenameEdit.text()) )
				return
			
	def onSave(self) :
		
		if self.editMode == "edit"  :			
			
			# Current project
			
			self.currentProject = Project()		
			
			self.currentProject.title = self.titleEdit.text()
			self.currentProject.description = self.summaryEdit.toPlainText()
			self.currentProject.dbFile = self.dbFilenameEdit.text()
			rootMediaPath = self.rootMediaEdit.text()
			if rootMediaPath[-1] != "/" :
				rootMediaPath = rootMediaPath + "/"
			self.currentProject.rootMedia = rootMediaPath
			
			self.currentProject.pageAdd = self.pageAddEdit.text()
			
			self.currentProject.authorName = self.authorNameEdit.text()
			self.currentProject.authorMail = self.authorMailEdit.text()
			self.currentProject.decuJus = self.decuJusEdit.text()
			
			# xxx
			self.currentProject.rootExportPath = self.rootExportPathEdit.text()

			lastProjectFileName = AppSetting.get('app/lastProject', '')			
			self.currentProject.save( lastProjectFileName )
		
			self.accept()
			
		else :
			# New project			
			# Select dir and name for projnameEdit			
			# Create new database (copy template db)  
			
			if os.path.isfile(self.dbFilenameEdit.text()) : 
				self.showCriticalMessage( "Erreur" , "Ce dossier contient déjà une base de donnée : " + str(self.dbFilenameEdit.text()) )
				return
						
			saveDir = ""
			projFileName = QFileDialog.getSaveFileName(self, 
													"Enregistrer un nouveau projet", 
													saveDir, 
													"Fichiers projet (*.pro);;Tous (*)")		

			if projFileName[0]:	
				
				try :
					
					newProjFileName = projFileName[0]
					if newProjFileName[-3] != ".pro" :
						newProjFileName = newProjFileName + ".pro"
						
					AppLog.log( LOG_INFO , "Création du nouveau projet : ", newProjFileName )	

					self.newProject = Project()	

					self.newProject.dbFile = self.dbFilenameEdit.text()
					
					AppLog.log( LOG_INFO , "copie template db : ", "data/template.sqlite", " => ", self.newProject.dbFile )
					shutil.copy("data/template.sqlite", self.newProject.dbFile)

					self.newProject.title = self.titleEdit.text()
					self.newProject.description = self.summaryEdit.toPlainText()

					AppLog.log( LOG_INFO , "Nouveau projet: title : ", self.newProject.title )
					AppLog.log( LOG_INFO , "Nouveau projet: dbFile : ", self.newProject.dbFile )

					rootMediaPath = self.rootMediaEdit.text()
					if rootMediaPath[-1] != "/" :
						rootMediaPath = rootMediaPath + "/"
					self.newProject.rootMedia = rootMediaPath									
					AppLog.log( LOG_INFO , "Nouveau projet: rootMedia : ", self.newProject.rootMedia )

					self.newProject.pageAdd = self.pageAddEdit.text()
					AppLog.log( LOG_INFO , "Nouveau projet: pageAdd : ", self.newProject.pageAdd )

					self.newProject.authorName = self.authorNameEdit.text()
					self.newProject.authorMail = self.authorMailEdit.text()
					self.newProject.decuJus = self.decuJusEdit.text()

					# xxx
					self.newProject.rootExportPath = self.rootExportPathEdit.text()

					self.newProject.save( newProjFileName )
					
					lastProjectFileName = AppSetting.set('app/lastProject', newProjFileName)			
					
					self.showMessage( "Information", "Le projet a été créé : " + newProjFileName )	
					
					self.accept()					
					
				except Exception as e:				
					AppLog.log( LOG_ERROR , "Création du nouveau projet : ", str(e) )
					self.showCriticalMessage( "Erreur" , str(e) )				
		

	def onCancel(self) :
		
		self.reject()
		
	def showMessage( self, titleText, message ) :
		
		msg = QMessageBox()

		msg.setIcon(QMessageBox.Information)
		
		msg.setText(message)
		#msg.setInformativeText('More information')
		msg.setWindowTitle(titleText)		

		msg.exec_()	

	def showCriticalMessage( self, titleText, message ) :
		
		msg = QMessageBox()

		msg.setIcon(QMessageBox.Critical)
		
		msg.setText(message)
		#msg.setInformativeText('More information')
		msg.setWindowTitle(titleText)		

		msg.exec_()	
