#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
    
    
"""

import re, os
from os.path import exists

from datetime import datetime
from PyQt5.QtCore import QDate, QDateTime

class Util() :
        
	debug = False        
        
	def getQDatetimeFromString(srtdatetime, defaultValue = QDateTime(1,1,1,0,0), allowqdate = False ):
			
		if srtdatetime is None :
			return defaultValue		
		
		if Util.debug is True : print("--- Format qdatetime [" + str(srtdatetime) + "]")					
		myQdate = QDateTime(1,1,1,0,0)		
		date_patterns = [ "%Y-%m-%d %H:%M", "%Y-%m-%d" ] # db date formats acceptés
		for pattern in date_patterns:
			try:
				pytonDateTime = datetime.strptime(srtdatetime, pattern)
				
				if Util.debug is True : 				
					print("Pattern = "+pattern)
					print(pytonDateTime)
					print("   day : "+str(pytonDateTime.day))
					print("   month : "+str(pytonDateTime.month))
					print("   year : "+str(pytonDateTime.year))
					print("   hour : "+str(pytonDateTime.hour))
					print("   minute : "+str(pytonDateTime.minute))
				
				if pytonDateTime.hour == 0 and pytonDateTime.minute == 0 and allowqdate is True :
					myQdate = QDate( 
											pytonDateTime.year,
											pytonDateTime.month,
											pytonDateTime.day
										)				
				else :
					myQdate = QDateTime( pytonDateTime.year,pytonDateTime.month, pytonDateTime.day, pytonDateTime.hour,	pytonDateTime.minute )	
				if Util.debug is True : print("  return " + str(myQdate.value()))
				return myQdate
						
			except:
				pass
				
		if Util.debug is True : print("   return default date : ",defaultValue.toString() )
		
		return myQdate		

	def fmtstr(fmt, strr):
		
		"""
		To format some string !!!
		"""

		res = []
		i = 0
		s=re.sub(r'[^\w]', '', strr)
		for c in fmt:
			if c == '#':
				res.append(s[i:i+1])
				i = i+1
			else:
				res.append(c)
		res.append(s[i:])
		return "".join(res)

	def titling(name):
		"""
		To display the names and surnames in uppercase for 1st letter.
		"""

		return name.title()

	def single_quote_remover(text):
		"""
		To remove single quotes from entry to prevent SQL crash.
		"""

		return text.replace ("'", "/")

	def single_quote_returner(text):
		"""
		To display the single quote for the user ex: cote d'or as chocolat:)))
		"""

		return text.replace("/", "'")

	def human_size(self, size_bytes):
		"""
		format a size in bytes into a 'human' file size, e.g. bytes, KB, MB, GB, TB, PB
		Note that bytes/KB will be reported in whole numbers but MB and above will have greater precision
		e.g. 1 byte, 43 bytes, 443 KB, 4.3 MB, 4.43 GB, etc
		"""
		if size_bytes == 1:
			# because I really hate unnecessary plurals
			return "1 byte"

		suffixes_table = [('bytes',0),('KB',0),('MB',1),('GB',2),('TB',2), ('PB',2)]

		num = float(size_bytes)
		for suffix, precision in suffixes_table:
			if num < 1000.0:
				break
			num /= 1000.0

		if precision == 0:
			formatted_size = "%d" % num
		else:
			formatted_size = str(round(num, ndigits=precision))

		return "%s %s" % (formatted_size, suffix)
				
	def get_size(self, start_path = '.'):
		total_file = 0
		total_size = 0
		for dirpath, dirnames, filenames in os.walk(start_path):
			for f in filenames:
				
				fp = os.path.join(dirpath, f)
				# skip if it is symbolic link
				if not os.path.islink(fp):
					total_size += os.path.getsize(fp)
					total_file = total_file +1
					# print(str(total_file) + "->" + str(os.path.join(dirpath, f)))
		
		return total_size, self.human_size(total_size)

	def get_file_size(self, file_path ):
		
		if not exists(file_path) :
			return 0,"0"
		
		try :
			total_size = os.path.getsize(file_path)
			return total_size, self.human_size(total_size)
			
		except Exception as e:		
			return 0,"0"
		
	def get_file_date(self, file_path ):

		if not exists(file_path) :
			return 0,"0"
			
		try:			
			stat = os.stat(file_path)			
			return stat.st_birthtime, self.formatTimeStamp(stat.st_mtime)
			
		except Exception as e:
			# We're probably on Linux. No easy way to get creation dates here,
			# so we'll settle for when its content was last modified.
			try:
				stat = os.stat(file_path)							
				return stat.st_mtime, self.formatTimeStamp(stat.st_mtime)					
			except Exception as e:
				return 0,"0"

	def formatTimeStamp( self , fileMtime) :
		import datetime
		timestamp_str = datetime.datetime.fromtimestamp(fileMtime).strftime('%d/%m/%Y %H:%M')		
		return timestamp_str
		

class zipUtil() :
		
	def archive( self, zipfilename, filesDirsList, compressionFlag = True ):
	
		import zipfile 
		
		import os
		from os.path import basename

		compression = zipfile.ZIP_STORED
		if compressionFlag is True :
			try:
				import zlib
				compression = zipfile.ZIP_DEFLATED
			except:
				compression = zipfile.ZIP_STORED		
			
		zipObj = zipfile.ZipFile( zipfilename, 'w')
		
		for key, elem in filesDirsList.items(): 

			rootOutDir = key
			
			if os.path.isdir(elem) : 					

				dbMediaRoot = elem
					
				empty_dirs = [] 				

				# Iterate over all the files in directory
				for folderName, subfolders, filenames in os.walk(elem):
					for filename in filenames:
					   
						empty_dirs.extend([dir for dir in subfolders if os.listdir(os.path.join(folderName, dir)) == []])  

						# Create complete filepath of file in directory
						filePath = os.path.join(folderName, filename)
						# print("filePath ", filePath)
						
						mediaFilePath = filePath[ len( os.path.dirname( dbMediaRoot) ):]
						if mediaFilePath[:1]=='/' :
							mediaFilePath=mediaFilePath[1:]
							
						# print("rootOutDir-mediaFilePath ", rootOutDir, ' - ' , mediaFilePath)
						
						outFullPathName = os.path.join( rootOutDir, mediaFilePath )

						print("add ", outFullPathName)

						# Add file to zip
						zipObj.write( filePath, outFullPathName, compress_type=compression )
						
						for dir in empty_dirs:  
							filePath = os.path.join(folderName, dir)
							mediaFilePath = filePath[ len( os.path.dirname( dbMediaRoot) ):]
							if mediaFilePath[:1]=='/' :
								mediaFilePath=mediaFilePath[1:]							
							outFullPathName = os.path.join( rootOutDir, mediaFilePath )
							zipObj.write( filePath, outFullPathName, compress_type=compression) 
						
						empty_dirs = []  						
						
			if os.path.isfile(elem) : 
				
			   # Add file to zip
			   outFullPathName = os.path.join( rootOutDir, basename(elem) )
			   # print("append file : ", elem, " -> ", outFullPathName)			   
			   zipObj.write( elem, outFullPathName, compress_type=compression )		
				
		zipObj.close()

