#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

        Saisie des données généalogiques (individus>evennements>sources>documents et médias)

        07/02/2021 18:40

        lvardon@laposte.net - 2021

        Licence libre

'''

from PyQt5.QtWidgets import QFileDialog, QWidget, QLabel, QVBoxLayout, QTabWidget
from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import QSettings			

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
import lib.setting as AppSetting
from lib.util import Util

from libview.person import PersonListWidget, PersonEditWidget				
from libview.event import EventListWidget, EventEditWidget
from libview.source import SourceListWidget, SourceEditWidget
from libview.document import DocumentListWidget, DocumentEditWidget				
from libview.media import MediaListWidget, MediaEditWidget	

from libdb.event_source import Event_source, Event_sourcesDb
from libdb.media_person import Media_person, Media_personsDb

class Dataenter( QWidget ) :

	db = None
	currentPersonId = None
	currentEventId = None
	currentSourceId = None
	currentDocumentId = None
	currentMediaId = None
	
	storedPersonId = None
	storedEventId = None
	storedSourceId = None
	storedDocumentId = None
	storedMediaId = None	
	
	myPersonListWidget = None
	myEventListWidget = None
	mySourceListWidget = None
	myDocumentListWidget = None
	myMediaListWidget = None

	tabPersonLabel = "Individus"
	tabEventLabel = "Evenements"
	tabSourceLabel = "Sources"
	tabDocumentLabel = "Documents"
	tabMediaLabel = "Médias"

	mainWindowEditMode =  pyqtSignal( int, str )
	mainWindowListMode = pyqtSignal()
	mainWindowUserMessage = pyqtSignal( int, str )
	
	myPersonListWidget = None
	myPersonEditWidget = None
	
	log = None
	iniEditReference = None
	
	def __init__(self , parent, db, rootMedia ):

		super().__init__()
		
		self.iniEditReference = QSettings(AppSetting.get('app/lastProject', ''), QSettings.IniFormat)			
		self.db = db
		self.parent = parent	
		
		self.rootMedia = rootMedia
		
		self.initUI()
		
		self.loadEditReferences()
		self.debugTrace("Initial lists selection")
		self.myPersonListWidget.selectRowById(self.storedPersonId)
		self.myEventListWidget.selectRowById(self.storedEventId)
		self.mySourceListWidget.selectRowById(self.storedSourceId)
		self.myDocumentListWidget.selectRowById(self.storedDocumentId)
		self.myMediaListWidget.selectRowById(self.storedMediaId)
		
		
						
	def initUI(self):	
							
		# Tabs		
		self.tabs = QTabWidget()
		self.tabPerson = QWidget()
		self.tabEvent = QWidget()
		self.tabSource = QWidget()
		self.tabDocument = QWidget()
		self.tabMedia = QWidget()						
		self.tabs.addTab(self.tabPerson, QIcon('img/menu_table_person.png'), "Individus")
		self.tabs.addTab(self.tabEvent, QIcon('img/menu_table_event.png'), "Evenement")
		self.tabs.addTab(self.tabSource, QIcon('img/menu_table_source.png'), "Source")
		self.tabs.addTab(self.tabDocument, QIcon('img/menu_table_document.png'), "Document")
		self.tabs.addTab(self.tabMedia, QIcon('img/menu_table_media.png'), "Media")

		# Add tabs to widget
		self.layout = QVBoxLayout()		
		self.layout.addWidget(self.tabs)
		self.setLayout(self.layout)
		
		# TAB 1
		self.tabPersonlayout = QVBoxLayout()		
		self.tabPerson.setLayout(self.tabPersonlayout)

		# TAB 2
		self.tabEventlayout = QVBoxLayout()		
		self.tabEvent.setLayout(self.tabEventlayout)
			
		# TAB 3
		self.tabSourcelayout = QVBoxLayout()
		self.tabSource.setLayout(self.tabSourcelayout)  

		# TAB 4
		self.tabDocumentlayout = QVBoxLayout()		
		self.tabDocument.setLayout(self.tabDocumentlayout)

		# TAB 5
		self.tabMedialayout = QVBoxLayout()		
		self.tabMedia.setLayout(self.tabMedialayout)
		
		self.tabs.currentChanged.connect(self.onTabChange) 

		self.setTabsHeaderReset()
		self.initWidgets()		
		self.initDataTabs()
		
	# Point d'entrée depuis le menu principal		
	def main( self ) :
		#self.parent.setWindowTitle( "" )
		self.myPersonListWidget.setFocus()
		pass

	def debugTrace(self, message) :		
		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)		
		AppLog.log( LOG_INFO, message)

	@pyqtSlot( )	
	def onCloseWidget( self ) :		
		self.debugTrace("Close event.")		
		self.storeEditReferences()
		
	#@pyqtSlot()  
	def onTabChange(self,i): 		
		self.debugTrace("onTabChange, #" + str(i))			

	def setTabsHeaderReset(self):
		
		bar = self.tabs.tabBar()

		bar.setTabTextColor(0, Qt.gray)
		self.tabs.setTabText(0, self.tabPersonLabel)
		bar.setTabTextColor(1, Qt.gray)
		self.tabs.setTabText(1, self.tabEventLabel)
		bar.setTabTextColor(2, Qt.gray)
		self.tabs.setTabText(2, self.tabSourceLabel)
		bar.setTabTextColor(3, Qt.gray)
		self.tabs.setTabText(3, self.tabDocumentLabel)
		bar.setTabTextColor(4, Qt.gray)
		self.tabs.setTabText(4, self.tabMediaLabel)

	def refreshTabsLabel( self) :
		
		self.setTabsHeaderReset()
		self.setTabsHeaderTextPerson()						
		self.setTabsHeaderTextEvent()			
		self.setTabsHeaderTextSource()						
		self.setTabsHeaderTextDocument()									
		self.setTabsHeaderTextMedia()	
		
	def setTabsHeaderTextPerson(self):
		bar = self.tabs.tabBar()

		bar.setTabTextColor(0, Qt.black)
		self.tabs.setTabText(0, self.tabPersonLabel + " (" + str(self.myPersonListWidget.model.rowCount(self)) + ")")
		if self.myPersonListWidget.model.rowCount(self) == 0:
			bar.setTabTextColor(0, Qt.red)

	def setTabsHeaderTextEvent(self, disable = False):
		bar = self.tabs.tabBar()

		if disable is True  :
			bar.setTabTextColor(1, Qt.gray)
			self.tabs.setTabText(1, self.tabEventLabel)	
		else :
			bar.setTabTextColor(1, Qt.black)
			self.tabs.setTabText(1, self.tabEventLabel + " (" + str(self.myEventListWidget.model.rowCount(self)) + ")")
			if self.myEventListWidget.model.rowCount(self) == 0:
				bar.setTabTextColor(1, Qt.red)

	def setTabsHeaderTextSource(self, disable = False):
		bar = self.tabs.tabBar()

		if disable is True :
			bar.setTabTextColor(2, Qt.gray)
			self.tabs.setTabText(2, self.tabSourceLabel)			
		else :
			bar.setTabTextColor(2, Qt.black)
			self.tabs.setTabText(2, self.tabSourceLabel + " (" + str(self.mySourceListWidget.model.rowCount(self)) + ")")
			if self.mySourceListWidget.model.rowCount(self) == 0:
				bar.setTabTextColor(2, Qt.red)
			
	def setTabsHeaderTextDocument(self, disable = False):
		bar = self.tabs.tabBar()

		if disable is True :
			bar.setTabTextColor(3, Qt.gray)
			self.tabs.setTabText(3, self.tabDocumentLabel)			
		else :
			bar.setTabTextColor(3, Qt.black)
			self.tabs.setTabText(3, self.tabDocumentLabel + " (" + str(self.myDocumentListWidget.model.rowCount(self)) + ")")
			if self.myDocumentListWidget.model.rowCount(self) == 0:
				bar.setTabTextColor(3, Qt.red)

	def setTabsHeaderTextMedia(self, disable = False):
		bar = self.tabs.tabBar()
		
		if disable is True  :
			bar.setTabTextColor(4, Qt.gray)
			self.tabs.setTabText(4, self.tabMediaLabel)	
		else :
			bar.setTabTextColor(4, Qt.black)
			self.tabs.setTabText(4, self.tabMediaLabel + " (" + str(self.myMediaListWidget.model.rowCount(self)) + ")")
			if self.myMediaListWidget.model.rowCount(self) == 0:
				bar.setTabTextColor(4, Qt.red)

	def clearLayout( self, layout ) :
		
		for i in reversed(range(layout.count())): 
			widgetToRemove = layout.itemAt( i ).widget()
			# remove it from the layout list
			layout.removeWidget( widgetToRemove )
			# remove it from the gui
			widgetToRemove.setParent( None )
			
	def setWindowTitle( self, message=""  ) :

		#self.parent.setWindowTitle( message + " / person={} / event={} / source={} / doc={}".format( self.currentPersonId, self.currentEventId, self.currentSourceId, self.currentDocumentId ) )

		winDowMessage = ""

		textDisplay = self.myPersonListWidget.getRowData(self.currentPersonId).getName()
		if textDisplay is not None :
			winDowMessage = winDowMessage + "[ "+ textDisplay + " ]"

		textDisplay = self.myEventListWidget.getRowData(self.currentEventId).getName()
		if textDisplay is not None :
			winDowMessage = winDowMessage + " -> " + textDisplay

		textDisplay = self.mySourceListWidget.getRowData(self.currentSourceId).getName()
		if textDisplay is not None :
			winDowMessage = winDowMessage + " -> " + textDisplay
			
		self.debugTrace("setWindowTitle = " + winDowMessage )
		
		self.parent.setWindowTitle( winDowMessage )

	@pyqtSlot( int, str )
	def onUserMessage(self, level, message ):
		
		self.parent.statusBar().showMessage(message)
		
	def initWidgets( self ) :
		
		# -- Person
		self.myPersonListWidget = PersonListWidget( self.db, self.rootMedia, "source_dataenter" )		
		
		self.myPersonListWidget.mainWindowEditMode.connect(self.onEditPerson)
		self.myPersonListWidget.mainWindowRemoveRow.connect(self.onPersonRemoveRow)		
		self.myPersonListWidget.selectionSignal.connect(self.onPersonSelectionSignal)				
		self.myPersonListWidget.mainWindowListMode.connect(self.onListPerson)
		self.myPersonListWidget.mainWindowUserMessage.connect(self.onUserMessage)
	
		# -- Event
		self.myEventListWidget = EventListWidget( self.db, self.rootMedia, "source_dataenter", "where person_1_id = ? or person_2_id = ? or person_3_id = ?" , [self.currentPersonId, self.currentPersonId, self.currentPersonId])
		self.myEventListWidget.mainWindowEditMode.connect(self.onEditEvent)
		self.myEventListWidget.mainWindowRemoveRow.connect(self.onEventRemoveRow)		
		self.myEventListWidget.selectionSignal.connect(self.onEventSelectionSignal)				
		self.myEventListWidget.mainWindowListMode.connect(self.onListEvent)
		self.myEventListWidget.mainWindowUserMessage.connect(self.onUserMessage)
				
		# - Source
		source_id_list = []			
		self.myEvent_SourceObj = Event_sourcesDb ( self.db )
		event_sources = self.myEvent_SourceObj.selectById( "where event_id = ?" , [self.currentEventId] )
		
		if event_sources : 
			for event_source in event_sources : 
				source_id_list.append(event_source.getSource_id())
		
		self.mySourceListWidget = SourceListWidget( self.db, self.rootMedia, "source_dataenter", "where a.id in ("+self.getSqlWherePlaceHolder( len(source_id_list) )+")" , source_id_list)					
		self.mySourceListWidget.mainWindowEditMode.connect(self.onEditSource)
		self.mySourceListWidget.mainWindowRemoveRow.connect(self.onSourceRemoveRow)		
		self.mySourceListWidget.selectionSignal.connect(self.onSourceSelectionSignal)						
		self.mySourceListWidget.mainWindowListMode.connect(self.onListSource)
		self.mySourceListWidget.mainWindowUserMessage.connect(self.onUserMessage)
						
		# -- Document
		document_id_list = []	
		self.myEvent_SourceObj = Event_sourcesDb ( self.db )
		event_sources = self.myEvent_SourceObj.selectById( "where event_id = ? and source_id = ?" , [ self.currentEventId, self.currentSourceId ] )		

		if event_sources : 
			for event_source in event_sources : 
				document_id_list.append(event_source.getDocument_id())

		self.myDocumentListWidget = DocumentListWidget( self.db, self.rootMedia, "source_dataenter", "where a.id in ("+self.getSqlWherePlaceHolder( len(document_id_list) )+")" , document_id_list)
		self.myDocumentListWidget.mainWindowEditMode.connect(self.onEditDocument)
		self.myDocumentListWidget.mainWindowRemoveRow.connect(self.onDocumentRemoveRow)				
		self.myDocumentListWidget.selectionSignal.connect(self.onDocumentSelectionSignal)								
		self.myDocumentListWidget.mainWindowListMode.connect(self.onListDocument)
		self.myDocumentListWidget.mainWindowUserMessage.connect(self.onUserMessage)		
		
		# -- Media
		self.media_id_list = []	
		self.myMedia_PersonObj = Media_personsDb ( self.db )
		media_persons = self.myMedia_PersonObj.selectById( "where person_id = ?" , [ self.currentPersonId ] )		

		if media_persons : 			
			for media_person in media_persons : 			
				self.media_id_list.append(media_person.getMedia_id())

		self.myMediaListWidget = MediaListWidget( self.db, self.rootMedia, "source_dataenter", "where a.id in ("+self.getSqlWherePlaceHolder( len(self.media_id_list) )+")" , self.media_id_list)
		self.myMediaListWidget.mainWindowEditMode.connect(self.onEditMedia)
		self.myMediaListWidget.mainWindowRemoveRow.connect(self.onMediaRemoveRow)				
		self.myMediaListWidget.selectionSignal.connect(self.onMediaSelectionSignal)								
		self.myMediaListWidget.mainWindowListMode.connect(self.onListMedia)
		self.myMediaListWidget.mainWindowUserMessage.connect(self.onUserMessage)	
						
	def initDataTabs( self ) :
		
		self.debugTrace("initDataTabs")
		
		# liste initiale Person
		self.myPersonListWidget.loadData( None, None )
		self.clearLayout(self.tabPersonlayout)					
		self.tabPersonlayout.addWidget(self.myPersonListWidget)	
		self.setTabsHeaderTextPerson()
	
	
	# ------------------------------------------------------------------	
	# --------------------------- Person -------------------------------
	# ------------------------------------------------------------------		
	# Après affichage liste	
	@pyqtSlot()
	def onListPerson( self ) :
		
		self.debugTrace("Person : onListPerson")
		
		self.myPersonListWidget.loadData( None, None )
		self.clearLayout(self.tabPersonlayout)					
		self.tabPersonlayout.addWidget(self.myPersonListWidget)	
		self.myPersonListWidget.selectRowById( self.currentPersonId )
		self.setTabsHeaderTextPerson()
		
	# Après affichage fiche edition
	@pyqtSlot( int, str )
	def onEditPerson(self, rowId, message ):

		self.debugTrace("Person : onEditPerson rowId=" + str(rowId))
		
		rowDataObj = self.myPersonListWidget.getRowData( rowId )
		
		self.myPersonEditWidget = PersonEditWidget( self.db, \
													self.rootMedia, \
													rowId, \
													None, \
													self.myPersonListWidget.getIdList(), \
													self.myPersonListWidget.getSqlOrderByField(), \
													self.myPersonListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		
		self.myPersonEditWidget.enterEditSignal.connect( self.onPersonEnterEditSignal )
		self.myPersonEditWidget.mainWindowUserMessage.connect( self.onUserMessage )
		self.myPersonEditWidget.validEditSignal.connect( self.onPersonValidEditSignal )		
		self.myPersonEditWidget.closeEdit.connect( self.onPersonCloseEditSignal )		
		
		
		self.clearLayout( self.tabPersonlayout )		
		
		self.tabPersonlayout.addWidget( self.myPersonEditWidget )			

	# Après validation des données fiche
	def onPersonValidEditSignal( self, validState, rowId ) :
		
		self.debugTrace("Person : onPersonValidEditSignal validState=" + str(validState))
		
		if validState is True :

			self.myPersonListWidget.reloadData()
			self.setTabsHeaderTextPerson()
						
	# Après fermeture fenêtre edition
	def onPersonCloseEditSignal(self, btnState, rowId ) :
		self.debugTrace("Person : onPersonCloseEditSignal btnState=" + str(btnState))
		self.clearLayout(self.tabPersonlayout)
				
		self.myPersonListWidget.tableView.restoreState = False		
		self.tabPersonlayout.addWidget(self.myPersonListWidget)	
		
		self.myPersonListWidget.selectRowById( self.currentPersonId )
		
	# Après suppression fiche
	def onPersonRemoveRow( self, rowId ) :

		self.debugTrace("Person : onPersonRemoveRow")

		self.currentPersonId = None
		
		self.clearLayout(self.tabEventlayout)	
		self.clearLayout(self.tabSourcelayout)	
		self.clearLayout(self.tabDocumentlayout)	
		self.clearLayout(self.tabMedialayout)		
		
		self.setTabsHeaderReset()
		self.setTabsHeaderTextPerson()	
					
	# Après affichage d'une nouvelle fiche 	
	def onPersonEnterEditSignal( self, rowId ) :
		
		self.debugTrace("Person : onPersonEnterEditSignal")
		
		self.currentPersonId = rowId
		self.currentEventId = None	
		self.currentSourceId = None	
		self.currentDocumentId = None
		self.currentMediaId = None
		
		self.onListEvent()		
		self.clearLayout(self.tabSourcelayout)	
		self.clearLayout(self.tabDocumentlayout)	
		self.onListMedia()
		
		self.setTabsHeaderTextEvent()			
		self.setTabsHeaderTextSource( disable=True )						
		self.setTabsHeaderTextDocument( disable=True )	
		self.setTabsHeaderTextMedia()	
		
		self.setWindowTitle()			
		
	# Après selection d'une ligne
	def onPersonSelectionSignal( self, rowId) :
		
		self.debugTrace("Person : select line rowId=" + str(rowId))

		self.currentPersonId = rowId
		self.currentEventId = None	
		self.currentSourceId = None	
		self.currentDocumentId = None
		self.currentMediaId = None
		
		self.onListEvent()		
		self.clearLayout(self.tabSourcelayout)	
		self.clearLayout(self.tabDocumentlayout)	
		self.onListMedia()
		
		self.setTabsHeaderTextEvent()			
		self.setTabsHeaderTextSource( disable=True )						
		self.setTabsHeaderTextDocument( disable=True )	
		self.setTabsHeaderTextMedia()	
		
		self.setWindowTitle()		
				
														
	def storeEditReferences( self ) :	
			
		self.debugTrace("Store edit references.")
		
		self.debugTrace(" currentPersonId = " + str(self.currentPersonId))
		self.debugTrace(" currentEventId = "+ str( self.currentEventId))
		self.debugTrace(" currentSourceId = "+ str( self.currentSourceId))
		self.debugTrace(" currentDocumentId = "+ str( self.currentDocumentId))
		self.debugTrace(" currentMediaId = "+ str( self.currentMediaId))
		
		if self.currentPersonId is not None :
			self.iniEditReference.setValue('editreferences/currentPersonId',self.currentPersonId)  
		if self.currentEventId is not None :	
			self.iniEditReference.setValue('editreferences/currentEventId',self.currentEventId)  
		if self.currentSourceId is not None :			
			self.iniEditReference.setValue('editreferences/currentSourceId',self.currentSourceId)  
		if self.currentDocumentId is not None :	
			self.iniEditReference.setValue('editreferences/currentDocumentId',self.currentDocumentId)  
		if self.currentMediaId is not None :	
			self.iniEditReference.setValue('editreferences/currentMediaId',self.currentMediaId)  
		
		self.iniEditReference.sync()			
	
	def loadEditReferences( self ) :
				
		self.debugTrace("Load edit references.")
						
		try :
			self.storedPersonId = int( self.iniEditReference.value('editreferences/currentPersonId'))
		except Exception as e:
			self.storedPersonId = None	
		try :
			self.storedEventId = int( self.iniEditReference.value('editreferences/currentEventId'))
		except Exception as e:
			self.storedEventId = None		
		try :
			self.storedSourceId = int( self.iniEditReference.value('editreferences/currentSourceId'))
		except Exception as e:
			self.storedSourceId = None	
		try :
			self.storedDocumentId = int( self.iniEditReference.value('editreferences/currentDocumentId'))
		except Exception as e:
			self.storedDocumentId = None				
		try :
			self.storedMediaId = int( self.iniEditReference.value('editreferences/currentMediaId'))
		except Exception as e:
			self.storedMediaId = None	

		self.debugTrace(" storedPersonId = " + str(self.storedPersonId))
		self.debugTrace(" storedEventId = "+ str( self.storedEventId))
		self.debugTrace(" storedSourceId = "+ str( self.storedSourceId))
		self.debugTrace(" storedDocumentId = "+ str( self.storedDocumentId))
		self.debugTrace(" storedMediaId = "+ str( self.storedMediaId))
		
	# ------------------------------------------------------------------
	# ---------------------------- Event -------------------------------
	# ------------------------------------------------------------------
	
	# Après affichage liste			
	@pyqtSlot()
	def onListEvent( self ) :

		if self.currentPersonId is not None : 			
			self.myEventListWidget.loadData( "where person_1_id = ? or person_2_id = ? or person_3_id = ?" , [self.currentPersonId, self.currentPersonId, self.currentPersonId])	
			self.clearLayout(self.tabEventlayout)						
			self.tabEventlayout.addWidget(self.myEventListWidget)				
			self.myEventListWidget.selectRowById( self.currentEventId )
			self.setTabsHeaderTextEvent()
			
	# Après affichage fiche edition				
	@pyqtSlot( int, str )
	def onEditEvent(self, rowId, message ):		

		self.debugTrace("Event : onEditEvent rowid=" + str(rowId) + " currentPersonId=" +str( self.currentPersonId))
		
		rowDataObj = self.myEventListWidget.getRowData( rowId )
		
		# Mode "Nouveau"
		# Par défaut : créer l'évènement pour l'individu selectionné
		if rowId == -1 :
			rowDataObj.setPerson_1_id( self.currentPersonId )

		self.myEventEditWidget = EventEditWidget( self.db, \
													self.rootMedia, \
													rowId, \
													None, \
													self.myEventListWidget.getIdList(), \
													self.myEventListWidget.getSqlOrderByField(), \
													self.myEventListWidget.getSqlOrderByOrder(), \
													rowDataObj )		
		
		self.myEventEditWidget.enterEditSignal.connect(self.onEventEnterEditSignal)
		self.myEventEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myEventEditWidget.validEditSignal.connect( self.onEventValidEditSignal )	
		self.myEventEditWidget.closeEdit.connect( self.onEventCloseEditSignal )		
		
		self.clearLayout(self.tabEventlayout)		
		self.tabEventlayout.addWidget(self.myEventEditWidget)		

	# Après validation des données fiche
	def onEventValidEditSignal(self, validState, rowId ) :
		self.debugTrace("Event : onEventValidEditSignal")
		
		if validState is True :

			self.myEventListWidget.reloadData()
			self.setTabsHeaderTextEvent()		
			
	# Après fermeture fenêtre edition
	def onEventCloseEditSignal(self, btnState, rowId ) :
		self.debugTrace("Event : onEventCloseEditSignal btnState=" + str(btnState))
		self.clearLayout(self.tabEventlayout)	
						
		self.myEventListWidget.tableView.restoreState = False
		self.tabEventlayout.addWidget(self.myEventListWidget)	
		self.myEventListWidget.selectRowById( self.currentEventId )		

	# Après suppression fiche
	def onEventRemoveRow( self, rowId ) :
		
		self.debugTrace("Event : onEventRemoveRow")
		self.currentEventId = None
		self.onListEvent()

	# Après affichage d'une nouvelle fiche 	
	def onEventEnterEditSignal(self, rowId) :

		self.currentEventId = rowId
		self.currentSourceId = None
		self.currentDocumentId = None	
		self.currentMediaId = None
								
		self.onListSource()
		self.clearLayout(self.tabDocumentlayout)	
		
		self.setTabsHeaderTextSource()						
		self.setTabsHeaderTextDocument( disable = True )	
		
		self.setWindowTitle()
		
	# Après selection d'une ligne			
	def onEventSelectionSignal( self, rowId) :		
		
		self.debugTrace("Event : select line rowId=" + str(rowId))
				
		self.currentEventId = rowId
		self.currentSourceId = None
		self.currentDocumentId = None	
		
								
		self.onListSource()
		self.clearLayout(self.tabDocumentlayout)	
		
		self.setTabsHeaderTextSource()						
		self.setTabsHeaderTextDocument( disable = True )	
		
		self.setWindowTitle()			

	# ------------------------------------------------------------------
	# ---------------------------- Source ------------------------------		
	# ------------------------------------------------------------------
	
	# Après affichage liste			
	@pyqtSlot()
	def onListSource( self ) :

		self.debugTrace("Source : onListSource")

		# Liste des id (affichés) depuis la table d'association
		myEvent_SourceObj = Event_sourcesDb ( self.db )
		event_sources = myEvent_SourceObj.selectById( "where event_id = ?" , [self.currentEventId] )	
		source_id_list = []				
		if event_sources : 
			for event_source in event_sources : 
				if type(event_source.getSource_id()) == int :
					source_id_list.append(event_source.getSource_id())
					
		self.mySourceListWidget.loadData( "where a.id in ("+self.getSqlWherePlaceHolder( len(source_id_list) )+")" , source_id_list )

		self.clearLayout(self.tabSourcelayout)					
		self.tabSourcelayout.addWidget(self.mySourceListWidget)				
		
		self.onListDocument()				
					
	# Après affichage fiche edition				
	@pyqtSlot( int, str )
	def onEditSource(self, rowId, message ):
		
		rowDataObj = self.mySourceListWidget.getRowData( rowId )

		self.mySourceEditWidget = SourceEditWidget( self.db, \
													self.rootMedia, \
													rowId, \
													None, \
													self.mySourceListWidget.getIdList(), \
													self.mySourceListWidget.getSqlOrderByField(), \
													self.mySourceListWidget.getSqlOrderByOrder(), \
													rowDataObj )
																	
		self.mySourceEditWidget.enterEditSignal.connect( self.onSourceEnterEditSignal )				
		self.mySourceEditWidget.validEditSignal.connect( self.onSourceValidEditSignal )	
		self.mySourceEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.mySourceEditWidget.closeEdit.connect( self.onSourceCloseEditSignal )		
		
		self.clearLayout(self.tabSourcelayout)		
		self.tabSourcelayout.addWidget(self.mySourceEditWidget)		

	# Après validation des données fiche
	def onSourceValidEditSignal( self, validState, rowId ) :
		if validState is True :
			
			myEvent_SourceObj = Event_source( 	id = rowId,
												event_id = self.currentEventId, 
												source_id = self.currentSourceId, 
												document_id = None,
												datecreated = self.getCurentDateTime()    
											)
			Event_sourcesDb ( self.db ).update( myEvent_SourceObj)
			
			# Refresh datas
			# Liste des id (affichés) depuis la table d'association
			myEvent_SourceObj = Event_sourcesDb ( self.db )
			event_sources = myEvent_SourceObj.selectById( "where event_id = ?" , [self.currentEventId] )	
			source_id_list = []				
			if event_sources : 
				for event_source in event_sources : 
					if type(event_source.getSource_id()) == int :
						source_id_list.append(event_source.getSource_id())
						
			self.mySourceListWidget.loadData( "where a.id in ("+self.getSqlWherePlaceHolder( len(source_id_list) )+")" , source_id_list )
					
		self.setTabsHeaderTextSource()

	# Après fermeture fenêtre edition
	def onSourceCloseEditSignal(self, btnState, rowId ) :
		self.debugTrace("Source : onSourceCloseEditSignal btnState=" + str(btnState))
		self.clearLayout(self.tabSourcelayout)	
		self.mySourceListWidget.tableView.restoreState = False				
		self.tabSourcelayout.addWidget(self.mySourceListWidget)	
		self.mySourceListWidget.selectRowById( self.currentSourceId )		

	# Après suppression fiche
	def onSourceRemoveRow( self, rowId ) :
		self.currentSourceId = None
		self.onListSource()

		self.setTabsHeaderTextSource()

	# Après affichage d'une nouvelle fiche 	
	def onSourceEnterEditSignal( self, rowId ) :
		
		self.debugTrace("Source : onSourceEnterEditSignal")
				
		self.currentSourceId = rowId
		self.currentDocumentId = None	
									
		self.onListDocument()
		
		self.setTabsHeaderTextDocument()						
		
		self.setWindowTitle()				

	# Après selection d'une ligne	
	def onSourceSelectionSignal( self, rowId) :		
		
		self.debugTrace("Source : select line rowId=" + str(rowId))
				
		self.currentSourceId = rowId
		self.currentDocumentId = None	
									
		self.onListDocument()
		
		self.setTabsHeaderTextDocument()						
		
		self.setWindowTitle()	


	# ------------------------------------------------------------------
	# ---------------------------- Document ----------------------------
 	# ------------------------------------------------------------------
 	
	# Après affichage liste	
	@pyqtSlot()
	def onListDocument( self ) :
		
		myEvent_SourceObj = Event_sourcesDb ( self.db )
		event_sources = myEvent_SourceObj.selectById( "where event_id = ? and source_id = ?" , [ self.currentEventId, self.currentSourceId ] )		
		
		document_id_list = []	
		if event_sources : 
			for event_source in event_sources : 
				if type(event_source.getDocument_id()) == int :
					document_id_list.append(event_source.getDocument_id())						
					
		self.myDocumentListWidget.loadData( "where a.id in ("+self.getSqlWherePlaceHolder( len(document_id_list) )+")" , document_id_list)
						
		self.clearLayout(self.tabDocumentlayout)					
		self.tabDocumentlayout.addWidget(self.myDocumentListWidget)	
								
	# Après affichage fiche edition	
	@pyqtSlot( int, str )
	def onEditDocument(self, rowId, message ):
		
		rowDataObj = self.myDocumentListWidget.getRowData( rowId )

		self.myDocumentEditWidget = DocumentEditWidget( self.db, \
													self.rootMedia, \
													rowId, \
													None, \
													self.myDocumentListWidget.getIdList(), \
													self.myDocumentListWidget.getSqlOrderByField(), \
													self.myDocumentListWidget.getSqlOrderByOrder(), \
													rowDataObj )		
				
		self.myDocumentEditWidget.enterEditSignal.connect( self.onDocumentEnterEditSignal )				
		self.myDocumentEditWidget.validEditSignal.connect( self.onDocumentValidEditSignal )	
		self.myDocumentEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myDocumentEditWidget.closeEdit.connect( self.onDocumentCloseEditSignal )		
		
		self.clearLayout(self.tabDocumentlayout)		
		self.tabDocumentlayout.addWidget(self.myDocumentEditWidget)		

	# Après validation des données fiche		
	def onDocumentValidEditSignal( self, validState, rowId ) :

		self.debugTrace("Document : onDocumentValidEditSignal validState=" + str(validState) + ", rowId="+str(rowId))
		
		if validState is True :

			myEvent_SourceObj = Event_source( 	id = rowId,
												event_id = self.currentEventId, 
												source_id = self.currentSourceId, 
												document_id = self.currentDocumentId,
												datecreated = self.getCurentDateTime()    
											)
			Event_sourcesDb ( self.db ).update( myEvent_SourceObj)
			
			# Refresh datas
			# Liste des id (affichés) depuis la table d'association
			myEvent_SourceObj = Event_sourcesDb ( self.db )
			event_sources = myEvent_SourceObj.selectById( "where event_id = ? and source_id = ?" , [ self.currentEventId, self.currentSourceId ] )		
			
			document_id_list = []	
			if event_sources : 
				for event_source in event_sources : 
					if type(event_source.getDocument_id()) == int :
						document_id_list.append(event_source.getDocument_id())						
						
			self.myDocumentListWidget.loadData( "where a.id in ("+self.getSqlWherePlaceHolder( len(document_id_list) )+")" , document_id_list)
							
		self.setTabsHeaderTextDocument()

	# Après fermeture fenêtre edition
	def onDocumentCloseEditSignal(self, btnState, rowId ) :
		self.debugTrace("Document : onDocumentCloseEditSignal btnState=" + str(btnState))
		self.clearLayout(self.tabDocumentlayout)
		self.myDocumentListWidget.tableView.restoreState = False				
		self.tabDocumentlayout.addWidget(self.myDocumentListWidget)	
		self.myDocumentListWidget.selectRowById( self.currentDocumentId )		

	# Après suppression fiche
	def onDocumentRemoveRow( self, rowId ) :
		self.currentDocumentId = None
		self.onListDocument()	

		self.setTabsHeaderTextDocument()		

	# Après affichage d'une nouvelle fiche 	
	def onDocumentEnterEditSignal( self, rowId ) :
		self.currentDocumentId = rowId
		self.debugTrace("Document : onDocumentEnterEditSignal" )

	# Après selection d'une ligne
	def onDocumentSelectionSignal( self, rowId) :				
		self.debugTrace("Document : select line rowId=" + str(rowId))
		self.currentDocumentId = rowId



	# ------------------------------------------------------------------
	# ---------------------------- Médias ------------------------------
	# ------------------------------------------------------------------
	
	# Après affichage liste		
	@pyqtSlot()
	def onListMedia( self ) :
		
		self.debugTrace("Media : onListMedia" )
		
		self.myMedia_PersonObj = Media_personsDb ( self.db )
		media_persons = self.myMedia_PersonObj.selectById( "where person_id = ?" , [ self.currentPersonId ] )		
		
		media_id_list = []	
		if media_persons : 
			for media_person in media_persons : 
				media_id_list.append(media_person.getMedia_id())									

		self.myMediaListWidget.loadData( "where a.id in ("+self.getSqlWherePlaceHolder( len(media_id_list) )+")", media_id_list )
						
		self.clearLayout(self.tabMedialayout)					
		self.tabMedialayout.addWidget(self.myMediaListWidget)	

	# Après affichage fiche edition	
	@pyqtSlot( int, str )
	def onEditMedia(self, rowId, message ):
		
		self.debugTrace("Media : onEditMedia" )
		
		rowDataObj = self.myMediaListWidget.getRowData( rowId )

		self.myMediaEditWidget = MediaEditWidget( self.db, \
													self.rootMedia, \
													rowId, \
													None, \
													self.myMediaListWidget.getIdList(), \
													self.myMediaListWidget.getSqlOrderByField(), \
													self.myMediaListWidget.getSqlOrderByOrder(), \
													rowDataObj )	
		
		self.myMediaEditWidget.enterEditSignal.connect( self.onMediaEnterEditSignal )				
		self.myMediaEditWidget.validEditSignal.connect( self.onMediaValidEditSignal )
		self.myMediaEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myMediaEditWidget.closeEdit.connect( self.onMediaCloseEditSignal )		
		
		self.clearLayout(self.tabMedialayout)		
		self.tabMedialayout.addWidget(self.myMediaEditWidget)

	# Après validation des données fiche	
	def onMediaValidEditSignal( self, validState, rowId ) :
		
		self.debugTrace("Media : onMedia Valid EditSignal" )
	
		if validState is True :	
			myMedia_PersonObj = Media_person( 	id = rowId,
									media_id = self.currentMediaId, 
									person_id = self.currentPersonId, 
									datecreated = self.getCurentDateTime()    
								)
			Media_personsDb ( self.db ).update( myMedia_PersonObj )			
	
			self.myMedia_PersonObj = Media_personsDb ( self.db )
			media_persons = self.myMedia_PersonObj.selectById( "where person_id = ?" , [ self.currentPersonId ] )		
			
			media_id_list = []	
			if media_persons : 
				for media_person in media_persons : 
					media_id_list.append(media_person.getMedia_id())									

			self.myMediaListWidget.loadData( "where a.id in ("+self.getSqlWherePlaceHolder( len(media_id_list) )+")", media_id_list )
		
		self.setTabsHeaderTextMedia()		
		
	# Après fermeture fenêtre edition
	def onMediaCloseEditSignal(self, btnState, rowId ) :
		self.debugTrace("Media : onMediaCloseEditSignal btnState=" + str(btnState))
		self.clearLayout(self.tabMedialayout)	
		self.myMediaListWidget.tableView.restoreState = False										
		self.tabMedialayout.addWidget(self.myMediaListWidget)	
		self.myMediaListWidget.selectRowById( self.currentMediaId )		

	# Après suppression fiche
	def onMediaRemoveRow( self, rowId ) :
		self.debugTrace("Media : onMediaRemoveRow" )
		self.currentMediaId = None
		self.onListMedia()
		
		self.setTabsHeaderTextMedia()
		
	# Après affichage d'une nouvelle fiche 	
	def onMediaEnterEditSignal( self, rowId ) :
		self.currentMediaId = rowId
		self.debugTrace("Media : onMediaEnterEditSignal" )
				
	# Après selection d'une ligne
	def onMediaSelectionSignal( self, rowId) :				
		self.debugTrace("Media : select line rowId=" + str(rowId))
		self.currentMediaId = rowId		
		
	# ------------------------------------------------------------------
	# ---------------------------- Utils ------------------------------
	# ------------------------------------------------------------------

	'''
		Retourne la date et heure système pour maj Sql
	'''
	def getCurentDateTime(self) :
		
		from datetime import datetime
		
		now = datetime.now()	
		dt_string = now.strftime("%Y-%m-%d %H:%M")		
		return dt_string

	'''
		Retourne une chaine contenant autant de '?' que de valeurs dans <where_id_list>
		Exemple : si <where_id_list> = [1,8,21] retourne : '?, ?, ?'
	'''
	def getSqlWherePlaceHolder( self , where_id_list) :		

		place_holder_str = ""
		place_holder_sep = ""
		for place_holder in range( 0 , where_id_list)  :
			place_holder_str = place_holder_str + place_holder_sep + "?"
			place_holder_sep = ", "		
			
		return place_holder_str
