#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		{{title}}

        Aide

        {{datetime}}

        {{author}}

        {{licence}}

'''
import os
import sys
import linecache

from PyQt5.QtWidgets import QFileDialog, QWidget, QLabel, QVBoxLayout
from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

from lib.util import Util
import lib.setting as AppSetting

from libdb.event_source import Event_source, Event_sourcesDb


class DbMetaInformation( QWidget ) :

	db = None
	lasterror = ""
	selectCount = -1

	log = None

	def __init__(self, db, cbPrivate ):

		AppLog.log( LOG_INFO , "DbMetaInformation init" )	
		
		self.db = db

		if cbPrivate is True :
			self.cbPrivate = 1
		else :
			self.cbPrivate = 0		

	def PrintException(self):
		
		exc_type, exc_obj, tb = sys.exc_info()
		f = tb.tb_frame
		lineno = tb.tb_lineno
		if tb.tb_next:
			tb_next = tb.tb_next
			while tb_next: # loop until the value is 'None'
			  lineno = tb_next.tb_lineno
			  tb_next = tb_next.tb_next
		filename = f.f_code.co_filename
		linecache.checkcache(filename)
		line = linecache.getline(filename, lineno, f.f_globals)
		AppLog.log( LOG_ERROR , 'EXCEPTION IN ({}, LINE {} "{}"): {}'.format(filename, lineno, line.strip(), exc_obj) )	


	def selectDateCreatedMax( self ) :

		sqlSt="""
			SELECT max(datecreated) as 'meta' FROM 
			(
				select max(datecreated) as datecreated  from Document
				UNION
				select max(datecreated) as datecreated  from Event
				UNION
				select max(datecreated) as datecreated  from Event_source
				UNION
				select max(datecreated) as datecreated  from Media
				UNION
				select max(datecreated) as datecreated  from Media_person
				UNION
				select max(datecreated) as datecreated  from Person
				UNION
				select max(datecreated) as datecreated  from Place
				UNION
				select max(datecreated) as datecreated  from Source
			)
				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
	

	def selectIndiCount( self ) :

		sqlSt="""
				select  count(*) as meta
				from person p
				where (( p.private is null or p.private = 0 ) or p.private = ? )				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
 
	def selectIndiMax( self ) :

		sqlSt="""
				select  count(*) as meta
				from person p				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False


	def selectEventCount( self ) :

		sqlSt="""
				select  count(*) as meta
				from event p
				where (( p.private is null or p.private = 0 ) or p.private = ? )				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
 
	def selectEventMax( self ) :

		sqlSt="""
				select  count(*) as meta
				from event p				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False


	def selectSourceCount( self ) :

		sqlSt="""
				select  count(*) as meta
				from source p
				where (( p.private is null or p.private = 0 ) or p.private = ? )				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
 
	def selectSourceMax( self ) :

		sqlSt="""
				select  count(*) as meta
				from source p				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False


	def selectDocumentCount( self ) :

		sqlSt="""
				select  count(*) as meta
				from document p
				where (( p.private is null or p.private = 0 ) or p.private = ? )				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
 
	def selectDocumentMax( self ) :

		sqlSt="""
				select  count(*) as meta
				from document p				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False


	def selectMediaCount( self ) :

		sqlSt="""
				select  count(*) as meta
				from media p
				where (( p.private is null or p.private = 0 ) or p.private = ? )				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
 
	def selectMediaMax( self ) :

		sqlSt="""
				select  count(*) as meta
				from media p				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False


	def selectEventDateMin( self ) :
		sqlSt="""
				select min(date_begin) as meta
				from event p				
				where (( p.private is null or p.private = 0 ) or p.private = ? )					
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
				
	def selectEventDateMax( self ) :
		
		sqlSt="""
				select max(date_begin) as meta
				from event p				
				where (( p.private is null or p.private = 0 ) or p.private = ? )				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False		
		
			
	def selectFamillyCount( self ) :
		
		sqlSt="""
				select count(*) as 'meta'				
				from event e , eventtype et
				left outer join person p1 on p1.id = e.person_1_id
				left outer join person p2 on p2.id = e.person_2_id
				where et.id = e.eventtype_id
				and et.category = 'FAM0'
				and (( e.private is null or e.private = 0 ) or e.private = ? )				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False					



	# Evénements sans date (ou 01-01-001 00:00:)
	def selectEventsWithoutDate( self ) :
		
		sqlSt="""
				select e.id, e.name, e.date_begin, e.datecreated, e.notesearch from Event e
				where e.date_begin is NULL
				or e.date_begin =''
				or e.date_begin = "0001-01-01 00:00"			
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			lineno = 0
			table = ""
			if	self.selectCount > 0 :
				for line in resultlist :
					lineno = lineno  + 1
					table = table + '<tr>'
					table = table + '<td>'
					table = table + str(lineno)
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['id'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['name'])
					table = table + '</td>'					
					table = table + '<td>'
					table = table + str(line['date_begin'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['datecreated'])
					table = table + '</td>'
					table = table + '<td>'
					if line['notesearch'] is None : line['notesearch']=""
					table = table + str(line['notesearch'])
					table = table + '</td>'
					table = table + '</tr>'
				return lineno, table
			else :
				return lineno, table

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False		



	# Individus sans événements
	'''

	'''
	def selectPersonWithoutEvent( self ) :
		
		sqlSt="""
				select 
				p.id, p.name, p.firstname, p.lastname, p.datecreated
				from person p 
				where 
					p.id not in ( select e.person_1_id from event e ) 
				and p.id not in ( select e.person_2_id from event e ) 
				and p.id not in ( select e.person_3_id from event e )  			
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			lineno = 0
			table = ""
			if	self.selectCount > 0 :
				for line in resultlist :
					lineno = lineno  + 1
					table = table + '<tr>'
					table = table + '<td>'
					table = table + str(lineno)
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['id'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['name'])
					table = table + '</td>'					
					table = table + '<td>'
					table = table + str(line['firstname'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['lasname'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['datecreated'])
					table = table + '</td>'
					table = table + '</tr>'
				return lineno, table
			else :
				return lineno, table

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False	



	# Evénements sans sources
	'''

	'''
	def selectEventsWithoutSource( self ) :
		
		sqlSt="""
				select e.id, e.name, e.date_begin, e.datecreated, e.notesearch from Event e
				where e.id not in (
					SELECT distinct
					es.event_id
					from event_source es					
				)			
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			lineno = 0
			table = ""
			if	self.selectCount > 0 :
				for line in resultlist :
					lineno = lineno  + 1
					table = table + '<tr>'
					table = table + '<td>'
					table = table + str(lineno)
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['id'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['name'])
					table = table + '</td>'					
					table = table + '<td>'
					table = table + str(line['date_begin'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['datecreated'])
					table = table + '</td>'
					table = table + '<td>'
					if line['notesearch'] is None : line['notesearch']=""
					table = table + str(line['notesearch'])
					table = table + '</td>'
					table = table + '</tr>'
				return lineno, table
			else :
				return lineno, table

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False	
		
		
		# Sources sans événements
		'''
		'''
	def selectSourcesWithoutEvent( self ) :
		
		sqlSt="""
				select s.id, s.name, s.localurl, s.datecreated from Source s
				where s.id not in (
					SELECT distinct
					es.source_id
					from event_source es					
				)			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			lineno = 0
			table = ""			
			if	self.selectCount > 0 :
				for line in resultlist :
					lineno = lineno  + 1
					table = table + '<tr>'
					table = table + '<td>'
					table = table + str(lineno)
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['id'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['name'])
					table = table + '</td>'					
					table = table + '<td>'
					table = table + str(line['localurl'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['datecreated'])
					table = table + '</td>'
					table = table + '</tr>'
				return lineno, table
			else :
				return lineno, table

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
		
		
		# Media sans lien individu 
		'''
		'''
	def selectMediasWithoutPerson( self ) :
		
		sqlSt="""
				select m.id, m.name, m.url, m.datecreated from Media m
				where m.id not in (
					SELECT distinct
					mp.media_id
					from media_person mp					
				)			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [] )
			self.selectCount = len(resultlist)
			lineno = 0
			table = ""			
			if	self.selectCount > 0 :
				for line in resultlist :
					lineno = lineno  + 1
					table = table + '<tr>'
					table = table + '<td>'
					table = table + str(lineno)
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['id'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['name'])
					table = table + '</td>'					
					table = table + '<td>'
					table = table + str(line['url'])
					table = table + '</td>'
					table = table + '<td>'
					table = table + str(line['datecreated'])
					table = table + '</td>'
					table = table + '</tr>'
				return lineno, table
			else :
				return lineno, table

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False		
		
	# Media fichiers non référencés en base		
	def getDiskOrphanMedias( self, start_path = '.') :
		
		mediaList=[]
		mediaTable=""
		
		total_file = 0
		total_file_orphan = 0
		total_file_skip = 0
		for dirpath, dirnames, filenames in os.walk(start_path):
			for f in filenames:				
				fp = os.path.join(dirpath, f)
				# skip if it is symbolic link
				if not os.path.islink(fp) :	
					filename, file_extension = os.path.splitext(f) 
					if file_extension not in ('.shp') :				
						total_file = total_file +1
						mediaList.append(os.path.join(dirpath, f))
					else :
						total_file_skip = total_file_skip + 1
						
		for mediaUrl in mediaList :
			
			mediaUrlCount = self.selectCountMedia( mediaUrl[len(start_path):] )
			if mediaUrlCount == 0 :
				
				from pathlib import Path
				mediaAbsoluteUrl = str(Path(mediaUrl).absolute())
				
				total_file_orphan = total_file_orphan + 1
				mediaTable = mediaTable + "<tr>"
				mediaTable = mediaTable + "<td>"
				mediaTable = mediaTable + str( total_file_orphan )
				mediaTable = mediaTable + "</td>"				
				mediaTable = mediaTable + "<td>"
				mediaTable = mediaTable + os.path.dirname(mediaUrl[len(start_path):])
				mediaTable = mediaTable + "</td>"

				mediaTable = mediaTable + "<td>"
				mediaTable = mediaTable + '<img style="width:64px;height:auto"src="'+mediaAbsoluteUrl+'" >'
				mediaTable = mediaTable + "</td>"

				mediaTable = mediaTable + "<td>"
				mediaTable = mediaTable + '<a title="'+mediaAbsoluteUrl+'" href="'+mediaAbsoluteUrl+'">'+os.path.basename(mediaUrl[len(start_path):])+'</a>'				
				mediaTable = mediaTable + "</td>"
				mediaTable = mediaTable + "</tr>"
						
		return total_file, total_file_skip, total_file_orphan, mediaTable
		
	def selectCountMedia( self, mediaurl ) :
		
		sqlSt="""
			select sum("count") as 'meta' FROM
			(
					select 'media', count(*) as 'count' from media
					where url like ?
				union
					select 'source', count(*) as 'count'from source
					where 
					localurl like ?
					or
					sourceurl like ?
					or
					icon = ?
				union
					select 'person', count(*) as 'count' from person
					where icon like ?
				union
					select 'event', count(*) as 'count' from event
					where icon like ?
				union
					select 'document', count(*) as 'count' from document
					where icon like ?
			)				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			mediaurl = '%' + str(mediaurl) + '%'
			resultlist = self.db.query( sqlSt, [mediaurl,mediaurl,mediaurl,mediaurl,mediaurl,mediaurl,mediaurl] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['meta']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False	


	# Medias fichiers référencés en base, inexistants sur le disque
	def getOrphanMedias( self ,start_path = '.') :
		
		total_file = 0
		total_file_orphan = 0
		mediaTable=""
		
		listMedia = self.selectAllMedia()
		if listMedia : 
			for media in listMedia :
				
				# Split comma, loop each, check on disk
				for fileNameUrl in  media['url'].split(',') :
					
					total_file = total_file + 1
					foundOnDisk = False
					fullPathName = os.path.join( start_path, fileNameUrl)
					
					if os.path.exists(fullPathName) :
						foundOnDisk = True
					
					if foundOnDisk is False : 
					
						total_file_orphan = total_file_orphan + 1
						mediaTable = mediaTable + "<tr>"
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( total_file_orphan )
						mediaTable = mediaTable + "</td>"									
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( media['origin'] )
						mediaTable = mediaTable + "</td>"
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( media['id'] )
						mediaTable = mediaTable + "</td>"
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( media['name'] )
						mediaTable = mediaTable + "</td>"
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( fileNameUrl )
						mediaTable = mediaTable + "</td>"
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( media['datecreated'] )
						mediaTable = mediaTable + "</td>"
						mediaTable = mediaTable + "</tr>"
						
		return total_file, total_file_orphan, mediaTable


	def selectAllMedia( self ) :

		sqlSt="""
			select * FROM
			(
					select 'Media' as origin, id, name, url, datecreated  from media
				union
					select 'Source (image)' as origin, id, name, localurl, datecreated from source
				union
					select 'Source (icône)' as origin, id, name, icon, datecreated from source		
				union
					select 'Individu (icône)' as origin, id, name, icon, datecreated from person
				union
					select 'Evénement (icône)' as origin, id, name, icon, datecreated from event
				union
					select 'Document (icône)' as origin, id, name, icon, datecreated from document
			)
			where url is not NULL and url <> ""				
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
		


	'''
		Notes de recherche
	'''
	def getAllNoteSearch( self ) :
		
		total_note = 0
		mediaTable=""
		
		listeNote = self.selectAllNotesearch()
		for notesearch in listeNote :

			total_note = total_note + 1
			
			mediaTable = mediaTable + "<tr>"
			mediaTable = mediaTable + "<td>"
			mediaTable = mediaTable + str( total_note )
			mediaTable = mediaTable + "</td>"									
			mediaTable = mediaTable + "<td>"
			mediaTable = mediaTable + str( notesearch['id'] )
			mediaTable = mediaTable + "</td>"
			mediaTable = mediaTable + "<td>"
			mediaTable = mediaTable + str( notesearch['type'] )
			mediaTable = mediaTable + "</td>"
			mediaTable = mediaTable + "<td>"
			mediaTable = mediaTable + str( notesearch['name'] )
			mediaTable = mediaTable + "</td>"
			mediaTable = mediaTable + "<td>"
			if notesearch['notesearch'] is None : notesearch['notesearch']=""
			mediaTable = mediaTable + str( notesearch['notesearch'] )
			mediaTable = mediaTable + "</td>"
			mediaTable = mediaTable + "<td>"
			mediaTable = mediaTable + str( notesearch['datecreated'] )
			mediaTable = mediaTable + "</td>"
			mediaTable = mediaTable + "</tr>"
					
		return total_note, mediaTable	
	
	def selectAllNotesearch( self ) :

		sqlSt="""
				select id, 'Individu' as type, name, notesearch, datecreated from person where notesearch is not null and notesearch != ''
				union 
				select id, 'Média' as type, name, notesearch, datecreated from media where notesearch is not null and notesearch != ''
				union 
				select id, 'Source' as type, name, notesearch, datecreated from source where notesearch is not null and notesearch != ''
				union 
				select id, 'Evénement' as type, name, notesearch, datecreated from event where notesearch is not null and notesearch != ''
				union 
				select id, 'Document' as type, name, notesearch, datecreated from document where notesearch  is not null and notesearch != ''			
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return []

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
			

	# Icones fichiers référencés en base, inexistants sur le disque
	def getOrphanIcon( self ,start_path = '.') :
		
		total_file = 0
		total_file_orphan = 0
		mediaTable=""
		
		listMedia = self.selectAllIcon()
		
		if listMedia is None or listMedia is False : return total_file, total_file_orphan, mediaTable
		
		for media in listMedia :
			
			# Split comma, loop each, check on disk
			for fileNameUrl in  media['icon'].split(',') :
				
				total_file = total_file + 1
				foundOnDisk = False
				fullPathName = os.path.join( start_path, fileNameUrl)
				
				if os.path.exists(fullPathName) :
					foundOnDisk = True
				
				if foundOnDisk is False : 
				
					total_file_orphan = total_file_orphan + 1
					mediaTable = mediaTable + "<tr>"
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( total_file_orphan )
					mediaTable = mediaTable + "</td>"									
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( media['origin'] )
					mediaTable = mediaTable + "</td>"
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( media['id'] )
					mediaTable = mediaTable + "</td>"
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( media['name'] )
					mediaTable = mediaTable + "</td>"
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( fileNameUrl )
					mediaTable = mediaTable + "</td>"
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( media['datecreated'] )
					mediaTable = mediaTable + "</td>"
					mediaTable = mediaTable + "</tr>"
						
		return total_file, total_file_orphan, mediaTable


	def selectAllIcon( self ) :

		sqlSt="""
			select * FROM
			(
				select 'Person' as origin, p.id, p.name, p.icon, p.datecreated from person p where p.icon is not null and p.icon != ''
				union
				select 'Event', e.id, e.name, e.icon, e.datecreated from event e where e.icon is not null and e.icon != ''
				union
				select 'Source', s.id, s.name, s.icon, s.datecreated from source s where s.icon is not null and s.icon != ''
				union
				select 'Document', d.id, d.name, d.icon, d.datecreated from document d where d.icon is not null and d.icon != ''
				union
				select 'Depot', d.id, d.name, d.icon, Null from depot d where d.icon is not null and d.icon != ''
				union
				select 'Depot', d.id, d.name, d.icon, Null from documenttype d where d.icon is not null and d.icon != ''
				union
				select 'Event', e.id, e.name, e.icon, Null from eventtype e where e.icon is not null and e.icon != ''
				union
				select 'Place', p.id, p.name, p.icon, p.datecreated from place p where p.icon is not null and p.icon != ''
			)			
			order by origin, id			
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
		
		
	# Soures url référencés en base, inexistants sur le disque
	def getOrphanSourceUrl( self ,start_path = '.') :
		
		total_file = 0
		total_file_orphan = 0
		mediaTable=""
		
		listMedia = self.selectAllSourceUrl()
		if listMedia : 
			for media in listMedia :
				
				# Part 1/2
				# --------
				
				# Split comma, loop each, check on disk
				fileNameIndex = 0
				for fileNameUrl in  media['localurl'].split(',') :
					
					if fileNameUrl == '' : continue
					
					total_file = total_file + 1
					foundOnDisk = False
					
					fileNameIndex = fileNameIndex + 1
					fullPathName = os.path.join( start_path, fileNameUrl)
					
					if os.path.exists(fullPathName) :
						foundOnDisk = True
					
					if foundOnDisk is False : 
					
						total_file_orphan = total_file_orphan + 1
						mediaTable = mediaTable + "<tr>"
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( total_file_orphan )
						mediaTable = mediaTable + "</td>"									
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( media['id'] )
						mediaTable = mediaTable + "</td>"
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( media['name'] )
						mediaTable = mediaTable + "</td>"
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( fileNameIndex )
						mediaTable = mediaTable + "</td>"
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( fileNameUrl )
						mediaTable = mediaTable + "</td>"
						mediaTable = mediaTable + "<td>"
						mediaTable = mediaTable + str( media['datecreated'] )
						mediaTable = mediaTable + "</td>"
						mediaTable = mediaTable + "</tr>"

				# Part 2/2
				# --------
				fileNameUrl = media['sourceurl']
				if fileNameUrl == '' : continue
				if fileNameUrl is None : continue
				if fileNameUrl[0:3] == 'http'[0:3] : continue
				
				total_file = total_file + 1
				foundOnDisk = False
				
				fileNameIndex = fileNameIndex + 1
				fullPathName = os.path.join( start_path, fileNameUrl)
				
				if os.path.exists(fullPathName) :
					foundOnDisk = True
				
				if foundOnDisk is False : 
				
					total_file_orphan = total_file_orphan + 1
					mediaTable = mediaTable + "<tr>"
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( total_file_orphan )
					mediaTable = mediaTable + "</td>"									
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( media['id'] )
					mediaTable = mediaTable + "</td>"
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( media['name'] )
					mediaTable = mediaTable + "</td>"
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( fileNameIndex )
					mediaTable = mediaTable + "</td>"
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( fileNameUrl )
					mediaTable = mediaTable + "</td>"
					mediaTable = mediaTable + "<td>"
					mediaTable = mediaTable + str( media['datecreated'] )
					mediaTable = mediaTable + "</td>"
					mediaTable = mediaTable + "</tr>"


						
		return total_file, total_file_orphan, mediaTable		
		
	def selectAllSourceUrl( self ) :

		sqlSt="""
			select * FROM
			(
				select s.id, s.name, s.localurl, s.sourceurl, s.datecreated
				from source s
			)			
			where
				( localurl != '' or sourceurl != '' )
			order by id			
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False		

