#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Classe des requettes SQL pour generation de fichier DOT

'''

import sys
import linecache
from lib.sqlitedb import Sqlitedb

from PyQt5.QtWidgets import QFileDialog, QMessageBox, QWidget, QLabel, QVBoxLayout
from PyQt5.QtWidgets import QDialog, QGridLayout, QCheckBox, QDialogButtonBox
from PyQt5.QtCore import Qt

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

from lib.util import Util
import lib.setting as AppSetting                   
from libdb.metadata import Metadata, MetadatasDb		                    

class Dotgraph() :
	
	db = None
	lasterror = ""
	errorLog =  []
	selectCount = -1

	log = None

	def __init__(self , db, cbPrivate ):
		
		AppLog.log( LOG_INFO , "Dotgraph init" )	
		
		self.db = db

		if cbPrivate is True :
			self.cbPrivate = 1
		else :
			self.cbPrivate = 0	

		self.errorLog = []	

	def PrintException(self):
		exc_type, exc_obj, tb = sys.exc_info()
		f = tb.tb_frame
		lineno = tb.tb_lineno
		filename = f.f_code.co_filename
		linecache.checkcache(filename)
		line = linecache.getline(filename, lineno, f.f_globals)
		AppLog.log( LOG_ERROR , 'EXCEPTION IN ({}, LINE {} "{}"): {}'.format(filename, lineno, line.strip(), exc_obj) )	
		self.errorLog.append('EXCEPTION IN ({}, LINE {} "{}"): {}'.format(filename, lineno, line.strip(), exc_obj))

	def setPrivate( self, private ) :
		if private :
			self.private = 2
		else :
			self.private = 0

	def echoFile( self, fp, Dotdata ) :		
	
		fp.write( str(Dotdata) )
		fp.write("\n")
	
	# Liste des mariages des individus avec id des parents
	def selectMariageTree( self) :

		sqlSt="""
			select
			e.id as mariage_id,
			e.name as event_name,
			et.name as eventtype_name,			
			--          Conjoint :
			e.person_1_id as conjoint_id,
			(
				select 
				count(*) +1 as conjoint_mariage_count
				from event i
				join eventtype ty on i.eventtype_id = ty.id
				where ty.category in ('FAM0') -- Mariage
				and i.person_1_id = e.person_1_id and i.id < e.id			
			) as conjoint_mariage_count,			
			(
				select
				p.person_3_id as pere_id
				from event p
				join eventtype et on p.eventtype_id = et.id
				where et.category in ('FAM2') -- Naissance
				and (
					p.person_1_id = e.person_1_id
				)
			) as conjoint_pere_id,
			(
				select
				p.person_2_id as mere_id
				from event p
				join eventtype et on p.eventtype_id = et.id
				where et.category in ('FAM2') -- Naissance
				and (
					p.person_1_id = e.person_1_id
				)
			) as conjoint_mere_id,

			(	
				select
				q.id as mariage_id
				from event q
				join eventtype et on q.eventtype_id = et.id
				where et.category in ('FAM0') -- Mariages
				and (
					--q.person_1_id = 16 -- conjoint_pere_id
					q.person_1_id = (			
							(
								select
								p.person_3_id as pere_id
								from event p
								join eventtype et on p.eventtype_id = et.id
								where et.category in ('FAM2') -- Naissance
								and (
									p.person_1_id = e.person_1_id
								)
							)
						)		
					or
					-- q.person_2_id = 17 -- conjoint_mere_id
					q.person_2_id = (			
							(
								select
								p.person_2_id as mere_id
								from event p
								join eventtype et on p.eventtype_id = et.id
								where et.category in ('FAM2') -- Naissance
								and (
									p.person_1_id = e.person_1_id
								)
							)
					)
				)
			) as conjoint_parent_mariage_id,

			--         Conjointe :
			e.person_2_id as conjointe_id,
			(
				select 
				count(*) +1 as conjoint_mariage_count
				from event i
				join eventtype ty on i.eventtype_id = ty.id
				where ty.category in ('FAM0') -- Mariage
				and i.person_2_id = e.person_1_id and i.id < e.id			
			) as conjointe_mariage_count,			
			(
				select
				p.person_3_id as pere_id
				from event p
				join eventtype et on p.eventtype_id = et.id
				where et.category in ('FAM2') -- Naissance
				and (
					p.person_1_id = e.person_2_id
				)
			) as conjointe_pere_id,
			(
				select
				p.person_2_id as mere_id
				from event p
				join eventtype et on p.eventtype_id = et.id
				where et.category in ('FAM2') -- Naissance
				and (
					p.person_1_id = e.person_2_id
				)
			) as conjointe_mere_id,
			(	
				select
				q.id as mariage_id
				from event q
				join eventtype et on q.eventtype_id = et.id
				where et.category in ('FAM0') -- Mariages
				and (
					--q.person_1_id = 18 -- conjointe_pere_id
					q.person_1_id = (			
							(
								select
								p.person_3_id as pere_id
								from event p
								join eventtype et on p.eventtype_id = et.id
								where et.category in ('FAM2') -- Naissance
								and (
									p.person_1_id = e.person_2_id
								)
							)
						)		
					or
					-- q.person_2_id = 19 -- conjointe_mere_id
					q.person_2_id = (			
							(
								select
								p.person_2_id as mere_id
								from event p
								join eventtype et on p.eventtype_id = et.id
								where et.category in ('FAM2') -- Naissance
								and (
									p.person_1_id = e.person_2_id
								)
							)
					)
				)
			) as conjointe_parent_mariage_id

			from event e
			join eventtype et on e.eventtype_id = et.id
			where et.category in ('FAM0') -- Mariages
			and (( e.private is null or e.private = 0 ) or e.private = ? )
			order by e.person_1_id, e.date_begin desc
			"""
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [ self.cbPrivate ]  )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False	

	def selectIndiDetById( self, person_id ) :	
	
		sqlSt = """	

				select p.id as 'id',
				( 
				 select m.person_3_id 
				 from event m 
				 join eventtype y on m.eventtype_id = y.id and y.category in ('FAM2')
				 where m.person_1_id = p.id 
				) as pere_id,
				( 
				 select m.person_2_id 
				 from event m 
				 join eventtype y on m.eventtype_id = y.id and y.category in ('FAM2')
				 where m.person_1_id = p.id 
				) as mere_id,				
				( 
				 select m.id 
				 from event m 
				 join eventtype y on m.eventtype_id = y.id and y.category in ('FAM0')
				 where m.person_1_id = p.id or person_2_id = p.id
				) as mariage_id,
				
			(	
				select
				q.id as mariage_id
				from event q
				join eventtype et on q.eventtype_id = et.id
				where et.category in ('FAM0') -- Mariages
				and (
					--q.person_1_id = 16 -- conjoint_pere_id
					q.person_1_id = (			
							(
								select
								p.person_3_id as pere_id
								from event p
								join eventtype et on p.eventtype_id = et.id
								where et.category in ('FAM2') -- Naissance
								and (
									p.person_1_id = e.person_1_id
								)
							)
						)		
					or
					-- q.person_2_id = 17 -- conjoint_mere_id
					q.person_2_id = (			
							(
								select
								p.person_2_id as mere_id
								from event p
								join eventtype et on p.eventtype_id = et.id
								where et.category in ('FAM2') -- Naissance
								and (
									p.person_1_id = e.person_1_id
								)
							)
					)
				)
			) as parent_mariage_id,				
								
				p.key,				
				p.name as 'name',				
				p.lastname as 'lastname',  -- Nom de famille
				p.firstname as 'firstname', -- Prénom
				p.firstnameprefix,
				p.lastnamesuffix,
				p.surname,				
				case
					when  p.firtsnameothers is null 
					then p.firstname || ' ' || p.lastname 
					else p.firstname || ', ' || p.firtsnameothers || ' ' || p.lastname 
				end as 'nameformated',
				p.firtsnameothers,					
				p.sex as 'sex',
				p.icon as 'icon',
				p.description,	
				CASE 
					WHEN e.date_begin != '0001-01-01 00:00' THEN
						CASE			
							WHEN length(SUBSTR(e.date_begin, 12,2)) == 2 THEN
								strftime('%Y-%m-%d %H:%M',e.date_begin) 
							ELSE
								strftime('%Y-%m-%d',e.date_begin) 
						END			
					ELSE '' 
				END as date_birth,	
				cast(strftime('%d', e.date_begin) as integer) as birth_day,
				cast(strftime('%m', e.date_begin) as integer) as birth_month,
				cast(strftime('%Y', e.date_begin) as integer) as birth_year,	
				e.date_begin_accurate as 'date_birth_accurate',
				e.date_begin_format as 'date_birth_format',
				
				CASE 
					WHEN f.date_begin != '0001-01-01 00:00' THEN
						CASE			
							WHEN length(SUBSTR(f.date_begin , 12,2)) == 2 THEN
								strftime('%Y-%m-%d %H:%M',f.date_begin ) 
							ELSE
								strftime('%Y-%m-%d',f.date_begin ) 
						END			
					ELSE '' 
				END as date_death,		
				cast(strftime('%d', f.date_begin) as integer) as death_day,
				cast(strftime('%m', f.date_begin) as integer) as death_month,
				cast(strftime('%Y', f.date_begin) as integer) as death_year,
				f.date_begin_accurate as 'date_death_accurate',
				f.date_begin_format as 'date_death_format',

				a.name as 'birth_place_name',
				a.accurate as 'birth_place_accurate',
				a.zipcode as 'birth_place_zipcode',
				a.gpslat as 'birth_place_gpslat',
				a.gpslon as 'birth_place_gpslon',			
				a.zoom as 'birth_place_zoom',	
				a.icon as 'birth_place_icon'				
				
				from person p
				left join event e on e.person_1_id = p.id and e.eventtype_id in ( select et.id from eventtype et where et.category = 'FAM2' )
				left join event f on f.person_1_id = p.id and f.eventtype_id in ( select et.id from eventtype et where et.code = 'DEAT' )				
				left join place a on a.id = e.place_id	
				where p.id = ?
				and (( p.private is null or p.private = 0 ) or p.private = ? )
				
				"""
				
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id, self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist 
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False				

	def selectIndiDetByParentId( self, indiId ) :
		
		sqlSt = """	
			select * from (
				select p.id as 'id',
				( 
				 select m.person_3_id 
				 from event m 
				 join eventtype y on m.eventtype_id = y.id and y.category in ('FAM2')
				 where m.person_1_id = p.id 
				) as pere_id,
				( 
				 select m.person_2_id 
				 from event m 
				 join eventtype y on m.eventtype_id = y.id and y.category in ('FAM2')
				 where m.person_1_id = p.id 
				) as mere_id,				
				( 
				 select m.id 
				 from event m 
				 join eventtype y on m.eventtype_id = y.id and y.category in ('FAM0')
				 where m.person_1_id = p.id or person_2_id = p.id
				) as mariage_id,
				
			(	
				select
				q.id as mariage_id
				from event q
				join eventtype et on q.eventtype_id = et.id
				where et.category in ('FAM0') -- Mariages
				and (
					--q.person_1_id = 16 -- conjoint_pere_id
					q.person_1_id = (			
							(
								select
								p.person_3_id as pere_id
								from event p
								join eventtype et on p.eventtype_id = et.id
								where et.category in ('FAM2') -- Naissance
								and (
									p.person_1_id = e.person_1_id
								)
							)
						)		
					or
					-- q.person_2_id = 17 -- conjoint_mere_id
					q.person_2_id = (			
							(
								select
								p.person_2_id as mere_id
								from event p
								join eventtype et on p.eventtype_id = et.id
								where et.category in ('FAM2') -- Naissance
								and (
									p.person_1_id = e.person_1_id
								)
							)
					)
				)
			) as parent_mariage_id,				
								
				p.key,				
				p.name as 'name',				
				p.lastname as 'lastname',  -- Nom de famille
				p.firstname as 'firstname', -- Prénom
				p.firstnameprefix,
				p.lastnamesuffix,
				p.surname,				
				p.sex as 'sex',
				p.icon as 'icon',
				p.description,	
				CASE 
					WHEN e.date_begin != '0001-01-01 00:00' THEN
						CASE			
							WHEN length(SUBSTR(e.date_begin, 12,2)) == 2 THEN
								strftime('%Y-%m-%d %H:%M',e.date_begin) 
							ELSE
								strftime('%Y-%m-%d',e.date_begin) 
						END			
					ELSE '' 
				END as date_birth,	
				cast(strftime('%d', e.date_begin) as integer) as birth_day,
				cast(strftime('%m', e.date_begin) as integer) as birth_month,
				cast(strftime('%Y', e.date_begin) as integer) as birth_year,	
				e.date_begin_accurate as 'date_birth_accurate',
				e.date_begin_format as 'date_birth_format',
				
				CASE 
					WHEN f.date_begin != '0001-01-01 00:00' THEN
						CASE			
							WHEN length(SUBSTR(f.date_begin , 12,2)) == 2 THEN
								strftime('%Y-%m-%d %H:%M',f.date_begin ) 
							ELSE
								strftime('%Y-%m-%d',f.date_begin ) 
						END			
					ELSE '' 
				END as date_death,		
				cast(strftime('%d', f.date_begin) as integer) as death_day,
				cast(strftime('%m', f.date_begin) as integer) as death_month,
				cast(strftime('%Y', f.date_begin) as integer) as death_year,
				f.date_begin_accurate as 'date_death_accurate',
				f.date_begin_format as 'date_death_format',

				a.name as 'birth_place_name',
				a.accurate as 'birth_place_accurate',
				a.zipcode as 'birth_place_zipcode',
				a.gpslat as 'birth_place_gpslat',
				a.gpslon as 'birth_place_gpslon',			
				a.zoom as 'birth_place_zoom',	
				a.icon as 'birth_place_icon'				
				
				from person p
				left join event e on e.person_1_id = p.id and e.eventtype_id in ( select et.id from eventtype et where et.category = 'FAM2' )
				left join event f on f.person_1_id = p.id and f.eventtype_id in ( select et.id from eventtype et where et.code = 'DEAT' )				
				left join place a on a.id = e.place_id					
				where (( p.private is null or p.private = 0 ) or p.private = ? )
			)
			where pere_id = ? or mere_id = ? 
				"""
				
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [ self.cbPrivate, indiId, indiId ] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist 
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False	

	def selectAllIndiDet( self) :	
	
		sqlSt = """	
				select p.id as 'id',
				( 
				 select m.person_3_id 
				 from event m 
				 join eventtype y on m.eventtype_id = y.id and y.category in ('FAM2')
				 where m.person_1_id = p.id 
				) as pere_id,
				( 
				 select m.person_2_id 
				 from event m 
				 join eventtype y on m.eventtype_id = y.id and y.category in ('FAM2')
				 where m.person_1_id = p.id 
				) as mere_id,				
				( 
				 select m.id 
				 from event m 
				 join eventtype y on m.eventtype_id = y.id and y.category in ('FAM0')
				 where m.person_1_id = p.id or person_2_id = p.id
				) as mariage_id,
				
			(	
				select
				q.id as mariage_id
				from event q
				join eventtype et on q.eventtype_id = et.id
				where et.category in ('FAM0') -- Mariages
				and (
					--q.person_1_id = 16 -- conjoint_pere_id
					q.person_1_id = (			
								 select r.person_3_id 
								 from event r
								 join eventtype y on r.eventtype_id = y.id and y.category in ('FAM2')
								 where r.person_1_id = p.id									
						)		
					and
					-- q.person_2_id = 17 -- conjoint_mere_id
					q.person_2_id = (			
							(
								 select t.person_2_id 
								 from event t
								 join eventtype y on t.eventtype_id = y.id and y.category in ('FAM2')
								 where t.person_1_id = p.id 
							)
					)
				)
			) as parent_mariage_id,				
								
				p.key,				
				p.name as 'name',				
				p.lastname as 'lastname',  -- Nom de famille
				p.firstname as 'firstname', -- Prénom
				p.firstnameprefix,
				p.lastnamesuffix,
				p.surname,		
				case
					when  p.firtsnameothers is null 
					then p.firstname || ' ' || p.lastname 
					else p.firstname || ', ' || p.firtsnameothers || ' ' || p.lastname 
				end as 'nameformated',
				p.firtsnameothers,	
				p.sex as 'sex',
				p.icon as 'icon',
				p.description,	
				CASE 
					WHEN e.date_begin != '0001-01-01 00:00' THEN
						CASE			
							WHEN length(SUBSTR(e.date_begin, 12,2)) == 2 THEN
								strftime('%Y-%m-%d %H:%M',e.date_begin) 
							ELSE
								strftime('%Y-%m-%d',e.date_begin) 
						END			
					ELSE '' 
				END as date_birth,	
				cast(strftime('%d', e.date_begin) as integer) as birth_day,
				cast(strftime('%m', e.date_begin) as integer) as birth_month,
				cast(strftime('%Y', e.date_begin) as integer) as birth_year,	
				e.date_begin_accurate as 'date_birth_accurate',
				e.date_begin_format as 'date_birth_format',
				
				CASE 
					WHEN f.date_begin != '0001-01-01 00:00' THEN
						CASE			
							WHEN length(SUBSTR(f.date_begin , 12,2)) == 2 THEN
								strftime('%Y-%m-%d %H:%M',f.date_begin ) 
							ELSE
								strftime('%Y-%m-%d',f.date_begin ) 
						END			
					ELSE '' 
				END as date_death,		
				cast(strftime('%d', f.date_begin) as integer) as death_day,
				cast(strftime('%m', f.date_begin) as integer) as death_month,
				cast(strftime('%Y', f.date_begin) as integer) as death_year,
				f.date_begin_accurate as 'date_death_accurate',
				f.date_begin_format as 'date_death_format',

				a.name as 'birth_place_name',
				a.accurate as 'birth_place_accurate',
				a.zipcode as 'birth_place_zipcode',
				a.gpslat as 'birth_place_gpslat',
				a.gpslon as 'birth_place_gpslon',			
				a.zoom as 'birth_place_zoom',	
				a.icon as 'birth_place_icon'				
				
				from person p
				left join event e on e.person_1_id = p.id and e.eventtype_id in ( select et.id from eventtype et where et.category = 'FAM2' )
				left join event f on f.person_1_id = p.id and f.eventtype_id in ( select et.id from eventtype et where et.code = 'DEAT' )				
				left join place a on a.id = e.place_id	
								
				where (( p.private is null or p.private = 0 ) or p.private = ? )
				
				"""
				
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [ self.cbPrivate ] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist 
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False				

	def selectIndiChildsById( self, person_id ) :

		sqlSt = """
			select 
			p.id,
			p.name,
			p.firstname,
			p.lastname,
			p.sex,
			p.icon,
			( select pb.name from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on pb.id = b.place_id where bt.category ='FAM2' and b.person_1_id = p.id) as birth_place,
			( select b.date_begin from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on pb.id = b.place_id where bt.category = 'FAM2' and b.person_1_id = p.id) as birth_date,
			( select cast(strftime('%d', b.date_begin) as integer) from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on pb.id = b.place_id where bt.category ='FAM2' and b.person_1_id = p.id ) as birth_day,
			( select cast(strftime('%m', b.date_begin) as integer) from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on pb.id = b.place_id where bt.category ='FAM2' and b.person_1_id = p.id ) as birth_month,
			( select cast(strftime ('%Y', b.date_begin) as integer) from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on b.place_id = p.id where bt.category ='FAM2' and b.person_1_id = p.id ) as birth_year,
			( select b.date_begin_accurate from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on b.place_id = p.id where bt.category ='FAM2' and b.person_1_id = p.id ) as birth_accurate,
			( select b.date_begin_format from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on b.place_id = p.id where bt.category ='FAM2' and b.person_1_id = p.id ) as birth_format,
			e.person_2_id as 'mere_id', 
			p2.name,
			e.person_3_id as 'pere_id',
			p3.name
			from event e, Eventtype et
			left join person p on p.id = person_1_id
			left join person p2 on p2.id = person_2_id
			left join person p3 on p3.id = person_3_id
			where e.eventtype_id = et.id
			and et.category = ('FAM2')
			and ( e.person_2_id = ? or e.person_3_id = ? )
			and e.person_1_id != ?		
			and (( e.private is null or e.private = 0 ) or e.private = ? )
			and (( p.private is null or p.private = 0 ) or p.private = ? )
			and (( p2.private is null or p2.private = 0 ) or p2.private = ? )
			and (( p3.private is null or p2.private = 0 ) or p3.private = ? )
			
		"""
		
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id, person_id, person_id, self.cbPrivate, self.cbPrivate, self.cbPrivate, self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist 
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False	

	def selectIndiAndParentById( self, person_id ) :

		sqlSt = """
		select 
			p.id, 
			p.name,
			p.firstname,
			p.lastname,
			p.sex,
			p.icon,
			( select pb.name from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on pb.id = b.place_id where bt.category ='FAM2' and b.person_1_id = p.id) as birth_place,
			( select b.date_begin from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on pb.id = b.place_id where bt.category = 'FAM2' and b.person_1_id = p.id) as birth_date,
			( select cast(strftime('%d', b.date_begin) as integer) from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on pb.id = b.place_id where bt.category ='FAM2' and b.person_1_id = p.id ) as birth_day,
			( select cast(strftime('%m', b.date_begin) as integer) from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on pb.id = b.place_id where bt.category ='FAM2' and b.person_1_id = p.id ) as birth_month,
			( select cast(strftime ('%Y', b.date_begin) as integer) from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on b.place_id = p.id where bt.category ='FAM2' and b.person_1_id = p.id ) as birth_year,
			( select b.date_begin_accurate from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on b.place_id = p.id where bt.category ='FAM2' and b.person_1_id = p.id ) as birth_accurate,
			( select b.date_begin_format from event b join eventtype bt on b.eventtype_id = bt.id left join place pb on b.place_id = p.id where bt.category ='FAM2' and b.person_1_id = p.id ) as birth_format,
			( select person_2_id from event e , eventtype et where e.eventtype_id = et.id and et.category in ('FAM2') and e.person_1_id = p.id ) as 'mere_id',
			( select person_3_id from event e , eventtype et where e.eventtype_id = et.id and et.category in ('FAM2') and e.person_1_id = p.id ) as 'pere_id'			
			from person p
			where 			
			p.id = ?		
			and (( p.private is null or p.private = 0 ) or p.private = ? )
		"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id, self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist 
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False	

	def getEmptyIndiAndParentById( self, person_id, sex, firstNameLib = "" ) :

		sqlSt = """
		select 
			' ' as id, 
			' ' as displayname,
			' ' as firstname,
			' ' as lastname,
			' ' as sex,
			' ' as icon,
			' ' as birth_place,
			' ' as birth_date,
			' ' as birth_day,
			' ' as birth_month,
			' ' as  birth_year,
			' ' as 'mere_id',
			' ' as 'pere_id'			
			from person p
			where 			
			p.id = ?
		"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				resultlist[0]['sex'] = sex
				resultlist[0]['firstname'] = firstNameLib				
				resultlist[0]['birth_month'] = 13
				return resultlist 
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False	

	def selectMariageByIndiIds( self, conjoint_id, conjointe_id ) :	

		sqlSt="""
					select
					e.id as id,			
					e.date_begin ,
					cast(strftime('%d', e.date_begin) as integer) as mar_day,
					cast(strftime('%m', e.date_begin) as integer) as mar_month,
					cast(strftime('%Y', e.date_begin) as integer) as mar_year,	
					e.date_begin_accurate as mar_accurate,
					e.date_begin_format as mar_format,					
					e.icon as mar_icon,
					i.id as i_id, i.sex as i_sex, i.firstname as i_firstname, i.lastname as i_lastname,
					m.id as m_id, m.sex as m_sex, m.firstname as m_firstname, m.lastname as m_lastname,
					f.id as f_id, f.sex as f_sex, f.firstname as f_firstname, f.lastname as f_lastname,
					et.code as et_code,
					et.name as et_name,
					p.name as mar_place,
					p.zipcode as mar_zipcode					
					from  event e
					join eventtype et on e.eventtype_id = et.id
					left join place p on p.id = e.place_id							
					left outer join person i on i.id = e.person_1_id
					left outer join person f on f.id = e.person_2_id			
					left outer join person m on m.id = e.person_3_id					
					where et.category = 'FAM0'
					and i.id = ? 
					and f.id = ? 
					and (( e.private is null or e.private = 0 ) or e.private = ? )
					and (( i.private is null or i.private = 0 ) or i.private = ? )
					and (( f.private is null or f.private = 0 ) or f.private = ? )
					and (( m.private is null or m.private = 0 ) or m.private = ? )

			"""
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [ conjoint_id, conjointe_id, self.cbPrivate, self.cbPrivate, self.cbPrivate, self.cbPrivate ]  )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False
	
	def selectIndiLMariageById( self, conjoint_id, conjointe_id ) :	

		sqlSt="""
					select
					e.id as id,			
					e.date_begin ,
					cast(strftime('%d', e.date_begin) as integer) as mar_day,
					cast(strftime('%m', e.date_begin) as integer) as mar_month,
					cast(strftime('%Y', e.date_begin) as integer) as mar_year,	
					e.date_begin_accurate as mar_accurate,
					e.date_begin_format as mar_format,					
					e.icon as mar_icon,
					i.id as i_id, i.sex as i_sex, i.firstname as i_firstname, i.lastname as i_lastname,
					m.id as m_id, m.sex as m_sex, m.firstname as m_firstname, m.lastname as m_lastname,
					f.id as f_id, f.sex as f_sex, f.firstname as f_firstname, f.lastname as f_lastname,
					et.code as et_code,
					et.name as et_name,
					p.name as mar_place,
					p.zipcode as mar_zipcode					
					from  event e
					join eventtype et on e.eventtype_id = et.id
					left join place p on p.id = e.place_id							
					left outer join person i on i.id = e.person_1_id
					left outer join person f on f.id = e.person_2_id			
					left outer join person m on m.id = e.person_3_id					
					where et.category = 'FAM0'
					and i.id = ?  -- conjoint
					and f.id = ?  -- conjointe	
					and (( e.private is null or e.private = 0 ) or e.private = ? )
					and (( i.private is null or i.private = 0 ) or i.private = ? )
					and (( f.private is null or f.private = 0 ) or f.private = ? )
					and (( m.private is null or m.private = 0 ) or m.private = ? )

			"""
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [ conjoint_id, conjointe_id, self.cbPrivate, self.cbPrivate, self.cbPrivate, self.cbPrivate ]  )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False
	
