#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		{{title}}

		Application entry

		{{datetime}}

		{{author}}

		{{licence}}

'''

import sys
import linecache
from lib.sqlitedb import Sqlitedb

#from lib.logmanager import LogManager,LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

from lib.util import Util
import lib.setting as AppSetting

# class gedcomParser :

class Gedcom() :

	db = None
	lasterror = ""
	errorLog =  []
	selectCount = -1
	

	outLineNumber = 0
 
	gedcomColors = {
		'linenumber' : '#ff33cc',
		'level' : '#db4dff',
		'id' : '#0020C2',
		'tag' : '#C35817',
		'data' : '#347C2C'
		}

	def __init__(self , db ):

		self.db = db
		print("INIT Class Gedcom")
		self.errorLog = []	

	def PrintException(self):
		exc_type, exc_obj, tb = sys.exc_info()
		f = tb.tb_frame
		lineno = tb.tb_lineno
		filename = f.f_code.co_filename
		linecache.checkcache(filename)
		line = linecache.getline(filename, lineno, f.f_globals)
		print ('EXCEPTION IN ({}, LINE {} "{}"): {}'.format(filename, lineno, line.strip(), exc_obj))
		self.errorLog.append('EXCEPTION IN ({}, LINE {} "{}"): {}'.format(filename, lineno, line.strip(), exc_obj))

	def setPrivate( self, private ) :
		if private :
			self.private = 1
		else :
			self.private = 0

	def selectMeta( self, code ) :

		sqlSt="""
				select
					a.id as 'id',
					a.code as 'code',
					a.name as 'name',
					a.value as 'value',
					a.type as 'type',
					a.description as 'description'
				from metadata a
				where a.code = ?
			"""
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [code] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['value']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectMetaByCode( self, code ) :

		sqlSt="""
				select
					a.id as 'id',
					a.code as 'code',
					a.name as 'name',
					a.value as 'value',
					a.type as 'type',
					a.description as 'description'
				from metadata a
				where a.code = ?
			"""
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [code] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectAllINDI( self ) :

		sqlSt="""
				select p.id as 'id',
				datecreated as 'date_created',
				p.lastname as 'lastname',  -- Nom de famille
				p.firstname as 'givenname', -- Prénom
				p.name as 'name',  -- Un mot ou une combinaison de mots utilisés pour aider à identifier un individu. Pour les femmes mariées, il s'agit du nom de jeune fille. Plus d'une ligne NOM devrait être utilisée pour les gens qui sont connus sous différents noms.
				p.firstnameprefix,
				p.lastnamesuffix,
				p.surname,
				p.firstname || ',' || p.lastname as 'name2' ,
				p.sex as 'sex'
				from person p
				where (( p.private is null or p.private = 0 ) or p.private = ? )
				order by p.lastname
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.private] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				self.lastError = "Liste individus vide."
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectIndiFAMS( self, person_id ) :

		sqlSt="""
				select
				e.id,
				et.code,
				e.date_begin as 'date',
				e.name,
				p1.id as 'wid',
				p1.name as 'Mere',
				p2.id as 'hid',
				p2.name as 'Pere'
				from event e , eventtype et
				left outer join person p1 on p1.id = e.person_1_id
				left outer join person p2 on p2.id = e.person_2_id
				where et.id = e.eventtype_id
				and (p1.id = ? or p2.id = ?)
				and e.eventtype_id in (select et.id from Eventtype et where et.category = 'FAM0' )
				and (( e.private is null or e.private = 0 ) or e.private = ? )
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id, person_id, self.private] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]['id']
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectIndiParents( self, person_id ) :

		sqlSt="""
				select
				e.person_1_id,
				p1.name,
				y.name as event_type_name,
				e.date_begin as event_date,
				e.datecreated as 'date_created',
				e.person_2_id,
				p2.name as Mère,
				e.person_3_id,
				p3.name as Père,
				pl.name as place_name
				from
				event e,
				person p1,
				eventtype y
				left outer join person p2 on e.person_2_id = p2.id
				left outer join person p3 on e.person_3_id = p3.id
				left outer join place pl on pl.id = e.place_id
				where y.id = e.eventtype_id
				and e.person_1_id = p1.id
				and e.eventtype_id in (select et.id from Eventtype et where et.code in( 'BIRT', 'ADOP' ))
				and e.person_1_id = ?
				order by e.date_begin
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectIndiEventPlace( self, person_id, event_code ) :

		sqlSt="""
				select p.id as 'id', p.city || ',' || ifnull(', ' || p.zipcode,'') || ',' || p.country as 'plac',
				p.gpslat, p.gpslon
				from event e, place p
				where
				e.place_id = p.id
				and e.person_1_id =  ?
				and e.eventtype_id = ( select et.id from Eventtype et where et.code = ? )
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id, event_code] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectIndiEventDate( self, person_id ) :

		sqlSt = """
			select e.id as 'id', et.code, e.date_begin as 'date_begin'
			from event e , eventtype et
			where
				e.eventtype_id = et.id
			and et.category = 'INDI'
			and e.person_1_id = ?
		"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist # {'date_begin' : resultlist[0]['date_begin'] }
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectEventDate( self, event_id, event_code ) :

		sqlSt="""
			select e.id as 'id', e.date_begin as 'date_begin'
			from event e
			where
			e.id = ?
			and e.eventtype_id = ( select et.id from Eventtype et where et.code = ? )
		"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [event_id, event_code] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				print(resultlist)
				return resultlist[0] # { 'date_begin' : resultlist[0]['date_begin'] }
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectEventPlace( self, event_id, event_code ) :

		sqlSt="""
				select p.id as 'id', p.city || ',' || ifnull(', ' || p.zipcode,'') || ',' || p.country as 'plac',
				p.gpslat, p.gpslon
				from event e, place p
				where
				e.place_id  = p.id
				and e.id =  ?
				and e.eventtype_id = ( select et.id from Eventtype  et where et.code = ? )
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		
		try :

			resultlist = self.db.query( sqlSt , [event_id, event_code] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectEventSource( self, event_id ) :

		sqlSt="""
				select
				s.id as 'id',
				s.quality  as 'quality',
				s.datecreated as 'date_created',
				s.name as 'name',
				s.page as 'page',
				s.citation as 'citation',
				s.description,
				s.quality as 'quality',
				d.id as 'document_id',
				d.name as 'document_name',
				e.id as 'envent_id',
				e.name as 'event_name'
				from event e, event_source es, source s
				left outer join document d on d.id = es.document_id
				where e.id = es.event_id
				and s.id = es.source_id
				and e.id = ?
				and (( e.private is null or e.private = 0 ) or e.private = ? )
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [event_id, self.private] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectAllFAM( self, event_category ) :

		sqlSt="""
				select
				e.id,
				e.datecreated,
				et.code,
				e.date_begin as 'date',
				e.name,
				p1.id as 'wid',
				p1.name as 'Mere',
				p2.id as 'hid',
				p2.name as 'Pere'
				from event e , eventtype et
				left outer join person p1 on p1.id = e.person_1_id
				left outer join person p2 on p2.id = e.person_2_id
				where et.id = e.eventtype_id
				and et.category = ?
				and (( e.private is null or e.private = 0 ) or e.private = ? )
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [event_category, self.private] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectAllFAM1( self, event_category , person_id ) :

		sqlSt="""
				select
				e.id,
				e.datecreated,
				et.code,
				e.date_begin as 'date',
				e.name,
				p1.id as 'wid',
				p1.name as 'Mere',
				p2.id as 'hid',
				p2.name as 'Pere'
				from event e , eventtype et
				left outer join person p1 on p1.id = e.person_1_id
				left outer join person p2 on p2.id = e.person_2_id
				where et.id = e.eventtype_id
				and ( p1.id = ? or p2.id = ?)
				and et.category = ?
				and (( e.private is null or e.private = 0 ) or e.private = ? )
		"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id, person_id, event_category, self.private] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectFAMCHILD( self, person_id ) :

		sqlSt="""
				select
				e.person_1_id as 'id',
				p1.name,
				y.name as event_type_name,
				e.date_begin as event_date,
				e.person_2_id,
				p2.name as Mère,
				e.person_3_id,
				p3.name as Père,
				pl.name as place_name
				from
				event e,
				person p1,
				eventtype y
				left outer join person p2 on e.person_2_id = p2.id
				left outer join person p3 on e.person_3_id = p3.id
				left outer join place pl on pl.id = e.place_id
				where y.id = e.eventtype_id
				and e.person_1_id = p1.id
				and e.eventtype_id in (select et.id from Eventtype et where et.code in ('BIRT', 'ADOP') )
				and ( e.person_3_id = ? or e.person_2_id = ?) -- Chercher père ou mère
				order by e.date_begin
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id, person_id] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectAllSOURCE(self) :

		sqlSt="""
			select distinct
			s.id as 'id', s.datecreated as 'date_created', 
			s.name as 'name',
			s.description,
			s.localurl as localurl,
			s.quality as 'quality',
			d.id as document_id,
			d.name as document_name,
			p.id as depot_id,
			p.name as depot_name
			from source s
			left outer join event_source es on  es.source_id = s.id
			left outer join document d on d.id = es.document_id
			left outer join depot p on p.id = d.depot_id
			where (( s.private is null or s.private = 0 ) or s.private = ? )
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.private] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False

	def selectAllREPO( self ) :

		sqlSt="""
				select d.id as 'id' , d.code, d.name as 'name', d.city, d.adress_1, d.adress_2, d.zip, d.country
				from depot d
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False


	def selectAllOBJ( self ) :

		sqlSt="""
			select
			d.id as 'id', d.name as 'name',
			d.description as 'description' ,
			d.url as 'url',
			t.code
			from document d, Documenttype t
			where d.documenttype_id = t.id
			and (( d.private is null or d.private = 0 ) or d.private = ? )
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [self.private] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False


	# indate : 1888-09-08 16:42     
	# return : 9 AUG 1642	
	def formatDate( self, indate ) :
		
		dateSep = "-"
		hourSep = " "
		
		months = {
			0:'', 1:'JAN', 2:'FEB', 3:'MAR', 4:'APR', 5:'MAY', 6:'JUN', 7:'JUL', 8:'AUG', 9:'SEP', 10:'OCT', 11:'NOV', 12:'DEC'
		};

		try:
			indatechunks = indate.split(hourSep)		
			datechunks = indatechunks[0].split(dateSep)	
			
			print("indate=", indate)
			print("indatechunks=", indatechunks)
			print("datechunks=", datechunks)

			year = str(datechunks[0])
			month = str(datechunks[1])
			day = str(datechunks[2])
				
			print("y,m,d = ", year," ", month," ", day)
			
			#return ( str(datechunks[0])+' '+months[(int)(datechunks[1])*1]+' '+datechunks[2])
			return ( day + ' '+months[(int)(month)*1] + ' '+year)

		except Exception as e:
			print(str(e))
			self.errorLog.append( "Exception formatdate : indate=[" + str(indate) +"] : " + str(e) )
			return ''
			

	def formatDateTime( self, indate ) :
		months = {
			0:'', 1:'JAN', 2:'FEB', 3:'MAR', 4:'APR', 5:'MAY', 6:'JUN', 7:'JUL', 8:'AUG', 9:'SEP', 10:'OCT', 11:'NOV', 12:'DEC'
		};

		try:
			
			indatechunks = indate.split(" ")		
			datechunks = indatechunks[0].split("/")		
			return ( str(datechunks[0])+' '+months[(int)(datechunks[1])*1]+' '+str(datechunks[2])+' '+str(indatechunks[1]))

		except Exception as e:
			self.errorLog.append( "Exception formatDateTime : indate=[" + str(indate) +"] : " + str(e) )
			print(str(e))
			return ''
			
	 # TIME HH:MM
	def formatHour( self, indate ) :
				
		datechunks = indate.split(" ") 
		try:
			if datechunks[1] : 
				return  datechunks[1]
		except Exception as e:
			self.errorLog.append( "Exception formatHour : indate=[" + str(indate) +"] : " + str(e) )
			
		return False

	def echoGedcom( self, fp, fpHtml, gedcomLevel, id, gedcomTag, gedcomData ) :
		
		self.echoGedcomFile( fp, gedcomLevel, id, gedcomTag, gedcomData ) 		
		self.echoGedcomHtml( fpHtml, gedcomLevel, id, gedcomTag, gedcomData ) 


	def echoGedcomFile( self, fp, gedcomLevel, id, gedcomTag, gedcomData ) :		
	
		if gedcomLevel is not None :
			fp.write( str(gedcomLevel) )
		
		if id is not None :
			fp.write( " ")
			fp.write( str(id) )	
		
		if gedcomTag is not None : 
			fp.write( " ")				
			fp.write( str(gedcomTag))
				
		if gedcomData is not None :
			fp.write( " ")
			fp.write( str(gedcomData))

		fp.write("\n")
						
	def echoGedcomHtml( self, fp, gedcomLevel, id, gedcomTag, gedcomData ) :

		if fp is None :
			return

		fp.write('<tr>')
		
		fp.write('<td>')
		fp.write('<span style="color:'+self.gedcomColors['linenumber']+'">')  
		fp.write('['+str(self.outLineNumber)+']')
		self.outLineNumber = self.outLineNumber + 1
		fp.write('</span>')
		fp.write('</td>')	
		
		fp.write('<td>')
		if gedcomLevel is not None :
			fp.write('<span style="color:'+self.gedcomColors['level'] +'">')  
			fp.write( str(gedcomLevel))
			fp.write('</span>')	
		
		fp.write('</td>')
		
		fp.write('<td>')
		if id is not None :
			fp.write('<span style="color:'+self.gedcomColors['id']+'">')  
			fp.write( str(id) )
			fp.write('</span>')			
		
		fp.write('</td>')		

		fp.write('<td>')	
		for i in range( 1, gedcomLevel) : #(i = gedcomLevel; i >0; i=i-1) 
			fp.write('&nbsp;&nbsp;')
		

		if gedcomTag is not None :
			fp.write('<span style="color:'+self.gedcomColors['tag']+'">')  
			fp.write( str(gedcomTag))
			fp.write('</span>')	

		
		fp.write('&nbsp;')

		if gedcomData is not None :
			fp.write('<span style="color:'+self.gedcomColors['data']+'">')  
			fp.write('</span>')	
			fp.write( str(gedcomData) )

		fp.write('</td>')
		
		fp.write('</tr>')

	def echoHtmlHead( self, fp, title ) :
		if fp is not None :
			fp.write('<html>')
			fp.write('<head>')
			fp.write('<title>')
			fp.write(title)
			fp.write('</title>')
			fp.write('</head>')
			fp.write('<body>')
			fp.write('<style> table#gedcomtable tr:nth-child(even) { background-color:#F0FFF0;  tr:nth-child(odd) { background-color:#EAF9EA; } </style>')
			fp.write('<div style="margin:auto;width:100%"><tt><table border=0 id="gedcomtable" style="margin-left: auto; margin-right: auto;">')

	def echoHtmlFooter( self, fp) :
		if fp is not None :
			fp.write('</body>')			
			fp.write('</table></tt></div>')
			fp.write('</html>')
		
	def fpHtmlClose( self, fp) : 
		if fp is not None :		
			fp.close()

