#!/usr/bin/env python3
# -*- coding: utf-8 -*-


'''

		{{title}}

        Application entry

        {{datetime}}

        {{author}}

        {{licence}}

'''

# http://easygensolutions.com/gedcom/gedcom.html

from PyQt5.QtWidgets import QFileDialog, QMessageBox, QWidget, QLabel, QVBoxLayout
from PyQt5.QtWidgets import QDialog, QGridLayout, QCheckBox, QDialogButtonBox
from PyQt5.QtCore import Qt

#from lib.logmanager import LogManager,LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

from lib.util import Util
import lib.setting as AppSetting
                    
from libdb.metadata import Metadata, MetadatasDb		                    
   
class ExportParametres(QDialog):
		
	cancel = False
		
	def __init__(self):

		QDialog.__init__(self)

		layout = QGridLayout()
		self.setLayout(layout)

		self.cbPrivate = QCheckBox('Inclure les données privées', self)
		self.cbPrivate.stateChanged.connect(self.onCbPrivate)
		layout.addWidget(self.cbPrivate, 0, 0)

		self.cbHtml = QCheckBox('Exporter au format Html', self)
		self.cbHtml.stateChanged.connect(self.onCbPrivate)
		layout.addWidget(self.cbHtml, 1, 0)

		# OK and Cancel buttons
		buttons = QDialogButtonBox(
			QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
			Qt.Horizontal, self)
		buttons.accepted.connect(self.onAccept)
		buttons.rejected.connect(self.onCancel)

		layout.addWidget(buttons)

		self.setWindowTitle("Options d'exportation")

	def onAccept( self) :
		self.accept()

	def onCancel( self) :
		self.cancel = True
		self.reject()

	def onCbPrivate(self, state):

		if state == Qt.Checked:
			pass
		else:
			pass
                        
class Gedcomexport( QWidget ):

	debugPrint = True
	
	statCountPerson = 0
	statCountEvent = 0
	statCountSource = 0
	statCountDocument = 0

	def __init__(self , parent, db ):
		
		print("INIT from Gedcomexport")
		
		super().__init__()
		self.db = db
		self.parent = parent		
		self.initUI()
		#self.main()
		
		
	def initUI(self):		
		pass
		
	def printDebug( self, *args ) :

		messagePrint=""
		for message in args:
			messagePrint = messagePrint + str(message)

		if self.debugPrint :
			print('Gedcomexport: ' , messagePrint)

					
	# Point d'entrée depuis le menu principal		
	def main( self ) :

		print("MAIN from Gedcomexport")	

		mediaInsertChoiceDlg = ExportParametres()
		mediaInsertChoiceDlg.exec()		
		if mediaInsertChoiceDlg.cancel : return
		
		if mediaInsertChoiceDlg.cbPrivate.isChecked() : 
			print("OPTION CHECKED")

		from libapp.gedcom import Gedcom			
		myGedcom = Gedcom( self.db )
		saveFile = myGedcom.selectMeta('GEDCOMFILE') 
		if saveFile is False : 
			saveFile = ""
			print("Pas de nom de fichier trouvé dans les paramètres GEDCOMFILE")
						
		gedfile = QFileDialog.getSaveFileName(self.parent, 
												"Exporter au format GED", 
												saveFile, 
												"Fichiers Gedcom (*.ged);;Tous (*)")		

		if gedfile[0]:
			self.saveDataAsGedFile( 
									gedfile[0], 
									myGedcom,
									mediaInsertChoiceDlg.cbPrivate.isChecked(),
									mediaInsertChoiceDlg.cbHtml.isChecked() 
									)

			myMetadatas = MetadatasDb( self.db )
			savedFile = myGedcom.selectMetaByCode('GEDCOMFILE') 
			myMetadata = Metadata( savedFile['id'], savedFile['name'], savedFile['code'], savedFile['value'], savedFile['type'], savedFile['description'] )
			myMetadata.setValue( gedfile[0] )
			myMetadatas.update( myMetadata )
			
			msg = QMessageBox()
			msg.setIcon(QMessageBox.Information)
			
			msg.setMinimumSize(1200, 0)

			msg.setWindowTitle("Exportation Gedcom")
			msg.setText("Export gedcom terminé : <b>" + str(gedfile[0]) + "</b>")
			msg.setInformativeText(
								"Nombre d'individus: " + str(self.statCountPerson) + "\n" + 
								"Nombre d'evenements: " + str(self.statCountEvent) + "\n" +
								"Nombre de sources: " + str(self.statCountSource) + "\n" +
								"Nombre de documents: " + str(self.statCountDocument) + "\n" + 
								"\n" + 
								"Nombre d'erreurs : " + str(len(myGedcom.errorLog))
								)
			
			msg.setDetailedText( "Log des erreurs : " + "\n" + "\n".join( myGedcom.errorLog) ) 
			msg.setStandardButtons(QMessageBox.Ok )

			retval = msg.exec_()
						
	def saveDataAsGedFile(self, gedfilename, myGedcom , cbPrivate, cbHtml)  :
			
		fp = open(gedfilename, 'w')
		
		if cbHtml : 
			fpHtml = open(gedfilename+'.html', 'w')
			myGedcom.echoHtmlHead( fpHtml, gedfilename )
		else :
			fpHtml = None
		
		myGedcom.setPrivate( cbPrivate )
			
		if myGedcom.selectMeta('GEDCOMDATE').strip() == '' : 
			from datetime import datetime
			today = datetime.now() # date.today()			
			dateNow = myGedcom.formatDate(today.strftime("%Y-%m-%d")) # .upper()
			hourNow = today.strftime("%H:%M")
		else :
			dateNow = myGedcom.formatDate(myGedcom.selectMeta('GEDCOMDATE'))
			hourNow = myGedcom.formatHour(myGedcom.selectMeta('GEDCOMDATE'))	

		myGedcom.echoGedcom( fp, fpHtml, 0, None,'HEAD', None )
		myGedcom.echoGedcom( fp, fpHtml, 1, None,'SOUR', myGedcom.selectMeta('GEDCOMSOUR') )
		myGedcom.echoGedcom( fp, fpHtml, 2, None,'VERS', myGedcom.selectMeta('GEDCOMSOURVERS') )
		myGedcom.echoGedcom( fp, fpHtml, 2, None,'NAME', myGedcom.selectMeta('GEDCOMSOURNAME') )
		myGedcom.echoGedcom( fp, fpHtml, 1, None,'DATE', dateNow )
		myGedcom.echoGedcom( fp, fpHtml, 2, None,'TIME', hourNow )
		myGedcom.echoGedcom( fp, fpHtml, 1, None,'CHAR', 'UTF-8' )
		myGedcom.echoGedcom( fp, fpHtml, 1, None,'FILE', gedfilename )	
		myGedcom.echoGedcom( fp, fpHtml, 1, None,'GEDC', None )
		myGedcom.echoGedcom( fp, fpHtml, 2, None,'FORM', 'LINEAGE-LINKED' )
		myGedcom.echoGedcom( fp, fpHtml, 2, None,'VERS', '5.5.1' )
		myGedcom.echoGedcom( fp, fpHtml, 1, None,'SUBM', '@SUBM@' ) 
		myGedcom.echoGedcom( fp, fpHtml, 0, '@SUBM@', 'SUBM', None )
		myGedcom.echoGedcom( fp, fpHtml, 1, None,'NAME', myGedcom.selectMeta('GEDCOMAUT') )
		
		# INDIVIDUS
		indiResults = myGedcom.selectAllINDI()
		if indiResults is False : 
			print( "Erreur : "+myGedcom.lasterror+"\n" )
		else :		
			self.statCountPerson = myGedcom.selectCount	
			for indival in indiResults :				
				myGedcom.echoGedcom( fp, fpHtml, 0, '@I'+str(indival['id'])+'@','INDI', None )
				myGedcom.echoGedcom( fp, fpHtml, 1, None,'NAME', indival['name'] ) 	 
				if indival['givenname'] : 
					myGedcom.echoGedcom( fp, fpHtml, 2, None,'GIVN', indival['givenname'] ) # Prénom
				if indival['lastname'] : 
					myGedcom.echoGedcom( fp, fpHtml, 2, None,'SURN', indival['lastname'] ) # Nom de famille
				if indival['surname'] : 
					myGedcom.echoGedcom( fp, fpHtml, 2, None,'NICK', indival['surname'] ) # surnom
				if indival['firstnameprefix'] : 
					myGedcom.echoGedcom( fp, fpHtml, 2, None,'NPFX', indival['firstnameprefix'] ) # Docteur, Général, Monseigneur
				if indival['lastnamesuffix'] : 
					myGedcom.echoGedcom( fp, fpHtml, 2, None,'NSFX', indival['lastnamesuffix'] ) # 	Jr

				myGedcom.echoGedcom( fp, fpHtml, 1, None,'SEX', indival['sex'] )

				indiEventResults = myGedcom.selectIndiEventDate( indival['id'] )	
				if indiEventResults : 
					self.statCountEvent += myGedcom.selectCount	
					for indiEvent in indiEventResults :
						if indiEvent['date_begin'] :
							myGedcom.echoGedcom( fp, fpHtml, 1, None, indiEvent['code'], None )
							myGedcom.echoGedcom( fp, fpHtml, 2, None,'DATE', myGedcom.formatDate(indiEvent['date_begin']))
							hour_begin = myGedcom.formatHour(indiEvent['date_begin'])

						if indiEvent['code'][0] != '_' :			
							eventPlace = myGedcom.selectIndiEventPlace( indival['id'], indiEvent['code'])	
							if eventPlace : 			
								myGedcom.echoGedcom( fp, fpHtml, 2, None,'PLAC', eventPlace['plac'] )		
								if eventPlace['gpslat'] and eventPlace['gpslon'] :
									myGedcom.echoGedcom( fp, fpHtml, 3, None,'MAP', None )	
									myGedcom.echoGedcom( fp, fpHtml, 4, None,'LATI', eventPlace['gpslat'] )	
									myGedcom.echoGedcom( fp, fpHtml, 4, None,'LONG', eventPlace['gpslon'] )				

						eventSources = myGedcom.selectEventSource( indiEvent['id'] )
						if eventSources :														
							for eventSourceValue in eventSources :
								myGedcom.echoGedcom( fp, fpHtml, 2, None, 'SOUR', '@S'+str(eventSourceValue['id'])+'@' )	
								
								if eventSourceValue['description'] :
									myGedcom.echoGedcom( fp, fpHtml, 3, None, 'DATA', None)	
									myGedcom.echoGedcom( fp, fpHtml, 4, None, 'TEXT', eventSourceValue['description'] )				

								if eventSourceValue['page'] :
									myGedcom.echoGedcom( fp, fpHtml, 3, None, 'PAGE', eventSourceValue['page'] )	

								if eventSourceValue['quality'] :
									myGedcom.echoGedcom( fp, fpHtml, 3, None, 'QUAY', eventSourceValue['quality'] )	
		
				
				indiFams = myGedcom.selectIndiFAMS( indival['id'] )	
				if indiFams: 
					myGedcom.echoGedcom( fp, fpHtml, 1, None, 'FAMS', '@F'+str(indiFams)+'@' ) # famille de son couple (pointeur vers famille @F3@)
					
				indiParentsId = myGedcom.selectIndiParents( indival['id'] )				
				if indiParentsId :
					indiMotherId = indiParentsId[0]['person_2_id']
					indiFams = myGedcom.selectIndiFAMS( indiMotherId)		
					if indiFams: 
						myGedcom.echoGedcom( fp, fpHtml, 1, None, 'FAMC', '@F'+str(indiFams)+'@' ) # famille de ses parents (pointeur vers famille @F1@)
				
				if indival['date_created'] :
					myGedcom.echoGedcom( fp, fpHtml, 1, None,'CHAN', None )
					myGedcom.echoGedcom( fp, fpHtml, 2, None,'DATE', myGedcom.formatDate(indival['date_created']))
			
		# FAMILLES
		fam0Results = myGedcom.selectAllFAM( 'FAM0') # Mariage, union ...
		if fam0Results is False :
			print("Erreur : "+myGedcom.lasterror+"\n")
		else :
			for famval in fam0Results : 			
				myGedcom.echoGedcom( fp, fpHtml, 0, '@F'+str(famval['id'])+'@', 'FAM', None )
				myGedcom.echoGedcom( fp, fpHtml, 1, None, 'HUSB', '@I'+str(famval['hid'])+'@' )
				myGedcom.echoGedcom( fp, fpHtml, 1, None, 'WIFE', '@I'+str(famval['wid'])+'@' )
				myGedcom.echoGedcom( fp, fpHtml, 1, None, famval['code'], None )  # MARR, UNION ...

				marDateEvent = myGedcom.selectEventDate( famval['id'], famval['code'] )
				if marDateEvent :
					dateEvent = myGedcom.formatDate(marDateEvent['date_begin'])
					if dateEvent != "" :
						myGedcom.echoGedcom( fp, fpHtml, 2, None,'DATE', dateEvent) # HOUR not allowed
				
				eventPlace = myGedcom.selectEventPlace( famval['id'], famval['code'] )
				if eventPlace :
					myGedcom.echoGedcom( fp, fpHtml, 2, None,'PLAC', eventPlace['plac'] )	
					if (eventPlace['gpslat'] and eventPlace['gpslon']) :
						myGedcom.echoGedcom( fp, fpHtml, 3, None,'MAP', None )	
						myGedcom.echoGedcom( fp, fpHtml, 4, None,'LATI', eventPlace['gpslat'] )	
						myGedcom.echoGedcom( fp, fpHtml, 4, None,'LONG', eventPlace['gpslon'] )				
				
				famchildResults = myGedcom.selectFAMCHILD( famval['wid'] ) # ou famval['hid']
				if famchildResults is False :
					print("Erreur : "+myGedcom.lasterror+"\n")
				else :
					for famchildval in famchildResults :
						myGedcom.echoGedcom( fp, fpHtml, 1, None, 'CHIL', '@I'+str(famchildval['id'])+'@' )				
				
				# Evenements suite au mariage, union : divorce, séparation ...
				fam1Results = myGedcom.selectAllFAM1( 'FAM1' , famval['wid'] ) # Mariage, union ...
				if fam1Results is False :
					print( "Erreur : "+myGedcom.lasterror+"\n")
				else :
					for fam1val in fam1Results :
						myGedcom.echoGedcom( fp, fpHtml, 1, None, fam1val['code'], None )  
						marDateEvent = myGedcom.selectEventDate( fam1val['id'], fam1val['code'] )
						if marDateEvent : 
							myGedcom.echoGedcom( fp, fpHtml, 2, None, 'DATE', myGedcom.formatDate(marDateEvent['date_begin'])) # HOUR not allowed
						
						eventPlace = myGedcom.selectEventPlace( fam1val['id'], fam1val['code'] )
						if eventPlace : 
							myGedcom.echoGedcom( fp, fpHtml, 2, None, 'PLAC', eventPlace['plac'] )	
							if eventPlace['gpslat'] and eventPlace['gpslon'] : 
								myGedcom.echoGedcom( fp, fpHtml, 3, None,'MAP', None)	
								myGedcom.echoGedcom( fp, fpHtml, 4, None,'LATI', eventPlace['gpslat'] )	
								myGedcom.echoGedcom( fp, fpHtml, 4, None,'LONG', eventPlace['gpslon'] )			
		
		# SOURCES
		sourceResults = myGedcom.selectAllSOURCE()
		if sourceResults is False : 
			print( "Erreur : "+myGedcom.lasterror+"\n")
		else :
			self.statCountSource = myGedcom.selectCount			
			for sourceval in sourceResults:
				myGedcom.echoGedcom( fp, fpHtml, 0, '@S'+str(sourceval['id'])+'@', 'SOUR', None )
				
				myGedcom.echoGedcom( fp, fpHtml, 1, None,  'TITL', sourceval['name'] )
				if sourceval['description'] :
					myGedcom.echoGedcom( fp, fpHtml, 1, None, 'TEXT', sourceval['description'] )
					
				if sourceval['date_created'] :
					myGedcom.echoGedcom( fp, fpHtml, 1, None,'CHAN', None )
					myGedcom.echoGedcom( fp, fpHtml, 2, None,'DATE', myGedcom.formatDate(sourceval['date_created']) )
					if myGedcom.formatHour(sourceval['date_created']) :
						myGedcom.echoGedcom( fp, fpHtml, 3, None,'TIME', myGedcom.formatHour(sourceval['date_created']) )
											
				if sourceval['depot_id'] :
					myGedcom.echoGedcom( fp, fpHtml, 1, None, 'REPO', '@R'+str(sourceval['depot_id'])+'@' )
				
		# MEDIAS / OBJECTS
		mediaResults = myGedcom.selectAllOBJ()
		if mediaResults is False : 
			print( "Erreur : "+myGedcom.lasterror+"\n")
		else : 
			self.statCountDocument = myGedcom.selectCount			
			for mediaval in mediaResults:
				myGedcom.echoGedcom( fp, fpHtml, 0, '@M'+str(mediaval['id'])+'@', 'OBJE', None )
				
				if mediaval['url'] :
					import os.path
					if os.path.basename(mediaval['url']) == '' : 
						mediaFile = mediaval['name']
					else :						
						mediaFile = os.path.basename(mediaval['url'])				
					myGedcom.echoGedcom( fp, fpHtml, 1, None, 'FILE', mediaFile )										
					myGedcom.echoGedcom( fp, fpHtml, 2, None, 'TITL', mediaval['name'] )						
					myGedcom.echoGedcom( fp, fpHtml, 2, None, 'FORM', mediaval['code'] )

				if mediaval['description'] :
					myGedcom.echoGedcom( fp, fpHtml, 1, None,'NOTE', mediaval['description'])

		# DEPOTS
		repoResults = myGedcom.selectAllREPO()		
		if repoResults is False : 
			print("Erreur : "+myGedcom.lasterror+"\n")
		else :
			for repoval in repoResults :
				myGedcom.echoGedcom( fp, fpHtml, 0, '@R'+str(repoval['id'])+'@', 'REPO', None )
				myGedcom.echoGedcom( fp, fpHtml, 1, None, 'NAME', repoval['name'] )
				if repoval['code'] : 
					myGedcom.echoGedcom( fp, fpHtml, 1, None, 'REFN', repoval['code'] )
				if repoval['adress_1']  :
					myGedcom.echoGedcom( fp, fpHtml, 1, None, 'ADDR', repoval['adress_1'] )
				if repoval['adress_2'] :
					myGedcom.echoGedcom( fp, fpHtml, 2, None, 'ADR1', repoval['adress_2'] )
				if repoval['zip'] :
					myGedcom.echoGedcom( fp, fpHtml, 2, None, 'POST', repoval['zip'] )
				if repoval['city'] : 
					myGedcom.echoGedcom( fp, fpHtml, 2, None, 'CITY', repoval['city'] )
				if repoval['country']: 
					myGedcom.echoGedcom( fp, fpHtml, 2, None, 'CTRY', repoval['country'] )	

		myGedcom.echoGedcom( fp, fpHtml, 0, 'TRLR', None, None )
			 
		myGedcom.echoHtmlFooter( fpHtml )
		myGedcom.fpHtmlClose( fpHtml )		
						
		fp.close()



