#!/usr/bin/env python3
# -*- coding: utf-8 -*-


'''
		Classe des requettes SQL d'extraion des données pour les exportations

'''

import sys
import linecache
from lib.sqlitedb import Sqlitedb

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

from lib.util import Util
import lib.setting as AppSetting

class Rapport() :

	db = None
	lasterror = ""
	selectCount = -1

	log = None

	def __init__(self , db, cbPrivate ):

		#self.log = LogManager( AppSetting.get('app/logfile', "defaultlog.txt") )
		AppLog.log( LOG_INFO , "Rapport init" )	
		
		self.db = db
		
		if cbPrivate is True :
			self.cbPrivate = 1
		else :
			self.cbPrivate = 0		

	def PrintException(self):
		exc_type, exc_obj, tb = sys.exc_info()
		f = tb.tb_frame
		lineno = tb.tb_lineno
		filename = f.f_code.co_filename
		linecache.checkcache(filename)
		line = linecache.getline(filename, lineno, f.f_globals)
		AppLog.log( LOG_ERROR , 'EXCEPTION IN ({}, LINE {} "{}"): {}'.format(filename, lineno, line.strip(), exc_obj) )	


	def selectINDIbyId( self , indiId ) :

		sqlSt="""
				select p.id as 'id',
				p.key,
				p.datecreated as 'date_created',
				p.lastname as 'lastname',  -- Nom de famille
				p.firstname as 'firstname', -- Prénom
				p.firstname || ',' || p.lastname as 'name2' ,
				p.name as 'name',  -- Un mot ou une combinaison de mots utilisés pour aider à identifier un individu. Pour les femmes mariées, il s'agit du nom de jeune fille. Plus d'une ligne NOM devrait être utilisée pour les gens qui sont connus sous différents noms.
				p.firstnameprefix,
				p.lastnamesuffix,
				p.surname,			
				case
					when  p.firtsnameothers is null 
					then p.firstname || ' ' || p.lastname 
					else p.firstname || ', ' || p.firtsnameothers || ' ' || p.lastname 
				end as 'nameformated',
				p.firtsnameothers,					
				p.sex as 'sex',
				p.icon as 'icon',
				p.description,	
				p.notesearch,		
				e.date_begin as 'e.datebirth',
				e.date_begin_accurate as 'e.datebirth_accurate',
				e.date_begin_format as 'e.datebirth_format',
				f.date_begin as 'f.datedeath'
				from person p
				left join event e on e.person_1_id = p.id and e.eventtype_id in ( select et.id from eventtype et where et.category = 'FAM2' )
				left join event f on f.person_1_id = p.id and f.eventtype_id in ( select et.id from eventtype et where et.code = 'DEAT' )
				where 
				p.id = ?
				and (( p.private is null or p.private = 0 ) or p.private = ? )
				order by e.date_begin, p.name
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [indiId, self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False

	def selectINDIAll( self ) :

		sqlSt="""
				select p.id as 'id',
				p.key,
				p.datecreated as 'date_created',
				p.lastname as 'lastname',  -- Nom de famille
				p.firstname as 'firstname', -- Prénom
				p.firstname || ',' || p.lastname as 'name2' ,
				p.name as 'name',  
				p.firstnameprefix,
				p.lastnamesuffix,
				p.surname,			
				case
					when  p.firtsnameothers is null 
					then p.firstname || ' ' || p.lastname 
					else p.firstname || ', ' || p.firtsnameothers || ' ' || p.lastname 
				end as 'nameformated',
				p.firtsnameothers,
				p.sex as 'sex',
				p.icon as 'icon',
				p.description,	
				p.notesearch,		
				e.date_begin as 'e.datebirth',
				e.date_begin_accurate as 'e.date_birth_accurate',
				e.date_begin_format as 'e.date_birth_format',
				f.date_begin as 'f.datedeath',
				f.date_begin_format as 'f.datedeath_format'
				from person p
				left join event e on e.person_1_id = p.id and e.eventtype_id in ( select et.id from eventtype et where et.category = 'FAM2' )
				left join event f on f.person_1_id = p.id and f.eventtype_id in ( select et.id from eventtype et where et.code = 'DEAT' )
				where (( p.private is null or p.private = 0 ) or p.private = ? )				
				order by e.date_begin, p.name
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			
			resultlist = self.db.query( sqlSt, [self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
		
		
	def selectEVENTSAll( self ) :
		sqlSt = """
		select 
			e.id as 'id',  
			e.person_1_id,
			e.name, 
			e.icon,		
			i.id as 'i.id',		
			i.name,
			i.firstname,
			i.lastname,
			i.sex,
			i.icon as 'i.icon',	
			e.person_2_id,
			m.private as "mother_private",	
			m.name as 'mother_displayname',
			m.firstname as 'mother_firstname',
			m.lastname as 'mother_lastname',
			e.person_3_id,
			f.private as "father_private",	
			f.name as 'father_displayname',
			f.firstname as 'father_firstname',
			f.lastname as 'father_lastname',
			CASE 
				WHEN e.date_begin != '0001-01-01 00:00' THEN
					CASE			
						WHEN length(SUBSTR(e.date_begin, 12,2)) == 2 THEN
							strftime('%Y-%m-%d %H:%M',e.date_begin) 
						ELSE
							strftime('%Y-%m-%d',e.date_begin) 
					END			
				ELSE '' 
			END as date_begin,		
			SUBSTR(e.date_begin, 1,4) as year_begin,
			e.date_begin_format as 'date_begin_format',		
			e.date_begin_accurate as 'date_begin_accurate',
			e.description as 'description',		
			e.notesearch,	
			et.code as 'et.code',
			et.name as 'et.name',
			p.name as 'p.name',
			p.accurate as 'p.accurate',
			p.zipcode as 'p.zipcode',			
			p.gpslat as 'p.gpslat',
			p.gpslon as 'p.gpslon',	
			p.zoom as 'p.zoom',		
			p.icon as 'p.icon'			
			from event e , eventtype et
			left join place p on p.id = e.place_id							
			left outer join person i on i.id = e.person_1_id
			left outer join person m on m.id = e.person_2_id			
			left outer join person f on f.id = e.person_3_id
			where
				e.eventtype_id = et.id			
			and p.gpslon != 0 and p.gpslat != 0 --> sans coord. impossible à placer sur une carte
			and (( e.private is null or e.private = 0 ) or e.private = ? )
			order by e.date_begin
		"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [ self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist 
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False		
		
	def selectEventsByPerson( self, person_id ) :

		sqlSt = """
		select 
			e.id as 'id',  
			e.person_1_id,
			e.name, 
			e.icon,					
			i.id as 'i.id',		
			i.name as 'indi_name',
			i.firstname as 'indi_firstname',
			i.lastname as 'indi_lastname',
			i.sex as 'i.sex',
			i.icon as 'i.icon',	
			e.person_2_id,
			m.private as "mother_private",			
			m.name as 'mother_displayname',
			m.firstname as 'mother_firstname',
			m.lastname as 'mother_lastname',
			m.sex as 'm.sex',
			m.icon as 'm.icon',
			e.person_3_id,
			f.private as "father_private",			
			f.name as 'father_displayname',
			f.firstname as 'father_firstname',
			f.lastname as 'father_lastname',
			f.sex as 'f.sex',
			f.icon as 'f.icon',
			CASE 
				WHEN e.date_begin != '0001-01-01 00:00' THEN
					CASE			
						WHEN length(SUBSTR(e.date_begin, 12,2)) == 2 THEN
							strftime('%Y-%m-%d %H:%M',e.date_begin) 
						ELSE
							strftime('%Y-%m-%d',e.date_begin) 
					END			
				ELSE '' 
			END as date_begin,		
			SUBSTR(e.date_begin, 1,4) as year_begin,		
			e.date_begin_accurate as 'date_begin_accurate',
			e.date_begin_format as 'date_begin_format',
			e.description as 'description',		
			e.notesearch,	
			et.code as 'et.code',
			et.name as 'et.name',
			et.category as 'et.category',
			p.name as 'p.name',
			p.accurate as 'p.accurate',
			p.zipcode as 'p.zipcode',
			p.gpslat as 'p.gpslat',
			p.gpslon as 'p.gpslon',			
			p.zoom as 'p.zoom',	
			p.icon as 'p.icon'			
			from event e , eventtype et
			left join place p on p.id = e.place_id							
			left outer join person i on i.id = e.person_1_id
			left outer join person m on m.id = e.person_2_id			
			left outer join person f on f.id = e.person_3_id
			where
				e.eventtype_id = et.id			
			and ( e.person_1_id = ? or e.person_2_id = ? or e.person_3_id = ? )
			and (( e.private is null or e.private = 0 ) or e.private = ? )
			order by e.sortorder, e.date_begin
		"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [person_id, person_id, person_id, self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist 
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False

		
				
	def selectSourcesByEvent( self, event_id ) :

		sqlSt = """
				select * from Source s
				where s.id in (
					SELECT distinct
					es.source_id
					from event_source es
					where es.event_id = ?
				)
				and (( s.private is null or s.private = 0 ) or s.private = ? )	
				order by s.sortorder, s.name	
		"""
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [event_id, self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False
		
	def selectDocumementsBySource( self, eventid, sourceid ) :

		sqlSt="""
				select 
				d.id,
				d.name,
				d.notesearch,
				t.name as 'typename',
				t.icon as 't.icon',
				p.name as 'depot',
				d.description,
				d.url,
				d.icon,
				d.cote,
				d.datecreated
				from document d
				left join depot p on p.id = d.depot_id
				left join documenttype t on t.id = d.documenttype_id
				where d.id  in (
					SELECT document_id FROM 
					event_source es
					where event_id = ? and source_id = ?
				)
				and (( d.private is null or d.private = 0 ) or d.private = ? )
				order by d.sortorder, d.name
			"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [eventid, sourceid, self.cbPrivate] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False		

	def selectIndiParents( self, person_id ):

		sqlSt = """
				select
				e.person_1_id as 'id',
				p1.name,
				y.name as event_type_name,
				e.date_begin as event_date,
				e.date_begin_accurate as 'date_begin_accurate',
				e.date_begin_format as 'date_begin_format',
				e.person_2_id,
				p2.name as mother_displayname,
				e.person_3_id,
				p3.name as father_displayname,
				pl.name as place_name
				from
				event e,
				person p1,
				eventtype y
				left outer join person p2 on e.person_2_id = p2.id
				left outer join person p3 on e.person_3_id = p3.id
				left outer join place pl on pl.id = e.place_id
				where y.id = e.eventtype_id
				and e.person_1_id = p1.id
				and e.eventtype_id in (select et.id from Eventtype et where et.category = 'FAM2' )
				and e.person_1_id = ?				
				and (( p1.private is null or p1.private = 0 ) or p1.private = ? )
				and (( e.private is null or e.private = 0 ) or e.private = ? )
				order by e.date_begin				
				"""

		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [person_id, self.cbPrivate, self.cbPrivate] )
			self.selectCount = len(resultlist)
			if self.selectCount > 0:
				return resultlist
			else:
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False


	def selectmediasByPerson(self, person_id ) :	

		sqlSt = """
			select 
			e.id as 'eventid',
			e.name as 'eventname',
			e.icon as 'eventicon',
			y.code as 'eventcode',
			d.name as 'depotname',
			i.id as 'personid',
			i.icon as 'personicon',
			i.name as 'persondisplayname',
			t.id as 'documenttypeid',
			t.name as 'documenttypename',
			l.name as 'place_name',
			l.city as 'place_city',
			l.zipcode as 'place_zipcode',
			l.accurate as 'place_accurate',
			l.gpslat as 'place_gpslat',
			l.gpslon as 'place_gpslon',
			l.zoom as 'place_zoom',
			m.id as 'id',
			m.name as 'displayname',
			m.url as 'url',
			m.datemedia as 'datemedia',
			m.datemedia_format as 'datemedia_format',
			m.datemedia_accurate as 'datemedia_accurate',
			m.description as 'description',
			m.dimension as 'dimension',
			m.origin as 'origin',
			m.keyword as 'keyword'
			from media m, media_person p, depot d, person i
			left join event e on e.id = m.event_id
			left join documenttype t on t.id = m.documenttype_id
			left join place l on l.id = m.place_id
			left join Eventtype y on y.id = e.eventtype_id
			where 
			m.id = p.media_id
			and i.id = p.person_id			
			and m.depot_id = d.id
			and p.person_id = ?
			and (( m.private is null or m.private = 0 ) or m.private = ? )				
			order by m.sortorder, m.name
				"""
		self.lastError = None
		self.selectCount = None
		resultlist = []
		
		try:
			resultlist = self.db.query( sqlSt, [person_id, self.cbPrivate] )
			self.selectCount = len(resultlist)
			if self.selectCount > 0:
				return resultlist
			else:
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return False

	def getMediaCountByPerson(self, person_id) :
		
		sqlSt = """
			select 
			count(*) as count
			from media m, media_person p
			left join event e on e.id = m.event_id
			where 
			m.id = p.media_id						
			and p.person_id = ?
			and (( m.private is null or m.private = 0 ) or m.private = ? )							
				"""
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt, [person_id, self.cbPrivate] )
			self.selectCount = len(resultlist)
			if self.selectCount > 0:
				return resultlist[0]['count']
			else:
				return 0

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()

		return 0
