#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

		Custom Class for table : metadata.

		07/05/2021 22:10

		lvardon@laposte.net - 2021

		Licence libre


'''
import sys
import linecache


from libdb.metadata import MetadatasDb, Metadata

class Metadatas ( MetadatasDb ):

	db = None
	lastError = None
	lastInsertId = None

	def __init__(self, db ):
		super().__init__( db )
		self.db = db		

	# Custom Sql
	
	def updateMetaValueByCode( self, code, value, name ) :
	
		self.lastError = None
		self.lastInsertId = None
		sqlSt = "UPDATE metadata set value = ?, name=? WHERE code = ?"

		try:
			metadataObj = Metadata()
			metadataObj.setCode( code )
			metadataObj.setValue( value )
			metadataObj.setName( name )			
			
			if metadataObj.id != -1 :
				
				result = self.db.execute(sqlSt,
					[ value, name, code ])

				if self.db.cursor.rowcount >= 1 :
					self.db.commit()
					self.lastInsertId = metadataObj.id
					return True
				else :
					
					result = self.insert ( metadataObj )
					if result is True :
						self.db.commit()
						return True
					else :
						self.lastError = str(self.db.lastError)
						self.dumpObj( metadataObj , self.lastError, sqlSt )
						return False
			else :
				result = self.insert ( metadataObj )
				if result is True :
					self.db.commit()
					return True
				else :
					self.lastError = str(self.db.lastError)
					self.dumpObj( metadataObj , self.lastError, sqlSt )
					return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.dumpObj( metadataObj , self.lastError, sqlSt )

		return False	
	
	def selectMetaByCode( self, code ) :

		sqlSt="""
				select
					a.id as 'id',
					a.code as 'code',
					a.name as 'name',
					a.value as 'value',
					a.type as 'type',
					a.description as 'description'
				from metadata a
				where a.code = ?
			"""
		self.lastError = None
		self.selectCount = None
		resultlist = []
		try:
			resultlist = self.db.query( sqlSt , [code] )
			self.selectCount = len(resultlist)
			if	self.selectCount > 0 :
				return resultlist[0]
			else :
				return False

		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
			self.errorLog.append( str(e) )

		return False	
	
	def selectCustom( self, rowPropertyId  ) :
		self.lastError = None
		self.lastInsertId = None
		self.selectCount = None
		resultlist = []
		try:
			resultrows = self.db.query( 'SELECT * FROM metadata where person_1_id = ?' , [ rowPropertyId ])
			self.selectCount = len(resultrows)
			if	self.selectCount > 0 :
				for dbvalue in resultrows:
					
					metadataObj = Metadata( dbvalue['id'], dbvalue['name'], dbvalue['code'], dbvalue['value'], dbvalue['type'], dbvalue['description']  )
					resultlist.append( metadataObj )

				return resultlist
			else :
				return False
		except Exception as e:
			self.lastError = str(e)
			self.PrintException()
		return False

	def PrintException(self):
		exc_type, exc_obj, tb = sys.exc_info()
		f = tb.tb_frame
		lineno = tb.tb_lineno
		filename = f.f_code.co_filename
		linecache.checkcache(filename)
		line = linecache.getline(filename, lineno, f.f_globals)
		print ('EXCEPTION IN ({}, LINE {} "{}"): {}'.format(filename, lineno, line.strip(), exc_obj))

	def dumpObj( self, eventObj , lastError, message ) :
		print("----------------------------------")
		print(message)
		print("LastError : " + str(lastError))

		print("Id = " + str(eventObj.id))
		print("  name = " + str( eventObj.name ))
		print("  code = " + str( eventObj.code ))
		print("  value = " + str( eventObj.value ))
		print("  type = " + str( eventObj.type ))
		print("  description = " + str( eventObj.description ))
		
