#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

		*** Source autogénéré *** 
		
		Pour la table : document.

		----------------------------
		Document.
		
		Contenu de la table Document.
		----------------------------		

		05/24/2023 14:31

		lvardon@laposte.net - 2021

		Licence libre

'''
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QAction, QMenu, QApplication, QWidget, QToolTip, QPushButton, QMessageBox, QFileDialog
from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, QHBoxLayout, QScrollArea
from PyQt5.QtWidgets import QLabel, QLineEdit, QTextEdit, QComboBox, QCheckBox, QDateTimeEdit
from PyQt5.QtGui import QCursor, QFont, QIcon
from PyQt5.QtCore import Qt, QEvent, pyqtSignal, pyqtSlot, QModelIndex, QDateTime, QMargins, QSettings
from PyQt5 import QtCore
		
from lib.sqlitedb import Sqlitedb
from libdbcustom.document import Document, Documents

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
from lib.util import Util
from libwidget.tableModel import TableModel, TableProxyModel
from libwidget.tableView import TableView,TableCellImageDelegate

import lib.setting as AppSetting
import lib.log as AppLog

class DocumentListWidget(QWidget):

	mainWindowEditMode =  pyqtSignal( int, str )
	mainWindowRemoveRow =  pyqtSignal( int, str )	
	mainWindowListMode = pyqtSignal()
	mainWindowUserMessage = pyqtSignal( int, str )
	cellClickSignal =  pyqtSignal( int )
	cellDoubleClickSignal =  pyqtSignal( int )
	selectionSignal = pyqtSignal( int )
	
	def __init__(self , db, rootMedia, sourceOrigin = "", where_clause = None, fields_values = None):
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia 
		self.sourceOrigin = sourceOrigin
		
		self.where_clause = where_clause
		self.fields_values = fields_values
		
		self.initUI()

		self.initDelegates(  self.tableView )
		
		iconW, iconH, rowHeight = self.getTableViewIniParams( "tablelistviewDocument" )
		self.tableView.verticalHeader().setDefaultSectionSize(int(rowHeight))

		try :
			self.iconImageDelegate.icoW = int(iconW) 
			self.iconImageDelegate.icoH = int(iconH)	
		except Exception as e:
			pass
		try :
			self.urlImageDelegate.icoW = int(iconW) 
			self.urlImageDelegate.icoH = int(iconH)					
		except Exception as e:
			pass
		
		self.setObjectName( "DocumentListWidget" )

	def initDelegates( self, thistableView ) :
				
		self.privateImageDelegate = TableCellImageDelegate(self, "document", {'0' : 'img/dataicons/empty.png', '1' : 'img/dataicons/private-yes.png', '2' : 'img/dataicons/private-yes.png'} , self.rootMedia, 26, 26)
		thistableView.setItemDelegateForColumn(3, self.privateImageDelegate)

		self.iconImageDelegate = TableCellImageDelegate(self, "document", None , self.rootMedia, 40, 40)
		thistableView.setItemDelegateForColumn(4, self.iconImageDelegate)

		
		thistableView.verticalHeader().setDefaultSectionSize(40)

	def initUI(self):
		
		self.layout = QVBoxLayout()
		self.setLayout(self.layout)
				
		self.proxyModel = TableProxyModel(self)
		
		self.proxyModel.setSortRole(Qt.UserRole + 2)
		
		self.tableView = TableView( self, "document", self.sourceOrigin )	
				
		self.loadData( self.where_clause , self.fields_values ) 
	
		self.tableView.setModel(self.proxyModel)
		
		self.tableView.resizeColumnsToContents()

		self.tableView.cellClickSignal.connect(self.onCellClic)
		self.tableView.cellDoubleClickSignal.connect(self.onCellDoubleClic)
	
		selection_model = self.tableView.selectionModel()
		selection_model.selectionChanged.connect(self.onSelectionChanged)      
		
		self.filterValue = QLineEdit()
		self.layout.addWidget(self.filterValue)
		self.filterValue.textChanged.connect(self.onfilterValueChanged)
		
		self.layout.addWidget(self.tableView)

	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
			

	def getTableViewIniParams ( self, tableViewName) :
		
		iniConfigFile = "config/app.ini"
		settings=QSettings(iniConfigFile, QSettings.IniFormat)

		rowHeight = 0
		if settings.value(tableViewName + '/rowHeight') is not None:			
			rowHeight = settings.value( tableViewName +'/rowHeight')						
			
		iconW = 0
		if settings.value( tableViewName + '/rowIconW') is not None:						
			iconW = settings.value( tableViewName +'/rowIconW')				
		
		iconH = 0			
		if settings.value( tableViewName +'/rowIconH') is not None:			
			iconH = settings.value( tableViewName +'/rowIconH')	

		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" rowHeight="+ str(rowHeight))
		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" iconW x IconH="+ str(iconW) + " x " + str(iconH))
		
		return iconW, iconH, rowHeight
		
	@pyqtSlot('QItemSelection', 'QItemSelection')
	def onSelectionChanged(self, selected, deselected):
		try :
			rowId =  selected.indexes()[0].data()
			self.selectionSignal.emit( rowId )
		except Exception as e:
			pass		
		
	def onfilterValueChanged(self , text) :
		self.proxyModel.setFilterValue( str(text) )		
		self.proxyModel.applyFilters()

	def onCellClic(self, rowId ):
		self.cellClickSignal.emit(rowId)

	def onCellDoubleClic(self, rowId ):
		self.cellDoubleClickSignal.emit(rowId)
		self.onEditRow(rowId)
		
	def reloadData(self) :
		self.loadData(self.where_clause, self.fields_values)		
		
	def loadData( self, where_clause , fields_values ) :
		
		self.where_clause = where_clause
		self.fields_values = fields_values
				
		dataSource = DocumentDataSources( self.db )
		dataRows, dataColumns, self.dataSqlNames, dataColumnsType, dataToolTips = dataSource.selectAll( self.where_clause, self.fields_values )
		
		self.model = TableModel("document", dataRows, dataColumns, dataColumnsType, dataToolTips)

		self.proxyModel.setSourceModel(self.model)
		
		self.tableView.resizeColumnsToContents()				
				
	def dumpDatas( self) :
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		for i in range(0, row) :
			self.printDebug("--- Row : ", i)
			for j in range(0, col) :
				self.printDebug( 
						self.model.data( self.model.index(i, j), Qt.UserRole + 1 ),
						" - ",
						self.model.data( self.model.index(i, j), Qt.UserRole ) 
					)

	# Nom de la colonne sur laquelle porte le tri 
	def getSqlOrderByField( self) :
		orderByFieldPos = self.tableView.hView.sortIndicatorSection()
		orderByField = self.dataSqlNames[orderByFieldPos]
		
		return orderByField
		
	# Index du sens de tri 'DESC' = 0 'ASC' = 1
	def getSqlOrderByOrder( self) :
		orderBy = self.tableView.hView.sortIndicatorOrder()
		return orderBy
		
		
	# Liste des id de la liste affichée
	def getIdList( self ) :
		
		idList = []
		
		indexId = 0 # id column is at pos 0
		
		row = self.proxyModel.rowCount()
		model = self.proxyModel
		
		for i in range(0, row) :			
			idList.append( model.data( model.index(i, indexId ), Qt.UserRole ) )
		
		return idList
			
	
	def selectRowById( self, rowId ) :

		if rowId is None :
			return
			
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel
		
		for i in range(0, row) :
			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :				
				self.tableView.selectRow( i )
				return

	def getRowData( self, rowId ) :

		myDocumentObj = Document( )

		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel

		for i in range(0, row) :
			#print('row:', i, " model rowid=", self.model.data( self.model.index(i, 0), Qt.UserRole ), ' == ',  rowId, ' **', self.model.data( self.model.index(i, 1), Qt.UserRole ) )

			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :

				myDocumentObj.setId( model.data( model.index(i, 0 ), Qt.UserRole  )  )
				myDocumentObj.setName( model.data( model.index(i, 1 ), Qt.UserRole  )  )
				myDocumentObj.setDescription( model.data( model.index(i, 2 ), Qt.UserRole  )  )
				myDocumentObj.setPrivate( model.data( model.index(i, 3 ), Qt.UserRole  )  )
				myDocumentObj.setIcon( model.data( model.index(i, 4 ), Qt.UserRole  )  )
				myDocumentObj.setUrl( model.data( model.index(i, 5 ), Qt.UserRole  )  )
				myDocumentObj.setCote( model.data( model.index(i, 6 ), Qt.UserRole  )  )
				myDocumentObj.setNotesearch( model.data( model.index(i, 7 ), Qt.UserRole  )  )
				myDocumentObj.setSortorder( model.data( model.index(i, 8 ), Qt.UserRole  )  )
				myDocumentObj.setDepot_id( model.data( model.index(i, 9 ), Qt.UserRole  )  )
				myDocumentObj.setDocumenttype_id( model.data( model.index(i, 10 ), Qt.UserRole  )  )
				myDocumentObj.setDatecreated( model.data( model.index(i, 11 ), Qt.UserRole  )  )

				return myDocumentObj

		return myDocumentObj

	def contextMenuEvent(self, event):

		gp = event.globalPos()
		vp_pos = self.tableView.viewport().mapFromGlobal(gp)
		rowNum = self.tableView.rowAt(vp_pos.y())
		colNum = self.tableView.columnAt(event.x())

		self.menu = QMenu(self)
			
		if rowNum >= 0 :

			index = self.tableView.indexAt(vp_pos)
			rowId = self.proxyModel.index(index.row(),0).data()

			cellValue = self.proxyModel.index(rowNum, colNum).data()

			deleteAction = QAction( QIcon('img/context_del.png'), 'Supprimer', self)
			deleteAction.triggered.connect(lambda : self.onDeleteRow(rowId, rowNum))
			self.menu.addAction(deleteAction)

			editAction = QAction( QIcon('img/context_edit.png'), 'Editer', self)
			editAction.triggered.connect(lambda : self.onEditRow(rowId))
			self.menu.addAction(editAction)

		newAction = QAction(QIcon('img/context_add.png'), 'Nouveau', self)
		newAction.triggered.connect(lambda checked : self.onNewRow())
		self.menu.addAction(newAction)			

		self.menu.addSeparator()
		
		storeStateAction = QAction(QIcon('img/context_save.png'), 'Enregistrer la disposition', self)
		storeStateAction.triggered.connect(lambda checked : self.onStoreState())
		self.menu.addAction(storeStateAction)

		self.menu.popup(QCursor.pos())

	def selectDocumentAction(self) : 
		
		from libwidget.selectDialog import SelectDialog
		from libview.document import DocumentDataSources	
		dataRows, dataColumns, self.dataSqlNames, dataColumsType, dataTooltips = DocumentDataSources( self.db ).selectAll()			
		dialog = SelectDialog()
		dialog.loadData( dataRows, dataColumns, dataColumsType, dataTooltips, 'Sélection: '+str( 'Document') , "document", DocumentDataSources(self).getFkIndexLibBack() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.mainWindowEditMode.emit( int(rowId) , "Edit mode selection")
				
	
	def onStoreState( self ) :
		self.tableView.storeState()

	def onNewRow(self):
		
		self.mainWindowEditMode.emit( -1 , "Edit mode New")

	def onEditRow(self, rowId):

		self.mainWindowEditMode.emit( int(rowId) , "Edit mode")

	def onDeleteRow(self, rowdbId, rowIndex):

		QApplication.setOverrideCursor(Qt.WaitCursor)
		self.myDocuments = Documents( self.db )
		documentObj = self.myDocuments.select( rowdbId )
		QApplication.restoreOverrideCursor()

		if documentObj :

			dlg = DocumentDeleteConfirm(self.db, self, rowdbId, documentObj.getName()) 
			
			returnCode = dlg.exec()
			if returnCode == QDialog.Accepted :
				self.proxyModel.removeRow(rowIndex)				
				self.mainWindowRemoveRow.emit( rowdbId, "row removed")
				
class DocumentDeleteConfirm(QDialog):

	parent = None

	def __init__(self, db, parent=None, rowid=-1, libRecord="" ):
		super().__init__(parent)

		self.db = db
		self.parent = parent
		self.rowId = rowid
		self.libRecord = libRecord

		self.setWindowTitle("Document: suppression de #" + str(self.rowId)  )		

		QBtn = QDialogButtonBox.Ok | QDialogButtonBox.Cancel

		self.buttonBox = QDialogButtonBox(QBtn)
		self.buttonBox.accepted.connect(self.onOk)
		self.buttonBox.rejected.connect(self.onCancel)

		self.layout = QGridLayout() 
		
		# http://srinikom.github.io/pyside-docs/PySide/QtGui/QStyle.html#PySide.QtGui.PySide.QtGui.QStyle.StandardPixmap
		from PyQt5.QtWidgets import QApplication, QStyle
		from PyQt5.QtGui import QIcon

		labelIcon = QLabel("icon")
		dialogIcon = QIcon(QApplication.style().standardIcon(QStyle.SP_MessageBoxQuestion) )       
		labelIcon.setPixmap( dialogIcon.pixmap(48,48)  )
		self.layout.addWidget(labelIcon,0,0)				
		
		message = QLabel("Confirmez-vous la suppression de : \n" )
		self.layout.addWidget(message,0,1)

		data = QLabel(str(self.libRecord))
		self.layout.addWidget(data,1,1)
			
		self.layout.addWidget(self.buttonBox,2,1)
		self.setLayout(self.layout)

	def onCancel( self ) :
		self.reject()

	def onOk( self ) :
		self.myDocuments = Documents( self.db )
		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.myDocuments.delete( self.rowId )
		QApplication.restoreOverrideCursor()

		if result is False :
			self.parent.mainWindowUserMessage.emit( 1, "ERREUR SQL : " + str(self.myDocuments.lastError) )
			self.reject()
		else :
			self.parent.mainWindowUserMessage.emit( 0 , "Suppresion effectuée. (id:"+str(self.rowId) + ")")
			self.parent.mainWindowListMode.emit()
			self.accept()


class DocumentEditWidget(QWidget):

	mainWindowUserMessage = pyqtSignal( int, str )
	mainWindowListMode = pyqtSignal( )
	enterEditSignal = pyqtSignal( int )
	closeEdit = pyqtSignal( int, int )	
	validEditSignal = pyqtSignal( bool, int )
	valuesChanged = False
	
	recPos = 0
	siblingIdList = []
	siblingOrderField = "1"
	siblingOrderBy = ""

	def __init__(self, db, rootMedia, rowId, parentId, siblingIdList, siblingOrderField, siblingOrderBy, rowDataObj = None ):		
		
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia
		self.parentId = parentId
		
		self.siblingIdList = siblingIdList		
		self.siblingOrderField = siblingOrderField # Nom Sql du champ sur lequel porte le tri		
		sqlOrderBy = ['DESC','ASC'][siblingOrderBy]
		self.siblingOrderBy = sqlOrderBy		
		
		self.rowDataObj = rowDataObj

		self.myDocuments = Documents( self.db )

		documentObj = self.myDocuments.select( rowId )
		if documentObj is False :
			self.mainWindowUserMessage.emit( 0 , "Cet enregistrement a été supprimé.")
			self.rowId = -1
		else :
			self.rowId = rowId

		if self.rowId in self.siblingIdList:
			self.recPos = self.siblingIdList.index(self.rowId)
		else:
			self.recPos = 0

		self.initUI()
		
		self.setObjectName( "DocumentEditWidget" )

	def showEvent(self, event):
		super(DocumentEditWidget, self).showEvent(event)
		self.WidgetsForm.setFocusOnFirstWidget()	
		self.setPrevNextButtonState()		
		
	def initUI(self):

		pageLayout = QGridLayout()
		pageButtonLayout = QHBoxLayout()
		
		self.closeBtn = QPushButton("Fermer", self)
		self.closeBtn.setToolTip( "Fermer et afficher la liste Document" )
		self.closeBtn.setIcon(QIcon('img/form_back.png'))
		self.closeBtn.clicked.connect(self.onClose)
		pageButtonLayout.addWidget(self.closeBtn)

		self.validBtn = QPushButton("Sauver", self)
		self.validBtn.setToolTip( "Enregistrer les modifications Document" )
		self.validBtn.setIcon(QIcon('img/form_save.png'))		
		self.validBtn.clicked.connect(self.onValid)
		pageButtonLayout.addWidget(self.validBtn)

		self.updateMessage = QLineEdit( self )
		self.updateMessage.setStyleSheet("QLineEdit { background-color : rgb(255, 255, 255); color: red; border : 0px ;}")
		self.updateMessage.setToolTip( "Dernière erreur rencontrée lors de l'édition" )
		pageButtonLayout.addWidget(self.updateMessage)

		self.prevBtn = QPushButton("Précédent", self)
		self.prevBtn.setToolTip( "Document précédent" )
		self.prevBtn.setIcon(QIcon('img/form_prev.png'))		
		self.prevBtn.clicked.connect(self.onPrev)
		pageButtonLayout.addWidget(self.prevBtn)

		self.nextBtn = QPushButton("Suivant", self)
		self.nextBtn.setToolTip( "Document suivant" )
		self.nextBtn.setIcon(QIcon('img/form_next.png'))		
		self.nextBtn.clicked.connect(self.onNext)
		pageButtonLayout.addWidget(self.nextBtn)
		self.nextBtn.setLayoutDirection(Qt.RightToLeft)		

		self.WidgetsForm = DocumentEditDisplayWidgets(self, self.db, self.rootMedia, self.rowId, self.parentId, self.rowDataObj)
		
		scroll = QScrollArea(self)
		scroll.setWidget(self.WidgetsForm)
		scroll.setWidgetResizable(True)
		pageLayout.addWidget(scroll)

		pageLayout.addLayout(pageButtonLayout, 1,0)

		self.setLayout(pageLayout)

	def enableNavButtons( self ) :
		self.nextBtn.setEnabled( True )
		self.prevBtn.setEnabled( True )

	def setPrevNextButtonState( self ) :
		
		if self.recPos <= 0 :
			self.prevBtn.setEnabled( False )
		else :
			self.prevBtn.setEnabled( True )

		if self.recPos >= len(self.siblingIdList)-1 :
			self.nextBtn.setEnabled( False )
		else :
			self.nextBtn.setEnabled( True )	
			
	def onPrev( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
					quitForm = False
					
		if quitForm is True :					
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos - 1			
			if self.recPos < 0 :
				self.recPos = 0		
			
			documentObj = self.myDocuments.select( self.siblingIdList[self.recPos]  )
			
			if documentObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.myDocuments.selectCount == 1 :
				self.rowDataObj = documentObj			
				self.rowId = documentObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()
		
	def onNext( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
				quitForm = False
			
		if quitForm is True :
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos + 1
			if self.recPos >= len(self.siblingIdList) :
				self.recPos = len(self.siblingIdList) - 1
			
			documentObj = self.myDocuments.select( self.siblingIdList[self.recPos]  )
			
			if documentObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.myDocuments.selectCount == 1 :
				self.rowDataObj = documentObj
				self.rowId = documentObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()

	def onValid( self ) :
		self.updateMessage.setText( "" )

		myDocumentObj = Document()

		myDocumentObj.setId( self.rowId )

			
		myDocumentObj.setName( self.WidgetsForm.InputName.text() )					

			
			
		myDocumentObj.setDescription( self.WidgetsForm.InputDescription.toPlainText() )					

								
			
		if self.WidgetsForm.InputPrivate.checkState() == Qt.CheckState.Unchecked :
			checkBoxState = 0
		else :
			checkBoxState = 1
		myDocumentObj.setPrivate( checkBoxState ) 
							

								
			
		myDocumentObj.setIcon( self.WidgetsForm.InputIcon.text() )					

								
			
		myDocumentObj.setUrl( self.WidgetsForm.InputUrl.text() )					

			
		myDocumentObj.setCote( self.WidgetsForm.InputCote.text() )					

			
			
		myDocumentObj.setNotesearch( self.WidgetsForm.InputNotesearch.toPlainText() )					

			
		myDocumentObj.setSortorder( self.WidgetsForm.InputSortorder.text() )					

						
			
		myDocumentObj.setDepot_id( self.rowDataObj.getDepot_id() ) 				

						
			
		myDocumentObj.setDocumenttype_id( self.rowDataObj.getDocumenttype_id() ) 				

			
			
		# dd/MM/yyyy hh:mm
		myDocumentObj.setDatecreated( self.WidgetsForm.InputDatecreated.dateTime().toString('yyyy-MM-dd hh:mm') )					

		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.myDocuments.update( myDocumentObj )
		QApplication.restoreOverrideCursor()

		self.updateMessage.setText( "" )
		self.mainWindowUserMessage.emit( -1 , "" )

		if (result is False) :
			self.updateMessage.setText( "ERREUR SQL : " + str(self.myDocuments.lastError) )
			self.updateMessage.setToolTip( "ERREUR SQL : " + str(self.myDocuments.lastError) )
			self.validEditSignal.emit( False, self.rowId )
		else :
			self.rowId = self.myDocuments.lastInsertId
			myDocumentObj.setId( self.rowId )
			self.mainWindowUserMessage.emit( 0 , "Mise à jour effectuée. (id:"+str(self.rowId) + ")")
			self.WidgetsForm.displayData( self.rowId )
			self.validEditSignal.emit( True, self.rowId )

	def onClose( self ) :

		if self.valuesChanged is True :

			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.Yes:
				self.mainWindowUserMessage.emit( -1 , "" )
				self.closeEdit.emit(True, self.rowId)
				
		else :
			self.mainWindowUserMessage.emit( -1 , "" )
			self.closeEdit.emit(False, self.rowId)
	
	def onValuesChanged( self, boolState ) :
		self.valuesChanged = boolState
		self.validBtn.setEnabled( boolState )
		
		
class DocumentEditDisplayWidgets(QWidget):

	def __init__(self, parent, db, rootMedia, rowId, rowParentId, rowDataObj):
		
		QWidget.__init__(self, parent)

		self.db = db
		self.rootMedia = rootMedia
		self.parent = parent
		self.rowId = rowId
		self.rowParentId = rowParentId
		self.rowDataObj = rowDataObj

		self.myDocuments = Documents( self.db )

		gridLayout = QGridLayout()

		gridLayout.setSpacing(10)

		rowNumber = 1
		
		labelId = QLabel('id')
		labelId.setToolTip( 'Identifiant interne (Id)' )
		labelId.setAlignment(Qt.AlignRight | Qt.AlignVCenter)		
		self.InputId = QLineEdit()
		self.InputId.setMaximumWidth( self.getWidgetWidth( self.InputId, 5 )  )		
		self.InputId.setEnabled( False )
		self.InputId.setStyleSheet("color: #CCCCCC;")
		gridLayout.addWidget(labelId, rowNumber, 0)
		gridLayout.addWidget(self.InputId, rowNumber, 1)

		rowNumber = rowNumber + 1
			
		labelName = QLabel('Nom')
		labelName.setToolTip( 'Nom du document' )
		labelName.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputName = QLineEdit()
		self.InputName.setObjectName( "DocumentEditName" )
		self.InputName.installEventFilter(self)
		gridLayout.addWidget(labelName, rowNumber, 0)
		gridLayout.addWidget(self.InputName, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Nom: valeur obligatoire' )
		self.InputName.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
				
			
		labelDescription = QLabel('description')
		labelDescription.setToolTip( 'Description du document' )
		labelDescription.setAlignment(Qt.AlignRight)
		self.InputDescription = QTextEdit()
		self.InputDescription.setObjectName( "DocumentEditDescription" )
		self.InputDescription.installEventFilter(self)
		gridLayout.addWidget(labelDescription, rowNumber, 0)
		gridLayout.addWidget(self.InputDescription, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'description: valeur obligatoire' )
		self.InputDescription.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
				

			
		labelPrivate = QLabel('Privé')
		labelPrivate.setToolTip( 'Donnée de statut privé' )
		labelPrivate.setAlignment(Qt.AlignRight)
		self.InputPrivate = QCheckBox()
		self.InputPrivate.setObjectName( "DocumentEditPrivate" )
		self.InputPrivate.installEventFilter(self)
		gridLayout.addWidget(labelPrivate, rowNumber, 0)
		gridLayout.addWidget(self.InputPrivate, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Privé: valeur obligatoire' )
		self.InputPrivate.stateChanged.connect(self.onCheckBoxValuesChanged) 
		rowNumber = rowNumber + 1
					

			
		labelIcon = QLabel('Icone')
		labelIcon.setToolTip( 'Icone du document' )
		labelIcon.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputIcon = QLineEdit()
		self.InputIcon.setObjectName( "DocumentEditIcon" )
		self.InputIcon.installEventFilter(self)
		gridLayout.addWidget(labelIcon, rowNumber, 0)
		gridLayout.addWidget(self.InputIcon, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Icone: valeur obligatoire' )
		self.InputIcon.textChanged.connect(self.onLineEditValuesChanged)	
		self.fileSelectIcon = QPushButton("...")
		self.fileSelectIcon.clicked.connect(self.getfilesMediaIcon)
		gridLayout.addWidget(self.fileSelectIcon, rowNumber, 3)

		rowNumber = rowNumber + 1
					

			
		labelUrl = QLabel('Url')
		labelUrl.setToolTip( 'Lien vers le document, ou url' )
		labelUrl.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputUrl = QLineEdit()
		self.InputUrl.setObjectName( "DocumentEditUrl" )
		self.InputUrl.installEventFilter(self)
		gridLayout.addWidget(labelUrl, rowNumber, 0)
		gridLayout.addWidget(self.InputUrl, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Url: valeur obligatoire' )
		self.InputUrl.textChanged.connect(self.onLineEditValuesChanged)	
		self.fileSelectUrl = QPushButton("...")
		self.fileSelectUrl.clicked.connect(self.getfilesMediaUrl)
		gridLayout.addWidget(self.fileSelectUrl, rowNumber, 3)

		rowNumber = rowNumber + 1
			
		labelCote = QLabel('Cote')
		labelCote.setToolTip( 'Identifiant unique du document' )
		labelCote.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputCote = QLineEdit()
		self.InputCote.setObjectName( "DocumentEditCote" )
		self.InputCote.installEventFilter(self)
		gridLayout.addWidget(labelCote, rowNumber, 0)
		gridLayout.addWidget(self.InputCote, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Cote: valeur obligatoire' )
		self.InputCote.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
				
			
		labelNotesearch = QLabel('Notes de recherche')
		labelNotesearch.setToolTip( 'Notes de recherches' )
		labelNotesearch.setAlignment(Qt.AlignRight)
		self.InputNotesearch = QTextEdit()
		self.InputNotesearch.setObjectName( "DocumentEditNotesearch" )
		self.InputNotesearch.installEventFilter(self)
		gridLayout.addWidget(labelNotesearch, rowNumber, 0)
		gridLayout.addWidget(self.InputNotesearch, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Notes de recherche: valeur obligatoire' )
		self.InputNotesearch.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
			
		labelSortorder = QLabel('Ordre de tri')
		labelSortorder.setToolTip( 'Ordre de tri' )
		labelSortorder.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputSortorder = QLineEdit()
		self.InputSortorder.setObjectName( "DocumentEditSortorder" )
		self.InputSortorder.installEventFilter(self)
		gridLayout.addWidget(labelSortorder, rowNumber, 0)
		gridLayout.addWidget(self.InputSortorder, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Ordre de tri: valeur obligatoire' )
		self.InputSortorder.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
			
		labelDepot_id = QLabel('Dépôt')
		labelDepot_id.setToolTip( 'Identifiant du dépot dans lequel se trouve le document' )
		labelDepot_id.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDepot_id = QLineEdit()
		self.InputDepot_id.setObjectName( "DocumentEditDepot_id" )
		self.InputDepot_id.installEventFilter(self)
		gridLayout.addWidget(labelDepot_id, rowNumber, 0)
		gridLayout.addWidget(self.InputDepot_id, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Dépôt: valeur obligatoire' )
		self.InputDepot_id.setReadOnly(True)
		self.selectBtn = QPushButton("...", self)
		self.selectBtn.clicked.connect(self.onSelectDepot_id)
		gridLayout.addWidget(self.selectBtn, rowNumber, 3) 
		rowNumber = rowNumber + 1
			
			
		labelDocumenttype_id = QLabel('Type')
		labelDocumenttype_id.setToolTip( 'Type de document' )
		labelDocumenttype_id.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDocumenttype_id = QLineEdit()
		self.InputDocumenttype_id.setObjectName( "DocumentEditDocumenttype_id" )
		self.InputDocumenttype_id.installEventFilter(self)
		gridLayout.addWidget(labelDocumenttype_id, rowNumber, 0)
		gridLayout.addWidget(self.InputDocumenttype_id, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Type: valeur obligatoire' )
		self.InputDocumenttype_id.setReadOnly(True)
		self.selectBtn = QPushButton("...", self)
		self.selectBtn.clicked.connect(self.onSelectDocumenttype_id)
		gridLayout.addWidget(self.selectBtn, rowNumber, 3) 
		rowNumber = rowNumber + 1
							
			
		labelDatecreated = QLabel('Création')
		labelDatecreated.setToolTip( 'Date de création ou modification' )
		labelDatecreated.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDatecreated = QDateTimeEdit()
		self.InputDatecreated.setObjectName( "DocumentEditDatecreated" )
		self.InputDatecreated.installEventFilter(self)
		self.InputDatecreated.setMaximumWidth( self.getWidgetWidth( self.InputDatecreated, 16 )  )		
		gridLayout.addWidget(labelDatecreated, rowNumber, 0)
		gridLayout.addWidget(self.InputDatecreated, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Création: valeur obligatoire' )
		self.InputDatecreated.dateTimeChanged.connect(self.onDateEditValuesChanged)	
		self.InputDatecreated.setDisplayFormat("dd/MM/yyyy hh:mm")
		self.InputDatecreated.setDateTimeRange(QDateTime(1, 1, 1, 0, 0), QDateTime(2999, 12, 31, 23, 59))
		self.InputDatecreated.setSpecialValueText(' ')
		self.InputDatecreated.setReadOnly(True)
		if True is False :
			self.InputDatecreated.setCalendarPopup(True)	


		self.setLayout(gridLayout)
		self.displayData( self.rowId, self.rowDataObj )
		

			

			

			

			

			

			

			

			

			
	 # Affichage de la fenêtre pop-up de selection de la clé étrangère
	def onSelectDepot_id( self ) :

		from libview.depotselect import DepotSelect
		dialog = DepotSelect( self, self.db, self.rootMedia  )
		dialog.selectRowById( self.parent.rowDataObj.getDepot_id() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.InputDepot_id.setText( str( rowLib ) )	
				self.parent.rowDataObj.setDepot_id( rowId )
				self.parent.onValuesChanged( True ) 
			else :
				self.InputDepot_id.setText( None )	
				self.parent.rowDataObj.setDepot_id( None )
				self.parent.onValuesChanged( True ) 

			
	 # Affichage de la fenêtre pop-up de selection de la clé étrangère
	def onSelectDocumenttype_id( self ) :

		from libview.documenttypeselect import DocumenttypeSelect
		dialog = DocumenttypeSelect( self, self.db, self.rootMedia  )
		dialog.selectRowById( self.parent.rowDataObj.getDocumenttype_id() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.InputDocumenttype_id.setText( str( rowLib ) )	
				self.parent.rowDataObj.setDocumenttype_id( rowId )
				self.parent.onValuesChanged( True ) 
			else :
				self.InputDocumenttype_id.setText( None )	
				self.parent.rowDataObj.setDocumenttype_id( None )
				self.parent.onValuesChanged( True ) 

			


	

	

	

	

	

	

	

	

	

	

	


	

	

	

	

	def getfilesMediaIcon( self ) :      

		from libwidget.mediamanager import MediaManager
		from lib.project import Project  
		currentProject = Project()			
		currentProject.load( AppSetting.get('app/lastProject', '') )
		myMediaSelector = MediaManager( rootMedia = currentProject.rootMedia )
		myMediaSelector.loadDataList( self.InputIcon.text().split(',') )		
		result = myMediaSelector.exec_()
		filenamelist = myMediaSelector.getValue()
		if filenamelist is not None :
			self.printDebug("filenamelist = " , filenamelist)
			self.InputIcon.setText( ",".join(filenamelist) )

	

	def getfilesMediaUrl( self ) :      

		from libwidget.mediamanager import MediaManager
		from lib.project import Project  
		currentProject = Project()			
		currentProject.load( AppSetting.get('app/lastProject', '') )
		myMediaSelector = MediaManager( rootMedia = currentProject.rootMedia )
		myMediaSelector.loadDataList( self.InputUrl.text().split(',') )		
		result = myMediaSelector.exec_()
		filenamelist = myMediaSelector.getValue()
		if filenamelist is not None :
			self.printDebug("filenamelist = " , filenamelist)
			self.InputUrl.setText( ",".join(filenamelist) )

	

	

	

	

	

	



	

	

	

	

	

	

	

	

	

	

	



	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
	
	# Retourne la largeur necessaire pour un widget (QlineEdit ...) 
	# en fonctiion du nombre de caractères souhaité.
	def getWidgetWidth( self, widget, charmax ) :
		
		fm = widget.fontMetrics()
		try :
			m = widget.textMargins()
		except Exception as e:
			m = QMargins(8,8,8,8)
		c = widget.contentsMargins()
		w = charmax*fm.width('x')+m.left()+m.right()+c.left()+c.right()
		w = w + 8 # margins
		return w
			
	def onLineEditValuesChanged(self , text) :
		
		self.parent.onValuesChanged( True )

	def onTextEditValuesChanged(self):
		
		self.parent.onValuesChanged( True )
		
	def onComboValuesChanged(self, newIndex):
		
		self.parent.onValuesChanged( True )
				
	def onCheckBoxValuesChanged( self ,state) :
		self.parent.onValuesChanged( True )
			
	def onDateEditValuesChanged( self, newDate ) :
		
		self.parent.onValuesChanged( True )
				
	def displayData( self, rowId, dataObj = None) :

		self.rowId = rowId		

		self.parent.enterEditSignal.emit( self.rowId )
		
		if self.rowId == -1 :

			self.parent.mainWindowUserMessage.emit( 1 , "Formulaire vide (creation ou recréation)" )
		
		if dataObj is None :			
			documentObj = self.myDocuments.select( self.rowId )
			if documentObj is False :
				documentObj = Document()			
		else :
			documentObj = dataObj
						
		self.InputId.setText( str(documentObj.getId()) )		

			
		defaultValue = ''
		self.InputName.setText( str( documentObj.getName() if documentObj.getName() is not None else defaultValue ) ) 

			
			
		self.InputDescription.setText( str( documentObj.getDescription() if documentObj.getDescription() is not None else '' ) ) 

			
			
		if documentObj.getPrivate() is None or documentObj.getPrivate() == 0 :
			self.InputPrivate.setCheckState( Qt.CheckState.Unchecked ) 			
		if documentObj.getPrivate() == 1 :
			self.InputPrivate.setCheckState( Qt.CheckState.Checked ) 


			
			
		self.InputIcon.setText( str( documentObj.getIcon() if documentObj.getIcon() is not None else '' ) ) 

			
			
		self.InputUrl.setText( str( documentObj.getUrl() if documentObj.getUrl() is not None else '' ) ) 

			
		defaultValue = ''
		self.InputCote.setText( str( documentObj.getCote() if documentObj.getCote() is not None else defaultValue ) ) 

			
			
		self.InputNotesearch.setText( str( documentObj.getNotesearch() if documentObj.getNotesearch() is not None else '' ) ) 

			
		defaultValue = ''
		self.InputSortorder.setText( str( documentObj.getSortorder() if documentObj.getSortorder() is not None else defaultValue ) ) 

			
			
		from libdbcustom.depot import Depots
		depot = Depots( self.db )
		depotObj = depot.select( documentObj.getDepot_id()  ) 
		if depotObj is not False :
			self.InputDepot_id.setText( str(  depotObj.getName() ) )
		else :
			self.InputDepot_id.setText( '' ) 

			
			
		from libdbcustom.documenttype import Documenttypes
		documenttype = Documenttypes( self.db )
		documenttypeObj = documenttype.select( documentObj.getDocumenttype_id()  ) 
		if documenttypeObj is not False :
			self.InputDocumenttype_id.setText( str(  documenttypeObj.getName() ) )
		else :
			self.InputDocumenttype_id.setText( '' ) 

									
			
		#myQdate = Util.getQDatetimeFromString( documentObj.getDatecreated(), defaultValue = QDateTime.currentDateTime().toString('dd/MM/yyyy H:i:s') )
		myQdate = Util.getQDatetimeFromString( documentObj.getDatecreated() )
		self.InputDatecreated.setDateTime( myQdate ) 

		self.parent.onValuesChanged( False )


	def setFocusOnFirstWidget( self ) :

		from PyQt5.QtWidgets import  QLineEdit, QTextEdit, QDateEdit, QDateTimeEdit, QDateTimeEdit, QCheckBox, QComboBox,QDial, QDoubleSpinBox, QRadioButton, QSlider, QSpinBox, QTimeEdit, QFontComboBox
		for widget in self.children():
			if widget.isEnabled() :
				if 	isinstance(widget, QLineEdit) or \
					isinstance(widget, QTextEdit) or \
					isinstance(widget, QDateEdit) or \
					isinstance(widget, QDateTimeEdit) or \
					isinstance(widget, QCheckBox) or \
					isinstance(widget, QComboBox) or \
					isinstance(widget, QDial) or \
					isinstance(widget, QDoubleSpinBox) or \
					isinstance(widget, QRadioButton) or \
					isinstance(widget, QSlider) or \
					isinstance(widget, QSpinBox) or \
					isinstance(widget, QTimeEdit) or \
					isinstance(widget, QFontComboBox) \
					:
					#AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					widget.setFocus()					
					return

	def eventFilter(self, widgetObj, event):
		
		if event.type() == QtCore.QEvent.FocusIn:			
			self.focusWidget = widgetObj
	
		return super(DocumentEditDisplayWidgets, self).eventFilter(widgetObj, event)
    
	def restoreFocusWidget( self ) :		
		#AppLog.log( LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		AppLog.log( LOG_DEBUG , LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		self.focusWidget.setFocus()	
            

class DocumentDataSources() :
	
	# Jointure sur table(s):  depot (b_name)  documenttype (c_name) 	
	dataColumns = [ 'id', 'Nom', 'description', 'Privé', 'Icone', 'Url', 'Cote', 'Notes de recherche', 'Ordre de tri', 'Dépôt', 'Type', 'Création',  'Dépôt', 'Type de document' ]
	dataSqlFields = [ 'id', 'name', 'description', 'private', 'icon', 'url', 'cote', 'notesearch', 'sortorder', 'depot_id', 'documenttype_id', 'datecreated',  'b1.name', 'c2.name' ]	
	dataColumsType = ['INTEGER', 'TEXT', 'TEXT', 'BOOLEAN', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'INTEGER', 'INTEGER', 'INTEGER', 'DATETIME',  'TEXT', 'TEXT' ]
	dataTooltips = [ 'Identifiant du document', 'Nom du document', 'Description du document', 'Donnée de statut privé', 'Icone du document', 'Lien vers le document, ou url', 'Identifiant unique du document', 'Notes de recherches', 'Ordre de tri', 'Identifiant du dépot dans lequel se trouve le document', 'Type de document', 'Date de création ou modification',  'Nom du dépôt', 'Type de document' ]
			
	def __init__(self, db ):
		self.db = db
		pass
		
	def selectAll( self , field_name = None, field_value = None ) :
		
		if field_name is None and field_value is None :
			return self.select()
		else :
			return self.selectById( field_name, field_value )
			
					
	def select( self ) :
		
		dataRows = []
		
		self.myDocuments = Documents( self.db )
		documentListAll = self.myDocuments.selectAll()
		if documentListAll :
			i = 0
			for document in documentListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(document.getId()))
				dataRows.append( [ document.getId(),  document.getName(),  document.getDescription(),  document.getPrivate(),  document.getIcon(),  document.getUrl(),  document.getCote(),  document.getNotesearch(),  document.getSortorder(),  document.getDepot_id(),  document.getDocumenttype_id(),  document.getDatecreated(),  document.getB1_name(), document.getC2_name() ] )
				i = i + 1
			
		return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )


	def selectById( self, where_clause, fields_values ) :
		
		dataRows = []
				
		self.myDocuments = Documents( self.db )
		documentListAll = self.myDocuments.selectById( where_clause, fields_values )
		if documentListAll : 
			i = 0
			for document in documentListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(document.getId()))
				dataRows.append( [ document.getId(),  document.getName(),  document.getDescription(),  document.getPrivate(),  document.getIcon(),  document.getUrl(),  document.getCote(),  document.getNotesearch(),  document.getSortorder(),  document.getDepot_id(),  document.getDocumenttype_id(),  document.getDatecreated(),  document.getB1_name(), document.getC2_name() ] )
				i = i + 1
				
			return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
		return ( None, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
	# Retourner le champ #x lors de la selection par pop-up (Fk)
	def getFkIndexLibBack( self ) :
		return 1
