#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

		*** Source autogénéré *** 
		
		Pour la table : event.

		----------------------------
		Evenement.
		
		Liste des évenements.
		----------------------------		

		05/24/2023 14:31

		lvardon@laposte.net - 2021

		Licence libre

'''
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QAction, QMenu, QApplication, QWidget, QToolTip, QPushButton, QMessageBox, QFileDialog
from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, QHBoxLayout, QScrollArea
from PyQt5.QtWidgets import QLabel, QLineEdit, QTextEdit, QComboBox, QCheckBox, QDateTimeEdit
from PyQt5.QtGui import QCursor, QFont, QIcon
from PyQt5.QtCore import Qt, QEvent, pyqtSignal, pyqtSlot, QModelIndex, QDateTime, QMargins, QSettings
from PyQt5 import QtCore
		
from lib.sqlitedb import Sqlitedb
from libdbcustom.event import Event, Events

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
from lib.util import Util
from libwidget.tableModel import TableModel, TableProxyModel
from libwidget.tableView import TableView,TableCellImageDelegate

import lib.setting as AppSetting
import lib.log as AppLog

class EventListWidget(QWidget):

	mainWindowEditMode =  pyqtSignal( int, str )
	mainWindowRemoveRow =  pyqtSignal( int, str )	
	mainWindowListMode = pyqtSignal()
	mainWindowUserMessage = pyqtSignal( int, str )
	cellClickSignal =  pyqtSignal( int )
	cellDoubleClickSignal =  pyqtSignal( int )
	selectionSignal = pyqtSignal( int )
	
	def __init__(self , db, rootMedia, sourceOrigin = "", where_clause = None, fields_values = None):
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia 
		self.sourceOrigin = sourceOrigin
		
		self.where_clause = where_clause
		self.fields_values = fields_values
		
		self.initUI()

		self.initDelegates(  self.tableView )
		
		iconW, iconH, rowHeight = self.getTableViewIniParams( "tablelistviewEvent" )
		self.tableView.verticalHeader().setDefaultSectionSize(int(rowHeight))

		try :
			self.iconImageDelegate.icoW = int(iconW) 
			self.iconImageDelegate.icoH = int(iconH)	
		except Exception as e:
			pass
		try :
			self.urlImageDelegate.icoW = int(iconW) 
			self.urlImageDelegate.icoH = int(iconH)					
		except Exception as e:
			pass
		
		self.setObjectName( "EventListWidget" )

	def initDelegates( self, thistableView ) :
				
		self.privateImageDelegate = TableCellImageDelegate(self, "event", {'0' : 'img/dataicons/empty.png', '1' : 'img/dataicons/private-yes.png', '2' : 'img/dataicons/private-yes.png'} , self.rootMedia, 26, 26)
		thistableView.setItemDelegateForColumn(3, self.privateImageDelegate)

		self.iconImageDelegate = TableCellImageDelegate(self, "event", None , self.rootMedia, 40, 40)
		thistableView.setItemDelegateForColumn(4, self.iconImageDelegate)

		
		thistableView.verticalHeader().setDefaultSectionSize(40)

	def initUI(self):
		
		self.layout = QVBoxLayout()
		self.setLayout(self.layout)
				
		self.proxyModel = TableProxyModel(self)
		
		self.proxyModel.setSortRole(Qt.UserRole + 2)
		
		self.tableView = TableView( self, "event", self.sourceOrigin )	
				
		self.loadData( self.where_clause , self.fields_values ) 
	
		self.tableView.setModel(self.proxyModel)
		
		self.tableView.resizeColumnsToContents()

		self.tableView.cellClickSignal.connect(self.onCellClic)
		self.tableView.cellDoubleClickSignal.connect(self.onCellDoubleClic)
	
		selection_model = self.tableView.selectionModel()
		selection_model.selectionChanged.connect(self.onSelectionChanged)      
		
		self.filterValue = QLineEdit()
		self.layout.addWidget(self.filterValue)
		self.filterValue.textChanged.connect(self.onfilterValueChanged)
		
		self.layout.addWidget(self.tableView)

	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
			

	def getTableViewIniParams ( self, tableViewName) :
		
		iniConfigFile = "config/app.ini"
		settings=QSettings(iniConfigFile, QSettings.IniFormat)

		rowHeight = 0
		if settings.value(tableViewName + '/rowHeight') is not None:			
			rowHeight = settings.value( tableViewName +'/rowHeight')						
			
		iconW = 0
		if settings.value( tableViewName + '/rowIconW') is not None:						
			iconW = settings.value( tableViewName +'/rowIconW')				
		
		iconH = 0			
		if settings.value( tableViewName +'/rowIconH') is not None:			
			iconH = settings.value( tableViewName +'/rowIconH')	

		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" rowHeight="+ str(rowHeight))
		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" iconW x IconH="+ str(iconW) + " x " + str(iconH))
		
		return iconW, iconH, rowHeight
		
	@pyqtSlot('QItemSelection', 'QItemSelection')
	def onSelectionChanged(self, selected, deselected):
		try :
			rowId =  selected.indexes()[0].data()
			self.selectionSignal.emit( rowId )
		except Exception as e:
			pass		
		
	def onfilterValueChanged(self , text) :
		self.proxyModel.setFilterValue( str(text) )		
		self.proxyModel.applyFilters()

	def onCellClic(self, rowId ):
		self.cellClickSignal.emit(rowId)

	def onCellDoubleClic(self, rowId ):
		self.cellDoubleClickSignal.emit(rowId)
		self.onEditRow(rowId)
		
	def reloadData(self) :
		self.loadData(self.where_clause, self.fields_values)		
		
	def loadData( self, where_clause , fields_values ) :
		
		self.where_clause = where_clause
		self.fields_values = fields_values
				
		dataSource = EventDataSources( self.db )
		dataRows, dataColumns, self.dataSqlNames, dataColumnsType, dataToolTips = dataSource.selectAll( self.where_clause, self.fields_values )
		
		self.model = TableModel("event", dataRows, dataColumns, dataColumnsType, dataToolTips)

		self.proxyModel.setSourceModel(self.model)
		
		self.tableView.resizeColumnsToContents()				
				
	def dumpDatas( self) :
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		for i in range(0, row) :
			self.printDebug("--- Row : ", i)
			for j in range(0, col) :
				self.printDebug( 
						self.model.data( self.model.index(i, j), Qt.UserRole + 1 ),
						" - ",
						self.model.data( self.model.index(i, j), Qt.UserRole ) 
					)

	# Nom de la colonne sur laquelle porte le tri 
	def getSqlOrderByField( self) :
		orderByFieldPos = self.tableView.hView.sortIndicatorSection()
		orderByField = self.dataSqlNames[orderByFieldPos]
		
		return orderByField
		
	# Index du sens de tri 'DESC' = 0 'ASC' = 1
	def getSqlOrderByOrder( self) :
		orderBy = self.tableView.hView.sortIndicatorOrder()
		return orderBy
		
		
	# Liste des id de la liste affichée
	def getIdList( self ) :
		
		idList = []
		
		indexId = 0 # id column is at pos 0
		
		row = self.proxyModel.rowCount()
		model = self.proxyModel
		
		for i in range(0, row) :			
			idList.append( model.data( model.index(i, indexId ), Qt.UserRole ) )
		
		return idList
			
	
	def selectRowById( self, rowId ) :

		if rowId is None :
			return
			
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel
		
		for i in range(0, row) :
			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :				
				self.tableView.selectRow( i )
				return

	def getRowData( self, rowId ) :

		myEventObj = Event( )

		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel

		for i in range(0, row) :
			#print('row:', i, " model rowid=", self.model.data( self.model.index(i, 0), Qt.UserRole ), ' == ',  rowId, ' **', self.model.data( self.model.index(i, 1), Qt.UserRole ) )

			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :

				myEventObj.setId( model.data( model.index(i, 0 ), Qt.UserRole  )  )
				myEventObj.setName( model.data( model.index(i, 1 ), Qt.UserRole  )  )
				myEventObj.setDescription( model.data( model.index(i, 2 ), Qt.UserRole  )  )
				myEventObj.setPrivate( model.data( model.index(i, 3 ), Qt.UserRole  )  )
				myEventObj.setIcon( model.data( model.index(i, 4 ), Qt.UserRole  )  )
				myEventObj.setDate_begin( model.data( model.index(i, 5 ), Qt.UserRole  )  )
				myEventObj.setDate_begin_format( model.data( model.index(i, 6 ), Qt.UserRole  )  )
				myEventObj.setDate_begin_accurate( model.data( model.index(i, 7 ), Qt.UserRole  )  )
				myEventObj.setNotesearch( model.data( model.index(i, 8 ), Qt.UserRole  )  )
				myEventObj.setPerson_1_id( model.data( model.index(i, 9 ), Qt.UserRole  )  )
				myEventObj.setPerson_2_id( model.data( model.index(i, 10 ), Qt.UserRole  )  )
				myEventObj.setPerson_3_id( model.data( model.index(i, 11 ), Qt.UserRole  )  )
				myEventObj.setEventtype_id( model.data( model.index(i, 12 ), Qt.UserRole  )  )
				myEventObj.setPlace_id( model.data( model.index(i, 13 ), Qt.UserRole  )  )
				myEventObj.setDatecreated( model.data( model.index(i, 14 ), Qt.UserRole  )  )
				myEventObj.setSortorder( model.data( model.index(i, 15 ), Qt.UserRole  )  )

				return myEventObj

		return myEventObj

	def contextMenuEvent(self, event):

		gp = event.globalPos()
		vp_pos = self.tableView.viewport().mapFromGlobal(gp)
		rowNum = self.tableView.rowAt(vp_pos.y())
		colNum = self.tableView.columnAt(event.x())

		self.menu = QMenu(self)
			
		if rowNum >= 0 :

			index = self.tableView.indexAt(vp_pos)
			rowId = self.proxyModel.index(index.row(),0).data()

			cellValue = self.proxyModel.index(rowNum, colNum).data()

			deleteAction = QAction( QIcon('img/context_del.png'), 'Supprimer', self)
			deleteAction.triggered.connect(lambda : self.onDeleteRow(rowId, rowNum))
			self.menu.addAction(deleteAction)

			editAction = QAction( QIcon('img/context_edit.png'), 'Editer', self)
			editAction.triggered.connect(lambda : self.onEditRow(rowId))
			self.menu.addAction(editAction)

		newAction = QAction(QIcon('img/context_add.png'), 'Nouveau', self)
		newAction.triggered.connect(lambda checked : self.onNewRow())
		self.menu.addAction(newAction)			

		self.menu.addSeparator()
		
		storeStateAction = QAction(QIcon('img/context_save.png'), 'Enregistrer la disposition', self)
		storeStateAction.triggered.connect(lambda checked : self.onStoreState())
		self.menu.addAction(storeStateAction)

		self.menu.popup(QCursor.pos())

	def selectEventAction(self) : 
		
		from libwidget.selectDialog import SelectDialog
		from libview.event import EventDataSources	
		dataRows, dataColumns, self.dataSqlNames, dataColumsType, dataTooltips = EventDataSources( self.db ).selectAll()			
		dialog = SelectDialog()
		dialog.loadData( dataRows, dataColumns, dataColumsType, dataTooltips, 'Sélection: '+str( 'Evenement') , "event", EventDataSources(self).getFkIndexLibBack() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.mainWindowEditMode.emit( int(rowId) , "Edit mode selection")
				
	
	def onStoreState( self ) :
		self.tableView.storeState()

	def onNewRow(self):
		
		self.mainWindowEditMode.emit( -1 , "Edit mode New")

	def onEditRow(self, rowId):

		self.mainWindowEditMode.emit( int(rowId) , "Edit mode")

	def onDeleteRow(self, rowdbId, rowIndex):

		QApplication.setOverrideCursor(Qt.WaitCursor)
		self.myEvents = Events( self.db )
		eventObj = self.myEvents.select( rowdbId )
		QApplication.restoreOverrideCursor()

		if eventObj :

			dlg = EventDeleteConfirm(self.db, self, rowdbId, eventObj.getName()) 
			
			returnCode = dlg.exec()
			if returnCode == QDialog.Accepted :
				self.proxyModel.removeRow(rowIndex)				
				self.mainWindowRemoveRow.emit( rowdbId, "row removed")
				
class EventDeleteConfirm(QDialog):

	parent = None

	def __init__(self, db, parent=None, rowid=-1, libRecord="" ):
		super().__init__(parent)

		self.db = db
		self.parent = parent
		self.rowId = rowid
		self.libRecord = libRecord

		self.setWindowTitle("Evenement: suppression de #" + str(self.rowId)  )		

		QBtn = QDialogButtonBox.Ok | QDialogButtonBox.Cancel

		self.buttonBox = QDialogButtonBox(QBtn)
		self.buttonBox.accepted.connect(self.onOk)
		self.buttonBox.rejected.connect(self.onCancel)

		self.layout = QGridLayout() 
		
		# http://srinikom.github.io/pyside-docs/PySide/QtGui/QStyle.html#PySide.QtGui.PySide.QtGui.QStyle.StandardPixmap
		from PyQt5.QtWidgets import QApplication, QStyle
		from PyQt5.QtGui import QIcon

		labelIcon = QLabel("icon")
		dialogIcon = QIcon(QApplication.style().standardIcon(QStyle.SP_MessageBoxQuestion) )       
		labelIcon.setPixmap( dialogIcon.pixmap(48,48)  )
		self.layout.addWidget(labelIcon,0,0)				
		
		message = QLabel("Confirmez-vous la suppression de : \n" )
		self.layout.addWidget(message,0,1)

		data = QLabel(str(self.libRecord))
		self.layout.addWidget(data,1,1)
			
		self.layout.addWidget(self.buttonBox,2,1)
		self.setLayout(self.layout)

	def onCancel( self ) :
		self.reject()

	def onOk( self ) :
		self.myEvents = Events( self.db )
		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.myEvents.delete( self.rowId )
		QApplication.restoreOverrideCursor()

		if result is False :
			self.parent.mainWindowUserMessage.emit( 1, "ERREUR SQL : " + str(self.myEvents.lastError) )
			self.reject()
		else :
			self.parent.mainWindowUserMessage.emit( 0 , "Suppresion effectuée. (id:"+str(self.rowId) + ")")
			self.parent.mainWindowListMode.emit()
			self.accept()


class EventEditWidget(QWidget):

	mainWindowUserMessage = pyqtSignal( int, str )
	mainWindowListMode = pyqtSignal( )
	enterEditSignal = pyqtSignal( int )
	closeEdit = pyqtSignal( int, int )	
	validEditSignal = pyqtSignal( bool, int )
	valuesChanged = False
	
	recPos = 0
	siblingIdList = []
	siblingOrderField = "1"
	siblingOrderBy = ""

	def __init__(self, db, rootMedia, rowId, parentId, siblingIdList, siblingOrderField, siblingOrderBy, rowDataObj = None ):		
		
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia
		self.parentId = parentId
		
		self.siblingIdList = siblingIdList		
		self.siblingOrderField = siblingOrderField # Nom Sql du champ sur lequel porte le tri		
		sqlOrderBy = ['DESC','ASC'][siblingOrderBy]
		self.siblingOrderBy = sqlOrderBy		
		
		self.rowDataObj = rowDataObj

		self.myEvents = Events( self.db )

		eventObj = self.myEvents.select( rowId )
		if eventObj is False :
			self.mainWindowUserMessage.emit( 0 , "Cet enregistrement a été supprimé.")
			self.rowId = -1
		else :
			self.rowId = rowId

		if self.rowId in self.siblingIdList:
			self.recPos = self.siblingIdList.index(self.rowId)
		else:
			self.recPos = 0

		self.initUI()
		
		self.setObjectName( "EventEditWidget" )

	def showEvent(self, event):
		super(EventEditWidget, self).showEvent(event)
		self.WidgetsForm.setFocusOnFirstWidget()	
		self.setPrevNextButtonState()		
		
	def initUI(self):

		pageLayout = QGridLayout()
		pageButtonLayout = QHBoxLayout()
		
		self.closeBtn = QPushButton("Fermer", self)
		self.closeBtn.setToolTip( "Fermer et afficher la liste Evenement" )
		self.closeBtn.setIcon(QIcon('img/form_back.png'))
		self.closeBtn.clicked.connect(self.onClose)
		pageButtonLayout.addWidget(self.closeBtn)

		self.validBtn = QPushButton("Sauver", self)
		self.validBtn.setToolTip( "Enregistrer les modifications Evenement" )
		self.validBtn.setIcon(QIcon('img/form_save.png'))		
		self.validBtn.clicked.connect(self.onValid)
		pageButtonLayout.addWidget(self.validBtn)

		self.updateMessage = QLineEdit( self )
		self.updateMessage.setStyleSheet("QLineEdit { background-color : rgb(255, 255, 255); color: red; border : 0px ;}")
		self.updateMessage.setToolTip( "Dernière erreur rencontrée lors de l'édition" )
		pageButtonLayout.addWidget(self.updateMessage)

		self.prevBtn = QPushButton("Précédent", self)
		self.prevBtn.setToolTip( "Evenement précédent" )
		self.prevBtn.setIcon(QIcon('img/form_prev.png'))		
		self.prevBtn.clicked.connect(self.onPrev)
		pageButtonLayout.addWidget(self.prevBtn)

		self.nextBtn = QPushButton("Suivant", self)
		self.nextBtn.setToolTip( "Evenement suivant" )
		self.nextBtn.setIcon(QIcon('img/form_next.png'))		
		self.nextBtn.clicked.connect(self.onNext)
		pageButtonLayout.addWidget(self.nextBtn)
		self.nextBtn.setLayoutDirection(Qt.RightToLeft)		

		self.WidgetsForm = EventEditDisplayWidgets(self, self.db, self.rootMedia, self.rowId, self.parentId, self.rowDataObj)
		
		scroll = QScrollArea(self)
		scroll.setWidget(self.WidgetsForm)
		scroll.setWidgetResizable(True)
		pageLayout.addWidget(scroll)

		pageLayout.addLayout(pageButtonLayout, 1,0)

		self.setLayout(pageLayout)

	def enableNavButtons( self ) :
		self.nextBtn.setEnabled( True )
		self.prevBtn.setEnabled( True )

	def setPrevNextButtonState( self ) :
		
		if self.recPos <= 0 :
			self.prevBtn.setEnabled( False )
		else :
			self.prevBtn.setEnabled( True )

		if self.recPos >= len(self.siblingIdList)-1 :
			self.nextBtn.setEnabled( False )
		else :
			self.nextBtn.setEnabled( True )	
			
	def onPrev( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
					quitForm = False
					
		if quitForm is True :					
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos - 1			
			if self.recPos < 0 :
				self.recPos = 0		
			
			eventObj = self.myEvents.select( self.siblingIdList[self.recPos]  )
			
			if eventObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.myEvents.selectCount == 1 :
				self.rowDataObj = eventObj			
				self.rowId = eventObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()
		
	def onNext( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
				quitForm = False
			
		if quitForm is True :
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos + 1
			if self.recPos >= len(self.siblingIdList) :
				self.recPos = len(self.siblingIdList) - 1
			
			eventObj = self.myEvents.select( self.siblingIdList[self.recPos]  )
			
			if eventObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.myEvents.selectCount == 1 :
				self.rowDataObj = eventObj
				self.rowId = eventObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()

	def onValid( self ) :
		self.updateMessage.setText( "" )

		myEventObj = Event()

		myEventObj.setId( self.rowId )

			
		myEventObj.setName( self.WidgetsForm.InputName.text() )					

			
			
		myEventObj.setDescription( self.WidgetsForm.InputDescription.toPlainText() )					

								
			
		if self.WidgetsForm.InputPrivate.checkState() == Qt.CheckState.Unchecked :
			checkBoxState = 0
		else :
			checkBoxState = 1
		myEventObj.setPrivate( checkBoxState ) 
							

								
			
		myEventObj.setIcon( self.WidgetsForm.InputIcon.text() )					

			
			
		# dd/MM/yyyy hh:mm
		myEventObj.setDate_begin( self.WidgetsForm.InputDate_begin.dateTime().toString('yyyy-MM-dd hh:mm') )					

			
			
		myEventObj.setDate_begin_format( self.WidgetsForm.InputDate_begin_format.itemData(self.WidgetsForm.InputDate_begin_format.currentIndex()) )					

			
			
		myEventObj.setDate_begin_accurate( self.WidgetsForm.InputDate_begin_accurate.itemData(self.WidgetsForm.InputDate_begin_accurate.currentIndex()) )					

			
			
		myEventObj.setNotesearch( self.WidgetsForm.InputNotesearch.toPlainText() )					

						
			
		myEventObj.setPerson_1_id( self.rowDataObj.getPerson_1_id() ) 				

						
			
		myEventObj.setPerson_2_id( self.rowDataObj.getPerson_2_id() ) 				

						
			
		myEventObj.setPerson_3_id( self.rowDataObj.getPerson_3_id() ) 				

						
			
		myEventObj.setEventtype_id( self.rowDataObj.getEventtype_id() ) 				

						
			
		myEventObj.setPlace_id( self.rowDataObj.getPlace_id() ) 				

			
			
		# dd/MM/yyyy hh:mm
		myEventObj.setDatecreated( self.WidgetsForm.InputDatecreated.dateTime().toString('yyyy-MM-dd hh:mm') )					

			
		myEventObj.setSortorder( self.WidgetsForm.InputSortorder.text() )					

		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.myEvents.update( myEventObj )
		QApplication.restoreOverrideCursor()

		self.updateMessage.setText( "" )
		self.mainWindowUserMessage.emit( -1 , "" )

		if (result is False) :
			self.updateMessage.setText( "ERREUR SQL : " + str(self.myEvents.lastError) )
			self.updateMessage.setToolTip( "ERREUR SQL : " + str(self.myEvents.lastError) )
			self.validEditSignal.emit( False, self.rowId )
		else :
			self.rowId = self.myEvents.lastInsertId
			myEventObj.setId( self.rowId )
			self.mainWindowUserMessage.emit( 0 , "Mise à jour effectuée. (id:"+str(self.rowId) + ")")
			self.WidgetsForm.displayData( self.rowId )
			self.validEditSignal.emit( True, self.rowId )

	def onClose( self ) :

		if self.valuesChanged is True :

			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.Yes:
				self.mainWindowUserMessage.emit( -1 , "" )
				self.closeEdit.emit(True, self.rowId)
				
		else :
			self.mainWindowUserMessage.emit( -1 , "" )
			self.closeEdit.emit(False, self.rowId)
	
	def onValuesChanged( self, boolState ) :
		self.valuesChanged = boolState
		self.validBtn.setEnabled( boolState )
		
		
class EventEditDisplayWidgets(QWidget):

	def __init__(self, parent, db, rootMedia, rowId, rowParentId, rowDataObj):
		
		QWidget.__init__(self, parent)

		self.db = db
		self.rootMedia = rootMedia
		self.parent = parent
		self.rowId = rowId
		self.rowParentId = rowParentId
		self.rowDataObj = rowDataObj

		self.myEvents = Events( self.db )

		gridLayout = QGridLayout()

		gridLayout.setSpacing(10)

		rowNumber = 1
		
		labelId = QLabel('id')
		labelId.setToolTip( 'Identifiant interne (Id)' )
		labelId.setAlignment(Qt.AlignRight | Qt.AlignVCenter)		
		self.InputId = QLineEdit()
		self.InputId.setMaximumWidth( self.getWidgetWidth( self.InputId, 5 )  )		
		self.InputId.setEnabled( False )
		self.InputId.setStyleSheet("color: #CCCCCC;")
		gridLayout.addWidget(labelId, rowNumber, 0)
		gridLayout.addWidget(self.InputId, rowNumber, 1)

		rowNumber = rowNumber + 1
			
		labelName = QLabel('Nom')
		labelName.setToolTip( 'Nom de lʼévènement' )
		labelName.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputName = QLineEdit()
		self.InputName.setObjectName( "EventEditName" )
		self.InputName.installEventFilter(self)
		gridLayout.addWidget(labelName, rowNumber, 0)
		gridLayout.addWidget(self.InputName, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Nom: valeur obligatoire' )
		self.InputName.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
				
			
		labelDescription = QLabel('Description')
		labelDescription.setToolTip( 'Description de lʼévènement' )
		labelDescription.setAlignment(Qt.AlignRight)
		self.InputDescription = QTextEdit()
		self.InputDescription.setObjectName( "EventEditDescription" )
		self.InputDescription.installEventFilter(self)
		gridLayout.addWidget(labelDescription, rowNumber, 0)
		gridLayout.addWidget(self.InputDescription, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Description: valeur obligatoire' )
		self.InputDescription.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
				

			
		labelPrivate = QLabel('Privé')
		labelPrivate.setToolTip( 'Événement de statut privé' )
		labelPrivate.setAlignment(Qt.AlignRight)
		self.InputPrivate = QCheckBox()
		self.InputPrivate.setObjectName( "EventEditPrivate" )
		self.InputPrivate.installEventFilter(self)
		gridLayout.addWidget(labelPrivate, rowNumber, 0)
		gridLayout.addWidget(self.InputPrivate, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Privé: valeur obligatoire' )
		self.InputPrivate.stateChanged.connect(self.onCheckBoxValuesChanged) 
		rowNumber = rowNumber + 1
					

			
		labelIcon = QLabel('Icône')
		labelIcon.setToolTip( 'Icône, image d’illustration' )
		labelIcon.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputIcon = QLineEdit()
		self.InputIcon.setObjectName( "EventEditIcon" )
		self.InputIcon.installEventFilter(self)
		gridLayout.addWidget(labelIcon, rowNumber, 0)
		gridLayout.addWidget(self.InputIcon, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Icône: valeur obligatoire' )
		self.InputIcon.textChanged.connect(self.onLineEditValuesChanged)	
		self.fileSelectIcon = QPushButton("...")
		self.fileSelectIcon.clicked.connect(self.getfilesMediaIcon)
		gridLayout.addWidget(self.fileSelectIcon, rowNumber, 3)

		rowNumber = rowNumber + 1
							
			
		labelDate_begin = QLabel('Date')
		labelDate_begin.setToolTip( 'Date et heure de événement (ou 01/01/0001 pour une date inexistante)' )
		labelDate_begin.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDate_begin = QDateTimeEdit()
		self.InputDate_begin.setObjectName( "EventEditDate_begin" )
		self.InputDate_begin.installEventFilter(self)
		self.InputDate_begin.setMaximumWidth( self.getWidgetWidth( self.InputDate_begin, 16 )  )		
		gridLayout.addWidget(labelDate_begin, rowNumber, 0)
		gridLayout.addWidget(self.InputDate_begin, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Date: valeur obligatoire' )
		self.InputDate_begin.dateTimeChanged.connect(self.onDateEditValuesChanged)	
		self.InputDate_begin.setDisplayFormat("dd/MM/yyyy hh:mm")
		self.InputDate_begin.setDateTimeRange(QDateTime(1, 1, 1, 0, 0), QDateTime(2999, 12, 31, 23, 59))
		self.InputDate_begin.setSpecialValueText(' ')
		self.InputDate_begin.setReadOnly(False)
		if False is False :
			self.InputDate_begin.setCalendarPopup(True)	

		rowNumber = rowNumber + 1
			
			
		labelDate_begin_format = QLabel('Format date')
		labelDate_begin_format.setToolTip( 'Format  de la date' )
		labelDate_begin_format.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDate_begin_format = QComboBox()
		self.InputDate_begin_format.setObjectName( "EventEditDate_begin_format" )
		self.InputDate_begin_format.installEventFilter(self)
		gridLayout.addWidget(labelDate_begin_format, rowNumber, 0)
		gridLayout.addWidget(self.InputDate_begin_format, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Format date: valeur obligatoire' )
		comboData =  { "" : "", "AAAA" : "aaaa", "MMAAAA" : "mm/aaaa",  } 
		self.InputDate_begin_format.clear()
		for key, value in comboData.items():
			self.InputDate_begin_format.addItem( str(value) , userData=key ) 
		self.InputDate_begin_format.currentIndexChanged.connect(self.onComboValuesChanged)
		rowNumber = rowNumber + 1
			
			
		labelDate_begin_accurate = QLabel('Précision')
		labelDate_begin_accurate.setToolTip( 'Précision de la date' )
		labelDate_begin_accurate.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDate_begin_accurate = QComboBox()
		self.InputDate_begin_accurate.setObjectName( "EventEditDate_begin_accurate" )
		self.InputDate_begin_accurate.installEventFilter(self)
		gridLayout.addWidget(labelDate_begin_accurate, rowNumber, 0)
		gridLayout.addWidget(self.InputDate_begin_accurate, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Précision: valeur obligatoire' )
		comboData =  { "" : "", "EXA" : "Exacte", "ABT" : "Environ", "BEF" : "Avant", "AFT" : "Après", "EST" : "Estimée", "CAL" : "Calculée", "UNK" : "Date inconnue",  } 
		self.InputDate_begin_accurate.clear()
		for key, value in comboData.items():
			self.InputDate_begin_accurate.addItem( str(value) , userData=key ) 
		self.InputDate_begin_accurate.currentIndexChanged.connect(self.onComboValuesChanged)
		rowNumber = rowNumber + 1
				
			
		labelNotesearch = QLabel('Notes de recherche')
		labelNotesearch.setToolTip( 'Notes de recherche' )
		labelNotesearch.setAlignment(Qt.AlignRight)
		self.InputNotesearch = QTextEdit()
		self.InputNotesearch.setObjectName( "EventEditNotesearch" )
		self.InputNotesearch.installEventFilter(self)
		gridLayout.addWidget(labelNotesearch, rowNumber, 0)
		gridLayout.addWidget(self.InputNotesearch, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Notes de recherche: valeur obligatoire' )
		self.InputNotesearch.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
			
			
		labelPerson_1_id = QLabel('Individu 👤')
		labelPerson_1_id.setToolTip( 'Individu concerné par l’événement (mariage:homme)' )
		labelPerson_1_id.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputPerson_1_id = QLineEdit()
		self.InputPerson_1_id.setObjectName( "EventEditPerson_1_id" )
		self.InputPerson_1_id.installEventFilter(self)
		gridLayout.addWidget(labelPerson_1_id, rowNumber, 0)
		gridLayout.addWidget(self.InputPerson_1_id, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Individu 👤: valeur obligatoire' )
		self.InputPerson_1_id.setReadOnly(True)
		self.selectBtn = QPushButton("...", self)
		self.selectBtn.clicked.connect(self.onSelectPerson_1_id)
		gridLayout.addWidget(self.selectBtn, rowNumber, 3) 
		rowNumber = rowNumber + 1
			
			
		labelPerson_2_id = QLabel('Femme 🤵🏻‍♀️')
		labelPerson_2_id.setToolTip( 'Mère / femme/ conjointe  : mariage, naissance...' )
		labelPerson_2_id.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputPerson_2_id = QLineEdit()
		self.InputPerson_2_id.setObjectName( "EventEditPerson_2_id" )
		self.InputPerson_2_id.installEventFilter(self)
		gridLayout.addWidget(labelPerson_2_id, rowNumber, 0)
		gridLayout.addWidget(self.InputPerson_2_id, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Femme 🤵🏻‍♀️: valeur obligatoire' )
		self.InputPerson_2_id.setReadOnly(True)
		self.selectBtn = QPushButton("...", self)
		self.selectBtn.clicked.connect(self.onSelectPerson_2_id)
		gridLayout.addWidget(self.selectBtn, rowNumber, 3) 
		rowNumber = rowNumber + 1
			
			
		labelPerson_3_id = QLabel('Homme 🧔🏻‍♂️')
		labelPerson_3_id.setToolTip( 'Père / homme : naissance...' )
		labelPerson_3_id.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputPerson_3_id = QLineEdit()
		self.InputPerson_3_id.setObjectName( "EventEditPerson_3_id" )
		self.InputPerson_3_id.installEventFilter(self)
		gridLayout.addWidget(labelPerson_3_id, rowNumber, 0)
		gridLayout.addWidget(self.InputPerson_3_id, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Homme 🧔🏻‍♂️: valeur obligatoire' )
		self.InputPerson_3_id.setReadOnly(True)
		self.selectBtn = QPushButton("...", self)
		self.selectBtn.clicked.connect(self.onSelectPerson_3_id)
		gridLayout.addWidget(self.selectBtn, rowNumber, 3) 
		rowNumber = rowNumber + 1
			
			
		labelEventtype_id = QLabel('Type')
		labelEventtype_id.setToolTip( 'Type evenement' )
		labelEventtype_id.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputEventtype_id = QLineEdit()
		self.InputEventtype_id.setObjectName( "EventEditEventtype_id" )
		self.InputEventtype_id.installEventFilter(self)
		gridLayout.addWidget(labelEventtype_id, rowNumber, 0)
		gridLayout.addWidget(self.InputEventtype_id, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Type: valeur obligatoire' )
		self.InputEventtype_id.setReadOnly(True)
		self.selectBtn = QPushButton("...", self)
		self.selectBtn.clicked.connect(self.onSelectEventtype_id)
		gridLayout.addWidget(self.selectBtn, rowNumber, 3) 
		rowNumber = rowNumber + 1
			
			
		labelPlace_id = QLabel('Lieu')
		labelPlace_id.setToolTip( 'Lieu de l’evenement' )
		labelPlace_id.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputPlace_id = QLineEdit()
		self.InputPlace_id.setObjectName( "EventEditPlace_id" )
		self.InputPlace_id.installEventFilter(self)
		gridLayout.addWidget(labelPlace_id, rowNumber, 0)
		gridLayout.addWidget(self.InputPlace_id, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Lieu: valeur obligatoire' )
		self.InputPlace_id.setReadOnly(True)
		self.selectBtn = QPushButton("...", self)
		self.selectBtn.clicked.connect(self.onSelectPlace_id)
		gridLayout.addWidget(self.selectBtn, rowNumber, 3) 
		rowNumber = rowNumber + 1
							
			
		labelDatecreated = QLabel('Modifié')
		labelDatecreated.setToolTip( 'Date de la dernière modification' )
		labelDatecreated.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDatecreated = QDateTimeEdit()
		self.InputDatecreated.setObjectName( "EventEditDatecreated" )
		self.InputDatecreated.installEventFilter(self)
		self.InputDatecreated.setMaximumWidth( self.getWidgetWidth( self.InputDatecreated, 16 )  )		
		gridLayout.addWidget(labelDatecreated, rowNumber, 0)
		gridLayout.addWidget(self.InputDatecreated, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Modifié: valeur obligatoire' )
		self.InputDatecreated.dateTimeChanged.connect(self.onDateEditValuesChanged)	
		self.InputDatecreated.setDisplayFormat("dd/MM/yyyy hh:mm")
		self.InputDatecreated.setDateTimeRange(QDateTime(1, 1, 1, 0, 0), QDateTime(2999, 12, 31, 23, 59))
		self.InputDatecreated.setSpecialValueText(' ')
		self.InputDatecreated.setReadOnly(True)
		if True is False :
			self.InputDatecreated.setCalendarPopup(True)	

		rowNumber = rowNumber + 1
			
		labelSortorder = QLabel('Ordre de tri')
		labelSortorder.setToolTip( 'Ordre de tri' )
		labelSortorder.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputSortorder = QLineEdit()
		self.InputSortorder.setObjectName( "EventEditSortorder" )
		self.InputSortorder.installEventFilter(self)
		gridLayout.addWidget(labelSortorder, rowNumber, 0)
		gridLayout.addWidget(self.InputSortorder, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Ordre de tri: valeur obligatoire' )
		self.InputSortorder.textChanged.connect(self.onLineEditValuesChanged)	

		self.setLayout(gridLayout)
		self.displayData( self.rowId, self.rowDataObj )
		

			

			

			

			

			

			

			

			

			
	 # Affichage de la fenêtre pop-up de selection de la clé étrangère
	def onSelectPerson_1_id( self ) :

		from libview.personselect import PersonSelect
		dialog = PersonSelect( self, self.db, self.rootMedia  )
		dialog.selectRowById( self.parent.rowDataObj.getPerson_1_id() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.InputPerson_1_id.setText( str( rowLib ) )	
				self.parent.rowDataObj.setPerson_1_id( rowId )
				self.parent.onValuesChanged( True ) 
			else :
				self.InputPerson_1_id.setText( None )	
				self.parent.rowDataObj.setPerson_1_id( None )
				self.parent.onValuesChanged( True ) 

			
	 # Affichage de la fenêtre pop-up de selection de la clé étrangère
	def onSelectPerson_2_id( self ) :

		from libview.personselect import PersonSelect
		dialog = PersonSelect( self, self.db, self.rootMedia  )
		dialog.selectRowById( self.parent.rowDataObj.getPerson_2_id() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.InputPerson_2_id.setText( str( rowLib ) )	
				self.parent.rowDataObj.setPerson_2_id( rowId )
				self.parent.onValuesChanged( True ) 
			else :
				self.InputPerson_2_id.setText( None )	
				self.parent.rowDataObj.setPerson_2_id( None )
				self.parent.onValuesChanged( True ) 

			
	 # Affichage de la fenêtre pop-up de selection de la clé étrangère
	def onSelectPerson_3_id( self ) :

		from libview.personselect import PersonSelect
		dialog = PersonSelect( self, self.db, self.rootMedia  )
		dialog.selectRowById( self.parent.rowDataObj.getPerson_3_id() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.InputPerson_3_id.setText( str( rowLib ) )	
				self.parent.rowDataObj.setPerson_3_id( rowId )
				self.parent.onValuesChanged( True ) 
			else :
				self.InputPerson_3_id.setText( None )	
				self.parent.rowDataObj.setPerson_3_id( None )
				self.parent.onValuesChanged( True ) 

			
	 # Affichage de la fenêtre pop-up de selection de la clé étrangère
	def onSelectEventtype_id( self ) :

		from libview.eventtypeselect import EventtypeSelect
		dialog = EventtypeSelect( self, self.db, self.rootMedia  )
		dialog.selectRowById( self.parent.rowDataObj.getEventtype_id() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.InputEventtype_id.setText( str( rowLib ) )	
				self.parent.rowDataObj.setEventtype_id( rowId )
				self.parent.onValuesChanged( True ) 
			else :
				self.InputEventtype_id.setText( None )	
				self.parent.rowDataObj.setEventtype_id( None )
				self.parent.onValuesChanged( True ) 

			
	 # Affichage de la fenêtre pop-up de selection de la clé étrangère
	def onSelectPlace_id( self ) :

		from libview.placeselect import PlaceSelect
		dialog = PlaceSelect( self, self.db, self.rootMedia  )
		dialog.selectRowById( self.parent.rowDataObj.getPlace_id() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.InputPlace_id.setText( str( rowLib ) )	
				self.parent.rowDataObj.setPlace_id( rowId )
				self.parent.onValuesChanged( True ) 
			else :
				self.InputPlace_id.setText( None )	
				self.parent.rowDataObj.setPlace_id( None )
				self.parent.onValuesChanged( True ) 

			

			


	

	

	

	

	

	

	

	

	

	

	

	

	

	

	


	

	

	

	

	def getfilesMediaIcon( self ) :      

		from libwidget.mediamanager import MediaManager
		from lib.project import Project  
		currentProject = Project()			
		currentProject.load( AppSetting.get('app/lastProject', '') )
		myMediaSelector = MediaManager( rootMedia = currentProject.rootMedia )
		myMediaSelector.loadDataList( self.InputIcon.text().split(',') )		
		result = myMediaSelector.exec_()
		filenamelist = myMediaSelector.getValue()
		if filenamelist is not None :
			self.printDebug("filenamelist = " , filenamelist)
			self.InputIcon.setText( ",".join(filenamelist) )

	

	

	

	

	

	

	

	

	

	

	



	

	

	

	

	

	

	

	

	

	

	

	

	

	

	



	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
	
	# Retourne la largeur necessaire pour un widget (QlineEdit ...) 
	# en fonctiion du nombre de caractères souhaité.
	def getWidgetWidth( self, widget, charmax ) :
		
		fm = widget.fontMetrics()
		try :
			m = widget.textMargins()
		except Exception as e:
			m = QMargins(8,8,8,8)
		c = widget.contentsMargins()
		w = charmax*fm.width('x')+m.left()+m.right()+c.left()+c.right()
		w = w + 8 # margins
		return w
			
	def onLineEditValuesChanged(self , text) :
		
		self.parent.onValuesChanged( True )

	def onTextEditValuesChanged(self):
		
		self.parent.onValuesChanged( True )
		
	def onComboValuesChanged(self, newIndex):
		
		self.parent.onValuesChanged( True )
				
	def onCheckBoxValuesChanged( self ,state) :
		self.parent.onValuesChanged( True )
			
	def onDateEditValuesChanged( self, newDate ) :
		
		self.parent.onValuesChanged( True )
				
	def displayData( self, rowId, dataObj = None) :

		self.rowId = rowId		

		self.parent.enterEditSignal.emit( self.rowId )
		
		if self.rowId == -1 :

			self.parent.mainWindowUserMessage.emit( 1 , "Formulaire vide (creation ou recréation)" )
		
		if dataObj is None :			
			eventObj = self.myEvents.select( self.rowId )
			if eventObj is False :
				eventObj = Event()			
		else :
			eventObj = dataObj
						
		self.InputId.setText( str(eventObj.getId()) )		

			
		defaultValue = ''
		self.InputName.setText( str( eventObj.getName() if eventObj.getName() is not None else defaultValue ) ) 

			
			
		self.InputDescription.setText( str( eventObj.getDescription() if eventObj.getDescription() is not None else '' ) ) 

			
			
		if eventObj.getPrivate() is None or eventObj.getPrivate() == 0 :
			self.InputPrivate.setCheckState( Qt.CheckState.Unchecked ) 			
		if eventObj.getPrivate() == 1 :
			self.InputPrivate.setCheckState( Qt.CheckState.Checked ) 


			
			
		self.InputIcon.setText( str( eventObj.getIcon() if eventObj.getIcon() is not None else '' ) ) 

									
			
		#myQdate = Util.getQDatetimeFromString( eventObj.getDate_begin(), defaultValue = QDateTime(1,1,1,0,0) )
		myQdate = Util.getQDatetimeFromString( eventObj.getDate_begin() )
		self.InputDate_begin.setDateTime( myQdate ) 

			
			
		currentKey = eventObj.getDate_begin_format() if eventObj.getDate_begin_format() is not None else '' 
		self.InputDate_begin_format.setCurrentIndex( self.InputDate_begin_format.findData(currentKey) ) 

			
			
		currentKey = eventObj.getDate_begin_accurate() if eventObj.getDate_begin_accurate() is not None else '' 
		self.InputDate_begin_accurate.setCurrentIndex( self.InputDate_begin_accurate.findData(currentKey) ) 

			
			
		self.InputNotesearch.setText( str( eventObj.getNotesearch() if eventObj.getNotesearch() is not None else '' ) ) 

			
			
		from libdbcustom.person import Persons
		person = Persons( self.db )
		personObj = person.select( eventObj.getPerson_1_id()  ) 
		if personObj is not False :
			self.InputPerson_1_id.setText( str(  personObj.getName() ) )
		else :
			self.InputPerson_1_id.setText( '' ) 

			
			
		from libdbcustom.person import Persons
		person = Persons( self.db )
		personObj = person.select( eventObj.getPerson_2_id()  ) 
		if personObj is not False :
			self.InputPerson_2_id.setText( str(  personObj.getName() ) )
		else :
			self.InputPerson_2_id.setText( '' ) 

			
			
		from libdbcustom.person import Persons
		person = Persons( self.db )
		personObj = person.select( eventObj.getPerson_3_id()  ) 
		if personObj is not False :
			self.InputPerson_3_id.setText( str(  personObj.getName() ) )
		else :
			self.InputPerson_3_id.setText( '' ) 

			
			
		from libdbcustom.eventtype import Eventtypes
		eventtype = Eventtypes( self.db )
		eventtypeObj = eventtype.select( eventObj.getEventtype_id()  ) 
		if eventtypeObj is not False :
			self.InputEventtype_id.setText( str(  eventtypeObj.getName() ) )
		else :
			self.InputEventtype_id.setText( '' ) 

			
			
		from libdbcustom.place import Places
		place = Places( self.db )
		placeObj = place.select( eventObj.getPlace_id()  ) 
		if placeObj is not False :
			self.InputPlace_id.setText( str(  placeObj.getName() ) )
		else :
			self.InputPlace_id.setText( '' ) 

									
			
		#myQdate = Util.getQDatetimeFromString( eventObj.getDatecreated(), defaultValue = QDateTime.currentDateTime().toString('dd/MM/yyyy') )
		myQdate = Util.getQDatetimeFromString( eventObj.getDatecreated() )
		self.InputDatecreated.setDateTime( myQdate ) 

			
		defaultValue = ''
		self.InputSortorder.setText( str( eventObj.getSortorder() if eventObj.getSortorder() is not None else defaultValue ) ) 

		self.parent.onValuesChanged( False )


	def setFocusOnFirstWidget( self ) :

		from PyQt5.QtWidgets import  QLineEdit, QTextEdit, QDateEdit, QDateTimeEdit, QDateTimeEdit, QCheckBox, QComboBox,QDial, QDoubleSpinBox, QRadioButton, QSlider, QSpinBox, QTimeEdit, QFontComboBox
		for widget in self.children():
			if widget.isEnabled() :
				if 	isinstance(widget, QLineEdit) or \
					isinstance(widget, QTextEdit) or \
					isinstance(widget, QDateEdit) or \
					isinstance(widget, QDateTimeEdit) or \
					isinstance(widget, QCheckBox) or \
					isinstance(widget, QComboBox) or \
					isinstance(widget, QDial) or \
					isinstance(widget, QDoubleSpinBox) or \
					isinstance(widget, QRadioButton) or \
					isinstance(widget, QSlider) or \
					isinstance(widget, QSpinBox) or \
					isinstance(widget, QTimeEdit) or \
					isinstance(widget, QFontComboBox) \
					:
					#AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					widget.setFocus()					
					return

	def eventFilter(self, widgetObj, event):
		
		if event.type() == QtCore.QEvent.FocusIn:			
			self.focusWidget = widgetObj
	
		return super(EventEditDisplayWidgets, self).eventFilter(widgetObj, event)
    
	def restoreFocusWidget( self ) :		
		#AppLog.log( LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		AppLog.log( LOG_DEBUG , LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		self.focusWidget.setFocus()	
            

class EventDataSources() :
	
	# Jointure sur table(s):  person (b_name)  person (c_name)  person (d_name)  eventtype (e_name)  place (f_name) 	
	dataColumns = [ 'id', 'Nom', 'Description', 'Privé', 'Icône', 'Date', 'Format date', 'Précision', 'Notes de recherche', 'Individu 👤', 'Femme 🤵🏻‍♀️', 'Homme 🧔🏻‍♂️', 'Type', 'Lieu', 'Modifié', 'Ordre de tri',  'Individu', 'Femme', 'Homme', 'Type evènement', 'Lieu' ]
	dataSqlFields = [ 'id', 'name', 'description', 'private', 'icon', 'date_begin', 'date_begin_format', 'date_begin_accurate', 'notesearch', 'person_1_id', 'person_2_id', 'person_3_id', 'eventtype_id', 'place_id', 'datecreated', 'sortorder',  'b1.name', 'c2.name', 'd3.name', 'e4.name', 'f5.name' ]	
	dataColumsType = ['INTEGER', 'TEXT', 'TEXT', 'BOOLEAN', 'TEXT', 'DATETIME', 'TEXT', 'TEXT', 'TEXT', 'INTEGER', 'INTEGER', 'INTEGER', 'INTEGER', 'INTEGER', 'DATETIME', 'int',  'TEXT', 'TEXT', 'TEXT', 'TEXT', 'TEXT' ]
	dataTooltips = [ 'id', 'Nom de lʼévènement', 'Description de lʼévènement', 'Événement de statut privé', 'Icône, image d’illustration', 'Date et heure de événement (ou 01/01/0001 pour une date inexistante)', 'Format  de la date', 'Précision de la date', 'Notes de recherche', 'Individu concerné par l’événement (mariage:homme)', 'Mère / femme/ conjointe  : mariage, naissance...', 'Père / homme : naissance...', 'Type evenement', 'Lieu de l’evenement', 'Date de la dernière modification', 'Ordre de tri',  'Nom de lʼindividu concerné par lʼévènement', 'Mère / femme / conjointe (mariage, naissance ...)', 'Père / homme / conjoint (mariage, naissance ...)', 'Type de lʼévènement', 'Lieu, emplacement de lʼévènement' ]
			
	def __init__(self, db ):
		self.db = db
		pass
		
	def selectAll( self , field_name = None, field_value = None ) :
		
		if field_name is None and field_value is None :
			return self.select()
		else :
			return self.selectById( field_name, field_value )
			
					
	def select( self ) :
		
		dataRows = []
		
		self.myEvents = Events( self.db )
		eventListAll = self.myEvents.selectAll()
		if eventListAll :
			i = 0
			for event in eventListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(event.getId()))
				dataRows.append( [ event.getId(),  event.getName(),  event.getDescription(),  event.getPrivate(),  event.getIcon(),  event.getDate_begin(),  event.getDate_begin_format(),  event.getDate_begin_accurate(),  event.getNotesearch(),  event.getPerson_1_id(),  event.getPerson_2_id(),  event.getPerson_3_id(),  event.getEventtype_id(),  event.getPlace_id(),  event.getDatecreated(),  event.getSortorder(),  event.getB1_name(), event.getC2_name(), event.getD3_name(), event.getE4_name(), event.getF5_name() ] )
				i = i + 1
			
		return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )


	def selectById( self, where_clause, fields_values ) :
		
		dataRows = []
				
		self.myEvents = Events( self.db )
		eventListAll = self.myEvents.selectById( where_clause, fields_values )
		if eventListAll : 
			i = 0
			for event in eventListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(event.getId()))
				dataRows.append( [ event.getId(),  event.getName(),  event.getDescription(),  event.getPrivate(),  event.getIcon(),  event.getDate_begin(),  event.getDate_begin_format(),  event.getDate_begin_accurate(),  event.getNotesearch(),  event.getPerson_1_id(),  event.getPerson_2_id(),  event.getPerson_3_id(),  event.getEventtype_id(),  event.getPlace_id(),  event.getDatecreated(),  event.getSortorder(),  event.getB1_name(), event.getC2_name(), event.getD3_name(), event.getE4_name(), event.getF5_name() ] )
				i = i + 1
				
			return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
		return ( None, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
	# Retourner le champ #x lors de la selection par pop-up (Fk)
	def getFkIndexLibBack( self ) :
		return 1
