#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

		*** Source autogénéré *** 
		
		Pour la table : media_person.

		----------------------------
		Média - individu.
		
		Associations média - individu.
		----------------------------		

		05/24/2023 14:31

		lvardon@laposte.net - 2021

		Licence libre

'''
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QAction, QMenu, QApplication, QWidget, QToolTip, QPushButton, QMessageBox, QFileDialog
from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, QHBoxLayout, QScrollArea
from PyQt5.QtWidgets import QLabel, QLineEdit, QTextEdit, QComboBox, QCheckBox, QDateTimeEdit
from PyQt5.QtGui import QCursor, QFont, QIcon
from PyQt5.QtCore import Qt, QEvent, pyqtSignal, pyqtSlot, QModelIndex, QDateTime, QMargins, QSettings
from PyQt5 import QtCore
		
from lib.sqlitedb import Sqlitedb
from libdbcustom.media_person import Media_person, Media_persons

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
from lib.util import Util
from libwidget.tableModel import TableModel, TableProxyModel
from libwidget.tableView import TableView,TableCellImageDelegate

import lib.setting as AppSetting
import lib.log as AppLog

class Media_personListWidget(QWidget):

	mainWindowEditMode =  pyqtSignal( int, str )
	mainWindowRemoveRow =  pyqtSignal( int, str )	
	mainWindowListMode = pyqtSignal()
	mainWindowUserMessage = pyqtSignal( int, str )
	cellClickSignal =  pyqtSignal( int )
	cellDoubleClickSignal =  pyqtSignal( int )
	selectionSignal = pyqtSignal( int )
	
	def __init__(self , db, rootMedia, sourceOrigin = "", where_clause = None, fields_values = None):
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia 
		self.sourceOrigin = sourceOrigin
		
		self.where_clause = where_clause
		self.fields_values = fields_values
		
		self.initUI()

		self.initDelegates(  self.tableView )
		
		iconW, iconH, rowHeight = self.getTableViewIniParams( "tablelistviewMedia_person" )
		self.tableView.verticalHeader().setDefaultSectionSize(int(rowHeight))

		try :
			self.iconImageDelegate.icoW = int(iconW) 
			self.iconImageDelegate.icoH = int(iconH)	
		except Exception as e:
			pass
		try :
			self.urlImageDelegate.icoW = int(iconW) 
			self.urlImageDelegate.icoH = int(iconH)					
		except Exception as e:
			pass
		
		self.setObjectName( "Media_personListWidget" )

	def initDelegates( self, thistableView ) :
				
		
		thistableView.verticalHeader().setDefaultSectionSize(40)

	def initUI(self):
		
		self.layout = QVBoxLayout()
		self.setLayout(self.layout)
				
		self.proxyModel = TableProxyModel(self)
		
		self.proxyModel.setSortRole(Qt.UserRole + 2)
		
		self.tableView = TableView( self, "media_person", self.sourceOrigin )	
				
		self.loadData( self.where_clause , self.fields_values ) 
	
		self.tableView.setModel(self.proxyModel)
		
		self.tableView.resizeColumnsToContents()

		self.tableView.cellClickSignal.connect(self.onCellClic)
		self.tableView.cellDoubleClickSignal.connect(self.onCellDoubleClic)
	
		selection_model = self.tableView.selectionModel()
		selection_model.selectionChanged.connect(self.onSelectionChanged)      
		
		self.filterValue = QLineEdit()
		self.layout.addWidget(self.filterValue)
		self.filterValue.textChanged.connect(self.onfilterValueChanged)
		
		self.layout.addWidget(self.tableView)

	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
			

	def getTableViewIniParams ( self, tableViewName) :
		
		iniConfigFile = "config/app.ini"
		settings=QSettings(iniConfigFile, QSettings.IniFormat)

		rowHeight = 0
		if settings.value(tableViewName + '/rowHeight') is not None:			
			rowHeight = settings.value( tableViewName +'/rowHeight')						
			
		iconW = 0
		if settings.value( tableViewName + '/rowIconW') is not None:						
			iconW = settings.value( tableViewName +'/rowIconW')				
		
		iconH = 0			
		if settings.value( tableViewName +'/rowIconH') is not None:			
			iconH = settings.value( tableViewName +'/rowIconH')	

		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" rowHeight="+ str(rowHeight))
		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" iconW x IconH="+ str(iconW) + " x " + str(iconH))
		
		return iconW, iconH, rowHeight
		
	@pyqtSlot('QItemSelection', 'QItemSelection')
	def onSelectionChanged(self, selected, deselected):
		try :
			rowId =  selected.indexes()[0].data()
			self.selectionSignal.emit( rowId )
		except Exception as e:
			pass		
		
	def onfilterValueChanged(self , text) :
		self.proxyModel.setFilterValue( str(text) )		
		self.proxyModel.applyFilters()

	def onCellClic(self, rowId ):
		self.cellClickSignal.emit(rowId)

	def onCellDoubleClic(self, rowId ):
		self.cellDoubleClickSignal.emit(rowId)
		self.onEditRow(rowId)
		
	def reloadData(self) :
		self.loadData(self.where_clause, self.fields_values)		
		
	def loadData( self, where_clause , fields_values ) :
		
		self.where_clause = where_clause
		self.fields_values = fields_values
				
		dataSource = Media_personDataSources( self.db )
		dataRows, dataColumns, self.dataSqlNames, dataColumnsType, dataToolTips = dataSource.selectAll( self.where_clause, self.fields_values )
		
		self.model = TableModel("media_person", dataRows, dataColumns, dataColumnsType, dataToolTips)

		self.proxyModel.setSourceModel(self.model)
		
		self.tableView.resizeColumnsToContents()				
				
	def dumpDatas( self) :
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		for i in range(0, row) :
			self.printDebug("--- Row : ", i)
			for j in range(0, col) :
				self.printDebug( 
						self.model.data( self.model.index(i, j), Qt.UserRole + 1 ),
						" - ",
						self.model.data( self.model.index(i, j), Qt.UserRole ) 
					)

	# Nom de la colonne sur laquelle porte le tri 
	def getSqlOrderByField( self) :
		orderByFieldPos = self.tableView.hView.sortIndicatorSection()
		orderByField = self.dataSqlNames[orderByFieldPos]
		
		return orderByField
		
	# Index du sens de tri 'DESC' = 0 'ASC' = 1
	def getSqlOrderByOrder( self) :
		orderBy = self.tableView.hView.sortIndicatorOrder()
		return orderBy
		
		
	# Liste des id de la liste affichée
	def getIdList( self ) :
		
		idList = []
		
		indexId = 0 # id column is at pos 0
		
		row = self.proxyModel.rowCount()
		model = self.proxyModel
		
		for i in range(0, row) :			
			idList.append( model.data( model.index(i, indexId ), Qt.UserRole ) )
		
		return idList
			
	
	def selectRowById( self, rowId ) :

		if rowId is None :
			return
			
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel
		
		for i in range(0, row) :
			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :				
				self.tableView.selectRow( i )
				return

	def getRowData( self, rowId ) :

		myMedia_personObj = Media_person( )

		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel

		for i in range(0, row) :
			#print('row:', i, " model rowid=", self.model.data( self.model.index(i, 0), Qt.UserRole ), ' == ',  rowId, ' **', self.model.data( self.model.index(i, 1), Qt.UserRole ) )

			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :

				myMedia_personObj.setId( model.data( model.index(i, 0 ), Qt.UserRole  )  )
				myMedia_personObj.setMedia_id( model.data( model.index(i, 1 ), Qt.UserRole  )  )
				myMedia_personObj.setPerson_id( model.data( model.index(i, 2 ), Qt.UserRole  )  )
				myMedia_personObj.setDatecreated( model.data( model.index(i, 3 ), Qt.UserRole  )  )

				return myMedia_personObj

		return myMedia_personObj

	def contextMenuEvent(self, event):

		gp = event.globalPos()
		vp_pos = self.tableView.viewport().mapFromGlobal(gp)
		rowNum = self.tableView.rowAt(vp_pos.y())
		colNum = self.tableView.columnAt(event.x())

		self.menu = QMenu(self)
			
		if rowNum >= 0 :

			index = self.tableView.indexAt(vp_pos)
			rowId = self.proxyModel.index(index.row(),0).data()

			cellValue = self.proxyModel.index(rowNum, colNum).data()

			deleteAction = QAction( QIcon('img/context_del.png'), 'Supprimer', self)
			deleteAction.triggered.connect(lambda : self.onDeleteRow(rowId, rowNum))
			self.menu.addAction(deleteAction)

			editAction = QAction( QIcon('img/context_edit.png'), 'Editer', self)
			editAction.triggered.connect(lambda : self.onEditRow(rowId))
			self.menu.addAction(editAction)

		newAction = QAction(QIcon('img/context_add.png'), 'Nouveau', self)
		newAction.triggered.connect(lambda checked : self.onNewRow())
		self.menu.addAction(newAction)			

		self.menu.addSeparator()
		
		storeStateAction = QAction(QIcon('img/context_save.png'), 'Enregistrer la disposition', self)
		storeStateAction.triggered.connect(lambda checked : self.onStoreState())
		self.menu.addAction(storeStateAction)

		self.menu.popup(QCursor.pos())

	def selectMedia_personAction(self) : 
		
		from libwidget.selectDialog import SelectDialog
		from libview.media_person import Media_personDataSources	
		dataRows, dataColumns, self.dataSqlNames, dataColumsType, dataTooltips = Media_personDataSources( self.db ).selectAll()			
		dialog = SelectDialog()
		dialog.loadData( dataRows, dataColumns, dataColumsType, dataTooltips, 'Sélection: '+str( 'Média - individu') , "media_person", Media_personDataSources(self).getFkIndexLibBack() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.mainWindowEditMode.emit( int(rowId) , "Edit mode selection")
				
	
	def onStoreState( self ) :
		self.tableView.storeState()

	def onNewRow(self):
		
		self.mainWindowEditMode.emit( -1 , "Edit mode New")

	def onEditRow(self, rowId):

		self.mainWindowEditMode.emit( int(rowId) , "Edit mode")

	def onDeleteRow(self, rowdbId, rowIndex):

		QApplication.setOverrideCursor(Qt.WaitCursor)
		self.myMedia_persons = Media_persons( self.db )
		media_personObj = self.myMedia_persons.select( rowdbId )
		QApplication.restoreOverrideCursor()

		if media_personObj :

			dlg = Media_personDeleteConfirm(self.db, self, rowdbId, media_personObj.getName()) 
			
			returnCode = dlg.exec()
			if returnCode == QDialog.Accepted :
				self.proxyModel.removeRow(rowIndex)				
				self.mainWindowRemoveRow.emit( rowdbId, "row removed")
				
class Media_personDeleteConfirm(QDialog):

	parent = None

	def __init__(self, db, parent=None, rowid=-1, libRecord="" ):
		super().__init__(parent)

		self.db = db
		self.parent = parent
		self.rowId = rowid
		self.libRecord = libRecord

		self.setWindowTitle("Média - individu: suppression de #" + str(self.rowId)  )		

		QBtn = QDialogButtonBox.Ok | QDialogButtonBox.Cancel

		self.buttonBox = QDialogButtonBox(QBtn)
		self.buttonBox.accepted.connect(self.onOk)
		self.buttonBox.rejected.connect(self.onCancel)

		self.layout = QGridLayout() 
		
		# http://srinikom.github.io/pyside-docs/PySide/QtGui/QStyle.html#PySide.QtGui.PySide.QtGui.QStyle.StandardPixmap
		from PyQt5.QtWidgets import QApplication, QStyle
		from PyQt5.QtGui import QIcon

		labelIcon = QLabel("icon")
		dialogIcon = QIcon(QApplication.style().standardIcon(QStyle.SP_MessageBoxQuestion) )       
		labelIcon.setPixmap( dialogIcon.pixmap(48,48)  )
		self.layout.addWidget(labelIcon,0,0)				
		
		message = QLabel("Confirmez-vous la suppression de : \n" )
		self.layout.addWidget(message,0,1)

		data = QLabel(str(self.libRecord))
		self.layout.addWidget(data,1,1)
			
		self.layout.addWidget(self.buttonBox,2,1)
		self.setLayout(self.layout)

	def onCancel( self ) :
		self.reject()

	def onOk( self ) :
		self.myMedia_persons = Media_persons( self.db )
		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.myMedia_persons.delete( self.rowId )
		QApplication.restoreOverrideCursor()

		if result is False :
			self.parent.mainWindowUserMessage.emit( 1, "ERREUR SQL : " + str(self.myMedia_persons.lastError) )
			self.reject()
		else :
			self.parent.mainWindowUserMessage.emit( 0 , "Suppresion effectuée. (id:"+str(self.rowId) + ")")
			self.parent.mainWindowListMode.emit()
			self.accept()


class Media_personEditWidget(QWidget):

	mainWindowUserMessage = pyqtSignal( int, str )
	mainWindowListMode = pyqtSignal( )
	enterEditSignal = pyqtSignal( int )
	closeEdit = pyqtSignal( int, int )	
	validEditSignal = pyqtSignal( bool, int )
	valuesChanged = False
	
	recPos = 0
	siblingIdList = []
	siblingOrderField = "1"
	siblingOrderBy = ""

	def __init__(self, db, rootMedia, rowId, parentId, siblingIdList, siblingOrderField, siblingOrderBy, rowDataObj = None ):		
		
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia
		self.parentId = parentId
		
		self.siblingIdList = siblingIdList		
		self.siblingOrderField = siblingOrderField # Nom Sql du champ sur lequel porte le tri		
		sqlOrderBy = ['DESC','ASC'][siblingOrderBy]
		self.siblingOrderBy = sqlOrderBy		
		
		self.rowDataObj = rowDataObj

		self.myMedia_persons = Media_persons( self.db )

		media_personObj = self.myMedia_persons.select( rowId )
		if media_personObj is False :
			self.mainWindowUserMessage.emit( 0 , "Cet enregistrement a été supprimé.")
			self.rowId = -1
		else :
			self.rowId = rowId

		if self.rowId in self.siblingIdList:
			self.recPos = self.siblingIdList.index(self.rowId)
		else:
			self.recPos = 0

		self.initUI()
		
		self.setObjectName( "Media_personEditWidget" )

	def showEvent(self, event):
		super(Media_personEditWidget, self).showEvent(event)
		self.WidgetsForm.setFocusOnFirstWidget()	
		self.setPrevNextButtonState()		
		
	def initUI(self):

		pageLayout = QGridLayout()
		pageButtonLayout = QHBoxLayout()
		
		self.closeBtn = QPushButton("Fermer", self)
		self.closeBtn.setToolTip( "Fermer et afficher la liste Média - individu" )
		self.closeBtn.setIcon(QIcon('img/form_back.png'))
		self.closeBtn.clicked.connect(self.onClose)
		pageButtonLayout.addWidget(self.closeBtn)

		self.validBtn = QPushButton("Sauver", self)
		self.validBtn.setToolTip( "Enregistrer les modifications Média - individu" )
		self.validBtn.setIcon(QIcon('img/form_save.png'))		
		self.validBtn.clicked.connect(self.onValid)
		pageButtonLayout.addWidget(self.validBtn)

		self.updateMessage = QLineEdit( self )
		self.updateMessage.setStyleSheet("QLineEdit { background-color : rgb(255, 255, 255); color: red; border : 0px ;}")
		self.updateMessage.setToolTip( "Dernière erreur rencontrée lors de l'édition" )
		pageButtonLayout.addWidget(self.updateMessage)

		self.prevBtn = QPushButton("Précédent", self)
		self.prevBtn.setToolTip( "Média - individu précédent" )
		self.prevBtn.setIcon(QIcon('img/form_prev.png'))		
		self.prevBtn.clicked.connect(self.onPrev)
		pageButtonLayout.addWidget(self.prevBtn)

		self.nextBtn = QPushButton("Suivant", self)
		self.nextBtn.setToolTip( "Média - individu suivant" )
		self.nextBtn.setIcon(QIcon('img/form_next.png'))		
		self.nextBtn.clicked.connect(self.onNext)
		pageButtonLayout.addWidget(self.nextBtn)
		self.nextBtn.setLayoutDirection(Qt.RightToLeft)		

		self.WidgetsForm = Media_personEditDisplayWidgets(self, self.db, self.rootMedia, self.rowId, self.parentId, self.rowDataObj)
		
		scroll = QScrollArea(self)
		scroll.setWidget(self.WidgetsForm)
		scroll.setWidgetResizable(True)
		pageLayout.addWidget(scroll)

		pageLayout.addLayout(pageButtonLayout, 1,0)

		self.setLayout(pageLayout)

	def enableNavButtons( self ) :
		self.nextBtn.setEnabled( True )
		self.prevBtn.setEnabled( True )

	def setPrevNextButtonState( self ) :
		
		if self.recPos <= 0 :
			self.prevBtn.setEnabled( False )
		else :
			self.prevBtn.setEnabled( True )

		if self.recPos >= len(self.siblingIdList)-1 :
			self.nextBtn.setEnabled( False )
		else :
			self.nextBtn.setEnabled( True )	
			
	def onPrev( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
					quitForm = False
					
		if quitForm is True :					
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos - 1			
			if self.recPos < 0 :
				self.recPos = 0		
			
			media_personObj = self.myMedia_persons.select( self.siblingIdList[self.recPos]  )
			
			if media_personObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.myMedia_persons.selectCount == 1 :
				self.rowDataObj = media_personObj			
				self.rowId = media_personObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()
		
	def onNext( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
				quitForm = False
			
		if quitForm is True :
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos + 1
			if self.recPos >= len(self.siblingIdList) :
				self.recPos = len(self.siblingIdList) - 1
			
			media_personObj = self.myMedia_persons.select( self.siblingIdList[self.recPos]  )
			
			if media_personObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.myMedia_persons.selectCount == 1 :
				self.rowDataObj = media_personObj
				self.rowId = media_personObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()

	def onValid( self ) :
		self.updateMessage.setText( "" )

		myMedia_personObj = Media_person()

		myMedia_personObj.setId( self.rowId )

						
			
		myMedia_personObj.setMedia_id( self.rowDataObj.getMedia_id() ) 				

						
			
		myMedia_personObj.setPerson_id( self.rowDataObj.getPerson_id() ) 				

			
			
		# dd/MM/yyyy hh:mm
		myMedia_personObj.setDatecreated( self.WidgetsForm.InputDatecreated.dateTime().toString('yyyy-MM-dd hh:mm') )					

		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.myMedia_persons.update( myMedia_personObj )
		QApplication.restoreOverrideCursor()

		self.updateMessage.setText( "" )
		self.mainWindowUserMessage.emit( -1 , "" )

		if (result is False) :
			self.updateMessage.setText( "ERREUR SQL : " + str(self.myMedia_persons.lastError) )
			self.updateMessage.setToolTip( "ERREUR SQL : " + str(self.myMedia_persons.lastError) )
			self.validEditSignal.emit( False, self.rowId )
		else :
			self.rowId = self.myMedia_persons.lastInsertId
			myMedia_personObj.setId( self.rowId )
			self.mainWindowUserMessage.emit( 0 , "Mise à jour effectuée. (id:"+str(self.rowId) + ")")
			self.WidgetsForm.displayData( self.rowId )
			self.validEditSignal.emit( True, self.rowId )

	def onClose( self ) :

		if self.valuesChanged is True :

			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.Yes:
				self.mainWindowUserMessage.emit( -1 , "" )
				self.closeEdit.emit(True, self.rowId)
				
		else :
			self.mainWindowUserMessage.emit( -1 , "" )
			self.closeEdit.emit(False, self.rowId)
	
	def onValuesChanged( self, boolState ) :
		self.valuesChanged = boolState
		self.validBtn.setEnabled( boolState )
		
		
class Media_personEditDisplayWidgets(QWidget):

	def __init__(self, parent, db, rootMedia, rowId, rowParentId, rowDataObj):
		
		QWidget.__init__(self, parent)

		self.db = db
		self.rootMedia = rootMedia
		self.parent = parent
		self.rowId = rowId
		self.rowParentId = rowParentId
		self.rowDataObj = rowDataObj

		self.myMedia_persons = Media_persons( self.db )

		gridLayout = QGridLayout()

		gridLayout.setSpacing(10)

		rowNumber = 1
		
		labelId = QLabel('Id')
		labelId.setToolTip( 'Identifiant interne (Id)' )
		labelId.setAlignment(Qt.AlignRight | Qt.AlignVCenter)		
		self.InputId = QLineEdit()
		self.InputId.setMaximumWidth( self.getWidgetWidth( self.InputId, 5 )  )		
		self.InputId.setEnabled( False )
		self.InputId.setStyleSheet("color: #CCCCCC;")
		gridLayout.addWidget(labelId, rowNumber, 0)
		gridLayout.addWidget(self.InputId, rowNumber, 1)

		rowNumber = rowNumber + 1
			
			
		labelMedia_id = QLabel('Média')
		labelMedia_id.setToolTip( 'Média' )
		labelMedia_id.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputMedia_id = QLineEdit()
		self.InputMedia_id.setObjectName( "Media_personEditMedia_id" )
		self.InputMedia_id.installEventFilter(self)
		gridLayout.addWidget(labelMedia_id, rowNumber, 0)
		gridLayout.addWidget(self.InputMedia_id, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Média: valeur obligatoire' )
		self.InputMedia_id.setReadOnly(True)
		self.selectBtn = QPushButton("...", self)
		self.selectBtn.clicked.connect(self.onSelectMedia_id)
		gridLayout.addWidget(self.selectBtn, rowNumber, 3) 
		rowNumber = rowNumber + 1
			
			
		labelPerson_id = QLabel('Individu')
		labelPerson_id.setToolTip( 'Individu lié au média' )
		labelPerson_id.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputPerson_id = QLineEdit()
		self.InputPerson_id.setObjectName( "Media_personEditPerson_id" )
		self.InputPerson_id.installEventFilter(self)
		gridLayout.addWidget(labelPerson_id, rowNumber, 0)
		gridLayout.addWidget(self.InputPerson_id, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Individu: valeur obligatoire' )
		self.InputPerson_id.setReadOnly(True)
		self.selectBtn = QPushButton("...", self)
		self.selectBtn.clicked.connect(self.onSelectPerson_id)
		gridLayout.addWidget(self.selectBtn, rowNumber, 3) 
		rowNumber = rowNumber + 1
							
			
		labelDatecreated = QLabel('Créé le')
		labelDatecreated.setToolTip( 'Date de création' )
		labelDatecreated.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDatecreated = QDateTimeEdit()
		self.InputDatecreated.setObjectName( "Media_personEditDatecreated" )
		self.InputDatecreated.installEventFilter(self)
		self.InputDatecreated.setMaximumWidth( self.getWidgetWidth( self.InputDatecreated, 16 )  )		
		gridLayout.addWidget(labelDatecreated, rowNumber, 0)
		gridLayout.addWidget(self.InputDatecreated, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Créé le: valeur obligatoire' )
		self.InputDatecreated.dateTimeChanged.connect(self.onDateEditValuesChanged)	
		self.InputDatecreated.setDisplayFormat("dd/MM/yyyy hh:mm")
		self.InputDatecreated.setDateTimeRange(QDateTime(1, 1, 1, 0, 0), QDateTime(2999, 12, 31, 23, 59))
		self.InputDatecreated.setSpecialValueText(' ')
		self.InputDatecreated.setReadOnly(True)
		if True is False :
			self.InputDatecreated.setCalendarPopup(True)	


		self.setLayout(gridLayout)
		self.displayData( self.rowId, self.rowDataObj )
		

			
	 # Affichage de la fenêtre pop-up de selection de la clé étrangère
	def onSelectMedia_id( self ) :

		from libview.mediaselect import MediaSelect
		dialog = MediaSelect( self, self.db, self.rootMedia  )
		dialog.selectRowById( self.parent.rowDataObj.getMedia_id() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.InputMedia_id.setText( str( rowLib ) )	
				self.parent.rowDataObj.setMedia_id( rowId )
				self.parent.onValuesChanged( True ) 
			else :
				self.InputMedia_id.setText( None )	
				self.parent.rowDataObj.setMedia_id( None )
				self.parent.onValuesChanged( True ) 

			
	 # Affichage de la fenêtre pop-up de selection de la clé étrangère
	def onSelectPerson_id( self ) :

		from libview.personselect import PersonSelect
		dialog = PersonSelect( self, self.db, self.rootMedia  )
		dialog.selectRowById( self.parent.rowDataObj.getPerson_id() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.InputPerson_id.setText( str( rowLib ) )	
				self.parent.rowDataObj.setPerson_id( rowId )
				self.parent.onValuesChanged( True ) 
			else :
				self.InputPerson_id.setText( None )	
				self.parent.rowDataObj.setPerson_id( None )
				self.parent.onValuesChanged( True ) 

			


	

	

	


	

	

	



	

	

	



	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
	
	# Retourne la largeur necessaire pour un widget (QlineEdit ...) 
	# en fonctiion du nombre de caractères souhaité.
	def getWidgetWidth( self, widget, charmax ) :
		
		fm = widget.fontMetrics()
		try :
			m = widget.textMargins()
		except Exception as e:
			m = QMargins(8,8,8,8)
		c = widget.contentsMargins()
		w = charmax*fm.width('x')+m.left()+m.right()+c.left()+c.right()
		w = w + 8 # margins
		return w
			
	def onLineEditValuesChanged(self , text) :
		
		self.parent.onValuesChanged( True )

	def onTextEditValuesChanged(self):
		
		self.parent.onValuesChanged( True )
		
	def onComboValuesChanged(self, newIndex):
		
		self.parent.onValuesChanged( True )
				
	def onCheckBoxValuesChanged( self ,state) :
		self.parent.onValuesChanged( True )
			
	def onDateEditValuesChanged( self, newDate ) :
		
		self.parent.onValuesChanged( True )
				
	def displayData( self, rowId, dataObj = None) :

		self.rowId = rowId		

		self.parent.enterEditSignal.emit( self.rowId )
		
		if self.rowId == -1 :

			self.parent.mainWindowUserMessage.emit( 1 , "Formulaire vide (creation ou recréation)" )
		
		if dataObj is None :			
			media_personObj = self.myMedia_persons.select( self.rowId )
			if media_personObj is False :
				media_personObj = Media_person()			
		else :
			media_personObj = dataObj
						
		self.InputId.setText( str(media_personObj.getId()) )		

			
			
		from libdbcustom.media import Medias
		media = Medias( self.db )
		mediaObj = media.select( media_personObj.getMedia_id()  ) 
		if mediaObj is not False :
			self.InputMedia_id.setText( str(  mediaObj.getName() ) )
		else :
			self.InputMedia_id.setText( '' ) 

			
			
		from libdbcustom.person import Persons
		person = Persons( self.db )
		personObj = person.select( media_personObj.getPerson_id()  ) 
		if personObj is not False :
			self.InputPerson_id.setText( str(  personObj.getName() ) )
		else :
			self.InputPerson_id.setText( '' ) 

									
			
		#myQdate = Util.getQDatetimeFromString( media_personObj.getDatecreated(), defaultValue = QDateTime.currentDateTime().toString('dd/MM/yyyy H:i:s') )
		myQdate = Util.getQDatetimeFromString( media_personObj.getDatecreated() )
		self.InputDatecreated.setDateTime( myQdate ) 

		self.parent.onValuesChanged( False )


	def setFocusOnFirstWidget( self ) :

		from PyQt5.QtWidgets import  QLineEdit, QTextEdit, QDateEdit, QDateTimeEdit, QDateTimeEdit, QCheckBox, QComboBox,QDial, QDoubleSpinBox, QRadioButton, QSlider, QSpinBox, QTimeEdit, QFontComboBox
		for widget in self.children():
			if widget.isEnabled() :
				if 	isinstance(widget, QLineEdit) or \
					isinstance(widget, QTextEdit) or \
					isinstance(widget, QDateEdit) or \
					isinstance(widget, QDateTimeEdit) or \
					isinstance(widget, QCheckBox) or \
					isinstance(widget, QComboBox) or \
					isinstance(widget, QDial) or \
					isinstance(widget, QDoubleSpinBox) or \
					isinstance(widget, QRadioButton) or \
					isinstance(widget, QSlider) or \
					isinstance(widget, QSpinBox) or \
					isinstance(widget, QTimeEdit) or \
					isinstance(widget, QFontComboBox) \
					:
					#AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					widget.setFocus()					
					return

	def eventFilter(self, widgetObj, event):
		
		if event.type() == QtCore.QEvent.FocusIn:			
			self.focusWidget = widgetObj
	
		return super(Media_personEditDisplayWidgets, self).eventFilter(widgetObj, event)
    
	def restoreFocusWidget( self ) :		
		#AppLog.log( LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		AppLog.log( LOG_DEBUG , LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		self.focusWidget.setFocus()	
            

class Media_personDataSources() :
	
	# Jointure sur table(s):  media (b_name)  person (c_name) 	
	dataColumns = [ 'Id', 'Média', 'Individu', 'Créé le',  'Nom du média', 'Individu' ]
	dataSqlFields = [ 'id', 'media_id', 'person_id', 'datecreated',  'b1.name', 'c2.name' ]	
	dataColumsType = ['INTEGER', 'INTEGER', 'INTEGER', 'DATETIME',  'TEXT', 'TEXT' ]
	dataTooltips = [ 'Id', 'Média', 'Individu lié au média', 'Date de création',  'Nom du média', 'Nom de lʼindividu' ]
			
	def __init__(self, db ):
		self.db = db
		pass
		
	def selectAll( self , field_name = None, field_value = None ) :
		
		if field_name is None and field_value is None :
			return self.select()
		else :
			return self.selectById( field_name, field_value )
			
					
	def select( self ) :
		
		dataRows = []
		
		self.myMedia_persons = Media_persons( self.db )
		media_personListAll = self.myMedia_persons.selectAll()
		if media_personListAll :
			i = 0
			for media_person in media_personListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(media_person.getId()))
				dataRows.append( [ media_person.getId(),  media_person.getMedia_id(),  media_person.getPerson_id(),  media_person.getDatecreated(),  media_person.getB1_name(), media_person.getC2_name() ] )
				i = i + 1
			
		return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )


	def selectById( self, where_clause, fields_values ) :
		
		dataRows = []
				
		self.myMedia_persons = Media_persons( self.db )
		media_personListAll = self.myMedia_persons.selectById( where_clause, fields_values )
		if media_personListAll : 
			i = 0
			for media_person in media_personListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(media_person.getId()))
				dataRows.append( [ media_person.getId(),  media_person.getMedia_id(),  media_person.getPerson_id(),  media_person.getDatecreated(),  media_person.getB1_name(), media_person.getC2_name() ] )
				i = i + 1
				
			return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
		return ( None, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
	# Retourner le champ #x lors de la selection par pop-up (Fk)
	def getFkIndexLibBack( self ) :
		return 1
