#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

		*** Source autogénéré ***

		Pour la table : metadata.

		----------------------------
		Paramètre.

		Paramètres et métadonnées.
		----------------------------

		05/24/2023 14:31

		lvardon@laposte.net - 2021

		Licence libre

'''
import sys

from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QAction, QMenu, QApplication, QWidget, QToolTip, QPushButton, QMessageBox,QTableView
from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, QHBoxLayout, QScrollArea
from PyQt5.QtWidgets import QLabel, QLineEdit, QTextEdit, QComboBox, QCheckBox, QDateTimeEdit
from PyQt5.QtGui import QCursor, QFont, QIcon
from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot, QModelIndex, QDateTime, QSettings

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
from lib.util import Util
from libwidget.tableModel import TableModel, TableHeaderView, TableProxyModel
import lib.setting as AppSetting
import lib.log as AppLog
from lib.sqlitedb import Sqlitedb
from libwidget.tableView import TableView,TableCellImageDelegate

from libview.metadata import MetadataDataSources

class MetadataSelect(QDialog):

	def __init__(self, parent, db, rootmedia  ):
		super(MetadataSelect, self).__init__(parent)

		self.parent = parent
		self.db = db
		self.rootMedia = rootmedia
		self.Reset = False
		self.dialogOk = True

		dataRows, dataColumns, self.dataSqlNames, dataColumsType, dataTooltips = MetadataDataSources( self.db ).selectAll()

		self.loadData( dataRows, dataColumns, dataColumsType, dataTooltips, 'Selection: '+str( 'Paramètre') , "select_metadata", MetadataDataSources(self).getFkIndexLibBack() )

	def loadData( self, dataRows, dataColumns, dataColumsType, dataTooltips, windowTitle, viewName, indexLibBack  = 1 ) :

		AppLog.log( LOG_INFO , 'Selectdialog, loadData()')

		self.indexLibBack = indexLibBack

		layout = QVBoxLayout(self)
		self.viewName = viewName
		self.tableView = TableView( self, viewName, "source_select" )

		model = TableModel(viewName, dataRows, dataColumns, dataColumsType, dataTooltips)

		self.proxyModel = TableProxyModel(self)
		self.proxyModel.setSortRole(Qt.UserRole + 2)
		self.proxyModel.setSourceModel(model)
		self.tableView.setModel(self.proxyModel)
		self.tableView.clicked.connect(self.viewClicked)

		self.tableView.resizeColumnsToContents()

		self.filterValue = QLineEdit()
		layout.addWidget(self.filterValue)
		self.filterValue.textChanged.connect(self.onfilterValueChanged)

		self.tableView.doubleClicked.connect(self.onRowDoubleClick)

		self.initDelegates(  self.tableView )

		layout.addWidget(self.tableView)

		# OK and Cancel buttons
		buttons = QDialogButtonBox(
			QDialogButtonBox.Reset | QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
			Qt.Horizontal, self)

		buttons.accepted.connect(self.accept)
		buttons.rejected.connect(self.reject)
		buttons.button(QDialogButtonBox.Reset).clicked.connect(self.onReset)

		buttons.button(QDialogButtonBox.Ok).clicked.connect(self.onOk)
		buttons.button(QDialogButtonBox.Cancel).clicked.connect(self.onCancel)

		buttons.button(QDialogButtonBox.Ok).setToolTip('Valider')
		buttons.button(QDialogButtonBox.Cancel).setToolTip('Annuler')
		buttons.button(QDialogButtonBox.Reset).setToolTip('Effacer la valeur')

		layout.addWidget(buttons)

		self.setGeometry(300, 300, 660, 480)
		self.setWindowTitle(windowTitle)

		iconW, iconH, rowHeight = self.getTableViewIniParams( "tablelistviewMetadata" )
		self.tableView.verticalHeader().setDefaultSectionSize(int(rowHeight))

	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)

	def getTableViewIniParams ( self, tableViewName) :

		iniConfigFile = "config/app.ini"
		settings=QSettings(iniConfigFile, QSettings.IniFormat)

		rowHeight = 0
		if settings.value(tableViewName + '/rowHeight') is not None:
			rowHeight = settings.value( tableViewName +'/rowHeight')

		iconW = 0
		if settings.value( tableViewName + '/rowIconW') is not None:
			iconW = settings.value( tableViewName +'/rowIconW')

		iconH = 0
		if settings.value( tableViewName +'/rowIconH') is not None:
			iconH = settings.value( tableViewName +'/rowIconH')

		AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" rowHeight="+ str(rowHeight))
		AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" iconW x IconH="+ str(iconW) + " x " + str(iconH))

		return iconW, iconH, rowHeight

	def initProperties ( self, tableName,  rootMedia )	:
		self.tableName = tableName
		self.rootMedia = rootMedia

	def initDelegates( self, thistableView ) :


		thistableView.verticalHeader().setDefaultSectionSize(40)

	def selectRowById( self, rowId ) :

		if rowId is None :
			return

		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel

		for i in range(0, row) :
			#print("SELECT ROW , db id="+ str(model.data( model.index(i, 0), Qt.UserRole )))
			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :
				self.tableView.selectRow( i )
				return


	def onOk( self ):
		AppLog.log( LOG_INFO , 'On Ok Button')
		self.dialogOk = True
		self.accept()

	def onCancel( self ):
		AppLog.log( LOG_INFO , 'On Cancel Button')
		self.dialogOk = False
		self.close()

	def onReset( self ) :
		AppLog.log( LOG_INFO , 'On Reset Button')
		self.dialogOk = True
		self.Reset = True
		self.accept()

	def showEvent( self, event ) :
		AppLog.log( LOG_INFO , 'Show event.')
		self.onLoadState()
		self.filterValue.setFocus()		

	@pyqtSlot(QModelIndex)
	def onRowDoubleClick(self, index):
		row = index.row()
		column = index.column()
		self.printDebug("row,col = ", row, column)

		rowId = self.proxyModel.index(index.row(),0).data()
		self.tableView.selectRow(row)
		self.accept()

	def contextMenuEvent(self, event):

		gp = event.globalPos()
		vp_pos = self.tableView.viewport().mapFromGlobal(gp)
		rowNum = self.tableView.rowAt(vp_pos.y())
		colNum = self.tableView.columnAt(event.x())

		# print("select dialog, context " , rowNum, " ", colNum)

		self.menu = QMenu(self)

		storeStateAction = QAction('Enregistrer la disposition', self)
		storeStateAction.triggered.connect(lambda checked : self.onStoreState())
		self.menu.addAction(storeStateAction)

		self.menu.popup(QCursor.pos())

	def onStoreState( self ) :

		self.tableView.storeState()

		width = self.frameGeometry().width()
		height = self.frameGeometry().height()
		left = self.frameGeometry().left()
		top = self.frameGeometry().top()

		AppSetting.set("source_select/" + self.viewName+ "_window" , str(left) + "," + str(top) + ","+str(width) + ","+ str(height))

	def onLoadState( self) :

		self.tableView.loadState()
		windowGeometry = AppSetting.get("source_select/" + self.viewName + "_window", "").split(",")

		if windowGeometry[0] != "" :
			self.resize( int(windowGeometry[2]), int(windowGeometry[3]) ) # W, h
			self.move( int(windowGeometry[0]), int(windowGeometry[1]) ) # top, left

	def onfilterValueChanged(self , text) :

		self.proxyModel.setFilterValue( str(text) )
		self.proxyModel.applyFilters()

	def viewClicked(self, clickedIndex):

		row=clickedIndex.row()
		self.tableView.selectRow(row)

	def getValue(self):

		AppLog.log( LOG_INFO , 'Recherche de la valeur selectionnée')

		# Cancel button
		if self.dialogOk is False :
			AppLog.log( LOG_INFO , 'Index de la valeur selectionnée: None (bouton Cancel)')
			return None, None

		indexes = self.tableView.selectionModel().selectedRows()

		try :
			if self.Reset is False :
				index = self.tableView.selectedIndexes()[0] # Rowid
				rowId = self.tableView.model().data(index)
				index = self.tableView.selectedIndexes()[self.indexLibBack] # Lib/Text à afficher
				rowLib = self.tableView.model().data(index)

				AppLog.log( LOG_INFO , 'Valeur selectionnée: '+str(rowId) +', libellé= ' + str(rowLib) )

				return (rowId, str(rowLib))
			else :
				AppLog.log( LOG_INFO , 'Index de la valeur selectionnée: -1, libellé= <vide>')
				return (-1, "")
		except :
			e = sys.exc_info()[0]
			AppLog.log( LOG_ERROR , str(e))

		AppLog.log( LOG_INFO , 'Index de la valeur selectionnée: None')
		return None, None


