#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

		*** Source autogénéré *** 
		
		Pour la table : person.

		----------------------------
		Individu.
		
		Liste des individus.
		----------------------------		

		05/24/2023 14:31

		lvardon@laposte.net - 2021

		Licence libre

'''
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QAction, QMenu, QApplication, QWidget, QToolTip, QPushButton, QMessageBox, QFileDialog
from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, QHBoxLayout, QScrollArea
from PyQt5.QtWidgets import QLabel, QLineEdit, QTextEdit, QComboBox, QCheckBox, QDateTimeEdit
from PyQt5.QtGui import QCursor, QFont, QIcon
from PyQt5.QtCore import Qt, QEvent, pyqtSignal, pyqtSlot, QModelIndex, QDateTime, QMargins, QSettings
from PyQt5 import QtCore
		
from lib.sqlitedb import Sqlitedb
from libdbcustom.person import Person, Persons

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
from lib.util import Util
from libwidget.tableModel import TableModel, TableProxyModel
from libwidget.tableView import TableView,TableCellImageDelegate

import lib.setting as AppSetting
import lib.log as AppLog

class PersonListWidget(QWidget):

	mainWindowEditMode =  pyqtSignal( int, str )
	mainWindowRemoveRow =  pyqtSignal( int, str )	
	mainWindowListMode = pyqtSignal()
	mainWindowUserMessage = pyqtSignal( int, str )
	cellClickSignal =  pyqtSignal( int )
	cellDoubleClickSignal =  pyqtSignal( int )
	selectionSignal = pyqtSignal( int )
	
	def __init__(self , db, rootMedia, sourceOrigin = "", where_clause = None, fields_values = None):
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia 
		self.sourceOrigin = sourceOrigin
		
		self.where_clause = where_clause
		self.fields_values = fields_values
		
		self.initUI()

		self.initDelegates(  self.tableView )
		
		iconW, iconH, rowHeight = self.getTableViewIniParams( "tablelistviewPerson" )
		self.tableView.verticalHeader().setDefaultSectionSize(int(rowHeight))

		try :
			self.iconImageDelegate.icoW = int(iconW) 
			self.iconImageDelegate.icoH = int(iconH)	
		except Exception as e:
			pass
		try :
			self.urlImageDelegate.icoW = int(iconW) 
			self.urlImageDelegate.icoH = int(iconH)					
		except Exception as e:
			pass
		
		self.setObjectName( "PersonListWidget" )

	def initDelegates( self, thistableView ) :
				
		self.privateImageDelegate = TableCellImageDelegate(self, "person", {'0' : 'img/dataicons/empty.png', '1' : 'img/dataicons/private-yes.png', '2' : 'img/dataicons/private-yes.png'} , self.rootMedia, 26, 26)
		thistableView.setItemDelegateForColumn(6, self.privateImageDelegate)

		self.iconImageDelegate = TableCellImageDelegate(self, "person", None , self.rootMedia, 40, 40)
		thistableView.setItemDelegateForColumn(7, self.iconImageDelegate)

		self.sexImageDelegate = TableCellImageDelegate(self, "person", {'F' : 'img/dataicons/ico-sexe-f.png', 'M' : 'img/dataicons/ico-sexe-m.png', 'I' : 'img/dataicons/unknown.png'} , self.rootMedia, 26, 26)
		thistableView.setItemDelegateForColumn(8, self.sexImageDelegate)

		
		thistableView.verticalHeader().setDefaultSectionSize(40)

	def initUI(self):
		
		self.layout = QVBoxLayout()
		self.setLayout(self.layout)
				
		self.proxyModel = TableProxyModel(self)
		
		self.proxyModel.setSortRole(Qt.UserRole + 2)
		
		self.tableView = TableView( self, "person", self.sourceOrigin )	
				
		self.loadData( self.where_clause , self.fields_values ) 
	
		self.tableView.setModel(self.proxyModel)
		
		self.tableView.resizeColumnsToContents()

		self.tableView.cellClickSignal.connect(self.onCellClic)
		self.tableView.cellDoubleClickSignal.connect(self.onCellDoubleClic)
	
		selection_model = self.tableView.selectionModel()
		selection_model.selectionChanged.connect(self.onSelectionChanged)      
		
		self.filterValue = QLineEdit()
		self.layout.addWidget(self.filterValue)
		self.filterValue.textChanged.connect(self.onfilterValueChanged)
		
		self.layout.addWidget(self.tableView)

	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
			

	def getTableViewIniParams ( self, tableViewName) :
		
		iniConfigFile = "config/app.ini"
		settings=QSettings(iniConfigFile, QSettings.IniFormat)

		rowHeight = 0
		if settings.value(tableViewName + '/rowHeight') is not None:			
			rowHeight = settings.value( tableViewName +'/rowHeight')						
			
		iconW = 0
		if settings.value( tableViewName + '/rowIconW') is not None:						
			iconW = settings.value( tableViewName +'/rowIconW')				
		
		iconH = 0			
		if settings.value( tableViewName +'/rowIconH') is not None:			
			iconH = settings.value( tableViewName +'/rowIconH')	

		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" rowHeight="+ str(rowHeight))
		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" iconW x IconH="+ str(iconW) + " x " + str(iconH))
		
		return iconW, iconH, rowHeight
		
	@pyqtSlot('QItemSelection', 'QItemSelection')
	def onSelectionChanged(self, selected, deselected):
		try :
			rowId =  selected.indexes()[0].data()
			self.selectionSignal.emit( rowId )
		except Exception as e:
			pass		
		
	def onfilterValueChanged(self , text) :
		self.proxyModel.setFilterValue( str(text) )		
		self.proxyModel.applyFilters()

	def onCellClic(self, rowId ):
		self.cellClickSignal.emit(rowId)

	def onCellDoubleClic(self, rowId ):
		self.cellDoubleClickSignal.emit(rowId)
		self.onEditRow(rowId)
		
	def reloadData(self) :
		self.loadData(self.where_clause, self.fields_values)		
		
	def loadData( self, where_clause , fields_values ) :
		
		self.where_clause = where_clause
		self.fields_values = fields_values
				
		dataSource = PersonDataSources( self.db )
		dataRows, dataColumns, self.dataSqlNames, dataColumnsType, dataToolTips = dataSource.selectAll( self.where_clause, self.fields_values )
		
		self.model = TableModel("person", dataRows, dataColumns, dataColumnsType, dataToolTips)

		self.proxyModel.setSourceModel(self.model)
		
		self.tableView.resizeColumnsToContents()				
				
	def dumpDatas( self) :
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		for i in range(0, row) :
			self.printDebug("--- Row : ", i)
			for j in range(0, col) :
				self.printDebug( 
						self.model.data( self.model.index(i, j), Qt.UserRole + 1 ),
						" - ",
						self.model.data( self.model.index(i, j), Qt.UserRole ) 
					)

	# Nom de la colonne sur laquelle porte le tri 
	def getSqlOrderByField( self) :
		orderByFieldPos = self.tableView.hView.sortIndicatorSection()
		orderByField = self.dataSqlNames[orderByFieldPos]
		
		return orderByField
		
	# Index du sens de tri 'DESC' = 0 'ASC' = 1
	def getSqlOrderByOrder( self) :
		orderBy = self.tableView.hView.sortIndicatorOrder()
		return orderBy
		
		
	# Liste des id de la liste affichée
	def getIdList( self ) :
		
		idList = []
		
		indexId = 0 # id column is at pos 0
		
		row = self.proxyModel.rowCount()
		model = self.proxyModel
		
		for i in range(0, row) :			
			idList.append( model.data( model.index(i, indexId ), Qt.UserRole ) )
		
		return idList
			
	
	def selectRowById( self, rowId ) :

		if rowId is None :
			return
			
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel
		
		for i in range(0, row) :
			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :				
				self.tableView.selectRow( i )
				return

	def getRowData( self, rowId ) :

		myPersonObj = Person( )

		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel

		for i in range(0, row) :
			#print('row:', i, " model rowid=", self.model.data( self.model.index(i, 0), Qt.UserRole ), ' == ',  rowId, ' **', self.model.data( self.model.index(i, 1), Qt.UserRole ) )

			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :

				myPersonObj.setId( model.data( model.index(i, 0 ), Qt.UserRole  )  )
				myPersonObj.setName( model.data( model.index(i, 1 ), Qt.UserRole  )  )
				myPersonObj.setKey( model.data( model.index(i, 2 ), Qt.UserRole  )  )
				myPersonObj.setDescription( model.data( model.index(i, 3 ), Qt.UserRole  )  )
				myPersonObj.setFirstname( model.data( model.index(i, 4 ), Qt.UserRole  )  )
				myPersonObj.setLastname( model.data( model.index(i, 5 ), Qt.UserRole  )  )
				myPersonObj.setPrivate( model.data( model.index(i, 6 ), Qt.UserRole  )  )
				myPersonObj.setIcon( model.data( model.index(i, 7 ), Qt.UserRole  )  )
				myPersonObj.setSex( model.data( model.index(i, 8 ), Qt.UserRole  )  )
				myPersonObj.setNotesearch( model.data( model.index(i, 9 ), Qt.UserRole  )  )
				myPersonObj.setFirstnameprefix( model.data( model.index(i, 10 ), Qt.UserRole  )  )
				myPersonObj.setFirtsnameothers( model.data( model.index(i, 11 ), Qt.UserRole  )  )
				myPersonObj.setLastnamesuffix( model.data( model.index(i, 12 ), Qt.UserRole  )  )
				myPersonObj.setSurname( model.data( model.index(i, 13 ), Qt.UserRole  )  )
				myPersonObj.setDatecreated( model.data( model.index(i, 14 ), Qt.UserRole  )  )

				return myPersonObj

		return myPersonObj

	def contextMenuEvent(self, event):

		gp = event.globalPos()
		vp_pos = self.tableView.viewport().mapFromGlobal(gp)
		rowNum = self.tableView.rowAt(vp_pos.y())
		colNum = self.tableView.columnAt(event.x())

		self.menu = QMenu(self)
			
		if rowNum >= 0 :

			index = self.tableView.indexAt(vp_pos)
			rowId = self.proxyModel.index(index.row(),0).data()

			cellValue = self.proxyModel.index(rowNum, colNum).data()

			deleteAction = QAction( QIcon('img/context_del.png'), 'Supprimer', self)
			deleteAction.triggered.connect(lambda : self.onDeleteRow(rowId, rowNum))
			self.menu.addAction(deleteAction)

			editAction = QAction( QIcon('img/context_edit.png'), 'Editer', self)
			editAction.triggered.connect(lambda : self.onEditRow(rowId))
			self.menu.addAction(editAction)

		newAction = QAction(QIcon('img/context_add.png'), 'Nouveau', self)
		newAction.triggered.connect(lambda checked : self.onNewRow())
		self.menu.addAction(newAction)			

		self.menu.addSeparator()
		
		storeStateAction = QAction(QIcon('img/context_save.png'), 'Enregistrer la disposition', self)
		storeStateAction.triggered.connect(lambda checked : self.onStoreState())
		self.menu.addAction(storeStateAction)

		self.menu.popup(QCursor.pos())

	def selectPersonAction(self) : 
		
		from libwidget.selectDialog import SelectDialog
		from libview.person import PersonDataSources	
		dataRows, dataColumns, self.dataSqlNames, dataColumsType, dataTooltips = PersonDataSources( self.db ).selectAll()			
		dialog = SelectDialog()
		dialog.loadData( dataRows, dataColumns, dataColumsType, dataTooltips, 'Sélection: '+str( 'Individu') , "person", PersonDataSources(self).getFkIndexLibBack() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.mainWindowEditMode.emit( int(rowId) , "Edit mode selection")
				
	
	def onStoreState( self ) :
		self.tableView.storeState()

	def onNewRow(self):
		
		self.mainWindowEditMode.emit( -1 , "Edit mode New")

	def onEditRow(self, rowId):

		self.mainWindowEditMode.emit( int(rowId) , "Edit mode")

	def onDeleteRow(self, rowdbId, rowIndex):

		QApplication.setOverrideCursor(Qt.WaitCursor)
		self.myPersons = Persons( self.db )
		personObj = self.myPersons.select( rowdbId )
		QApplication.restoreOverrideCursor()

		if personObj :

			dlg = PersonDeleteConfirm(self.db, self, rowdbId, personObj.getName()) 
			
			returnCode = dlg.exec()
			if returnCode == QDialog.Accepted :
				self.proxyModel.removeRow(rowIndex)				
				self.mainWindowRemoveRow.emit( rowdbId, "row removed")
				
class PersonDeleteConfirm(QDialog):

	parent = None

	def __init__(self, db, parent=None, rowid=-1, libRecord="" ):
		super().__init__(parent)

		self.db = db
		self.parent = parent
		self.rowId = rowid
		self.libRecord = libRecord

		self.setWindowTitle("Individu: suppression de #" + str(self.rowId)  )		

		QBtn = QDialogButtonBox.Ok | QDialogButtonBox.Cancel

		self.buttonBox = QDialogButtonBox(QBtn)
		self.buttonBox.accepted.connect(self.onOk)
		self.buttonBox.rejected.connect(self.onCancel)

		self.layout = QGridLayout() 
		
		# http://srinikom.github.io/pyside-docs/PySide/QtGui/QStyle.html#PySide.QtGui.PySide.QtGui.QStyle.StandardPixmap
		from PyQt5.QtWidgets import QApplication, QStyle
		from PyQt5.QtGui import QIcon

		labelIcon = QLabel("icon")
		dialogIcon = QIcon(QApplication.style().standardIcon(QStyle.SP_MessageBoxQuestion) )       
		labelIcon.setPixmap( dialogIcon.pixmap(48,48)  )
		self.layout.addWidget(labelIcon,0,0)				
		
		message = QLabel("Confirmez-vous la suppression de : \n" )
		self.layout.addWidget(message,0,1)

		data = QLabel(str(self.libRecord))
		self.layout.addWidget(data,1,1)
			
		self.layout.addWidget(self.buttonBox,2,1)
		self.setLayout(self.layout)

	def onCancel( self ) :
		self.reject()

	def onOk( self ) :
		self.myPersons = Persons( self.db )
		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.myPersons.delete( self.rowId )
		QApplication.restoreOverrideCursor()

		if result is False :
			self.parent.mainWindowUserMessage.emit( 1, "ERREUR SQL : " + str(self.myPersons.lastError) )
			self.reject()
		else :
			self.parent.mainWindowUserMessage.emit( 0 , "Suppresion effectuée. (id:"+str(self.rowId) + ")")
			self.parent.mainWindowListMode.emit()
			self.accept()


class PersonEditWidget(QWidget):

	mainWindowUserMessage = pyqtSignal( int, str )
	mainWindowListMode = pyqtSignal( )
	enterEditSignal = pyqtSignal( int )
	closeEdit = pyqtSignal( int, int )	
	validEditSignal = pyqtSignal( bool, int )
	valuesChanged = False
	
	recPos = 0
	siblingIdList = []
	siblingOrderField = "1"
	siblingOrderBy = ""

	def __init__(self, db, rootMedia, rowId, parentId, siblingIdList, siblingOrderField, siblingOrderBy, rowDataObj = None ):		
		
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia
		self.parentId = parentId
		
		self.siblingIdList = siblingIdList		
		self.siblingOrderField = siblingOrderField # Nom Sql du champ sur lequel porte le tri		
		sqlOrderBy = ['DESC','ASC'][siblingOrderBy]
		self.siblingOrderBy = sqlOrderBy		
		
		self.rowDataObj = rowDataObj

		self.myPersons = Persons( self.db )

		personObj = self.myPersons.select( rowId )
		if personObj is False :
			self.mainWindowUserMessage.emit( 0 , "Cet enregistrement a été supprimé.")
			self.rowId = -1
		else :
			self.rowId = rowId

		if self.rowId in self.siblingIdList:
			self.recPos = self.siblingIdList.index(self.rowId)
		else:
			self.recPos = 0

		self.initUI()
		
		self.setObjectName( "PersonEditWidget" )

	def showEvent(self, event):
		super(PersonEditWidget, self).showEvent(event)
		self.WidgetsForm.setFocusOnFirstWidget()	
		self.setPrevNextButtonState()		
		
	def initUI(self):

		pageLayout = QGridLayout()
		pageButtonLayout = QHBoxLayout()
		
		self.closeBtn = QPushButton("Fermer", self)
		self.closeBtn.setToolTip( "Fermer et afficher la liste Individu" )
		self.closeBtn.setIcon(QIcon('img/form_back.png'))
		self.closeBtn.clicked.connect(self.onClose)
		pageButtonLayout.addWidget(self.closeBtn)

		self.validBtn = QPushButton("Sauver", self)
		self.validBtn.setToolTip( "Enregistrer les modifications Individu" )
		self.validBtn.setIcon(QIcon('img/form_save.png'))		
		self.validBtn.clicked.connect(self.onValid)
		pageButtonLayout.addWidget(self.validBtn)

		self.updateMessage = QLineEdit( self )
		self.updateMessage.setStyleSheet("QLineEdit { background-color : rgb(255, 255, 255); color: red; border : 0px ;}")
		self.updateMessage.setToolTip( "Dernière erreur rencontrée lors de l'édition" )
		pageButtonLayout.addWidget(self.updateMessage)

		self.prevBtn = QPushButton("Précédent", self)
		self.prevBtn.setToolTip( "Individu précédent" )
		self.prevBtn.setIcon(QIcon('img/form_prev.png'))		
		self.prevBtn.clicked.connect(self.onPrev)
		pageButtonLayout.addWidget(self.prevBtn)

		self.nextBtn = QPushButton("Suivant", self)
		self.nextBtn.setToolTip( "Individu suivant" )
		self.nextBtn.setIcon(QIcon('img/form_next.png'))		
		self.nextBtn.clicked.connect(self.onNext)
		pageButtonLayout.addWidget(self.nextBtn)
		self.nextBtn.setLayoutDirection(Qt.RightToLeft)		

		self.WidgetsForm = PersonEditDisplayWidgets(self, self.db, self.rootMedia, self.rowId, self.parentId, self.rowDataObj)
		
		scroll = QScrollArea(self)
		scroll.setWidget(self.WidgetsForm)
		scroll.setWidgetResizable(True)
		pageLayout.addWidget(scroll)

		pageLayout.addLayout(pageButtonLayout, 1,0)

		self.setLayout(pageLayout)

	def enableNavButtons( self ) :
		self.nextBtn.setEnabled( True )
		self.prevBtn.setEnabled( True )

	def setPrevNextButtonState( self ) :
		
		if self.recPos <= 0 :
			self.prevBtn.setEnabled( False )
		else :
			self.prevBtn.setEnabled( True )

		if self.recPos >= len(self.siblingIdList)-1 :
			self.nextBtn.setEnabled( False )
		else :
			self.nextBtn.setEnabled( True )	
			
	def onPrev( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
					quitForm = False
					
		if quitForm is True :					
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos - 1			
			if self.recPos < 0 :
				self.recPos = 0		
			
			personObj = self.myPersons.select( self.siblingIdList[self.recPos]  )
			
			if personObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.myPersons.selectCount == 1 :
				self.rowDataObj = personObj			
				self.rowId = personObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()
		
	def onNext( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
				quitForm = False
			
		if quitForm is True :
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos + 1
			if self.recPos >= len(self.siblingIdList) :
				self.recPos = len(self.siblingIdList) - 1
			
			personObj = self.myPersons.select( self.siblingIdList[self.recPos]  )
			
			if personObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.myPersons.selectCount == 1 :
				self.rowDataObj = personObj
				self.rowId = personObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()

	def onValid( self ) :
		self.updateMessage.setText( "" )

		myPersonObj = Person()

		myPersonObj.setId( self.rowId )

			
		myPersonObj.setName( self.WidgetsForm.InputName.text() )					

			
		myPersonObj.setKey( self.WidgetsForm.InputKey.text() )					

			
			
		myPersonObj.setDescription( self.WidgetsForm.InputDescription.toPlainText() )					

			
		myPersonObj.setFirstname( self.WidgetsForm.InputFirstname.text() )					

			
		myPersonObj.setLastname( self.WidgetsForm.InputLastname.text() )					

								
			
		if self.WidgetsForm.InputPrivate.checkState() == Qt.CheckState.Unchecked :
			checkBoxState = 0
		else :
			checkBoxState = 1
		myPersonObj.setPrivate( checkBoxState ) 
							

								
			
		myPersonObj.setIcon( self.WidgetsForm.InputIcon.text() )					

			
			
		myPersonObj.setSex( self.WidgetsForm.InputSex.itemData(self.WidgetsForm.InputSex.currentIndex()) )					

			
			
		myPersonObj.setNotesearch( self.WidgetsForm.InputNotesearch.toPlainText() )					

			
		myPersonObj.setFirstnameprefix( self.WidgetsForm.InputFirstnameprefix.text() )					

			
		myPersonObj.setFirtsnameothers( self.WidgetsForm.InputFirtsnameothers.text() )					

			
		myPersonObj.setLastnamesuffix( self.WidgetsForm.InputLastnamesuffix.text() )					

			
		myPersonObj.setSurname( self.WidgetsForm.InputSurname.text() )					

			
			
		# dd/MM/yyyy hh:mm
		myPersonObj.setDatecreated( self.WidgetsForm.InputDatecreated.dateTime().toString('yyyy-MM-dd hh:mm') )					

		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.myPersons.update( myPersonObj )
		QApplication.restoreOverrideCursor()

		self.updateMessage.setText( "" )
		self.mainWindowUserMessage.emit( -1 , "" )

		if (result is False) :
			self.updateMessage.setText( "ERREUR SQL : " + str(self.myPersons.lastError) )
			self.updateMessage.setToolTip( "ERREUR SQL : " + str(self.myPersons.lastError) )
			self.validEditSignal.emit( False, self.rowId )
		else :
			self.rowId = self.myPersons.lastInsertId
			myPersonObj.setId( self.rowId )
			self.mainWindowUserMessage.emit( 0 , "Mise à jour effectuée. (id:"+str(self.rowId) + ")")
			self.WidgetsForm.displayData( self.rowId )
			self.validEditSignal.emit( True, self.rowId )

	def onClose( self ) :

		if self.valuesChanged is True :

			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.Yes:
				self.mainWindowUserMessage.emit( -1 , "" )
				self.closeEdit.emit(True, self.rowId)
				
		else :
			self.mainWindowUserMessage.emit( -1 , "" )
			self.closeEdit.emit(False, self.rowId)
	
	def onValuesChanged( self, boolState ) :
		self.valuesChanged = boolState
		self.validBtn.setEnabled( boolState )
		
		
class PersonEditDisplayWidgets(QWidget):

	def __init__(self, parent, db, rootMedia, rowId, rowParentId, rowDataObj):
		
		QWidget.__init__(self, parent)

		self.db = db
		self.rootMedia = rootMedia
		self.parent = parent
		self.rowId = rowId
		self.rowParentId = rowParentId
		self.rowDataObj = rowDataObj

		self.myPersons = Persons( self.db )

		gridLayout = QGridLayout()

		gridLayout.setSpacing(10)

		rowNumber = 1
		
		labelId = QLabel('Id')
		labelId.setToolTip( 'Identifiant interne (Id)' )
		labelId.setAlignment(Qt.AlignRight | Qt.AlignVCenter)		
		self.InputId = QLineEdit()
		self.InputId.setMaximumWidth( self.getWidgetWidth( self.InputId, 5 )  )		
		self.InputId.setEnabled( False )
		self.InputId.setStyleSheet("color: #CCCCCC;")
		gridLayout.addWidget(labelId, rowNumber, 0)
		gridLayout.addWidget(self.InputId, rowNumber, 1)

		rowNumber = rowNumber + 1
			
		labelName = QLabel('Nom')
		labelName.setToolTip( 'Prénom et nom de lʼindividu' )
		labelName.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputName = QLineEdit()
		self.InputName.setObjectName( "PersonEditName" )
		self.InputName.installEventFilter(self)
		gridLayout.addWidget(labelName, rowNumber, 0)
		gridLayout.addWidget(self.InputName, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Nom: valeur obligatoire' )
		self.InputName.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelKey = QLabel('Clé')
		labelKey.setToolTip( 'Clé identifiant unique' )
		labelKey.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputKey = QLineEdit()
		self.InputKey.setObjectName( "PersonEditKey" )
		self.InputKey.installEventFilter(self)
		gridLayout.addWidget(labelKey, rowNumber, 0)
		gridLayout.addWidget(self.InputKey, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Clé: valeur obligatoire' )
		self.InputKey.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
				
			
		labelDescription = QLabel('Description')
		labelDescription.setToolTip( 'Description de l’individu' )
		labelDescription.setAlignment(Qt.AlignRight)
		self.InputDescription = QTextEdit()
		self.InputDescription.setObjectName( "PersonEditDescription" )
		self.InputDescription.installEventFilter(self)
		gridLayout.addWidget(labelDescription, rowNumber, 0)
		gridLayout.addWidget(self.InputDescription, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Description: valeur obligatoire' )
		self.InputDescription.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
			
		labelFirstname = QLabel('Prénom')
		labelFirstname.setToolTip( 'Prénom' )
		labelFirstname.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputFirstname = QLineEdit()
		self.InputFirstname.setObjectName( "PersonEditFirstname" )
		self.InputFirstname.installEventFilter(self)
		gridLayout.addWidget(labelFirstname, rowNumber, 0)
		gridLayout.addWidget(self.InputFirstname, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Prénom: valeur obligatoire' )
		self.InputFirstname.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelLastname = QLabel('Nom de famille')
		labelLastname.setToolTip( 'Nom de famille' )
		labelLastname.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputLastname = QLineEdit()
		self.InputLastname.setObjectName( "PersonEditLastname" )
		self.InputLastname.installEventFilter(self)
		gridLayout.addWidget(labelLastname, rowNumber, 0)
		gridLayout.addWidget(self.InputLastname, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Nom de famille: valeur obligatoire' )
		self.InputLastname.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
				

			
		labelPrivate = QLabel('Privé')
		labelPrivate.setToolTip( 'Donnée de statut privé' )
		labelPrivate.setAlignment(Qt.AlignRight)
		self.InputPrivate = QCheckBox()
		self.InputPrivate.setObjectName( "PersonEditPrivate" )
		self.InputPrivate.installEventFilter(self)
		gridLayout.addWidget(labelPrivate, rowNumber, 0)
		gridLayout.addWidget(self.InputPrivate, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Privé: valeur obligatoire' )
		self.InputPrivate.stateChanged.connect(self.onCheckBoxValuesChanged) 
		rowNumber = rowNumber + 1
					

			
		labelIcon = QLabel('Icône')
		labelIcon.setToolTip( 'Icône, image d’illustration' )
		labelIcon.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputIcon = QLineEdit()
		self.InputIcon.setObjectName( "PersonEditIcon" )
		self.InputIcon.installEventFilter(self)
		gridLayout.addWidget(labelIcon, rowNumber, 0)
		gridLayout.addWidget(self.InputIcon, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Icône: valeur obligatoire' )
		self.InputIcon.textChanged.connect(self.onLineEditValuesChanged)	
		self.fileSelectIcon = QPushButton("...")
		self.fileSelectIcon.clicked.connect(self.getfilesMediaIcon)
		gridLayout.addWidget(self.fileSelectIcon, rowNumber, 3)

		rowNumber = rowNumber + 1
			
			
		labelSex = QLabel('Sexe')
		labelSex.setToolTip( 'Sexe' )
		labelSex.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputSex = QComboBox()
		self.InputSex.setObjectName( "PersonEditSex" )
		self.InputSex.installEventFilter(self)
		gridLayout.addWidget(labelSex, rowNumber, 0)
		gridLayout.addWidget(self.InputSex, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Sexe: valeur obligatoire' )
		comboData =  { "" : "[sélectionner une valeur]", "F" : "Féminin", "M" : "Masculin", "I" : "Indéterminé",  } 
		self.InputSex.clear()
		for key, value in comboData.items():
			self.InputSex.addItem( str(value) , userData=key ) 
		self.InputSex.currentIndexChanged.connect(self.onComboValuesChanged)
		rowNumber = rowNumber + 1
				
			
		labelNotesearch = QLabel('Notes de recherche')
		labelNotesearch.setToolTip( 'Notes de recherche' )
		labelNotesearch.setAlignment(Qt.AlignRight)
		self.InputNotesearch = QTextEdit()
		self.InputNotesearch.setObjectName( "PersonEditNotesearch" )
		self.InputNotesearch.installEventFilter(self)
		gridLayout.addWidget(labelNotesearch, rowNumber, 0)
		gridLayout.addWidget(self.InputNotesearch, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Notes de recherche: valeur obligatoire' )
		self.InputNotesearch.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
			
		labelFirstnameprefix = QLabel('Préfixe prénom')
		labelFirstnameprefix.setToolTip( 'Préfixe du prénom (Me, Mlle, M, Dr ..)' )
		labelFirstnameprefix.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputFirstnameprefix = QLineEdit()
		self.InputFirstnameprefix.setObjectName( "PersonEditFirstnameprefix" )
		self.InputFirstnameprefix.installEventFilter(self)
		gridLayout.addWidget(labelFirstnameprefix, rowNumber, 0)
		gridLayout.addWidget(self.InputFirstnameprefix, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Préfixe prénom: valeur obligatoire' )
		self.InputFirstnameprefix.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelFirtsnameothers = QLabel('Autres prénoms')
		labelFirtsnameothers.setToolTip( 'Prénoms secondaires' )
		labelFirtsnameothers.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputFirtsnameothers = QLineEdit()
		self.InputFirtsnameothers.setObjectName( "PersonEditFirtsnameothers" )
		self.InputFirtsnameothers.installEventFilter(self)
		gridLayout.addWidget(labelFirtsnameothers, rowNumber, 0)
		gridLayout.addWidget(self.InputFirtsnameothers, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Autres prénoms: valeur obligatoire' )
		self.InputFirtsnameothers.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelLastnamesuffix = QLabel('Suffixe nom de famille')
		labelLastnamesuffix.setToolTip( 'Suffixe du nom de famille (Junior, 1er ...)' )
		labelLastnamesuffix.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputLastnamesuffix = QLineEdit()
		self.InputLastnamesuffix.setObjectName( "PersonEditLastnamesuffix" )
		self.InputLastnamesuffix.installEventFilter(self)
		gridLayout.addWidget(labelLastnamesuffix, rowNumber, 0)
		gridLayout.addWidget(self.InputLastnamesuffix, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Suffixe nom de famille: valeur obligatoire' )
		self.InputLastnamesuffix.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelSurname = QLabel('Surnom')
		labelSurname.setToolTip( 'Surnom' )
		labelSurname.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputSurname = QLineEdit()
		self.InputSurname.setObjectName( "PersonEditSurname" )
		self.InputSurname.installEventFilter(self)
		gridLayout.addWidget(labelSurname, rowNumber, 0)
		gridLayout.addWidget(self.InputSurname, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Surnom: valeur obligatoire' )
		self.InputSurname.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
							
			
		labelDatecreated = QLabel('Modifié')
		labelDatecreated.setToolTip( 'Date de création ou de dernière modification' )
		labelDatecreated.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDatecreated = QDateTimeEdit()
		self.InputDatecreated.setObjectName( "PersonEditDatecreated" )
		self.InputDatecreated.installEventFilter(self)
		self.InputDatecreated.setMaximumWidth( self.getWidgetWidth( self.InputDatecreated, 16 )  )		
		gridLayout.addWidget(labelDatecreated, rowNumber, 0)
		gridLayout.addWidget(self.InputDatecreated, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Modifié: valeur obligatoire' )
		self.InputDatecreated.dateTimeChanged.connect(self.onDateEditValuesChanged)	
		self.InputDatecreated.setDisplayFormat("dd/MM/yyyy hh:mm")
		self.InputDatecreated.setDateTimeRange(QDateTime(1, 1, 1, 0, 0), QDateTime(2999, 12, 31, 23, 59))
		self.InputDatecreated.setSpecialValueText(' ')
		self.InputDatecreated.setReadOnly(True)
		if True is False :
			self.InputDatecreated.setCalendarPopup(True)	


		self.setLayout(gridLayout)
		self.displayData( self.rowId, self.rowDataObj )
		

			

			

			

			

			

			

			

			

			

			

			

			

			

			


	

	

	

	

	

	

	

	

	

	

	

	

	

	


	

	

	

	

	

	

	

	def getfilesMediaIcon( self ) :      

		from libwidget.mediamanager import MediaManager
		from lib.project import Project  
		currentProject = Project()			
		currentProject.load( AppSetting.get('app/lastProject', '') )
		myMediaSelector = MediaManager( rootMedia = currentProject.rootMedia )
		myMediaSelector.loadDataList( self.InputIcon.text().split(',') )		
		result = myMediaSelector.exec_()
		filenamelist = myMediaSelector.getValue()
		if filenamelist is not None :
			self.printDebug("filenamelist = " , filenamelist)
			self.InputIcon.setText( ",".join(filenamelist) )

	

	

	

	

	

	

	



	

	

	

	

	

	

	

	

	

	

	

	

	

	



	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
	
	# Retourne la largeur necessaire pour un widget (QlineEdit ...) 
	# en fonctiion du nombre de caractères souhaité.
	def getWidgetWidth( self, widget, charmax ) :
		
		fm = widget.fontMetrics()
		try :
			m = widget.textMargins()
		except Exception as e:
			m = QMargins(8,8,8,8)
		c = widget.contentsMargins()
		w = charmax*fm.width('x')+m.left()+m.right()+c.left()+c.right()
		w = w + 8 # margins
		return w
			
	def onLineEditValuesChanged(self , text) :
		
		self.parent.onValuesChanged( True )

	def onTextEditValuesChanged(self):
		
		self.parent.onValuesChanged( True )
		
	def onComboValuesChanged(self, newIndex):
		
		self.parent.onValuesChanged( True )
				
	def onCheckBoxValuesChanged( self ,state) :
		self.parent.onValuesChanged( True )
			
	def onDateEditValuesChanged( self, newDate ) :
		
		self.parent.onValuesChanged( True )
				
	def displayData( self, rowId, dataObj = None) :

		self.rowId = rowId		

		self.parent.enterEditSignal.emit( self.rowId )
		
		if self.rowId == -1 :

			self.parent.mainWindowUserMessage.emit( 1 , "Formulaire vide (creation ou recréation)" )
		
		if dataObj is None :			
			personObj = self.myPersons.select( self.rowId )
			if personObj is False :
				personObj = Person()			
		else :
			personObj = dataObj
						
		self.InputId.setText( str(personObj.getId()) )		

			
		defaultValue = ''
		self.InputName.setText( str( personObj.getName() if personObj.getName() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputKey.setText( str( personObj.getKey() if personObj.getKey() is not None else defaultValue ) ) 

			
			
		self.InputDescription.setText( str( personObj.getDescription() if personObj.getDescription() is not None else '' ) ) 

			
		defaultValue = ''
		self.InputFirstname.setText( str( personObj.getFirstname() if personObj.getFirstname() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputLastname.setText( str( personObj.getLastname() if personObj.getLastname() is not None else defaultValue ) ) 

			
			
		if personObj.getPrivate() is None or personObj.getPrivate() == 0 :
			self.InputPrivate.setCheckState( Qt.CheckState.Unchecked ) 			
		if personObj.getPrivate() == 1 :
			self.InputPrivate.setCheckState( Qt.CheckState.Checked ) 


			
			
		self.InputIcon.setText( str( personObj.getIcon() if personObj.getIcon() is not None else '' ) ) 

			
			
		currentKey = personObj.getSex() if personObj.getSex() is not None else '' 
		self.InputSex.setCurrentIndex( self.InputSex.findData(currentKey) ) 

			
			
		self.InputNotesearch.setText( str( personObj.getNotesearch() if personObj.getNotesearch() is not None else '' ) ) 

			
		defaultValue = ''
		self.InputFirstnameprefix.setText( str( personObj.getFirstnameprefix() if personObj.getFirstnameprefix() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputFirtsnameothers.setText( str( personObj.getFirtsnameothers() if personObj.getFirtsnameothers() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputLastnamesuffix.setText( str( personObj.getLastnamesuffix() if personObj.getLastnamesuffix() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputSurname.setText( str( personObj.getSurname() if personObj.getSurname() is not None else defaultValue ) ) 

									
			
		#myQdate = Util.getQDatetimeFromString( personObj.getDatecreated(), defaultValue = QDateTime.currentDateTime().toString('dd/MM/yyyy') )
		myQdate = Util.getQDatetimeFromString( personObj.getDatecreated() )
		self.InputDatecreated.setDateTime( myQdate ) 

		self.parent.onValuesChanged( False )


	def setFocusOnFirstWidget( self ) :

		from PyQt5.QtWidgets import  QLineEdit, QTextEdit, QDateEdit, QDateTimeEdit, QDateTimeEdit, QCheckBox, QComboBox,QDial, QDoubleSpinBox, QRadioButton, QSlider, QSpinBox, QTimeEdit, QFontComboBox
		for widget in self.children():
			if widget.isEnabled() :
				if 	isinstance(widget, QLineEdit) or \
					isinstance(widget, QTextEdit) or \
					isinstance(widget, QDateEdit) or \
					isinstance(widget, QDateTimeEdit) or \
					isinstance(widget, QCheckBox) or \
					isinstance(widget, QComboBox) or \
					isinstance(widget, QDial) or \
					isinstance(widget, QDoubleSpinBox) or \
					isinstance(widget, QRadioButton) or \
					isinstance(widget, QSlider) or \
					isinstance(widget, QSpinBox) or \
					isinstance(widget, QTimeEdit) or \
					isinstance(widget, QFontComboBox) \
					:
					#AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					widget.setFocus()					
					return

	def eventFilter(self, widgetObj, event):
		
		if event.type() == QtCore.QEvent.FocusIn:			
			self.focusWidget = widgetObj
	
		return super(PersonEditDisplayWidgets, self).eventFilter(widgetObj, event)
    
	def restoreFocusWidget( self ) :		
		#AppLog.log( LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		AppLog.log( LOG_DEBUG , LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		self.focusWidget.setFocus()	
            

class PersonDataSources() :
	
	# Jointure sur table(s): 	
	dataColumns = [ 'Id', 'Nom', 'Clé', 'Description', 'Prénom', 'Nom de famille', 'Privé', 'Icône', 'Sexe', 'Notes de recherche', 'Préfixe prénom', 'Autres prénoms', 'Suffixe nom de famille', 'Surnom', 'Modifié'  ]
	dataSqlFields = [ 'id', 'name', 'key', 'description', 'firstname', 'lastname', 'private', 'icon', 'sex', 'notesearch', 'firstnameprefix', 'firtsnameothers', 'lastnamesuffix', 'surname', 'datecreated'  ]	
	dataColumsType = ['INTEGER', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'BOOLEAN', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'DATETIME'  ]
	dataTooltips = [ 'Identifiant interne', 'Prénom et nom de lʼindividu', 'Clé identifiant unique', 'Description de l’individu', 'Prénom', 'Nom de famille', 'Donnée de statut privé', 'Icône, image d’illustration', 'Sexe', 'Notes de recherche', 'Préfixe du prénom (Me, Mlle, M, Dr ..)', 'Prénoms secondaires', 'Suffixe du nom de famille (Junior, 1er ...)', 'Surnom', 'Date de création ou de dernière modification'  ]
			
	def __init__(self, db ):
		self.db = db
		pass
		
	def selectAll( self , field_name = None, field_value = None ) :
		
		if field_name is None and field_value is None :
			return self.select()
		else :
			return self.selectById( field_name, field_value )
			
					
	def select( self ) :
		
		dataRows = []
		
		self.myPersons = Persons( self.db )
		personListAll = self.myPersons.selectAll()
		if personListAll :
			i = 0
			for person in personListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(person.getId()))
				dataRows.append( [ person.getId(),  person.getName(),  person.getKey(),  person.getDescription(),  person.getFirstname(),  person.getLastname(),  person.getPrivate(),  person.getIcon(),  person.getSex(),  person.getNotesearch(),  person.getFirstnameprefix(),  person.getFirtsnameothers(),  person.getLastnamesuffix(),  person.getSurname(),  person.getDatecreated()  ] )
				i = i + 1
			
		return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )


	def selectById( self, where_clause, fields_values ) :
		
		dataRows = []
				
		self.myPersons = Persons( self.db )
		personListAll = self.myPersons.selectById( where_clause, fields_values )
		if personListAll : 
			i = 0
			for person in personListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(person.getId()))
				dataRows.append( [ person.getId(),  person.getName(),  person.getKey(),  person.getDescription(),  person.getFirstname(),  person.getLastname(),  person.getPrivate(),  person.getIcon(),  person.getSex(),  person.getNotesearch(),  person.getFirstnameprefix(),  person.getFirtsnameothers(),  person.getLastnamesuffix(),  person.getSurname(),  person.getDatecreated()  ] )
				i = i + 1
				
			return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
		return ( None, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
	# Retourner le champ #x lors de la selection par pop-up (Fk)
	def getFkIndexLibBack( self ) :
		return 1
