#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

		*** Source autogénéré *** 
		
		Pour la table : place.

		----------------------------
		Lieu.
		
		Liste des adresses des événements.
		----------------------------		

		05/24/2023 14:31

		lvardon@laposte.net - 2021

		Licence libre

'''
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QAction, QMenu, QApplication, QWidget, QToolTip, QPushButton, QMessageBox, QFileDialog
from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, QHBoxLayout, QScrollArea
from PyQt5.QtWidgets import QLabel, QLineEdit, QTextEdit, QComboBox, QCheckBox, QDateTimeEdit
from PyQt5.QtGui import QCursor, QFont, QIcon
from PyQt5.QtCore import Qt, QEvent, pyqtSignal, pyqtSlot, QModelIndex, QDateTime, QMargins, QSettings
from PyQt5 import QtCore
		
from lib.sqlitedb import Sqlitedb
from libdbcustom.place import Place, Places

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
from lib.util import Util
from libwidget.tableModel import TableModel, TableProxyModel
from libwidget.tableView import TableView,TableCellImageDelegate

import lib.setting as AppSetting
import lib.log as AppLog

class PlaceListWidget(QWidget):

	mainWindowEditMode =  pyqtSignal( int, str )
	mainWindowRemoveRow =  pyqtSignal( int, str )	
	mainWindowListMode = pyqtSignal()
	mainWindowUserMessage = pyqtSignal( int, str )
	cellClickSignal =  pyqtSignal( int )
	cellDoubleClickSignal =  pyqtSignal( int )
	selectionSignal = pyqtSignal( int )
	
	def __init__(self , db, rootMedia, sourceOrigin = "", where_clause = None, fields_values = None):
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia 
		self.sourceOrigin = sourceOrigin
		
		self.where_clause = where_clause
		self.fields_values = fields_values
		
		self.initUI()

		self.initDelegates(  self.tableView )
		
		iconW, iconH, rowHeight = self.getTableViewIniParams( "tablelistviewPlace" )
		self.tableView.verticalHeader().setDefaultSectionSize(int(rowHeight))

		try :
			self.iconImageDelegate.icoW = int(iconW) 
			self.iconImageDelegate.icoH = int(iconH)	
		except Exception as e:
			pass
		try :
			self.urlImageDelegate.icoW = int(iconW) 
			self.urlImageDelegate.icoH = int(iconH)					
		except Exception as e:
			pass
		
		self.setObjectName( "PlaceListWidget" )

	def initDelegates( self, thistableView ) :
				
		self.privateImageDelegate = TableCellImageDelegate(self, "place", {'0' : 'img/dataicons/empty.png', '1' : 'img/dataicons/private-yes.png', '2' : 'img/dataicons/private-yes.png'} , self.rootMedia, 26, 26)
		thistableView.setItemDelegateForColumn(3, self.privateImageDelegate)

		self.iconImageDelegate = TableCellImageDelegate(self, "place", None , self.rootMedia, 40, 40)
		thistableView.setItemDelegateForColumn(4, self.iconImageDelegate)

		
		thistableView.verticalHeader().setDefaultSectionSize(40)

	def initUI(self):
		
		self.layout = QVBoxLayout()
		self.setLayout(self.layout)
				
		self.proxyModel = TableProxyModel(self)
		
		self.proxyModel.setSortRole(Qt.UserRole + 2)
		
		self.tableView = TableView( self, "place", self.sourceOrigin )	
				
		self.loadData( self.where_clause , self.fields_values ) 
	
		self.tableView.setModel(self.proxyModel)
		
		self.tableView.resizeColumnsToContents()

		self.tableView.cellClickSignal.connect(self.onCellClic)
		self.tableView.cellDoubleClickSignal.connect(self.onCellDoubleClic)
	
		selection_model = self.tableView.selectionModel()
		selection_model.selectionChanged.connect(self.onSelectionChanged)      
		
		self.filterValue = QLineEdit()
		self.layout.addWidget(self.filterValue)
		self.filterValue.textChanged.connect(self.onfilterValueChanged)
		
		self.layout.addWidget(self.tableView)

	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
			

	def getTableViewIniParams ( self, tableViewName) :
		
		iniConfigFile = "config/app.ini"
		settings=QSettings(iniConfigFile, QSettings.IniFormat)

		rowHeight = 0
		if settings.value(tableViewName + '/rowHeight') is not None:			
			rowHeight = settings.value( tableViewName +'/rowHeight')						
			
		iconW = 0
		if settings.value( tableViewName + '/rowIconW') is not None:						
			iconW = settings.value( tableViewName +'/rowIconW')				
		
		iconH = 0			
		if settings.value( tableViewName +'/rowIconH') is not None:			
			iconH = settings.value( tableViewName +'/rowIconH')	

		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" rowHeight="+ str(rowHeight))
		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" iconW x IconH="+ str(iconW) + " x " + str(iconH))
		
		return iconW, iconH, rowHeight
		
	@pyqtSlot('QItemSelection', 'QItemSelection')
	def onSelectionChanged(self, selected, deselected):
		try :
			rowId =  selected.indexes()[0].data()
			self.selectionSignal.emit( rowId )
		except Exception as e:
			pass		
		
	def onfilterValueChanged(self , text) :
		self.proxyModel.setFilterValue( str(text) )		
		self.proxyModel.applyFilters()

	def onCellClic(self, rowId ):
		self.cellClickSignal.emit(rowId)

	def onCellDoubleClic(self, rowId ):
		self.cellDoubleClickSignal.emit(rowId)
		self.onEditRow(rowId)
		
	def reloadData(self) :
		self.loadData(self.where_clause, self.fields_values)		
		
	def loadData( self, where_clause , fields_values ) :
		
		self.where_clause = where_clause
		self.fields_values = fields_values
				
		dataSource = PlaceDataSources( self.db )
		dataRows, dataColumns, self.dataSqlNames, dataColumnsType, dataToolTips = dataSource.selectAll( self.where_clause, self.fields_values )
		
		self.model = TableModel("place", dataRows, dataColumns, dataColumnsType, dataToolTips)

		self.proxyModel.setSourceModel(self.model)
		
		self.tableView.resizeColumnsToContents()				
				
	def dumpDatas( self) :
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		for i in range(0, row) :
			self.printDebug("--- Row : ", i)
			for j in range(0, col) :
				self.printDebug( 
						self.model.data( self.model.index(i, j), Qt.UserRole + 1 ),
						" - ",
						self.model.data( self.model.index(i, j), Qt.UserRole ) 
					)

	# Nom de la colonne sur laquelle porte le tri 
	def getSqlOrderByField( self) :
		orderByFieldPos = self.tableView.hView.sortIndicatorSection()
		orderByField = self.dataSqlNames[orderByFieldPos]
		
		return orderByField
		
	# Index du sens de tri 'DESC' = 0 'ASC' = 1
	def getSqlOrderByOrder( self) :
		orderBy = self.tableView.hView.sortIndicatorOrder()
		return orderBy
		
		
	# Liste des id de la liste affichée
	def getIdList( self ) :
		
		idList = []
		
		indexId = 0 # id column is at pos 0
		
		row = self.proxyModel.rowCount()
		model = self.proxyModel
		
		for i in range(0, row) :			
			idList.append( model.data( model.index(i, indexId ), Qt.UserRole ) )
		
		return idList
			
	
	def selectRowById( self, rowId ) :

		if rowId is None :
			return
			
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel
		
		for i in range(0, row) :
			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :				
				self.tableView.selectRow( i )
				return

	def getRowData( self, rowId ) :

		myPlaceObj = Place( )

		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel

		for i in range(0, row) :
			#print('row:', i, " model rowid=", self.model.data( self.model.index(i, 0), Qt.UserRole ), ' == ',  rowId, ' **', self.model.data( self.model.index(i, 1), Qt.UserRole ) )

			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :

				myPlaceObj.setId( model.data( model.index(i, 0 ), Qt.UserRole  )  )
				myPlaceObj.setName( model.data( model.index(i, 1 ), Qt.UserRole  )  )
				myPlaceObj.setDescription( model.data( model.index(i, 2 ), Qt.UserRole  )  )
				myPlaceObj.setPrivate( model.data( model.index(i, 3 ), Qt.UserRole  )  )
				myPlaceObj.setIcon( model.data( model.index(i, 4 ), Qt.UserRole  )  )
				myPlaceObj.setAdress1( model.data( model.index(i, 5 ), Qt.UserRole  )  )
				myPlaceObj.setAdress2( model.data( model.index(i, 6 ), Qt.UserRole  )  )
				myPlaceObj.setZipcode( model.data( model.index(i, 7 ), Qt.UserRole  )  )
				myPlaceObj.setCity( model.data( model.index(i, 8 ), Qt.UserRole  )  )
				myPlaceObj.setCityinsee( model.data( model.index(i, 9 ), Qt.UserRole  )  )
				myPlaceObj.setCountry( model.data( model.index(i, 10 ), Qt.UserRole  )  )
				myPlaceObj.setUrl( model.data( model.index(i, 11 ), Qt.UserRole  )  )
				myPlaceObj.setAccurate( model.data( model.index(i, 12 ), Qt.UserRole  )  )
				myPlaceObj.setGpslat( model.data( model.index(i, 13 ), Qt.UserRole  )  )
				myPlaceObj.setGpslon( model.data( model.index(i, 14 ), Qt.UserRole  )  )
				myPlaceObj.setZoom( model.data( model.index(i, 15 ), Qt.UserRole  )  )
				myPlaceObj.setDatecreated( model.data( model.index(i, 16 ), Qt.UserRole  )  )

				return myPlaceObj

		return myPlaceObj

	def contextMenuEvent(self, event):

		gp = event.globalPos()
		vp_pos = self.tableView.viewport().mapFromGlobal(gp)
		rowNum = self.tableView.rowAt(vp_pos.y())
		colNum = self.tableView.columnAt(event.x())

		self.menu = QMenu(self)
			
		if rowNum >= 0 :

			index = self.tableView.indexAt(vp_pos)
			rowId = self.proxyModel.index(index.row(),0).data()

			cellValue = self.proxyModel.index(rowNum, colNum).data()

			deleteAction = QAction( QIcon('img/context_del.png'), 'Supprimer', self)
			deleteAction.triggered.connect(lambda : self.onDeleteRow(rowId, rowNum))
			self.menu.addAction(deleteAction)

			editAction = QAction( QIcon('img/context_edit.png'), 'Editer', self)
			editAction.triggered.connect(lambda : self.onEditRow(rowId))
			self.menu.addAction(editAction)

		newAction = QAction(QIcon('img/context_add.png'), 'Nouveau', self)
		newAction.triggered.connect(lambda checked : self.onNewRow())
		self.menu.addAction(newAction)			

		self.menu.addSeparator()
		
		storeStateAction = QAction(QIcon('img/context_save.png'), 'Enregistrer la disposition', self)
		storeStateAction.triggered.connect(lambda checked : self.onStoreState())
		self.menu.addAction(storeStateAction)

		self.menu.popup(QCursor.pos())

	def selectPlaceAction(self) : 
		
		from libwidget.selectDialog import SelectDialog
		from libview.place import PlaceDataSources	
		dataRows, dataColumns, self.dataSqlNames, dataColumsType, dataTooltips = PlaceDataSources( self.db ).selectAll()			
		dialog = SelectDialog()
		dialog.loadData( dataRows, dataColumns, dataColumsType, dataTooltips, 'Sélection: '+str( 'Lieu') , "place", PlaceDataSources(self).getFkIndexLibBack() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.mainWindowEditMode.emit( int(rowId) , "Edit mode selection")
				
	
	def onStoreState( self ) :
		self.tableView.storeState()

	def onNewRow(self):
		
		self.mainWindowEditMode.emit( -1 , "Edit mode New")

	def onEditRow(self, rowId):

		self.mainWindowEditMode.emit( int(rowId) , "Edit mode")

	def onDeleteRow(self, rowdbId, rowIndex):

		QApplication.setOverrideCursor(Qt.WaitCursor)
		self.myPlaces = Places( self.db )
		placeObj = self.myPlaces.select( rowdbId )
		QApplication.restoreOverrideCursor()

		if placeObj :

			dlg = PlaceDeleteConfirm(self.db, self, rowdbId, placeObj.getName()) 
			
			returnCode = dlg.exec()
			if returnCode == QDialog.Accepted :
				self.proxyModel.removeRow(rowIndex)				
				self.mainWindowRemoveRow.emit( rowdbId, "row removed")
				
class PlaceDeleteConfirm(QDialog):

	parent = None

	def __init__(self, db, parent=None, rowid=-1, libRecord="" ):
		super().__init__(parent)

		self.db = db
		self.parent = parent
		self.rowId = rowid
		self.libRecord = libRecord

		self.setWindowTitle("Lieu: suppression de #" + str(self.rowId)  )		

		QBtn = QDialogButtonBox.Ok | QDialogButtonBox.Cancel

		self.buttonBox = QDialogButtonBox(QBtn)
		self.buttonBox.accepted.connect(self.onOk)
		self.buttonBox.rejected.connect(self.onCancel)

		self.layout = QGridLayout() 
		
		# http://srinikom.github.io/pyside-docs/PySide/QtGui/QStyle.html#PySide.QtGui.PySide.QtGui.QStyle.StandardPixmap
		from PyQt5.QtWidgets import QApplication, QStyle
		from PyQt5.QtGui import QIcon

		labelIcon = QLabel("icon")
		dialogIcon = QIcon(QApplication.style().standardIcon(QStyle.SP_MessageBoxQuestion) )       
		labelIcon.setPixmap( dialogIcon.pixmap(48,48)  )
		self.layout.addWidget(labelIcon,0,0)				
		
		message = QLabel("Confirmez-vous la suppression de : \n" )
		self.layout.addWidget(message,0,1)

		data = QLabel(str(self.libRecord))
		self.layout.addWidget(data,1,1)
			
		self.layout.addWidget(self.buttonBox,2,1)
		self.setLayout(self.layout)

	def onCancel( self ) :
		self.reject()

	def onOk( self ) :
		self.myPlaces = Places( self.db )
		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.myPlaces.delete( self.rowId )
		QApplication.restoreOverrideCursor()

		if result is False :
			self.parent.mainWindowUserMessage.emit( 1, "ERREUR SQL : " + str(self.myPlaces.lastError) )
			self.reject()
		else :
			self.parent.mainWindowUserMessage.emit( 0 , "Suppresion effectuée. (id:"+str(self.rowId) + ")")
			self.parent.mainWindowListMode.emit()
			self.accept()


class PlaceEditWidget(QWidget):

	mainWindowUserMessage = pyqtSignal( int, str )
	mainWindowListMode = pyqtSignal( )
	enterEditSignal = pyqtSignal( int )
	closeEdit = pyqtSignal( int, int )	
	validEditSignal = pyqtSignal( bool, int )
	valuesChanged = False
	
	recPos = 0
	siblingIdList = []
	siblingOrderField = "1"
	siblingOrderBy = ""

	def __init__(self, db, rootMedia, rowId, parentId, siblingIdList, siblingOrderField, siblingOrderBy, rowDataObj = None ):		
		
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia
		self.parentId = parentId
		
		self.siblingIdList = siblingIdList		
		self.siblingOrderField = siblingOrderField # Nom Sql du champ sur lequel porte le tri		
		sqlOrderBy = ['DESC','ASC'][siblingOrderBy]
		self.siblingOrderBy = sqlOrderBy		
		
		self.rowDataObj = rowDataObj

		self.myPlaces = Places( self.db )

		placeObj = self.myPlaces.select( rowId )
		if placeObj is False :
			self.mainWindowUserMessage.emit( 0 , "Cet enregistrement a été supprimé.")
			self.rowId = -1
		else :
			self.rowId = rowId

		if self.rowId in self.siblingIdList:
			self.recPos = self.siblingIdList.index(self.rowId)
		else:
			self.recPos = 0

		self.initUI()
		
		self.setObjectName( "PlaceEditWidget" )

	def showEvent(self, event):
		super(PlaceEditWidget, self).showEvent(event)
		self.WidgetsForm.setFocusOnFirstWidget()	
		self.setPrevNextButtonState()		
		
	def initUI(self):

		pageLayout = QGridLayout()
		pageButtonLayout = QHBoxLayout()
		
		self.closeBtn = QPushButton("Fermer", self)
		self.closeBtn.setToolTip( "Fermer et afficher la liste Lieu" )
		self.closeBtn.setIcon(QIcon('img/form_back.png'))
		self.closeBtn.clicked.connect(self.onClose)
		pageButtonLayout.addWidget(self.closeBtn)

		self.validBtn = QPushButton("Sauver", self)
		self.validBtn.setToolTip( "Enregistrer les modifications Lieu" )
		self.validBtn.setIcon(QIcon('img/form_save.png'))		
		self.validBtn.clicked.connect(self.onValid)
		pageButtonLayout.addWidget(self.validBtn)

		self.updateMessage = QLineEdit( self )
		self.updateMessage.setStyleSheet("QLineEdit { background-color : rgb(255, 255, 255); color: red; border : 0px ;}")
		self.updateMessage.setToolTip( "Dernière erreur rencontrée lors de l'édition" )
		pageButtonLayout.addWidget(self.updateMessage)

		self.prevBtn = QPushButton("Précédent", self)
		self.prevBtn.setToolTip( "Lieu précédent" )
		self.prevBtn.setIcon(QIcon('img/form_prev.png'))		
		self.prevBtn.clicked.connect(self.onPrev)
		pageButtonLayout.addWidget(self.prevBtn)

		self.nextBtn = QPushButton("Suivant", self)
		self.nextBtn.setToolTip( "Lieu suivant" )
		self.nextBtn.setIcon(QIcon('img/form_next.png'))		
		self.nextBtn.clicked.connect(self.onNext)
		pageButtonLayout.addWidget(self.nextBtn)
		self.nextBtn.setLayoutDirection(Qt.RightToLeft)		

		self.WidgetsForm = PlaceEditDisplayWidgets(self, self.db, self.rootMedia, self.rowId, self.parentId, self.rowDataObj)
		
		scroll = QScrollArea(self)
		scroll.setWidget(self.WidgetsForm)
		scroll.setWidgetResizable(True)
		pageLayout.addWidget(scroll)

		pageLayout.addLayout(pageButtonLayout, 1,0)

		self.setLayout(pageLayout)

	def enableNavButtons( self ) :
		self.nextBtn.setEnabled( True )
		self.prevBtn.setEnabled( True )

	def setPrevNextButtonState( self ) :
		
		if self.recPos <= 0 :
			self.prevBtn.setEnabled( False )
		else :
			self.prevBtn.setEnabled( True )

		if self.recPos >= len(self.siblingIdList)-1 :
			self.nextBtn.setEnabled( False )
		else :
			self.nextBtn.setEnabled( True )	
			
	def onPrev( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
					quitForm = False
					
		if quitForm is True :					
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos - 1			
			if self.recPos < 0 :
				self.recPos = 0		
			
			placeObj = self.myPlaces.select( self.siblingIdList[self.recPos]  )
			
			if placeObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.myPlaces.selectCount == 1 :
				self.rowDataObj = placeObj			
				self.rowId = placeObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()
		
	def onNext( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
				quitForm = False
			
		if quitForm is True :
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos + 1
			if self.recPos >= len(self.siblingIdList) :
				self.recPos = len(self.siblingIdList) - 1
			
			placeObj = self.myPlaces.select( self.siblingIdList[self.recPos]  )
			
			if placeObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.myPlaces.selectCount == 1 :
				self.rowDataObj = placeObj
				self.rowId = placeObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()

	def onValid( self ) :
		self.updateMessage.setText( "" )

		myPlaceObj = Place()

		myPlaceObj.setId( self.rowId )

			
		myPlaceObj.setName( self.WidgetsForm.InputName.text() )					

			
			
		myPlaceObj.setDescription( self.WidgetsForm.InputDescription.toPlainText() )					

								
			
		if self.WidgetsForm.InputPrivate.checkState() == Qt.CheckState.Unchecked :
			checkBoxState = 0
		else :
			checkBoxState = 1
		myPlaceObj.setPrivate( checkBoxState ) 
							

								
			
		myPlaceObj.setIcon( self.WidgetsForm.InputIcon.text() )					

			
		myPlaceObj.setAdress1( self.WidgetsForm.InputAdress1.text() )					

			
		myPlaceObj.setAdress2( self.WidgetsForm.InputAdress2.text() )					

			
		myPlaceObj.setZipcode( self.WidgetsForm.InputZipcode.text() )					

			
		myPlaceObj.setCity( self.WidgetsForm.InputCity.text() )					

			
		myPlaceObj.setCityinsee( self.WidgetsForm.InputCityinsee.text() )					

			
		myPlaceObj.setCountry( self.WidgetsForm.InputCountry.text() )					

			
		myPlaceObj.setUrl( self.WidgetsForm.InputUrl.text() )					

			
			
		myPlaceObj.setAccurate( self.WidgetsForm.InputAccurate.itemData(self.WidgetsForm.InputAccurate.currentIndex()) )					

			
		myPlaceObj.setGpslat( self.WidgetsForm.InputGpslat.text() )					

			
		myPlaceObj.setGpslon( self.WidgetsForm.InputGpslon.text() )					

			
		myPlaceObj.setZoom( self.WidgetsForm.InputZoom.text() )					

			
			
		# dd/MM/yyyy hh:mm
		myPlaceObj.setDatecreated( self.WidgetsForm.InputDatecreated.dateTime().toString('yyyy-MM-dd hh:mm') )					

		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.myPlaces.update( myPlaceObj )
		QApplication.restoreOverrideCursor()

		self.updateMessage.setText( "" )
		self.mainWindowUserMessage.emit( -1 , "" )

		if (result is False) :
			self.updateMessage.setText( "ERREUR SQL : " + str(self.myPlaces.lastError) )
			self.updateMessage.setToolTip( "ERREUR SQL : " + str(self.myPlaces.lastError) )
			self.validEditSignal.emit( False, self.rowId )
		else :
			self.rowId = self.myPlaces.lastInsertId
			myPlaceObj.setId( self.rowId )
			self.mainWindowUserMessage.emit( 0 , "Mise à jour effectuée. (id:"+str(self.rowId) + ")")
			self.WidgetsForm.displayData( self.rowId )
			self.validEditSignal.emit( True, self.rowId )

	def onClose( self ) :

		if self.valuesChanged is True :

			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.Yes:
				self.mainWindowUserMessage.emit( -1 , "" )
				self.closeEdit.emit(True, self.rowId)
				
		else :
			self.mainWindowUserMessage.emit( -1 , "" )
			self.closeEdit.emit(False, self.rowId)
	
	def onValuesChanged( self, boolState ) :
		self.valuesChanged = boolState
		self.validBtn.setEnabled( boolState )
		
		
class PlaceEditDisplayWidgets(QWidget):

	def __init__(self, parent, db, rootMedia, rowId, rowParentId, rowDataObj):
		
		QWidget.__init__(self, parent)

		self.db = db
		self.rootMedia = rootMedia
		self.parent = parent
		self.rowId = rowId
		self.rowParentId = rowParentId
		self.rowDataObj = rowDataObj

		self.myPlaces = Places( self.db )

		gridLayout = QGridLayout()

		gridLayout.setSpacing(10)

		rowNumber = 1
		
		labelId = QLabel('id')
		labelId.setToolTip( 'Identifiant interne (Id)' )
		labelId.setAlignment(Qt.AlignRight | Qt.AlignVCenter)		
		self.InputId = QLineEdit()
		self.InputId.setMaximumWidth( self.getWidgetWidth( self.InputId, 5 )  )		
		self.InputId.setEnabled( False )
		self.InputId.setStyleSheet("color: #CCCCCC;")
		gridLayout.addWidget(labelId, rowNumber, 0)
		gridLayout.addWidget(self.InputId, rowNumber, 1)

		rowNumber = rowNumber + 1
			
		labelName = QLabel('Nom')
		labelName.setToolTip( 'Nom du lieu' )
		labelName.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputName = QLineEdit()
		self.InputName.setObjectName( "PlaceEditName" )
		self.InputName.installEventFilter(self)
		gridLayout.addWidget(labelName, rowNumber, 0)
		gridLayout.addWidget(self.InputName, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Nom: valeur obligatoire' )
		self.InputName.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
				
			
		labelDescription = QLabel('Description')
		labelDescription.setToolTip( 'Description du lieu' )
		labelDescription.setAlignment(Qt.AlignRight)
		self.InputDescription = QTextEdit()
		self.InputDescription.setObjectName( "PlaceEditDescription" )
		self.InputDescription.installEventFilter(self)
		gridLayout.addWidget(labelDescription, rowNumber, 0)
		gridLayout.addWidget(self.InputDescription, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Description: valeur obligatoire' )
		self.InputDescription.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
				

			
		labelPrivate = QLabel('Privé')
		labelPrivate.setToolTip( 'Donnée de type privé' )
		labelPrivate.setAlignment(Qt.AlignRight)
		self.InputPrivate = QCheckBox()
		self.InputPrivate.setObjectName( "PlaceEditPrivate" )
		self.InputPrivate.installEventFilter(self)
		gridLayout.addWidget(labelPrivate, rowNumber, 0)
		gridLayout.addWidget(self.InputPrivate, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Privé: valeur obligatoire' )
		self.InputPrivate.stateChanged.connect(self.onCheckBoxValuesChanged) 
		rowNumber = rowNumber + 1
					

			
		labelIcon = QLabel('Icone')
		labelIcon.setToolTip( 'Icône du lieu' )
		labelIcon.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputIcon = QLineEdit()
		self.InputIcon.setObjectName( "PlaceEditIcon" )
		self.InputIcon.installEventFilter(self)
		gridLayout.addWidget(labelIcon, rowNumber, 0)
		gridLayout.addWidget(self.InputIcon, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Icone: valeur obligatoire' )
		self.InputIcon.textChanged.connect(self.onLineEditValuesChanged)	
		self.fileSelectIcon = QPushButton("...")
		self.fileSelectIcon.clicked.connect(self.getfilesMediaIcon)
		gridLayout.addWidget(self.fileSelectIcon, rowNumber, 3)

		rowNumber = rowNumber + 1
			
		labelAdress1 = QLabel('Adresse 1')
		labelAdress1.setToolTip( 'Première ligne adresse ' )
		labelAdress1.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputAdress1 = QLineEdit()
		self.InputAdress1.setObjectName( "PlaceEditAdress1" )
		self.InputAdress1.installEventFilter(self)
		gridLayout.addWidget(labelAdress1, rowNumber, 0)
		gridLayout.addWidget(self.InputAdress1, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Adresse 1: valeur obligatoire' )
		self.InputAdress1.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelAdress2 = QLabel('Adresse 2')
		labelAdress2.setToolTip( 'Seconde ligne adresse' )
		labelAdress2.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputAdress2 = QLineEdit()
		self.InputAdress2.setObjectName( "PlaceEditAdress2" )
		self.InputAdress2.installEventFilter(self)
		gridLayout.addWidget(labelAdress2, rowNumber, 0)
		gridLayout.addWidget(self.InputAdress2, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Adresse 2: valeur obligatoire' )
		self.InputAdress2.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelZipcode = QLabel('Code postal')
		labelZipcode.setToolTip( 'Code postal' )
		labelZipcode.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputZipcode = QLineEdit()
		self.InputZipcode.setObjectName( "PlaceEditZipcode" )
		self.InputZipcode.installEventFilter(self)
		gridLayout.addWidget(labelZipcode, rowNumber, 0)
		gridLayout.addWidget(self.InputZipcode, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Code postal: valeur obligatoire' )
		self.InputZipcode.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelCity = QLabel('Ville')
		labelCity.setToolTip( 'Ville' )
		labelCity.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputCity = QLineEdit()
		self.InputCity.setObjectName( "PlaceEditCity" )
		self.InputCity.installEventFilter(self)
		gridLayout.addWidget(labelCity, rowNumber, 0)
		gridLayout.addWidget(self.InputCity, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Ville: valeur obligatoire' )
		self.InputCity.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelCityinsee = QLabel('Code insee')
		labelCityinsee.setToolTip( 'Code insee de la ville' )
		labelCityinsee.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputCityinsee = QLineEdit()
		self.InputCityinsee.setObjectName( "PlaceEditCityinsee" )
		self.InputCityinsee.installEventFilter(self)
		gridLayout.addWidget(labelCityinsee, rowNumber, 0)
		gridLayout.addWidget(self.InputCityinsee, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Code insee: valeur obligatoire' )
		self.InputCityinsee.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelCountry = QLabel('Pays')
		labelCountry.setToolTip( 'Pays' )
		labelCountry.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputCountry = QLineEdit()
		self.InputCountry.setObjectName( "PlaceEditCountry" )
		self.InputCountry.installEventFilter(self)
		gridLayout.addWidget(labelCountry, rowNumber, 0)
		gridLayout.addWidget(self.InputCountry, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Pays: valeur obligatoire' )
		self.InputCountry.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelUrl = QLabel('Url')
		labelUrl.setToolTip( 'Url' )
		labelUrl.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputUrl = QLineEdit()
		self.InputUrl.setObjectName( "PlaceEditUrl" )
		self.InputUrl.installEventFilter(self)
		gridLayout.addWidget(labelUrl, rowNumber, 0)
		gridLayout.addWidget(self.InputUrl, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Url: valeur obligatoire' )
		self.InputUrl.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
			
		labelAccurate = QLabel('Précision')
		labelAccurate.setToolTip( 'Précision du lieu' )
		labelAccurate.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputAccurate = QComboBox()
		self.InputAccurate.setObjectName( "PlaceEditAccurate" )
		self.InputAccurate.installEventFilter(self)
		gridLayout.addWidget(labelAccurate, rowNumber, 0)
		gridLayout.addWidget(self.InputAccurate, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Précision: valeur obligatoire' )
		comboData =  { "" : "", "EXA" : "Exactement", "ENV" : "Approximativement",  } 
		self.InputAccurate.clear()
		for key, value in comboData.items():
			self.InputAccurate.addItem( str(value) , userData=key ) 
		self.InputAccurate.currentIndexChanged.connect(self.onComboValuesChanged)
		rowNumber = rowNumber + 1
			
		labelGpslat = QLabel('Latitude')
		labelGpslat.setToolTip( 'Latitude du lieu' )
		labelGpslat.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputGpslat = QLineEdit()
		self.InputGpslat.setObjectName( "PlaceEditGpslat" )
		self.InputGpslat.installEventFilter(self)
		gridLayout.addWidget(labelGpslat, rowNumber, 0)
		gridLayout.addWidget(self.InputGpslat, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Latitude: valeur obligatoire' )
		self.InputGpslat.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelGpslon = QLabel('Longitude')
		labelGpslon.setToolTip( 'Longitude du lieu' )
		labelGpslon.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputGpslon = QLineEdit()
		self.InputGpslon.setObjectName( "PlaceEditGpslon" )
		self.InputGpslon.installEventFilter(self)
		gridLayout.addWidget(labelGpslon, rowNumber, 0)
		gridLayout.addWidget(self.InputGpslon, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Longitude: valeur obligatoire' )
		self.InputGpslon.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelZoom = QLabel('Zoom')
		labelZoom.setToolTip( 'Niveau de zoom lors de lʼaffichage de la carte' )
		labelZoom.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputZoom = QLineEdit()
		self.InputZoom.setObjectName( "PlaceEditZoom" )
		self.InputZoom.installEventFilter(self)
		gridLayout.addWidget(labelZoom, rowNumber, 0)
		gridLayout.addWidget(self.InputZoom, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Zoom: valeur obligatoire' )
		self.InputZoom.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
							
			
		labelDatecreated = QLabel('Crée le')
		labelDatecreated.setToolTip( 'Date de création' )
		labelDatecreated.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDatecreated = QDateTimeEdit()
		self.InputDatecreated.setObjectName( "PlaceEditDatecreated" )
		self.InputDatecreated.installEventFilter(self)
		self.InputDatecreated.setMaximumWidth( self.getWidgetWidth( self.InputDatecreated, 16 )  )		
		gridLayout.addWidget(labelDatecreated, rowNumber, 0)
		gridLayout.addWidget(self.InputDatecreated, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Crée le: valeur obligatoire' )
		self.InputDatecreated.dateTimeChanged.connect(self.onDateEditValuesChanged)	
		self.InputDatecreated.setDisplayFormat("dd/MM/yyyy hh:mm")
		self.InputDatecreated.setDateTimeRange(QDateTime(1, 1, 1, 0, 0), QDateTime(2999, 12, 31, 23, 59))
		self.InputDatecreated.setSpecialValueText(' ')
		self.InputDatecreated.setReadOnly(True)
		if True is False :
			self.InputDatecreated.setCalendarPopup(True)	


		self.setLayout(gridLayout)
		self.displayData( self.rowId, self.rowDataObj )
		

			

			

			

			

			

			

			

			

			

			

			

			

			

			

			

			


	

	

	

	

	

	

	

	

	

	

	

	

	

	

	

	


	

	

	

	

	def getfilesMediaIcon( self ) :      

		from libwidget.mediamanager import MediaManager
		from lib.project import Project  
		currentProject = Project()			
		currentProject.load( AppSetting.get('app/lastProject', '') )
		myMediaSelector = MediaManager( rootMedia = currentProject.rootMedia )
		myMediaSelector.loadDataList( self.InputIcon.text().split(',') )		
		result = myMediaSelector.exec_()
		filenamelist = myMediaSelector.getValue()
		if filenamelist is not None :
			self.printDebug("filenamelist = " , filenamelist)
			self.InputIcon.setText( ",".join(filenamelist) )

	

	

	

	

	

	

	

	

	

	

	

	



	

	

	

	

	

	

	

	

	

	

	

	

	

	

	

	



	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
	
	# Retourne la largeur necessaire pour un widget (QlineEdit ...) 
	# en fonctiion du nombre de caractères souhaité.
	def getWidgetWidth( self, widget, charmax ) :
		
		fm = widget.fontMetrics()
		try :
			m = widget.textMargins()
		except Exception as e:
			m = QMargins(8,8,8,8)
		c = widget.contentsMargins()
		w = charmax*fm.width('x')+m.left()+m.right()+c.left()+c.right()
		w = w + 8 # margins
		return w
			
	def onLineEditValuesChanged(self , text) :
		
		self.parent.onValuesChanged( True )

	def onTextEditValuesChanged(self):
		
		self.parent.onValuesChanged( True )
		
	def onComboValuesChanged(self, newIndex):
		
		self.parent.onValuesChanged( True )
				
	def onCheckBoxValuesChanged( self ,state) :
		self.parent.onValuesChanged( True )
			
	def onDateEditValuesChanged( self, newDate ) :
		
		self.parent.onValuesChanged( True )
				
	def displayData( self, rowId, dataObj = None) :

		self.rowId = rowId		

		self.parent.enterEditSignal.emit( self.rowId )
		
		if self.rowId == -1 :

			self.parent.mainWindowUserMessage.emit( 1 , "Formulaire vide (creation ou recréation)" )
		
		if dataObj is None :			
			placeObj = self.myPlaces.select( self.rowId )
			if placeObj is False :
				placeObj = Place()			
		else :
			placeObj = dataObj
						
		self.InputId.setText( str(placeObj.getId()) )		

			
		defaultValue = ''
		self.InputName.setText( str( placeObj.getName() if placeObj.getName() is not None else defaultValue ) ) 

			
			
		self.InputDescription.setText( str( placeObj.getDescription() if placeObj.getDescription() is not None else '' ) ) 

			
			
		if placeObj.getPrivate() is None or placeObj.getPrivate() == 0 :
			self.InputPrivate.setCheckState( Qt.CheckState.Unchecked ) 			
		if placeObj.getPrivate() == 1 :
			self.InputPrivate.setCheckState( Qt.CheckState.Checked ) 


			
			
		self.InputIcon.setText( str( placeObj.getIcon() if placeObj.getIcon() is not None else '' ) ) 

			
		defaultValue = ''
		self.InputAdress1.setText( str( placeObj.getAdress1() if placeObj.getAdress1() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputAdress2.setText( str( placeObj.getAdress2() if placeObj.getAdress2() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputZipcode.setText( str( placeObj.getZipcode() if placeObj.getZipcode() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputCity.setText( str( placeObj.getCity() if placeObj.getCity() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputCityinsee.setText( str( placeObj.getCityinsee() if placeObj.getCityinsee() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputCountry.setText( str( placeObj.getCountry() if placeObj.getCountry() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputUrl.setText( str( placeObj.getUrl() if placeObj.getUrl() is not None else defaultValue ) ) 

			
			
		currentKey = placeObj.getAccurate() if placeObj.getAccurate() is not None else '' 
		self.InputAccurate.setCurrentIndex( self.InputAccurate.findData(currentKey) ) 

			
		defaultValue = ''
		self.InputGpslat.setText( str( placeObj.getGpslat() if placeObj.getGpslat() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputGpslon.setText( str( placeObj.getGpslon() if placeObj.getGpslon() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputZoom.setText( str( placeObj.getZoom() if placeObj.getZoom() is not None else defaultValue ) ) 

									
			
		#myQdate = Util.getQDatetimeFromString( placeObj.getDatecreated(), defaultValue = QDateTime.currentDateTime().toString('dd/MM/yyyy') )
		myQdate = Util.getQDatetimeFromString( placeObj.getDatecreated() )
		self.InputDatecreated.setDateTime( myQdate ) 

		self.parent.onValuesChanged( False )


	def setFocusOnFirstWidget( self ) :

		from PyQt5.QtWidgets import  QLineEdit, QTextEdit, QDateEdit, QDateTimeEdit, QDateTimeEdit, QCheckBox, QComboBox,QDial, QDoubleSpinBox, QRadioButton, QSlider, QSpinBox, QTimeEdit, QFontComboBox
		for widget in self.children():
			if widget.isEnabled() :
				if 	isinstance(widget, QLineEdit) or \
					isinstance(widget, QTextEdit) or \
					isinstance(widget, QDateEdit) or \
					isinstance(widget, QDateTimeEdit) or \
					isinstance(widget, QCheckBox) or \
					isinstance(widget, QComboBox) or \
					isinstance(widget, QDial) or \
					isinstance(widget, QDoubleSpinBox) or \
					isinstance(widget, QRadioButton) or \
					isinstance(widget, QSlider) or \
					isinstance(widget, QSpinBox) or \
					isinstance(widget, QTimeEdit) or \
					isinstance(widget, QFontComboBox) \
					:
					#AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					widget.setFocus()					
					return

	def eventFilter(self, widgetObj, event):
		
		if event.type() == QtCore.QEvent.FocusIn:			
			self.focusWidget = widgetObj
	
		return super(PlaceEditDisplayWidgets, self).eventFilter(widgetObj, event)
    
	def restoreFocusWidget( self ) :		
		#AppLog.log( LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		AppLog.log( LOG_DEBUG , LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		self.focusWidget.setFocus()	
            

class PlaceDataSources() :
	
	# Jointure sur table(s): 	
	dataColumns = [ 'id', 'Nom', 'Description', 'Privé', 'Icone', 'Adresse 1', 'Adresse 2', 'Code postal', 'Ville', 'Code insee', 'Pays', 'Url', 'Précision', 'Latitude', 'Longitude', 'Zoom', 'Crée le'  ]
	dataSqlFields = [ 'id', 'name', 'description', 'private', 'icon', 'adress1', 'adress2', 'zipcode', 'city', 'cityinsee', 'country', 'url', 'accurate', 'gpslat', 'gpslon', 'zoom', 'datecreated'  ]	
	dataColumsType = ['INTEGER', 'TEXT', 'TEXT', 'BOOLEAN', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'INTEGER', 'TEXT', 'TEXT', 'TEXT', 'DECIMAL(16, 16)', 'DECIMAL(16, 16)', 'INTEGER', 'DATETIME'  ]
	dataTooltips = [ 'Identifiant du lieu', 'Nom du lieu', 'Description du lieu', 'Donnée de type privé', 'Icône du lieu', 'Première ligne adresse ', 'Seconde ligne adresse', 'Code postal', 'Ville', 'Code insee de la ville', 'Pays', 'Url', 'Précision du lieu', 'Latitude du lieu', 'Longitude du lieu', 'Niveau de zoom lors de lʼaffichage de la carte', 'Date de création'  ]
			
	def __init__(self, db ):
		self.db = db
		pass
		
	def selectAll( self , field_name = None, field_value = None ) :
		
		if field_name is None and field_value is None :
			return self.select()
		else :
			return self.selectById( field_name, field_value )
			
					
	def select( self ) :
		
		dataRows = []
		
		self.myPlaces = Places( self.db )
		placeListAll = self.myPlaces.selectAll()
		if placeListAll :
			i = 0
			for place in placeListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(place.getId()))
				dataRows.append( [ place.getId(),  place.getName(),  place.getDescription(),  place.getPrivate(),  place.getIcon(),  place.getAdress1(),  place.getAdress2(),  place.getZipcode(),  place.getCity(),  place.getCityinsee(),  place.getCountry(),  place.getUrl(),  place.getAccurate(),  place.getGpslat(),  place.getGpslon(),  place.getZoom(),  place.getDatecreated()  ] )
				i = i + 1
			
		return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )


	def selectById( self, where_clause, fields_values ) :
		
		dataRows = []
				
		self.myPlaces = Places( self.db )
		placeListAll = self.myPlaces.selectById( where_clause, fields_values )
		if placeListAll : 
			i = 0
			for place in placeListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(place.getId()))
				dataRows.append( [ place.getId(),  place.getName(),  place.getDescription(),  place.getPrivate(),  place.getIcon(),  place.getAdress1(),  place.getAdress2(),  place.getZipcode(),  place.getCity(),  place.getCityinsee(),  place.getCountry(),  place.getUrl(),  place.getAccurate(),  place.getGpslat(),  place.getGpslon(),  place.getZoom(),  place.getDatecreated()  ] )
				i = i + 1
				
			return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
		return ( None, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
	# Retourner le champ #x lors de la selection par pop-up (Fk)
	def getFkIndexLibBack( self ) :
		return 1
