#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

		*** Source autogénéré *** 
		
		Pour la table : source.

		----------------------------
		Source.
		
		Liste des sources.
		----------------------------		

		05/24/2023 14:31

		lvardon@laposte.net - 2021

		Licence libre

'''
from PyQt5 import QtWidgets
from PyQt5.QtWidgets import QAction, QMenu, QApplication, QWidget, QToolTip, QPushButton, QMessageBox, QFileDialog
from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, QHBoxLayout, QScrollArea
from PyQt5.QtWidgets import QLabel, QLineEdit, QTextEdit, QComboBox, QCheckBox, QDateTimeEdit
from PyQt5.QtGui import QCursor, QFont, QIcon
from PyQt5.QtCore import Qt, QEvent, pyqtSignal, pyqtSlot, QModelIndex, QDateTime, QMargins, QSettings
from PyQt5 import QtCore
		
from lib.sqlitedb import Sqlitedb
from libdbcustom.source import Source, Sources

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
from lib.util import Util
from libwidget.tableModel import TableModel, TableProxyModel
from libwidget.tableView import TableView,TableCellImageDelegate

import lib.setting as AppSetting
import lib.log as AppLog

class SourceListWidget(QWidget):

	mainWindowEditMode =  pyqtSignal( int, str )
	mainWindowRemoveRow =  pyqtSignal( int, str )	
	mainWindowListMode = pyqtSignal()
	mainWindowUserMessage = pyqtSignal( int, str )
	cellClickSignal =  pyqtSignal( int )
	cellDoubleClickSignal =  pyqtSignal( int )
	selectionSignal = pyqtSignal( int )
	
	def __init__(self , db, rootMedia, sourceOrigin = "", where_clause = None, fields_values = None):
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia 
		self.sourceOrigin = sourceOrigin
		
		self.where_clause = where_clause
		self.fields_values = fields_values
		
		self.initUI()

		self.initDelegates(  self.tableView )
		
		iconW, iconH, rowHeight = self.getTableViewIniParams( "tablelistviewSource" )
		self.tableView.verticalHeader().setDefaultSectionSize(int(rowHeight))

		try :
			self.iconImageDelegate.icoW = int(iconW) 
			self.iconImageDelegate.icoH = int(iconH)	
		except Exception as e:
			pass
		try :
			self.urlImageDelegate.icoW = int(iconW) 
			self.urlImageDelegate.icoH = int(iconH)					
		except Exception as e:
			pass
		
		self.setObjectName( "SourceListWidget" )

	def initDelegates( self, thistableView ) :
				
		self.privateImageDelegate = TableCellImageDelegate(self, "source", {'0' : 'img/dataicons/empty.png', '1' : 'img/dataicons/private-yes.png', '2' : 'img/dataicons/private-yes.png'} , self.rootMedia, 26, 26)
		thistableView.setItemDelegateForColumn(3, self.privateImageDelegate)

		self.iconImageDelegate = TableCellImageDelegate(self, "source", None , self.rootMedia, 40, 40)
		thistableView.setItemDelegateForColumn(4, self.iconImageDelegate)

		self.qualityImageDelegate = TableCellImageDelegate(self, "source", {'0' : 'img/dataicons/quality_0.png', '1' : 'img/dataicons/quality_1.png', '2' : 'img/dataicons/quality_2.png', '3' : 'img/dataicons/quality_3.png', '4' : 'img/dataicons/quality_4.png' } , self.rootMedia, 26, 26)
		thistableView.setItemDelegateForColumn(5, self.qualityImageDelegate)

		
		thistableView.verticalHeader().setDefaultSectionSize(40)

	def initUI(self):
		
		self.layout = QVBoxLayout()
		self.setLayout(self.layout)
				
		self.proxyModel = TableProxyModel(self)
		
		self.proxyModel.setSortRole(Qt.UserRole + 2)
		
		self.tableView = TableView( self, "source", self.sourceOrigin )	
				
		self.loadData( self.where_clause , self.fields_values ) 
	
		self.tableView.setModel(self.proxyModel)
		
		self.tableView.resizeColumnsToContents()

		self.tableView.cellClickSignal.connect(self.onCellClic)
		self.tableView.cellDoubleClickSignal.connect(self.onCellDoubleClic)
	
		selection_model = self.tableView.selectionModel()
		selection_model.selectionChanged.connect(self.onSelectionChanged)      
		
		self.filterValue = QLineEdit()
		self.layout.addWidget(self.filterValue)
		self.filterValue.textChanged.connect(self.onfilterValueChanged)
		
		self.layout.addWidget(self.tableView)

	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
			

	def getTableViewIniParams ( self, tableViewName) :
		
		iniConfigFile = "config/app.ini"
		settings=QSettings(iniConfigFile, QSettings.IniFormat)

		rowHeight = 0
		if settings.value(tableViewName + '/rowHeight') is not None:			
			rowHeight = settings.value( tableViewName +'/rowHeight')						
			
		iconW = 0
		if settings.value( tableViewName + '/rowIconW') is not None:						
			iconW = settings.value( tableViewName +'/rowIconW')				
		
		iconH = 0			
		if settings.value( tableViewName +'/rowIconH') is not None:			
			iconH = settings.value( tableViewName +'/rowIconH')	

		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" rowHeight="+ str(rowHeight))
		#AppLog.log( LOG_INFO , "Selectdialog "+tableViewName+" iconW x IconH="+ str(iconW) + " x " + str(iconH))
		
		return iconW, iconH, rowHeight
		
	@pyqtSlot('QItemSelection', 'QItemSelection')
	def onSelectionChanged(self, selected, deselected):
		try :
			rowId =  selected.indexes()[0].data()
			self.selectionSignal.emit( rowId )
		except Exception as e:
			pass		
		
	def onfilterValueChanged(self , text) :
		self.proxyModel.setFilterValue( str(text) )		
		self.proxyModel.applyFilters()

	def onCellClic(self, rowId ):
		self.cellClickSignal.emit(rowId)

	def onCellDoubleClic(self, rowId ):
		self.cellDoubleClickSignal.emit(rowId)
		self.onEditRow(rowId)
		
	def reloadData(self) :
		self.loadData(self.where_clause, self.fields_values)		
		
	def loadData( self, where_clause , fields_values ) :
		
		self.where_clause = where_clause
		self.fields_values = fields_values
				
		dataSource = SourceDataSources( self.db )
		dataRows, dataColumns, self.dataSqlNames, dataColumnsType, dataToolTips = dataSource.selectAll( self.where_clause, self.fields_values )
		
		self.model = TableModel("source", dataRows, dataColumns, dataColumnsType, dataToolTips)

		self.proxyModel.setSourceModel(self.model)
		
		self.tableView.resizeColumnsToContents()				
				
	def dumpDatas( self) :
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		for i in range(0, row) :
			self.printDebug("--- Row : ", i)
			for j in range(0, col) :
				self.printDebug( 
						self.model.data( self.model.index(i, j), Qt.UserRole + 1 ),
						" - ",
						self.model.data( self.model.index(i, j), Qt.UserRole ) 
					)

	# Nom de la colonne sur laquelle porte le tri 
	def getSqlOrderByField( self) :
		orderByFieldPos = self.tableView.hView.sortIndicatorSection()
		orderByField = self.dataSqlNames[orderByFieldPos]
		
		return orderByField
		
	# Index du sens de tri 'DESC' = 0 'ASC' = 1
	def getSqlOrderByOrder( self) :
		orderBy = self.tableView.hView.sortIndicatorOrder()
		return orderBy
		
		
	# Liste des id de la liste affichée
	def getIdList( self ) :
		
		idList = []
		
		indexId = 0 # id column is at pos 0
		
		row = self.proxyModel.rowCount()
		model = self.proxyModel
		
		for i in range(0, row) :			
			idList.append( model.data( model.index(i, indexId ), Qt.UserRole ) )
		
		return idList
			
	
	def selectRowById( self, rowId ) :

		if rowId is None :
			return
			
		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel
		
		for i in range(0, row) :
			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :				
				self.tableView.selectRow( i )
				return

	def getRowData( self, rowId ) :

		mySourceObj = Source( )

		row = self.proxyModel.rowCount()
		col = self.proxyModel.columnCount()
		model = self.proxyModel

		for i in range(0, row) :
			#print('row:', i, " model rowid=", self.model.data( self.model.index(i, 0), Qt.UserRole ), ' == ',  rowId, ' **', self.model.data( self.model.index(i, 1), Qt.UserRole ) )

			if model.data( model.index(i, 0), Qt.UserRole ) == rowId :

				mySourceObj.setId( model.data( model.index(i, 0 ), Qt.UserRole  )  )
				mySourceObj.setName( model.data( model.index(i, 1 ), Qt.UserRole  )  )
				mySourceObj.setDescription( model.data( model.index(i, 2 ), Qt.UserRole  )  )
				mySourceObj.setPrivate( model.data( model.index(i, 3 ), Qt.UserRole  )  )
				mySourceObj.setIcon( model.data( model.index(i, 4 ), Qt.UserRole  )  )
				mySourceObj.setQuality( model.data( model.index(i, 5 ), Qt.UserRole  )  )
				mySourceObj.setLocalurl( model.data( model.index(i, 6 ), Qt.UserRole  )  )
				mySourceObj.setSourceurl( model.data( model.index(i, 7 ), Qt.UserRole  )  )
				mySourceObj.setSourcetext( model.data( model.index(i, 8 ), Qt.UserRole  )  )
				mySourceObj.setPage( model.data( model.index(i, 9 ), Qt.UserRole  )  )
				mySourceObj.setCitation( model.data( model.index(i, 10 ), Qt.UserRole  )  )
				mySourceObj.setTextdraduction( model.data( model.index(i, 11 ), Qt.UserRole  )  )
				mySourceObj.setNotesearch( model.data( model.index(i, 12 ), Qt.UserRole  )  )
				mySourceObj.setSortorder( model.data( model.index(i, 13 ), Qt.UserRole  )  )
				mySourceObj.setDatecreated( model.data( model.index(i, 14 ), Qt.UserRole  )  )

				return mySourceObj

		return mySourceObj

	def contextMenuEvent(self, event):

		gp = event.globalPos()
		vp_pos = self.tableView.viewport().mapFromGlobal(gp)
		rowNum = self.tableView.rowAt(vp_pos.y())
		colNum = self.tableView.columnAt(event.x())

		self.menu = QMenu(self)
			
		if rowNum >= 0 :

			index = self.tableView.indexAt(vp_pos)
			rowId = self.proxyModel.index(index.row(),0).data()

			cellValue = self.proxyModel.index(rowNum, colNum).data()

			deleteAction = QAction( QIcon('img/context_del.png'), 'Supprimer', self)
			deleteAction.triggered.connect(lambda : self.onDeleteRow(rowId, rowNum))
			self.menu.addAction(deleteAction)

			editAction = QAction( QIcon('img/context_edit.png'), 'Editer', self)
			editAction.triggered.connect(lambda : self.onEditRow(rowId))
			self.menu.addAction(editAction)

		newAction = QAction(QIcon('img/context_add.png'), 'Nouveau', self)
		newAction.triggered.connect(lambda checked : self.onNewRow())
		self.menu.addAction(newAction)			

		self.menu.addSeparator()
		
		storeStateAction = QAction(QIcon('img/context_save.png'), 'Enregistrer la disposition', self)
		storeStateAction.triggered.connect(lambda checked : self.onStoreState())
		self.menu.addAction(storeStateAction)

		self.menu.popup(QCursor.pos())

	def selectSourceAction(self) : 
		
		from libwidget.selectDialog import SelectDialog
		from libview.source import SourceDataSources	
		dataRows, dataColumns, self.dataSqlNames, dataColumsType, dataTooltips = SourceDataSources( self.db ).selectAll()			
		dialog = SelectDialog()
		dialog.loadData( dataRows, dataColumns, dataColumsType, dataTooltips, 'Sélection: '+str( 'Source') , "source", SourceDataSources(self).getFkIndexLibBack() )		
		result = dialog.exec_()
		rowId, rowLib = dialog.getValue()
		if rowId is not None :
			if rowId >= 0 :
				self.mainWindowEditMode.emit( int(rowId) , "Edit mode selection")
				
	
	def onStoreState( self ) :
		self.tableView.storeState()

	def onNewRow(self):
		
		self.mainWindowEditMode.emit( -1 , "Edit mode New")

	def onEditRow(self, rowId):

		self.mainWindowEditMode.emit( int(rowId) , "Edit mode")

	def onDeleteRow(self, rowdbId, rowIndex):

		QApplication.setOverrideCursor(Qt.WaitCursor)
		self.mySources = Sources( self.db )
		sourceObj = self.mySources.select( rowdbId )
		QApplication.restoreOverrideCursor()

		if sourceObj :

			dlg = SourceDeleteConfirm(self.db, self, rowdbId, sourceObj.getName()) 
			
			returnCode = dlg.exec()
			if returnCode == QDialog.Accepted :
				self.proxyModel.removeRow(rowIndex)				
				self.mainWindowRemoveRow.emit( rowdbId, "row removed")
				
class SourceDeleteConfirm(QDialog):

	parent = None

	def __init__(self, db, parent=None, rowid=-1, libRecord="" ):
		super().__init__(parent)

		self.db = db
		self.parent = parent
		self.rowId = rowid
		self.libRecord = libRecord

		self.setWindowTitle("Source: suppression de #" + str(self.rowId)  )		

		QBtn = QDialogButtonBox.Ok | QDialogButtonBox.Cancel

		self.buttonBox = QDialogButtonBox(QBtn)
		self.buttonBox.accepted.connect(self.onOk)
		self.buttonBox.rejected.connect(self.onCancel)

		self.layout = QGridLayout() 
		
		# http://srinikom.github.io/pyside-docs/PySide/QtGui/QStyle.html#PySide.QtGui.PySide.QtGui.QStyle.StandardPixmap
		from PyQt5.QtWidgets import QApplication, QStyle
		from PyQt5.QtGui import QIcon

		labelIcon = QLabel("icon")
		dialogIcon = QIcon(QApplication.style().standardIcon(QStyle.SP_MessageBoxQuestion) )       
		labelIcon.setPixmap( dialogIcon.pixmap(48,48)  )
		self.layout.addWidget(labelIcon,0,0)				
		
		message = QLabel("Confirmez-vous la suppression de : \n" )
		self.layout.addWidget(message,0,1)

		data = QLabel(str(self.libRecord))
		self.layout.addWidget(data,1,1)
			
		self.layout.addWidget(self.buttonBox,2,1)
		self.setLayout(self.layout)

	def onCancel( self ) :
		self.reject()

	def onOk( self ) :
		self.mySources = Sources( self.db )
		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.mySources.delete( self.rowId )
		QApplication.restoreOverrideCursor()

		if result is False :
			self.parent.mainWindowUserMessage.emit( 1, "ERREUR SQL : " + str(self.mySources.lastError) )
			self.reject()
		else :
			self.parent.mainWindowUserMessage.emit( 0 , "Suppresion effectuée. (id:"+str(self.rowId) + ")")
			self.parent.mainWindowListMode.emit()
			self.accept()


class SourceEditWidget(QWidget):

	mainWindowUserMessage = pyqtSignal( int, str )
	mainWindowListMode = pyqtSignal( )
	enterEditSignal = pyqtSignal( int )
	closeEdit = pyqtSignal( int, int )	
	validEditSignal = pyqtSignal( bool, int )
	valuesChanged = False
	
	recPos = 0
	siblingIdList = []
	siblingOrderField = "1"
	siblingOrderBy = ""

	def __init__(self, db, rootMedia, rowId, parentId, siblingIdList, siblingOrderField, siblingOrderBy, rowDataObj = None ):		
		
		super().__init__()

		self.db = db
		self.rootMedia = rootMedia
		self.parentId = parentId
		
		self.siblingIdList = siblingIdList		
		self.siblingOrderField = siblingOrderField # Nom Sql du champ sur lequel porte le tri		
		sqlOrderBy = ['DESC','ASC'][siblingOrderBy]
		self.siblingOrderBy = sqlOrderBy		
		
		self.rowDataObj = rowDataObj

		self.mySources = Sources( self.db )

		sourceObj = self.mySources.select( rowId )
		if sourceObj is False :
			self.mainWindowUserMessage.emit( 0 , "Cet enregistrement a été supprimé.")
			self.rowId = -1
		else :
			self.rowId = rowId

		if self.rowId in self.siblingIdList:
			self.recPos = self.siblingIdList.index(self.rowId)
		else:
			self.recPos = 0

		self.initUI()
		
		self.setObjectName( "SourceEditWidget" )

	def showEvent(self, event):
		super(SourceEditWidget, self).showEvent(event)
		self.WidgetsForm.setFocusOnFirstWidget()	
		self.setPrevNextButtonState()		
		
	def initUI(self):

		pageLayout = QGridLayout()
		pageButtonLayout = QHBoxLayout()
		
		self.closeBtn = QPushButton("Fermer", self)
		self.closeBtn.setToolTip( "Fermer et afficher la liste Source" )
		self.closeBtn.setIcon(QIcon('img/form_back.png'))
		self.closeBtn.clicked.connect(self.onClose)
		pageButtonLayout.addWidget(self.closeBtn)

		self.validBtn = QPushButton("Sauver", self)
		self.validBtn.setToolTip( "Enregistrer les modifications Source" )
		self.validBtn.setIcon(QIcon('img/form_save.png'))		
		self.validBtn.clicked.connect(self.onValid)
		pageButtonLayout.addWidget(self.validBtn)

		self.updateMessage = QLineEdit( self )
		self.updateMessage.setStyleSheet("QLineEdit { background-color : rgb(255, 255, 255); color: red; border : 0px ;}")
		self.updateMessage.setToolTip( "Dernière erreur rencontrée lors de l'édition" )
		pageButtonLayout.addWidget(self.updateMessage)

		self.prevBtn = QPushButton("Précédent", self)
		self.prevBtn.setToolTip( "Source précédent" )
		self.prevBtn.setIcon(QIcon('img/form_prev.png'))		
		self.prevBtn.clicked.connect(self.onPrev)
		pageButtonLayout.addWidget(self.prevBtn)

		self.nextBtn = QPushButton("Suivant", self)
		self.nextBtn.setToolTip( "Source suivant" )
		self.nextBtn.setIcon(QIcon('img/form_next.png'))		
		self.nextBtn.clicked.connect(self.onNext)
		pageButtonLayout.addWidget(self.nextBtn)
		self.nextBtn.setLayoutDirection(Qt.RightToLeft)		

		self.WidgetsForm = SourceEditDisplayWidgets(self, self.db, self.rootMedia, self.rowId, self.parentId, self.rowDataObj)
		
		scroll = QScrollArea(self)
		scroll.setWidget(self.WidgetsForm)
		scroll.setWidgetResizable(True)
		pageLayout.addWidget(scroll)

		pageLayout.addLayout(pageButtonLayout, 1,0)

		self.setLayout(pageLayout)

	def enableNavButtons( self ) :
		self.nextBtn.setEnabled( True )
		self.prevBtn.setEnabled( True )

	def setPrevNextButtonState( self ) :
		
		if self.recPos <= 0 :
			self.prevBtn.setEnabled( False )
		else :
			self.prevBtn.setEnabled( True )

		if self.recPos >= len(self.siblingIdList)-1 :
			self.nextBtn.setEnabled( False )
		else :
			self.nextBtn.setEnabled( True )	
			
	def onPrev( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
					quitForm = False
					
		if quitForm is True :					
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos - 1			
			if self.recPos < 0 :
				self.recPos = 0		
			
			sourceObj = self.mySources.select( self.siblingIdList[self.recPos]  )
			
			if sourceObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.mySources.selectCount == 1 :
				self.rowDataObj = sourceObj			
				self.rowId = sourceObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()
		
	def onNext( self ) :
		
		quitForm = True
		
		if self.valuesChanged is True :
			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.No:	
				quitForm = False
			
		if quitForm is True :
			self.enableNavButtons()
			self.updateMessage.setText( "" )
			self.mainWindowUserMessage.emit( -1 , "" )
			QApplication.setOverrideCursor(Qt.WaitCursor)
						
			bck_recPos = self.recPos
			self.recPos = self.recPos + 1
			if self.recPos >= len(self.siblingIdList) :
				self.recPos = len(self.siblingIdList) - 1
			
			sourceObj = self.mySources.select( self.siblingIdList[self.recPos]  )
			
			if sourceObj is False :
				self.recPos = bck_recPos
							
			QApplication.restoreOverrideCursor()

			if self.mySources.selectCount == 1 :
				self.rowDataObj = sourceObj
				self.rowId = sourceObj.getId()
				self.WidgetsForm.displayData(  self.rowId  )

			self.setPrevNextButtonState()

		self.WidgetsForm.restoreFocusWidget()

	def onValid( self ) :
		self.updateMessage.setText( "" )

		mySourceObj = Source()

		mySourceObj.setId( self.rowId )

			
		mySourceObj.setName( self.WidgetsForm.InputName.text() )					

			
			
		mySourceObj.setDescription( self.WidgetsForm.InputDescription.toPlainText() )					

								
			
		if self.WidgetsForm.InputPrivate.checkState() == Qt.CheckState.Unchecked :
			checkBoxState = 0
		else :
			checkBoxState = 1
		mySourceObj.setPrivate( checkBoxState ) 
							

								
			
		mySourceObj.setIcon( self.WidgetsForm.InputIcon.text() )					

			
			
		mySourceObj.setQuality( self.WidgetsForm.InputQuality.itemData(self.WidgetsForm.InputQuality.currentIndex()) )					

								
			
		mySourceObj.setLocalurl( self.WidgetsForm.InputLocalurl.text() )					

								
			
		mySourceObj.setSourceurl( self.WidgetsForm.InputSourceurl.text() )					

			
			
		mySourceObj.setSourcetext( self.WidgetsForm.InputSourcetext.toPlainText() )					

			
		mySourceObj.setPage( self.WidgetsForm.InputPage.text() )					

			
		mySourceObj.setCitation( self.WidgetsForm.InputCitation.text() )					

			
			
		mySourceObj.setTextdraduction( self.WidgetsForm.InputTextdraduction.toPlainText() )					

			
			
		mySourceObj.setNotesearch( self.WidgetsForm.InputNotesearch.toPlainText() )					

			
		mySourceObj.setSortorder( self.WidgetsForm.InputSortorder.text() )					

			
			
		# dd/MM/yyyy hh:mm
		mySourceObj.setDatecreated( self.WidgetsForm.InputDatecreated.dateTime().toString('yyyy-MM-dd hh:mm') )					

		QApplication.setOverrideCursor(Qt.WaitCursor)
		result = self.mySources.update( mySourceObj )
		QApplication.restoreOverrideCursor()

		self.updateMessage.setText( "" )
		self.mainWindowUserMessage.emit( -1 , "" )

		if (result is False) :
			self.updateMessage.setText( "ERREUR SQL : " + str(self.mySources.lastError) )
			self.updateMessage.setToolTip( "ERREUR SQL : " + str(self.mySources.lastError) )
			self.validEditSignal.emit( False, self.rowId )
		else :
			self.rowId = self.mySources.lastInsertId
			mySourceObj.setId( self.rowId )
			self.mainWindowUserMessage.emit( 0 , "Mise à jour effectuée. (id:"+str(self.rowId) + ")")
			self.WidgetsForm.displayData( self.rowId )
			self.validEditSignal.emit( True, self.rowId )

	def onClose( self ) :

		if self.valuesChanged is True :

			self.mainWindowUserMessage.emit( -1 , "Les données ont été modifiées." )
			
			quit_msg = "Quitter sans sauvegarder ?"
			reply = QMessageBox.question(self, 'Message', 
								 quit_msg, QMessageBox.Yes, QMessageBox.No)

			if reply == QMessageBox.Yes:
				self.mainWindowUserMessage.emit( -1 , "" )
				self.closeEdit.emit(True, self.rowId)
				
		else :
			self.mainWindowUserMessage.emit( -1 , "" )
			self.closeEdit.emit(False, self.rowId)
	
	def onValuesChanged( self, boolState ) :
		self.valuesChanged = boolState
		self.validBtn.setEnabled( boolState )
		
		
class SourceEditDisplayWidgets(QWidget):

	def __init__(self, parent, db, rootMedia, rowId, rowParentId, rowDataObj):
		
		QWidget.__init__(self, parent)

		self.db = db
		self.rootMedia = rootMedia
		self.parent = parent
		self.rowId = rowId
		self.rowParentId = rowParentId
		self.rowDataObj = rowDataObj

		self.mySources = Sources( self.db )

		gridLayout = QGridLayout()

		gridLayout.setSpacing(10)

		rowNumber = 1
		
		labelId = QLabel('id')
		labelId.setToolTip( 'Identifiant interne (Id)' )
		labelId.setAlignment(Qt.AlignRight | Qt.AlignVCenter)		
		self.InputId = QLineEdit()
		self.InputId.setMaximumWidth( self.getWidgetWidth( self.InputId, 5 )  )		
		self.InputId.setEnabled( False )
		self.InputId.setStyleSheet("color: #CCCCCC;")
		gridLayout.addWidget(labelId, rowNumber, 0)
		gridLayout.addWidget(self.InputId, rowNumber, 1)

		rowNumber = rowNumber + 1
			
		labelName = QLabel('Nom')
		labelName.setToolTip( 'Nom de la source' )
		labelName.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputName = QLineEdit()
		self.InputName.setObjectName( "SourceEditName" )
		self.InputName.installEventFilter(self)
		gridLayout.addWidget(labelName, rowNumber, 0)
		gridLayout.addWidget(self.InputName, rowNumber, 1)
		mandatoryFlag = QLabel('*')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Nom: valeur obligatoire' )
		self.InputName.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
				
			
		labelDescription = QLabel('Description')
		labelDescription.setToolTip( 'Description de la source' )
		labelDescription.setAlignment(Qt.AlignRight)
		self.InputDescription = QTextEdit()
		self.InputDescription.setObjectName( "SourceEditDescription" )
		self.InputDescription.installEventFilter(self)
		gridLayout.addWidget(labelDescription, rowNumber, 0)
		gridLayout.addWidget(self.InputDescription, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Description: valeur obligatoire' )
		self.InputDescription.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
				

			
		labelPrivate = QLabel('Privée')
		labelPrivate.setToolTip( 'Source de type privée' )
		labelPrivate.setAlignment(Qt.AlignRight)
		self.InputPrivate = QCheckBox()
		self.InputPrivate.setObjectName( "SourceEditPrivate" )
		self.InputPrivate.installEventFilter(self)
		gridLayout.addWidget(labelPrivate, rowNumber, 0)
		gridLayout.addWidget(self.InputPrivate, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Privée: valeur obligatoire' )
		self.InputPrivate.stateChanged.connect(self.onCheckBoxValuesChanged) 
		rowNumber = rowNumber + 1
					

			
		labelIcon = QLabel('Icône')
		labelIcon.setToolTip( 'Icône, image d’illustration' )
		labelIcon.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputIcon = QLineEdit()
		self.InputIcon.setObjectName( "SourceEditIcon" )
		self.InputIcon.installEventFilter(self)
		gridLayout.addWidget(labelIcon, rowNumber, 0)
		gridLayout.addWidget(self.InputIcon, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Icône: valeur obligatoire' )
		self.InputIcon.textChanged.connect(self.onLineEditValuesChanged)	
		self.fileSelectIcon = QPushButton("...")
		self.fileSelectIcon.clicked.connect(self.getfilesMediaIcon)
		gridLayout.addWidget(self.fileSelectIcon, rowNumber, 3)

		rowNumber = rowNumber + 1
			
			
		labelQuality = QLabel('Qualité')
		labelQuality.setToolTip( 'Qualité, crédibilité de la source (0-3)' )
		labelQuality.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputQuality = QComboBox()
		self.InputQuality.setObjectName( "SourceEditQuality" )
		self.InputQuality.installEventFilter(self)
		gridLayout.addWidget(labelQuality, rowNumber, 0)
		gridLayout.addWidget(self.InputQuality, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Qualité: valeur obligatoire' )
		comboData =  { "" : "", "0" : "Source non fiable ou estimation", "1" : "Source de fiabilité douteuse", "2" : "Source indirecte", "3" : "Preuve indiscutable",  } 
		self.InputQuality.clear()
		for key, value in comboData.items():
			self.InputQuality.addItem( str(value) , userData=key ) 
		self.InputQuality.currentIndexChanged.connect(self.onComboValuesChanged)
		rowNumber = rowNumber + 1
						
			
		labelLocalurl = QLabel('Source image')
		labelLocalurl.setToolTip( 'Images locales : png, jpg, et annotations' )
		labelLocalurl.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputLocalurl = QLineEdit()
		self.InputLocalurl.setObjectName( "SourceEditLocalurl" )
		self.InputLocalurl.installEventFilter(self)
		gridLayout.addWidget(labelLocalurl, rowNumber, 0)
		gridLayout.addWidget(self.InputLocalurl, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Source image: valeur obligatoire' )
		self.InputLocalurl.textChanged.connect(self.onLineEditValuesChanged)	
		self.fileSelectLocalurl = QPushButton("...")
		self.fileSelectLocalurl.clicked.connect(self.getfilesPicLocalurl)
		gridLayout.addWidget(self.fileSelectLocalurl, rowNumber, 3)

		rowNumber = rowNumber + 1
					

			
		labelSourceurl = QLabel('Source url')
		labelSourceurl.setToolTip( 'Url de la source (http, .pdf, .docx, .txt ...)' )
		labelSourceurl.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputSourceurl = QLineEdit()
		self.InputSourceurl.setObjectName( "SourceEditSourceurl" )
		self.InputSourceurl.installEventFilter(self)
		gridLayout.addWidget(labelSourceurl, rowNumber, 0)
		gridLayout.addWidget(self.InputSourceurl, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Source url: valeur obligatoire' )
		self.InputSourceurl.textChanged.connect(self.onLineEditValuesChanged)	
		self.fileSelectSourceurl = QPushButton("...")
		self.fileSelectSourceurl.clicked.connect(self.getfilesMediaSourceurl)
		gridLayout.addWidget(self.fileSelectSourceurl, rowNumber, 3)

		rowNumber = rowNumber + 1
				
			
		labelSourcetext = QLabel('Source texte')
		labelSourcetext.setToolTip( 'Texte de la source' )
		labelSourcetext.setAlignment(Qt.AlignRight)
		self.InputSourcetext = QTextEdit()
		self.InputSourcetext.setObjectName( "SourceEditSourcetext" )
		self.InputSourcetext.installEventFilter(self)
		gridLayout.addWidget(labelSourcetext, rowNumber, 0)
		gridLayout.addWidget(self.InputSourcetext, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Source texte: valeur obligatoire' )
		self.InputSourcetext.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
			
		labelPage = QLabel('Page')
		labelPage.setToolTip( 'Numéro de la page (ou vue) dans la source' )
		labelPage.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputPage = QLineEdit()
		self.InputPage.setObjectName( "SourceEditPage" )
		self.InputPage.installEventFilter(self)
		gridLayout.addWidget(labelPage, rowNumber, 0)
		gridLayout.addWidget(self.InputPage, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Page: valeur obligatoire' )
		self.InputPage.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
			
		labelCitation = QLabel('Citation')
		labelCitation.setToolTip( 'Précision sur lʼemplacement dans la page (en bas, à gauche ...)' )
		labelCitation.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputCitation = QLineEdit()
		self.InputCitation.setObjectName( "SourceEditCitation" )
		self.InputCitation.installEventFilter(self)
		gridLayout.addWidget(labelCitation, rowNumber, 0)
		gridLayout.addWidget(self.InputCitation, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Citation: valeur obligatoire' )
		self.InputCitation.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
				
			
		labelTextdraduction = QLabel('Transcription')
		labelTextdraduction.setToolTip( 'Traduction du texte' )
		labelTextdraduction.setAlignment(Qt.AlignRight)
		self.InputTextdraduction = QTextEdit()
		self.InputTextdraduction.setObjectName( "SourceEditTextdraduction" )
		self.InputTextdraduction.installEventFilter(self)
		gridLayout.addWidget(labelTextdraduction, rowNumber, 0)
		gridLayout.addWidget(self.InputTextdraduction, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Transcription: valeur obligatoire' )
		self.InputTextdraduction.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
				
			
		labelNotesearch = QLabel('Notes de recherche')
		labelNotesearch.setToolTip( 'Notes de recherche' )
		labelNotesearch.setAlignment(Qt.AlignRight)
		self.InputNotesearch = QTextEdit()
		self.InputNotesearch.setObjectName( "SourceEditNotesearch" )
		self.InputNotesearch.installEventFilter(self)
		gridLayout.addWidget(labelNotesearch, rowNumber, 0)
		gridLayout.addWidget(self.InputNotesearch, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Notes de recherche: valeur obligatoire' )
		self.InputNotesearch.textChanged.connect(self.onTextEditValuesChanged) 
		rowNumber = rowNumber + 1
			
		labelSortorder = QLabel('Ordre de tri')
		labelSortorder.setToolTip( 'Ordre de tri' )
		labelSortorder.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputSortorder = QLineEdit()
		self.InputSortorder.setObjectName( "SourceEditSortorder" )
		self.InputSortorder.installEventFilter(self)
		gridLayout.addWidget(labelSortorder, rowNumber, 0)
		gridLayout.addWidget(self.InputSortorder, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Ordre de tri: valeur obligatoire' )
		self.InputSortorder.textChanged.connect(self.onLineEditValuesChanged)	
		rowNumber = rowNumber + 1
							
			
		labelDatecreated = QLabel('Crée le')
		labelDatecreated.setToolTip( 'Date de création de la source' )
		labelDatecreated.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
		self.InputDatecreated = QDateTimeEdit()
		self.InputDatecreated.setObjectName( "SourceEditDatecreated" )
		self.InputDatecreated.installEventFilter(self)
		self.InputDatecreated.setMaximumWidth( self.getWidgetWidth( self.InputDatecreated, 16 )  )		
		gridLayout.addWidget(labelDatecreated, rowNumber, 0)
		gridLayout.addWidget(self.InputDatecreated, rowNumber, 1)
		mandatoryFlag = QLabel(' ')
		gridLayout.addWidget( mandatoryFlag, rowNumber, 2)
		mandatoryFlag.setToolTip( 'Crée le: valeur obligatoire' )
		self.InputDatecreated.dateTimeChanged.connect(self.onDateEditValuesChanged)	
		self.InputDatecreated.setDisplayFormat("dd/MM/yyyy hh:mm")
		self.InputDatecreated.setDateTimeRange(QDateTime(1, 1, 1, 0, 0), QDateTime(2999, 12, 31, 23, 59))
		self.InputDatecreated.setSpecialValueText(' ')
		self.InputDatecreated.setReadOnly(True)
		if True is False :
			self.InputDatecreated.setCalendarPopup(True)	


		self.setLayout(gridLayout)
		self.displayData( self.rowId, self.rowDataObj )
		

			

			

			

			

			

			

			

			

			

			

			

			

			

			


	

	

	

	

	

	

	def getfilesPicLocalurl( self ) :      

		from libwidget.photomanager import PhotoManager
		from lib.project import Project  
		currentProject = Project()			
		currentProject.load( AppSetting.get('app/lastProject', '') )
		myPhotoSelector = PhotoManager( rootMedia = currentProject.rootMedia )
		myPhotoSelector.loadDataList( self.InputLocalurl.text().split(',') )		
		result = myPhotoSelector.exec_()
		filenamelist = myPhotoSelector.getValue()
		if filenamelist is not None :
			self.printDebug("filenamelist = " , filenamelist)
			self.InputLocalurl.setText( ",".join(filenamelist) )

	

	

	

	

	

	

	

	


	

	

	

	

	def getfilesMediaIcon( self ) :      

		from libwidget.mediamanager import MediaManager
		from lib.project import Project  
		currentProject = Project()			
		currentProject.load( AppSetting.get('app/lastProject', '') )
		myMediaSelector = MediaManager( rootMedia = currentProject.rootMedia )
		myMediaSelector.loadDataList( self.InputIcon.text().split(',') )		
		result = myMediaSelector.exec_()
		filenamelist = myMediaSelector.getValue()
		if filenamelist is not None :
			self.printDebug("filenamelist = " , filenamelist)
			self.InputIcon.setText( ",".join(filenamelist) )

	

	

	

	def getfilesMediaSourceurl( self ) :      

		from libwidget.mediamanager import MediaManager
		from lib.project import Project  
		currentProject = Project()			
		currentProject.load( AppSetting.get('app/lastProject', '') )
		myMediaSelector = MediaManager( rootMedia = currentProject.rootMedia )
		myMediaSelector.loadDataList( self.InputSourceurl.text().split(',') )		
		result = myMediaSelector.exec_()
		filenamelist = myMediaSelector.getValue()
		if filenamelist is not None :
			self.printDebug("filenamelist = " , filenamelist)
			self.InputSourceurl.setText( ",".join(filenamelist) )

	

	

	

	

	

	

	



	

	

	

	

	

	

	

	

	

	

	

	

	

	



	def printDebug( self, *args ) :

		message=""
		for messageElem in args:
			message = message + str(messageElem)			

		import inspect
		callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
		frame = callerframerecord[0]
		AppLog.logContext = inspect.getframeinfo(frame)	

		AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)
	
	# Retourne la largeur necessaire pour un widget (QlineEdit ...) 
	# en fonctiion du nombre de caractères souhaité.
	def getWidgetWidth( self, widget, charmax ) :
		
		fm = widget.fontMetrics()
		try :
			m = widget.textMargins()
		except Exception as e:
			m = QMargins(8,8,8,8)
		c = widget.contentsMargins()
		w = charmax*fm.width('x')+m.left()+m.right()+c.left()+c.right()
		w = w + 8 # margins
		return w
			
	def onLineEditValuesChanged(self , text) :
		
		self.parent.onValuesChanged( True )

	def onTextEditValuesChanged(self):
		
		self.parent.onValuesChanged( True )
		
	def onComboValuesChanged(self, newIndex):
		
		self.parent.onValuesChanged( True )
				
	def onCheckBoxValuesChanged( self ,state) :
		self.parent.onValuesChanged( True )
			
	def onDateEditValuesChanged( self, newDate ) :
		
		self.parent.onValuesChanged( True )
				
	def displayData( self, rowId, dataObj = None) :

		self.rowId = rowId		

		self.parent.enterEditSignal.emit( self.rowId )
		
		if self.rowId == -1 :

			self.parent.mainWindowUserMessage.emit( 1 , "Formulaire vide (creation ou recréation)" )
		
		if dataObj is None :			
			sourceObj = self.mySources.select( self.rowId )
			if sourceObj is False :
				sourceObj = Source()			
		else :
			sourceObj = dataObj
						
		self.InputId.setText( str(sourceObj.getId()) )		

			
		defaultValue = ''
		self.InputName.setText( str( sourceObj.getName() if sourceObj.getName() is not None else defaultValue ) ) 

			
			
		self.InputDescription.setText( str( sourceObj.getDescription() if sourceObj.getDescription() is not None else '' ) ) 

			
			
		if sourceObj.getPrivate() is None or sourceObj.getPrivate() == 0 :
			self.InputPrivate.setCheckState( Qt.CheckState.Unchecked ) 			
		if sourceObj.getPrivate() == 1 :
			self.InputPrivate.setCheckState( Qt.CheckState.Checked ) 


			
			
		self.InputIcon.setText( str( sourceObj.getIcon() if sourceObj.getIcon() is not None else '' ) ) 

			
			
		currentKey = sourceObj.getQuality() if sourceObj.getQuality() is not None else '' 
		self.InputQuality.setCurrentIndex( self.InputQuality.findData(currentKey) ) 

			
			
		self.InputLocalurl.setText( str( sourceObj.getLocalurl() if sourceObj.getLocalurl() is not None else '' ) ) 

			
			
		self.InputSourceurl.setText( str( sourceObj.getSourceurl() if sourceObj.getSourceurl() is not None else '' ) ) 

			
			
		self.InputSourcetext.setText( str( sourceObj.getSourcetext() if sourceObj.getSourcetext() is not None else '' ) ) 

			
		defaultValue = ''
		self.InputPage.setText( str( sourceObj.getPage() if sourceObj.getPage() is not None else defaultValue ) ) 

			
		defaultValue = ''
		self.InputCitation.setText( str( sourceObj.getCitation() if sourceObj.getCitation() is not None else defaultValue ) ) 

			
			
		self.InputTextdraduction.setText( str( sourceObj.getTextdraduction() if sourceObj.getTextdraduction() is not None else '' ) ) 

			
			
		self.InputNotesearch.setText( str( sourceObj.getNotesearch() if sourceObj.getNotesearch() is not None else '' ) ) 

			
		defaultValue = ''
		self.InputSortorder.setText( str( sourceObj.getSortorder() if sourceObj.getSortorder() is not None else defaultValue ) ) 

									
			
		#myQdate = Util.getQDatetimeFromString( sourceObj.getDatecreated(), defaultValue = QDateTime.currentDateTime().toString('dd/MM/yyyy') )
		myQdate = Util.getQDatetimeFromString( sourceObj.getDatecreated() )
		self.InputDatecreated.setDateTime( myQdate ) 

		self.parent.onValuesChanged( False )


	def setFocusOnFirstWidget( self ) :

		from PyQt5.QtWidgets import  QLineEdit, QTextEdit, QDateEdit, QDateTimeEdit, QDateTimeEdit, QCheckBox, QComboBox,QDial, QDoubleSpinBox, QRadioButton, QSlider, QSpinBox, QTimeEdit, QFontComboBox
		for widget in self.children():
			if widget.isEnabled() :
				if 	isinstance(widget, QLineEdit) or \
					isinstance(widget, QTextEdit) or \
					isinstance(widget, QDateEdit) or \
					isinstance(widget, QDateTimeEdit) or \
					isinstance(widget, QCheckBox) or \
					isinstance(widget, QComboBox) or \
					isinstance(widget, QDial) or \
					isinstance(widget, QDoubleSpinBox) or \
					isinstance(widget, QRadioButton) or \
					isinstance(widget, QSlider) or \
					isinstance(widget, QSpinBox) or \
					isinstance(widget, QTimeEdit) or \
					isinstance(widget, QFontComboBox) \
					:
					#AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					AppLog.log( LOG_DEBUG , 'Set focus on : '+ widget.objectName() )
					widget.setFocus()					
					return

	def eventFilter(self, widgetObj, event):
		
		if event.type() == QtCore.QEvent.FocusIn:			
			self.focusWidget = widgetObj
	
		return super(SourceEditDisplayWidgets, self).eventFilter(widgetObj, event)
    
	def restoreFocusWidget( self ) :		
		#AppLog.log( LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		AppLog.log( LOG_DEBUG , LOG_DEBUG , 'Restore focus widget : '+ self.focusWidget.objectName() )
		self.focusWidget.setFocus()	
            

class SourceDataSources() :
	
	# Jointure sur table(s): 	
	dataColumns = [ 'id', 'Nom', 'Description', 'Privée', 'Icône', 'Qualité', 'Source image', 'Source url', 'Source texte', 'Page', 'Citation', 'Transcription', 'Notes de recherche', 'Ordre de tri', 'Crée le'  ]
	dataSqlFields = [ 'id', 'name', 'description', 'private', 'icon', 'quality', 'localurl', 'sourceurl', 'sourcetext', 'page', 'citation', 'textdraduction', 'notesearch', 'sortorder', 'datecreated'  ]	
	dataColumsType = ['INTEGER', 'TEXT', 'TEXT', 'BOOLEAN', 'TEXT', 'INTEGER', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'TEXT', 'INTEGER', 'DATETIME'  ]
	dataTooltips = [ 'Identifiant de la source', 'Nom de la source', 'Description de la source', 'Source de type privée', 'Icône, image d’illustration', 'Qualité, crédibilité de la source (0-3)', 'Images locales : png, jpg, et annotations', 'Url de la source (http, .pdf, .docx, .txt ...)', 'Texte de la source', 'Numéro de la page (ou vue) dans la source', 'Précision sur lʼemplacement dans la page (en bas, à gauche ...)', 'Traduction du texte', 'Notes de recherche', 'Ordre de tri', 'Date de création de la source'  ]
			
	def __init__(self, db ):
		self.db = db
		pass
		
	def selectAll( self , field_name = None, field_value = None ) :
		
		if field_name is None and field_value is None :
			return self.select()
		else :
			return self.selectById( field_name, field_value )
			
					
	def select( self ) :
		
		dataRows = []
		
		self.mySources = Sources( self.db )
		sourceListAll = self.mySources.selectAll()
		if sourceListAll :
			i = 0
			for source in sourceListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(source.getId()))
				dataRows.append( [ source.getId(),  source.getName(),  source.getDescription(),  source.getPrivate(),  source.getIcon(),  source.getQuality(),  source.getLocalurl(),  source.getSourceurl(),  source.getSourcetext(),  source.getPage(),  source.getCitation(),  source.getTextdraduction(),  source.getNotesearch(),  source.getSortorder(),  source.getDatecreated()  ] )
				i = i + 1
			
		return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )


	def selectById( self, where_clause, fields_values ) :
		
		dataRows = []
				
		self.mySources = Sources( self.db )
		sourceListAll = self.mySources.selectById( where_clause, fields_values )
		if sourceListAll : 
			i = 0
			for source in sourceListAll:
				#AppLog.log( LOG_DEBUG , 'Sql data #'+str(i)+' .id = ' + str(source.getId()))
				dataRows.append( [ source.getId(),  source.getName(),  source.getDescription(),  source.getPrivate(),  source.getIcon(),  source.getQuality(),  source.getLocalurl(),  source.getSourceurl(),  source.getSourcetext(),  source.getPage(),  source.getCitation(),  source.getTextdraduction(),  source.getNotesearch(),  source.getSortorder(),  source.getDatecreated()  ] )
				i = i + 1
				
			return ( dataRows, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
		return ( None, self.dataColumns, self.dataSqlFields, self.dataColumsType, self.dataTooltips )
		
	# Retourner le champ #x lors de la selection par pop-up (Fk)
	def getFkIndexLibBack( self ) :
		return 1
