#!/usr/bin/python
# -*- coding: utf-8 -*-

'''

		Media Manager : audio type

		07/07/2021 08:40

		lvardon@laposte.net - 2021

		Licence libre

'''

import os
from os.path import exists
import datetime

from PyQt5.QtGui import QImage, QPixmap
from PyQt5.QtCore import Qt, QSize

from lib.util import Util

class MediaTypeAudio( ):

	fileSize = None
	fileSizeH = None	
	fileModiftimeH = None	
	fileModiftimestamp = None	
	
	fileDuration = None
	fileDurationTime = None	
	fileMetaDate = None
	
	
	def __init__(self, mediaSrc, fileName, tempDir = "temp", iconDir="libwidget/mediastype/icons" ):
		self.mediaSrc = mediaSrc		
		self.fileName = fileName
		self.tempDir = tempDir
		self.iconDir = iconDir
		
	def getPixmap( self) :		
		
		fullFileTemp = os.path.join(self.iconDir, "audio.png")				
		
		self.initFileInfo()
		self.iniMediaInfo()
		
		return self.getPixmapFromFile( fullFileTemp )
		
	def getPixmapFromFile( self, filename ) :
		
		self.image_qt = QImage(filename )
		self.pixmap = QPixmap.fromImage(self.image_qt)
		return self.pixmap

	def getPixmapResised( self , width, height ) :
		
		newsize = QSize(width, height)
		
		self.pixmap = self.getPixmap()
		
		pixmap_resized = self.pixmap.scaled(
			newsize,
			Qt.KeepAspectRatio,
			Qt.SmoothTransformation
		)		
		return pixmap_resized
		

	def initFileInfo( self ) :
		utilObj = Util()
		self.fileSize, self.fileSizeH = utilObj.get_file_size( self.fileName )
		self.fileModiftimestamp, self.fileModiftimeH = utilObj.get_file_date( self.mediaSrc )
		
	def iniMediaInfo ( self) :
		
		# https://pypi.org/project/audio-metadata/
		import audio_metadata
		metadata = audio_metadata.load(self.fileName)
		self.fileDuration = round(metadata.streaminfo['duration'], 2)
		self.fileDurationTime = datetime.timedelta(seconds= round(metadata.streaminfo['duration'],0))
		try :
			self.fileMetaDate = metadata['date']
		except Exception as e:
			print("soundfile : err timedelta")
			self.fileMetaDate = ""
		
		

	def getMetaInformationList( self ):
		rowMetas = {	
					'type'		: { 'value':"Audio", 'lib':"Type", 'tooltip':"Type detecté (basé sur l'extention)" } ,					
					'name' 		: { 'value':os.path.basename(self.fileName), 'lib': "Nom", 'tooltip' : "Nom du fichier"  } , 
					'size' 		: { 'value':self.fileSizeH, 'lib': "Poids", 'tooltip':"Poids du fichier en octets"  } ,
					'fmodif' 	: { 'value':self.fileModiftimeH, 'lib':"Modifié le" , 'tooltip':"Date de denière modification du fichier" } , 
					'duration' 	: { 'value':self.fileDuration, 'lib':"Durée (s)", 'tooltip':"Durée de l'audio en secondes" } ,
					'durationHms': { 'value':self.fileDurationTime, 'lib':"Durée (h:m:s)", 'tooltip':"Durée de l'audio en heure:minutes:secondes" } ,
					'metaDate'	: { 'value':self.fileMetaDate, 'lib':"Date (tag)", 'tooltip':"Date du fichier (métadonnée)" } 

					}
					
		return rowMetas
