#!/usr/bin/python
# -*- coding: utf-8 -*-

'''

		Media Manager : pdf type

		07/07/2021 08:40

		lvardon@laposte.net - 2021

		Licence libre

'''

import os
from os.path import exists

from pdf2image import convert_from_path

from PyQt5.QtGui import QImage, QPixmap
from PyQt5.QtCore import Qt, QSize

from lib.util import Util

class MediaTypePdf( ):

	fileSize = None
	fileSizeH = None	
	fileModiftimeH = None	
	fileModiftimestamp = None
	pageCount = None
	
	fileTempPrefix = 'pdf-preview'

	def __init__(self, mediaSrc, fileName, tempDir = "temp", iconDir="libwidget/mediastype/icons" ):		
		self.mediaSrc = mediaSrc		
		self.fileName = fileName
		self.tempDir = tempDir
		self.iconDir = iconDir
		
	def getPixmap( self, pageNumber = 0) :		
		
		if exists(self.fileName) :		 

			# Store Pdf with convert_from_path function
			images = convert_from_path(self.fileName)
			 
			# Save page as image
			fileTemp =  self.fileTempPrefix + '-' + 'page-'+str(pageNumber) + '-'+ os.path.basename(self.fileName) + '.jpg'
			fullFileTemp = os.path.join(self.tempDir, fileTemp)

			images[pageNumber].save(fullFileTemp, 'JPEG')
			self.pageCount = len(images)

			self.initFileInfo()

			return self.getPixmapFromFile( fullFileTemp )

		else :
			fullFileTemp = os.path.join(self.iconDir, "missing.png")				
			return self.getPixmapFromFile( fullFileTemp )			
		
	def getPixmapFromFile( self, filename ) :
		
		self.image_qt = QImage(filename )
		self.pixmap = QPixmap.fromImage(self.image_qt)
		return self.pixmap

	def getPixmapResised( self , width, height, pageNumber=0 ) :
		
		newsize = QSize(width, height)
		
		self.pixmap = self.getPixmap(pageNumber)
		
		pixmap_resized = self.pixmap.scaled(
			newsize,
			Qt.KeepAspectRatio,
			Qt.SmoothTransformation
		)		
		return pixmap_resized

	def initFileInfo( self ) :
		utilObj = Util()
		self.fileSize, self.fileSizeH = utilObj.get_file_size( self.fileName )
		self.fileModiftimestamp, self.fileModiftimeH = utilObj.get_file_date( self.mediaSrc )
		
	def getMetaInformationList( self ):
		rowMetas = {	
					'type'		: { 'value':"Document", 'lib':"Type", 'tooltip':"Type detecté (basé sur l'extention)" } ,					
					'name' 		: { 'value':os.path.basename(self.fileName), 'lib': "Nom", 'tooltip' : "Nom du fichier"  } , 
					'size' 		: { 'value':self.fileSizeH, 'lib': "Poids", 'tooltip':"Poids du fichier en octets"  } ,
					'fmodif' 	: { 'value':self.fileModiftimeH, 'lib':"Modifié le" , 'tooltip':"Date de denière modification du fichier" } ,
					'pagecount' : { 'value':self.pageCount, 'lib':"Pages" , 'tooltip':"Nombre de pages du document" } 
					
					}
					
		return rowMetas		

