#!/usr/bin/python
# -*- coding: utf-8 -*-

'''

		Media Manager : photo type

		07/07/2021 08:40

		lvardon@laposte.net - 2021

		Licence libre

'''

import os
from os.path import exists

from PyQt5.QtGui import QImage, QPixmap
from PyQt5.QtCore import Qt, QSize
from PIL import Image

from lib.util import Util

class MediaTypePhoto( ):

	fileSize = None
	fileSizeH = None	
	fileModiftimeH = None
	fileModiftimestamp = None
	imgW = None
	imgH = None

	def __init__(self, mediaSrc, fileName, tempDir = "temp", iconDir="libwidget/mediastype/icons" ):		
		self.mediaSrc = mediaSrc
		self.fileName = fileName
		self.tempDir = tempDir
		self.iconDir = iconDir
		
		self.exifDateOriginal = self.getExifDateOriginal()
				
	def getPixmap( self) :
		
		if exists(self.fileName) :
			
			self.image_qt = QImage(self.fileName)
			self.pixmap = QPixmap.fromImage(self.image_qt)
			
			self.initFileInfo()
			self.iniMediaInfo()
			
			return self.pixmap
		else :
			fullFileTemp = os.path.join(self.iconDir, "missing.png")				
			return self.getPixmapFromFile( fullFileTemp )			

	def getPixmapFromFile( self, filename ) :
		
		self.image_qt = QImage(filename )
		self.pixmap = QPixmap.fromImage(self.image_qt)
		return self.pixmap

	def getPixmapResised( self , width, height ) :
		
		newsize = QSize(width, height)
		
		self.pixmap = self.getPixmap()
		
		pixmap_resized = self.pixmap.scaled(
			newsize,
			Qt.KeepAspectRatio,
			Qt.SmoothTransformation
		)		
		return pixmap_resized

	def initFileInfo( self ) :
		utilObj = Util()
		self.fileSize, self.fileSizeH = utilObj.get_file_size( self.fileName )
		self.fileModiftimestamp, self.fileModiftimeH = utilObj.get_file_date( self.mediaSrc )
		
		
	def iniMediaInfo ( self) :
		img = Image.open(self.fileName)													
		self.imgW = img.width
		self.imgH = img.height		

	def getExifDateOriginal( self ) :
		
		try :		
			self.exifData = Image.open(self.mediaSrc)._getexif()[36867]
		except Exception as e:
			self.exifData = None
			
		return self.exifData
		
	def getMetaInformationList( self ):
		
		rowMetas = {	
					'type'		: { 'value':"Photo", 'lib':"Type", 'tooltip':"Type detecté (basé sur l'extention)" } ,					
					'name' 		: { 'value':os.path.basename(self.fileName), 'lib': "Nom", 'tooltip' : "Nom du fichier"  } , 
					'size' 		: { 'value':self.fileSizeH, 'lib': "Poids", 'tooltip':"Poids du fichier en octets"  } ,
					'fmodif' 	: { 'value':self.fileModiftimeH, 'lib':"Modifié le" , 'tooltip':"Date de denière modification du fichier" } ,
					'width' 	: { 'value':self.imgW, 'lib':"Largeur (pixels)", 'tooltip':"Largeur du média en pixels" } ,
					'height' 	: { 'value':self.imgH, 'lib':"Hauteur (pixels)", 'tooltip':"Hauteur du média en pixels" } ,
					'exifdateoriginal'	: { 'value':self.exifDateOriginal, 'lib':"Date original", 'tooltip':"Date de prise de vue (Exif)" } ,
					
					}

		if self.exifDateOriginal is None :
			del rowMetas['exifdateoriginal']
					
		return rowMetas

			
