#!/usr/bin/python
# -*- coding: utf-8 -*-

'''

		Media Manager : video type

		07/07/2021 08:40

		lvardon@laposte.net - 2021

		Licence libre

'''

import os
from os.path import exists
import cv2
import datetime

from PyQt5.QtGui import QImage, QPixmap
from PyQt5.QtCore import Qt, QSize

from lib.util import Util

class MediaTypeVideo( ):

	fileSize = None
	fileSizeH = None	
	fileModiftimeH = None
	fileModiftimestamp = None
	imgW = None
	imgH = None	

	fileDuration = None
	fileDurationTime = None
	
	fileTempPrefix = 'video-preview'

	def __init__(self, mediaSrc, fileName, tempDir = "temp/", iconDir="libwidget/mediastype/icons" ):	
		self.mediaSrc = mediaSrc		
		self.fileName = fileName
		self.tempDir = tempDir
		self.iconDir = iconDir
		
	def getPixmap( self, frameNumber = 100 ) :		
		
		if exists(self.fileName) :
			
			# Opens the Video file
			cap= cv2.VideoCapture(self.fileName)
			i=0
			while(cap.isOpened() and i < frameNumber ):
				ret, frame = cap.read()
				if ret == False:
					break				
				i+=1

			fileTemp =  self.fileTempPrefix + '-' + 'frame-'+str(i) + '-'+ os.path.basename(self.fileName) + '.jpg'
			fullFileTemp = os.path.join(self.tempDir, fileTemp)
			cv2.imwrite(fullFileTemp,frame)

			self.initFileInfo()
			self.imgW = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))   # float `width`
			self.imgH = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))  # float `height`

			# count the number of frames
			frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
			fps = int(cap.get(cv2.CAP_PROP_FPS))
			  
			# calculate dusration of the video
			self.fileDuration = int(frames / fps)
			self.fileDurationTime = str(datetime.timedelta(seconds=self.fileDuration))			

			cap.release()
			cv2.destroyAllWindows()
			
			return self.getPixmapFromFile( fullFileTemp )
			
		else :
			fullFileTemp = os.path.join(self.iconDir, "missing.png")				
			return self.getPixmapFromFile( fullFileTemp )			
		
	def getPixmapFromFile( self, filename ) :
		
		self.image_qt = QImage(filename )
		self.pixmap = QPixmap.fromImage(self.image_qt)
		return self.pixmap
		
	def getPixmapResised( self , width, height, framenumber=100 ) :
		
		newsize = QSize(width, height)
		
		self.pixmap = self.getPixmap(framenumber)
		
		pixmap_resized = self.pixmap.scaled(
			newsize,
			Qt.KeepAspectRatio,
			Qt.SmoothTransformation
		)		
		return pixmap_resized		
		
	def initFileInfo( self ) :
		utilObj = Util()
		self.fileSize, self.fileSizeH = utilObj.get_file_size( self.fileName )
		self.fileModiftimestamp, self.fileModiftimeH = utilObj.get_file_date( self.mediaSrc )

		
	def getMetaInformationList( self ):
		rowMetas = {	
					'type'		: { 'value':"Vidéo", 'lib':"Type", 'tooltip':"Type detecté (basé sur l'extention)" } ,					
					'name' 		: { 'value':os.path.basename(self.fileName), 'lib': "Nom", 'tooltip' : "Nom du fichier"  } , 
					'size' 		: { 'value':self.fileSizeH, 'lib': "Poids", 'tooltip':"Poids du fichier en octets"  } ,
					'fmodif' 	: { 'value':self.fileModiftimeH, 'lib':"Modifié le" , 'tooltip':"Date de denière modification du fichier" } ,
					'width' 	: { 'value':self.imgW, 'lib':"Largeur (pixels)", 'tooltip':"Largeur du média en pixels" } ,
					'height' 	: { 'value':self.imgH, 'lib':"Hauteur (pixels)", 'tooltip':"Hauteur du média en pixels" } , 
					'duration' 	: { 'value':self.fileDuration, 'lib':"Durée (s)", 'tooltip':"Durée de la vidéo en secondes" } ,
					'durationHms': { 'value':self.fileDurationTime, 'lib':"Durée (h:m:s)", 'tooltip':"Durée de la vidéo en heure:minutes:secondes" } 
					}
					
		return rowMetas		

