#!/usr/bin/python
# -*- coding: utf-8 -*-

'''

		Photo Manager

		07/07/2021 08:40

		lvardon@laposte.net - 2021

		Licence libre

'''
import time,platform, os, sys
from pathlib import Path

from configparser import ConfigParser

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL

import lib.setting as AppSetting

from PyQt5 import QtWidgets
from PyQt5.Qt import QStandardItem
from PyQt5.QtWidgets import QMainWindow, QAction, QMenu, QApplication, QWidget, QToolTip, QPushButton, QMessageBox,QTableView, QToolBar, QFileDialog
from PyQt5.QtWidgets import QDialog, QColorDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, QHBoxLayout, QBoxLayout, QScrollArea, QGraphicsRectItem, QGraphicsScene, QGraphicsItem, QGraphicsLineItem
from PyQt5.QtWidgets import QLabel, QLineEdit, QTextEdit, QComboBox, QCheckBox, QRadioButton, QDateTimeEdit, QToolButton, QSplitter,QFrame
from PyQt5.QtGui import QCursor, QFont, QIcon, QPixmap, QBrush, QColor, QTransform, QPen, QPainterPath, QPainter
from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot, QSettings, QModelIndex, QDateTime, QRect, QPoint, QRectF, QPointF, QLineF

class GraphicsRectItem(QGraphicsRectItem):

	debugPrint = True

	handleTopLeft = 1
	handleTopMiddle = 2
	handleTopRight = 3
	handleMiddleLeft = 4
	handleMiddleRight = 5
	handleBottomLeft = 6
	handleBottomMiddle = 7
	handleBottomRight = 8

	itemId = None
	itemTitle = ""
	itemText = ""

	itemColor = QColor(255, 255, 0, 100)
	itemLineColor = QColor(255, 0, 0, 255)
	handleColor = QColor(0, 0, 0, 255)

	handleZoom = 1
	handleSize = +8.0*handleZoom
	handleSpace = -(handleSize / 2)
	
	handleCursors = {
		handleTopLeft: Qt.SizeFDiagCursor,
		handleTopMiddle: Qt.SizeVerCursor,
		handleTopRight: Qt.SizeBDiagCursor,
		handleMiddleLeft: Qt.SizeHorCursor,
		handleMiddleRight: Qt.SizeHorCursor,
		handleBottomLeft: Qt.SizeBDiagCursor,
		handleBottomMiddle: Qt.SizeVerCursor,
		handleBottomRight: Qt.SizeFDiagCursor,
	}

	def __init__(self, *args):
		"""
		Initialize the shape.
		"""
		super().__init__(*args)
		self.handles = {}
		self.handleSelected = None
		self.mousePressPos = None
		self.mousePressRect = None
		self.setAcceptHoverEvents(True)
		self.setFlag(QGraphicsItem.ItemIsMovable, True)
		self.setFlag(QGraphicsItem.ItemIsSelectable, True)
		self.setFlag(QGraphicsItem.ItemSendsGeometryChanges, True)
		self.setFlag(QGraphicsItem.ItemIsFocusable, True)
		self.updateHandlesPos()

	def printDebug( self, *args ) :

		if self.debugPrint :
			message=""
			for messageElem in args:
				message = message + str(messageElem)			

			import inspect
			callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
			frame = callerframerecord[0]
			AppLog.logContext = inspect.getframeinfo(frame)	

			AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)

	def setItemParams( self, itemParams ) :

		for key, value in itemParams.items():
			self.printDebug( 'key=', key, ' ::: ', value)
			if key == 'itemColor' : self.itemColor = QColor(value)
			if key == 'itemLineColor' : self.itemLineColor = QColor(value)
			if key == 'handleZoom' :
				self.handleZoom = value
				self.handleSize = +8.0*self.handleZoom
				self.handleSpace = -(self.handleSize / 2)
			if key == 'handleColor' : self.handleColor = QColor(value	)
			
		self.updateHandlesPos()

	def setItemVisible(self, visibleState ) :
		self.setVisible(visibleState)

	def handleAt(self, point):
		"""
		Returns the resize handle below the given point.
		"""
		for k, v, in self.handles.items():
			if v.contains(point):
				return k
		return None

	def hoverMoveEvent(self, moveEvent):
		"""
		Executed when the mouse moves over the shape (NOT PRESSED).
		"""
		if self.isSelected():
			handle = self.handleAt(moveEvent.pos())
			cursor = Qt.ArrowCursor if handle is None else self.handleCursors[handle]
			self.setCursor(cursor)
		super().hoverMoveEvent(moveEvent)

	def hoverLeaveEvent(self, moveEvent):
		"""
		Executed when the mouse leaves the shape (NOT PRESSED).
		"""
		self.setCursor(Qt.ArrowCursor)
		super().hoverLeaveEvent(moveEvent)

	def mousePressEvent(self, mouseEvent):
		"""
		Executed when the mouse is pressed on the item.
		"""
		self.printDebug("mousePressEvent")

		self.handleSelected = self.handleAt(mouseEvent.pos())
		if self.handleSelected:
			self.printDebug("handleSelected")
			self.mousePressPos = mouseEvent.pos()
			self.mousePressRect = self.boundingRect()

		super().mousePressEvent(mouseEvent)

	def mouseMoveEvent(self, mouseEvent):
		"""
		Executed when the mouse is being moved over the item while being pressed.
		"""
		self.printDebug("item scene pos : ", self.scenePos().x(), " ,", self.scenePos().y())

		if self.handleSelected is not None:
			self.interactiveResize(mouseEvent.pos())
		else:
			super().mouseMoveEvent(mouseEvent)

	def mouseReleaseEvent(self, mouseEvent):
		"""
		Executed when the mouse is released from the item.
		"""
		super().mouseReleaseEvent(mouseEvent)
		self.handleSelected = None
		self.mousePressPos = None
		self.mousePressRect = None
		self.update()

	def boundingRect(self):
		"""
		Returns the bounding rect of the shape (including the resize handles).
		"""
		o = self.handleSize + self.handleSpace
		return self.rect().adjusted(-o, -o, o, o)

	def updateHandlesPos(self):
		"""
		Update current resize handles according to the shape size and position.
		"""		
		self.handleSize = +8.0*self.handleZoom		
		self.printDebug("updateHandlesPos, self.handleZoom=",self.handleZoom)
		self.printDebug("updateHandlesPos, self.handleSize=",self.handleSize)
		
		s = self.handleSize
		b = self.boundingRect()
		self.handles[self.handleTopLeft] = QRectF(b.left(), b.top(), s, s)
		self.handles[self.handleTopMiddle] = QRectF(b.center().x() - s / 2, b.top(), s, s)
		self.handles[self.handleTopRight] = QRectF(b.right() - s, b.top(), s, s)
		self.handles[self.handleMiddleLeft] = QRectF(b.left(), b.center().y() - s / 2, s, s)
		self.handles[self.handleMiddleRight] = QRectF(b.right() - s, b.center().y() - s / 2, s, s)
		self.handles[self.handleBottomLeft] = QRectF(b.left(), b.bottom() - s, s, s)
		self.handles[self.handleBottomMiddle] = QRectF(b.center().x() - s / 2, b.bottom() - s, s, s)
		self.handles[self.handleBottomRight] = QRectF(b.right() - s, b.bottom() - s, s, s)

	def interactiveResize(self, mousePos):
		"""
		Perform shape interactive resize.
		"""
		offset = self.handleSize + self.handleSpace
		boundingRect = self.boundingRect()
		rect = self.rect()
		diff = QPointF(0, 0)

		self.prepareGeometryChange()

		if self.handleSelected == self.handleTopLeft:

			fromX = self.mousePressRect.left()
			fromY = self.mousePressRect.top()
			toX = fromX + mousePos.x() - self.mousePressPos.x()
			toY = fromY + mousePos.y() - self.mousePressPos.y()
			diff.setX(toX - fromX)
			diff.setY(toY - fromY)
			boundingRect.setLeft(toX)
			boundingRect.setTop(toY)
			rect.setLeft(boundingRect.left() + offset)
			rect.setTop(boundingRect.top() + offset)
			self.setRect(rect)

		elif self.handleSelected == self.handleTopMiddle:

			fromY = self.mousePressRect.top()
			toY = fromY + mousePos.y() - self.mousePressPos.y()
			diff.setY(toY - fromY)
			boundingRect.setTop(toY)
			rect.setTop(boundingRect.top() + offset)
			self.setRect(rect)

		elif self.handleSelected == self.handleTopRight:

			fromX = self.mousePressRect.right()
			fromY = self.mousePressRect.top()
			toX = fromX + mousePos.x() - self.mousePressPos.x()
			toY = fromY + mousePos.y() - self.mousePressPos.y()
			diff.setX(toX - fromX)
			diff.setY(toY - fromY)
			boundingRect.setRight(toX)
			boundingRect.setTop(toY)
			rect.setRight(boundingRect.right() - offset)
			rect.setTop(boundingRect.top() + offset)
			self.setRect(rect)

		elif self.handleSelected == self.handleMiddleLeft:

			fromX = self.mousePressRect.left()
			toX = fromX + mousePos.x() - self.mousePressPos.x()
			diff.setX(toX - fromX)
			boundingRect.setLeft(toX)
			rect.setLeft(boundingRect.left() + offset)
			self.setRect(rect)

		elif self.handleSelected == self.handleMiddleRight:
			fromX = self.mousePressRect.right()
			toX = fromX + mousePos.x() - self.mousePressPos.x()
			diff.setX(toX - fromX)
			boundingRect.setRight(toX)
			rect.setRight(boundingRect.right() - offset)
			self.setRect(rect)

		elif self.handleSelected == self.handleBottomLeft:

			fromX = self.mousePressRect.left()
			fromY = self.mousePressRect.bottom()
			toX = fromX + mousePos.x() - self.mousePressPos.x()
			toY = fromY + mousePos.y() - self.mousePressPos.y()
			diff.setX(toX - fromX)
			diff.setY(toY - fromY)
			boundingRect.setLeft(toX)
			boundingRect.setBottom(toY)
			rect.setLeft(boundingRect.left() + offset)
			rect.setBottom(boundingRect.bottom() - offset)
			self.setRect(rect)

		elif self.handleSelected == self.handleBottomMiddle:

			fromY = self.mousePressRect.bottom()
			toY = fromY + mousePos.y() - self.mousePressPos.y()
			diff.setY(toY - fromY)
			boundingRect.setBottom(toY)
			rect.setBottom(boundingRect.bottom() - offset)
			self.setRect(rect)

		elif self.handleSelected == self.handleBottomRight:

			fromX = self.mousePressRect.right()
			fromY = self.mousePressRect.bottom()
			toX = fromX + mousePos.x() - self.mousePressPos.x()
			toY = fromY + mousePos.y() - self.mousePressPos.y()
			diff.setX(toX - fromX)
			diff.setY(toY - fromY)
			boundingRect.setRight(toX)
			boundingRect.setBottom(toY)
			rect.setRight(boundingRect.right() - offset)
			rect.setBottom(boundingRect.bottom() - offset)
			self.setRect(rect)

		self.updateHandlesPos()

	def shape(self):
		"""
		Returns the shape of this item as a QPainterPath in local coordinates.
		"""
		path = QPainterPath()
		path.addRect(self.rect())
		if self.isSelected():
			for shape in self.handles.values():
				path.addEllipse(shape)
		return path


	def paint(self, painter, option, widget=None):
		"""
		Paint the node in the graphic view.
		"""
		painter.setBrush(QBrush(self.itemColor))
		painter.setPen(QPen(QColor(0, 0, 0), 1.0, Qt.SolidLine))
		painter.drawRect(self.rect())

		painter.setRenderHint(QPainter.Antialiasing)
		painter.setBrush(QBrush( self.itemLineColor ))
		painter.setPen(QPen( self.handleColor , 1.0, Qt.SolidLine, Qt.RoundCap, Qt.RoundJoin))

		for handle, rect in self.handles.items():
			if self.isSelected():
				if self.handleSelected is None or handle == self.handleSelected:
					painter.drawEllipse(rect)

class GraphicsScene(QGraphicsScene):

	drawMode = False
	debugPrint = True

	minimalItemWidth = 7
	minimalItemHeight = 7

	handleSizeFactor = 400

	currentItem = None
	sceneModified = False

	def __init__(self, parent=None):
		super(GraphicsScene, self).__init__(QRectF(-500, -500, 1000, 1000), parent)
		self._start = QPointF()
		self._current_rect_item = None
		self.parent = parent

	def printDebug( self, *args ) :

		if self.debugPrint :
			message=""
			for messageElem in args:
				message = message + str(messageElem)			

			import inspect
			callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
			frame = callerframerecord[0]
			AppLog.logContext = inspect.getframeinfo(frame)	

			AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)

	def isModified( self) :
		return self.sceneModified

	def setModified( self , modifiedState ) :
		self.sceneModified = modifiedState

	def getItemsText( self) :
		itemTexts = ""
		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pass
			else :
				if item.itemTitle is not None :
					itemTexts = itemTexts + "[" + str(item.itemTitle) + "]\n"
				else :
					itemTexts = itemTexts + "[]\n"

				if item.itemText is not None :
					itemTexts = itemTexts + "" + str(item.itemText) + "\n"
				else :
					itemTexts = itemTexts + "\n"

		return itemTexts

	def setItemVisible(self, item, visibleState ) :
		item.setVisible(visibleState)

	def setItemsVisible(self, visibleState ) :

		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pass
			else :
				item.setVisible(visibleState)

	def setSelected(self, index) :
		countIndex = 0
		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pass
			else :
				countIndex +=1
				if index == countIndex :
					item.setSelected(True)
					self.currentItem = item
					self.parent.onItemSelected( self.currentItem )
				else :
					item.setSelected(False)

	def setItemsDefaultParams( self, itemParams ) :
				
		self.itemDefaultParams = itemParams

		minPixmapSize = min([ self.pixmapBackground.boundingRect().width(), self.pixmapBackground.boundingRect().height()] )
		self.itemDefaultParams['handleZoom'] = minPixmapSize / self.handleSizeFactor
		
		for key, value in self.itemDefaultParams.items():
			self.printDebug( 'itemDefaultParams : ', key, ' -> ', value)


	'''
	Appliquer les paramètres à tous les items
	'''
	def setItemsParams( self, itemParams ) :

		self.sceneModified = True
		
		minPixmapSize = min([ self.pixmapBackground.boundingRect().width(), self.pixmapBackground.boundingRect().height()] )
		itemParams['handleZoom'] = minPixmapSize / self.handleSizeFactor
			
		for key, value in itemParams.items():
			self.printDebug( 'itemParams : ', key, ' => ', value)

		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pass
			else :
				item.setItemParams( itemParams )

		self.update()

	'''
	Appliquer les paramètres au seul item selectionné
	'''
	def setCurrentItemParams( self, itemParams ) :

		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pass
			else :
				if item.isSelected() is True :
					item.setItemParams( itemParams )
		self.update()

	def setBackgroundImage( self, itemBackgroundImage) :
		self.addItem( itemBackgroundImage )
		self.pixmapBackground = itemBackgroundImage

	def setDrawMode( self, drawMode ) :
		self.drawMode = drawMode

	def deleteShape(self) :

		self.sceneModified = True

		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pass # pixmap
			else :
				if item.isSelected() is True :
					self.removeItem(item)
		self.update()

	def getShapeList( self ) :

		shapeList = []
		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pass # pixmap
			else :
				shapeList.append(item)

		return shapeList

	def deleteAllShapes(self) :

		self.sceneModified = True

		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pass # pixmap
			else :
				self.removeItem(item)
		self.update()

	def getRect(self) :

		self.printDebug("dump shapes")
		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pixmap = item

		itemCount = 0
		for item in self.items():

			itemCount +=1

			if type(item) == QtWidgets.QGraphicsPixmapItem :
				continue

			if type(item) == QtWidgets.QGraphicsLineItem	:
				continue

			self.printDebug(itemCount, "  x=", item.x(), " y=", item.y(), ", w=", item.rect().width(), ", ", item.rect().height() )

			centerStart = self.pixmapBackground.mapToScene(self.pixmapBackground.boundingRect().center())
			centerEnd = item.mapToScene(item.boundingRect().center())
			self.printDebug('  delta centers x,y : ', centerEnd.x() - centerStart.x(), ', ', centerStart.y() - centerEnd.y() )

	def getNextShapeId(self) :

		nextShapeId=1
		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pass # pixmap
			else :
				nextShapeId +=1
		return nextShapeId


	def toFile(self, filename ) :

		self.sceneModified = False

		shapecount = 0
		config = ConfigParser()

		for item in self.items():
			if type(item) == QtWidgets.QGraphicsPixmapItem :
				pass # pixmap
			else :
				shapecount +=1
				# add a new section
				section = 'shape_'+str(shapecount)

				if config.has_section(section) == False :
					config.add_section( section )

				config.set(section, 'type', str(type(item)))
				config.set(section, 'id', str(item.itemId))
				config.set(section, 'title', str(item.itemTitle))
				config.set(section, 'text', str(item.itemText))

				centerPixmap = self.pixmapBackground.mapToScene(self.pixmapBackground.boundingRect().center())
				centerItem = item.mapToScene(item.boundingRect().center())
				self.printDebug('  delta centers x,y : ', centerItem.x() - centerPixmap.x(), ', ', centerPixmap.y() - centerItem.y() )

				config.set(section, 'px', str( centerPixmap.x() ))
				config.set(section, 'py', str( centerPixmap.y() ))
				config.set(section, 'ix', str( centerItem.x() ))
				config.set(section, 'iy', str( centerItem.y() ))
				config.set(section, 'dx', str( centerItem.x() - centerPixmap.x() ))
				config.set(section, 'dy', str( centerPixmap.y() - centerItem.y() ))
				config.set(section, 'w', str( item.rect().width() ))
				config.set(section, 'h', str( item.rect().height() ))

				config.set(section, 'itemColor', str( item.itemColor.name(QColor.HexArgb) ))
				config.set(section, 'itemLineColor', str( item.itemLineColor.name(QColor.HexArgb) ))
				config.set(section, 'handleZoom', str( item.handleZoom ))
				config.set(section, 'handleColor', str( item.handleColor.name(QColor.HexArgb) ))

				self.printDebug(" w=", item.rect().width(), ", ", item.rect().height() )

		self.printDebug( "Creation/maj de : ",filename, " shapecount=", shapecount)

		try :
			with open(filename, 'w') as configfile:
				config.write(configfile)
		except Exception as e:
			self.printDebug(e)
			QMessageBox.critical(self.parent, "Erreur", str(e))

	def fromFile(self, filename ) :

		self.sceneModified = False

		itemPos = QPointF()
		itemSize = QPointF()
		config = ConfigParser()
		config.read(filename)

		for each_section in config.sections():

			itemParams = {

				'itemColor' : '#5bf9ff00',
				'itemLineColor' : '#aeff1b00',
				'handleZoom' : 1,
				'handleColor' : '#ffef2929',
				'minimalItemWidth' : 7,
				'minimalItemHeight' : 7
			}
			itemId = None
			itemTitle = None
			itemText = None

			for (each_key, each_val) in config.items(each_section):
				self.printDebug ("k=", each_key, ", val=" , each_val.replace("\n",""))
				if each_key == "id" : itemId = int( each_val )
				if each_key == "title" : itemTitle = each_val
				if each_key == "text" : itemText = each_val
				if each_key == "dx" : itemPos.setX( float(each_val ))
				if each_key == "dy" : itemPos.setY( float(each_val ))
				if each_key == "w" : itemSize.setX( float(each_val ))
				if each_key == "h" : itemSize.setY( float(each_val ))
				if each_key == "itemcolor" : itemParams['itemColor'] = QColor(each_val)
				if each_key == "itemLineColor" : itemParams['itemLineColor'] = QColor(each_val)
				if each_key == "handleZoom" : itemParams['handleZoom'] = int(each_val)
				if each_key == "handleColor" : itemParams['handleColor'] = QColor(each_val)

			self.printDebug("Add shape. itemId=", itemId)

			minPixmapSize = min([ self.pixmapBackground.boundingRect().width(), self.pixmapBackground.boundingRect().height()] )
			itemParams['handleZoom'] = minPixmapSize / self.handleSizeFactor
			
			# Create pixmap
			r = QRectF(QPointF(0,0), itemSize)
			item = GraphicsRectItem( r )
			self.addItem(item)
			item.setItemParams( itemParams )
			item.itemId = itemId
			item.itemTitle = itemTitle
			item.itemText = itemText
			item.modifiedItem = False

			# Move pixmap
			pixmapCenterX = self.pixmapBackground.boundingRect().width() / 2
			pixmapCenterY = self.pixmapBackground.boundingRect().height() / 2
			itemWidth = itemSize.x()
			itemHeight = itemSize.y()
			item.setPos( (pixmapCenterX-itemWidth/2)+itemPos.x(), (pixmapCenterY-itemHeight/2)-itemPos.y() )

		self.update()

	def setCurrentItemTitle( self, itemTitle) :

		if self.currentItem :
			self.currentItem.itemTitle = itemTitle

	def setCurrentItemText( self, itemText) :

		if self.currentItem :
			self.currentItem.itemText = itemText

	def getCurrentItemTitle( self ) :
		return self.currentItem.itemTitle

	def getCurrentItemText( self) :
		return self.currentItem.itemText

	def mousePressEvent(self, event):

		if self.drawMode == False :
			return

		item = self.itemAt(event.scenePos(), QTransform())

		if type(item) == QtWidgets.QGraphicsPixmapItem  : # sur l'image de fond ?

			self._start = event.scenePos()
			r = QRectF(self._start, self._start)
			self.printDebug(" On create shape , scene x,y : ", self._start.x(), ", ", self._start.y())
			self._current_rect_item = GraphicsRectItem( r )
			self.addItem(self._current_rect_item)
			self._current_rect_item.setItemParams( self.itemDefaultParams )
			self._current_rect_item.updateHandlesPos()
		else :
			if item is not None :
				self._start = event.scenePos()
				self.printDebug("  mousePressEvent shape found, event pos  ", self._start.x(), ", ", self._start.y())
				self.currentItem = item
				self.parent.onItemSelected( self.currentItem )

		super(GraphicsScene, self).mousePressEvent(event)

	def mouseMoveEvent(self, event):

		if self._current_rect_item is not None:

			self.sceneModified = True

			#self.printDebug("  growing item, at creation time...")
			r = QRectF(self._start, event.scenePos()).normalized()
			self._current_rect_item.setRect(r)
			self._current_rect_item.updateHandlesPos()

		super(GraphicsScene, self).mouseMoveEvent(event)

	def mouseReleaseEvent(self, event):

		if self._current_rect_item is not None :
			self._current_rect_item.updateHandlesPos()
			self.printDebug("mouseReleaseEvent (on shape) ",self._current_rect_item.pos().x() )
			self.printDebug("mouseReleaseEvent : item w,h : " ,self. _current_rect_item.rect().width(), " , ", self._current_rect_item.rect().height())
			if self._current_rect_item.rect().width() < self.minimalItemWidth or self._current_rect_item.rect().height() < self.minimalItemHeight :
				self.printDebug("mouseReleaseEvent : remove item (item too small)" )

				self.removeItem(self._current_rect_item)
				#sip.delete(self._current_rect_item)
				self.update()
			else :
				self.printDebug("mouseReleaseEvent : item created." )
				self._current_rect_item.itemId = self.getNextShapeId()
				self.parent.onItemCreated()

		item = self.itemAt(event.scenePos(), QTransform())

		if type(item) == QtWidgets.QGraphicsPixmapItem  : # image de fond
			pass
		else :
			if item is not None :
				self.sceneModified = True
				itemPosScene = item.mapToScene(item.pos())
				self.printDebug(" mouseReleaseEvent (mapToScene) ",itemPosScene.x(), ", ", itemPosScene.y() )
				self.currentItem = item
				self.parent.onItemSelected( self.currentItem )

		self._current_rect_item = None

		super(GraphicsScene, self).mouseReleaseEvent(event)

class PhotoViewer(QtWidgets.QGraphicsView):

	photoClicked = pyqtSignal(QPoint)
	drawMode = False
	debugPrint = True

	def __init__(self, parent):
		super(PhotoViewer, self).__init__(parent)
		self._zoom = 0
		self._empty = True
		self.parent = parent

		self._scene = GraphicsScene(self)
		self._photo = QtWidgets.QGraphicsPixmapItem()

		self._scene.setBackgroundImage(self._photo)

		self.setScene(self._scene)
		self.setTransformationAnchor(QtWidgets.QGraphicsView.AnchorUnderMouse)
		self.setResizeAnchor(QtWidgets.QGraphicsView.AnchorUnderMouse)
		self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
		self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
		self.setBackgroundBrush(QBrush(QColor(100, 100, 100)))
		self.setFrameShape(QtWidgets.QFrame.NoFrame)

	def printDebug( self, *args ) :

		if self.debugPrint :
			message=""
			for messageElem in args:
				message = message + str(messageElem)			

			import inspect
			callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
			frame = callerframerecord[0]
			AppLog.logContext = inspect.getframeinfo(frame)	

			AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)

	def isModified( self ) :
		return self._scene.isModified()

	def setModified( self , modifiedState ) :
		self._scene.setModified( modifiedState )

	def getItemsText( self) :
		return self._scene.getItemsText()

	def centerOnItem( self, item) :
		self.centerOn( item )

	def fitInViewItem( self, item) :
		self.fitInView( item , Qt.KeepAspectRatio)

	def rotateView( self , angleDegree = None) :
		if angleDegree :
			self.rotate( angleDegree )
		else :
			self.rotate( 90 )

	def setItemVisible(self, item, visibleState ) :
		self._scene.setVisible(visibleState)

	def setItemsVisible(self, visibleState ) :
		self._scene.setItemsVisible(visibleState )

	def onItemSelected( self, currentItem )	:
		self.parent.onItemSelected( currentItem )

	def setCurrentItemTitle( self, itemTitle) :
		self._scene.setCurrentItemTitle( itemTitle )

	def setCurrentItemText( self, itemText) :
		self._scene.setCurrentItemText( itemText )

	def getCurrentItemTitle( self ) :
		return self._scene.getCurrentItemTitle()

	def getCurrentItemText( self) :
		return self._scene.getCurrentItemText()

	def onItemCreated( self )	 :
		self.parent.onItemCreated()

	def setSelected(self, index) :
		self._scene.setSelected(index)

	def setItemsDefaultParams( self, itemParams ) :
		self._scene.setItemsDefaultParams( itemParams )

	def setItemsParams( self, itemParams ) :
		for key, value in itemParams.items():
			self.printDebug( key, ' ::: ', value)
		self._scene.setItemParams( itemParams )

	def setCurrentItemParams( self, itemParams ) :
		for key, value in itemParams.items():
			self.printDebug( key, ' ::: ', value)
		self._scene.setCurrentItemParams( itemParams )

	def setViewParams( self, viewParams ) :
		self.viewParams = viewParams

	def showEvent( self, event) :
		self.printDebug('showEvent')

	def fromFile(self, filename ) :
		self._scene.fromFile( filename )

	def toFile(self, filename ) :
		self._scene.toFile( filename )

	def getRect( self ) :
		self._scene.getRect()

	def deleteShape( self ) :
		self._scene.deleteShape()

	def deleteAllShapes( self ) :
		self._scene.deleteAllShapes()

	def hasPhoto(self):
		return not self._empty

	def adjustToView(self) :

		self.printDebug("adjustToView")

		rect = QRectF(self._photo.pixmap().rect())
		self._scene.setSceneRect( 0,0 , rect.width(), rect.height() )
		self.fitInView( self._scene.itemsBoundingRect() ,Qt.KeepAspectRatio)

	def getShapeList( self ) :
		return self._scene.getShapeList()

	def setPhoto(self, pixmap=None):
		self._zoom = 0
		if pixmap and not pixmap.isNull():
			self._empty = False
			self.setDragMode(QtWidgets.QGraphicsView.ScrollHandDrag)
			self._photo.setPixmap(pixmap)
		else:
			self._empty = True
			self.setDragMode(QtWidgets.QGraphicsView.NoDrag)
			self._photo.setPixmap(QPixmap())

		QRectFrect = self._scene.itemsBoundingRect()
		self.printDebug("SetPhoto, sceneRect pixmap = ", pixmap.width(), ', ' , pixmap.height())
		self.printDebug("SetPhoto, sceneRect = ", QRectFrect.width(), ', ' , QRectFrect.height())
		self._scene.setSceneRect(QRectFrect)

		self.printDebug( "view : " , self.frameSize().width() )

	def deleteAllShapes(self) :
		self._scene.deleteAllShapes()

	def wheelEvent(self, event):
		if self.hasPhoto():

			if event.angleDelta().y() > 0:
				if self._zoom <= self.viewParams['zoomMax'] :
					factor = 1.25
					self._zoom += 1
					self.scale(factor, factor)
			else:
				if self._zoom >= self.viewParams['zoomMin'] :
					factor = 0.8
					self._zoom -= 1
					self.scale(factor, factor)

		self.printDebug("Zoom ", self._zoom)

	def getDrawMode(self):
		return self.drawMode


	def setDrawMode(self, drawModeState):
		if drawModeState is True :
			self.drawMode = True
			self.setDragMode(QtWidgets.QGraphicsView.NoDrag)

		else :
			self.drawMode = False
			self.setDragMode(QtWidgets.QGraphicsView.ScrollHandDrag)

		self._scene.setDrawMode( self.drawMode	)
		return self.drawMode

	def toggleDragMode(self):
		if self.drawMode is True :

			self.drawMode = False
			self.setDragMode(QtWidgets.QGraphicsView.ScrollHandDrag)

		else :
			self.drawMode = True
			self.setDragMode(QtWidgets.QGraphicsView.NoDrag)

		self._scene.setDrawMode( self.drawMode	)
		return self.drawMode

	def mousePressEvent(self, event):

		pos = self.mapToScene(event.pos())
		item = self._scene.itemAt(pos.x(), pos.y(), QTransform())

		if type(item) != QtWidgets.QGraphicsPixmapItem :
			if item is not None :
				self.printDebug(" item : ", item.itemTitle)
				self.itemUnderMouse = item

				coorItem = item.mapToScene(item.boundingRect().center())
				self.printDebug( " ccord : ",coorItem.x() , ", ", coorItem.y())
				self.printDebug( " wxh   : ",item.rect().width() , ", ", item.rect().height())

			else :
				self.printDebug(" item : aucun")
				self.itemUnderMouse = None
		else :
			self.itemUnderMouse = None

		super(PhotoViewer, self).mousePressEvent(event)

	def setZoomZero(self) :
		
		self.resetTransform();()
		self._zoom  = 0	
		self.printDebug("Zoom zero, zoom=", self._zoom)

	def setZoomFit(self) :
		self.fitInView(self._scene.sceneRect(), Qt.KeepAspectRatio)
		self.printDebug("Zoom fit, zoom=", self._zoom)

class PhotoInsertChoice(QDialog):

	insertMode = "APRES"
	debugPrint = True

	def __init__(self):

		QDialog.__init__(self)

		layout = QGridLayout()
		self.setLayout(layout)

		radiobutton = QRadioButton("Insérer avant")
		radiobutton.insertMode = "AVANT"
		radiobutton.toggled.connect(self.onClicked)
		layout.addWidget(radiobutton, 0, 0)

		radiobutton = QRadioButton("Remplacer")
		radiobutton.insertMode = "REMPLACE"
		radiobutton.toggled.connect(self.onClicked)
		layout.addWidget(radiobutton, 1, 0)

		radiobutton = QRadioButton("Insérer après")
		radiobutton.insertMode = "APRES"
		radiobutton.toggled.connect(self.onClicked)
		layout.addWidget(radiobutton, 2, 0)
		radiobutton.setChecked(True)

		# OK and Cancel buttons
		buttons = QDialogButtonBox(
			QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
			Qt.Horizontal, self)
		buttons.accepted.connect(self.onAccept)
		buttons.rejected.connect(self.onCancel)

		layout.addWidget(buttons)

		self.setWindowTitle("Selectionner où insérer ce média")

	def printDebug( self, *args ) :

		if self.debugPrint :
			message=""
			for messageElem in args:
				message = message + str(messageElem)			

			import inspect
			callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
			frame = callerframerecord[0]
			AppLog.logContext = inspect.getframeinfo(frame)	

			AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)

	def onAccept( self) :
		self.accept()

	def onCancel( self) :
		self.reject()

	def onClicked(self):
		radioButton = self.sender()
		if radioButton.isChecked():
			self.printDebug("InsertMode : %s" % (radioButton.insertMode))
			self.insertMode = radioButton.insertMode


class PhotoManager(QDialog):

	debugPrint = True

	iniConfig = "config/photomanager.ini"
	iconPath = "img/photomanager/"
	fileExplorer = "nemo"

	parent = None
	isModified = False

	Reset = False
	Accept = None
	currentItem = None
	itemsVisibleState = True

	filename = ""
	filenamelist = []
	filenamelistIndex = None

	itemParams = {

		'handleZoom' : 1,
		'handleColor' : '#ffef2929',

		'itemColor' : '#5bf9ff00',
		'itemLineColor' : '#aeff1b00',

		'minimalItemWidth' : 7,
		'minimalItemHeight' : 7
	}

	viewParams = {

		'zoomMin' : -16,
		'zoomMax' : 16
	}

	rootMedia = ""

	def __init__(self,  parent = None, filenamelist=None, rootMedia=None ):
		super(PhotoManager, self).__init__(parent)

		self.rootMedia = rootMedia
					
		self.parent = parent

		self.setupUi()

		self.viewer.setViewParams( self.viewParams )
		self.viewer.setItemsDefaultParams( self.itemParams )

		self.filenamelistIndex = 0
		self.filenamelist = filenamelist
		if filenamelist is not None :
			self.filename = filenamelist[self.filenamelistIndex]
		else :
			self.filename = None

		settings=QSettings(self.iniConfig, QSettings.IniFormat)
		
		if platform.system() == 'Darwin':       # macOS
			if settings.value('fileExplorerTux') is not None:			
				self.fileExplorer = settings.value('fileExplorerTux')				
		elif platform.system() == 'Windows':    # Windows
			if settings.value('fileExplorerWin') is not None:			
				self.fileExplorer = settings.value('fileExplorerWin')				
		else: # linux variants
			if settings.value('fileExplorerTux') is not None:			
				self.fileExplorer = settings.value('fileExplorerTux')						

		self.printDebug( "fileExplorer = " + str(self.fileExplorer))

	def setupUi( self) :

		layout = QVBoxLayout(self)

		toolBar = QToolBar()
		layout.addWidget(toolBar)

		self.buttonTofile = QToolButton()
		self.buttonTofile.setIcon(QIcon(QPixmap( self.iconPath + "annot-save.png")))
		self.buttonTofile.clicked.connect(self.onToFile)
		self.buttonTofile.setToolTip("Enregistrer les annotations")
		toolBar.addWidget(self.buttonTofile)

		self.buttonSelectFile = QToolButton()
		self.buttonSelectFile.clicked.connect(self.onSelectFile)
		self.buttonSelectFile.setIcon(QIcon(QPixmap(self.iconPath + "photo-add.png")))
		self.buttonSelectFile.setToolTip('Ajouter un média')
		toolBar.addWidget(self.buttonSelectFile)

		self.buttonRemovePage = QToolButton()
		self.buttonRemovePage.clicked.connect(self.onRemovePage)
		self.buttonRemovePage.setIcon(QIcon(QPixmap(self.iconPath + "photo-remove.png")))
		self.buttonRemovePage.setToolTip('Retirer ce média')
		toolBar.addWidget(self.buttonRemovePage)

		self.buttonPrev = QToolButton()
		self.buttonPrev.clicked.connect(self.onDocPrev)
		self.buttonPrev.setIcon(QIcon(QPixmap(self.iconPath + "photo-prev.png")))
		self.buttonPrev.setToolTip("Média précédent")
		toolBar.addWidget(self.buttonPrev)

		self.buttonNext = QToolButton()
		self.buttonNext.clicked.connect(self.onDocNext)
		self.buttonNext.setIcon(QIcon(QPixmap(self.iconPath + "photo-next.png")))
		self.buttonNext.setToolTip("Média suivant")
		toolBar.addWidget(self.buttonNext)

		self.buttonDrawTogle = QToolButton()
		self.buttonDrawTogle.setIcon(QIcon(QPixmap(self.iconPath + "annot-edit.png")))
		self.buttonDrawTogle.clicked.connect(self.onDrawTogle)
		self.buttonDrawTogle.setToolTip("Editer les annotations")
		toolBar.addWidget(self.buttonDrawTogle)

		self.buttonDel = QToolButton()
		self.buttonDel.setIcon(QIcon(QPixmap(self.iconPath + "annot-del.png")))
		self.buttonDel.clicked.connect(self.onDel)
		self.buttonDel.setToolTip("Supprimer cette annotation")
		toolBar.addWidget(self.buttonDel)

		self.buttonDelAll = QToolButton()
		self.buttonDelAll.setIcon(QIcon(QPixmap(self.iconPath + "annot-delall.png")))
		self.buttonDelAll.clicked.connect(self.onDelAll)
		self.buttonDelAll.setToolTip("Supprimer <b>toutes</b> les annotations")
		toolBar.addWidget(self.buttonDelAll)

		self.buttonItemColor = QToolButton()
		self.buttonItemColor.setIcon(QIcon(QPixmap(self.iconPath + "annot-color.png")))
		self.buttonItemColor.clicked.connect(self.onItemColor)
		self.buttonItemColor.setToolTip("Appliquer ou choisir une couleur d'annotation")
		toolBar.addWidget(self.buttonItemColor)

		self.photoName = QLineEdit()
		self.photoName.textChanged.connect(self.onPhotoNameEditChanged)
		self.photoNameValid = QToolButton()
		self.photoNameValid.setIcon(QIcon(QPixmap(self.iconPath + "photo-rename.png")))
		self.photoNameValid.clicked.connect(self.onPhotoNameEditValid)
		self.photoNameValid.setToolTip("Valider le changement de nom du média")
		toolBar.addWidget(self.photoName)
		toolBar.addWidget(self.photoNameValid)

		self.shapeCombo = QComboBox()
		self.shapeCombo.currentIndexChanged.connect(self.onShapeChange)
		toolBar.addWidget(self.shapeCombo)

		self.setGeometry(80, 300, 800, 640)

		self.viewer = PhotoViewer(self)
		self.shapeTitle = QLineEdit()
		self.shapeText = QTextEdit()

		self.rightSplitter = QSplitter(Qt.Vertical)
		self.rightSplitter.addWidget(self.shapeTitle)
		self.rightSplitter.addWidget(self.shapeText)
		self.mainSplitter = QSplitter(Qt.Horizontal)
		self.mainSplitter.addWidget(self.viewer)
		self.mainSplitter.addWidget(self.rightSplitter)

		layout.addWidget(self.mainSplitter)

		self.shapeTitle.textChanged.connect(self.onShapeTitleChanged)
		self.shapeText.textChanged.connect(self.onShapeTextChanged)

		# OK and Cancel buttons
		buttons = QDialogButtonBox(
			QDialogButtonBox.Reset | QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
			Qt.Horizontal, self)
		buttons.accepted.connect(self.onAccept)
		buttons.rejected.connect(self.onReject)
		buttons.button(QDialogButtonBox.Reset).clicked.connect(self.onReset)
		buttons.button(QDialogButtonBox.Ok).setToolTip('Valider')
		buttons.button(QDialogButtonBox.Cancel).setToolTip('Annuler')
		buttons.button(QDialogButtonBox.Reset).setToolTip('Effacer la valeur')

		layout.addWidget(buttons)

		self.desktop = QApplication.desktop()
		self.screenRect = self.desktop.screenGeometry()
		self.screenHeight = self.screenRect.height()
		self.screenWidth = self.screenRect.width()
		self.printDebug("screen size: ", self.screenWidth, " x ", self.screenHeight)

	def printDebug( self, *args ) :

		if self.debugPrint :
			message=""
			for messageElem in args:
				message = message + str(messageElem)			

			import inspect
			callerframerecord = inspect.stack()[1]    # 0 represents this line, # 1 represents line at caller
			frame = callerframerecord[0]
			AppLog.logContext = inspect.getframeinfo(frame)	

			AppLog.log( LOG_DEBUG, self.__class__.__name__, ' : ' , message)

	# UI simplifiée 
	def disableButtons( self ) :
		
		self.printDebug("Hide UI buttons")
		self.buttonNext.setEnabled( False )
		self.buttonPrev.setEnabled( False )
		self.buttonRemovePage.setEnabled( False )
		self.buttonSelectFile.setEnabled( False )
		self.photoNameValid.setEnabled( False )
		
	def showEvent( self, event) :

		self.printDebug('showEvent')
		self.mainSplitter.setSizes([100,5])
		self.rightSplitter.setSizes([1,75])

		self.loadWindowState()

		if self.filename is not None :
			self.loadData( self.filename )

		self.viewer.adjustToView()

		self.setWindowTitle("Editeur de photo")

	def contextMenuEvent(self, event):

		if self.viewer.getDrawMode() is False :

			cmenu = QMenu(self)

			modeAct = cmenu.addAction("Mode édition")
			modeAct.triggered.connect(self.onDrawTogle)

			cmenu.addSeparator()

			showHide = cmenu.addAction("Cacher, afficher les annotations")
			showHide.triggered.connect(self.onItemsVisible)
			showMaximizedAct = cmenu.addAction("Maximiser")
			showMaximizedAct.triggered.connect(self.onShowMaximized)
			rotateView = cmenu.addAction("Rotation")
			rotateView.triggered.connect(self.onRotateView)

			zoomZeroAct = cmenu.addAction("Zoom zero")  
			zoomZeroAct.triggered.connect(self.onSetZoomZero)

			zoomFitAct = cmenu.addAction("Zoom idéal")  
			zoomFitAct.triggered.connect(self.onSetZoomFit)
			
			cmenu.addSeparator()

			storeWindowStateAct = cmenu.addAction("Enregistrer la disposition")  # zoom, rotate, splitter
			storeWindowStateAct.triggered.connect(self.onStoreWindowState)

			cmenu.addSeparator()

			photoInfoAct = cmenu.addAction("Informations sur le média")
			photoInfoAct.triggered.connect(self.onShowPhotoInfo)

			opendirFileAct = cmenu.addAction("Ouvrir le dossier du média ...")
			opendirFileAct.triggered.connect(self.onOpendirFile)

			startFileAct = cmenu.addAction("Ouvrir le média ...")
			startFileAct.triggered.connect(self.onStartFile)

			cmenu.addSeparator()

			copyItemsAct = cmenu.addAction("Copier les annotations")
			copyItemsAct.triggered.connect(self.onCopyItems)

			copyPhotoNameAct = cmenu.addAction("Copier le nom du média")
			copyPhotoNameAct.triggered.connect(self.onCopyPhotoName)

			currentItemShpAct = cmenu.addAction("Ouvrir le fichier .shp" )
			currentItemShpAct.triggered.connect(lambda checked : self.onCurrentItemOpenShp() )

			if self.viewer.itemUnderMouse is not None :

				from os.path import splitext
				outFile, outFileExt = splitext(self.filename)
				if outFile[-1] == '/' :
					outFile = outFile[:-1]

				if self.viewer.itemUnderMouse.itemTitle is None :
					itemName = str(self.viewer.itemUnderMouse.itemId)
				else :
					itemName = str(self.viewer.itemUnderMouse.itemTitle)

				propFileName = os.path.join( outFile + "-" + itemName.replace(" ", "_") + outFileExt )
				currentItemAct = cmenu.addAction("Sauver l'annotation sous ..." )
				currentItemAct.triggered.connect(lambda checked, item=self.viewer.itemUnderMouse, propFileName=propFileName : self.onCurrentItemSave(item, propFileName) )

			action = cmenu.exec_(self.mapToGlobal(event.pos()))

	def onRemovePage( self) :

		self.printDebug("remove filenamelistIndex ", self.filenamelistIndex)
		self.printDebug("len(self.filenamelist) ", len(self.filenamelist))

		if len(self.filenamelist) > 0 :

			if self.onToFileCheck() is False :
				return

			ret = QMessageBox.question(self, 'Confirmez', "Confirmez vous la suppression de cette page ?", QMessageBox.Yes | QMessageBox.Cancel, QMessageBox.Cancel)

			if ret == QMessageBox.Yes:

				del self.filenamelist[self.filenamelistIndex]

				if self.filenamelistIndex + 1 > len(self.filenamelist) :

					if len(self.filenamelist) > 0 :
						self.filenamelistIndex = len(self.filenamelist) -1
						self.filename = self.filenamelist[self.filenamelistIndex]
					else  :
						self.filenamelistIndex = 0
						self.filename = None
				else :
					self.filename = self.filenamelist[self.filenamelistIndex]
					self.photoName.setText( "" )

				self.printDebug("self.filenamelistIndex = ", self.filenamelistIndex)

				self.loadData( self.filename )

	def onDocPrev( self) :

		if self.filenamelist is None :
			return

		if self.onToFileCheck() is True :

			if self.filenamelistIndex > 0 :
				self.filenamelistIndex -=1
				self.filename = self.filenamelist[self.filenamelistIndex]
				self.loadData( self.filename )

	def onDocNext( self) :

		if self.filenamelist is None :
			return

		if self.onToFileCheck() is True :

			if self.filenamelistIndex < len(self.filenamelist)-1 :
				self.filenamelistIndex +=1
				self.filename = self.filenamelist[self.filenamelistIndex]
				self.loadData( self.filename )

	def onItemCreated( self )	 :
		self.printDebug( "onItemCreated" )
		self.shapeComboLoad()

	def onRotateView (self) :
		self.viewer.rotateView( 90 )

	def onShowMaximized( self ) :
		self.printDebug( "onShowMaximized")
		self.setGeometry(0, 0, self.screenWidth, self.screenHeight)

	def onShapeChange(self, index ) :
		self.printDebug( "shapeComboChange ", index)
		#for count in range(self.shapeCombo.count()):
		#	self.printDebug( self.shapeCombo.itemText(count) )
		self.printDebug ("Current index ",index," selection changed ",self.shapeCombo.currentText() )

		self.viewer.setSelected(index)

	def onItemColor( self) :
		color = QColorDialog.getColor(options=QColorDialog.ShowAlphaChannel)

		if color.isValid():
			self.printDebug("current item color change : ", color.name(QColor.HexArgb) )
			self.itemParams['itemColor'] = color.name(QColor.HexArgb)
			self.viewer.setCurrentItemParams( self.itemParams )
			self.shapeComboLoad()
		else :
			self.printDebug("Color not valid")

	def onFromFile(self) :
		self.viewer.fromFile( str(self.filename) + ".shp"  )

	def onToFile(self) :
		self.viewer.toFile(str(self.filename) + ".shp"  )
		self.shapeComboLoad()
		self.setModified(False)

	def onToFileCheck( self ) :

		if self.isModified is True :

			self.onPhotoNameEditValid()

			ret = QMessageBox.question(self, 'Confirmez', "Sauvegarder avant de quitter ?", QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.Cancel)

			if ret == QMessageBox.Yes:
				self.printDebug('Button QMessageBox.Yes clicked.')
				self.onToFile()
				return True

			if ret == QMessageBox.No:
				return True

			if ret == QMessageBox.Cancel:
				return False

		return True

	def setModified (self, modifiedState) :
		self.isModified  = modifiedState
		self.buttonTofile.setEnabled( self.isModified )

	def onGetRect( self) :
		self.viewer.getRect()

	def onItemsVisible(self ) :

		if self.itemsVisibleState :
			self.itemsVisibleState = False
		else :
			self.itemsVisibleState = True

		self.viewer.setItemsVisible( self.itemsVisibleState )

	def onDel(self) :
		self.printDebug("delete shape")

		ret = QMessageBox.question(self, 'Confirmez', "Confirmez vous la suppression de l'annotation ?", QMessageBox.Yes | QMessageBox.Cancel, QMessageBox.Cancel)

		if ret == QMessageBox.Yes:
			self.printDebug('Button QMessageBox.Yes clicked.')
			self.viewer.deleteShape()
			self.shapeComboLoad()

	def onDelAll(self) :
		self.printDebug("delete all shapes")

		ret = QMessageBox.question(self, 'Confirmez', "Confirmez vous la suppression de <b>toutes</b> les annotations ?", QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel, QMessageBox.No)

		if ret == QMessageBox.Yes:
			self.printDebug('Button QMessageBox.Yes clicked.')
			self.viewer.deleteAllShapes()
			self.shapeComboLoad()

	def onDrawTogle(self) :

		self.printDebug("onDrawTogle")
		drawMode = self.viewer.toggleDragMode()

		self.setModified(True)

		if drawMode :
			self.uiEnable( True )
			self.setShapeTexts( self.currentItem )
			self.viewer.setItemsVisible( True )

			self.buttonDrawTogle.setIcon(QIcon(QPixmap(self.iconPath + "annot-select.png")))
			self.buttonDrawTogle.setToolTip("Basculer en mode <b>consultation</b>")

		else :
			self.uiEnable( False )
			self.setShapeTexts( self.currentItem )

			self.buttonDrawTogle.setIcon(QIcon(QPixmap(self.iconPath + "annot-edit.png")))
			self.buttonDrawTogle.setToolTip("Basculer en mode <b>édition</b>")

	def onSelectFile(self) :

		if self.onToFileCheck() is False :
			return

		defaultDir = self.rootMedia

		filedialog = QFileDialog(self,
									caption = "Selectionner le nom du média à ouvrir",
									directory = defaultDir
									)

		filedialog.setDefaultSuffix("png")
		filedialog.setNameFilter("Images (*.png *.tiff *.jpg);;Tous (*.*)")
		selected = filedialog.exec()
		if selected:

			if self.filenamelist[0] !="" and self.filename != "" and len(self.filenamelist) > 0 :

				photoInsertChoiceDlg = PhotoInsertChoice()
				photoInsertChoiceDlg.exec()

				self.printDebug( "Insert mode : ", photoInsertChoiceDlg.insertMode )

				if photoInsertChoiceDlg.insertMode == "AVANT" :
					self.printDebug( "Insert AVANT , pos : " , self.filenamelistIndex )
					self.filenamelist.insert(self.filenamelistIndex, filedialog.selectedFiles()[0])
					self.filename  = filedialog.selectedFiles()[0]

				if photoInsertChoiceDlg.insertMode == "REMPLACE" :
					self.printDebug( "Insert REPLACE , pos : " , self.filenamelistIndex )
					self.filenamelist[self.filenamelistIndex] = filedialog.selectedFiles()[0]
					self.filename  = filedialog.selectedFiles()[0]

				if photoInsertChoiceDlg.insertMode == "APRES" :
					self.filenamelistIndex +=1
					self.printDebug( "Insert APRES , pos : " , self.filenamelistIndex )
					self.filenamelist.insert(self.filenamelistIndex, filedialog.selectedFiles()[0])
					self.filename  = filedialog.selectedFiles()[0]
			else :
				self.filenamelistIndex = 0
				self.filenamelist = [""]
				self.printDebug( "Insert REPLACE , pos : " , self.filenamelistIndex )
				self.filenamelist[self.filenamelistIndex] = filedialog.selectedFiles()[0]
				self.filename  = filedialog.selectedFiles()[0]

			self.loadData( self.filename )

		else:
			return

	def onAccept(self) :

		if self.onToFileCheck() is False :
			return

		self.printDebug("onAccept")
		self.Accept = True
		self.accept()

	def onReject(self) :

		if self.onToFileCheck() is False :
			return

		self.printDebug("onReject")
		self.Accept = False
		self.reject()

	def onReset( self ) :

		if self.onToFileCheck() is False :
			return

		self.Reset = True
		self.accept()

	def loadDataList( self, filenamelist ) :

		self.filenamelistIndex = 0

		self.filenamelist = filenamelist

		# Add rootMedia to each photo
		if filenamelist is not None and self.rootMedia != '' :
			index=0
			for fileName in filenamelist :
				
				if len(fileName) >=1 and fileName[0] == "/" : 
					fileName = fileName[1:]
				
				if fileName.find(self.rootMedia) == -1 :
					self.printDebug("Add rootmedia in file name : ", fileName)
					self.printDebug("Before : ", filenamelist[index])
					filenamelist[index] = os.path.join(self.rootMedia, fileName)
					self.printDebug("After : ", filenamelist[index])
				if fileName.find(self.rootMedia) == 0 :
					self.printDebug("rootmedia found @0 : ", fileName)
				if fileName.find(self.rootMedia) > 0 :
					self.printDebug("rootmedia >0 : ", fileName)
				index=index+1
		else :
			self.printDebug("filenamelist is null (loadDataList)")

		self.printDebug("Load photo : ", filenamelist[self.filenamelistIndex])
		if not os.path.isfile(filenamelist[self.filenamelistIndex]) :
			self.printDebug("ERR : cant access photo.")

		self.loadData( filenamelist[self.filenamelistIndex] )


	def loadData( self, filename ) :

		self.filename = filename

		self.printDebug("loadData, image :" , self.filename )

		self.setWindowTitle(self.filename)

		if self.filename is not None :
			self.photoName.setText( os.path.basename(self.filename) )
		else :
			self.photoName.setText( "" )

		self.viewer.deleteAllShapes()

		pixmap = QPixmap(self.filename)
		self.viewer.setPhoto(pixmap)
		pixmapHeight = pixmap.height()
		pixmapWidth = pixmap.width()
		
		self.viewer.setItemsDefaultParams( self.itemParams )

		self.viewer.fromFile( str(self.filename) + ".shp" )

		self.currentItem = None

		drawModeState = False
		self.viewer.setDrawMode(drawModeState)

		self.shapeComboLoad()
		self.setShapeTexts( None )
		self.uiEnable( False )

		self.itemsVisibleState = True
		self.viewer.setItemsVisible( self.itemsVisibleState )

		self.viewer.setModified( False )

		self.viewer.adjustToView()

		self.setModified(False)

		self.photoName.setStyleSheet("color: black;")

	def shapeComboLoad( self ) :

		self.shapeCombo.clear()
		self.shapeCombo.addItem("[Annotation]")

		self.shapeList = self.viewer.getShapeList()
		model = self.shapeCombo.model()
		indexShape=0
		for shapeItem in self.shapeList :
			entry = QStandardItem('#' + str(shapeItem.itemId) + ": " + str(shapeItem.itemTitle))
			entry.setForeground(QColor('black'))
			entry.setBackground(QColor( shapeItem.itemColor))
			model.appendRow(entry)
			indexShape +=1

	def onShapeTitleChanged(self):

		self.printDebug('onShapeTitleChanged ', self.shapeTitle.text().replace("\n",""))
		self.viewer.setCurrentItemTitle( self.shapeTitle.text() )

	def onShapeTextChanged(self):

		self.printDebug('onShapeTextChanged ', self.shapeText.toPlainText().replace("\n",""))
		self.viewer.setCurrentItemText( self.shapeText.toPlainText() )

	def onItemSelected( self, currentItem )	:
		self.printDebug('onItemSelected id is : ', currentItem.itemId)
		indexItemSelected = 0
		for shapeItem in self.shapeList :
			indexItemSelected +=1
			if 	currentItem.itemId == shapeItem.itemId :
					break

		self.printDebug('onItemSelected, combo index set : ', indexItemSelected)

		self.shapeCombo.currentIndexChanged.disconnect()
		self.shapeCombo.setCurrentIndex( indexItemSelected )
		self.shapeCombo.currentIndexChanged.connect(self.onShapeChange)

		self.setShapeTexts( currentItem )

		self.currentItem = currentItem

		if self.viewer.getDrawMode() is False :
			self.viewer.centerOnItem( self.currentItem )

	def setShapeTexts( self, currentItem) :

		if currentItem is not None :
			self.shapeTitle.setText( currentItem.itemTitle )
			self.shapeTitle.setStyleSheet('color: black; background-color: ' + currentItem.itemColor.name(QColor.HexArgb)  )
			self.shapeText.setText( currentItem.itemText)
		else :
			self.shapeTitle.setText( "" )
			self.shapeTitle.setStyleSheet('color: black; background-color: white' )
			self.shapeText.setText( "")

	def uiEnable( self, uiEnableMode ) :

		self.shapeTitle.setEnabled(uiEnableMode)
		self.shapeTitle.setStyleSheet('color: black' )
		self.shapeText.setEnabled(uiEnableMode)
		self.shapeText.setStyleSheet('color: black' )
		self.buttonDel.setEnabled(uiEnableMode)
		self.buttonDelAll.setEnabled(uiEnableMode)
		self.buttonItemColor.setEnabled(uiEnableMode)

	def onStoreWindowState(self) :

		settings=QSettings(self.iniConfig, QSettings.IniFormat)

		settings.setValue("geometry", self.saveGeometry())

		mainSplitterSetting = self.mainSplitter.saveState()
		if mainSplitterSetting:
			settings.setValue("mainSplitterSetting", self.mainSplitter.saveState())

		rightSplitterSetting = self.rightSplitter.saveState()
		if rightSplitterSetting:
			settings.setValue("rightSplitterSetting", self.rightSplitter.saveState())

	def loadWindowState(self) :

		settings=QSettings(self.iniConfig, QSettings.IniFormat)

		if settings.value("geometry") is not None:
			self.restoreGeometry(settings.value("geometry", ""))

		try :
			rightSplitterSetting=settings.value("rightSplitterSetting")
			self.rightSplitter.restoreState(rightSplitterSetting)
		except Exception as e:
			self.printDebug(str(e))

		try :
			mainSplitterSetting=settings.value("mainSplitterSetting") #.toByteArray()
			self.mainSplitter.restoreState(mainSplitterSetting)
		except Exception as e:
			self.printDebug(str(e))

		font = QFont()
		fontSize=8
		if settings.value("shapeTitleFontSize") is not None:
			fontSize = int(settings.value("shapeTitleFontSize", "12"))
		font.setPointSize(fontSize)
		self.shapeTitle.setFont(font)

		fontSize=8
		if settings.value("shapeTextFontSize") is not None:
			fontSize = int(settings.value("shapeTextFontSize", "12"))
		font.setPointSize(fontSize)
		self.shapeText.setFont(font)

	def onShowPhotoInfo( self ):

		self.printDebug('showPhotoInfo')

		msg = QMessageBox()
		msg.setIcon(QMessageBox.Information)

		pixmap = QPixmap(self.filename)
		pixmapHeight = pixmap.height()
		pixmapWidth = pixmap.width()
		self.printDebug("pixmap WxH = ", pixmapWidth, " x ", pixmapHeight)

		msg.setWindowTitle("Informations sur le média")
		msg.setText(
					"Fichier : <b>"+str(self.filename)+"</b>" + "<br>" +
					"Modifié : <b>" + str(self.viewer.isModified()) + "</b>"
					)
		msg.setInformativeText(
							"Largeur et hauteur (pixels) : <b>"+str(pixmapWidth)+" x "+str(pixmapHeight) + "</b>" + "<br>" +
							"Dossier : <b>" + str(os.path.dirname(self.filename) ) + "</b>"
							)
		filemtime = time.strftime("%d/%m/%Y %H:%M",time.localtime(os.path.getmtime(self.filename)))
		filesize = self.sizeof_fmt( os.path.getsize(self.filename) )

		msg.setDetailedText(
								"Dernière modification : " + str(filemtime) + "\n" +
								"Taille : " + str(filesize)
							)

		msg.setStandardButtons(QMessageBox.Ok)

		retval = msg.exec_()

	def onSetZoomZero(self) :
		self.viewer.setZoomZero()

	def onSetZoomFit(self) :
		self.viewer.setZoomFit()
				
	def onStartFile( self ):
		
		p = str(Path(self.filename).resolve())

		AppLog.log( LOG_INFO , "Ouverture du fichier : " + p)					
		
		import subprocess, platform
		if platform.system() == 'Darwin':       # macOS
			AppLog.log( LOG_INFO , "Ouverture ...")									
			subprocess.call(('open', p ))			
		elif platform.system() == 'Windows':    # Windows
			AppLog.log( LOG_INFO , "Ouverture ...")									
			os.startfile( p )
		else:                                   # linux variants
			AppLog.log( LOG_INFO , "Ouverture ...")									
			subprocess.call(('xdg-open', p ))

	def onOpendirFile( self ):

		p = str(Path(self.filename).resolve())
		
		if platform.system() == 'Darwin':       # macOS
			AppLog.log( LOG_INFO , "Ouverture dossier ... ", p)					
			os.system( self.fileExplorer + ' "' + p + '"')
		elif platform.system() == 'Windows':    # Windows
			p = os.path.dirname(p)
			AppLog.log( LOG_INFO , "Ouverture dossier ... ", p)					
			os.system( self.fileExplorer + ' "' + p + '"')
		else:                                   # linux variants
			AppLog.log( LOG_INFO , "Ouverture dossier ... ", p)					
			os.system( self.fileExplorer + ' "' + p + '"')

	def onCopyItems(self) :

		itemTexts = self.viewer.getItemsText()
		self.printDebug("itemTexts : ", itemTexts)
		QApplication.clipboard().setText(itemTexts)

	def onCopyPhotoName(self) :

		p = str(Path(self.filename).resolve())
		QApplication.clipboard().setText(p)

	def onCurrentItemOpenShp( self ) :

		filepath = self.filename + ".shp"
		p = str(Path(filepath).resolve())

		import subprocess
		if platform.system() == 'Darwin':       # macOS
			subprocess.call(('open', p))
		elif platform.system() == 'Windows':    # Windows
			os.startfile(p)
		else:                                   # linux variants
			subprocess.call(('xdg-open', p))			

	def onPhotoNameEditChanged( self) :

		if self.filename is not None and self.photoName.text() != os.path.basename( self.filename ) :
			photoNewFileName = os.path.join( os.path.dirname(self.filename), self.photoName.text())
			if os.path.exists( photoNewFileName ) :
				self.photoName.setStyleSheet("color: red;")
			else :
				self.photoName.setStyleSheet("color: green;")
				self.setModified(True)

	def onPhotoNameEditValid( self ) :

		if self.filename is not None and self.photoName.text() != os.path.basename( self.filename ) :
			photoNewFileName = os.path.join( os.path.dirname(self.filename), self.photoName.text())
			photoCurrentName = self.filename
			if os.path.exists( photoNewFileName ) :
				msg = QMessageBox()
				msg.setIcon(QMessageBox.Critical)
				msg.setText("Ce fichier existe déjà.")
				msg.setInformativeText(photoNewFileName)
				msg.setWindowTitle("Error")
				x = msg.exec_()
			else :
				ret = QMessageBox.question(self, 'Confirmez', "Confirmez vous le changement de nom du média ?", QMessageBox.Yes | QMessageBox.Cancel, QMessageBox.Cancel)
				if ret == QMessageBox.Yes:

					try:
						import shutil
						shutil.move( photoCurrentName, photoNewFileName)
						if os.path.exists( photoCurrentName + ".shp" ) :
							shutil.move( photoCurrentName + ".shp", photoNewFileName + ".shp")
						self.filenamelist[self.filenamelistIndex] = photoNewFileName
						self.filename  = photoNewFileName
						self.setModified(False)
						self.photoName.setStyleSheet("color: black;")

					except Exception as e:
						self.printDebug(str(e.errno))
						self.printDebug(str(e))
						msg = QMessageBox()
						msg.setIcon(QMessageBox.Critical)
						msg.setText("Error")
						msg.setInformativeText(e)
						msg.setWindowTitle("Error")
						x = msg.exec_()
		else:
			self.photoName.setStyleSheet("color: black;")

	def onCurrentItemSave(self, itemUnderMouse, propFileName ) :
		self.printDebug("onCurrentItemSave : ", itemUnderMouse.itemTitle)

		coorItem = itemUnderMouse.mapToScene(itemUnderMouse.boundingRect().center())
		self.printDebug( " center coord : ",coorItem.x() , ", ", coorItem.y())
		self.printDebug( " wxh   : ",itemUnderMouse.rect().width() , ", ", itemUnderMouse.rect().height())

		rect = QRect(
						coorItem.x() - (itemUnderMouse.rect().width()/2), # Center - ( larg. / 2)
						coorItem.y() - (itemUnderMouse.rect().height()/2),
						itemUnderMouse.rect().width(),
						itemUnderMouse.rect().height()
					)
		pixmapOriginal = QPixmap(self.filename);
		pixmapCropped = pixmapOriginal.copy(rect)

		filedialog = QFileDialog(self,
								caption = "Selectionner le nom de l'image à sauvegarder",
								directory = str(propFileName)
								#option =
								)
		filedialog.setDefaultSuffix("png")
		filedialog.setNameFilter("Image Png (*.png);;All files (*.*)")
		filedialog.setAcceptMode(QFileDialog.AcceptSave)

		selected = filedialog.exec()
		if selected :
			filename = filedialog.selectedFiles()[0]
			self.printDebug("Saved as : " , filename )

			pixmapCropped.save(filename)

			msg = QMessageBox()
			msg.setIcon(QMessageBox.Information)

			pixmapHeight = pixmapCropped.height()
			pixmapWidth = pixmapCropped.width()
			self.printDebug("pixmap WxH = ", pixmapWidth, " x ", pixmapHeight)

			msg.setWindowTitle("Informations sur le média")
			msg.setText(
						"Fichier : <b>"+str(filename)+"</b>" + "<br>"
						)
			msg.setInformativeText(
								"Largeur et hauteur (pixels) : <b>"+str(pixmapWidth)+" x "+str(pixmapHeight) + "</b>" + "<br>" +
								"Dossier : <b>" + str(os.path.dirname(filename) ) + "</b>"
								)

			filemtime = time.strftime("%d/%m/%Y %H:%M",time.localtime(os.path.getmtime(filename)))
			filesize = self.sizeof_fmt( os.path.getsize(filename) )
			self.printDebug("Taille : " + str(filesize) )

			msg.setDetailedText(
									"Dernière modification : " + str(filemtime) + "\n" +
									"Taille : " + str(filesize)
								)

			msg.setStandardButtons(QMessageBox.Ok)

			retval = msg.exec_()

		else:
			self.printDebug("Sauvegarde annulée.")


	def sizeof_fmt(self, num, suffix='B'):

		for unit in ['','Ki','Mi','Gi','Ti','Pi','Ei','Zi']:
			if abs(num) < 1024.0:
				return "%3.1f%s%s" % (num, unit, suffix)
			num /= 1024.0
		return "%.1f%s%s" % (num, 'Yi', suffix)

	def getValue(self):

		if self.Reset is False :
			if self.Accept is False :
				return (None)
			else :
							
				# Remove rootMedia to each photo
				if self.filenamelist is not None and self.rootMedia != '' :
					index=0
					for fileName in self.filenamelist :
						
						if fileName == '' : continue
						
						rootMediaPos = fileName.find(self.rootMedia)
						if rootMediaPos == -1 :
							self.printDebug("NO rootmedia : ", fileName)
						if rootMediaPos == 0 :							
							self.printDebug("rootmedia @0 : ", fileName)
							self.filenamelist[index] = self.filenamelist[index].replace( self.rootMedia, '')						
						if rootMediaPos > 0 :
							self.printDebug("rootmedia >0 : ", rootMediaPos)
							self.filenamelist[index]  = self.filenamelist[index][rootMediaPos + len(self.rootMedia):]
						index=index+1
				else :
					self.printDebug("filenamelist is null (loadDataList)")				
					
				return (self.filenamelist)
		else :
			return ( [""] )



