#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# -----------------------------------------------------------
import sys
from lib.util import Util

from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import Qt, QSortFilterProxyModel, QDate, QDateTime
from PyQt5.QtWidgets import QHeaderView
from PyQt5.QtGui import QFont,QIcon, QPixmap , QImage, QColor, QPalette

from PyQt5.QtCore import QVariant

class TableModel(QtCore.QAbstractTableModel): 
	
	def __init__(self, tablename, datain, headerdata, dataColumsType, datatooltips, parent=None, *args): 
		QtCore.QAbstractTableModel.__init__(self, parent, *args) 
		self.arraydata = datain
		self.headerdata = headerdata
		self.tooltips = datatooltips
		self.dataColumsType = dataColumsType 
		self.tablename = tablename
		
	def rowCount(self, parent): 
		if self.arraydata :		
			return len(self.arraydata) 
		else : 
			return 0
 
	def columnCount(self, parent): 
		if self.arraydata :
			return len(self.arraydata[0]) 
		else : 
			return 0
 
	def data(self, index, role): 
		if not index.isValid(): 
			return QVariant() 
								
		if role == Qt.ToolTipRole: 			
			return QVariant(self.arraydata[index.row()][index.column()] ) 	

		if role == Qt.DisplayRole: 
			
			# Icones. Pas de texte			
			if self.tablename == "person" or self.tablename == "select_person":
				if index.column() == 6 :
					return QVariant()
				if index.column() == 7 :
					return QVariant()
				if index.column() == 8 :
					return QVariant()
					
			# Précision date
			if self.tablename == "event" or self.tablename == "select_event":
				if index.column() == 3 :
					return QVariant()
				if index.column() == 4 :
					return QVariant()	
				if index.column() == 6 :
					if self.arraydata[index.row()][index.column()] == "AAAA" : return "aaaa"
					if self.arraydata[index.row()][index.column()] == "MMAAAA" : return "mm/aaaa"
				if index.column() == 7 :
					if self.arraydata[index.row()][index.column()] == "ABT" : return "Environ"
					if self.arraydata[index.row()][index.column()] == "EXA" : return "Exacte"
					if self.arraydata[index.row()][index.column()] == "BEF" : return "Avant"
					if self.arraydata[index.row()][index.column()] == "AFT" : return "Après"
					if self.arraydata[index.row()][index.column()] == "EST" : return "Estimée"
					if self.arraydata[index.row()][index.column()] == "CAL" : return "Calculée"
					if self.arraydata[index.row()][index.column()] == "UNK" : return "Inconnue"					

					return QVariant( self.arraydata[index.row()][index.column()] ) 

			if self.tablename == "source" or self.tablename == "select_source":
				if index.column() == 3 :
					return QVariant()
				if index.column() == 4 :
					return QVariant()					
				if index.column() == 5 :
					return QVariant()									

			if self.tablename == "document" or self.tablename == "select_document":
				if index.column() == 3 :
					return QVariant()
				if index.column() == 4 :
					return QVariant()	

			# Précision lieu
			if self.tablename == "place" or self.tablename == "select_place":
				if index.column() == 12 :
					if self.arraydata[index.row()][index.column()] == "EXA" : return "exactement"
					if self.arraydata[index.row()][index.column()] == "ENV" : return "approximativement"

					return QVariant( self.arraydata[index.row()][index.column()] ) 
			
			# Précision date media
			if self.tablename == "media" or self.tablename == "select_media":
				if index.column() == 2 :
					return QVariant()
				if index.column() == 9 :
					return QVariant()	
				if index.column() == 4 :
					if self.arraydata[index.row()][index.column()] == "AAAA" : return "aaaa"
					if self.arraydata[index.row()][index.column()] == "MMAAAA" : return "mm/aaaa"								
				if index.column() == 5 :
					if self.arraydata[index.row()][index.column()] == "ABT" : return "Environ"
					if self.arraydata[index.row()][index.column()] == "EXA" : return "Exacte"
					if self.arraydata[index.row()][index.column()] == "BEF" : return "Avant"
					if self.arraydata[index.row()][index.column()] == "AFT" : return "Après"
					if self.arraydata[index.row()][index.column()] == "EST" : return "Estimée"
					if self.arraydata[index.row()][index.column()] == "CAL" : return "Calculée"
					if self.arraydata[index.row()][index.column()] == "UNK" : return "Inconnue"					

					return QVariant( self.arraydata[index.row()][index.column()] ) 
									
			if self.dataColumsType[index.column()] == 'DATETIME' or self.dataColumsType[index.column()] == 'DATE'  :
				
				myQdate = Util.getQDatetimeFromString( self.arraydata[index.row()][index.column()] , allowqdate = True )
				#print("Sql date value = ", self.arraydata[index.row()][index.column()])
				if self.arraydata[index.row()][index.column()] == '0001-01-01 00:00' :
					return ''
				return myQdate

			return QVariant(self.arraydata[index.row()][index.column()]) 

		if role == Qt.BackgroundRole:
			pass
			
		if role == Qt.FontRole :
			if self.tablename == "person" or self.tablename == "select_person":
				if index.column() == 7 :
					pass
			
		if role == Qt.TextColorRole :
			if self.tablename == "person" or self.tablename == "select_person":
				if index.column() == 7 :					
					pass
					#return QVariant( QColor(Qt.red) )
			
		if role == Qt.UserRole: # Données brutes					
			return self.arraydata[index.row()][index.column()]
			
		if role == Qt.UserRole + 1 : # Retourne l'index				
			return index.column()

		if role == Qt.UserRole + 2 : # Sort Role	
			return QVariant(self.arraydata[index.row()][index.column()]) 
		
		return QVariant() 

	def headerData(self, colsection, orientation, role):

		if role == Qt.ToolTipRole  :
			return str(self.tooltips[colsection] + "\n (idx " + str(colsection) + ")" )		
		
		if orientation == Qt.Horizontal and role == Qt.DisplayRole:
			return QVariant(self.headerdata[colsection] )
			
		return QVariant()

	def removeRows(self, position, rows, QModelIndex):
		
		self.layoutAboutToBeChanged.emit()
		self.beginRemoveRows(QModelIndex, position, position+rows-1)
		#print("remove row in model  position: " + str(position) + " row count=" + str(rows))
		for i in range(rows):
			del self.arraydata[position]
		self.layoutChanged.emit()
		return True
		
		
class TableHeaderView(QHeaderView):
	
	def __init__(self, parent):
		QHeaderView.__init__(self, Qt.Horizontal, parent)

		self.setStyleSheet("::section{background-color: #bfcee0; color: black; }")
		self.setSectionsClickable(True)

class TableProxyModel ( QSortFilterProxyModel ) :
	
	def __init__(self, parent=None):
		super(TableProxyModel, self).__init__(parent)
		self.filterValue = None
	   
	def sort(self, column, order):		
		super(TableProxyModel, self).sort(column, order)

	def setFilterValue( self, filterValue) :
		self.filterValue = filterValue
		
	def applyFilters(self):
		self.invalidateFilter()

	def filterAcceptsRow(self, source_row, source_parent):
		
		if self.filterValue is None :
			return True		
		
		model = self.sourceModel()
						
		for columnkey in range (0,model.columnCount(self)) :
			ix = model.index(source_row, columnkey, source_parent)
			if ix.isValid():
				
				if isinstance(model.data(ix, Qt.UserRole), QDate):
					text = model.data(ix, Qt.UserRole).toString('dd/MM/yyyy hh:mm')					
					
				else :
					if isinstance(model.data(ix, Qt.UserRole), QDateTime):
						text = model.data(ix, Qt.UserRole).toString('dd/MM/yyyy hh:mm')					
					else :
						if isinstance(model.data(ix, Qt.UserRole), str):
							text = model.data(ix, Qt.UserRole)							
						else :
							text = model.data(ix, Qt.UserRole)
				
				import re	
				'''			
				result="False"				
				if re.search( str(self.filterValue), str(text),  re.IGNORECASE) :					
					result="True"
				'''
				if re.search( str(self.filterValue), str(text),  re.IGNORECASE) :					
					return True
									
		return False

