#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Généalogik

		*** Source autogénéré *** 

        Application entry.

        05/24/2023 14:31

        lvardon@laposte.net - 2021

        Licence libre

'''

import os, sys, locale, platform, configparser

from PyQt5 import QtCore
from PyQt5.QtWidgets import QMainWindow, QAction, QApplication, QMessageBox
from PyQt5.QtWidgets import QTextEdit
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import Qt, pyqtSlot, pyqtSignal

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL
import lib.setting as AppSetting
import lib.log as AppLog


from libdbcustom.person import Persons
from libdbcustom.event import Events
from libdbcustom.source import Sources
from libdbcustom.document import Documents
from libdbcustom.media import Medias
from libdbcustom.place import Places
from libdbcustom.depot import Depots
from libdbcustom.eventtype import Eventtypes
from libdbcustom.documenttype import Documenttypes
from libdbcustom.event_source import Event_sources
from libdbcustom.media_person import Media_persons
from libdbcustom.metadata import Metadatas

from libview.person import PersonListWidget, PersonEditWidget
from libview.event import EventListWidget, EventEditWidget
from libview.source import SourceListWidget, SourceEditWidget
from libview.document import DocumentListWidget, DocumentEditWidget
from libview.media import MediaListWidget, MediaEditWidget
from libview.place import PlaceListWidget, PlaceEditWidget
from libview.depot import DepotListWidget, DepotEditWidget
from libview.eventtype import EventtypeListWidget, EventtypeEditWidget
from libview.documenttype import DocumenttypeListWidget, DocumenttypeEditWidget
from libview.event_source import Event_sourceListWidget, Event_sourceEditWidget
from libview.media_person import Media_personListWidget, Media_personEditWidget
from libview.metadata import MetadataListWidget, MetadataEditWidget

from lib.project import Project
from customapp.customApp import customApp

class myAppWindow(QMainWindow):
	
	db = None
	rootMedia = None	
	signalAppCloseEvent = pyqtSignal( )		
	
	def __init__(self):
		super().__init__()
		
		self.customAppObj = customApp( self )
		
		lastProjectFileName = AppSetting.get('app/lastProject', '')	
		
		self.customAppObj.loadProjectFile( lastProjectFileName )		

		self.initUI()
		
		self.customAppObj.main( )		

	def initUI(self):

		textEdit = QTextEdit()
		self.setCentralWidget(textEdit)

		self.statusBar()
		menubar = self.menuBar()
	
		# Création des entêtes de menu et des barres d'outils
		fichierMenu = menubar.addMenu('Fichier')
		fichierToolbar = self.addToolBar('Fichier')
		editMenu = menubar.addMenu('Editer')
		editToolbar = self.addToolBar('Editer')
		tableMenu = menubar.addMenu('Tables')
		tableToolbar = self.addToolBar('Tables')
		aideMenu = menubar.addMenu('Aide')
		aideToolbar = self.addToolBar('Aide')


		# Ajout dynamique des entrées de menu suplémentaires, pour chacune des entrées de menus
		from customapp.menuOptions import MenuOptions	
		menuOptionObj = MenuOptions( self, self.customAppObj )						
		menuOptionObj.AppendMenuOptions( fichierMenu, "fichier", fichierToolbar, "Fichier" )
		menuOptionObj.AppendMenuOptions( editMenu, "edit", editToolbar, "Editer" )
		menuOptionObj.AppendMenuOptions( tableMenu, "table", tableToolbar, "Tables" )
		menuOptionObj.AppendMenuOptions( aideMenu, "aide", aideToolbar, "Aide" )
			
	
		# Ajout dynamique des raccourcis, (non liés aux entrées de menu)
		from customapp.menuShorts import MenuShorts	
		menuShortObj = 	MenuShorts( self,self.customAppObj )
		menuShortObj.appendShortcuts( )			
					
		# Tables menu - Actions definitions
		personAct = QAction(QIcon('img/menu_table_person.png'), 'Individu', self)
		personAct.setStatusTip('Liste des individus')
		personAct.triggered.connect(self.onPerson)
		eventAct = QAction(QIcon('img/menu_table_event.png'), 'Evenement', self)
		eventAct.setStatusTip('Liste des évenements')
		eventAct.triggered.connect(self.onEvent)
		sourceAct = QAction(QIcon('img/menu_table_source.png'), 'Source', self)
		sourceAct.setStatusTip('Liste des sources')
		sourceAct.triggered.connect(self.onSource)
		documentAct = QAction(QIcon('img/menu_table_document.png'), 'Document', self)
		documentAct.setStatusTip('Contenu de la table Document')
		documentAct.triggered.connect(self.onDocument)
		mediaAct = QAction(QIcon('img/menu_table_media.png'), 'Média', self)
		mediaAct.setStatusTip('Référentiel des médias')
		mediaAct.triggered.connect(self.onMedia)
		placeAct = QAction(QIcon('img/menu_table_place.png'), 'Lieu', self)
		placeAct.setStatusTip('Liste des adresses des événements')
		placeAct.triggered.connect(self.onPlace)
		depotAct = QAction(QIcon('img/menu_table_depot.png'), 'Dépot', self)
		depotAct.setStatusTip('Liste des dépôts pour les sources')
		depotAct.triggered.connect(self.onDepot)
		eventtypeAct = QAction(QIcon('img/menu_table_eventtype.png'), 'Type evenement', self)
		eventtypeAct.setStatusTip('Référentiel des types évenements')
		eventtypeAct.triggered.connect(self.onEventtype)
		documenttypeAct = QAction(QIcon('img/menu_table_documenttype.png'), 'Type de document', self)
		documenttypeAct.setStatusTip('Référentiel des types de documents')
		documenttypeAct.triggered.connect(self.onDocumenttype)
		event_sourceAct = QAction(QIcon('img/menu_table_event_source.png'), 'Evénement - source', self)
		event_sourceAct.setStatusTip('Association des événements aux sources')
		event_sourceAct.triggered.connect(self.onEvent_source)
		media_personAct = QAction(QIcon('img/menu_table_media_person.png'), 'Média - individu', self)
		media_personAct.setStatusTip('Associations média - individu')
		media_personAct.triggered.connect(self.onMedia_person)
		metadataAct = QAction(QIcon('img/menu_table_metadata.png'), 'Paramètre', self)
		metadataAct.setStatusTip('Paramètres et métadonnées')
		metadataAct.triggered.connect(self.onMetadata)
	
		# Tables menu - Menu add actions
		tableMenu.addAction(personAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(eventAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(sourceAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(documentAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(mediaAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(placeAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(depotAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(eventtypeAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(documenttypeAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(event_sourceAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(media_personAct)
		#tableMenu.addSeparator()
		tableMenu.addAction(metadataAct)
		#tableMenu.addSeparator()

		# Tables toolbar - Toolbar add actions 
		toolbarTable = self.addToolBar('Tables')		
		toolbarTable.setObjectName('Tables')
		toolbarTable.addAction(personAct)
		toolbarTable.addAction(eventAct)
		toolbarTable.addAction(sourceAct)
		toolbarTable.addAction(documentAct)
		toolbarTable.addAction(mediaAct)
		toolbarTable.addAction(placeAct)
		toolbarTable.addAction(depotAct)
		toolbarTable.addAction(eventtypeAct)
		toolbarTable.addAction(documenttypeAct)
		toolbarTable.addAction(event_sourceAct)
		toolbarTable.addAction(media_personAct)
		toolbarTable.addAction(metadataAct)


		self.setGeometry(300, 300, 600, 450)
		self.setWindowIcon(QIcon('img/app64.png'))
		self.show()

	def updateWindowTitle( self, message="", isProjectname = False ) :
		if self.windowTitle() == "" and message == ""  :
			self.setWindowTitle('Généalogik')
		else :
			if isProjectname is False :
				self.setWindowTitle(message)
			else :
				self.setWindowTitle("Projet : [" + os.path.basename(message) + "] - " + message)						

	# Additionnals menu - actions events
	def onRapportexport( self ) :
		RapportexportObj = Rapportexport( self, self.db )
		self.setCentralWidget( RapportexportObj )
		RapportexportObj.main()

	def onGedcomexport( self ) :
		GedcomexportObj = Gedcomexport( self, self.db )
		self.setCentralWidget( GedcomexportObj )
		GedcomexportObj.main()

	def onDataenter( self ) :
		DataenterObj = Dataenter( self, self.db )
		self.setCentralWidget( DataenterObj )
		DataenterObj.main()

	def onAide( self ) :
		AideObj = Aide( self, self.db )
		self.setCentralWidget( AideObj )
		AideObj.main()

	def onApropos( self ) :
		AproposObj = Apropos( self, self.db )
		self.setCentralWidget( AproposObj )
		AproposObj.main()


	
	def onPerson( self ) :
		self.rowId = None
		self.onListPerson()
	def onEvent( self ) :
		self.rowId = None
		self.onListEvent()
	def onSource( self ) :
		self.rowId = None
		self.onListSource()
	def onDocument( self ) :
		self.rowId = None
		self.onListDocument()
	def onMedia( self ) :
		self.rowId = None
		self.onListMedia()
	def onPlace( self ) :
		self.rowId = None
		self.onListPlace()
	def onDepot( self ) :
		self.rowId = None
		self.onListDepot()
	def onEventtype( self ) :
		self.rowId = None
		self.onListEventtype()
	def onDocumenttype( self ) :
		self.rowId = None
		self.onListDocumenttype()
	def onEvent_source( self ) :
		self.rowId = None
		self.onListEvent_source()
	def onMedia_person( self ) :
		self.rowId = None
		self.onListMedia_person()
	def onMetadata( self ) :
		self.rowId = None
		self.onListMetadata()


	@pyqtSlot()
	def onListPerson( self ) :
		self.myPersonListWidget = PersonListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myPersonListWidget )
		self.myPersonListWidget.selectRowById( self.rowId )
		self.myPersonListWidget.mainWindowEditMode.connect(self.onEditPerson)
		self.myPersonListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Persons( self.db ).getCount()
		self.setWindowTitle('Individu ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListEvent( self ) :
		self.myEventListWidget = EventListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myEventListWidget )
		self.myEventListWidget.selectRowById( self.rowId )
		self.myEventListWidget.mainWindowEditMode.connect(self.onEditEvent)
		self.myEventListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Events( self.db ).getCount()
		self.setWindowTitle('Evenement ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListSource( self ) :
		self.mySourceListWidget = SourceListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.mySourceListWidget )
		self.mySourceListWidget.selectRowById( self.rowId )
		self.mySourceListWidget.mainWindowEditMode.connect(self.onEditSource)
		self.mySourceListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Sources( self.db ).getCount()
		self.setWindowTitle('Source ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListDocument( self ) :
		self.myDocumentListWidget = DocumentListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myDocumentListWidget )
		self.myDocumentListWidget.selectRowById( self.rowId )
		self.myDocumentListWidget.mainWindowEditMode.connect(self.onEditDocument)
		self.myDocumentListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Documents( self.db ).getCount()
		self.setWindowTitle('Document ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListMedia( self ) :
		self.myMediaListWidget = MediaListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myMediaListWidget )
		self.myMediaListWidget.selectRowById( self.rowId )
		self.myMediaListWidget.mainWindowEditMode.connect(self.onEditMedia)
		self.myMediaListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Medias( self.db ).getCount()
		self.setWindowTitle('Média ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListPlace( self ) :
		self.myPlaceListWidget = PlaceListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myPlaceListWidget )
		self.myPlaceListWidget.selectRowById( self.rowId )
		self.myPlaceListWidget.mainWindowEditMode.connect(self.onEditPlace)
		self.myPlaceListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Places( self.db ).getCount()
		self.setWindowTitle('Lieu ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListDepot( self ) :
		self.myDepotListWidget = DepotListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myDepotListWidget )
		self.myDepotListWidget.selectRowById( self.rowId )
		self.myDepotListWidget.mainWindowEditMode.connect(self.onEditDepot)
		self.myDepotListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Depots( self.db ).getCount()
		self.setWindowTitle('Dépot ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListEventtype( self ) :
		self.myEventtypeListWidget = EventtypeListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myEventtypeListWidget )
		self.myEventtypeListWidget.selectRowById( self.rowId )
		self.myEventtypeListWidget.mainWindowEditMode.connect(self.onEditEventtype)
		self.myEventtypeListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Eventtypes( self.db ).getCount()
		self.setWindowTitle('Type evenement ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListDocumenttype( self ) :
		self.myDocumenttypeListWidget = DocumenttypeListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myDocumenttypeListWidget )
		self.myDocumenttypeListWidget.selectRowById( self.rowId )
		self.myDocumenttypeListWidget.mainWindowEditMode.connect(self.onEditDocumenttype)
		self.myDocumenttypeListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Documenttypes( self.db ).getCount()
		self.setWindowTitle('Type de document ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListEvent_source( self ) :
		self.myEvent_sourceListWidget = Event_sourceListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myEvent_sourceListWidget )
		self.myEvent_sourceListWidget.selectRowById( self.rowId )
		self.myEvent_sourceListWidget.mainWindowEditMode.connect(self.onEditEvent_source)
		self.myEvent_sourceListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Event_sources( self.db ).getCount()
		self.setWindowTitle('Evénement - source ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListMedia_person( self ) :
		self.myMedia_personListWidget = Media_personListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myMedia_personListWidget )
		self.myMedia_personListWidget.selectRowById( self.rowId )
		self.myMedia_personListWidget.mainWindowEditMode.connect(self.onEditMedia_person)
		self.myMedia_personListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Media_persons( self.db ).getCount()
		self.setWindowTitle('Média - individu ('+ str(statcount['count']) + ')')
	@pyqtSlot()
	def onListMetadata( self ) :
		self.myMetadataListWidget = MetadataListWidget( self.db, self.rootMedia, "source_main" )
		self.setCentralWidget( self.myMetadataListWidget )
		self.myMetadataListWidget.selectRowById( self.rowId )
		self.myMetadataListWidget.mainWindowEditMode.connect(self.onEditMetadata)
		self.myMetadataListWidget.mainWindowUserMessage.connect(self.onUserMessage)
		statcount = Metadatas( self.db ).getCount()
		self.setWindowTitle('Paramètre ('+ str(statcount['count']) + ')')



	@pyqtSlot( int, str )
	def onEditPerson(self, rowId, message ):
		rowDataObj = self.myPersonListWidget.getRowData( rowId )
		self.myPersonEditWidget = PersonEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myPersonListWidget.getIdList(), \
													self.myPersonListWidget.getSqlOrderByField(), \
													self.myPersonListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myPersonEditWidget )
		self.myPersonEditWidget.mainWindowListMode.connect(self.onListPerson)
		self.myPersonEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myPersonEditWidget.closeEdit.connect( self.onPersonCloseEditSignal )	
		self.setWindowTitle('Edition : Individu')

	@pyqtSlot( int, str )
	def onEditEvent(self, rowId, message ):
		rowDataObj = self.myEventListWidget.getRowData( rowId )
		self.myEventEditWidget = EventEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myEventListWidget.getIdList(), \
													self.myEventListWidget.getSqlOrderByField(), \
													self.myEventListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myEventEditWidget )
		self.myEventEditWidget.mainWindowListMode.connect(self.onListEvent)
		self.myEventEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myEventEditWidget.closeEdit.connect( self.onEventCloseEditSignal )	
		self.setWindowTitle('Edition : Evenement')

	@pyqtSlot( int, str )
	def onEditSource(self, rowId, message ):
		rowDataObj = self.mySourceListWidget.getRowData( rowId )
		self.mySourceEditWidget = SourceEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.mySourceListWidget.getIdList(), \
													self.mySourceListWidget.getSqlOrderByField(), \
													self.mySourceListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.mySourceEditWidget )
		self.mySourceEditWidget.mainWindowListMode.connect(self.onListSource)
		self.mySourceEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.mySourceEditWidget.closeEdit.connect( self.onSourceCloseEditSignal )	
		self.setWindowTitle('Edition : Source')

	@pyqtSlot( int, str )
	def onEditDocument(self, rowId, message ):
		rowDataObj = self.myDocumentListWidget.getRowData( rowId )
		self.myDocumentEditWidget = DocumentEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myDocumentListWidget.getIdList(), \
													self.myDocumentListWidget.getSqlOrderByField(), \
													self.myDocumentListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myDocumentEditWidget )
		self.myDocumentEditWidget.mainWindowListMode.connect(self.onListDocument)
		self.myDocumentEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myDocumentEditWidget.closeEdit.connect( self.onDocumentCloseEditSignal )	
		self.setWindowTitle('Edition : Document')

	@pyqtSlot( int, str )
	def onEditMedia(self, rowId, message ):
		rowDataObj = self.myMediaListWidget.getRowData( rowId )
		self.myMediaEditWidget = MediaEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myMediaListWidget.getIdList(), \
													self.myMediaListWidget.getSqlOrderByField(), \
													self.myMediaListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myMediaEditWidget )
		self.myMediaEditWidget.mainWindowListMode.connect(self.onListMedia)
		self.myMediaEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myMediaEditWidget.closeEdit.connect( self.onMediaCloseEditSignal )	
		self.setWindowTitle('Edition : Média')

	@pyqtSlot( int, str )
	def onEditPlace(self, rowId, message ):
		rowDataObj = self.myPlaceListWidget.getRowData( rowId )
		self.myPlaceEditWidget = PlaceEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myPlaceListWidget.getIdList(), \
													self.myPlaceListWidget.getSqlOrderByField(), \
													self.myPlaceListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myPlaceEditWidget )
		self.myPlaceEditWidget.mainWindowListMode.connect(self.onListPlace)
		self.myPlaceEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myPlaceEditWidget.closeEdit.connect( self.onPlaceCloseEditSignal )	
		self.setWindowTitle('Edition : Lieu')

	@pyqtSlot( int, str )
	def onEditDepot(self, rowId, message ):
		rowDataObj = self.myDepotListWidget.getRowData( rowId )
		self.myDepotEditWidget = DepotEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myDepotListWidget.getIdList(), \
													self.myDepotListWidget.getSqlOrderByField(), \
													self.myDepotListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myDepotEditWidget )
		self.myDepotEditWidget.mainWindowListMode.connect(self.onListDepot)
		self.myDepotEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myDepotEditWidget.closeEdit.connect( self.onDepotCloseEditSignal )	
		self.setWindowTitle('Edition : Dépot')

	@pyqtSlot( int, str )
	def onEditEventtype(self, rowId, message ):
		rowDataObj = self.myEventtypeListWidget.getRowData( rowId )
		self.myEventtypeEditWidget = EventtypeEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myEventtypeListWidget.getIdList(), \
													self.myEventtypeListWidget.getSqlOrderByField(), \
													self.myEventtypeListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myEventtypeEditWidget )
		self.myEventtypeEditWidget.mainWindowListMode.connect(self.onListEventtype)
		self.myEventtypeEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myEventtypeEditWidget.closeEdit.connect( self.onEventtypeCloseEditSignal )	
		self.setWindowTitle('Edition : Type evenement')

	@pyqtSlot( int, str )
	def onEditDocumenttype(self, rowId, message ):
		rowDataObj = self.myDocumenttypeListWidget.getRowData( rowId )
		self.myDocumenttypeEditWidget = DocumenttypeEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myDocumenttypeListWidget.getIdList(), \
													self.myDocumenttypeListWidget.getSqlOrderByField(), \
													self.myDocumenttypeListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myDocumenttypeEditWidget )
		self.myDocumenttypeEditWidget.mainWindowListMode.connect(self.onListDocumenttype)
		self.myDocumenttypeEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myDocumenttypeEditWidget.closeEdit.connect( self.onDocumenttypeCloseEditSignal )	
		self.setWindowTitle('Edition : Type de document')

	@pyqtSlot( int, str )
	def onEditEvent_source(self, rowId, message ):
		rowDataObj = self.myEvent_sourceListWidget.getRowData( rowId )
		self.myEvent_sourceEditWidget = Event_sourceEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myEvent_sourceListWidget.getIdList(), \
													self.myEvent_sourceListWidget.getSqlOrderByField(), \
													self.myEvent_sourceListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myEvent_sourceEditWidget )
		self.myEvent_sourceEditWidget.mainWindowListMode.connect(self.onListEvent_source)
		self.myEvent_sourceEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myEvent_sourceEditWidget.closeEdit.connect( self.onEvent_sourceCloseEditSignal )	
		self.setWindowTitle('Edition : Evénement - source')

	@pyqtSlot( int, str )
	def onEditMedia_person(self, rowId, message ):
		rowDataObj = self.myMedia_personListWidget.getRowData( rowId )
		self.myMedia_personEditWidget = Media_personEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myMedia_personListWidget.getIdList(), \
													self.myMedia_personListWidget.getSqlOrderByField(), \
													self.myMedia_personListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myMedia_personEditWidget )
		self.myMedia_personEditWidget.mainWindowListMode.connect(self.onListMedia_person)
		self.myMedia_personEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myMedia_personEditWidget.closeEdit.connect( self.onMedia_personCloseEditSignal )	
		self.setWindowTitle('Edition : Média - individu')

	@pyqtSlot( int, str )
	def onEditMetadata(self, rowId, message ):
		rowDataObj = self.myMetadataListWidget.getRowData( rowId )
		self.myMetadataEditWidget = MetadataEditWidget( self.db,\
													self.rootMedia, \
													rowId, \
													None, \
													self.myMetadataListWidget.getIdList(), \
													self.myMetadataListWidget.getSqlOrderByField(), \
													self.myMetadataListWidget.getSqlOrderByOrder(), \
													rowDataObj )
		self.setCentralWidget( self.myMetadataEditWidget )
		self.myMetadataEditWidget.mainWindowListMode.connect(self.onListMetadata)
		self.myMetadataEditWidget.mainWindowUserMessage.connect(self.onUserMessage)
		self.myMetadataEditWidget.closeEdit.connect( self.onMetadataCloseEditSignal )	
		self.setWindowTitle('Edition : Paramètre')



	@pyqtSlot( int, int )
	def onPersonCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListPerson() 

	@pyqtSlot( int, int )
	def onEventCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListEvent() 

	@pyqtSlot( int, int )
	def onSourceCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListSource() 

	@pyqtSlot( int, int )
	def onDocumentCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListDocument() 

	@pyqtSlot( int, int )
	def onMediaCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListMedia() 

	@pyqtSlot( int, int )
	def onPlaceCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListPlace() 

	@pyqtSlot( int, int )
	def onDepotCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListDepot() 

	@pyqtSlot( int, int )
	def onEventtypeCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListEventtype() 

	@pyqtSlot( int, int )
	def onDocumenttypeCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListDocumenttype() 

	@pyqtSlot( int, int )
	def onEvent_sourceCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListEvent_source() 

	@pyqtSlot( int, int )
	def onMedia_personCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListMedia_person() 

	@pyqtSlot( int, int )
	def onMetadataCloseEditSignal(self, btnState, rowId) :		
		self.rowId = rowId
		self.onListMetadata() 
			

	@pyqtSlot( int, str )
	def onUserMessage(self, level, message ):
		self.statusBar().showMessage(message)

	# slot 2
	@pyqtSlot( int, str )
	def onUserMessage(self, level, message ):
		self.statusBar().showMessage(message)
		
	def showEvent(self,event):
		self.loadWindowState()
	
	def closeEvent(self,event):
		result = QMessageBox.question(self,
                      "Confirmer la fermeture",
                      "Voulez-vous fermer le programme ?",
                      QMessageBox.Yes| QMessageBox.No)
     
		event.ignore()
		if result == QMessageBox.Yes:
			self.storeWindowState()
			self.signalAppCloseEvent.emit()
			event.accept()
	
	def storeWindowState(self) :

		AppSetting.set("app/geometry", self.saveGeometry())
		AppSetting.set("app/state", self.saveState())
		
	def loadWindowState(self) :

		if AppSetting.get("app/geometry", None) is not None:			
			self.restoreGeometry(AppSetting.get("app/geometry", ""))
			
		if AppSetting.get("app/state", None) is not None:			
			self.restoreState(AppSetting.get("app/state", ""))			
		
def main():

	# Windows hack (taskbar icon)
	if platform.system() == 'Windows':    # Windows
		import ctypes
		myappid = u'mycompany.myproduct.subproduct.version' # arbitrary string
		ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
		
	# Set Current directory to the one containing this file.
	os.chdir(os.path.dirname(os.path.abspath(__file__)))

	AppSetting.init()
	
	# Locales
	if platform.system() == 'Darwin':       # macOS
		locale.setlocale(locale.LC_ALL, ('fr_FR', 'UTF8'))
		
	elif platform.system() == 'Windows':    # Windows
		#locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8')
		pass
		
	else: # linux variants
		locale.setlocale(locale.LC_ALL, ('fr_FR', 'UTF8'))
		
	logfile = AppSetting.get('app/logfile', "defaultlog.txt")
	AppLog.init( logfile, True )
	AppLog.log( LOG_INFO, "Init log file: ", logfile)
	
	app = QApplication(sys.argv)

    # Traductions de qt (pour QtDialog, les boutons yes/no/cancel...)
	localeName = QtCore.QLocale.system().name()
	
	translator = QtCore.QTranslator(app)
	if translator.load("locale/locale_" + localeName + ".qm"):
		app.installTranslator(translator)
        
	translator2 = QtCore.QTranslator (app)
	if translator2.load("locale/qt_fr.qm"):
		app.installTranslator(translator2)

	translatorQt = QtCore.QTranslator (app)
	if translatorQt.load("locale/qtbase_fr.qm"):
		app.installTranslator(translatorQt)

	f = app.font()
	f.setPointSize( AppSetting.getInt('app/fontsize', 12) )
	app.setFont(f)

	ex = myAppWindow()
	sys.exit(app.exec_())

if __name__ == '__main__':
	main()

