import sys, os, re, platform
import configparser
from PySide2.QtCore import Qt, QUrl, QSettings, QItemSelectionModel, QDir, QDirIterator, QTimer, QSortFilterProxyModel, QModelIndex, QSize
from PySide2.QtWidgets import  QMenu, QShortcut, QAction, QApplication, QSizePolicy, QSplitter, QFileSystemModel, QTreeView, QWidget, QGridLayout, QVBoxLayout, QHBoxLayout, QGroupBox, QFormLayout, QLabel, QLineEdit, QComboBox, QPushButton, QLabel, QAbstractItemView, QMessageBox, QDialog, QTextEdit,QStyle
from PySide2.QtGui import QDesktopServices, QIcon, QFont, QPixmap, QKeySequence, QCursor

from PySide2.QtWidgets import QGraphicsView, QGraphicsScene, QGraphicsPixmapItem, QFrame
from PySide2.QtCore import Signal, Slot
from PySide2.QtCore import QPoint, QRectF
from PySide2.QtGui import QBrush, QColor

from lib.sqlitedb import Sqlitedb
from libdb.source import SourcesDb
from libdbcustom.source import Source, Sources
from libdb.event import EventsDb
from libdbcustom.event import Event, Events
from libdb.person import PersonsDb
from libdbcustom.person import Person, Persons
from libdb.document import DocumentsDb
from libdbcustom.document import Document, Documents
from libdb.media import MediasDb
from libdbcustom.media import Media, Medias
from libdb.place import PlacesDb
from libdbcustom.place import Place, Places
from libdb.depot import DepotsDb
from libdbcustom.depot import Depot, Depots
		
class iniParamFile() :

	def __init__(self, inifile):
		self.inifile = inifile
		self.settings = QSettings(self.inifile, QSettings.IniFormat)
		self.settings.setIniCodec("UTF-8");

	def getValue (self, section, key ) :
		value = self.settings.value( str(section) + "/" + str(key) )
		return value

	def setValue (self, section, key, value ) :
		self.settings.setValue( str(section) + "/" + str(key), value )

class MyFilterModel(QSortFilterProxyModel):

	filterValue = ""

	def __init__(self, *args, **kwargs):
		super(MyFilterModel, self).__init__(*args, **kwargs)

	def path_from_index(self, index):
		path = []
		while index.isValid():
			path.insert(0, index.data())
			index = index.parent()
		return path

	def filterAcceptsRow(self, sourceRow, sourceParent):

		index = self.sourceModel().index(sourceRow, 0, sourceParent)
		fileName = index.data()

		path=""
		sep=""
		paths = self.path_from_index(sourceParent)
		for pathelem in paths :
			path = path + str(pathelem) + sep
			sep="/"

		fullFilename = os.path.join(path, fileName)

		subOffset = -1
		if os.path.isfile( fullFilename ) :
			import re
			substringOffset = re.search(self.filterValue, fileName)
			if substringOffset :
				subOffset = substringOffset.start()

			#print('  filterAcceptsRow data =',subOffset, " [",fileName,"]")

			if self.filterValue != "" and subOffset < 0  :
				#print('  FALSE : filterAcceptsRow data =',"[",fileName,"] " ,self.filterValue, " :", subOffset)
				return False
			else :
				pass
				#print('        : filterAcceptsRow data =',"[",fileName,"] " ,self.filterValue, " :", subOffset)

		rowCount = self.sourceModel().rowCount(index)

		accepted = QSortFilterProxyModel.filterAcceptsRow(self, sourceRow, sourceParent)

		if rowCount > 0 and not accepted:
			for row in range(rowCount):
				if self.filterAcceptsRow(row, index):
					return True

		return accepted

	def setFilterValue( self, filterValue ) :
		self.filterValue = filterValue

class App(QDialog):

	currentFile = None
	currentDir = None
	toggleFlat = False

	fileExplorer = "nemo"

	def __init__(self , iniFile, dbFile, rootPath, currentFile ):
		super().__init__()

		self.dbFile = dbFile
		self.rootPath = rootPath
		self.currentFile = currentFile
		self.param = iniParamFile( iniFile )

		q = QShortcut( QKeySequence("F2"), self, self.scrollToSelectedFile, Qt.WindowShortcut )

		self.initUI()

		self.restoreStates()

		self.setWindowIcon(QIcon('img/menu_editbasedocument.png'))

		self.setModal(True)
		#self.setWindowModality(Qt.ApplicationModal)
		#widget.show()

	def initUI(self):

		new_font = QFont()
		new_font.setPointSize( int(self.param.getValue("app" ,"fontsize") ))
		self.setFont( new_font )

		#self.initPath = self.param.getValue( 'app', 'rootPath' )
		self.initPath = self.rootPath
		self.setWindowTitle(self.initPath)

		self.model = QFileSystemModel()
		print("--- iniUI : self.initPath")
		self.model.setRootPath(self.initPath) #  The model have a rootPath that tells them which part of the model monitor changes.
		self.model.directoryLoaded.connect(self.onDirectoryLoaded)

		self.proxy_model = MyFilterModel()
		#self.proxy_model.setDynamicSortFilter(True)
		self.proxy_model.setSourceModel(self.model)

		filefilter = []
		filefilterList = self.param.getValue( 'app', 'filefilter' )
		for filefilterElem in filefilterList :
			filefilter.append(filefilterElem)
		self.model.setNameFilters(filefilter)
		self.model.setFilter( QDir.Files | QDir.AllDirs | QDir.NoDotAndDotDot )  # QtCore.QDir.AllDirs | QtCore.QDir.NoDotAndDotDot | QtCore.QDir.AllEntries)
		self.model.setNameFilterDisables(False)

		self.tree = QTreeView()
		self.tree.setModel(self.proxy_model)

		print("--- iniUI : rootIndex")
		modelindex = self.model.index(self.initPath)
		rootIndex = self.proxy_model.mapFromSource( modelindex ) # Returns the source model index corresponding to the given proxyIndex from the sorting filter model.
		self.tree.setRootIndex(rootIndex) #  The views have a rootIndex that tells them which part of the model to show.

		self.tree.setWindowTitle("Tree View")
		self.tree.selectionModel().selectionChanged.connect(self.onFileSelected)
		#self.tree.clicked.connect(self.onFileClicked)
		self.tree.clicked.connect(self.onTreeClicked)
		self.tree.setAnimated(False)
		self.tree.setSortingEnabled(True)
		self.tree.setContextMenuPolicy(Qt.CustomContextMenu)
		self.tree.customContextMenuRequested.connect(self.onOpenMenu)

		leftLayout = QVBoxLayout()

		# ---- Tree view
		leftLayout.addWidget(self.tree)

		# ---- Search
		searchWidget = QWidget()
		searchLayout = QHBoxLayout()

		self.filterLineEdit = QLineEdit("")
		searchLayout.addWidget(self.filterLineEdit)
		self.filterLineEdit.textChanged.connect(self.onFilterLineEdit)

		'''
		self.searchPushButton = QPushButton("Rechercher")
		searchLayout.addWidget(self.searchPushButton)
		'''

		searchWidget.setLayout(searchLayout)
		leftLayout.addWidget(searchWidget)

		# ---- File Form
		self.createFormGroupBox()
		leftLayout.addWidget(self.formGroupBox)

		# ---- File Form buttons
		buttonWidget = QWidget()
		buttonLayout = QHBoxLayout()

		self.validPushButton = QPushButton("Valider")
		buttonLayout.addWidget(self.validPushButton)
		self.validPushButton.clicked.connect(self.onValidPushButton)

		delPushButton = QPushButton("Supprimer")
		buttonLayout.addWidget(delPushButton)
		copyPushButton = QPushButton("Copier")
		buttonLayout.addWidget(copyPushButton)
		pastePushButton = QPushButton("Coller")
		buttonLayout.addWidget(pastePushButton)
		buttonWidget.setLayout(buttonLayout)

		leftLayout.addWidget(buttonWidget)

		# Preview
		rightLayout = QVBoxLayout()
		rgbR = int(self.param.getValue( 'app', 'previewRGBR' ))
		rgbG = int(self.param.getValue( 'app', 'previewRGBG' ))
		rgbB = int(self.param.getValue( 'app', 'previewRGBB' ))
		self.previewLabel = PhotoViewer(self, QColor(rgbR,rgbG,rgbB))
		rightLayout.addWidget(self.previewLabel)

		# Main layout
		mainLayout = QHBoxLayout()

		leftWidget = QWidget()
		leftWidget.setLayout(leftLayout)
		rightWidget = QWidget()
		rightWidget.setLayout(rightLayout)

		self.splitter = QSplitter(Qt.Horizontal)
		self.splitter.addWidget(leftWidget)
		self.splitter.addWidget(rightWidget)

		mainLayout.addWidget(self.splitter)
		self.setLayout(mainLayout)

		self.splitter.setHandleWidth(15)

		#splitter.setSizes([splitter.size().height() * 0.3, splitter.size().height() * 0.7])
		#splitter.setSizes([100,400])
		#splitter.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
		#splitter.setOpaqueResize(True)
		#splitter.setChildrenCollapsible(True)
		#splitter.setStretchFactor( 1, 0 )

		print("END ui")

	def activeApp( self) :

		self.exec()

	def showEvent(self, event):

		print("--- Event : showEvent")

		try:
			if self.currentFile is None :
				self.currentFile = self.param.getValue( 'app', 'currentpath' )
			if os.path.isfile( self.currentFile ) :
				self.gotoPath( self.currentFile )
				self.previewLabel.fitInView()
				self.scrollToSelectedFile()
			else :
				print("Fichier inexistant : ["+str( self.currentFile)+"]")

		except Exception as e:
			print("ERR showEvent currentFile ")
			print(str(e))

	def closeEvent(self, event):

		print("CLOSE EVENT")

		self.saveStates()

	def saveStates(self) :
		self.param.setValue("app", "geometry", self.saveGeometry())
		self.param.setValue("app", "splitterstate", self.splitter.saveState())
		self.param.setValue("app", "treegeometry", self.tree.header().saveState())
		self.param.setValue("app", "currentpath", self.currentFile)

	def restoreStates(self) :
		try:
			self.restoreGeometry( self.param.getValue( 'app', 'geometry' ) )
		except:
			print("ERR restoreStates geometry")

		try:
			self.splitter.restoreState( self.param.getValue( 'app', 'splitterstate' ) )
		except:
			print("ERR restoreStates splitterstate")

		try:
			self.tree.header().restoreState( self.param.getValue( 'app', 'treegeometry' ) )
		except:
			print("ERR restoreStates treegeometry")

	def onFilterLineEdit(self, newtext) :
		
		self.proxy_model.setFilterValue(newtext)

		# Refresh
		self.model.setRootPath("")
		self.model.setRootPath(self.initPath) #  The model have a rootPath that tells them which part of the model monitor changes.

	def onDirectoryLoaded(self, path ):
		#print("onDirectoryLoaded : ", path)
		pass

	def traverseDirectories(self, parentindex, level=0, expandMode=True):

		spaces = ""
		for i in range(level): spaces = spaces + "  "

		path = self.model.filePath(parentindex)

		self.expandPath( path, expandMode )

		for subdirname in os.listdir(path) :
			fullSubdirName = os.path.join(path, subdirname)

			if os.path.isdir(fullSubdirName) :
				subdirIndex =  self.getModelIndexFromPath(fullSubdirName)
				self.traverseDirectories(subdirIndex, level + 1, expandMode)

	def onCollapseAllNodes( self ) :
		modelindex = self.model.index(self.initPath)
		self.traverseDirectories( modelindex, level=0, expandMode=False )

	def onExpandAllNodes( self ) :
		modelindex = self.model.index(self.initPath)
		self.traverseDirectories( modelindex, level=0, expandMode=True )

	def onCollapseAllNodesExceptThis( self ):
		self.onCollapseAllNodes()
		self.onExpandCurrentNode()

	def onExpandCurrentNode( self) :
		print("Expand from ",self.currentDir)
		modelindex = self.model.index(self.currentDir)
		self.traverseDirectories( modelindex, level=0, expandMode=True )

	def onCollapseCurrentNode( self) :
		modelindex = self.model.index(self.currentDir)
		self.traverseDirectories( modelindex, level=0, expandMode=False )

	def onToggleFlat( self) :

		if self.toggleFlat is False :
			print("set flat")
			self.toggleFlat = True
			self.tree.setIndentation (0)
		else:
			print("set TREE")
			self.toggleFlat = False
			self.tree.setIndentation (20)

	def onOpenMenu(self, position):

		print("---onOpenMenu")

		selectedindexes = self.tree.selectedIndexes()

		nodeType = "?"

		for selectindex in selectedindexes:

			modelindex = self.proxy_model.mapToSource( selectindex )
			path = self.model.filePath(modelindex)

			if os.path.isfile(path) :
				nodeType = "File " + path
			if os.path.isdir(path) :
				nodeType = "Dir "  + path

		menu = QMenu()

		self.expandAllNodesAction = QAction("Ouvrir toutes les branches")
		menu.addAction(self.expandAllNodesAction)
		self.expandAllNodesAction.triggered.connect(self.onExpandAllNodes)

		self.collapseAllNodesAction = QAction("Fermer toutes les branches")
		menu.addAction(self.collapseAllNodesAction)
		self.collapseAllNodesAction.triggered.connect(self.onCollapseAllNodes)

		self.collapseAllNodesExceptThisAction = QAction("Fermer les autres braches")
		menu.addAction(self.collapseAllNodesExceptThisAction)
		self.collapseAllNodesExceptThisAction.triggered.connect(self.onCollapseAllNodesExceptThis)

		menu.addSeparator()
		self.expandCurrentNodeAction = QAction("Ouvrir cette branche")
		menu.addAction(self.expandCurrentNodeAction)
		self.expandCurrentNodeAction.triggered.connect(self.onExpandCurrentNode)

		self.collapseCurrentNodeAction = QAction("Fermer cette branche")
		menu.addAction(self.collapseCurrentNodeAction)
		self.collapseCurrentNodeAction.triggered.connect(self.onCollapseCurrentNode)

		menu.addSeparator()
		self.toggleFlatAction = QAction("Vue à plat/arborescente")
		menu.addAction(self.toggleFlatAction)
		self.toggleFlatAction.triggered.connect(self.onToggleFlat)

		menu.addSeparator()
		self.openDirAction = QAction("Ouvrir dans le dossier")
		menu.addAction(self.openDirAction)
		self.openDirAction.triggered.connect(self.onOpenDirAction)


		menu.addSeparator()
		self.copyFileNameAction = QAction("Copier le nom du fichier")
		menu.addAction(self.copyFileNameAction)
		self.copyFileNameAction.triggered.connect(self.onCopyFileNameAction)

		self.copyPathFileNameAction = QAction("Copier le nom complet du fichier")
		menu.addAction(self.copyPathFileNameAction)
		self.copyPathFileNameAction.triggered.connect(self.onCopyPathFileNameAction)


		menu.exec_(self.tree.viewport().mapToGlobal(position))

	def onCopyPathFileNameAction( self) :
		QApplication.clipboard().setText(self.currentFile)

	def onCopyFileNameAction( self) :
		QApplication.clipboard().setText(os.path.basename(self.currentFile))

	def onOpenDirAction(self) :

		p = self.currentFile

		self.fileexplorer = self.param.getValue( 'explorer', 'command' )

		if platform.system() == 'Darwin':       # macOS
			os.system( self.fileExplorer + ' "' + p + '"')
		elif platform.system() == 'Windows':    # Windows
			p = os.path.dirname(p)
			os.system( self.fileExplorer + ' "' + p + '"')
		else:                                   # linux variants
			os.system( self.fileExplorer + ' "' + p + '"')

	def gotoPath( self, lastFullPath ) :

		print("--- START gotoPath")

		index = self.getModelIndexFromPath(lastFullPath)

		# Expand tree subdirs
		print("    Expand tree subdirs")
		self.expandPath( lastFullPath, True )

		# Highlight row
		# print("    Highlight row")
		#self.tree.selectionModel().select(index, QItemSelectionModel.ClearAndSelect | QItemSelectionModel.Rows)

		# Set focus on seleted tree row
		print("    Set focus on seleted node")
		modelindex = self.model.index(self.currentFile)
		proxyindex = self.proxy_model.mapFromSource( modelindex )
		self.tree.setCurrentIndex(proxyindex)

		# Preview
		print("    Preview")
		QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
		self.previewLabel.setPhoto(QPixmap(self.currentFile))
		QApplication.restoreOverrideCursor()

		# Scroll treeview
		#QTimer.singleShot(800, lambda : self.scrollToSelectedFile())
		self.scrollToSelectedFile()

		print("END gotopath")

	def scrollToSelectedFile( self)  :
		print("---scrollToSelectedFile")
		print("file : ", self.currentFile)
		#index = self.getModelIndexFromPath(self.currentFile)
		index = self.tree.currentIndex()
		self.tree.scrollTo(index, QAbstractItemView.EnsureVisible)

	def getModelIndexFromPath( self, path ) :
		print("*** getModelIndexFromPath path=", path)
		modelindex = self.model.index(path)
		return modelindex

	def expandPath(self, filePath, expandMode = True):

		indexes = []

		modelindex = self.model.index(filePath)
		proxyindex = self.proxy_model.mapFromSource( modelindex )

		while proxyindex.isValid():
			indexes.insert(0, proxyindex)
			proxyindex = proxyindex.parent()

		for proxyindex in indexes:
			modelindex = self.proxy_model.mapToSource( proxyindex )
			path = self.model.filePath(modelindex)
			#print("expand path ", path)
			if expandMode is True :
				self.tree.expand(proxyindex)
			else :
				self.tree.collapse(proxyindex)

	def createFormGroupBox(self):

		self.formGroupBox = QGroupBox("")
		layout = QFormLayout()
		filePartList = self.param.getValue( 'fileparam', 'partslabel' )
		self.formTypeList = self.param.getValue( 'fileparam', 'partstype' )
		self.formValuesList = self.param.getValue( 'fileparam', 'partsvalues' ).split(",")
		self.formFields=[]
		self.formLabels=[]
		idxtype = 0
		for filePartLabel in filePartList :
			#print(filePartLabel)
			#print("Type = ", self.formTypeList[idxtype])

			if self.formTypeList[idxtype]== "Combo" :
				formField = QComboBox()
				formValuesListElemList = self.formValuesList[idxtype].split("/")
				for formValuesElem in formValuesListElemList :
					formValuesElemKeyVal = formValuesElem.split("-")
					formField.addItem( formValuesElemKeyVal[1], formValuesElemKeyVal[0] )

			if self.formTypeList[idxtype]== "Edit" :
				formField = QLineEdit()

			idxtype = idxtype + 1

			self.formFields.append(formField)
			formLabel = QLabel(filePartLabel)
			self.formLabels.append(filePartLabel)
			layout.addRow(formLabel, formField )

		self.formGroupBox.setLayout(layout)

	def enableFormFieldsConnect( self, enable=True) :

		idxtype = 0
		for formField in self.formFields :
			if enable is True :
				if self.formTypeList[idxtype]== "Edit" :
					formField.textChanged.connect(self.onFormFieldChanged)
				if self.formTypeList[idxtype]== "Combo" :
					formField.currentIndexChanged.connect(self.onCurrentIndexChanged)
			else :
				try :
					if self.formTypeList[idxtype]== "Edit" :
						formField.textChanged.disconnect()
					if self.formTypeList[idxtype]== "Combo" :
						formField.currentIndexChanged.disconnect()
				except :
					pass
			idxtype = idxtype + 1

	def isValidFileName( self ):

		#print("oldFileName = ", os.path.basename(self.currentFile))
		newFileName = self.getNewFileName()
		#print("newFileName = ", newFileName)

		fullNewFileName = os.path.join(self.currentDir , newFileName)
		print(fullNewFileName)

		if os.path.isfile(fullNewFileName) :
			return False

		return True

	def onFormFieldChanged(self, newtext) :

		self.validPushButton.setEnabled(True)

		sender = self.sender()
		if self.isValidFileName() is False :
			print("ERREUR nom de fichier")
			sender.setStyleSheet("background-color: rgba(255, 107, 107, 150);")
		else :
			sender.setStyleSheet("")


	def onCurrentIndexChanged(self, ix):
		self.validPushButton.setEnabled(True)

	def onFileSelected(self, index):

		print("--- Event : onFileSelected")

		indexes = self.tree.selectedIndexes()

		fileList = []
		for index in indexes:

			index = self.proxy_model.mapToSource(index)
			path = self.model.filePath(index)

			if os.path.isfile(path) :
				if path not in fileList :
					self.currentFile = path
					self.currentDir = os.path.dirname(path)
					fileList.append(path)
					self.formClearValues()

			if os.path.isdir(path) :
				self.currentFile = None
				self.currentDir = path
				self.formClearValues()

		for fullFileName in fileList :
			fileName = os.path.basename(fullFileName)
			print( "fileName=[" + str(fileName) + "]")

			# Preview
			QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
			self.previewLabel.setPhoto(QPixmap(fullFileName))
			QApplication.restoreOverrideCursor()

			self.formFileFill()

			print("Click current dir  = ", self.currentDir)
			print("Click current file = ", self.currentFile)

	def onTreeClicked( self, index) :

		print("--- Event : onTreeClicked")

		indexItem = self.model.index(index.row(), 0, index.parent())
		fileName = self.model.fileName(indexItem)
		filePath = self.model.filePath(indexItem)

		if os.path.isdir(filePath) is True :
			self.formClearValues()

	def onFileClicked(self, index ) :

		print("--- Event : onFileClicked")

		indexItem = self.model.index(index.row(), 0, index.parent())
		fileName = self.model.fileName(indexItem)
		filePath = self.model.filePath(indexItem)

		if os.path.isdir(filePath) :
			self.currentFile = None
			self.currentDir = filePath

			shortFileName = ""
			self.formGroupBox.setTitle( shortFileName )

		if os.path.isfile(filePath) :

			self.currentFile = filePath
			self.currentDir = os.path.dirname(filePath)

			self.formFileFill()

			# Preview
			QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
			self.previewLabel.setPhoto(QPixmap(self.currentFile))
			QApplication.restoreOverrideCursor()

		print("")
		print("currentDir = ", self.currentDir)
		print("currentFile = ", self.currentFile)

	def formClearValues(self):

		self.enableFormFieldsConnect(False)

		idxtype = 0
		for formInput in self.formFields :
			if self.formTypeList[idxtype]== "Edit" :
				formInput.setText("")
			if self.formTypeList[idxtype]== "Combo" :
				formInput.setCurrentIndex(formInput.findData("X"))
			idxtype = idxtype  + 1

		self.formGroupBox.setTitle("")

		QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
		self.previewLabel.setPhoto(QPixmap())
		QApplication.restoreOverrideCursor()

		self.validPushButton.setEnabled(False)

		self.enableFormFieldsConnect()

	def formFileFill( self) :

		self.enableFormFieldsConnect(False)

		shortFileName = os.path.basename(self.currentFile)

		pixmap = QPixmap(self.currentFile)
		pixmapHeight = pixmap.height()
		pixmapWidth = pixmap.width()
		#filesize = self.sizeof_fmt( os.path.getsize(self.currentFile) )


		db = Sqlitedb( self.dbFile )
		dbSourceCount, dbPersonCount, dbEventCount, dbDocumentCount, dbMediaCount, dbPlaceCount, dbDepotCount = self.countDbDocument( db, shortFileName ) 
		dbCountTotal = dbSourceCount + dbPersonCount+ dbEventCount+ dbDocumentCount+ dbMediaCount+ dbPlaceCount+ dbDepotCount

		self.formGroupBox.setTitle( shortFileName + " ("+ str(pixmapWidth) + "x"+str(pixmapHeight) + ") [ ref:"+str(dbCountTotal) + "]")

		shortFileName = os.path.basename(self.currentFile)

		strPart = shortFileName.split('.')
		strList = strPart[0].split(self.param.getValue( 'fileparam' , 'sep') )
		i = 0
		maxi = len(self.formFields)
		extratext = ""

		for strElem in strList :
			formPart = self.getFormElemFromIndex(i)

			if formPart is not None and i <= maxi :
				try:
					#print("Form #" + str(i) + " : " + str(formPart) + " : " + str(strElem))
					if self.formTypeList[i]== "Edit" :
						#print("Edit set.")
						self.formFields[i].setText(str(strElem))
					if self.formTypeList[i]== "Combo" :
						self.formFields[i].setCurrentIndex(self.formFields[i].findData(str(strElem)))

					i=i+1

				except:
					# Filename has extra text
					print("Filename has extra text")
					break
			else :
				#if str(strElem) != '' :
				print("Extra text = " + str(strElem))
				self.formFields[maxi-1].setText( str(self.formFields[maxi-1].text()) + self.param.getValue( 'fileparam' , 'sep') + str(strElem))

		self.validPushButton.setEnabled(False)

		self.enableFormFieldsConnect()

	def getFormElemFromIndex( self, index ) :

		i = 0
		for formPart in self.param.getValue( 'fileparam', 'partslabel' ) :
			if i == index :
				return formPart
			i = i + 1

		return None

	def sizeof_fmt(self, num, suffix='B'):

		for unit in ['','Ki','Mi','Gi','Ti','Pi','Ei','Zi']:
			if abs(num) < 1024.0:
				return "%3.1f%s%s" % (num, unit, suffix)
			num /= 1024.0
		return "%.1f%s%s" % (num, 'Yi', suffix)

	def onValidPushButton(self) :

		oldFileName = os.path.basename(self.currentFile)
		newFileName = self.getNewFileName()

		print("oldFileName = ", oldFileName)
		print("newFileName = ", newFileName)

		# Text difference in color
		diffText = self.show_diff(oldFileName, newFileName)
		
		db = Sqlitedb( self.dbFile )
		db.isolation_level = None

		dbSourceCount, dbPersonCount, dbEventCount, dbDocumentCount, dbMediaCount, dbPlaceCount, dbDepotCount = self.countDbDocument( db, oldFileName ) 

		if self.isValidFileName() is False :
			print("ERR : file exists ", newFileName)
			#self.messageDialog("ERREUR", "Le fichier existe déjà :" + str(newFileName))
			confirmDialog = CustomDialog("Erreur", "Le fichier existe déjà :" + str(newFileName), 1, buttonOk=True, ButtonCancel=False)
			confirmDialog.exec()
			return

		userMessage = "<b>Fichier (ancien nom, nouveau nom)</b> :<br>" + str(oldFileName) + "<br>" + str(diffText) +\
		"<hr><b>Mise à jour en base de données (tables)</b> :<br>" +\
		"Source : " + str(dbSourceCount) + "<br>" +\
		"Person : " + str(dbPersonCount) + "<br>" +\
		"Event : " + str(dbEventCount) + "<br>" +\
		"Document : " + str(dbDocumentCount) + "<br>" +\
		"Media : " + str(dbMediaCount) + "<br>" +\
		"Place : " + str(dbPlaceCount) + "<br>" +\
		"Depot : " + str(dbDepotCount)


		print("Confirm dialog creation")

		confirmDialog = CustomDialog("Confirmez la mise à jour", userMessage )
		confirmDialog.setToolTipMessage( str(oldFileName) +"\n" + str(newFileName) )
		r = confirmDialog.exec()
		if r == QDialog.Rejected : return

		try:

			directory = os.path.dirname(self.currentFile)

			srcFile = os.path.join(directory, oldFileName)
			dstFile = os.path.join(directory, newFileName)

			print("Src=", srcFile)
			print("Dst=", dstFile)

			# Rename main file
			os.rename(srcFile, dstFile)

			# Rename sibbling files
			fileRenameList = self.param.getValue( 'app', 'filerename' )			
			for fileRenameExt in fileRenameList :
				if os.path.exists( srcFile+fileRenameExt) : 
					print("Rename : " + srcFile+fileRenameExt + " -> " + dstFile+fileRenameExt)
					os.rename(srcFile+fileRenameExt, dstFile+fileRenameExt)

			with db._conn:
				db.execute("begin")
				# Do NOT use 'executescript'

				dbSources = Sources( db )
				r = dbSources.updateCustomImageFileName( oldFileName, newFileName )
				print("Result r=", r)
				if r is False :
					raise Exception("Update Source : " + str(dbSources.lastError))

				r = dbSources.updateCustomIconFileName( oldFileName, newFileName )
				print("Result r=", r)
				if r is False :
					raise Exception("Update Source icon : " + str(dbSources.lastError))

				dbPersons = Persons( db )
				r = dbPersons.updateCustomImageFileName( oldFileName, newFileName )
				print("Result r=", r)
				if r is False :
					raise Exception("Update Persons : " + str(dbSources.lastError))

				dbDocuments = Documents( db )
				r = dbDocuments.updateCustomIconFileName( oldFileName, newFileName )
				print("Result r=", r)
				if r is False :
					raise Exception("Update Documents : " + dbSources.lastError)

				dbMedias = Medias( db )
				r = dbMedias.updateCustomUrlFileName( oldFileName, newFileName )
				print("Result r=", r)
				if r is False :
					raise Exception("Update Media : " + dbSources.lastError)

				dbPlaces = Places( db )
				r = dbPlaces.updateCustomIconFileName( oldFileName, newFileName )
				print("Result r=", r)
				if r is False :
					raise Exception("Update Place : " + dbSources.lastError)

				dbDepots = Depots( db )
				r = dbDepots.updateCustomIconFileName( oldFileName, newFileName )
				print("Result r=", r)
				if r is False :
					raise Exception("Update Depot : " + dbSources.lastError)

				db.execute("commit")
				print("commit done.")

			self.validPushButton.setEnabled(False)

			self.currentFile = dstFile
			self.gotoPath( self.currentFile )
			self.formFileFill()

			confirmDialog = CustomDialog("Mise à jour", "Mise à jour effectuée.", 1, buttonOk=True, ButtonCancel=False)
			confirmDialog.exec()

		except Exception as e:
			
			db.execute("rollback")
			os.rename(dstFile, srcFile)
			fileRenameList = self.param.getValue( 'app', 'filerename' )			
			for fileRenameExt in fileRenameList :
				if os.path.exists( dstFile+fileRenameExt) : 
					print("Rename : " + srcFile+fileRenameExt + " -> " + dstFile+fileRenameExt)
					os.rename(dstFile+fileRenameExt, srcFile+fileRenameExt)	
							
			print("**** ROLLBACK done **** ")
			
			confirmDialog = CustomDialog("Erreur", "Erreur lors de la mise à jour : \n\n" +str(e), 2, buttonOk=True, ButtonCancel=False)
			confirmDialog.exec()

	def countDbDocument(self, db, oldFileName ) :

		sourcesDbObj = SourcesDb( db )
		dbSourceCount = 0
		dbLines = sourcesDbObj.selectById( "where a.localurl like ?", [ "%"+oldFileName+"%" ])
		if dbLines is not False :
			dbSourceCount = len(dbLines)
			for dbLine in dbLines :
				print("update source id = ", dbLine.getId())

		personsDbObj = PersonsDb( db )
		dbPersonCount = 0
		dbLines = personsDbObj.selectById( "where a.icon like ?", [ "%"+oldFileName+"%" ])
		if dbLines is not False :
			dbPersonCount = len(dbLines)
			for dbLine in dbLines :
				print("update person id = ", dbLine.getId())

		eventsDbObj = EventsDb( db )
		dbEventCount = 0
		dbLines = eventsDbObj.selectById( "where a.icon like ?", [ "%"+oldFileName+"%" ])
		if dbLines is not False :
			dbEventCount = len(dbLines)
			for dbLine in dbLines :
				print("update events id = ", dbLine.getId())

		documentsDbObj = DocumentsDb( db )
		dbDocumentCount = 0
		dbLines = documentsDbObj.selectById( "where a.icon like ? or a.url like ?", [ "%"+oldFileName+"%", "%"+oldFileName+"%" ])
		if dbLines is not False :
			dbDocumentCount = len(dbLines)
			for dbLine in dbLines :
				print("update document id = ", dbLine.getId())

		mediasDbObj = MediasDb( db )
		dbMediaCount = 0
		dbLines = mediasDbObj.selectById( "where a.url like ?", [ "%"+oldFileName+"%" ])
		if dbLines is not False :
			dbMediaCount = len(dbLines)
			for dbLine in dbLines :
				print("update media id = ", dbLine.getId())

		placesDbObj = PlacesDb( db )
		dbPlaceCount = 0
		dbLines = placesDbObj.selectById( "where a.icon like ?", [ "%"+oldFileName+"%" ])
		if dbLines is not False :
			dbPlaceCount = len(dbLines)
			for dbLine in dbLines :
				print("update place id = ", dbLine.getId())

		depotsDbObj = DepotsDb( db )
		dbDepotCount = 0
		dbLines = depotsDbObj.selectById( "where a.icon like ?", [ "%"+oldFileName+"%" ])
		if dbLines is not False :
			dbDepotCount = len(dbLines)
			for dbLine in dbLines :
				print("update depot id = ", dbLine.getId())
		
		return dbSourceCount, dbPersonCount, dbEventCount, dbDocumentCount, dbMediaCount, dbPlaceCount, dbDepotCount

	def show_diff(self, text, n_text):
		"""
		http://stackoverflow.com/a/788780
		Unify operations between two compared strings seqm is a difflib.
		SequenceMatcher instance whose a & b are strings
		"""
		import difflib
		seqm = difflib.SequenceMatcher(None, text, n_text)
		output= []
		for opcode, a0, a1, b0, b1 in seqm.get_opcodes():
			if opcode == 'equal':
				output.append(seqm.a[a0:a1])
			elif opcode == 'insert':
				output.append("<font color=magenta>" + seqm.b[b0:b1] + "</font>")
			elif opcode == 'delete':
				output.append("<font color=lime>" + seqm.a[a0:a1] + "</font>")
			elif opcode == 'replace':
				# seqm.a[a0:a1] -> seqm.b[b0:b1]
				output.append("<font color=red>" + seqm.b[b0:b1] + "</font>")
			else:
				print("raise RuntimeError, unexpected opcode")
		return ''.join(output)

	def getNewFileName(self) :

		filename, file_extension = os.path.splitext(self.currentFile)

		newFileName=""

		sep = self.param.getValue( 'fileparam', 'sep' )
		maxtype = len(self.formFields)
		fileSep = ""
		idxtype = 0
		for formInput in self.formFields :
			if self.formTypeList[idxtype]== "Edit" :

				if idxtype+1 >= maxtype and formInput.text() == "" :
					pass
				else :

					newFileName = str(newFileName) + str(fileSep) + str(formInput.text())
					formValue = str(formInput.text())

				print("Add : #"+str(idxtype) + "/" + str(maxtype) + " :" + str(formValue))

			if self.formTypeList[idxtype]== "Combo" :
				newFileName = str(newFileName) + str(fileSep) + str(formInput.itemData(formInput.currentIndex()))
				formValue = formInput.itemData(formInput.currentIndex())

			#print("rename "+ str(idxtype) + " / " + str(maxtype) + " : [" + formValue + "]" + " " + str(newFileName) )
			idxtype = idxtype  + 1
			fileSep = sep

		return newFileName + file_extension

class CustomDialog(QDialog):

	def __init__(self, title="", message="", dialogtype=0, buttonOk=True, ButtonCancel=True, fontsize=14, parent = None):
		QDialog.__init__(self, parent)

		self.title = title
		self.message = message
		self.dialogtype = dialogtype
		self.fontSize = fontsize

		font = QFont()
		font.setPointSize(self.fontSize)
		self.setFont(font)

		self.setWindowTitle(self.title)

		# Message text
		self.messageWidget = QLabel(self.message)
		self.messageWidget.setFont(font)

		# Buttons
		self.buttonWidget = QWidget()
		buttonLayout = QGridLayout()

		if ButtonCancel is True :
			self.cancelButton = QPushButton("Annuler")
			self.cancelButton.clicked.connect(self.onButtonCancel)
			self.cancelButton.adjustSize()
			# https://www.pythonguis.com/faq/built-in-qicons-pyqt/
			pixmapi = QStyle.SP_DialogCancelButton
			icon = QApplication.style().standardIcon(pixmapi)
			self.cancelButton.setIcon(icon)
			buttonLayout.addWidget(self.cancelButton,0,0,Qt.AlignLeft)

		if buttonOk is True :
			self.validButton = QPushButton("Valider")
			self.validButton.clicked.connect(self.onButtonOk)
			self.validButton.adjustSize()
			pixmapi = QStyle.SP_DialogApplyButton
			icon = QApplication.style().standardIcon(pixmapi)
			self.validButton.setIcon(icon)
			buttonLayout.addWidget(self.validButton,0,1,Qt.AlignRight)

		self.buttonWidget.setLayout(buttonLayout)

		# Icon widget
		self.iconWidget = QLabel()

		iconStyle = QStyle.SP_MessageBoxQuestion
		if self.dialogtype == 0 : iconStyle = QStyle.SP_MessageBoxQuestion
		if self.dialogtype == 1 : iconStyle = QStyle.SP_MessageBoxInformation
		if self.dialogtype == 2 : iconStyle = QStyle.SP_MessageBoxWarning
		if self.dialogtype == 3 : iconStyle = QStyle.SP_MessageBoxCritical

		icon = QIcon( QApplication.style().standardIcon(iconStyle))
		pixmap = QPixmap(icon.pixmap(QSize(60, 60)))
		self.iconWidget.setPixmap(pixmap)
		#self.iconWidget.setScaledContents(True)

		# Main
		winWidget = QWidget()
		winLayout = QGridLayout()

		winLayout.addWidget(self.iconWidget,0, 0, 1, 1, Qt.AlignLeft)
		winLayout.addWidget(self.messageWidget,0, 1, 1, 1, Qt.AlignLeft)
		winLayout.addWidget(self.buttonWidget,1, 0, 1, 2, Qt.AlignRight)
		winWidget.setLayout(winLayout)

		self.setLayout(winLayout)
		self.resize(600, 90)
		self.adjustSize()

	def onButtonCancel(self, s):
		self.reject()

	def onButtonOk(self, s):
		self.accept()

	def setTitle( self, title) :
		self.setWindowTitle(title)

	def setTextMessage( self, message) :
		self.messageWidget.setText(message)

	def setToolTipMessage(self, tooltipMessage) :
		self.messageWidget.setToolTip(tooltipMessage)

class PhotoViewer(QGraphicsView):

	photoClicked = Signal(QPoint)

	def __init__(self, parent, QcolorBackground = QColor(90, 30, 30) ):
		super(PhotoViewer, self).__init__(parent)
		self._zoom = 0
		self._empty = True
		self._scene = QGraphicsScene(self)
		self._photo = QGraphicsPixmapItem()
		self._scene.addItem(self._photo)
		self.setScene(self._scene)
		self.setTransformationAnchor(QGraphicsView.AnchorUnderMouse)
		self.setResizeAnchor(QGraphicsView.AnchorUnderMouse)
		self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
		self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
		self.setBackgroundBrush(QBrush( QcolorBackground))
		self.setFrameShape(QFrame.NoFrame)

		self.setContextMenuPolicy(Qt.CustomContextMenu)
		self.customContextMenuRequested.connect(self.onOpenMenu)

	def setPhoto(self, pixmap=None):

		self._zoom = 0
		if pixmap and not pixmap.isNull():
			self._empty = False
			self.setDragMode(QGraphicsView.ScrollHandDrag)
			self._photo.setPixmap(pixmap)
		else:
			self._empty = True
			self.setDragMode(QGraphicsView.NoDrag)
			self._photo.setPixmap(QPixmap())

		self.fitInView()

	def hasPhoto(self):
		return not self._empty

	def mousePressEvent(self, event):
		if self._photo.isUnderMouse():
			self.photoClicked.emit(self.mapToScene(event.pos()).toPoint())
		super(PhotoViewer, self).mousePressEvent(event)

	def wheelEvent(self, event):
		"""
		Zoom in or out of the view.
		"""
		zoomInFactor = 1.25
		zoomOutFactor = 1 / zoomInFactor

		# Save the scene pos
		oldPos = self.mapToScene(event.pos())

		# Zoom
		if event.angleDelta().y() > 0:
			zoomFactor = zoomInFactor
		else:
			zoomFactor = zoomOutFactor
		self.scale(zoomFactor, zoomFactor)

		# Get the new position
		newPos = self.mapToScene(event.pos())

		# Move scene to old position
		delta = newPos - oldPos
		self.translate(delta.x(), delta.y())

	def onOpenMenu(self, position):

		menu = QMenu()

		self.rotateResetAction = QAction("Init")
		menu.addAction(self.rotateResetAction)
		self.rotateResetAction.triggered.connect(self.rotateReset)

		self.zoomZeroAction = QAction("Zoom zéro")
		menu.addAction(self.zoomZeroAction)
		self.zoomZeroAction.triggered.connect(self.setZoomZero)

		self.zoomFitAction = QAction("Zoom optimal")
		menu.addAction(self.zoomFitAction)
		self.zoomFitAction.triggered.connect(self.fitInView)

		self.rotateRightAction = QAction("Rotation à droite")
		menu.addAction(self.rotateRightAction)
		self.rotateRightAction.triggered.connect(self.rotateRight)

		self.rotateLeftAction = QAction("Rotation à gauche")
		menu.addAction(self.rotateLeftAction)
		self.rotateLeftAction.triggered.connect(self.rotateLeft)

		self.rotateRight90Action = QAction("Rotation de 90°")
		menu.addAction(self.rotateRight90Action)
		self.rotateRight90Action.triggered.connect(self.rotateRight90)

		menu.exec_(self.viewport().mapToGlobal(position))

	def toggleDragMode(self):
		if self.dragMode() == QGraphicsView.ScrollHandDrag:
			self.setDragMode(QGraphicsView.NoDrag)
		elif not self._photo.pixmap().isNull():
			self.setDragMode(QGraphicsView.ScrollHandDrag)

	def fitInView(self, scale=True):
		rect = QRectF(self._photo.pixmap().rect())
		if not rect.isNull():
			self.setSceneRect(rect)
			if self.hasPhoto():
				print("fitInView")
				unity = self.transform().mapRect(QRectF(0, 0, 1, 1))
				self.scale(1 / unity.width(), 1 / unity.height())
				viewrect = self.viewport().rect()
				scenerect = self.transform().mapRect(rect)
				factor = min(viewrect.width() / scenerect.width(),
							 viewrect.height() / scenerect.height())
				self.scale(factor, factor)
			self._zoom = 0

	def setZoomFit(self) :

		self.fitInView(self._scene.sceneRect(), Qt.KeepAspectRatio)

	def setZoomZero(self) :

		self.resetTransform()
		self._zoom  = 0

	def rotateRight(self) :
		self.rotate(1)

	def rotateLeft(self) :
		self.rotate(-1)

	def rotateRight90(self) :
		self.rotate(90)

	def rotateReset(self) :
		self.resetTransform()
		self.fitInView()


if __name__ == '__main__':

	app = QApplication(sys.argv)

	ex = App( 	"main.ini" ,
				"data/genea.sqlite",
				"/media/laurent/MesDocuments/home/Documents/Perso/home - Généalogie/mediadbCOPIE",
				None
			)

	ex.activeApp()

	sys.exit(app.exec_())

