#!/usr/bin/env python3
# -*- coding: utf-8 -*-

'''

		Photo Manager

		07/07/2021 08:40

		lvardon@laposte.net - 2021

		Licence libre

'''

from datetime import datetime, date
import glob, os, time, sys, platform
import shutil
import re
from urllib.parse import quote
import configparser

from pathlib import Path
from PIL import Image

from PyQt5.QtWidgets import QDialog, QFileDialog, QWidget, QLabel, QHBoxLayout, QVBoxLayout, QListWidget, QGridLayout, QDialogButtonBox, QListWidgetItem, QCheckBox, QLineEdit, QPushButton, QMessageBox
from PyQt5.QtCore import Qt, pyqtSignal, pyqtSlot, QSettings
from PyQt5.QtGui import QPixmap
from PyQt5 import (QtWidgets, QtCore)
from PIL import Image

import lib.log as AppLog
from lib.log import LOG_ARGVAL,LOG_INFO,LOG_INFOSETTING,LOG_WARNING,LOG_ERROR,LOG_DEBUG,LOG_FATAL,LOG_DATA

from lib.util import Util
import lib.setting as AppSetting
from lib.project import Project

from libdb.event_source import Event_source, Event_sourcesDb
from lib.sqlitedb import Sqlitedb

from libapp.rapport import Rapport
from libapp.dotgraph import Dotgraph

from lib.project import Project

from libdbcustom.metadata import Metadatas


class templateManager():


	logDatas = True

	month_lib = {
		0:"", 1: "janvier", 2: "février", 3: "mars", 4: "avril", 5: "mai", 6: "juin", 7: "juillet", 8: "août",
		9: "septembre", 10: "octobre", 11: "novembre", 12: "décembre", 13: ""
	}

	day_lib = {
		0:"lundi", 1:"mardi", 2:"mercredi", 3:"jeudi", 4:"vendredi", 5:"samedi", 6:"dimanche"
	}

	date_format = {
		"" : "Date complète",
		"AAAA" : "Année seule",
		"MMAAAA" : "Mois et année"
	}

	date_event_accurate = {
		"" : {
				"" : "",
				"EXA" : "Le ",
				"ABT" : "Vers le ",
				"BEF" : "Avant le ",
				"AFT" : "Après le ",
				"EST" : "Estimé le ",
				"CAL" : "Calculé le ",
				"UNK" : "Date inconnue "
			},
		"AAAA" : {
				"" : "",
				"EXA" : "En ",
				"ABT" : "Vers ",
				"BEF" : "Avant ",
				"AFT" : "Après ",
				"EST" : "Estimé à ",
				"CAL" : "Calculé à ",
				"UNK" : "Date inconnue "
			},
		"MMAAAA" : {
				"" : "",
				"EXA" : "En ",
				"ABT" : "Vers ",
				"BEF" : "Avant ",
				"AFT" : "Après ",
				"EST" : "Estimé à ",
				"CAL" : "Calculé à ",
				"UNK" : "Date inconnue "
			}

	}

	source_quality = {
		"" : "_NONE_",
		"0": "source non fiable, estimation",
		"1": "source de fiabilité douteuse",
		"2": "source secondaire",
		"3": "source indiscutable"
	}

	place_accurate = {
		"" : "_NONE_",
		"EXA": "exactement",
		"ENV": "approximativement"
	}

	cbPrivate = None

	projSetting = None
	outDirMediaName = None
	matchTitle = None
	matchDes = None
	tplDir = None

	log = None

	projMetaInfo = {}

	mediaTypesVideoList = ["mp4", "xvid"]
	mediaTypesAudioList = ["flac", "mp3", "ogg"]
	mediaTypesPhotoList = ["png", "jpg", "tiff"]
	mediaTypesDocList = ["pdf"]

	tempdir = "temp"

	pageAddUrl = None

	def __init__(self):
		AppLog.log( LOG_INFO , "templateManager init" )

		self.loadSettings( )

	def printDebugData( self, *args ) :

		messagePrint=""
		for message in args:
			messagePrint = messagePrint + str(message)

		if self.logDatas :
			from inspect import getframeinfo, stack
			caller = getframeinfo(stack()[1][0])
			messageData = os.path.basename(caller.filename)+ ' ' + str(caller.lineno) + ' ' + messagePrint
			AppLog.log( LOG_DATA, messageData)

	def loadSettings(self) :

		iniConfigFile = "config/app.ini"

		settings=QSettings(iniConfigFile, QSettings.IniFormat)

		self.tempdir = "temp"
		if settings.value('app/tempDir') is not None:
			self.tempdir = settings.value('app/tempDir')
		AppLog.log( LOG_INFO, "tempdir = " + str(self.tempdir))

		self.mediaTypesVideoList = "mp4,xvid"
		if settings.value('mediastypes/mediaTypesVideoList') is not None:
			self.mediaTypesVideoList = settings.value('mediastypes/mediaTypesVideoList')
		AppLog.log( LOG_INFO, "mediaTypesVideoList = " + str(self.mediaTypesVideoList))

		self.mediaTypesAudioList = "flac,mp3,ogg"
		if settings.value('mediastypes/mediaTypesAudioList') is not None:
			self.mediaTypesAudioList = settings.value('mediastypes/mediaTypesAudioList')
		AppLog.log( LOG_INFO, "mediaTypesAudioList = " + str(self.mediaTypesAudioList))

		self.mediaTypesPhotoList = "png, jpg, tiff"
		if settings.value('mediastypes/mediaTypesPhotoList') is not None:
			self.mediaTypesPhotoList = settings.value('mediastypes/mediaTypesPhotoList')
		AppLog.log( LOG_INFO, "mediaTypesPhotoList = " + str(self.mediaTypesPhotoList))

		self.mediaTypesDocList = "pdf"
		if settings.value('mediastypes/mediaTypesDocList') is not None:
			self.mediaTypesDocList = settings.value('mediastypes/mediaTypesDocList')
		AppLog.log( LOG_INFO, "mediaTypesDocList = " + str(self.mediaTypesDocList))

	def copy_and_overwrite( self, from_path, to_path):

		if os.path.exists(to_path):
			shutil.rmtree(to_path)
		try:
			shutil.copytree(from_path, to_path)
		except Exception as e:
			AppLog.log( LOG_ERROR, str(e) )

	# https://docs.python.org/fr/3/library/pathlib.html
	def makePath( self, *args ) :
		retPath = ""

		for pathElem in args:
			if retPath is not None :
				retPath = os.path.join ( retPath, str(pathElem) )

		if platform.system() == 'Darwin':       # macOS
			pass
		elif platform.system() == 'Windows':    # Windows
			retPath = retPath.replace("\\", "/")
		else: # linux variants
			pass

		return retPath

	def getMatchByPattern( self, dataIn, pattern ) :
		pattern = re.compile( pattern, flags=re.DOTALL + re.MULTILINE)
		matchList = re.findall( pattern, dataIn)
		if matchList :
			return matchList[0]

		return None

	'''
		Redirection selon le type de données à traiter en entrée
	'''
	def genRapport( self, dbFile, rootMedia, tplDir, srcFile, outFile, cbPrivate, traceDatas, projSetting ) :

		self.cbPrivate = cbPrivate
		self.traceDatas = traceDatas

		fpi = open( srcFile, 'r', encoding='utf8')
		tplFileContentR = fpi.read()
		fpi.close()

		self.projSetting = projSetting
		self.tplDir = tplDir

		matchDatas = re.search("#DATAS=(.*)", tplFileContentR)
		if matchDatas is not None:
			matchDatas = matchDatas[1]
		else:
			matchDatas = ""
		AppLog.log( LOG_INFO , "Type d'export : matchDatas = " + str(matchDatas) )

		if matchDatas == "PERSONS" :
			self.genRapportPersons(dbFile, rootMedia, tplDir, srcFile, outFile, cbPrivate, projSetting)

		if matchDatas == "MAPEVENTS" :
			self.genRapportMapEvents(dbFile, rootMedia, tplDir, srcFile, outFile, cbPrivate, projSetting)

		if matchDatas == "DOTSIMPLE" :
			self.genRapportMariageTree(dbFile, rootMedia, tplDir, srcFile, outFile, cbPrivate, projSetting)

		if matchDatas == "DOT" :
			self.genRapportTree(dbFile, rootMedia, tplDir, srcFile, outFile, cbPrivate, projSetting)

		if matchDatas == "ASCEN" :
			self.genRapportAscendant(dbFile, rootMedia, tplDir, srcFile, outFile, cbPrivate, projSetting)

		if matchDatas == "INDEX" :
			self.genRapportIndex(dbFile, rootMedia, tplDir, srcFile, outFile, cbPrivate, projSetting)

	def genRapportPageAdd( self, dirExport ) :

		if self.pageAddUrl is None : return
		if self.pageAddUrl == "" : return
		if os.path.isfile(self.pageAddUrl) is False : return

		AppLog.log( LOG_INFO , "Pageadd. pageAddUrl = " + str(self.pageAddUrl) )
		AppLog.log( LOG_INFO , "Pageadd. dirExport = " + str(dirExport) )

		try:
			os.mkdir( dirExport )
		except Exception as e:
			pass

		# Copy main file to root dst, insert menu, clean tags
		dstPage = self.makePath( dirExport, os.path.basename(self.pageAddUrl))
		AppLog.log( LOG_INFO , "Pageadd. dst page = ", dstPage)

		fpi = open( self.makePath( self.pageAddUrl ), 'r', encoding='utf8')
		templateContent = fpi.read()
		fpi.close()

		templateContent = self.databaseMetaInfoFiller( templateContent )

		currentMenuOptionId=""
		AppLog.log(LOG_INFO, "Pageadd. Insert menu : ", currentMenuOptionId)
		templateContent = self.insertMenu( templateContent, currentMenuOptionId)
		templateContent = self.cleanFinalData( templateContent )

		templateContent = self.cleanTemplateMetas(templateContent)

		fpo = open( dstPage , 'w', encoding='utf8')
		fpo.write( templateContent )
		fpo.close()

		# Copy subdirs datas
		(pageAddDirSrc, outFileExt) = os.path.splitext(self.pageAddUrl)
		pageAddDirDst = self.makePath( dirExport , os.path.basename(pageAddDirSrc) )
		AppLog.log( LOG_INFO , "Pageadd. copytree : ", pageAddDirSrc, "-->",pageAddDirDst)
		shutil.copytree(pageAddDirSrc, pageAddDirDst)

	def getFileTypeIcon (self, filename ) :

		if filename is None :
			return '_NONE_'

		if filename[0:4] == 'http' :
			return 'http.png'

		file_name, file_extension = os.path.splitext(filename)
		file_extension = file_extension[1:] # remove prefix dot

		if file_extension in self.mediaTypesAudioList :
			return 'audio.png'

		if file_extension in self.mediaTypesVideoList :
			return 'video.png'

		if file_extension in self.mediaTypesPhotoList :
			return 'photo.png'

		return 'default.png'

	def getFileType (self, filename ) :

		if filename is None :
			return '_NONE_'

		if filename[0:4] == 'http' :
			return 'http',""

		file_name, file_extension = os.path.splitext(filename)
		file_extension = file_extension[1:] # remove prefix dot

		# .flac, .mp3, .ogg
		if file_extension in self.mediaTypesAudioList :
			return 'audio', file_extension

		# 'mp4' ,xvid
		if file_extension in self.mediaTypesVideoList :
			return 'video', file_extension

		# jpg, png
		if file_extension in self.mediaTypesPhotoList :
			return 'photo', file_extension

		# pdf
		if file_extension in self.mediaTypesDocList :
			return 'pdf', file_extension

		return 'default', file_extension

	def databaseMetaInfoInit( self, projSetting, cbPrivate ) :

		self.cbPrivate = cbPrivate

		from libapp.dbmetaInformation import DbMetaInformation
		db = Sqlitedb( projSetting.load((AppSetting.get('app/lastProject', ''))).dbFile )
		DbMetaObj = DbMetaInformation( db, self.cbPrivate )

		self.projMetaInfo['projecttitle'] = projSetting.load((AppSetting.get('app/lastProject', ''))).title
		self.projMetaInfo['projectdescription'] = projSetting.load((AppSetting.get('app/lastProject', ''))).description
		self.projMetaInfo['projectauthorname'] = projSetting.load((AppSetting.get('app/lastProject', ''))).authorName
		self.projMetaInfo['projectauthormail'] = projSetting.load((AppSetting.get('app/lastProject', ''))).authorMail

		self.projMetaInfo['projectdbFile'] = projSetting.load((AppSetting.get('app/lastProject', ''))).dbFile
		rootMEdiaDir = projSetting.load((AppSetting.get('app/lastProject', ''))).rootMedia
		self.projMetaInfo['projectrootmedia'] = rootMEdiaDir
		utilObj = Util()
		rootMediaDirSize, rootMediaDirSizeH = utilObj.get_size( rootMEdiaDir )
		self.projMetaInfo['rootmediasize'] = rootMediaDirSize
		self.projMetaInfo['rootmediasizehuman'] = rootMediaDirSizeH

		self.projMetaInfo['datetimecreatedmax'] = self.formatDateTimeHuman( DbMetaObj.selectDateCreatedMax() )
		self.projMetaInfo['indicount'] = DbMetaObj.selectIndiCount()
		self.projMetaInfo['indimax'] = DbMetaObj.selectIndiMax()

		self.projMetaInfo['eventcount'] = DbMetaObj.selectEventCount()
		self.projMetaInfo['eventmax'] = DbMetaObj.selectEventMax()
		self.projMetaInfo['sourcecount'] = DbMetaObj.selectSourceCount()
		self.projMetaInfo['sourcemax'] = DbMetaObj.selectSourceMax()
		self.projMetaInfo['documentcount'] = DbMetaObj.selectDocumentCount()
		self.projMetaInfo['documentmax'] = DbMetaObj.selectDocumentMax()
		self.projMetaInfo['mediacount'] = DbMetaObj.selectMediaCount()
		self.projMetaInfo['mediamax'] = DbMetaObj.selectMediaMax()

		self.projMetaInfo['eventdatemin'] = self.formatDateHuman( DbMetaObj.selectEventDateMin() )
		self.projMetaInfo['eventdatemax'] = self.formatDateHuman( DbMetaObj.selectEventDateMax() )
		self.projMetaInfo['famillycount'] = DbMetaObj.selectFamillyCount()

		mediasWithoutPersonCount, mediasWithoutPerson = DbMetaObj.selectMediasWithoutPerson()
		self.projMetaInfo['mediaswithoutperson'] = mediasWithoutPerson
		self.projMetaInfo['mediaswithoutpersoncount'] = mediasWithoutPersonCount

		eventsWithoutSourceCount, eventsWithoutSource = DbMetaObj.selectEventsWithoutSource()
		self.projMetaInfo['eventwithoutsource'] = eventsWithoutSource
		self.projMetaInfo['eventwithoutsourcecount'] = eventsWithoutSourceCount

		eventsWithoutDateCount, eventsWithoutDate = DbMetaObj.selectEventsWithoutDate()
		self.projMetaInfo['eventwithoutdate'] = eventsWithoutDate
		self.projMetaInfo['eventwithoutdatecount'] = eventsWithoutDateCount

		sourcesWithoutEventCount, sourcesWithoutEvent = DbMetaObj.selectSourcesWithoutEvent()
		self.projMetaInfo['sourcewithoutevent'] = sourcesWithoutEvent
		self.projMetaInfo['sourcewithouteventcount'] = sourcesWithoutEventCount

		personWithoutEventCount, personWithoutEvent = DbMetaObj.selectPersonWithoutEvent()
		self.projMetaInfo['personwithoutevent'] = personWithoutEvent
		self.projMetaInfo['personwithouteventcount'] = personWithoutEventCount

		diskMediaCount, diskMediaCountSkip, diskOrphanMediaCount, diskOrphanMediaList = DbMetaObj.getDiskOrphanMedias( projSetting.load((AppSetting.get('app/lastProject', ''))).rootMedia )
		self.projMetaInfo['diskorphanmediatable'] = diskOrphanMediaList
		self.projMetaInfo['diskmediacount'] = diskMediaCount
		self.projMetaInfo['diskorphanmediacount'] = diskOrphanMediaCount
		self.projMetaInfo['diskmediacountkip'] = diskMediaCountSkip

		mediaCount, orphanMediaCount, orphanMediaList = DbMetaObj.getOrphanMedias( projSetting.load((AppSetting.get('app/lastProject', ''))).rootMedia )
		self.projMetaInfo['orphanmediatable'] = orphanMediaList
		self.projMetaInfo['orphanmediacount'] = orphanMediaCount
		self.projMetaInfo['orphanmediacounttotal'] = mediaCount


		mediaCount, orphanIconCount, orphanIconList = DbMetaObj.getOrphanIcon( projSetting.load((AppSetting.get('app/lastProject', ''))).rootMedia )
		self.projMetaInfo['orphanicontable'] = orphanIconList
		self.projMetaInfo['orphaniconcount'] = orphanIconCount
		self.projMetaInfo['orphaniconcounttotal'] = mediaCount


		mediaCount, orphanSourceUrlCount, orphanSourceUrlList = DbMetaObj.getOrphanSourceUrl( projSetting.load((AppSetting.get('app/lastProject', ''))).rootMedia )
		self.projMetaInfo['orphansourceurltable'] = orphanSourceUrlList
		self.projMetaInfo['orphansourceurlcount'] = orphanSourceUrlCount
		self.projMetaInfo['orphansourceurlcounttotal'] = mediaCount

		noteSearchCount, noteSearchTable = DbMetaObj.getAllNoteSearch()
		self.projMetaInfo['notesearchtable'] = noteSearchTable
		self.projMetaInfo['notesearchcount'] = noteSearchCount

	def databaseMetaInfoFiller( self, tplStr ) :

		archiveDir = os.path.basename(self.outDirMediaName)
		utilObj = Util()
		archiveSize, archiveSizeH = utilObj.get_size( archiveDir )
		tplStr = tplStr.replace('{{archivesize}}', str(archiveSize))
		tplStr = tplStr.replace('{{archivesizehuman}}', str(archiveSizeH))
		tplStr = tplStr.replace('{{baseoutdirmedia}}', str(archiveDir))

		tplStr = tplStr.replace('{{datetimesys}}', datetime.now().strftime('%d/%m/%Y %H:%I'))
		tplStr = tplStr.replace('{{filetitle}}', self.matchTitle)
		tplStr = tplStr.replace('{{filedescription}}', self.matchDes)
		tplStr = tplStr.replace('{{outdirmedianame}}', self.outDirMediaName)
		tplStr = tplStr.replace('{{mediasubdir}}', os.path.basename(self.outDirMediaName))

		tplStr = tplStr.replace('{{archivedir}}', os.path.dirname(self.outDirMediaName))

		tplStr = tplStr.replace('{{projecttitle}}', str(self.projMetaInfo['projecttitle']) )
		tplStr = tplStr.replace('{{projectauthorname}}', str(self.projMetaInfo['projectauthorname']) )
		tplStr = tplStr.replace('{{projectauthormail}}', str(self.projMetaInfo['projectauthormail']) )
		tplStr = tplStr.replace('{{projectdescription}}', str(self.projMetaInfo['projectdescription']) )
		tplStr = tplStr.replace('{{projectdbFile}}', str(self.projMetaInfo['projectdbFile']) )
		tplStr = tplStr.replace('{{projectrootmedia}}', str(self.projMetaInfo['projectrootmedia']) )

		tplStr = tplStr.replace('{{rootmediasize}}', str(self.projMetaInfo['rootmediasize']) )
		tplStr = tplStr.replace('{{rootmediasizehuman}}', str(self.projMetaInfo['rootmediasizehuman']) )

		if self.cbPrivate is True :
			tplStr = tplStr.replace('{{privatecontent}}', "avec")
			tplStr = tplStr.replace('{{isnotprivatemessage}}', "_NONE_")
			tplStr = tplStr.replace('{{isprivatemessage}}', "")
		else :
			tplStr = tplStr.replace('{{privatecontent}}', "sans")
			tplStr = tplStr.replace('{{isnotprivatemessage}}', "")
			tplStr = tplStr.replace('{{isprivatemessage}}', "_NONE_")

		tplStr = tplStr.replace('{{datetimecreatedmax}}', str(self.projMetaInfo['datetimecreatedmax']) )
		tplStr = tplStr.replace('{{indicount}}',str(self.projMetaInfo['indicount']))
		tplStr = tplStr.replace('{{indimax}}', str(self.projMetaInfo['indimax']) )
		tplStr = tplStr.replace('{{eventcount}}', str(self.projMetaInfo['eventcount']) )
		tplStr = tplStr.replace('{{eventmax}}', str(self.projMetaInfo['eventmax']) )
		tplStr = tplStr.replace('{{sourcecount}}', str(self.projMetaInfo['sourcecount']) )
		tplStr = tplStr.replace('{{sourcemax}}', str(self.projMetaInfo['sourcemax']) )
		tplStr = tplStr.replace('{{documentcount}}', str(self.projMetaInfo['documentcount']) )
		tplStr = tplStr.replace('{{documentmax}}', str(self.projMetaInfo['documentmax']) )
		tplStr = tplStr.replace('{{mediacount}}', str(self.projMetaInfo['mediacount']) )
		tplStr = tplStr.replace('{{mediamax}}', str(self.projMetaInfo['mediamax']) )
		tplStr = tplStr.replace('{{eventdatemin}}', str(self.projMetaInfo['eventdatemin']) )
		tplStr = tplStr.replace('{{eventdatemax}}', str(self.projMetaInfo['eventdatemax']) )
		tplStr = tplStr.replace('{{famillycount}}', str(self.projMetaInfo['famillycount']) )

		tplStr = tplStr.replace('{{mediaswithoutperson}}', str(self.projMetaInfo['mediaswithoutperson']) )
		tplStr = tplStr.replace('{{mediaswithoutpersoncount}}', str(self.projMetaInfo['mediaswithoutpersoncount']) )

		tplStr = tplStr.replace('{{eventwithoutsource}}', str(self.projMetaInfo['eventwithoutsource']) )
		tplStr = tplStr.replace('{{eventwithoutsourcecount}}', str(self.projMetaInfo['eventwithoutsourcecount']) )

		tplStr = tplStr.replace('{{eventwithoutdate}}', str(self.projMetaInfo['eventwithoutdate']) )
		tplStr = tplStr.replace('{{eventwithoutdatecount}}', str(self.projMetaInfo['eventwithoutdatecount']) )

		tplStr = tplStr.replace('{{sourcewithoutevent}}', str(self.projMetaInfo['sourcewithoutevent']) )
		tplStr = tplStr.replace('{{sourcewithouteventcount}}', str(self.projMetaInfo['sourcewithouteventcount']) )

		tplStr = tplStr.replace('{{personwithoutevent}}', str(self.projMetaInfo['personwithoutevent']) )
		tplStr = tplStr.replace('{{personwithouteventcount}}', str(self.projMetaInfo['personwithouteventcount']) )

		tplStr = tplStr.replace('{{diskorphanmediatable}}', str(self.projMetaInfo['diskorphanmediatable']) )
		tplStr = tplStr.replace('{{diskmediacount}}', str(self.projMetaInfo['diskmediacount']) )
		tplStr = tplStr.replace('{{diskorphanmediacount}}', str(self.projMetaInfo['diskorphanmediacount']) )
		tplStr = tplStr.replace('{{diskmediacountkip}}', str(self.projMetaInfo['diskmediacountkip']) )

		tplStr = tplStr.replace('{{orphanmediatable}}', str(self.projMetaInfo['orphanmediatable']) )
		tplStr = tplStr.replace('{{mediacount}}', str(self.projMetaInfo['mediacount']) )
		tplStr = tplStr.replace('{{orphanmediacount}}', str(self.projMetaInfo['orphanmediacount']) )

		tplStr = tplStr.replace('{{orphanicontable}}', str(self.projMetaInfo['orphanicontable']) )
		tplStr = tplStr.replace('{{orphaniconcounttotal}}', str(self.projMetaInfo['orphaniconcounttotal']) )
		tplStr = tplStr.replace('{{orphaniconcount}}', str(self.projMetaInfo['orphaniconcount']) )

		tplStr = tplStr.replace('{{orphansourceurltable}}', str(self.projMetaInfo['orphansourceurltable']) )
		tplStr = tplStr.replace('{{orphansourceurlcounttotal}}', str(self.projMetaInfo['orphansourceurlcounttotal']) )
		tplStr = tplStr.replace('{{orphansourceurlcount}}', str(self.projMetaInfo['orphansourceurlcount']) )

		tplStr = tplStr.replace('{{notesearchtable}}', str(self.projMetaInfo['notesearchtable']) )
		tplStr = tplStr.replace('{{notesearchcount}}', str(self.projMetaInfo['notesearchcount']) )

		return tplStr

	def fileCopyAndReplace(self, fileIn, fileOut ) :

		f=open(fileIn, encoding='utf8')
		f1=open(fileOut,'w', encoding='utf8')
		for line in f.readlines():

			line = self.databaseMetaInfoFiller( line )
			f1.write(line)

		f.close()
		f1.close()

	def renderValue( self, value, defvalue='' ) :

		if value is None or value == '': rvalue = defvalue
		else : rvalue = str(value)

		return rvalue

	def rvalue( self , value ) :

		if value is None or value == '' : rvalue = '_NONE_'
		else : rvalue = str(value)

		return rvalue

	def renderIcoSex(self, sex) :

		icosex = "_NONE_"

		if sex == "M" :
			icosex = "ico-sexe-m.png"

		if sex == "F" :
			icosex = "ico-sexe-f.png"

		if sex == "I" :
			icosex = "ico-sexe-i.png"

		return icosex

	# Formater une Date Sqlite
	# Retourne : « jeudi 8 septembre 1875 »
	def formatDateHuman( self, dateStr, dateStrFormat = '%Y-%m-%d %H:%M') :

		if dateStr[0:10] == "0001-01-01" :
				return ""

		try :

			date_time_obj = datetime. strptime(dateStr, dateStrFormat )
			day_lib = self.day_lib[int(date_time_obj.weekday())]
			day = date_time_obj.day
			month_lib = self.month_lib[int(date_time_obj.month)]
			year = date_time_obj.year
			dt_h = 	day_lib +' '+ str(day) +' '+ self.month_lib[ int(date_time_obj.month) ] +' '+ str(year)

			return dt_h

		except Exception as e:
			AppLog.log( LOG_ERROR, "formatDateHuman : Erreur de format date = " + str(dateStr))

		return "formatDateHuman: err format ", dateStr

	# Formater une DateTime Sqlite
	# Retourne : « jeudi 8 septembre 1875 18:52»
	def formatDateTimeHuman( self, dateStr, dateStrFormat = '%Y-%m-%d %H:%M') :

		if dateStr == "0001-01-01 00:00" :
				AppLog.log( LOG_WARNING, "datestr est NULL = " + str(dateStr))
				return ""

		try :

			date_time_obj = datetime. strptime(dateStr, dateStrFormat )
			day_lib = self.day_lib[int(date_time_obj.weekday())]
			day = date_time_obj.day
			month_lib = self.month_lib[int(date_time_obj.month)]
			year = date_time_obj.year
			dt_h = 	day_lib +' '+ str(day) +' '+ self.month_lib[ int(date_time_obj.month) ] +' '+ str(year) + ' ' + str(date_time_obj.hour).rjust(2, '0')+ ':' + str(date_time_obj.minute).rjust(2, '0')

			return dt_h

		except Exception as e:
			AppLog.log( LOG_ERROR, "formatDateTimeHuman : Erreur de format date = " + str(dateStr))

		return "formatDateTimeHuman: err format ", dateStr

	def renderDate( self, datValue, dateFormat = None, datePrec = None, datValueFrom = None ):

		rDate = "_NONE_"
		rDateDay = "_NONE_"
		rDateDayLib = "_NONE_"
		rDateMonth = "_NONE_"
		rDateMonthLib = "_NONE_"
		rDateYear = "_NONE_"
		rDateMin = "_NONE_"
		rDateHour = "_NONE_"
		deltaYear = "_NONE_"

		if datePrec == "UNK" :
			return rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear,rDateHour, rDateMin, deltaYear

		if datValue == "0001-01-01 00:00" :
				return rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear,rDateHour, rDateMin, deltaYear

		try :
			if datValue is not None :
				rDate = datValue
				rDateDay = str(int(datValue[8:10])) # 0:2
				rDateMonth = str(int(datValue[5:7])) # 3:5
				rDateMonthLib = self.month_lib[int(datValue[5:7])] # 3:5
				rDateYear = datValue[0:4] # 6:10
				# HH:MM or HH:MM:ss
				if len(datValue) == 16 or len(datValue) == 19 :
					rDateMin = str(int(datValue[14:16]))
					rDateHour = str(int(datValue[11:13]))
					if rDateMin == "0" and rDateHour == "0" :
						rDateMin = "_NONE_"
						rDateHour = "_NONE_"

				AppLog.log( LOG_DEBUG, "renderDate year, month, day: ", int(rDateYear), ",", int(rDateMonth), ",",  int(rDateDay) )
				dayweek = date(int(rDateYear), int(rDateMonth), int(rDateDay)).weekday()
				AppLog.log( LOG_DEBUG, "renderDate dayweek: ", dayweek )

				rDateDayLib = self.day_lib[dayweek]

			AppLog.log( LOG_DEBUG, "renderDate datValueFrom: ", datValueFrom )
			# Delta year between dates
			if datValue is not None and datValueFrom is not None and datValueFrom != '':
				input_str = str(int(datValueFrom[8:10])) + "/" + str(int(datValueFrom[5:7])) + "/" + datValueFrom[0:4]
				dt_datebirth = datetime.strptime(input_str, '%d/%m/%Y')
				input_str = str(int(datValue[8:10])) + "/" + str(int(datValue[5:7])) + "/" + datValue[0:4]
				dt_datedeath = datetime.strptime(input_str, '%d/%m/%Y')
				delta = dt_datedeath - dt_datebirth
				deltaYear = str(int(delta.days/365.25))

		except Exception as e:
			AppLog.log( LOG_ERROR, "renderDate : Erreur de format date [" + str(datValue) + "] - " + str(e))

		if dateFormat == "AAAA" :
			rDate = "_NONE_"
			rDateDay = "_NONE_"
			rDateDayLib = "_NONE_"
			rDateMonth = "_NONE_"
			rDateMonthLib = "_NONE_"
			rDateMin = "_NONE_"
			rDateHour = "_NONE_"
			deltaYear = "_NONE_"

		if dateFormat == "MMAAAA" :
			rDate = "_NONE_"
			rDateDay = "_NONE_"
			rDateDayLib = "_NONE_"
			rDateMin = "_NONE_"
			rDateHour = "_NONE_"
			deltaYear = "_NONE_"

		return rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear,rDateHour, rDateMin, deltaYear

	def renderIcon( self, rootMedia, iconUrl, outDirMediaName, matchImageDir, indiId, defaultIconUrl, matchImageMaxWidth, matchImageMaxHeight ) :

		if matchImageMaxWidth == 0 or matchImageMaxHeight == 0 : return defaultIconUrl
		if matchImageMaxWidth == '' or matchImageMaxHeight == '' : return defaultIconUrl

		if iconUrl is None : iconUrl =''

		persontSrcIcon = self.makePath( rootMedia, iconUrl)
		personDstIcon = self.makePath( matchImageDir, str(indiId) )

		try:
			AppLog.log( LOG_INFO , "mkdir : " + str(personDstIcon) )
			os.makedirs( personDstIcon, exist_ok=True ) # -p equivalent
		except Exception as e:
			AppLog.log( LOG_ERROR, "renderIcon : Erreur : " + str(e))

		personDstIcon = self.makePath( personDstIcon, os.path.basename(iconUrl) )

		if persontSrcIcon != '' and persontSrcIcon is not None and os.path.isfile( persontSrcIcon ) :

			try:
				shutil.copyfile( persontSrcIcon , personDstIcon)
			except Exception as e:
				AppLog.log( LOG_ERROR, str(e) )

			# Make thumb (if params sizes are not None)
			if matchImageMaxWidth is not None and matchImageMaxHeight is not None :
				img = Image.open(personDstIcon)
				img.thumbnail((int(matchImageMaxWidth), int(matchImageMaxHeight)), Image.ANTIALIAS)
				img.save(personDstIcon)

			commonIconPath = os.path.commonpath( [outDirMediaName, personDstIcon] )
			personIconUrl = self.makePath( os.path.basename(outDirMediaName), personDstIcon[ len(commonIconPath)+1:])
		else :
			personIconUrl = self.makePath( os.path.basename(outDirMediaName), defaultIconUrl)

		personIconUrl = personIconUrl[ personIconUrl.find('/') + 1: ]

		if 1 == 0 :
			print("------------ render image -----------------")
			print(" IN rootMedia= ", rootMedia)
			print(" IN iconUrl= ", iconUrl)
			print(" IN outDirMediaName= ", outDirMediaName)
			print(" IN matchImageDir= ", matchImageDir)
			print(" IN indiId= ", indiId)
			print(" IN defaultIconUrl= ", defaultIconUrl)
			print(" IN matchImageMaxWidth= ", matchImageMaxWidth)
			print(" IN matchImageMaxHeight= ", matchImageMaxHeight)
			print(" persontSrcIcon= ", persontSrcIcon)
			print("   ** File saved : personDstIcon ...: ", personDstIcon)
			print("   ** return personIconUrl..........: ", personIconUrl)

		return personIconUrl

	'''
	Render dot file : /home/.../Bureau/OUTAS/arbre/tree.dot
	Svg file (no ext) : /home/.../Bureau/OUTAS/arbre/tree
	Svg file: /home/.../Bureau/OUTAS/arbre/tree.svg
	'''
	def renderDotAsSvg( self, filenamedot ) :

		from graphviz import Source

		with open(filenamedot, 'r', encoding='utf8') as f:
			contents = f.read()
		AppLog.log( LOG_INFO , "Render dot file : " + str(filenamedot) )

		# Get file without ext
		svgfile = os.path.splitext( filenamedot)[0]
		AppLog.log( LOG_INFO , "Svg file (no ext) : "+  str(svgfile) )

		graph = Source(contents, format="svg")

		#graph.engine = 'neato'
		#graph.engine = 'fdp'

		# filenmaeResult == "svgfile" + extention .svg
		filenameresult = graph.render(filename = svgfile, view = 0)

		# unlink file "svgfile",  optional
		AppLog.log( LOG_INFO , "Svg file: " + str(filenameresult) )

		return filenameresult

	def initTargetFromTemplateMeta( self, tplFileContentR, tplDir, outFile )  :

		matchTitle = re.search("#TITRE=(.*)", tplFileContentR)
		if matchTitle is not None :
			matchTitle = matchTitle[1]
		else:
			matchTitle = ""
		AppLog.log( LOG_INFO , "Template Titre = " + str(matchTitle) )
		self.matchTitle = matchTitle

		matchDes = re.search("#DESCRIPTION=(.*)", tplFileContentR)
		if matchDes is not None :
			matchDes = matchDes[1]
		else:
			matchDes = ""
		AppLog.log( LOG_INFO , "Template Description = " + str(matchDes) )

		self.matchDes = matchDes

		matchFormat = re.search("#FORMAT=(.*)", tplFileContentR)
		if matchFormat is not None:
			matchFormat = matchFormat[1]
		else:
			matchFormat = ""
		AppLog.log( LOG_INFO , "Template Format = " + str(matchFormat) )

		matchSourcemediaFile = re.search("#SOURCEMEDIATEMPLATE=(.*)", tplFileContentR)
		if matchSourcemediaFile is not None:
			matchSourcemediaFile = matchSourcemediaFile[1]
		else:
			matchSourcemediaFile = ""
		AppLog.log( LOG_INFO , "Template SourcemediaFile = " + str(matchSourcemediaFile) )

		matchSourcemediaType = re.search("#SOURCEMEDIATYPE=(.*)", tplFileContentR)
		if matchSourcemediaType is not None:
			matchSourcemediaType = matchSourcemediaType[1]
		else:
			matchSourcemediaType = ""
		AppLog.log( LOG_INFO , "Template Sourcemedia = " + str(matchSourcemediaType) )

		matchSourcemediaWidth = re.search("#SOURCEMEDIAWIDTH=(.*)", tplFileContentR)
		if matchSourcemediaWidth is not None:
			try :
				matchSourcemediaWidth = int(matchSourcemediaWidth[1])
			except Exception as e:
				matchSourcemediaWidth = 0
		else:
			matchSourcemediaWidth = 0
		AppLog.log( LOG_INFO , "Template SourcemediaWidth = " + str(matchSourcemediaWidth) )

		matchSourcemediaHeight = re.search("#SOURCEMEDIAHEIGHT=(.*)", tplFileContentR)
		if matchSourcemediaHeight is not None:
			try :
				matchSourcemediaHeight = int(matchSourcemediaHeight[1])
			except Exception as e:
				matchSourcemediaHeight = 0
		else:
			matchSourcemediaHeight = 0
		AppLog.log( LOG_INFO , "Template SourcemediaHeight = " + str(matchSourcemediaHeight) )

		matchFiles = re.search("#COPYFILES=(.*)", tplFileContentR)
		if matchFiles is not None:
			matchFiles = matchFiles[1].split(",")
		else:
			matchFiles = 0
		AppLog.log( LOG_INFO , "Template matchFiles = " + str(matchFiles) )

		matchDirs = re.search("#COPYDIRS=(.*)", tplFileContentR)
		if matchDirs is not None:
			matchDirs = matchDirs[1].split(",")
		else:
			matchDirs = 0
		AppLog.log( LOG_INFO , "Template matchDirs = " + str(matchDirs) )

		matchDirsToRoot = re.search("#COPYDIRSTOROOT=(.*)", tplFileContentR)
		if matchDirsToRoot is not None:
			matchDirsToRoot = matchDirsToRoot[1].split(",")
		else:
			matchDirsToRoot = 0
		AppLog.log( LOG_INFO , "Template matchDirsToRoot = " + str(matchDirsToRoot) )

		# Create dir for medias
		outDirName = os.path.dirname(outFile)
		(outFileBaseName, outFileExt) = os.path.splitext(outFile)
		outDirMediaName = self.makePath(outDirName, outFileBaseName)
		try:
			os.mkdir( outDirMediaName )
		except Exception as e:
			pass
		AppLog.log( LOG_INFO , "Template outDirMediaName = " + str(outDirMediaName) )
		self.outDirMediaName = outDirMediaName

		# Copy  files
		for fileElem in matchFiles :
			if fileElem != '' :
				AppLog.log( LOG_INFO , "Copy file from "+ self.makePath(tplDir, fileElem)+ " -> "+ self.makePath(outDirMediaName, fileElem) )
				self.fileCopyAndReplace( self.makePath(tplDir, fileElem), self.makePath(outDirMediaName, fileElem) )

		# Copy template dirs
		for dirElem in matchDirs :
			if dirElem != '' :
				AppLog.log( LOG_INFO ,"Copy dir from "+ self.makePath(tplDir, dirElem)+ " => "+ self.makePath(outDirMediaName, dirElem) )
				self.copy_and_overwrite(self.makePath(tplDir, dirElem), self.makePath(outDirMediaName, dirElem))

		# Copy dirs to root
		for dirElem in matchDirsToRoot :
			if dirElem != '' :
				AppLog.log( LOG_INFO , "Template outDirMediaName = " + outDirMediaName )
				dstDir = self.makePath( os.path.dirname( outDirMediaName) , dirElem)
				if os.path.isdir( self.makePath(tplDir, dirElem) ) is True :
					AppLog.log( LOG_INFO ,"Copy to root. Dir from "+ self.makePath(tplDir, dirElem)+ " >> "+ dstDir )
					try:
						AppLog.log( LOG_INFO , "Create dir : " + dstDir )
						os.mkdir( dstDir )
					except Exception as e:
						AppLog.log( LOG_ERROR , "Create dir = " + dstDir )
					self.copy_and_overwrite(self.makePath(tplDir, dirElem), dstDir)

				if os.path.isfile( self.makePath(tplDir, dirElem) ) is True :
					dstFile = self.makePath( os.path.dirname( outDirMediaName) , dirElem)
					AppLog.log( LOG_INFO ,"Copy to root. File from "+ self.makePath(tplDir, dirElem)+ " >> "+ dstFile )
					shutil.copy(self.makePath(tplDir, dirElem), dstFile)


		matchImageDir = re.search("#IMAGESUBDIR=(.*)", tplFileContentR)
		if matchImageDir is not None:
			try :
				matchImageDir = matchImageDir[1].split(",")[0]
				# Create main dir for medias
				matchImageDir = self.makePath(outDirMediaName, matchImageDir)
				try:
					os.mkdir( matchImageDir )
				except Exception as e:
					pass
			except Exception as e:
				matchImageDir = 0
		else:
			matchImageDir = 0
		AppLog.log( LOG_INFO , "Template matchImageDir = " + matchImageDir )

		matchImageMaxHeight = re.search("#IMAGEMAXHEIGHT=(.*)", tplFileContentR)
		if matchImageMaxHeight is not None:
			try :
				matchImageMaxHeight = matchImageMaxHeight[1].split(",")[0]
			except Exception as e:
				matchImageMaxHeight = 0
		else:
			matchImageMaxHeight = 0
		AppLog.log( LOG_INFO , "Template matchImageMaxHeight = " + str(matchImageMaxHeight) )

		matchImageMaxWidth = re.search("#IMAGEMAXWIDTH=(.*)", tplFileContentR)
		if matchImageMaxWidth is not None:
			try :
				matchImageMaxWidth = matchImageMaxWidth[1].split(",")[0]
			except Exception as e:
				matchImageMaxWidth = 0

		else:
			matchImageMaxWidth = 0
		AppLog.log( LOG_INFO , "Template matchImageMaxWidth = " + str(matchImageMaxWidth) )

		return \
			outDirMediaName, \
			matchFormat, \
			matchSourcemediaFile, \
			matchSourcemediaType, \
			matchSourcemediaWidth, \
			matchSourcemediaHeight, \
			matchFiles, \
			matchDirs, \
			matchTitle, \
			matchDes, \
			matchImageDir, \
			matchImageMaxHeight, \
			matchImageMaxWidth


	def readTargetFromTemplateMeta( self, tplFileContentR )  :

		matchTitle = re.search("#TITRE=(.*)", tplFileContentR)
		if matchTitle is not None :
			matchTitle = matchTitle[1]
		else:
			matchTitle = ""
		AppLog.log( LOG_INFO , "Template Titre = " + str(matchTitle) )
		self.matchTitle = matchTitle

		matchDes = re.search("#DESCRIPTION=(.*)", tplFileContentR)
		if matchDes is not None :
			matchDes = matchDes[1]
		else:
			matchDes = ""
		AppLog.log( LOG_INFO , "Template Description = " + str(matchDes) )

		self.matchDes = matchDes

		matchFormat = re.search("#FORMAT=(.*)", tplFileContentR)
		if matchFormat is not None:
			matchFormat = matchFormat[1]
		else:
			matchFormat = ""
		AppLog.log( LOG_INFO , "Template Format = " + str(matchFormat) )

		matchSourcemediaFile = re.search("#SOURCEMEDIATEMPLATE=(.*)", tplFileContentR)
		if matchSourcemediaFile is not None:
			matchSourcemediaFile = matchSourcemediaFile[1]
		else:
			matchSourcemediaFile = ""
		AppLog.log( LOG_INFO , "Template SourcemediaFile = " + str(matchSourcemediaFile) )

		matchSourcemediaType = re.search("#SOURCEMEDIATYPE=(.*)", tplFileContentR)
		if matchSourcemediaType is not None:
			matchSourcemediaType = matchSourcemediaType[1]
		else:
			matchSourcemediaType = ""
		AppLog.log( LOG_INFO , "Template Sourcemedia = " + str(matchSourcemediaType) )

		matchSourcemediaWidth = re.search("#SOURCEMEDIAWIDTH=(.*)", tplFileContentR)
		if matchSourcemediaWidth is not None:
			try :
				matchSourcemediaWidth = int(matchSourcemediaWidth[1])
			except Exception as e:
				matchSourcemediaWidth = 0
		else:
			matchSourcemediaWidth = 0
		AppLog.log( LOG_INFO , "Template SourcemediaWidth = " + str(matchSourcemediaWidth) )

		matchSourcemediaHeight = re.search("#SOURCEMEDIAHEIGHT=(.*)", tplFileContentR)
		if matchSourcemediaHeight is not None:
			try :
				matchSourcemediaHeight = int(matchSourcemediaHeight[1])
			except Exception as e:
				matchSourcemediaHeight = 0
		else:
			matchSourcemediaHeight = 0
		AppLog.log( LOG_INFO , "Template SourcemediaHeight = " + str(matchSourcemediaHeight) )

		matchFiles = re.search("#COPYFILES=(.*)", tplFileContentR)
		if matchFiles is not None:
			matchFiles = matchFiles[1].split(",")
		else:
			matchFiles = 0
		AppLog.log( LOG_INFO , "Template matchFiles = " + str(matchFiles) )

		matchDirs = re.search("#COPYDIRS=(.*)", tplFileContentR)
		if matchDirs is not None:
			matchDirs = matchDirs[1].split(",")
		else:
			matchDirs = 0
		AppLog.log( LOG_INFO , "Template matchDirs = " + str(matchDirs) )

		matchDirsToRoot = re.search("#COPYDIRSTOROOT=(.*)", tplFileContentR)
		if matchDirsToRoot is not None:
			matchDirsToRoot = matchDirsToRoot[1].split(",")
		else:
			matchDirsToRoot = 0
		AppLog.log( LOG_INFO , "Template matchDirsToRoot = " + str(matchDirsToRoot) )

		matchImageDir = re.search("#IMAGESUBDIR=(.*)", tplFileContentR)
		if matchImageDir is not None:
			matchImageDir = matchImageDir[1].split(",")[0]
		else:
			matchImageDir = 0
		AppLog.log( LOG_INFO , "Template matchImageDir = " + matchImageDir )

		matchImageMaxHeight = re.search("#IMAGEMAXHEIGHT=(.*)", tplFileContentR)
		if matchImageMaxHeight is not None:
			try :
				matchImageMaxHeight = matchImageMaxHeight[1].split(",")[0]
			except Exception as e:
				matchImageMaxHeight = 0
		else:
			matchImageMaxHeight = 0
		AppLog.log( LOG_INFO , "Template matchImageMaxHeight = " + str(matchImageMaxHeight) )

		matchImageMaxWidth = re.search("#IMAGEMAXWIDTH=(.*)", tplFileContentR)
		if matchImageMaxWidth is not None:
			try :
				matchImageMaxWidth = matchImageMaxWidth[1].split(",")[0]
			except Exception as e:
				matchImageMaxWidth = 0
		else:
			matchImageMaxWidth = 0
		AppLog.log( LOG_INFO , "Template matchImageMaxWidth = " + str(matchImageMaxWidth) )

		return \
			matchFormat, \
			matchSourcemediaFile, \
			matchSourcemediaType, \
			matchSourcemediaWidth, \
			matchSourcemediaHeight, \
			matchFiles, \
			matchDirs, \
			matchTitle, \
			matchDes, \
			matchImageDir, \
			matchImageMaxHeight, \
			matchImageMaxWidth


	def cleanTemplateMetas( self, tplFileContentR ) :

		tplFileContentR = re.sub("#TITRE=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#DESCRIPTION=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#FORMAT=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#DATAS=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#SOURCEMEDIATEMPLATE=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#SOURCEMEDIA=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#SOURCEMEDIAWIDTH=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#SOURCEMEDIATYPE=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#SOURCEMEDIAHEIGHT=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#IMAGESUBDIR=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#IMAGEMAXHEIGHT=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#IMAGEMAXWIDTH=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#COPYDIRS=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#COPYFILES=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#COPYDIRSTOROOT=(.*[\r\n])", '', tplFileContentR)
		tplFileContentR = re.sub("#MENUADD=(.*[\r\n])", '', tplFileContentR)


		return tplFileContentR


	def cleanFinalData( self, tplFileContent) :

		#tplFileContent = tplFileContent.replace( globalMatchR, '')
		tplFileContent = tplFileContent.replace( '%%', '')

		# Supprimer les champs de valeur vides
		#_NONE_ : (?<=<prefix>)(?:.(?!<prefix>))*NONE.*(?=<suffix>)
		tplFileContent = re.sub('(?<=<ifnotnull>)(?:.(?!<ifnotnull>))*NONE.*(?=</ifnotnull>)', '', tplFileContent)
		#tplFileContent = re.sub('(?<=<ifnotnull>)(.|\n)*?(?=</ifnotnull>)', '', tplFileContent)

		tplFileContent = tplFileContent.replace('<ifnotnull>','')
		tplFileContent = tplFileContent.replace('</ifnotnull>', '')

		tplFileContent = tplFileContent.replace('_NONE_','')

		tplFileContent = re.sub('{{.*}}', '', tplFileContent)


		tplFileContent = tplFileContent.replace('££u','')
		tplFileContent = tplFileContent.replace('££','')

		return tplFileContent

	def copyTemplateFile( self, fileSrc, fileDst, currentMenuOptionId="") :

		AppLog.log( LOG_INFO, "copyTemplateFile : copy template from : " +str(fileSrc) + " -> "+ str(fileDst) )

		fpi = open( self.makePath( fileSrc ), 'r', encoding='utf8')
		templateContent = fpi.read()
		fpi.close()

		templateContent = self.databaseMetaInfoFiller( templateContent )

		AppLog.log(LOG_INFO, "Insert menu : ", currentMenuOptionId)
		templateContent = self.insertMenu( templateContent, currentMenuOptionId)
		templateContent = self.cleanFinalData( templateContent )

		fpo = open( self.makePath( fileDst ), 'w', encoding='utf8')
		fpo.write( templateContent )
		fpo.close()

		return templateContent

	'''
		Insertion du menu (tag {{menu}} )

		currentMenuOptionId = identifiant de l'option du menu à mettre en valeur (option de la page affichée)
	'''
	def insertMenu( self, templateContent, currentMenuOptionId=""  ) :

		fpi = open( self.makePath( self.tplDir ,"index/menu.html"), 'r', encoding='utf8')
		menuContent = fpi.read()
		fpi.close()

		if self.pageAddUrl is not None and self.pageAddUrl != "" and os.path.isfile(self.pageAddUrl) is True  :

			fpi = open( self.pageAddUrl, 'r', encoding='utf8')
			pageAddContent = fpi.read()
			fpi.close()
			pageAddMenu = re.search("#MENUADD=(.*)", pageAddContent)

			if pageAddMenu is not None:
				try :
					pageAddMenu = pageAddMenu[1]
				except Exception as e:
					pageAddMenu = "_NONE_"
			else:
				pageAddMenu = "_NONE_"

			menuContent = menuContent.replace("{{menuadd}}", pageAddMenu)

		# Pathroot value
		pathHome = re.search("#PATHROOT=(.*)", templateContent)
		if pathHome is not None:
			try :
				pathHome = pathHome[1]
			except Exception as e:
				pathHome = "_NONE_"
		else:
			pathHome = "_NONE_"
		templateContent = re.sub("#PATHROOT=(.*[\r\n])", '', templateContent)
		menuContent = menuContent.replace('{{pathhome}}', pathHome)

		# Menu current option (apply class "mainMenuCurrent"_current" to current option)
		menuCurrentOption = re.search("#MENUCURRENT=(.*)", templateContent)
		if menuCurrentOption is not None:
			try :
				menuCurrentOption = menuCurrentOption[1]
			except Exception as e:
				menuCurrentOption = currentMenuOptionId
		else:
			menuCurrentOption = currentMenuOptionId
		templateContent = re.sub("#MENUCURRENT=(.*[\r\n])", '', templateContent)
		menuContent = menuContent.replace('{{'+menuCurrentOption+'_current}}', "menutabcurrent")
		menuContent = re.sub('{{(.*)_current}}', 'menutab', menuContent)

		# Insert menu in template
		templateContent = templateContent.replace('{{menu}}', menuContent)
		templateContent = self.cleanFinalData( templateContent )

		return templateContent

	def genRapportIndex(self, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting) :

		AppLog.log( LOG_INFO , "rootMedia = " + str(rootMedia) )
		AppLog.log( LOG_INFO , "dbFile = " + str(dbFile) )

		db = Sqlitedb( dbFile )

		indicount = {}
		eventcount = {}
		sourcecount = {}
		doccount = {}

		indiNum = 0

		myRapport = Dotgraph( db, cbPrivate )
		myRapport.setPrivate( cbPrivate )

		# Read template
		tplDir = os.path.dirname(tplFile)
		fpi = open( tplFile, 'r', encoding='utf8')
		tplFileContentR = fpi.read()
		fpi.close()

		outDirMediaName, \
		matchFormat, \
		matchSourcemediaFile, \
		matchSourcemediaType, \
		matchSourcemediaWidth, \
		matchSourcemediaHeight, \
		matchFiles, \
		matchDirs, \
		matchTitle, \
		matchDes, \
		matchImageDir, \
		matchImageMaxHeight, \
		matchImageMaxWidth = self.initTargetFromTemplateMeta( tplFileContentR, tplDir, outFile)

		tplFileContentR = self.cleanTemplateMetas( tplFileContentR )
		tplFileContent = tplFileContentR

		tplFileContent = self.databaseMetaInfoFiller( tplFileContent )

		AppLog.log(LOG_INFO, "Insert menu : ", "index")
		tplFileContent = self.insertMenu( tplFileContent , "index" )

		# rename : indexfile.html -> index.html
		#outFile = self.makePath( os.path.dirname(outFile), 'index.html')

		tplFileContent = self.cleanFinalData( tplFileContent )

		dirname = os.path.dirname(outFile)
		if not os.path.exists(dirname):
			os.makedirs(dirname)

		fpo = open(outFile, 'w', encoding='utf8')
		fpo.write(tplFileContent)
		fpo.close()


	# Rapport arbre gobal
	'''
		Rapport arbre des ascendants - type image SVG
	'''
	def genRapportTree(self, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting) :

		AppLog.log( LOG_INFO , "rootMedia = " + str(rootMedia) )
		AppLog.log( LOG_INFO , "dbFile = " + str(dbFile) )

		db = Sqlitedb( dbFile )

		indicount = {}
		eventcount = {}
		sourcecount = {}
		doccount = {}

		indiNum = 0

		myRapport = Dotgraph( db, cbPrivate )
		myRapport.setPrivate( cbPrivate )

		# Read template
		tplDir = os.path.dirname(tplFile)
		fpi = open( tplFile, 'r', encoding='utf8')
		tplFileContentR = fpi.read()
		fpi.close()

		outDirMediaName, \
		matchFormat, \
		matchSourcemediaFile, \
		matchSourcemediaType, \
		matchSourcemediaWidth, \
		matchSourcemediaHeight, \
		matchFiles, \
		matchDirs, \
		matchTitle, \
		matchDes, \
		matchImageDir, \
		matchImageMaxHeight, \
		matchImageMaxWidth = self.initTargetFromTemplateMeta( tplFileContentR, tplDir, outFile )

		tplFileContentR = self.cleanTemplateMetas( tplFileContentR )
		tplFileContent = tplFileContentR

		# Dot file creation

		# Read dot template
		f=open(self.makePath( os.path.dirname(tplFile), "modele.dot"), encoding='utf8')
		dotmodeleContent = f.read()
		f.close()

		# liste des mariages
		nodeList = myRapport.selectMariageTree()
		matchDotElemR = self.getMatchByPattern( dotmodeleContent, "%%U(.*?)%%" )
		if nodeList :
			for node in nodeList :

				matchDotElem = matchDotElemR

				# Conjoint ou conjointe
				mariage = myRapport.selectIndiLMariageById(node['conjoint_id'], node['conjointe_id'])
				if mariage : mariage = mariage[0]

				matchDotElem = matchDotElem.replace ('{{eventtype_name}}', str(node['eventtype_name']))
				matchDotElem = matchDotElem.replace ('{{mariage_id}}', str(node['mariage_id']))
				matchDotElem = matchDotElem.replace ('{{event_name}}', str(node['event_name']))

				matchDotElem = matchDotElem.replace ('{{mar_place}}', str(mariage['mar_place']))
				matchDotElem = matchDotElem.replace ('{{mar_zipcode}}', str(mariage['mar_zipcode']))
				matchDotElem = matchDotElem.replace ('{{i_firstname}}', str(mariage['i_firstname']))
				matchDotElem = matchDotElem.replace ('{{i_lastname}}', str(mariage['i_lastname']))
				matchDotElem = matchDotElem.replace ('{{f_firstname}}', str(mariage['f_firstname']))
				matchDotElem = matchDotElem.replace ('{{f_lastname}}', str(mariage['f_lastname']))

				dateprec = self.date_event_accurate[self.renderValue(mariage['mar_format'])][ self.renderValue(mariage['mar_accurate']) ]
				matchDotElem = matchDotElem.replace("{{dateprec}}", dateprec)

				if mariage['mar_accurate'] == "UNK" :
					matchDotElem = matchDotElem.replace ('{{mar_day}}', "Date inconnue")
					matchDotElem = matchDotElem.replace ('{{mar_month}}', "")
					matchDotElem = matchDotElem.replace ('{{mar_month_lib}}', "" )
					matchDotElem = matchDotElem.replace ('{{mar_year}}', "")
				else :
					if mariage['mar_day'] == 1 and mariage['mar_month'] == 1 and mariage['mar_year'] == 1 :
						matchDotElem = matchDotElem.replace ('{{mar_day}}', "")
						matchDotElem = matchDotElem.replace ('{{mar_month}}', "")
						matchDotElem = matchDotElem.replace ('{{mar_month_lib}}', "" )
						matchDotElem = matchDotElem.replace ('{{mar_year}}', "")
					else :
						rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( mariage['date_begin'], mariage['mar_format'],  mariage['mar_accurate'] )
						matchDotElem = matchDotElem.replace("{{mar_day}}", rDateDay)
						matchDotElem = matchDotElem.replace("{{mar_month_lib}}", rDateMonthLib)
						matchDotElem = matchDotElem.replace("{{mar_year}}", rDateYear)

				dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

		dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, '' )

		# Liens mariages; liens vers le(s) mariage(s) de(s) parent(s)
		matchDotElemR = self.getMatchByPattern( dotmodeleContent, "%%L(.*?)%%" )
		if nodeList :
			for node in nodeList :

				if str(node['conjoint_parent_mariage_id']) != "None" :
					matchDotElem = matchDotElemR
					matchDotElem = matchDotElem.replace ('{{mariage_src_id}}', str(node['mariage_id']))
					mariage_dst_id = node['conjoint_parent_mariage_id']
					matchDotElem = matchDotElem.replace ('{{mariage_dst_id}}', str(mariage_dst_id))
					dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

				if str(node['conjointe_parent_mariage_id']) != "None" :
					matchDotElem = matchDotElemR
					matchDotElem = matchDotElem.replace ('{{mariage_src_id}}', str(node['mariage_id']))
					mariage_dst_id = node['conjointe_parent_mariage_id']
					matchDotElem = matchDotElem.replace ('{{mariage_dst_id}}', str(mariage_dst_id))
					dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

		dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, '' )

		# Noeud Hommes (conjoint mariage)
		matchDotElemR = self.getMatchByPattern( dotmodeleContent, "%%M(.*?)%%" )
		if nodeList :
			for node in nodeList :

				if node['conjoint_id'] is None : continue

				indidet = myRapport.selectIndiDetById( node['conjoint_id']  )
				if indidet : indidet = indidet[0]

				matchDotElem = matchDotElemR
				
				matchDotElem = matchDotElem.replace ('{{conjoint_id}}', str(node['conjoint_id']))
				matchDotElem = matchDotElem.replace ('{{conjoint_mariage_count}}', str(node['conjoint_mariage_count']))
				matchDotElem = matchDotElem.replace ('{{mariage_id}}', str(node['mariage_id']))

				matchDotElem = matchDotElem.replace ('{{firstname}}', str(indidet['firstname']))
				matchDotElem = matchDotElem.replace ('{{lastname}}', str(indidet['lastname']))

				matchDotElem = matchDotElem.replace("{{nameformated}}", self.rvalue(indidet['nameformated']) )
				matchDotElem = matchDotElem.replace("{{firtsnameothers}}", self.rvalue(indidet['firtsnameothers']) )				
				
				matchDotElem = matchDotElem.replace ('{{sex}}', str(indidet['sex']))
				matchDotElem = matchDotElem.replace ('{{iconsex}}', self.renderIcoSex(indidet['sex']))

				iconUrl = self.renderIcon( rootMedia, indidet['icon'], outDirMediaName, matchImageDir, node['conjoint_id'], "icons/ico-person-m.png", matchImageMaxWidth, matchImageMaxHeight)
				matchDotElem = matchDotElem.replace ('{{icon}}', iconUrl)

				# Lien vers le mariage parents
				if node['conjoint_parent_mariage_id'] is not None :
					matchDotElem = matchDotElem.replace ('{{conjoint_parent_mariage_id}}', str(node['conjoint_parent_mariage_id']))
				else :
					matchDotElem = matchDotElem.replace ('{{conjoint_parent_mariage_id}}', "_NONE_")

				dateprec = self.date_event_accurate[self.renderValue(indidet['date_birth_format'])][ self.renderValue(indidet['date_birth_accurate']) ]
				matchDotElem = matchDotElem.replace("{{dateprec}}", dateprec)

				if indidet['date_birth_accurate'] == "UNK" :
					matchDotElem = matchDotElem.replace ('{{birth_day}}', "Date inconnue")
					matchDotElem = matchDotElem.replace ('{{birth_month}}', "")
					matchDotElem = matchDotElem.replace ('{{birth_month_lib}}', "" )
					matchDotElem = matchDotElem.replace ('{{birth_year}}', "")
				else :
					if indidet['birth_day'] == "1" and indidet['birth_month'] == "1" and indidet['birth_year'] == "1" :
						matchDotElem = matchDotElem.replace ('{{birth_day}}', "")
						matchDotElem = matchDotElem.replace ('{{birth_month}}', "")
						matchDotElem = matchDotElem.replace ('{{birth_month_lib}}', "" )
						matchDotElem = matchDotElem.replace ('{{birth_year}}', "")
					else :
						rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( indidet['date_birth'], indidet['date_birth_format'],  indidet['date_birth_accurate'] )
						matchDotElem = matchDotElem.replace("{{birth_day}}", rDateDay)
						matchDotElem = matchDotElem.replace("{{birth_month_lib}}", rDateMonthLib)
						matchDotElem = matchDotElem.replace("{{birth_year}}", rDateYear)

				matchDotElem = matchDotElem.replace ('{{birth_place}}', self.renderValue(indidet['birth_place_name']))

				dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

		# Noeud enfants M sans mariage
		nodeListChildNoMar = myRapport.selectAllIndiDet()
		if nodeListChildNoMar :
			for node in nodeListChildNoMar :

				if node['mariage_id'] is not None : continue
				if node['sex'] != 'M' : continue

				matchDotElem = matchDotElemR

				matchDotElem = matchDotElem.replace ('{{conjoint_id}}', str(node['id']))
				matchDotElem = matchDotElem.replace ('{{conjoint_mariage_count}}', "0")												
				matchDotElem = matchDotElem.replace ('{{conjoint_parent_mariage_id}}', str(node['parent_mariage_id']))
				matchDotElem = matchDotElem.replace ('{{firstname}}', str(node['firstname']))
				matchDotElem = matchDotElem.replace ('{{lastname}}', str(node['lastname']))
				matchDotElem = matchDotElem.replace("{{nameformated}}", self.rvalue(node['nameformated']) )
				matchDotElem = matchDotElem.replace("{{firtsnameothers}}", self.rvalue(node['firtsnameothers']) )								
				matchDotElem = matchDotElem.replace ('{{sex}}', str(node['sex']))
				matchDotElem = matchDotElem.replace ('{{iconsex}}', self.renderIcoSex(node['sex']))

				iconUrl = self.renderIcon( rootMedia, node['icon'], outDirMediaName, matchImageDir, node['id'], "icons/ico-person-m.png", matchImageMaxWidth, matchImageMaxHeight)
				matchDotElem = matchDotElem.replace ('{{icon}}', iconUrl)

				dateprec = self.date_event_accurate[self.renderValue(node['date_birth_format'])][ self.renderValue(node['date_birth_accurate']) ]
				matchDotElem = matchDotElem.replace("{{dateprec}}", dateprec)

				if node['date_birth_accurate'] == "UNK" :
					matchDotElem = matchDotElem.replace ('{{birth_day}}', "Date inconnue")
					matchDotElem = matchDotElem.replace ('{{birth_month}}', "")
					matchDotElem = matchDotElem.replace ('{{birth_month_lib}}', "" )
					matchDotElem = matchDotElem.replace ('{{birth_year}}', "")
				else :
					if node['birth_day'] == "1" and node['birth_month'] == "1" and node['birth_year'] == "1" :
						matchDotElem = matchDotElem.replace ('{{birth_day}}', "")
						matchDotElem = matchDotElem.replace ('{{birth_month}}', "")
						matchDotElem = matchDotElem.replace ('{{birth_month_lib}}', "" )
						matchDotElem = matchDotElem.replace ('{{birth_year}}', "")
					else :
						rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( node['date_birth'], node['date_birth_format'],  node['date_birth_accurate'] )
						matchDotElem = matchDotElem.replace("{{birth_day}}", rDateDay)
						matchDotElem = matchDotElem.replace("{{birth_month_lib}}", rDateMonthLib)
						matchDotElem = matchDotElem.replace("{{birth_year}}", rDateYear)

				matchDotElem = matchDotElem.replace ('{{birth_place}}', self.renderValue(node['birth_place_name']))

				dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

		dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, '' )

		# Noeud Femmes (conjointe mariage)
		matchDotElemR = self.getMatchByPattern( dotmodeleContent, "%%F(.*?)%%" )
		if nodeList :
			for node in nodeList :

				if node['conjointe_id'] is None : continue
				indidet = myRapport.selectIndiDetById( node['conjointe_id']  )
				if indidet : indidet = indidet[0]

				matchDotElem = matchDotElemR

				matchDotElem = matchDotElem.replace ('{{conjointe_id}}', str(node['conjointe_id']))
				matchDotElem = matchDotElem.replace ('{{conjointe_mariage_count}}', str(node['conjointe_mariage_count']))				
				matchDotElem = matchDotElem.replace ('{{mariage_id}}', str(node['mariage_id']))

				matchDotElem = matchDotElem.replace ('{{firstname}}', str(indidet['firstname']))
				matchDotElem = matchDotElem.replace ('{{lastname}}', str(indidet['lastname']))
				matchDotElem = matchDotElem.replace("{{nameformated}}", self.rvalue(indidet['nameformated']) )
				matchDotElem = matchDotElem.replace("{{firtsnameothers}}", self.rvalue(indidet['firtsnameothers']) )								
				matchDotElem = matchDotElem.replace ('{{sex}}', str(indidet['sex']))
				matchDotElem = matchDotElem.replace ('{{iconsex}}', self.renderIcoSex(indidet['sex']))

				iconUrl = self.renderIcon( rootMedia, indidet['icon'], outDirMediaName, matchImageDir, node['conjointe_id'], "icons/ico-person-w.png", matchImageMaxWidth, matchImageMaxHeight)
				matchDotElem = matchDotElem.replace ('{{icon}}', iconUrl)

				# Lien vers le mariage parents
				if node['conjointe_parent_mariage_id'] is not None :
					matchDotElem = matchDotElem.replace ('{{conjointe_parent_mariage_id}}', str(node['conjointe_parent_mariage_id']))
				else :
					matchDotElem = matchDotElem.replace ('{{conjointe_parent_mariage_id}}', "_NONE_")

				dateprec = self.date_event_accurate[self.renderValue(indidet['date_birth_format'])][ self.renderValue(indidet['date_birth_accurate']) ]
				matchDotElem = matchDotElem.replace("{{dateprec}}", dateprec)

				if indidet['date_birth_accurate'] == "UNK" :
					matchDotElem = matchDotElem.replace ('{{birth_day}}', "Date inconnue")
					matchDotElem = matchDotElem.replace ('{{birth_month}}', "")
					matchDotElem = matchDotElem.replace ('{{birth_month_lib}}', "" )
					matchDotElem = matchDotElem.replace ('{{birth_year}}', "")
				else :
					if indidet['birth_day'] == "1" and indidet['birth_month'] == "1" and indidet['birth_year'] == "1" :
						matchDotElem = matchDotElem.replace ('{{birth_day}}', "")
						matchDotElem = matchDotElem.replace ('{{birth_month}}', "")
						matchDotElem = matchDotElem.replace ('{{birth_month_lib}}', "" )
						matchDotElem = matchDotElem.replace ('{{birth_year}}', "")
					else :
						rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( indidet['date_birth'], indidet['date_birth_format'],  indidet['date_birth_accurate'] )
						matchDotElem = matchDotElem.replace("{{birth_day}}", rDateDay)
						matchDotElem = matchDotElem.replace("{{birth_month_lib}}", rDateMonthLib)
						matchDotElem = matchDotElem.replace("{{birth_year}}", rDateYear)

				matchDotElem = matchDotElem.replace ('{{birth_place}}', self.renderValue(indidet['birth_place_name']))

				dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

		# Noeud enfants F sans mariage
		nodeListChildNoMar = myRapport.selectAllIndiDet()
		if nodeListChildNoMar :
			for node in nodeListChildNoMar :

				if node['mariage_id'] is not None : continue
				if node['sex'] != 'F' : continue

				matchDotElem = matchDotElemR

				matchDotElem = matchDotElem.replace ('{{conjointe_id}}', str(node['id']))
				matchDotElem = matchDotElem.replace ('{{conjointe_mariage_count}}', "0")								
				matchDotElem = matchDotElem.replace ('{{conjointe_parent_mariage_id}}', str(node['parent_mariage_id']))
				matchDotElem = matchDotElem.replace ('{{firstname}}', str(node['firstname']))
				matchDotElem = matchDotElem.replace ('{{lastname}}', str(node['lastname']))
				matchDotElem = matchDotElem.replace("{{nameformated}}", self.rvalue(node['nameformated']) )
				matchDotElem = matchDotElem.replace("{{firtsnameothers}}", self.rvalue(node['firtsnameothers']) )								
				matchDotElem = matchDotElem.replace ('{{sex}}', str(node['sex']))
				matchDotElem = matchDotElem.replace ('{{iconsex}}', self.renderIcoSex(node['sex']))

				iconUrl = self.renderIcon( rootMedia, node['icon'], outDirMediaName, matchImageDir, node['id'], "icons/ico-person-m.png", matchImageMaxWidth, matchImageMaxHeight)
				matchDotElem = matchDotElem.replace ('{{icon}}', iconUrl)

				dateprec = self.date_event_accurate[self.renderValue(node['date_birth_format'])][ self.renderValue(node['date_birth_accurate']) ]
				matchDotElem = matchDotElem.replace("{{dateprec}}", dateprec)

				if node['date_birth_accurate'] == "UNK" :
					matchDotElem = matchDotElem.replace ('{{birth_day}}', "Date inconnue")
					matchDotElem = matchDotElem.replace ('{{birth_month}}', "")
					matchDotElem = matchDotElem.replace ('{{birth_month_lib}}', "" )
					matchDotElem = matchDotElem.replace ('{{birth_year}}', "")
				else :
					if node['birth_day'] == "1" and node['birth_month'] == "1" and node['birth_year'] == "1" :
						matchDotElem = matchDotElem.replace ('{{birth_day}}', "")
						matchDotElem = matchDotElem.replace ('{{birth_month}}', "")
						matchDotElem = matchDotElem.replace ('{{birth_month_lib}}', "" )
						matchDotElem = matchDotElem.replace ('{{birth_year}}', "")
					else :
						rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( node['date_birth'], node['date_birth_format'],  node['date_birth_accurate'] )
						matchDotElem = matchDotElem.replace("{{birth_day}}", rDateDay)
						matchDotElem = matchDotElem.replace("{{birth_month_lib}}", rDateMonth)
						matchDotElem = matchDotElem.replace("{{birth_year}}", rDateYear)

				matchDotElem = matchDotElem.replace ('{{birth_place}}', self.renderValue(node['birth_place_name']))

				dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

		dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, '' )

		# Liens  : individu -> mariages <- individu
		matchDotElemR = self.getMatchByPattern( dotmodeleContent, "%%A(.*?)%%" )
		cluster_id = 0
		if nodeList :
			for node in nodeList :

				cluster_id = cluster_id + 1

				matchDotElem = matchDotElemR
				
				matchDotElem = matchDotElem.replace ('{{cluster_id}}', str(cluster_id))
				matchDotElem = matchDotElem.replace ('{{conjoint_mariage_count}}', str(node['conjoint_mariage_count']))
				matchDotElem = matchDotElem.replace ('{{conjointe_mariage_count}}', str(node['conjointe_mariage_count']))				
				matchDotElem = matchDotElem.replace ('{{conjointe_mariage_count}}', str(node['conjointe_mariage_count']))
				matchDotElem = matchDotElem.replace ('{{conjoint_mariage_count}}', str(node['conjoint_mariage_count']))				
				matchDotElem = matchDotElem.replace ('{{name}}',str(node['event_name']))
				matchDotElem = matchDotElem.replace ('{{conjoint_id}}', str(node['conjoint_id']))
				matchDotElem = matchDotElem.replace ('{{conjointe_id}}', str(node['conjointe_id']))
				matchDotElem = matchDotElem.replace ('{{mariage_id}}', str(node['mariage_id']))

				dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

		dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, '' )

		# Clean _NONE_
		dotmodeleContent = re.sub('(?<=<ifnotnull>)(?:.(?!<ifnotnull>))*NONE.*(?=</ifnotnull>)', '', dotmodeleContent)
		dotmodeleContent = dotmodeleContent.replace ('<ifnotnull>', "")
		dotmodeleContent = dotmodeleContent.replace ('</ifnotnull>', "")
		dotmodeleContent = dotmodeleContent.replace ('_NONE_', "")

		# ----------------------
		# Dot other replacements
		dotmodeleContent = dotmodeleContent.replace ('{{outdir}}', os.path.dirname(outDirMediaName))
		dotmodeleContent = dotmodeleContent.replace ('{{mediasubdir}}', os.path.basename(outDirMediaName))

		# Dot file creation
		dotmodeleContent = dotmodeleContent.replace(  '%U', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%L', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%M', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%F', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%A', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%N', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%', '' )

		dotfilename = self.makePath( outDirMediaName, "tree.dot")
		fp = open(dotfilename, 'w', encoding='utf8')
		fp.write(dotmodeleContent)
		fp.close()

		# Html file creation
		svgurl = self.renderDotAsSvg( dotfilename )
		subdir = os.path.basename(os.path.dirname(svgurl))
		svgfilename = self.makePath(subdir, os.path.basename(svgurl))
		tplFileContent = tplFileContent.replace('{{svgfilename}}' , svgfilename )
		tplFileContent = tplFileContent.replace('{{svgdata}}' , open(svgurl, encoding='utf8').read() )

		tplFileContent = self.databaseMetaInfoFiller( tplFileContent )

		AppLog.log(LOG_INFO, "Insert menu : ", "arbre_mariages")
		tplFileContent = self.insertMenu( tplFileContent, "arbre" )

		tplFileContent = self.cleanFinalData( tplFileContent )
		fpo = open(outFile, 'w', encoding='utf8')
		fpo.write(tplFileContent)
		fpo.close()

	'''
		Rapport Ascendants - type image SVG (full tree)
	'''

	dotDataFinalNodes = ""
	dotDataFinalLinks = ""

	matchDotElemRM = ""
	matchDotElemRF = ""
	matchDotElemRI = ""
	matchDotElemLinks = ""

	outDirMediaName = None
	rootMedia = None
	matchImageMaxWidth = None
	matchImageMaxHeight = None
	matchImageDir = None

	# --------------------------------------------------------------------------------------

	'''
	Rapport ascendant de l'individu Decujus
	'''
	def genRapportAscendant(self, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting) :

		db = Sqlitedb( dbFile )
		levelMax = Metadatas( db ).selectMetaByCode("TREASCLEVELMAX")
		if levelMax :
			levelMax = int(Metadatas( db ).selectMetaByCode("TREASCLEVELMAX")['value'])

		indiId = projSetting.load((AppSetting.get('app/lastProject', ''))).decuJus

		# Read html template
		tplDir = os.path.dirname(tplFile)
		fpi = open( tplFile, 'r', encoding='utf8')
		tplFileContent = fpi.read()
		fpi.close()

		outDirMediaName, \
		matchFormat, \
		matchSourcemediaFile, \
		matchSourcemediaType, \
		matchSourcemediaWidth, \
		matchSourcemediaHeight, \
		matchFiles, \
		matchDirs, \
		matchTitle, \
		matchDes, \
		matchImageDir, \
		matchImageMaxHeight, \
		matchImageMaxWidth = self.initTargetFromTemplateMeta( tplFileContent, tplDir, outFile )
		tplFileContent = self.cleanTemplateMetas( tplFileContent )

		self.tplFileContent = tplFileContent
		self.outDirMediaName = outDirMediaName
		self.rootMedia = rootMedia
		self.matchImageMaxWidth = matchImageMaxWidth
		self.matchImageMaxHeight = matchImageMaxHeight
		self.matchImageDir = matchImageDir

		dotFileModel = self.makePath( os.path.dirname(tplFile), "modele.dot")
		dotmodeleContent = self.rapportAscendants( indiId, levelMax, dotFileModel, dbFile, cbPrivate, False )

		dotfilename = self.makePath( self.outDirMediaName, "tree.dot" )
		AppLog.log(LOG_INFO, "dot dotfilename : ", dotfilename)
		fp = open(dotfilename, 'w', encoding='utf8')
		fp.write(dotmodeleContent)
		fp.close()

		# Render Svg from Dot
		svgurl = self.renderDotAsSvg( dotfilename )
		subdir = os.path.basename(os.path.dirname(svgurl))
		svgfilename = self.makePath(subdir, os.path.basename(svgurl))

		# Insert svg in Html
		AppLog.log(LOG_INFO, "dot svgfilename : ", svgfilename)
		self.tplFileContent = self.tplFileContent.replace('{{svgfilename}}' , svgfilename )  # Relative path
		AppLog.log(LOG_INFO, "dot svgurl : ", svgurl)
		self.tplFileContent = self.tplFileContent.replace('{{svgdata}}' , open(svgurl, encoding='utf8').read() ) # Full path

		self.tplFileContent = self.databaseMetaInfoFiller( self.tplFileContent )

		AppLog.log(LOG_INFO, "Insert menu : ", "arbre")
		self.tplFileContent = self.insertMenu( self.tplFileContent, "arbre" )

		self.tplFileContent = self.cleanFinalData( self.tplFileContent )
		fpo = open(outFile, 'w', encoding='utf8')
		fpo.write(self.tplFileContent)
		fpo.close()


	# --------------------------------------------------------------------------------------
	'''
	Rapport ascendants d'un individu donné - Fiche individu
	'''
	# Mini-arbre des proches ascendants
	def rapportIndiTree(self, indiId, levelMax, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting) :

		deCujusId = indiId

		db = Sqlitedb( dbFile )
		rapport = Dotgraph( db, cbPrivate )
		rapport.setPrivate( cbPrivate )

		dotFileModel = self.makePath( tplDir, "tree/modele.dot")

		# Read html template
		tplDir = os.path.dirname(tplFile)
		fpi = open( tplFile, 'r', encoding='utf8')
		tplFileContent = fpi.read()
		fpi.close()

		matchFormat, \
		matchSourcemediaFile, \
		matchSourcemediaType, \
		matchSourcemediaWidth, \
		matchSourcemediaHeight, \
		matchFiles, \
		matchDirs, \
		matchTitle, \
		matchDes, \
		matchImageDir, \
		matchImageMaxHeight, \
		matchImageMaxWidth = self.readTargetFromTemplateMeta( tplFileContent )
		self.tplFileContent = tplFileContent

		self.rootMedia = rootMedia
		self.matchImageMaxWidth = matchImageMaxWidth
		self.matchImageMaxHeight = matchImageMaxHeight

		# self.matchImageDir = "/home/.../Bureau/OUTTMP/annuaire/images"
		self.matchImageDir = self.makePath( os.path.dirname( outFile ), "annuaire/images")

		AppLog.log(LOG_INFO, " rootMedia : ", self.rootMedia)
		AppLog.log(LOG_INFO, " matchImageDir : ", self.matchImageDir)

		# Read dot template
		f=open( dotFileModel, encoding='utf8' )
		dotmodeleContent = f.read()
		f.close()
		self.matchDotElemRM = self.getMatchByPattern( dotmodeleContent, "%%M(.*?)%%" )
		self.matchDotElemRF = self.getMatchByPattern( dotmodeleContent, "%%F(.*?)%%" )
		self.matchDotElemU = self.getMatchByPattern( dotmodeleContent, "%%U(.*?)%%" )
		self.matchDotElemC = self.getMatchByPattern( dotmodeleContent, "%%C(.*?)%%" )

		self.dotElemRM = ""
		self.dotElemRF = ""
		self.dotElemU = ""
		self.dotElemC = ""

		# Fill node START --------------------
		# Dejucus Node
		nodeList = rapport.selectIndiDetById( indiId )
		node = nodeList[0]
		decujusRoot = "" # decujusRoot = "_NONE_"
		self.rapportIndiTreeFillNodeIndi( node, decujusRoot = "" )

		levelCurrent = 1
		# Parents nodes
		#print("---------- J=",indiId )
		levelMax = 2
		self.rapportParent( rapport, indiId, levelCurrent, levelMax, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting )

		# Childs nodes
		levelMax = 1
		self.rapportChild( rapport, indiId , levelCurrent, levelMax, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting )

		#print("---")

		dotmodeleContent = dotmodeleContent.replace( self.matchDotElemRM, self.dotElemRM)
		dotmodeleContent = dotmodeleContent.replace( self.matchDotElemRF, self.dotElemRF)
		dotmodeleContent = dotmodeleContent.replace( self.matchDotElemU, self.dotElemU)
		dotmodeleContent = dotmodeleContent.replace( self.matchDotElemC, self.dotElemC)

		dirTreeFullPathName =  self.makePath( os.path.dirname(tplFile), "tree" )
		dotfilename = self.makePath( os.path.dirname(outFile), "annuaire/" + "tree" + "/tree_"+str(indiId)+".dot" )

		# ----------------------
		# Dot other replacements
		dotmodeleContent = dotmodeleContent.replace ('{{outdir}}', os.path.dirname(self.outDirMediaName))
		dotmodeleContent = dotmodeleContent.replace ('{{mediasubdir}}', os.path.basename(self.outDirMediaName))

		# Clean dot
		#dotmodeleContent = dotmodeleContent.replace(  '_NONE_', '' )
		dotmodeleContent = re.sub('(?<=<ifnotnull>)(?:.(?!<ifnotnull>))*NONE.*(?=</ifnotnull>)', '', dotmodeleContent)
		dotmodeleContent = dotmodeleContent.replace(  '<ifnotnull>', '' )
		dotmodeleContent = dotmodeleContent.replace(  '</ifnotnull>', '' )

		dotmodeleContent = dotmodeleContent.replace(  '%%M', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%%F', '' )

		dotmodeleContent = dotmodeleContent.replace(  '%%U', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%%C', '' )

		dotmodeleContent = dotmodeleContent.replace(  '%%', '' )

		fp = open(dotfilename, 'w', encoding='utf8')
		fp.write(dotmodeleContent)
		fp.close()

		# Render Svg from Dot
		svgurl = self.renderDotAsSvg( dotfilename )
		AppLog.log(LOG_INFO, " svgurl : ", svgurl)

		svgfilename = self.makePath(dirTreeFullPathName, os.path.basename(svgurl))

		return svgurl

	# Mini-arbre des proches ascendants
	def rapportParent( self, rapport, indiId, levelCurrent, levelMax, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting ):

		if levelCurrent > levelMax : return

		ascendant = rapport.selectIndiDetById( indiId )

		# Père
		if ascendant and ascendant[0]['pere_id'] is not None :
			#print( "PL=", levelCurrent, " pere_id=", str(ascendant[0]['pere_id']))
			nodeList = rapport.selectIndiDetById( ascendant[0]['pere_id']  )
			if nodeList :
				node = nodeList[0]
				self.rapportIndiTreeFillNodeIndi( node )
				self.rapportParent( rapport, ascendant[0]['pere_id'], levelCurrent+1, levelMax, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting )

		# Mère
		if ascendant and ascendant[0]['mere_id'] is not None :
			#print( "PL=", levelCurrent, " mere_id=", str(ascendant[0]['mere_id']))
			nodeList = rapport.selectIndiDetById( ascendant[0]['mere_id'] )
			if nodeList :
				node = nodeList[0]
				self.rapportIndiTreeFillNodeIndi( node )
				self.rapportParent( rapport, ascendant[0]['mere_id'], levelCurrent+1, levelMax, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting )

		mariage = rapport.selectMariageByIndiIds( ascendant[0]['pere_id'], ascendant[0]['mere_id'])
		if mariage :
			self.rapportIndiTreeFillNodeMariage( ascendant[0]['sex'], indiId, mariage[0] )

	# Mini-arbre des proches ascendants
	def rapportChild( self, rapport, indiId, levelCurrent, levelMax, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting ):

		if levelCurrent > levelMax : return

		childList = rapport.selectIndiDetByParentId( indiId )
		if childList :
			for child in childList :
				#print( "CL=", levelCurrent, " child_id=", str(child['id']))
				nodeList = rapport.selectIndiDetById( child['id']  )
				node = nodeList[0]
				self.rapportIndiTreeFillNodeIndi( node )
				self.rapportChild( rapport, indiId, levelCurrent+1, levelMax, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting )
				self.rapportIndiTreeFillChildLink( rapport, indiId, child['id'], child['sex'] )

	# Mini-arbre des proches ascendants
	def rapportIndiTreeFillNodeIndi( self, node, decujusRoot = "_NONE_" ) :

		if node['sex'] == 'M' : tmpDotData = self.matchDotElemRM
		if node['sex'] == 'F' : tmpDotData = self.matchDotElemRF

		tmpDotData = tmpDotData.replace("{{id}}", str(node['id']) )
		tmpDotData = tmpDotData.replace("{{noderoot}}", decujusRoot)

		tmpDotData = tmpDotData.replace("{{lastname}}", str(node['lastname']))
		tmpDotData = tmpDotData.replace("{{firstname}}", str(node['firstname']))

		if node['sex'] == 'M' :
			iconUrl = self.renderIcon( self.rootMedia, node['icon'], self.outDirMediaName, self.matchImageDir, node['id'], "icons/ico-person-m.png", self.matchImageMaxWidth, self.matchImageMaxHeight)
		if node['sex'] == 'F' :
			iconUrl = self.renderIcon( self.rootMedia, node['icon'], self.outDirMediaName, self.matchImageDir, node['id'], "icons/ico-person-w.png", self.matchImageMaxWidth, self.matchImageMaxHeight)

		tmpDotData = tmpDotData.replace ('{{icon}}', iconUrl)

		tmpDotData = tmpDotData.replace ('{{birth_place}}', self.renderValue(node['birth_place_name']) )

		dateprec = self.date_event_accurate[self.renderValue(node['date_birth_format'])][ self.renderValue(node['date_birth_accurate']) ]
		tmpDotData = tmpDotData.replace("{{dateprec}}", dateprec)

		if node['date_birth_accurate'] == "UNK" :
			tmpDotData = tmpDotData.replace ('{{birth_day}}', "Date inconnue")
			tmpDotData = tmpDotData.replace ('{{birth_month}}', "")
			tmpDotData = tmpDotData.replace ('{{birth_month_lib}}', "" )
			tmpDotData = tmpDotData.replace ('{{birth_year}}', "")
		else :
			if node['birth_day'] == 1 and node['birth_month'] == 1 and node['birth_year'] == 1 :
				tmpDotData = tmpDotData.replace ('{{birth_day}}', "")
				tmpDotData = tmpDotData.replace ('{{birth_month}}', "")
				tmpDotData = tmpDotData.replace ('{{birth_month_lib}}', "" )
				tmpDotData = tmpDotData.replace ('{{birth_year}}', "")
			else :
				rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( node['date_birth'], node['date_birth_format'],  node['date_birth_accurate'] )
				tmpDotData = tmpDotData.replace ('{{birth_day}}',rDateDay)
				tmpDotData = tmpDotData.replace ('{{birth_month}}', rDateMonth)
				tmpDotData = tmpDotData.replace ('{{birth_month_lib}}', rDateMonthLib )
				tmpDotData = tmpDotData.replace ('{{birth_year}}', rDateYear)

		#tmpDotData = tmpDotData.replace('_NONE_','')

		self.dotElemRM = self.dotElemRM + tmpDotData

	# Mini-arbre des proches ascendants
	def rapportIndiTreeFillNodeMariage( self, indiSex, indiId, mariage ) :

		tmpDotData = self.matchDotElemU

		tmpDotData = tmpDotData.replace("{{id}}", str(mariage['id']))
		tmpDotData = tmpDotData.replace("{{et_name}}", str(mariage['et_name']))
		tmpDotData = tmpDotData.replace("{{mar_place}}", str(mariage['mar_place']))
		tmpDotData = tmpDotData.replace("{{mar_zipcode}}", str(mariage['mar_zipcode']))
		tmpDotData = tmpDotData.replace("{{i_firstname}}", str(mariage['i_firstname']))
		tmpDotData = tmpDotData.replace("{{i_lastname}}", str(mariage['i_lastname']))
		tmpDotData = tmpDotData.replace("{{f_lastname}}", str(mariage['f_firstname']))
		tmpDotData = tmpDotData.replace("{{f_lastname}}", str(mariage['i_lastname']))

		dateprec = self.date_event_accurate[self.renderValue(mariage['mar_format'])][ self.renderValue(mariage['mar_accurate']) ]
		tmpDotData = tmpDotData.replace("{{dateprec}}", dateprec)

		if mariage['mar_accurate'] == "UNK" :
			tmpDotData = tmpDotData.replace ('{{mar_day}}', "Date inconnue")
			tmpDotData = tmpDotData.replace ('{{mar_month}}', "")
			tmpDotData = tmpDotData.replace ('{{mar_month_lib}}', "" )
			tmpDotData = tmpDotData.replace ('{{mar_year}}', "")
		else :
			if mariage['mar_day'] == 1 and mariage['mar_month'] == 1 and mariage['mar_year'] == 1 :
				tmpDotData = tmpDotData.replace ('{{mar_day}}', "")
				tmpDotData = tmpDotData.replace ('{{mar_month}}', "")
				tmpDotData = tmpDotData.replace ('{{mar_month_lib}}', "" )
				tmpDotData = tmpDotData.replace ('{{mar_year}}', "")
			else :
				rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( mariage['date_begin'], mariage['mar_format'],  mariage['mar_accurate'] )
				tmpDotData = tmpDotData.replace ('{{mar_day}}', rDateDay)
				tmpDotData = tmpDotData.replace ('{{mar_month}}', rDateMonth)
				tmpDotData = tmpDotData.replace ('{{mar_month_lib}}', rDateMonthLib )
				tmpDotData = tmpDotData.replace ('{{mar_year}}', rDateYear)

		tmpDotData = tmpDotData.replace("{{mar_year}}", str(mariage['mar_year']))

		tmpDotData = tmpDotData.replace ('{{node_i}}', indiSex + str(indiId))
		tmpDotData = tmpDotData.replace("{{node_f}}", 'F' +str(mariage['f_id']))
		tmpDotData = tmpDotData.replace("{{node_m}}", 'M' +str(mariage['i_id']))
		tmpDotData = tmpDotData.replace("{{node_u}}", 'U' +str(mariage['id']))

		

		self.dotElemU = self.dotElemU + tmpDotData

	# Mini-arbre des proches ascendants
	def rapportIndiTreeFillChildLink( self, rapport, indiId, childId, childSex) :

		nodeList = rapport.selectIndiDetById( indiId  )
		if nodeList :
			tmpDotData = self.matchDotElemC
			tmpDotData = tmpDotData.replace ('{{node_c}}', nodeList[0]['sex'] + str(indiId))
			tmpDotData = tmpDotData.replace ('{{node_i}}', childSex + str(childId))
			self.dotElemC = self.dotElemC + tmpDotData

	# --------------------------------------------------------------------------------------
	'''
	Point d'entrée pour la création d'un arbre graphique "ascendants"
	'''
	def rapportAscendants(self, indiId, levelMax, dotFileModel, dbFile, cbPrivate, showDescendants=True ) :

		deCujusId = indiId

		db = Sqlitedb( dbFile )
		myRapport = Dotgraph( db, cbPrivate )
		myRapport.setPrivate( cbPrivate )

		AppLog.log(LOG_INFO, "dot indiId : ", indiId)
		AppLog.log(LOG_INFO, "dot levelMax : ", levelMax)
		AppLog.log(LOG_INFO, "dot dotFileModel : ", dotFileModel)

		self.dotDataFinalNodes = ""
		self.dotDataFinalLinks = ""
		self.dotElemChildLinks = ""

		# Read dot template
		f=open( dotFileModel, encoding='utf8' )
		dotmodeleContent = f.read()
		f.close()
		self.matchDotElemRM = self.getMatchByPattern( dotmodeleContent, "%%M(.*?)%%" )
		self.matchDotElemRF = self.getMatchByPattern( dotmodeleContent, "%%F(.*?)%%" )
		self.matchDotElemRI = self.getMatchByPattern( dotmodeleContent, "%%I(.*?)%%" )
		self.matchDotElemLinks = self.getMatchByPattern( dotmodeleContent, "%%U(.*?)%%" )
		self.matchDotElemChildLinks = self.getMatchByPattern( dotmodeleContent, "%%C(.*?)%%" )

		level = 0

		nodeListParents = myRapport.selectIndiAndParentById( deCujusId )
		if nodeListParents :

			# Traitement des descendants
			if showDescendants is True :
				nodeListChilds = myRapport.selectIndiChildsById( deCujusId )
				if nodeListChilds :
					noderoot = "_NONE_"
					for node in nodeListChilds :
						self.rapportAscendantCreateNode( myRapport, node['id'], node['sex'], level, noderoot )
						self.rapportAscendantCreateChildLink( myRapport, node['id'], node['sex'], deCujusId, node['mere_id'], node['pere_id'] )

			# Traitement des ascendants
			noderoot = ""
			for node in nodeListParents :
				self.rapportAscendantCreateNode( myRapport, node['id'], node['sex'], level, noderoot )
				noderoot = "_NONE_"
				self.rapportAscendantParse( myRapport, node['pere_id'], 'M', levelMax, level, noderoot )
				self.rapportAscendantCreateLink( myRapport, node['id'], node['sex'], node['mere_id'], 'F', node['pere_id'], 'M')
				self.rapportAscendantParse( myRapport, node['mere_id'], 'F', levelMax, level, noderoot )

			# Dot template datas replacement
			dotmodeleContent = dotmodeleContent.replace( self.matchDotElemRM, self.dotDataFinalNodes)
			dotmodeleContent = dotmodeleContent.replace( self.matchDotElemRF, "")
			dotmodeleContent = dotmodeleContent.replace( self.matchDotElemRI, "")
			dotmodeleContent = dotmodeleContent.replace( self.matchDotElemLinks, self.dotDataFinalLinks)
			dotmodeleContent = dotmodeleContent.replace( self.matchDotElemChildLinks, self.dotElemChildLinks)

			# ----------------------
			# Dot other replacements
			dotmodeleContent = dotmodeleContent.replace ('{{outdir}}', os.path.dirname(self.outDirMediaName))
			dotmodeleContent = dotmodeleContent.replace ('{{mediasubdir}}', os.path.basename(self.outDirMediaName))

			# Clean dot
			dotmodeleContent = re.sub('(?<=<ifnotnull>)(?:.(?!<ifnotnull>))*NONE.*(?=</ifnotnull>)', '', dotmodeleContent)
			dotmodeleContent = dotmodeleContent.replace(  '<ifnotnull>', '' )
			dotmodeleContent = dotmodeleContent.replace(  '</ifnotnull>', '' )

			dotmodeleContent = dotmodeleContent.replace(  '%%M', '' )
			dotmodeleContent = dotmodeleContent.replace(  '%%F', '' )
			dotmodeleContent = dotmodeleContent.replace(  '%%I', '' )

			dotmodeleContent = dotmodeleContent.replace(  '%%U', '' )
			dotmodeleContent = dotmodeleContent.replace(  '%%C', '' )

			dotmodeleContent = dotmodeleContent.replace(  '%%', '' )

		return dotmodeleContent

	'''
		Traitement des parents d'un noeud
		Retourne False si le niveau max a été atteind
	'''
	def rapportAscendantParse(self, myRapport, indiId, sexe, levelMax, level, noderoot  ) :

		level = level + 1
		if level > levelMax :
			return False

		nodeList = myRapport.selectIndiAndParentById( indiId )
		if nodeList :
			for node in nodeList :
				self.rapportAscendantCreateNode( myRapport, node['id'], sexe, level, noderoot )
				trtNodeM = self.rapportAscendantParse( myRapport, node['mere_id'], 'F', levelMax, level, noderoot )
				trtNodeP = self.rapportAscendantParse( myRapport, node['pere_id'], 'M', levelMax, level, noderoot )
				if level < levelMax :
					self.rapportAscendantCreateLink( myRapport, node['id'], sexe, node['mere_id'], 'F', node['pere_id'], 'M')

				if trtNodeP is False or trtNodeM is False:
					return False

		return True

	# Création d'un noeud
	def rapportAscendantCreateNode (self, myRapport, indiId, sexe, level, noderoot ) :

		nodeData = myRapport.selectIndiAndParentById( indiId )[0]
		matchDotFilled = self.rapportAscendantFillNode( nodeData, noderoot )
		self.dotDataFinalNodes = self.dotDataFinalNodes + matchDotFilled

	# Liens vers le mariage des parents
	def rapportAscendantCreateLink (self, myRapport, indiId, sexe, indiMere, sexeM, indiPere, sexeP ) :

		# fix 1 un des 2 parents peut exister. à afficher...
		if indiMere is None or indiPere is None :
			return

		# Mariage Mere + Pere
		try :
			nodeDataMariage = myRapport.selectIndiLMariageById( indiId, indiMere )[0]
		except Exception as e:
			nodeDataMariage = False

		if nodeDataMariage :
			matchDotFilled = self.rapportAscendantFillLink( nodeDataMariage, indiId, sexe, indiMere, sexeM, indiPere, sexeP )
			self.dotDataFinalLinks = self.dotDataFinalLinks + matchDotFilled

	# Liens direct enfant parent
	def rapportAscendantCreateChildLink (self, myRapport, indidChild, indiSex, decujusID, mereId, pereId ) :

		if decujusID == mereId: decujusSex='F'
		if decujusID == pereId: decujusSex='M'

		matchDotFilled = self.matchDotElemChildLinks
		if matchDotFilled :
			matchDotFilled = matchDotFilled.replace("{{node_c}}", indiSex+str(indidChild))
			matchDotFilled = matchDotFilled.replace("{{node_i}}", decujusSex+str(decujusID) )
			self.dotElemChildLinks = self.dotElemChildLinks + matchDotFilled

	def rapportAscendantFillNode(self, node, noderoot="" ) :

		if node['sex'] == 'M' : matchDotElem = self.matchDotElemRM
		if node['sex'] == 'F' : matchDotElem = self.matchDotElemRF
		if node['sex'] == 'I' : matchDotElem = self.matchDotElemRI

		matchDotElem = matchDotElem.replace ('{{mediasubdir}}', os.path.basename(self.outDirMediaName))

		matchDotElem = matchDotElem.replace ('{{id}}', str(node['id']))
		matchDotElem = matchDotElem.replace ('{{firstname}}', str(node['firstname']))
		matchDotElem = matchDotElem.replace ('{{lastname}}', str(node['lastname']))
		matchDotElem = matchDotElem.replace ('{{sex}}', str(node['sex']))
		matchDotElem = matchDotElem.replace ('{{iconsex}}', self.renderIcoSex(node['sex']))

		if node['sex'] == 'M' :
			iconUrl = self.renderIcon( self.rootMedia, node['icon'], self.outDirMediaName, self.matchImageDir, node['id'], "icons/ico-person-m.png", self.matchImageMaxWidth, self.matchImageMaxHeight)
		if node['sex'] == 'F' :
			iconUrl = self.renderIcon( self.rootMedia, node['icon'], self.outDirMediaName, self.matchImageDir, node['id'], "icons/ico-person-w.png", self.matchImageMaxWidth, self.matchImageMaxHeight)
		if node['sex'] == 'I' :
			iconUrl = self.renderIcon( self.rootMedia, node['icon'], self.outDirMediaName, self.matchImageDir, node['id'], "icons/ico-person-i.png", self.matchImageMaxWidth, self.matchImageMaxHeight)

		matchDotElem = matchDotElem.replace ('{{icon}}', iconUrl)
		matchDotElem = matchDotElem.replace ('{{noderoot}}', noderoot)

		matchDotElem = matchDotElem.replace ('{{birth_place}}', self.renderValue(node['birth_place']) )
		matchDotElem = matchDotElem.replace ('{{birth_day}}', self.renderValue(node['birth_day']))
		matchDotElem = matchDotElem.replace ('{{birth_month}}', self.renderValue(node['birth_month']))
		matchDotElem = matchDotElem.replace ('{{birth_month_lib}}', self.month_lib[ int(self.renderValue(node['birth_month'], "0"))] )
		matchDotElem = matchDotElem.replace ('{{birth_year}}', self.renderValue(node['birth_year']))

		return matchDotElem

	def rapportAscendantFillLink( self, node, indiId, sexe, indiMere, sexeM, indiPere, sexeP ) :

		matchDotElem = self.matchDotElemLinks

		matchDotElem = matchDotElem.replace ('{{mediasubdir}}', os.path.basename(self.outDirMediaName))

		matchDotElem = matchDotElem.replace ('{{id}}', str(node['id']))
		matchDotElem = matchDotElem.replace ('{{et_name}}', str(node['et_name']))
		matchDotElem = matchDotElem.replace ('{{mar_place}}', str(node['mar_place']))
		matchDotElem = matchDotElem.replace ('{{mar_zipcode}}', str(node['mar_zipcode']))
		iconUrl = self.renderIcon( self.rootMedia, node['mar_icon'], self.outDirMediaName, self.matchImageDir, node['id'], "icons/ico-person-w.png", self.matchImageMaxWidth, self.matchImageMaxHeight)
		matchDotElem = matchDotElem.replace ('{{icon}}', iconUrl)

		matchDotElem = matchDotElem.replace ('{{i_id}}', self.renderValue(node['i_id']))
		matchDotElem = matchDotElem.replace ('{{i_sex}}', self.renderValue(node['i_sex']))
		matchDotElem = matchDotElem.replace ('{{i_firstname}}', self.renderValue(node['i_firstname']))
		matchDotElem = matchDotElem.replace ('{{i_lastname}}', self.renderValue(node['i_lastname']))
		matchDotElem = matchDotElem.replace ('{{m_id}}', self.renderValue(node['m_id']))
		matchDotElem = matchDotElem.replace ('{{m_sex}}', self.renderValue(node['m_sex']))
		matchDotElem = matchDotElem.replace ('{{m_firstname}}', self.renderValue(node['m_firstname']))
		matchDotElem = matchDotElem.replace ('{{m_lastname}}', self.renderValue(node['m_lastname']))
		matchDotElem = matchDotElem.replace ('{{f_id}}', self.renderValue(node['f_id']))
		matchDotElem = matchDotElem.replace ('{{f_sex}}', self.renderValue(node['f_sex']))
		matchDotElem = matchDotElem.replace ('{{f_firstname}}', self.renderValue(node['f_firstname']))
		matchDotElem = matchDotElem.replace ('{{f_lastname}}', self.renderValue(node['f_lastname']))

		matchDotElem = matchDotElem.replace ('{{mar_day}}', self.renderValue(node['mar_day']))
		matchDotElem = matchDotElem.replace ('{{mar_month}}', self.renderValue(node['mar_month']))
		matchDotElem = matchDotElem.replace ('{{mar_month_lib}}', self.month_lib[ int(self.renderValue(node['mar_month'], "0"))] )
		matchDotElem = matchDotElem.replace ('{{mar_year}}', self.renderValue(node['mar_year']))

		matchDotElem = matchDotElem.replace ('{{node_i}}', sexe + str(indiId))
		matchDotElem = matchDotElem.replace ('{{node_m}}', 'M' + str(indiPere))
		matchDotElem = matchDotElem.replace ('{{node_f}}', 'F' + str(indiMere))
		matchDotElem = matchDotElem.replace ('{{node_u}}', 'U' + str(node['id']))

		return matchDotElem


	# -------------------------------------------------------------------
	'''
		Rapport des mariages - type image SVG (simple tree)
	'''
	def genRapportMariageTree(self, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting) :

		AppLog.log( LOG_INFO , "rootMedia = " + str(rootMedia) )
		AppLog.log( LOG_INFO , "dbFile = " + str(dbFile) )

		db = Sqlitedb( dbFile )

		indicount = {}
		eventcount = {}
		sourcecount = {}
		doccount = {}

		indiNum = 0

		myRapport = Dotgraph( db, cbPrivate )
		myRapport.setPrivate( cbPrivate )

		# Read template
		tplDir = os.path.dirname(tplFile)
		fpi = open( tplFile, 'r', encoding='utf8')
		tplFileContentR = fpi.read()
		fpi.close()

		outDirMediaName, \
		matchFormat, \
		matchSourcemediaFile, \
		matchSourcemediaType, \
		matchSourcemediaWidth, \
		matchSourcemediaHeight, \
		matchFiles, \
		matchDirs, \
		matchTitle, \
		matchDes, \
		matchImageDir, \
		matchImageMaxHeight, \
		matchImageMaxWidth = self.initTargetFromTemplateMeta( tplFileContentR, tplDir, outFile )

		tplFileContentR = self.cleanTemplateMetas( tplFileContentR )
		tplFileContent = tplFileContentR

		# Dot file creation

		# Read dot template
		f=open(self.makePath( os.path.dirname(tplFile), "modele.dot"), encoding='utf8')
		dotmodeleContent = f.read()
		f.close()

		# liste des mariages
		nodeList = myRapport.selectMariageTree()
		matchDotElemR = self.getMatchByPattern( dotmodeleContent, "%%M(.*?)%%" )
		if nodeList :
			for node in nodeList :

				matchDotElem = matchDotElemR

				# Conjoint ou conjointe
				mariage = myRapport.selectIndiLMariageById(node['conjoint_id'], node['conjointe_id'])
				if mariage : mariage = mariage[0]

				matchDotElem = matchDotElem.replace ('{{eventtype_name}}', str(node['eventtype_name']))
				matchDotElem = matchDotElem.replace ('{{mariage_id}}', str(node['mariage_id']))
				matchDotElem = matchDotElem.replace ('{{event_name}}', str(node['event_name']))

				matchDotElem = matchDotElem.replace ('{{mar_place}}', str(mariage['mar_place']))
				matchDotElem = matchDotElem.replace ('{{mar_zipcode}}', str(mariage['mar_zipcode']))
				matchDotElem = matchDotElem.replace ('{{i_firstname}}', str(mariage['i_firstname']))
				matchDotElem = matchDotElem.replace ('{{i_lastname}}', str(mariage['i_lastname']))
				matchDotElem = matchDotElem.replace ('{{f_firstname}}', str(mariage['f_firstname']))
				matchDotElem = matchDotElem.replace ('{{f_lastname}}', str(mariage['f_lastname']))

				dateprec = self.date_event_accurate[self.renderValue(mariage['mar_format'])][ self.renderValue(mariage['mar_accurate']) ]
				matchDotElem = matchDotElem.replace("{{dateprec}}", dateprec)

				if mariage['mar_accurate'] == "UNK" :
					matchDotElem = matchDotElem.replace ('{{mar_day}}', "Date inconnue")
					matchDotElem = matchDotElem.replace ('{{mar_month}}', "")
					matchDotElem = matchDotElem.replace ('{{mar_month_lib}}', "" )
					matchDotElem = matchDotElem.replace ('{{mar_year}}', "")
				else :
					if mariage['mar_day'] == 1 and mariage['mar_month'] == 1 and mariage['mar_year'] == 1 :
						matchDotElem = matchDotElem.replace ('{{mar_day}}', "")
						matchDotElem = matchDotElem.replace ('{{mar_month}}', "")
						matchDotElem = matchDotElem.replace ('{{mar_month_lib}}', "" )
						matchDotElem = matchDotElem.replace ('{{mar_year}}', "")
					else :
						rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( mariage['date_begin'], mariage['mar_format'],  mariage['mar_accurate'] )
						matchDotElem = matchDotElem.replace ('{{mar_day}}', rDateDay)
						matchDotElem = matchDotElem.replace ('{{mar_month}}', rDateMonth)
						matchDotElem = matchDotElem.replace ('{{mar_month_lib}}', rDateMonthLib )
						matchDotElem = matchDotElem.replace ('{{mar_year}}', rDateYear)

				dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

		dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, '' )

		# Liens mariages; liens vers le(s) mariage(s) de(s) parent(s)
		matchDotElemR = self.getMatchByPattern( dotmodeleContent, "%%L(.*?)%%" )
		if nodeList :
			for node in nodeList :

				if str(node['conjoint_parent_mariage_id']) != "None" :
					matchDotElem = matchDotElemR
					matchDotElem = matchDotElem.replace ('{{mariage_src_id}}', str(node['mariage_id']))
					mariage_dst_id = node['conjoint_parent_mariage_id']
					matchDotElem = matchDotElem.replace ('{{mariage_dst_id}}', str(mariage_dst_id))
					dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

				if str(node['conjointe_parent_mariage_id']) != "None" :
					matchDotElem = matchDotElemR
					matchDotElem = matchDotElem.replace ('{{mariage_src_id}}', str(node['mariage_id']))
					mariage_dst_id = node['conjointe_parent_mariage_id']
					matchDotElem = matchDotElem.replace ('{{mariage_dst_id}}', str(mariage_dst_id))
					dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, matchDotElem + matchDotElemR )

		dotmodeleContent = dotmodeleContent.replace(  matchDotElemR, '' )

		# Dot file creation
		dotmodeleContent = dotmodeleContent.replace(  '%M', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%N', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%L', '' )
		dotmodeleContent = dotmodeleContent.replace(  '%', '' )

		dotmodeleContent = re.sub('(?<=<ifnotnull>)(?:.(?!<ifnotnull>))*NONE.*(?=</ifnotnull>)', '', dotmodeleContent)
		dotmodeleContent = dotmodeleContent.replace(  '<ifnotnull>', '' )
		dotmodeleContent = dotmodeleContent.replace(  '</ifnotnull>', '' )
		dotmodeleContent = dotmodeleContent.replace(  '_NONE_', '' )

		dotfilename = self.makePath( outDirMediaName, "tree.dot")
		fp = open(dotfilename, 'w', encoding='utf8')
		fp.write(dotmodeleContent)
		fp.close()

		# Html file creation
		svgurl = self.renderDotAsSvg( dotfilename )
		subdir = os.path.basename(os.path.dirname(svgurl))
		svgfilename = self.makePath(subdir, os.path.basename(svgurl))
		tplFileContent = tplFileContent.replace('{{svgfilename}}' , svgfilename )
		tplFileContent = tplFileContent.replace('{{svgdata}}' , open(svgurl, encoding='utf8').read() )

		tplFileContent = self.databaseMetaInfoFiller( tplFileContent )

		AppLog.log(LOG_INFO, "Insert menu : ", "arbre_mariages")
		tplFileContent = self.insertMenu( tplFileContent, "arbre_mariages" )

		tplFileContent = self.cleanFinalData( tplFileContent )
		fpo = open(outFile, 'w', encoding='utf8')
		fpo.write(tplFileContent)
		fpo.close()


	'''
		Rapport de type carte des evevements
	'''
	def genRapportMapEvents(self, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting) :

		AppLog.log( LOG_INFO , "rootMedia = " + str(rootMedia) )
		AppLog.log( LOG_INFO , "dbFile = " + str(dbFile) )

		db = Sqlitedb( dbFile )

		indicount = {}
		eventcount = {}
		sourcecount = {}
		doccount = {}

		indiNum = 0

		myRapport = Rapport( db, cbPrivate )

		tplDir = os.path.dirname(tplFile)
		fpi = open( tplFile, 'r', encoding='utf8')
		tplFileContentR = fpi.read()
		fpi.close()

		outDirMediaName, \
		matchFormat, \
		matchSourcemediaFile, \
		matchSourcemediaType, \
		matchSourcemediaWidth, \
		matchSourcemediaHeight, \
		matchFiles, \
		matchDirs, \
		matchTitle, \
		matchDes, \
		matchImageDir, \
		matchImageMaxHeight, \
		matchImageMaxWidth = self.initTargetFromTemplateMeta( tplFileContentR, tplDir, outFile )

		tplFileContentR = self.cleanTemplateMetas( tplFileContentR )
		tplFileContent = tplFileContentR

		# Blocs de repetitions
		globalMatchR   = self.getMatchByPattern( tplFileContentR, "%%(.*?)%%" )
		personMatchR   = self.getMatchByPattern( tplFileContentR, ";;I(.*?);;" )

		eventMatchR    = self.getMatchByPattern( tplFileContentR, "--E(.*?)--" )
		sourceMatchR   = self.getMatchByPattern( tplFileContentR, "==S(.*?)==" )
		documentMatchR = self.getMatchByPattern( tplFileContentR, ",,D(.*?),," )

		# Events ------------------------------------------------------------------------
		eventList = myRapport.selectEVENTSAll()

		eventMatchList = ""
		if eventList :
			for event in eventList :

				eventMatch = eventMatchR

				self.printDebugData( '  EV name #' , event['id'], ' :' , str(event['name']))
				eventcount[event['id']] = eventcount.get(event['id'], 0) + 1

				eventMatch = eventMatch.replace("{{id}}", str(event['id']) )
				eventMatch = eventMatch.replace("{{name}}", str(event['name']) )
				eventMatch = eventMatch.replace("{{zipcode}}", str(event['p.zipcode']) )

				eventMatch = eventMatch.replace("{{e.person_1_id}}", str(event['person_1_id']) )
				eventMatch = eventMatch.replace("{{lastname}}", str(event['lastname']) )
				eventMatch = eventMatch.replace("{{firstname}}", str(event['firstname']) )

				eventMatch = eventMatch.replace("{{e.person_2_id}}", str(event['person_2_id']) )
				eventMatch = eventMatch.replace("{{e.person_3_id}}", str(event['person_3_id']) )

				if event['description'] != '' and event['description'] is not None  :
					eventMatch = eventMatch.replace("{{description}}", str(event['description']) )
				else :
					eventMatch = eventMatch.replace("{{description}}", '_NONE_' )

				rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( event['date_begin'], event['date_begin_format'],  event['date_begin_accurate'] )
				eventMatch = eventMatch.replace("{{date_begin}}",rDate)
				eventMatch = eventMatch.replace("{{date_begin_day}}", rDateDay)
				eventMatch = eventMatch.replace("{{date_begin_day_week}}", rDateDayLib)
				eventMatch = eventMatch.replace("{{date_begin_month}}", rDateMonth)
				eventMatch = eventMatch.replace("{{date_begin_year}}", rDateYear)
				eventMatch = eventMatch.replace("{{date_begin_month_lib}}",rDateMonthLib)

				if rDateHour != "_NONE_" and rDateHour is not None:
					eventMatch = eventMatch.replace("{{date_begin_hour}}", rDateHour + "h")
				else :
					eventMatch = eventMatch.replace("{{date_begin_hour}}", rDateHour)
				eventMatch = eventMatch.replace("{{date_begin_minute}}",rDateMin)

				dateprec = self.date_event_accurate[self.renderValue(event['date_begin_format'])][ self.renderValue(event['date_begin_accurate']) ]
				eventMatch = eventMatch.replace("{{dateprec}}", dateprec)

				eventMatch = eventMatch.replace("{{et.code}}", event['et.code'] )
				eventMatch = eventMatch.replace("{{et.name}}", event['et.name'] )

				if event['p.name'] is not None and event['p.name'] != '':
					eventMatch = eventMatch.replace("{{p.name}}", event['p.name'])
					eventMatch = eventMatch.replace("{{p.placeprec}}", self.place_accurate[ self.renderValue(event['p.accurate']) ] )
				else:
					eventMatch = eventMatch.replace("{{p.name}}", "_NONE_")
					eventMatch = eventMatch.replace("{{p.placeprec}}", "_NONE_" )

				if event['p.gpslat'] is not None and event['p.gpslat']  != '' :
					eventMatch = eventMatch.replace("{{p.gpslat}}", str(event['p.gpslat']))
				else :
					eventMatch = eventMatch.replace("{{p.gpslat}}", "_NONE_")

				if event['p.gpslon'] is not None and  event['p.gpslon'] != '' :
					eventMatch = eventMatch.replace("{{p.gpslon}}",str(event['p.gpslon']))
				else :
					eventMatch = eventMatch.replace("{{p.gpslon}}", "_NONE_")

				personIconList =  str(event['i.icon']).split(',')
				persontSrcIcon = self.makePath( rootMedia, personIconList[0])

				# Icone Individu
				if persontSrcIcon != '' and persontSrcIcon is not None and os.path.isfile( persontSrcIcon ) :
					personIconUrl = self.renderIcon( rootMedia, personIconList[0], outDirMediaName, matchImageDir, event['person_1_id'], "", matchImageMaxWidth, matchImageMaxHeight)
					personIconUrl = self.makePath ( os.path.basename(outDirMediaName), personIconUrl)
				else :
					if event['sex'] == 'M' :
						personIconUrl = self.makePath( os.path.basename(outDirMediaName), "icons/ico-person-m.png")
					if event['sex'] == 'F' :
						personIconUrl = self.makePath( os.path.basename(outDirMediaName), "icons/ico-person-w.png")
					if event['sex'] == 'I' :
						personIconUrl = self.makePath( os.path.basename(outDirMediaName), "icons/ico-person-i.png")

				eventMatch = eventMatch.replace("{{i.icon}}", str(personIconUrl) )

				# Icone événement
				evenIconList =  str(event['icon']).split(',')
				eventSrcIcon = self.makePath( rootMedia, evenIconList[0])
				if eventSrcIcon != '' and eventSrcIcon is not None and os.path.isfile( eventSrcIcon ) :
					qEventIconUrl = self.renderIcon( rootMedia, evenIconList[0], outDirMediaName, matchImageDir, event['person_1_id'], "icons/ico-event.png", matchImageMaxWidth, matchImageMaxHeight)
					qEventIconUrl = self.makePath ( os.path.basename(outDirMediaName), qEventIconUrl)
					eventMatch = eventMatch.replace("{{icon}}",str( qEventIconUrl))
				else :
					qEventIconUrl = ""
					eventMatch = eventMatch.replace("{{icon}}", "_NONE_")

				if  event['p.gpslat'] is not None and event['p.gpslon'] is not None and event['p.gpslon'] != '' :

					if event['father_displayname'] is not None :
						if cbPrivate is True or event['father_private'] == 0 :
							father_displayname = event['father_displayname']
						else :
							father_displayname = ""
					else:
						father_displayname = ""

					if event['mother_displayname'] is not None :
						if cbPrivate is True or event['mother_private'] == 0 :
							mother_displayname = event['mother_displayname']
						else :
							mother_displayname = ""
					else:
						mother_displayname = ""

					eventMatch = eventMatch.replace("{{father_displayname}}", father_displayname )
					eventMatch = eventMatch.replace("{{mother_displayname}}", mother_displayname )

					try :
						dateEventLib = 	str(int(event['date_begin'][8:10])) + " " + str(self.month_lib[int(event['date_begin'][5:7])]) + " " + str(event['date_begin'][0:4])
					except Exception as e:
						dateEventLib = ""

					fieldSep = '£'

					localmapquery = str(event['id']) + fieldSep +\
									str(event['person_1_id']) + fieldSep +\
									str(event['name']) + fieldSep +\
									str(event['p.gpslat']) + fieldSep +\
									str(event['p.gpslon'])+ fieldSep +\
									str(event['et.code']) + fieldSep +\
									str(event['et.name']) + fieldSep +\
									str(rDateYear) + fieldSep +\
									str(rDateDay) + " " + str(rDateMonthLib) + " " + str(rDateYear) + fieldSep +\
									str(rDateHour) + str(rDateMin) + fieldSep +\
									str(dateprec) + fieldSep +\
									str(event['p.zipcode']) + fieldSep +\
									str(event['p.name']) + fieldSep +\
									str(self.place_accurate[ self.renderValue(event['p.accurate']) ]) + fieldSep +\
									str(event['lastname']) + fieldSep +\
									str(event['firstname']) + fieldSep +\
									str(event['person_2_id']) + fieldSep +\
									str(father_displayname) + fieldSep +\
									str(event['person_3_id']) + fieldSep +\
									str(mother_displayname) + fieldSep +\
									str(personIconUrl) + fieldSep +\
									str(qEventIconUrl)


					from urllib.parse import quote
					eventMatch = eventMatch.replace("{{localmapquery}}",quote(localmapquery))

				else :
					eventMatch = eventMatch.replace("{{localmapquery}}", "_NONE_")

				if event['p.icon'] is not None:
					eventMatch = eventMatch.replace("{{p.icon}}",event['p.icon'])

				# Conserver l'evennement en cours avant de traiter le suivant
				eventMatchList = eventMatchList + eventMatch
				eventMatch = eventMatchR

		# Remplacement de la liste finale des evenemlents
		tplFileContent = tplFileContent.replace( eventMatchR, eventMatchList)
		tplFileContent = tplFileContent.replace( '--E', '')
		tplFileContent = tplFileContent.replace( '--', '')

		# Compteurs d'entités
		tplFileContent = tplFileContent.replace('{{indicount}}', str(len(indicount)))
		tplFileContent = tplFileContent.replace('{{eventcount}}', str(len(eventcount)))
		tplFileContent = tplFileContent.replace('{{sourcecount}}', str(len(sourcecount)))
		tplFileContent = tplFileContent.replace('{{doccount}}', str(len(doccount)))

		tplFileContent = self.databaseMetaInfoFiller( tplFileContent )

		AppLog.log(LOG_INFO, "Insert menu : ", "carte_evenements")
		tplFileContent = self.insertMenu( tplFileContent, "carte_evenements" )

		tplFileContent = self.cleanFinalData( tplFileContent )
		fpo = open(outFile, 'w', encoding='utf8')
		fpo.write(tplFileContent)
		fpo.close()

	# **********************************************************************************
	#	Annuaire - rapport principal, type annuaire: individus -> evennements -> sources
	# **********************************************************************************
	def genRapportPersons(self, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting) :

		AppLog.log( LOG_INFO , "rootMedia = " + str(rootMedia) )
		AppLog.log( LOG_INFO , "dbFile = " + str(dbFile) )

		db = Sqlitedb( dbFile )

		indicount = {}
		eventcount = {}
		sourcecount = {}
		doccount = {}

		indiNum = 0

		myRapport = Rapport( db, cbPrivate )

		tplDir = os.path.dirname(tplFile)
		fpi = open( tplFile, 'r', encoding='utf8')
		tplFileContentR = fpi.read()
		fpi.close()

		outDirMediaName, \
		matchFormat, \
		matchSourcemediaFile, \
		matchSourcemediaType, \
		matchSourcemediaWidth, \
		matchSourcemediaHeight, \
		matchFiles, \
		matchDirs, \
		matchTitle, \
		matchDes, \
		matchImageDir, \
		matchImageMaxHeight, \
		matchImageMaxWidth = self.initTargetFromTemplateMeta( tplFileContentR, tplDir, outFile )

		tplFileContentR = self.cleanTemplateMetas( tplFileContentR )
		tplFileContentR = self.databaseMetaInfoFiller( tplFileContentR )

		tplFileContent = tplFileContentR

		# Bloc global pour un individu
		globalMatchR   = self.getMatchByPattern( tplFileContentR, "%%(.*?)%%" ) # Bloc global - Un individu (y compris sous bloc events)

		# Dans le Bloc global : entête (sans repetition)
		personMatchR   = self.getMatchByPattern( tplFileContentR, ";;I(.*?);;" )

		# Dans le Sous-Bloc : event, sources, doc. (avec repetition)
		blocPersonR    = self.getMatchByPattern( tplFileContentR, "µµ(.*?)µµ" ) # Sous-Bloc - repetition events + sources + docs d'UN individu
		eventMatchR    = self.getMatchByPattern( tplFileContentR, "--E(.*?)--" )
		sourceMatchR   = self.getMatchByPattern( tplFileContentR, "==S(.*?)==" )
		documentMatchR = self.getMatchByPattern( tplFileContentR, ",,D(.*?),," )

		indiList = myRapport.selectINDIAll()
		if indiList is False :
			AppLog.log( LOG_ERROR, +myRapport.lasterror )
		else :
			# Persons ------------------------------------------------------------------------
			for indi in indiList :

				self.printDebugData( 'Person name : #', indi['id'], ' : ' , str(indi['name']))
				indicount[indi['id']] = indicount.get(indi['id'], 0) + 1
				indiNum +=1

				personMatch = personMatchR

				personMatch = personMatch.replace("{{indinum}}", self.rvalue(indiNum) )

				personMatch = personMatch.replace("{{id}}", self.rvalue(indi['id']) )
				personMatch = personMatch.replace("{{key}}", self.rvalue(indi['key']))
				personMatch = personMatch.replace("{{name}}", self.rvalue(indi['name']) )
				
				personMatch = personMatch.replace("{{nameformated}}", self.rvalue(indi['nameformated']) )
				personMatch = personMatch.replace("{{firtsnameothers}}", self.rvalue(indi['firtsnameothers']) )				
				personMatch = personMatch.replace("{{firstname}}", self.rvalue(indi['firstname']) )
				personMatch = personMatch.replace("{{lastname}}", self.rvalue(indi['lastname']) )
				personMatch = personMatch.replace("{{firstname}}", self.rvalue(indi['firstname']) )
				personMatch = personMatch.replace("{{lastname}}", self.rvalue(indi['lastname']) )
				personMatch = personMatch.replace("{{firstnameprefix}}", self.rvalue(indi['firstnameprefix']) )
				personMatch = personMatch.replace("{{lastnamesuffix}}", self.rvalue(indi['lastnamesuffix']) )
				personMatch = personMatch.replace("{{surname}}", self.rvalue(indi['surname']) )
				personMatch = personMatch.replace("{{sex}}", self.rvalue(indi['sex']) )
				personMatch = personMatch.replace ('{{iconsex}}', self.renderIcoSex(indi['sex']))

				rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( indi['e.datebirth'], indi['e.date_birth_format'], indi['e.date_birth_accurate'] )
				personMatch = personMatch.replace("{{datebirth}}",rDate)
				personMatch = personMatch.replace("{{datebirth_day}}", rDateDay)
				personMatch = personMatch.replace("{{datebirth_day_week}}", rDateDayLib)
				personMatch = personMatch.replace("{{datebirth_month}}", rDateMonth)
				personMatch = personMatch.replace("{{datebirth_year}}", rDateYear)
				personMatch = personMatch.replace("{{datebirth_month_lib}}",rDateMonthLib)
				personMatch = personMatch.replace("{{datebirth_begin_hour}}", rDateHour)
				personMatch = personMatch.replace("{{datebirth_begin_minute}}",rDateMin)

				rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( indi['f.datedeath'], indi['e.date_birth_format'], indi['e.date_birth_accurate'], indi['e.datebirth'] )
				personMatch = personMatch.replace("{{datedeath}}",rDate)
				personMatch = personMatch.replace("{{datedeath_day}}", rDateDay)
				personMatch = personMatch.replace("{{datedeath_day_week}}", rDateDayLib)
				personMatch = personMatch.replace("{{datedeath_month}}", rDateMonth)
				personMatch = personMatch.replace("{{datedeath_year}}", rDateYear)
				personMatch = personMatch.replace("{{datedeath_month_lib}}",rDateMonthLib)
				personMatch = personMatch.replace("{{datedeath_begin_hour}}", rDateHour)
				personMatch = personMatch.replace("{{datedeath_begin_minute}}",rDateMin)
				personMatch = personMatch.replace("{{datedeath_years}}", rDeltaYears)

				defIconPath = "icons/ico-person-i.png"
				if indi['sex'] == 'M' : defIconPath = "icons/ico-person-m.png"
				if indi['sex'] == 'F' : defIconPath = "icons/ico-person-w.png"
				personIconUrl = self.renderIcon( rootMedia, indi['icon'], outDirMediaName, matchImageDir, indi['id'], defIconPath, matchImageMaxWidth, matchImageMaxHeight)
				personMatch = personMatch.replace ('{{iconperson}}', personIconUrl)

				personMatch = personMatch.replace("{{description}}", self.rvalue(indi['description']) )

				indiParentList = myRapport.selectIndiParents( indi['id'] )
				if indiParentList :
					for indiParent in indiParentList :
						personMatch = personMatch.replace("{{indimother}}", self.rvalue(indiParent['person_2_id']) )
						personMatch = personMatch.replace("{{mother_displayname}}", self.rvalue(indiParent['mother_displayname']) )
						personMatch = personMatch.replace("{{indifather}}", self.rvalue(indiParent['person_3_id']) )
						personMatch = personMatch.replace("{{father_displayname}}", self.rvalue(indiParent['father_displayname']) )
				else :
					personMatch = personMatch.replace("{{indimother}}", "_NONE_")
					personMatch = personMatch.replace("{{indifather}}", "_NONE_")
					personMatch = personMatch.replace("{{mother_displayname}}", "_NONE_")
					personMatch = personMatch.replace("{{father_displayname}}", "_NONE_")


				# Page(s) des médias de cet individu
				tplMediaTemplate = self.makePath ( os.path.dirname(tplFile), "medias_index.html")
				mediaCount = self.genRapportMedias( indi['id'], dbFile, rootMedia, tplMediaTemplate, outFile, cbPrivate, projSetting)
				# Lien vers cet index.html de la galerie utilisateur, depuis la page Annuaire
				if mediaCount != 0 :
					linkToGaleryIndex = self.makePath( os.path.basename(outDirMediaName), "medias/" + str(indi['id']) + "/index.html")
					personMatch = personMatch.replace("{{mediagalerieindex}}", linkToGaleryIndex)
				else :
					personMatch = personMatch.replace("{{mediagalerieindex}}", "_NONE_")

				personMatch = personMatch.replace("{{mediaCount}}", str(mediaCount))

				personDataSections = ""

				# Events ------------------------------------------------------------------------
				eventList = myRapport.selectEventsByPerson(indi['id'])
				eventMatch = eventMatchR
				if eventList :
					for event in eventList :
						self.printDebugData( '  EV name #' , event['id'], ' :' , str(event['name']))
						eventcount[event['id']] = eventcount.get(event['id'], 0) + 1

						eventMatch = eventMatch.replace("{{id}}", self.rvalue(event['id']) )
						eventMatch = eventMatch.replace("{{name}}", self.rvalue(event['name']) )
						eventMatch = eventMatch.replace("{{description}}", self.rvalue(event['description']) )

						rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( event['date_begin'], event['date_begin_format'],  event['date_begin_accurate'] )
						eventMatch = eventMatch.replace("{{date_begin}}",rDate)
						eventMatch = eventMatch.replace("{{date_begin_day}}", rDateDay)
						eventMatch = eventMatch.replace("{{date_begin_day_week}}", rDateDayLib)
						eventMatch = eventMatch.replace("{{date_begin_month}}", rDateMonth)
						eventMatch = eventMatch.replace("{{date_begin_year}}", rDateYear)
						eventMatch = eventMatch.replace("{{date_begin_month_lib}}",rDateMonthLib)
						if rDateHour != "_NONE_" and rDateHour is not None :
							rDateHour = rDateHour + "h"
						eventMatch = eventMatch.replace("{{date_begin_hour}}", rDateHour)
						eventMatch = eventMatch.replace("{{date_begin_minute}}",rDateMin)

						dateprec = self.date_event_accurate[self.renderValue(event['date_begin_format'])][ self.renderValue(event['date_begin_accurate']) ]
						if dateprec != '' :
							eventMatch = eventMatch.replace("{{eventdateprecision}}", dateprec)
						else :
							eventMatch = eventMatch.replace("{{eventdateprecision}}", "_NONE_")

						eventMatch = eventMatch.replace("{{et.code}}", self.rvalue(event['et.code']) )
						eventMatch = eventMatch.replace("{{et.name}}", self.rvalue(event['et.name']) )
						eventMatch = eventMatch.replace("{{p.name}}", self.rvalue(event['p.name']) )
						eventMatch = eventMatch.replace("{{eventplaceprecision}}", self.place_accurate[ self.renderValue(event['p.accurate']) ] )

						eventMatch = eventMatch.replace("{{zipcode}}", self.rvalue(event['p.zipcode']) )

						eventMatch = eventMatch.replace("{{displayname}}", self.rvalue(event['name']) )

						# MARR
						iconDet = '?'
						if event['et.category'] in ('FAM0', 'FAM1' ) :

							if event['i.id'] != indi['id'] :
								eventMatch = eventMatch.replace("{{i.id}}", self.rvalue(event['i.id']) )
								eventMatch = eventMatch.replace("{{i-1}}", "mariage-m" )
								eventMatch = eventMatch.replace("{{indi_displayname}}", self.rvalue(event['indi_name']) )

							# pas de lien pour l'indiv actuel
							if (cbPrivate is True or event['mother_private'] == 0)  and event['person_2_id'] != '' and event['person_2_id'] is not None and event['person_2_id'] != indi['id'] :
								eventMatch = eventMatch.replace("{{person_2_id}}", self.rvalue(event['person_2_id']) )
								eventMatch = eventMatch.replace("{{mother_displayname}}", self.rvalue(event['mother_displayname']) )
								eventMatch = eventMatch.replace("{{i-2}}", "mariage-w" )

							if (cbPrivate is True or event['father_private'] == 0) and event['person_3_id'] != '' and event['person_3_id'] is not None:
								eventMatch = eventMatch.replace("{{person_3_id}}", self.rvalue(event['person_3_id']) )
								eventMatch = eventMatch.replace("{{father_displayname}}", self.rvalue(event['father_displayname']) )
								eventMatch = eventMatch.replace("{{i-3}}", "i-3" )


							eventMatch = eventMatch.replace("{{i.id}}", "_NONE_" )
							eventMatch = eventMatch.replace("{{person_2_id}}", "_NONE_" )
							eventMatch = eventMatch.replace("{{person_3_id}}", "_NONE_" )

						# BIRT
						if event['et.category'] == 'FAM2' :

							if event['i.id'] != indi['id'] :
								eventMatch = eventMatch.replace("{{i.id}}", self.rvalue(event['i.id']) )
								eventMatch = eventMatch.replace("{{indi_displayname}}", self.rvalue(event['indi_name']) )
								defIconName = "mariage-i"
								if event['i.sex'] == 'M' : defIconName = "mariage-m"
								if event['i.sex'] == 'F' : defIconName = "mariage-w"
								eventMatch = eventMatch.replace("{{i-1}}", defIconName )

						eventMatch = eventMatch.replace("{{i.id}}", "_NONE_"  )
						eventMatch = eventMatch.replace("{{person_2_id}}", "_NONE_" )
						eventMatch = eventMatch.replace("{{person_3_id}}", "_NONE_" )

						eventMatch = eventMatch.replace("{{p.gpslat}}", self.rvalue(event['p.gpslat']) )
						eventMatch = eventMatch.replace("{{p.gpslon}}", self.rvalue(event['p.gpslon']) )

						qEventIcon = self.renderIcon( rootMedia, event['icon'], outDirMediaName, matchImageDir, indi['id'], "_NONE_", matchImageMaxWidth, matchImageMaxHeight)
						eventMatch = eventMatch.replace ('{{iconevent}}', qEventIcon)

						# Url for "localmap" display
						if  event['p.gpslat'] is not None and event['p.gpslon'] is not None and event['p.gpslon'] != '' :

							fieldSep = '£'
							localmapquery = str(event['id']) + fieldSep +\
											str(indi['id']) + fieldSep +\
											str(event['name']) + fieldSep +\
											str(event['p.gpslat']) + fieldSep +\
											str(event['p.gpslon'])+ fieldSep +\
											str(event['et.code']) + fieldSep +\
											str(event['et.name']) + fieldSep +\
											str(rDateYear) + fieldSep +\
											str(rDateDay) + " " + str(rDateMonthLib) + " " + str(rDateYear) + fieldSep +\
											str(rDateHour) + str(rDateMin) + fieldSep +\
											str(dateprec) + fieldSep +\
											str(event['p.zipcode']) + fieldSep +\
											str(event['p.name']) + fieldSep +\
											str(self.place_accurate[ self.renderValue(event['p.accurate']) ]) + fieldSep +\
											str(event['indi_lastname']) + fieldSep +\
											str(event['indi_firstname']) + fieldSep +\
											str(self.renderValue(event['father_displayname'], '')) + fieldSep +\
											str(self.renderValue(event['mother_displayname'], '')) + fieldSep +\
											str(personIconUrl) + fieldSep +\
											str(qEventIcon)

							eventMatch = eventMatch.replace("{{localmapquery}}",quote(localmapquery))

							try :
								zoom = int(event['p.zoom'])
							except Exception as e:
								zoom = 12
							eventMatch = eventMatch.replace("{{mapzoom}}",quote(str(zoom)))

						else :
							eventMatch = eventMatch.replace("{{localmapquery}}", "_NONE_")

						personDataSections = personDataSections +  eventMatch
						eventMatch = eventMatchR

						# Sources ------------------------------------------------------------------------
						sourceList = myRapport.selectSourcesByEvent( event['id'] )
						if sourceList :
							for source in sourceList :

								sourceMatch = sourceMatchR

								self.printDebugData('    SOURCE # ', str(source['id']), ' : ' , str(source['name']))
								sourcecount[source['id']] = sourcecount.get(source['id'], 0) + 1

								sourceMatch = sourceMatch.replace("{{id}}", self.rvalue(source['id']) )
								sourceMatch = sourceMatch.replace("{{name}}", self.rvalue(source['name']) )
								sourceMatch = sourceMatch.replace("{{page}}", self.rvalue(source['page']) )
								sourceMatch = sourceMatch.replace("{{citation}}", self.rvalue(source['citation']) )
								sourceMatch = sourceMatch.replace("{{traduction}}", self.rvalue(source['textdraduction']) )
								sourceMatch = sourceMatch.replace("{{description}}", self.rvalue(source['description']) )
								sourceMatch = sourceMatch.replace("{{quality}}", self.rvalue(source['quality']) )

								qualitylib = self.source_quality[ self.renderValue(source['quality']) ]
								sourceMatch = sourceMatch.replace("{{qualitylib}}", qualitylib )

								sourceIndiSubdir = self.makePath(outDirMediaName,  "sources/" + str(indi['id']) )
								AppLog.log(LOG_INFO, "mkdir sourceIndiSubdir : " + str(sourceIndiSubdir))

								try:
									os.makedirs( sourceIndiSubdir, exist_ok=True ) # -p equivalent
								except Exception as e:
									AppLog.log( LOG_ERROR, str(e))

								# 1/3 Source type text
								sourceMatch = sourceMatch.replace("{{sourcetext}}", self.rvalue(source['sourcetext']) )

								# 2/3 Source type url+file
								if source['sourceurl'] is not None and source['sourceurl'] != "" :
									# copy media file
									try:
										mediaUrlSrcDir = self.makePath(rootMedia, source['sourceurl'])
										mediaUrlDstDir = self.makePath(sourceIndiSubdir, os.path.basename(source['sourceurl']))
										mediaUrlDstDirLink = self.makePath( os.path.basename(outDirMediaName), "sources/" + str(indi['id']) + '/' + str(os.path.basename(source['sourceurl'])))
										AppLog.log( LOG_INFO, "Copy media file : " + str(mediaUrlSrcDir) + "  to :" + str(mediaUrlDstDir) )
										try:
											shutil.copyfile( mediaUrlSrcDir , mediaUrlDstDir)
										except Exception as e:
											AppLog.log( LOG_ERROR, str(e) )

									except Exception as e:
										AppLog.log( LOG_ERROR, str(e))

									sourceMatch = sourceMatch.replace("{{sourceurl}}", self.rvalue(mediaUrlDstDirLink) )
									sourceMatch = sourceMatch.replace("{{sourceicon}}", self.getFileTypeIcon( source['sourceurl'] ) )
									sourceMatch = sourceMatch.replace("{{sourcetitle}}", os.path.basename( self.rvalue(source['sourceurl']) ) )

								# 3/3 Source type images locales
								if source['localurl'] is not None and source['localurl'] != "" :

									# Annotations
									localUrlList = str(source['localurl']).split(',')
									sourceMatchImageR = self.getMatchByPattern(tplFileContentR, "££u(.*?)££")
									sourceMatchImageFinal = ""

									# Lien vers le premier média de la source
									mediaUrl = "NONE"
									for localUrlelem in localUrlList :
										localUrlelem = self.makePath( rootMedia,localUrlelem)
										if localUrlelem != "None":
											mediaUrlBase, mediaUrlBaseExt = os.path.splitext(os.path.basename(str(localUrlList[0])))
											mediaUrl = os.path.basename(mediaUrlBase)
										break

									sourceIndiFirstUrl = self.makePath( os.path.basename(outDirMediaName), "sources/" + str(indi['id']) + '/' + str(mediaUrl + ".html"))

									self.printDebugData("sourceIndiFirstUrl : ", sourceIndiFirstUrl)

									sourceMatch = sourceMatch.replace("{{localurl}}", sourceIndiFirstUrl )
									sourceMatch = sourceMatch.replace("{{localurlname}}", "Src.")

									mediaCount = 0
									mediaMax = len(localUrlList)
									mediaIndex = 1

									# Pour chaque image de la source

									mediaUrlDstDir = ""
									for localUrlelem in localUrlList :

										localUrlelem = self.makePath( rootMedia,localUrlelem)
										self.printDebugData('      photo = ', os.path.basename(localUrlelem))

										if not os.path.isfile(localUrlelem ) :
											AppLog.log( LOG_WARNING, "file not found", str(localUrlelem))

										if localUrlelem != "None" and os.path.isfile(localUrlelem ) :

											mediaCount = mediaCount + 1

											medianext = mediaIndex + 1
											mediaprev = mediaIndex - 1

											if medianext > mediaMax :
												medianext = mediaMax

											if mediaprev < 1 :
												mediaprev = 1

											mediaIndex = mediaIndex + 1

											mediaUrlDstDir = self.makePath(sourceIndiSubdir, os.path.basename(localUrlelem))

											img = Image.open(localUrlelem)
											imW = int(img.width)
											imH = int(img.height)
											imRatio = imW / imH

											if matchSourcemediaType == "A" :
												# MediaManager. Media files with annotations
												AppLog.log( LOG_INFO, "Creation : ", localUrlelem)
												from libwidget.photomanager import PhotoManagerFile
												myPhoto = PhotoManagerFile(self)
												myPhoto.toFile( localUrlelem, mediaUrlDstDir, matchSourcemediaWidth, matchSourcemediaHeight)
											else :
												if matchSourcemediaWidth == 0 and matchSourcemediaHeight == 0 :
													try:
														shutil.copyfile(str(localUrlelem), mediaUrlDstDir)
													except Exception as e:
														AppLog.log( LOG_ERROR, str(e) )

													im = Image.open(str(localUrlelem))
													image_size = im.size
													imagefileWidth = image_size[0]
													imagefileHeight = image_size[1]

												else :
													im = Image.open(str(localUrlelem))
													image_size = im.size
													imWidth = image_size[0]
													imHeight = image_size[1]
													imRatio = imWidth / imHeight
													if matchSourcemediaWidth == 0:
														imW = int( matchSourcemediaHeight * imRatio )
														imH = int(matchSourcemediaHeight)
													else:
														imW = int(matchSourcemediaWidth)
														imH = int( matchSourcemediaWidth / imRatio )

													im.thumbnail( (imW, imH ))

													filename, fileext = os.path.splitext(localUrlelem)
													fileext = fileext[1:]
													if fileext == "jpg" :
														im.save(mediaUrlDstDir, "JPEG", quality=100)
													else :
														im.save(mediaUrlDstDir, fileext.upper() )

													image = Image.open(mediaUrlDstDir)
													imagefileWidth, imagefileHeight = image.size

										else :
											AppLog.log( LOG_WARNING, "mediaUrl : None")

										sourceMatchImage = sourceMatchImageR
										filename, fileext = os.path.splitext(localUrlelem)
										fileext = fileext[1:]

										# Source photo template ---------------------------------------------------------
										mediaTempatleFile = self.makePath(os.path.dirname(tplFile), matchSourcemediaFile)
										tplFileContentMedia = open(mediaTempatleFile, 'r', encoding='utf8').read()

										# Souce subdir destination
										mediaTemplateUrlDstDir = os.path.dirname(mediaUrlDstDir)
										mediaTemplateUrlDstDir = self.makePath(mediaTemplateUrlDstDir, sourceIndiSubdir)
										self.printDebugData("Source subdir destination: ", mediaTemplateUrlDstDir)

										# Page média : zone des annotations
										sourceMatchAnnotationR = self.getMatchByPattern(tplFileContentMedia, "@@(.*?)@@")
										sourceMatchAnnotation = sourceMatchAnnotationR

										# Page média : zone menu des annotations
										sourceMatchMenuAnnotationR = self.getMatchByPattern(tplFileContentMedia, "==(.*?)==")
										sourceMatchMenuAnnotation = sourceMatchMenuAnnotationR

										# Page *principale* : zone des annotations
										fileMatchAnnotationR = self.getMatchByPattern(tplFileContent, "@@(.*?)@@")
										fileMatchAnnotation = fileMatchAnnotationR

										# Annotations -------------------------------------------------------------------
										if os.path.isfile(localUrlelem + ".shp"):

											img = Image.open(localUrlelem)

											# Loop annotations sections
											config = configparser.ConfigParser()
											config.read(localUrlelem + ".shp")
											templateFileName, templateFileExt = os.path.splitext(mediaUrlDstDir)
											templateFileName = os.path.basename(templateFileName)

											annoRatioX =  imagefileWidth / img.width
											annoRatioY = imagefileHeight / img.height

											for each_section in config.sections():

												self.printDebugData("        annotation, titre = " + str(config[each_section]['title']).replace("'","\\x27"))

												# Dans la page rapport
												fileMatchAnnotation = fileMatchAnnotation.replace("{{annoid}}", str(config[each_section]['id']))

												fileMatchAnnotation = fileMatchAnnotation.replace("{{annotitle}}", self.renderValue(config[each_section]['title'], config[each_section]['id']) )
												fileMatchAnnotation = fileMatchAnnotation.replace("{{annotext}}", str(config[each_section]['text']))

												# Dans la page photo
												sourceMatchAnnotation = sourceMatchAnnotation.replace("{{annoid}}", str(config[each_section]['id']))
												sourceMatchAnnotation = sourceMatchAnnotation.replace("{{annotitle}}", self.renderValue(config[each_section]['title'], config[each_section]['id']).replace("'","\\x27").replace("\n","<br>"))
												sourceMatchAnnotation = sourceMatchAnnotation.replace("{{annotext}}", str(config[each_section]['text']).replace("'","\\x27").replace("\n","<br>"))
												sourceMatchAnnotation = sourceMatchAnnotation.replace("{{annocolor}}", str(config[each_section]['itemcolor'][3:9]))

												transparency = int(config[each_section]['itemcolor'][1:3], 16)
												transparency = int ((transparency / 255) * 100)
												sourceMatchAnnotation = sourceMatchAnnotation.replace("{{annotransparency}}", str(transparency))

												# Dans la page photo (menu de toutes les annotation, colonne de gauche)
												sourceMatchMenuAnnotation = sourceMatchMenuAnnotation.replace("{{annoid}}", str(config[each_section]['id']))
												sourceMatchMenuAnnotation = sourceMatchMenuAnnotation.replace("{{annotitle}}", self.renderValue(config[each_section]['title'], config[each_section]['id']).replace("\n","<br>"))
												sourceMatchMenuAnnotation = sourceMatchMenuAnnotation.replace("{{annotext}}", str(config[each_section]['text']).replace("'","\\x27").replace("\n","<br>"))

												annotTopX = (img.width / 2) - ( float(config[each_section]['w']) / 2) + float(config[each_section]['dx'])
												annotTopY = (img.height / 2) - ( float(config[each_section]['h']) / 2) - float(config[each_section]['dy'])
												annotBottomX = annotTopX + float(config[each_section]['w'])
												annotBottomY = annotTopY + float(config[each_section]['h'])

												annotTopX = int(annotTopX * annoRatioX)
												annotTopY = int(annotTopY * annoRatioY)
												annotBottomX = int(annotBottomX * annoRatioX)
												annotBottomY = int(annotBottomY * annoRatioY)

												sourceMatchAnnotation = sourceMatchAnnotation.replace("{{topx}}", str(int(annotTopX)))
												sourceMatchAnnotation = sourceMatchAnnotation.replace("{{topy}}", str(int(annotTopY)))
												sourceMatchAnnotation = sourceMatchAnnotation.replace("{{bottomx}}", str(int(annotBottomX)))
												sourceMatchAnnotation = sourceMatchAnnotation.replace("{{bottomy}}", str(int(annotBottomY)))

												sourceMatchAnnotation = sourceMatchAnnotation + sourceMatchAnnotationR
												sourceMatchMenuAnnotation = sourceMatchMenuAnnotation + sourceMatchMenuAnnotationR
												fileMatchAnnotation = fileMatchAnnotation + fileMatchAnnotationR

											tplFileContentMedia = tplFileContentMedia.replace(sourceMatchAnnotationR, sourceMatchAnnotation)
											tplFileContentMedia = tplFileContentMedia.replace(sourceMatchAnnotationR, "")

											tplFileContentMedia = tplFileContentMedia.replace(sourceMatchMenuAnnotationR, sourceMatchMenuAnnotation)
											tplFileContentMedia = tplFileContentMedia.replace(sourceMatchMenuAnnotationR, "")

											tplFileContentMedia = tplFileContentMedia.replace("@@", "")
											tplFileContentMedia = tplFileContentMedia.replace("==", "")
											tplFileContentMedia = tplFileContentMedia.replace("{{mediaurl}}", os.path.basename(mediaUrlDstDir))
											tplFileContentMedia = tplFileContentMedia.replace("{{mediawidth}}", str(imagefileWidth ))
											tplFileContentMedia = tplFileContentMedia.replace("{{mediaheight}}", str(imagefileHeight))
											tplFileContentMedia = tplFileContentMedia.replace("{{filename}}", os.path.basename(mediaUrlDstDir))
											tplFileContentMedia = tplFileContentMedia.replace("{{sourcename}}", os.path.basename(source['name']))
											tplFileContentMedia = tplFileContentMedia.replace("{{sourceid}}", str(source['id']))
											tplFileContentMedia = tplFileContentMedia.replace("{{sourcedescription}}", str(self.rvalue(source['description'])) )

											tplFileContentMedia = tplFileContentMedia.replace("{{page}}", self.rvalue(source['page']))
											tplFileContentMedia = tplFileContentMedia.replace("{{citation}}", self.rvalue(source['citation']))

											tplFileContentMedia = tplFileContentMedia.replace("{{sourceurl}}", self.rvalue(source['sourceurl']) )
											tplFileContentMedia = tplFileContentMedia.replace("{{sourceicon}}", self.getFileTypeIcon( source['sourceurl'] ) )
											tplFileContentMedia = tplFileContentMedia.replace("{{sourcetitle}}", os.path.basename( self.rvalue(source['sourceurl']) ) )

											qualitylib = self.source_quality[ self.renderValue(source['quality']) ]
											tplFileContentMedia = tplFileContentMedia.replace("{{qualitylib}}", qualitylib )
											tplFileContentMedia = tplFileContentMedia.replace("{{quality}}", self.rvalue(source['quality']) )

											tplFileContentMedia = tplFileContentMedia.replace("{{eventid}}", str(event['id']))
											tplFileContentMedia = tplFileContentMedia.replace("{{eventname}}", str(event['name']))

											tplFileContentMedia = tplFileContentMedia.replace("{{p.name}}", self.rvalue(event['p.name']) )
											tplFileContentMedia = tplFileContentMedia.replace("{{eventplaceprecision}}", self.place_accurate[ self.renderValue(event['p.accurate']) ] )

											tplFileContentMedia = tplFileContentMedia.replace("{{zipcode}}", self.rvalue(event['p.zipcode']) )

											rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( event['date_begin'], event['date_begin_format'], event['date_begin_accurate'] )
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate}}",rDate)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_day}}", rDateDay)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_day_week}}", rDateDayLib)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_month}}", rDateMonth)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_year}}", rDateYear)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_month_lib}}",rDateMonthLib)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_hour}}", rDateHour)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_minute}}",rDateMin)

											# page média avec anno
											dateprec = self.date_event_accurate[self.renderValue(event['date_begin_format'])][ self.renderValue(event['date_begin_accurate']) ]
											if dateprec != '' :
												tplFileContentMedia = tplFileContentMedia.replace("{{eventdateprecision}}", dateprec)
											else :
												tplFileContentMedia = tplFileContentMedia.replace("{{eventdateprecision}}", "_NONE_")

											mediaUrlBaseCurrent, mediaUrlBaseExtCurrent = os.path.splitext( os.path.basename(str(mediaUrlDstDir)) )
											mediaUrlBaseCurrent = mediaUrlBaseCurrent + ".html"

											mediaUrlBase, mediaUrlBaseExt = os.path.splitext( os.path.basename(str(localUrlList[mediaprev-1])) )
											mediaprevLink = os.path.basename(mediaUrlBase + ".html")

											if mediaUrlBaseCurrent !=  mediaprevLink :
												tplFileContentMedia = tplFileContentMedia.replace("{{mediaprev}}",  mediaprevLink )
												tplFileContentMedia = tplFileContentMedia.replace("{{mediaprevtext}}",  "Prec." )
											else :
												tplFileContentMedia = tplFileContentMedia.replace("{{mediaprev}}",  "_NONE_" )
												tplFileContentMedia = tplFileContentMedia.replace("{{mediaprevtext}}",  "_NONE_" )

											mediaUrlBase, mediaUrlBaseExt = os.path.splitext( os.path.basename(str(localUrlList[medianext-1])) )
											medianextLink = os.path.basename(mediaUrlBase+ ".html")

											if mediaUrlBaseCurrent !=  medianextLink :
												tplFileContentMedia = tplFileContentMedia.replace("{{medianext}}", medianextLink )
												tplFileContentMedia = tplFileContentMedia.replace("{{medianexttext}}", "Suiv." )
											else :
												tplFileContentMedia = tplFileContentMedia.replace("{{medianext}}", "_NONE_" )
												tplFileContentMedia = tplFileContentMedia.replace("{{medianexttext}}", "_NONE_" )

											tplFileContentMedia = tplFileContentMedia.replace("{{mediaCount}}", str(mediaCount) )
											tplFileContentMedia = tplFileContentMedia.replace("{{mediaMax}}", str(mediaMax) )

											outFileIndex = os.path.basename(str(outFile))
											tplFileContentMedia = tplFileContentMedia.replace("{{outFile}}",  self.makePath("..", str(outFileIndex)) )
											tplFileContentMedia = tplFileContentMedia.replace("{{indianchor}}", str(indi['id']))

										# Source without annotations
										else :

											templateFileName, templateFileExt = os.path.splitext(mediaUrlDstDir)
											templateFileName = os.path.basename(templateFileName)
											outFileIndex = os.path.basename(str(outFile))
											tplFileContentMedia = tplFileContentMedia.replace("{{outFile}}",  self.makePath("..", str(outFileIndex)) )
											tplFileContentMedia = tplFileContentMedia.replace("{{indianchor}}", str(indi['id']))

											mediaUrlBaseCurrent, mediaUrlBaseExtCurrent = os.path.splitext( os.path.basename(str(mediaUrlDstDir)) )
											mediaUrlBaseCurrent = mediaUrlBaseCurrent + ".html"

											mediaUrlBase, mediaUrlBaseExt = os.path.splitext( os.path.basename(str(localUrlList[mediaprev-1])) )
											mediaprevLink = os.path.basename(mediaUrlBase + ".html")

											if mediaUrlBaseCurrent !=  mediaprevLink :
												tplFileContentMedia = tplFileContentMedia.replace("{{mediaprev}}",  mediaprevLink )
												tplFileContentMedia = tplFileContentMedia.replace("{{mediaprevtext}}",  "Prec." )
											else :
												tplFileContentMedia = tplFileContentMedia.replace("{{mediaprev}}",  "_NONE_" )
												tplFileContentMedia = tplFileContentMedia.replace("{{mediaprevtext}}",  "_NONE_" )

											mediaUrlBase, mediaUrlBaseExt = os.path.splitext( os.path.basename(str(localUrlList[medianext-1])) )
											medianextLink = os.path.basename(mediaUrlBase+ ".html")

											if mediaUrlBaseCurrent !=  medianextLink :
												tplFileContentMedia = tplFileContentMedia.replace("{{medianext}}", medianextLink )
												tplFileContentMedia = tplFileContentMedia.replace("{{medianexttext}}", "Suiv." )
											else :
												tplFileContentMedia = tplFileContentMedia.replace("{{medianext}}", "_NONE_" )
												tplFileContentMedia = tplFileContentMedia.replace("{{medianexttext}}", "_NONE_" )

											tplFileContentMedia = tplFileContentMedia.replace("{{mediaCount}}", str(mediaCount) )
											tplFileContentMedia = tplFileContentMedia.replace("{{mediaMax}}", str(mediaMax) )

											tplFileContentMedia = tplFileContentMedia.replace("{{mediaurl}}", os.path.basename(mediaUrlDstDir))
											tplFileContentMedia = tplFileContentMedia.replace("{{mediawidth}}", str(imagefileWidth ))
											tplFileContentMedia = tplFileContentMedia.replace("{{mediaheight}}", str(imagefileHeight))
											tplFileContentMedia = tplFileContentMedia.replace("{{filename}}", os.path.basename(mediaUrlDstDir))
											tplFileContentMedia = tplFileContentMedia.replace("{{sourcename}}", os.path.basename(source['name']))
											tplFileContentMedia = tplFileContentMedia.replace("{{sourceid}}", str(source['id']))
											tplFileContentMedia = tplFileContentMedia.replace("{{sourcedescription}}", str(self.rvalue(source['description'])))

											tplFileContentMedia = tplFileContentMedia.replace("{{page}}", self.rvalue(source['page']))
											tplFileContentMedia = tplFileContentMedia.replace("{{citation}}", self.rvalue(source['citation']))

											tplFileContentMedia = tplFileContentMedia.replace("{{sourceurl}}", self.rvalue(source['sourceurl']) )
											tplFileContentMedia = tplFileContentMedia.replace("{{sourceicon}}", self.getFileTypeIcon( source['sourceurl'] ) )
											tplFileContentMedia = tplFileContentMedia.replace("{{sourcetitle}}", os.path.basename( self.rvalue(source['sourceurl']) ) )

											qualitylib = self.source_quality[ self.renderValue(source['quality']) ]
											tplFileContentMedia = tplFileContentMedia.replace("{{qualitylib}}", qualitylib )
											tplFileContentMedia = tplFileContentMedia.replace("{{quality}}", self.rvalue(source['quality']) )

											tplFileContentMedia = tplFileContentMedia.replace("{{eventid}}", str(event['id']))
											tplFileContentMedia = tplFileContentMedia.replace("{{eventname}}", str(event['name']))

											tplFileContentMedia = tplFileContentMedia.replace("{{p.name}}", self.rvalue(event['p.name']) )
											tplFileContentMedia = tplFileContentMedia.replace("{{eventplaceprecision}}", self.place_accurate[ self.renderValue(event['p.accurate']) ] )

											tplFileContentMedia = tplFileContentMedia.replace("{{zipcode}}", self.rvalue(event['p.zipcode']) )

											rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( event['date_begin'], event['date_begin_format'],  event['date_begin_accurate'] )
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate}}",rDate)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_day}}", rDateDay)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_day_week}}", rDateDayLib)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_month}}", rDateMonth)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_year}}", rDateYear)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_month_lib}}",rDateMonthLib)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_hour}}", rDateHour)
											tplFileContentMedia = tplFileContentMedia.replace("{{eventdate_minute}}",rDateMin)

											# page media (sans annotation)
											dateprec = self.date_event_accurate[self.renderValue(event['date_begin_format'])][ self.renderValue(event['date_begin_accurate']) ]
											if dateprec != '' :
												tplFileContentMedia = tplFileContentMedia.replace("{{eventdateprecision}}", dateprec)
											else :
												tplFileContentMedia = tplFileContentMedia.replace("{{eventdateprecision}}", "_NONE_")

											# Nettoyer la page des blocs d'annnotations non necessaires
											tplFileContentMedia = re.sub('==(.*)==', '', tplFileContentMedia, 0, re.MULTILINE | re.DOTALL)
											tplFileContentMedia = re.sub('@@(.*)@@', '', tplFileContentMedia, 0, re.MULTILINE | re.DOTALL)

										tplFileContentMedia = self.databaseMetaInfoFiller( tplFileContentMedia )

										# Erase empty placeholders in photo html file
										tplFileContentMedia = tplFileContentMedia.replace(sourceMatchMenuAnnotationR, "")
										tplFileContentMedia = tplFileContentMedia.replace("==", "")
										tplFileContentMedia = tplFileContentMedia.replace(sourceMatchAnnotationR, "")
										tplFileContentMedia = tplFileContentMedia.replace("@@", "")

										tplFileContentMedia = re.sub('(?<=<ifnotnull>)(?:.(?!<ifnotnull>))*NONE.*(?=</ifnotnull>)', '', tplFileContentMedia)
										tplFileContentMedia = tplFileContentMedia.replace('<ifnotnull>','')
										tplFileContentMedia = tplFileContentMedia.replace('</ifnotnull>', '')

										AppLog.log(LOG_INFO, "Insert menu : ", "annuaire")
										tplFileContentMedia = self.insertMenu( tplFileContentMedia, "annuaire" )

										AppLog.log( LOG_INFO , "Creation media file: " + templateFileName + ".html" )

										f = open( self.makePath(mediaTemplateUrlDstDir, templateFileName + ".html" ), "w", encoding='utf8')
										f.write(tplFileContentMedia)
										f.close()

										sourceMatchImage = sourceMatchImage.replace(sourceMatchAnnotationR, sourceMatchAnnotation)
										sourceMatchImage = sourceMatchImage.replace("@@", "")

										mediaUrlBase, mediaUrlBaseExt = os.path.splitext(os.path.basename(str(mediaUrl)))
										sourceMatch = sourceMatch.replace("{{localurl}}", self.makePath(os.path.basename(outDirMediaName), str(mediaUrl + ".html")))

										sourceMatchImage = sourceMatchImage.replace("{{localurlname}}", "Lien")

										sourceMatchImageFinal = sourceMatchImageFinal + sourceMatchImage

										fileMatchAnnotation = fileMatchAnnotation.replace(fileMatchAnnotationR, "")

										sourceMatchImageFinal = sourceMatchImageFinal.replace(fileMatchAnnotationR, fileMatchAnnotation)

									sourceMatch = sourceMatch + sourceMatchImageR
									sourceMatch = sourceMatch.replace(sourceMatchImageR, sourceMatchImageFinal)

								else :
									sourceMatch = sourceMatch.replace("{{localurl}}", "_NONE_")
									sourceMatch = sourceMatch.replace("{{localurlname}}", "_NONE_")

								sourceIconUrl = self.renderIcon( rootMedia, source['icon'], outDirMediaName, matchImageDir, indi['id'], "_NONE_", matchImageMaxWidth, matchImageMaxHeight)
								sourceMatch = sourceMatch.replace ('{{iconsource}}', sourceIconUrl)

								personDataSections = personDataSections +  sourceMatch

								# End source type image local

								# Documents
								documentList = myRapport.selectDocumementsBySource( event['id'], source['id'] )
								documentMatch = documentMatchR
								if documentList :
									for document in documentList :
										self.printDebugData( '       DOC #', document['id'], ' name = ' , str(document['name']))
										doccount[document['id']] = doccount.get(document['id'], 0) + 1

										documentMatch = documentMatch.replace("{{id}}", self.rvalue(document['id']) )
										documentMatch = documentMatch.replace("{{name}}", self.rvalue(document['name']) )
										documentMatch = documentMatch.replace("{{typename}}", self.rvalue(document['typename']) )
										documentMatch = documentMatch.replace("{{depot}}", self.rvalue(document['depot']) )
										documentMatch = documentMatch.replace("{{description}}", self.rvalue(document['description']) )
										documentMatch = documentMatch.replace("{{cote}}", self.rvalue(document['cote']) )
										documentMatch = documentMatch.replace("{{url}}", self.rvalue(document['url']) )

										sourceIconUrl = self.renderIcon( rootMedia, document['icon'], outDirMediaName, matchImageDir, indi['id'], "_NONE_", matchImageMaxWidth, matchImageMaxHeight)
										documentMatch = documentMatch.replace ('{{icon}}', sourceIconUrl)

										sourceIconUrl = self.renderIcon( rootMedia, document['t.icon'], outDirMediaName, matchImageDir, indi['id'], "icons/link-document.png", matchImageMaxWidth, matchImageMaxHeight)
										documentMatch = documentMatch.replace ('{{t.icon}}', sourceIconUrl)

										personDataSections = personDataSections +  documentMatch
										documentMatch = documentMatchR

									# print('documentMatchFinal')

							# print('sourceMatchFinal')

					# print('eventMatchFinal')

				# End each person
				self.printDebugData("--------------- end person")

				# ----------- Mini-arbre des proches ascendants
				levelMax = 1
				svgurl = self.rapportIndiTree( indi['id'], levelMax, dbFile, rootMedia, tplDir, tplFile, outFile, cbPrivate, projSetting)

				# ----------- Fiche individuelle
				tplFicheTemplate = self.makePath ( os.path.dirname(tplFile), "fiche.html")
				self.genRapportPersonFiche( indi['id'], dbFile, rootMedia, tplFicheTemplate, outFile, cbPrivate, projSetting, svgurl)

				# Remplacement final pour un individu
				tplFileContent = tplFileContent.replace( personMatchR, personMatch )
				tplFileContent = tplFileContent.replace( blocPersonR, personDataSections )

				tplFileContent = tplFileContent.replace( "{{NEXTINDI}}", globalMatchR + "{{NEXTINDI}}" )

		#print("** FINAL FILE : ", globalMatchR)
		tplFileContent = tplFileContent.replace( globalMatchR, '')

		tplFileContent = tplFileContent.replace( ';;I', '')
		tplFileContent = tplFileContent.replace( ';;', '')
		tplFileContent = tplFileContent.replace( '%%', '')
		tplFileContent = tplFileContent.replace( 'µµ', '')

		# Compteurs d'entités
		tplFileContent = tplFileContent.replace('{{indicount}}', str(len(indicount)))
		tplFileContent = tplFileContent.replace('{{eventcount}}', str(len(eventcount)))
		tplFileContent = tplFileContent.replace('{{sourcecount}}', str(len(sourcecount)))
		tplFileContent = tplFileContent.replace('{{doccount}}', str(len(doccount)))

		AppLog.log(LOG_INFO, "Insert menu : ", "annuaire")
		tplFileContent = self.insertMenu( tplFileContent, "annuaire" )

		tplFileContent = tplFileContent.replace( '{{NEXTINDI}}', '')
		tplFileContent = self.cleanFinalData(tplFileContent)

		fpo = open(outFile, 'w', encoding='utf8')
		fpo.write(tplFileContent)
		fpo.close()

		# Fichiers complementaires
		self.copyTemplateFile( self.makePath( tplDir ,"map.html") , self.makePath( outDirMediaName, "map.html"), "annuaire")


	def getMediaCountByPerson( self, indiId, dbFile, cbPrivate  ) :

		mediaCount = -1

		db = Sqlitedb( dbFile )
		myRapport = Rapport( db, cbPrivate )

		mediaCount = myRapport.getMediaCountByPerson( indiId )

		return mediaCount


	# ***************************************************************
	#	Médias - rapport des médias d'un individu
	# ***************************************************************
	def genRapportMedias( self, indiId, dbFile, rootMedia, tplFile, outFile, cbPrivate, projSetting) :

		AppLog.log( LOG_INFO , "rootMedia = " + str(rootMedia) )
		AppLog.log( LOG_INFO , "dbFile = " + str(dbFile) )

		mediaCount = 0

		db = Sqlitedb( dbFile )
		myRapport = Rapport( db, cbPrivate )

		tplDir = os.path.dirname(tplFile)

		self.printDebugData("  MEDIAS indi id=", indiId)
		mediaList = myRapport.selectmediasByPerson( indiId )
		if mediaList :

			# Create dir for medias
			outDirName = os.path.dirname(outFile)
			(outFileBaseName, outFileExt) = os.path.splitext(outFile)
			outDirMediaName = self.makePath(outDirName, outFileBaseName)

			mediasDstDir = self.makePath(outDirMediaName, "medias/"+str(indiId))
			AppLog.log(LOG_INFO, "mkdir mediasDstDir : ", mediasDstDir)
			try:
				os.makedirs( mediasDstDir, exist_ok=True ) # -p equivalent
			except Exception as e:
				AppLog.log( LOG_ERROR, str(e) )

			# -----------------------------
			# INDEX : Create main medias index html
			AppLog.log( LOG_INFO, "Open index media template :", tplFile)
			fpi = open( tplFile, 'r', encoding='utf8')
			mediaTemplateIndexR = fpi.read()
			fpi.close()

			outDirMediaName, \
			matchFormat, \
			matchSourcemediaFile, \
			matchSourcemediaType, \
			matchSourcemediaWidth, \
			matchSourcemediaHeight, \
			matchFiles, \
			matchDirs, \
			matchTitle, \
			matchDes, \
			matchImageDir, \
			matchImageMaxHeight, \
			matchImageMaxWidth = self.initTargetFromTemplateMeta( mediaTemplateIndexR, tplDir, outFile )

			mediaTemplateIndexR = self.cleanTemplateMetas( mediaTemplateIndexR )
			mediaTemplateIndex = mediaTemplateIndexR

			mediaTemplateIndex = self.databaseMetaInfoFiller( mediaTemplateIndex )

			# Extract "each" media sub template
			mediaIndexMatchR   = self.getMatchByPattern( mediaTemplateIndex,  "°°M(.*?)°°"  )
			# -----------------------------

			# Each media : create thumbs, index ...
			for media in mediaList :

				mediaCount = mediaCount + 1
				mediaIndexMatch = mediaIndexMatchR

				self.printDebugData("   #" + str(media['id']) + ' ' + str(media['displayname']) + ' [' + str(media['url']) + ']' )

				# Copy and overwrite media
				mediaSrc = self.makePath( rootMedia, str(media['url']))
				mediaDst = self.makePath( mediasDstDir, os.path.basename(str(media['url'])))
				AppLog.log(LOG_INFO, "Copy: ", mediaSrc, ' => ', mediaDst)
				try:
					shutil.copy(mediaSrc, mediaDst)
				except Exception as e:
					AppLog.log( LOG_ERROR, str(e) )

				# Create media thumb
				filetype, fileext = self.getFileType( mediaDst )

				qEventIconUrl = "icons/doctype/default.png"

				if filetype == "photo" :

					tplSingleMediaFile = self.makePath ( os.path.dirname(tplFile), "media_photo.html")
					fpi = open( tplSingleMediaFile, 'r', encoding='utf8')
					mediaTemplateR = fpi.read()
					fpi.close()

					qEventIconUrl = "icons/doctype/photo.png"

					outDirMediaName, \
					matchFormat, \
					matchSourcemediaFile, \
					matchSourcemediaType, \
					matchSourcemediaWidth, \
					matchSourcemediaHeight, \
					matchFiles, \
					matchDirs, \
					matchTitle, \
					matchDes, \
					matchImageDir, \
					matchImageMaxHeight, \
					matchImageMaxWidth = self.initTargetFromTemplateMeta( mediaTemplateR, tplDir, outFile )

					from libwidget.mediastype.mediatypephoto import MediaTypePhoto
					mediaMetaObj = MediaTypePhoto( mediaSrc, mediaDst, tempDir = self.tempdir)
					thumbMediaFile = self.makePath( mediasDstDir, os.path.basename(str(media['url']))+"-thumb.jpeg")
					AppLog.log(LOG_INFO, "Create thumb:", thumbMediaFile)
					mediaMetaObj.getPixmapResised(int(matchImageMaxWidth),int(matchImageMaxHeight)).save(thumbMediaFile, "JPG")

					mediaTemplateR = self.cleanTemplateMetas( mediaTemplateR )
					mediaTemplate = mediaTemplateR

					exDate, exDateDay, exDateDayLib, exDateMonth, exDateMonthLib, exDateYear, exDateHour, exDateMin, exDeltaYears = self.renderDate( mediaMetaObj.exifDateOriginal )

					# Single media page
					mediaTemplate = mediaTemplate.replace("{{fileSize}}", self.rvalue(mediaMetaObj.fileSize) )
					mediaTemplate = mediaTemplate.replace("{{fileSizeH}}", self.rvalue(mediaMetaObj.fileSizeH) )
					mediaTemplate = mediaTemplate.replace("{{fileModiftimeH}}", self.rvalue(mediaMetaObj.fileModiftimeH) )
					mediaTemplate = mediaTemplate.replace("{{fileModiftimestamp}}", self.rvalue(mediaMetaObj.fileModiftimestamp) )
					mediaTemplate = mediaTemplate.replace("{{imgW}}", self.rvalue(mediaMetaObj.imgW) )
					mediaTemplate = mediaTemplate.replace("{{imgH}}", self.rvalue(mediaMetaObj.imgH) )

					mediaTemplate = mediaTemplate.replace("{{exifdateoriginal}}", self.rvalue(mediaMetaObj.exifDateOriginal) )
					mediaTemplate = mediaTemplate.replace("{{exifdate_day_week}}", exDateDayLib)
					mediaTemplate = mediaTemplate.replace("{{exifdate_day}}", exDateDay)
					mediaTemplate = mediaTemplate.replace("{{exifdate_month_lib}}",exDateMonthLib)
					mediaTemplate = mediaTemplate.replace("{{exifdate_month}}", exDateMonth)
					mediaTemplate = mediaTemplate.replace("{{exifdate_year}}", exDateYear)
					mediaTemplate = mediaTemplate.replace("{{exifdate_hour}}", exDateHour)
					mediaTemplate = mediaTemplate.replace("{{exifdate_minute}}",exDateMin)

					# Index page
					mediaIndexMatch = mediaIndexMatch.replace("{{fileSize}}", self.rvalue(mediaMetaObj.fileSize) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileSizeH}}", self.rvalue(mediaMetaObj.fileSizeH) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileModiftimeH}}", self.rvalue(mediaMetaObj.fileModiftimeH) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileModiftimestamp}}", self.rvalue(mediaMetaObj.fileModiftimestamp) )
					mediaIndexMatch = mediaIndexMatch.replace("{{imgW}}", self.rvalue(mediaMetaObj.imgW) )
					mediaIndexMatch = mediaIndexMatch.replace("{{imgH}}", self.rvalue(mediaMetaObj.imgH) )

					mediaIndexMatch = mediaIndexMatch.replace("{{exifdateoriginal}}", self.rvalue(mediaMetaObj.exifDateOriginal) )
					mediaIndexMatch = mediaIndexMatch.replace("{{exifdate_day_week}}", exDateDayLib)
					mediaIndexMatch = mediaIndexMatch.replace("{{exifdate_day}}", exDateDay)
					mediaIndexMatch = mediaIndexMatch.replace("{{exifdate_month_lib}}",exDateMonthLib)
					mediaIndexMatch = mediaIndexMatch.replace("{{exifdate_month}}", exDateMonth)
					mediaIndexMatch = mediaIndexMatch.replace("{{exifdate_year}}", exDateYear)
					mediaIndexMatch = mediaIndexMatch.replace("{{exifdate_hour}}", exDateHour)
					mediaIndexMatch = mediaIndexMatch.replace("{{exifdate_minute}}", exDateMin)

					mediaIndexMatch = mediaIndexMatch.replace("{{fileDuration}}", "_NONE_" )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileDurationTime}}", "_NONE_")
					mediaIndexMatch = mediaIndexMatch.replace("{{pageCount}}", "_NONE_" )

					mediaIndexMatch = mediaIndexMatch.replace("{{media_icon_type}}", "../../icons/doctype/photo.png")

				if filetype == "video" :

					tplSingleMediaFile = self.makePath ( os.path.dirname(tplFile), "media_video.html")
					fpi = open( tplSingleMediaFile, 'r', encoding='utf8')
					mediaTemplateR = fpi.read()
					fpi.close()

					qEventIconUrl = "icons/doctype/video.png"

					outDirMediaName, \
					matchFormat, \
					matchSourcemediaFile, \
					matchSourcemediaType, \
					matchSourcemediaWidth, \
					matchSourcemediaHeight, \
					matchFiles, \
					matchDirs, \
					matchTitle, \
					matchDes, \
					matchImageDir, \
					matchImageMaxHeight, \
					matchImageMaxWidth = self.initTargetFromTemplateMeta( mediaTemplateR, tplDir, outFile )

					from libwidget.mediastype.mediatypevideo import MediaTypeVideo
					mediaMetaObj = MediaTypeVideo( mediaSrc, mediaDst, tempDir = self.tempdir)
					thumbMediaFile = self.makePath( mediasDstDir, os.path.basename(str(media['url']))+"-thumb.jpeg")
					AppLog.log(LOG_INFO, "Create thumb:", thumbMediaFile)
					mediaMetaObj.getPixmapResised(int(matchImageMaxWidth),int(matchImageMaxHeight)).save(thumbMediaFile, "JPG")
					mediaTemplateR = self.cleanTemplateMetas( mediaTemplateR )
					mediaTemplate = mediaTemplateR

					mediaTemplate = mediaTemplate.replace("{{fileSize}}", self.rvalue(mediaMetaObj.fileSize) )
					mediaTemplate = mediaTemplate.replace("{{fileSizeH}}", self.rvalue(mediaMetaObj.fileSizeH) )
					mediaTemplate = mediaTemplate.replace("{{fileModiftimeH}}", self.rvalue(mediaMetaObj.fileModiftimeH) )
					mediaTemplate = mediaTemplate.replace("{{fileModiftimestamp}}", self.rvalue(mediaMetaObj.fileModiftimestamp) )
					mediaTemplate = mediaTemplate.replace("{{imgW}}", self.rvalue(mediaMetaObj.imgW) )
					mediaTemplate = mediaTemplate.replace("{{imgH}}", self.rvalue(mediaMetaObj.imgH) )

					mediaTemplate = mediaTemplate.replace("{{fileDuration}}", self.rvalue(mediaMetaObj.fileDuration) )
					mediaTemplate = mediaTemplate.replace("{{fileDurationTime}}", self.rvalue(mediaMetaObj.fileDurationTime) )

					# Index page
					mediaIndexMatch = mediaIndexMatch.replace("{{fileSize}}", self.rvalue(mediaMetaObj.fileSize) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileSizeH}}", self.rvalue(mediaMetaObj.fileSizeH) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileModiftimeH}}", self.rvalue(mediaMetaObj.fileModiftimeH) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileModiftimestamp}}", self.rvalue(mediaMetaObj.fileModiftimestamp) )
					mediaIndexMatch = mediaIndexMatch.replace("{{imgW}}", self.rvalue(mediaMetaObj.imgW) )
					mediaIndexMatch = mediaIndexMatch.replace("{{imgH}}", self.rvalue(mediaMetaObj.imgH) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileDuration}}", self.rvalue(mediaMetaObj.fileDuration) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileDurationTime}}", self.rvalue(mediaMetaObj.fileDurationTime) )
					mediaIndexMatch = mediaIndexMatch.replace("{{pageCount}}", "_NONE_" )
					mediaIndexMatch = mediaIndexMatch.replace("{{exifdateoriginal}}", "_NONE_" )

					mediaIndexMatch = mediaIndexMatch.replace("{{media_icon_type}}", "../../icons/doctype/video.png")


				if filetype == "audio" :

					tplSingleMediaFile = self.makePath ( os.path.dirname(tplFile), "media_audio.html")
					fpi = open( tplSingleMediaFile, 'r', encoding='utf8')
					mediaTemplateR = fpi.read()
					fpi.close()

					qEventIconUrl = "icons/doctype/audio.png"

					outDirMediaName, \
					matchFormat, \
					matchSourcemediaFile, \
					matchSourcemediaType, \
					matchSourcemediaWidth, \
					matchSourcemediaHeight, \
					matchFiles, \
					matchDirs, \
					matchTitle, \
					matchDes, \
					matchImageDir, \
					matchImageMaxHeight, \
					matchImageMaxWidth = self.initTargetFromTemplateMeta( mediaTemplateR, tplDir, outFile )

					from libwidget.mediastype.mediatypeaudio import MediaTypeAudio
					mediaMetaObj = MediaTypeAudio( mediaSrc, mediaDst, tempDir = self.tempdir)
					thumbMediaFile = self.makePath( mediasDstDir, os.path.basename(str(media['url']))+"-thumb.jpeg")
					AppLog.log(LOG_INFO,"  Create thumb:", thumbMediaFile)
					mediaMetaObj.getPixmapResised(int(matchImageMaxWidth),int(matchImageMaxHeight)).save(thumbMediaFile, "JPG")

					mediaTemplateR = self.cleanTemplateMetas( mediaTemplateR )
					mediaTemplate = mediaTemplateR

					mediaTemplate = mediaTemplate.replace("{{fileSize}}", self.rvalue(mediaMetaObj.fileSize) )
					mediaTemplate = mediaTemplate.replace("{{fileSizeH}}", self.rvalue(mediaMetaObj.fileSizeH) )
					mediaTemplate = mediaTemplate.replace("{{fileModiftimeH}}", self.rvalue(mediaMetaObj.fileModiftimeH) )
					mediaTemplate = mediaTemplate.replace("{{fileModiftimestamp}}", self.rvalue(mediaMetaObj.fileModiftimestamp) )

					mediaTemplate = mediaTemplate.replace("{{fileDuration}}", self.rvalue(mediaMetaObj.fileDuration) )
					mediaTemplate = mediaTemplate.replace("{{fileDurationTime}}", self.rvalue(mediaMetaObj.fileDurationTime) )

					# Index page
					mediaIndexMatch = mediaIndexMatch.replace("{{fileSize}}", self.rvalue(mediaMetaObj.fileSize) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileSizeH}}", self.rvalue(mediaMetaObj.fileSizeH) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileModiftimeH}}", self.rvalue(mediaMetaObj.fileModiftimeH) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileModiftimestamp}}", self.rvalue(mediaMetaObj.fileModiftimestamp) )
					mediaIndexMatch = mediaIndexMatch.replace("{{imgW}}", "_NONE_" )
					mediaIndexMatch = mediaIndexMatch.replace("{{imgH}}", "_NONE_" )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileDuration}}", self.rvalue(mediaMetaObj.fileDuration) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileDurationTime}}", self.rvalue(mediaMetaObj.fileDurationTime) )
					mediaIndexMatch = mediaIndexMatch.replace("{{pageCount}}", "_NONE_" )
					mediaIndexMatch = mediaIndexMatch.replace("{{exifdateoriginal}}", "_NONE_" )

					mediaIndexMatch = mediaIndexMatch.replace("{{media_icon_type}}", "../../icons/doctype/audio.png")


				if filetype == "pdf" :

					tplSingleMediaFile = self.makePath ( os.path.dirname(tplFile), "media_pdf.html")
					fpi = open( tplSingleMediaFile, 'r', encoding='utf8')
					mediaTemplateR = fpi.read()
					fpi.close()

					qEventIconUrl = "icons/doctype/document.png"

					outDirMediaName, \
					matchFormat, \
					matchSourcemediaFile, \
					matchSourcemediaType, \
					matchSourcemediaWidth, \
					matchSourcemediaHeight, \
					matchFiles, \
					matchDirs, \
					matchTitle, \
					matchDes, \
					matchImageDir, \
					matchImageMaxHeight, \
					matchImageMaxWidth = self.initTargetFromTemplateMeta( mediaTemplateR, tplDir, outFile )

					from libwidget.mediastype.mediatypepdf import MediaTypePdf
					mediaMetaObj = MediaTypePdf( mediaSrc, mediaDst, tempDir = self.tempdir)
					thumbMediaFile = self.makePath( mediasDstDir, os.path.basename(str(media['url']))+"-thumb.jpeg")
					AppLog.log(LOG_INFO,"  Create thumb:", thumbMediaFile)
					mediaMetaObj.getPixmapResised(int(matchImageMaxWidth),int(matchImageMaxHeight)).save(thumbMediaFile, "JPG")

					mediaTemplateR = self.cleanTemplateMetas( mediaTemplateR )
					mediaTemplate = mediaTemplateR

					mediaTemplate = mediaTemplate.replace("{{fileSize}}", self.rvalue(mediaMetaObj.fileSize) )
					mediaTemplate = mediaTemplate.replace("{{fileSizeH}}", self.rvalue(mediaMetaObj.fileSizeH) )
					mediaTemplate = mediaTemplate.replace("{{fileModiftimeH}}", self.rvalue(mediaMetaObj.fileModiftimeH) )
					mediaTemplate = mediaTemplate.replace("{{fileModiftimestamp}}", self.rvalue(mediaMetaObj.fileModiftimestamp) )

					mediaTemplate = mediaTemplate.replace("{{pageCount}}", self.rvalue(mediaMetaObj.pageCount) )

					# Index page
					mediaIndexMatch = mediaIndexMatch.replace("{{fileSize}}", self.rvalue(mediaMetaObj.fileSize) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileSizeH}}", self.rvalue(mediaMetaObj.fileSizeH) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileModiftimeH}}", self.rvalue(mediaMetaObj.fileModiftimeH) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileModiftimestamp}}", self.rvalue(mediaMetaObj.fileModiftimestamp) )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileDuration}}", "_NONE_" )
					mediaIndexMatch = mediaIndexMatch.replace("{{fileDurationTime}}", "_NONE_")
					mediaIndexMatch = mediaIndexMatch.replace("{{imgW}}", "_NONE_" )
					mediaIndexMatch = mediaIndexMatch.replace("{{imgH}}", "_NONE_" )
					mediaIndexMatch = mediaIndexMatch.replace("{{pageCount}}", self.rvalue(mediaMetaObj.pageCount) )
					mediaIndexMatch = mediaIndexMatch.replace("{{exifdateoriginal}}", "_NONE_" )

					mediaIndexMatch = mediaIndexMatch.replace("{{media_icon_type}}", "../../icons/doctype/document.png")


				# -----------------------------
				# INDEX page replacement

				mediaIndexMatch = mediaIndexMatch.replace("{{indiid}}", self.rvalue(indiId) )

				mediaIndexMatch = mediaIndexMatch.replace("{{eventid}}", self.rvalue(media['eventid']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{eventname}}", self.rvalue(media['eventname']) )

				mediaIndexMatch = mediaIndexMatch.replace("{{depotname}}", self.rvalue(media['depotname']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{personid}}", self.rvalue(media['personid']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{persondisplayname}}", self.rvalue(media['persondisplayname']) )

				mediaIndexMatch = mediaIndexMatch.replace("{{datemedia}}", self.rvalue(media['datemedia']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{datemediaH}}", self.rvalue(self.formatDateTimeHuman(media['datemedia'])) )

				rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( media['datemedia'], media['datemedia_format'], media['datemedia_accurate'] )

				mediaIndexMatch = mediaIndexMatch.replace("{{datemedia_day}}", self.rvalue(rDateDay) )
				mediaIndexMatch = mediaIndexMatch.replace("{{datemedia_day_week}}", self.rvalue(rDateDayLib) )
				mediaIndexMatch = mediaIndexMatch.replace("{{datemedia_month}}", self.rvalue(rDateMonth))
				mediaIndexMatch = mediaIndexMatch.replace("{{datemedia_year}}", self.rvalue(rDateYear) )
				mediaIndexMatch = mediaIndexMatch.replace("{{datemedia_month_lib}}", self.rvalue(rDateMonthLib))
				mediaIndexMatch = mediaIndexMatch.replace("{{datemedia_begin_hour}}", self.rvalue(rDateHour))

				if rDateHour != "_NONE_" and rDateHour is not None :
					rDateHour = rDateHour + "h"
				mediaIndexMatch = mediaIndexMatch.replace("{{datemedia_begin_minute}}",self.rvalue(rDateMin))
				mediaIndexMatch = mediaIndexMatch.replace("{{datemedia_years}}", self.rvalue(rDeltaYears))
				mediaIndexMatch = mediaIndexMatch.replace("{{datemedia_hourminutelib}}", self.rvalue(str(rDateHour) + ":" + str(rDateMin)))

				dateprec = self.date_event_accurate[self.renderValue(media['datemedia_format'])][ self.renderValue(media['datemedia_accurate']) ]
				if dateprec != '' :
					mediaIndexMatch = mediaIndexMatch.replace("{{mediadateprecision}}", dateprec)
				else :
					mediaIndexMatch = mediaIndexMatch.replace("{{mediadateprecision}}", "_NONE_")

				mediaIndexMatch = mediaIndexMatch.replace("{{place_name}}", self.rvalue(media['place_name']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{place_city}}", self.rvalue(media['place_city']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{place_zipcode}}", self.rvalue(media['place_zipcode']) )

				mediaIndexMatch = mediaIndexMatch.replace("{{place_gpslat}}", self.rvalue(media['place_gpslat']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{place_gpslon}}", self.rvalue(media['place_gpslon']) )

				mediaIndexMatch = mediaIndexMatch.replace("{{place_accurate}}", self.place_accurate[ self.renderValue( media['place_accurate'] ) ] )

				personIconUrl = ""

				self.printDebugData("about to replace localmapquery lat,lon :", str(media['place_gpslat']), " , ", str(media['place_gpslon']))

				if  media['place_gpslat'] is not None and media['place_gpslon'] is not None and media['place_gpslon'] != '' :

					if rDateYear  == "_NONE_" : rDateYear = ""
					if rDateDay  == "_NONE_" : rDateDay = ""
					if rDateMonthLib  == "_NONE_" : rDateMonthLib = ""
					if rDateHour  == "_NONE_" : rDateHour = ""
					if rDateMin  == "_NONE_" : rDateMin = ""
					if media['place_accurate']  is None :
						placeAccurateLib = media['place_accurate'] = ""
					else :
						placeAccurateLib = self.place_accurate[ media['place_accurate'] ]

					fieldSep = '£'
					localmapquery = str(media['id']) + fieldSep +\
									str(self.renderValue(indiId)) + fieldSep +\
									str(self.renderValue(media['displayname'])) + fieldSep +\
									str(self.renderValue(media['place_gpslat'])) + fieldSep +\
									str(self.renderValue(media['place_gpslon']))+ fieldSep +\
									str(self.renderValue(media['eventcode'])) + fieldSep +\
									str(self.renderValue(media['eventname'])) + fieldSep +\
									str(self.renderValue(rDateYear)) + fieldSep +\
									str(self.renderValue(rDateDay)) + " " + self.renderValue(str(rDateMonthLib)) + " " + str(self.renderValue(rDateYear)) + fieldSep +\
									str(self.renderValue(rDateHour)) + str(self.renderValue(rDateMin)) + fieldSep +\
									str(self.renderValue(dateprec)) + fieldSep +\
									str(self.renderValue(media['place_zipcode'])) + fieldSep +\
									str(self.renderValue(media['place_name'])) + fieldSep +\
									str(self.renderValue( placeAccurateLib ) ) + fieldSep +\
									"" + fieldSep +\
									"" + fieldSep +\
									"" + fieldSep +\
									"" + fieldSep +\
									str(self.renderValue(personIconUrl)) + fieldSep +\
									str(self.renderValue(qEventIconUrl))

					mediaIndexMatch = mediaIndexMatch.replace("{{localmapquery}}",quote(localmapquery))
					self.printDebugData("Replace localmapquery", localmapquery)

					try :
						zoom = int(media['place_zoom'])
					except Exception as e:
						zoom = 12
					mediaIndexMatch = mediaIndexMatch.replace("{{place_zoom}}",quote(str(zoom)))

				else :
					mediaIndexMatch = mediaIndexMatch.replace("{{localmapquery}}", "_NONE_")


				mediaIndexMatch = mediaIndexMatch.replace("{{description}}", self.rvalue(media['description']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{dimension}}", self.rvalue(media['dimension']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{origin}}", self.rvalue(media['origin']) )

				mediaIndexMatch = mediaIndexMatch.replace("{{mediaid}}", self.rvalue(media['id']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{medianame}}", self.rvalue(media['displayname']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{mediathumb}}", os.path.basename(self.rvalue(media['url']))+"-thumb.jpeg" )

				mediaIndexMatch = mediaIndexMatch.replace("{{mediafileurl}}", os.path.basename(self.rvalue(media['url'])) )

				if filetype == "photo" :
					# image annotée (png, jpg)
					mediaIndexMatch = mediaIndexMatch.replace("{{mediaindex}}", os.path.basename(self.rvalue(media['url']))+".html" )
					mediaIndexMatch = mediaIndexMatch.replace("{{mediaurl}}", "_NONE_" )
					mediaIndexMatch = mediaIndexMatch.replace("{{mediaurlindex}}", "_NONE_")

				else :
					mediaIndexMatch = mediaIndexMatch.replace("{{mediaindex}}", "_NONE_" )
					mediaIndexMatch = mediaIndexMatch.replace("{{mediaurl}}", os.path.basename(self.rvalue(media['url'])) )
					mediaIndexMatch = mediaIndexMatch.replace("{{mediaurlindex}}", os.path.basename(self.rvalue(media['url'])) )

				mediaIndexMatch = mediaIndexMatch.replace("{{mediathumbH}}", str(matchImageMaxHeight) )
				mediaIndexMatch = mediaIndexMatch.replace("{{mediathumbW}}", str(matchImageMaxWidth) )

				mediaIndexMatch = mediaIndexMatch.replace("{{documenttypename}}", self.rvalue(media['documenttypename']) )
				mediaIndexMatch = mediaIndexMatch.replace("{{documenttypeid}}", self.rvalue(media['documenttypeid']) )

				# Cumuler le résultat du remplacement dans l'index principal
				mediaTemplateIndex = mediaTemplateIndex.replace( mediaIndexMatchR,  mediaIndexMatch + mediaIndexMatchR  )

				# -----------------------------
				# SINGLE MEDIA PAGE replacement
				mediaTemplate = mediaTemplate.replace("{{indiid}}", self.rvalue(indiId) )

				mediaTemplate = mediaTemplate.replace("{{eventid}}", self.rvalue(media['eventid']) )
				mediaTemplate = mediaTemplate.replace("{{eventname}}", self.rvalue(media['eventname']) )

				mediaTemplate = mediaTemplate.replace("{{depotname}}", self.rvalue(media['depotname']) )
				mediaTemplate = mediaTemplate.replace("{{personid}}", self.rvalue(media['personid']) )
				mediaTemplate = mediaTemplate.replace("{{persondisplayname}}", self.rvalue(media['persondisplayname']) )

				mediaTemplate = mediaTemplate.replace("{{datemedia}}", self.rvalue(media['datemedia']) )
				mediaTemplate = mediaTemplate.replace("{{datemediaH}}", self.rvalue(self.formatDateTimeHuman(media['datemedia'])) )

				mediaTemplate = mediaTemplate.replace("{{description}}", self.rvalue(media['description']) )
				mediaTemplate = mediaTemplate.replace("{{dimension}}", self.rvalue(media['dimension']) )
				mediaTemplate = mediaTemplate.replace("{{origin}}", self.rvalue(media['origin']) )

				mediaTemplate = mediaTemplate.replace("{{mediaid}}", self.rvalue(media['id']) )
				mediaTemplate = mediaTemplate.replace("{{medianame}}", self.rvalue(media['displayname']) )
				mediaTemplate = mediaTemplate.replace("{{mediaurl}}", os.path.basename(self.rvalue(media['url'])) )
				mediaTemplate = mediaTemplate.replace("{{mediaurlindex}}", os.path.basename(self.rvalue(media['url']))+".html" )

				mediaTemplate = mediaTemplate.replace("{{mediathumb}}", os.path.basename(self.rvalue(media['url']))+"-thumb.jpeg" )

				mediaTemplate = mediaTemplate.replace("{{documenttypename}}", self.rvalue(media['documenttypename']) )
				mediaTemplate = mediaTemplate.replace("{{documenttypeid}}", self.rvalue(media['documenttypeid']) )

				# Page média : zone des annotations
				sourceMatchAnnotationR = self.getMatchByPattern(mediaTemplate, "@@(.*?)@@")
				sourceMatchAnnotation = sourceMatchAnnotationR

				# Page média : zone menu des annotations
				sourceMatchMenuAnnotationR = self.getMatchByPattern(mediaTemplate, "==(.*?)==")
				sourceMatchMenuAnnotation = sourceMatchMenuAnnotationR

				# Annotations -------------------------------------------------------------------
				localUrlelem = self.makePath( rootMedia, self.rvalue(media['url']))

				mediaUrlDstDir = self.makePath(outDirMediaName, os.path.basename(localUrlelem))

				try:
					img = Image.open(localUrlelem)

					image = Image.open(localUrlelem)
					imagefileWidth, imagefileHeight = image.size

					annoRatioX =  imagefileWidth / img.width
					annoRatioY = imagefileHeight / img.height

				except Exception as e:
					# Error opening image, or not an img
					pass

				if os.path.isfile(localUrlelem + ".shp"):

					# Loop annotations sections
					config = configparser.ConfigParser()
					config.read(localUrlelem + ".shp")
					templateFileName, templateFileExt = os.path.splitext(mediaUrlDstDir)
					templateFileName = os.path.basename(templateFileName)

					for each_section in config.sections():

						self.printDebugData("        annotation, titre = " + str(config[each_section]['title']).replace("'","\\x27"))

						# Dans la page photo
						sourceMatchAnnotation = sourceMatchAnnotation.replace("{{annoid}}", str(config[each_section]['id']))
						sourceMatchAnnotation = sourceMatchAnnotation.replace("{{annotitle}}", self.renderValue(config[each_section]['title'], config[each_section]['id']).replace("'","\\x27").replace("\n","<br>"))
						sourceMatchAnnotation = sourceMatchAnnotation.replace("{{annotext}}", str(config[each_section]['text']).replace("'","\\x27").replace("\n","<br>"))
						sourceMatchAnnotation = sourceMatchAnnotation.replace("{{annocolor}}", str(config[each_section]['itemcolor'][3:9]))

						transparency = int(config[each_section]['itemcolor'][1:3], 16)
						transparency = int ((transparency / 255) * 100)
						sourceMatchAnnotation = sourceMatchAnnotation.replace("{{annotransparency}}", str(transparency))

						# Dans la page photo (menu de toutes les annotation, colonne de gauche)
						sourceMatchMenuAnnotation = sourceMatchMenuAnnotation.replace("{{annoid}}", str(config[each_section]['id']))
						sourceMatchMenuAnnotation = sourceMatchMenuAnnotation.replace("{{annotitle}}", self.renderValue(config[each_section]['title'], config[each_section]['id']).replace("\n","<br>"))
						sourceMatchMenuAnnotation = sourceMatchMenuAnnotation.replace("{{annotext}}", str(config[each_section]['text']).replace("'","\\x27").replace("\n","<br>"))

						annotTopX = (img.width / 2) - ( float(config[each_section]['w']) / 2) + float(config[each_section]['dx'])
						annotTopY = (img.height / 2) - ( float(config[each_section]['h']) / 2) - float(config[each_section]['dy'])
						annotBottomX = annotTopX + float(config[each_section]['w'])
						annotBottomY = annotTopY + float(config[each_section]['h'])

						annotTopX = int(annotTopX * annoRatioX)
						annotTopY = int(annotTopY * annoRatioY)
						annotBottomX = int(annotBottomX * annoRatioX)
						annotBottomY = int(annotBottomY * annoRatioY)

						sourceMatchAnnotation = sourceMatchAnnotation.replace("{{topx}}", str(int(annotTopX)))
						sourceMatchAnnotation = sourceMatchAnnotation.replace("{{topy}}", str(int(annotTopY)))
						sourceMatchAnnotation = sourceMatchAnnotation.replace("{{bottomx}}", str(int(annotBottomX)))
						sourceMatchAnnotation = sourceMatchAnnotation.replace("{{bottomy}}", str(int(annotBottomY)))

						sourceMatchAnnotation = sourceMatchAnnotation + sourceMatchAnnotationR
						sourceMatchMenuAnnotation = sourceMatchMenuAnnotation + sourceMatchMenuAnnotationR

					mediaTemplate = mediaTemplate.replace(sourceMatchAnnotationR, sourceMatchAnnotation)
					mediaTemplate = mediaTemplate.replace(sourceMatchAnnotationR, "")

					mediaTemplate = mediaTemplate.replace(sourceMatchMenuAnnotationR, sourceMatchMenuAnnotation)
					mediaTemplate = mediaTemplate.replace(sourceMatchMenuAnnotationR, "")

				mediaTemplate = mediaTemplate.replace("@@", "")
				mediaTemplate = mediaTemplate.replace("==", "")
				mediaTemplate = mediaTemplate.replace("{{mediaurl}}", os.path.basename(mediaUrlDstDir))
				mediaTemplate = mediaTemplate.replace("{{mediawidth}}", str(imagefileWidth ))
				mediaTemplate = mediaTemplate.replace("{{mediaheight}}", str(imagefileHeight))
				mediaTemplate = mediaTemplate.replace("{{filename}}", os.path.basename(mediaUrlDstDir))

				mediaCount = 1
				mediaMax = 1
				mediaTemplate = mediaTemplate.replace("{{mediaCount}}", str(mediaCount) )
				mediaTemplate = mediaTemplate.replace("{{mediaMax}}", str(mediaMax) )

				mediaTemplate = self.databaseMetaInfoFiller( mediaTemplate )
				AppLog.log(LOG_INFO, "Insert menu : ", "annuaire")
				mediaTemplate = self.insertMenu( mediaTemplate, "annuaire" )
				mediaTemplate = self.cleanFinalData(mediaTemplate)

				singleMediaIndexFile = self.makePath( mediasDstDir, os.path.basename(str(media['url']))+".html")
				AppLog.log(LOG_INFO, "Create single media index :", singleMediaIndexFile)
				f = open( singleMediaIndexFile, "w", encoding='utf8')
				f.write(mediaTemplate)
				f.close()

			# INDEX : Create main medias index html

			# Global replacements
			mediaTemplateIndex = mediaTemplateIndex.replace("{{indiid}}", self.rvalue(indiId) )
			mediaTemplateIndex = mediaTemplateIndex.replace("{{personname}}", self.rvalue(media['persondisplayname']) )

			# 3. clean and insert menu
			mediaTemplateIndex = mediaTemplateIndex.replace(mediaIndexMatchR, "" )
			mediaTemplateIndex = mediaTemplateIndex.replace("°°M", "" )
			mediaTemplateIndex = mediaTemplateIndex.replace("°°", "" )

			AppLog.log(LOG_INFO, "Insert menu : ", "annuaire")
			mediaTemplateIndex = self.insertMenu( mediaTemplateIndex, "annuaire" )
			mediaTemplateIndex = self.cleanFinalData(mediaTemplateIndex)

			# 4. create final media index
			mediaHtmlIndex = self.makePath( mediasDstDir, "index.html")
			AppLog.log(LOG_INFO,"Create media index :", mediaHtmlIndex)
			f = open( mediaHtmlIndex, "w", encoding='utf8')
			f.write(mediaTemplateIndex)
			f.close()

		return mediaCount


	# ***************************************************************
	#	Fiche - rapport de type fiche: individus -> evennements -> sources
	# ***************************************************************
	def genRapportPersonFiche(self, indiId, dbFile, rootMedia, tplFile, outFile, cbPrivate, projSetting, svgurl) :

		AppLog.log( LOG_INFO , "rootMedia = " + str(rootMedia) )
		AppLog.log( LOG_INFO , "dbFile = " + str(dbFile) )

		db = Sqlitedb( dbFile )

		indicount = {}
		eventcount = {}
		sourcecount = {}
		doccount = {}

		indiNum = 0

		myRapport = Rapport( db, cbPrivate )

		tplDir = os.path.dirname(tplFile)
		fpi = open( tplFile, 'r', encoding='utf8')
		tplFileContentR = fpi.read()
		fpi.close()

		outDirMediaName, \
		matchFormat, \
		matchSourcemediaFile, \
		matchSourcemediaType, \
		matchSourcemediaWidth, \
		matchSourcemediaHeight, \
		matchFiles, \
		matchDirs, \
		matchTitle, \
		matchDes, \
		matchImageDir, \
		matchImageMaxHeight, \
		matchImageMaxWidth = self.initTargetFromTemplateMeta( tplFileContentR, tplDir, outFile )

		tplFileContentR = self.cleanTemplateMetas( tplFileContentR )
		tplFileContent = tplFileContentR

		# Bloc global pour un individu
		globalMatchR   = self.getMatchByPattern( tplFileContentR, "%%(.*?)%%" ) # Bloc global - Un individu (y compris sous bloc events)

		# Dans le Bloc global : entête (sans repetition)
		personMatchR   = self.getMatchByPattern( tplFileContentR, ";;I(.*?);;" )

		# Dans le Sous-Bloc : event, sources, doc. (avec repetition)
		blocPersonR    = self.getMatchByPattern( tplFileContentR, "µµ(.*?)µµ" ) # Sous-Bloc - repetition events + sources + docs d'UN individu
		eventMatchR    = self.getMatchByPattern( tplFileContentR, "--E(.*?)--" )
		sourceMatchR   = self.getMatchByPattern( tplFileContentR, "==S(.*?)==" )
		documentMatchR = self.getMatchByPattern( tplFileContentR, ",,D(.*?),," )

		indiList = myRapport.selectINDIbyId( indiId )
		if indiList is False :
			AppLog.log( LOG_ERROR, myRapport.lasterror )
		else :
			# Persons ------------------------------------------------------------------------
			indi = indiList[0]

			self.printDebugData( 'Fiche Person name : #', indi['id'], ' : ' , str(indi['name']))
			indicount[indi['id']] = indicount.get(indi['id'], 0) + 1
			indiNum +=1

			personMatch = personMatchR

			personMatch = personMatch.replace("{{indinum}}", self.rvalue(indiNum) )
			personMatch = personMatch.replace("{{id}}", self.rvalue(indi['id']) )
			personMatch = personMatch.replace("{{key}}", self.rvalue(indi['key']))
			personMatch = personMatch.replace("{{name}}", self.rvalue(indi['name']) )
			personMatch = personMatch.replace("{{nameformated}}", self.rvalue(indi['nameformated']) )
			personMatch = personMatch.replace("{{firtsnameothers}}", self.rvalue(indi['firtsnameothers']) )				
			personMatch = personMatch.replace("{{firstname}}", self.rvalue(indi['firstname']) )
			personMatch = personMatch.replace("{{lastname}}", self.rvalue(indi['lastname']) )
			personMatch = personMatch.replace("{{firstname}}", self.rvalue(indi['firstname']) )
			personMatch = personMatch.replace("{{lastname}}", self.rvalue(indi['lastname']) )
			personMatch = personMatch.replace("{{firstnameprefix}}", self.rvalue(indi['firstnameprefix']) )
			personMatch = personMatch.replace("{{lastnamesuffix}}", self.rvalue(indi['lastnamesuffix']) )
			personMatch = personMatch.replace("{{surname}}", self.rvalue(indi['surname']) )
			personMatch = personMatch.replace("{{sex}}", self.rvalue(indi['sex']) )
			personMatch = personMatch.replace('{{iconsex}}', self.renderIcoSex(indi['sex']))

			rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( indi['e.datebirth'], indi['e.datebirth_format'], indi['e.datebirth_accurate'] )
			personMatch = personMatch.replace("{{datebirth}}",rDate)
			personMatch = personMatch.replace("{{datebirth_day}}", rDateDay)
			personMatch = personMatch.replace("{{datebirth_day_week}}", rDateDayLib)
			personMatch = personMatch.replace("{{datebirth_month}}", rDateMonth)
			personMatch = personMatch.replace("{{datebirth_year}}", rDateYear)
			personMatch = personMatch.replace("{{datebirth_month_lib}}",rDateMonthLib)
			personMatch = personMatch.replace("{{datebirth_begin_hour}}", rDateHour)
			personMatch = personMatch.replace("{{datebirth_begin_minute}}",rDateMin)

			rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( indi['f.datedeath'], indi['e.datebirth'], indi['e.datebirth_format'], indi['e.datebirth_accurate'] )
			personMatch = personMatch.replace("{{datedeath}}",rDate)
			personMatch = personMatch.replace("{{datedeath_day}}", rDateDay)
			personMatch = personMatch.replace("{{datedeath_day_week}}", rDateDayLib)
			personMatch = personMatch.replace("{{datedeath_month}}", rDateMonth)
			personMatch = personMatch.replace("{{datedeath_year}}", rDateYear)
			personMatch = personMatch.replace("{{datedeath_month_lib}}",rDateMonthLib)
			personMatch = personMatch.replace("{{datedeath_begin_hour}}", rDateHour)
			personMatch = personMatch.replace("{{datedeath_begin_minute}}",rDateMin)
			personMatch = personMatch.replace("{{datedeath_years}}", rDeltaYears)

			if indi['sex'] == 'M' : defIconPath = "icons/ico-person-m.png"
			else : defIconPath = "icons/ico-person-w.png"
			personIconUrl = self.renderIcon( rootMedia, indi['icon'], outDirMediaName, matchImageDir, indi['id'], defIconPath, matchImageMaxWidth, matchImageMaxHeight)
			personMatch = personMatch.replace ('{{iconperson}}', personIconUrl)

			personMatch = personMatch.replace("{{description}}", self.rvalue(indi['description']) )

			indiParentList = myRapport.selectIndiParents( indi['id'] )
			if indiParentList :
				for indiParent in indiParentList :
					personMatch = personMatch.replace("{{indimother}}", self.rvalue(indiParent['person_2_id']) )
					personMatch = personMatch.replace("{{mother_displayname}}", self.rvalue(indiParent['mother_displayname']) )
					personMatch = personMatch.replace("{{indifather}}", self.rvalue(indiParent['person_3_id']) )
					personMatch = personMatch.replace("{{father_displayname}}", self.rvalue(indiParent['father_displayname']) )
			else :
				personMatch = personMatch.replace("{{indimother}}", "_NONE_")
				personMatch = personMatch.replace("{{indifather}}", "_NONE_")
				personMatch = personMatch.replace("{{mother_displayname}}", "_NONE_")
				personMatch = personMatch.replace("{{father_displayname}}", "_NONE_")

			# Lien vers cet index.html de la galerie utilisateur, depuis la fiche individu
			mediaCount = self.getMediaCountByPerson( indi['id'], dbFile, cbPrivate  )
			if mediaCount != 0 :
				linkToGaleryIndex =  "../medias/" + str(indi['id']) + "/index.html"
				personMatch = personMatch.replace("{{mediagalerieindex}}", linkToGaleryIndex)
			else :
				personMatch = personMatch.replace("{{mediagalerieindex}}", "NONE_")

			personMatch = personMatch.replace("{{mediaCount}}", str(mediaCount))

			personDataSections = ""

			# Events ------------------------------------------------------------------------
			eventList = myRapport.selectEventsByPerson( indi['id'] )
			eventMatch = eventMatchR
			if eventList :
				for event in eventList :
					self.printDebugData( '  EV name #' , event['id'], ' :' , str(event['name']))
					eventcount[event['id']] = eventcount.get(event['id'], 0) + 1

					eventMatch = eventMatch.replace("{{id}}", self.rvalue(event['id']) )
					eventMatch = eventMatch.replace("{{name}}", self.rvalue(event['name']) )
					eventMatch = eventMatch.replace("{{description}}", self.rvalue(event['description']) )

					rDate, rDateDay, rDateDayLib, rDateMonth, rDateMonthLib, rDateYear, rDateHour, rDateMin, rDeltaYears = self.renderDate( event['date_begin'], event['date_begin_format'], event['date_begin_accurate'] )
					eventMatch = eventMatch.replace("{{date_begin}}",rDate)
					eventMatch = eventMatch.replace("{{date_begin_day}}", rDateDay)
					eventMatch = eventMatch.replace("{{date_begin_dayweek}}", rDateDayLib)
					eventMatch = eventMatch.replace("{{date_begin_month}}", rDateMonth)
					eventMatch = eventMatch.replace("{{date_begin_year}}", rDateYear)
					eventMatch = eventMatch.replace("{{date_begin_month_lib}}",rDateMonthLib)
					eventMatch = eventMatch.replace("{{date_begin_hour}}", rDateHour)

					if rDateHour != "_NONE_" and rDateHour is not None :
						rDateHour = rDateHour + "h"
					eventMatch = eventMatch.replace("{{date_begin_minute}}",rDateMin)

					dateprec = self.date_event_accurate[self.renderValue(event['date_begin_format'])][ self.renderValue(event['date_begin_accurate']) ]
					if dateprec != '' :
						eventMatch = eventMatch.replace("{{eventdateprecision}}", dateprec)
					else :
						eventMatch = eventMatch.replace("{{eventdateprecision}}", "_NONE_")

					eventMatch = eventMatch.replace("{{et.code}}", self.rvalue(event['et.code']) )
					eventMatch = eventMatch.replace("{{et.name}}", self.rvalue(event['et.name']) )
					eventMatch = eventMatch.replace("{{p.name}}", self.rvalue(event['p.name']) )
					eventMatch = eventMatch.replace("{{eventplaceprecision}}", self.place_accurate[ self.renderValue(event['p.accurate']) ] )

					eventMatch = eventMatch.replace("{{zipcode}}", self.rvalue(event['p.zipcode']) )

					# MARR
					iconDet = '?'
					if event['et.category'] in ('FAM0', 'FAM1' ) :

						if event['i.id'] != indi['id'] :
							eventMatch = eventMatch.replace("{{i.id}}", self.rvalue(event['i.id']) )
							eventMatch = eventMatch.replace("{{i-1}}", "mariage-m" )
							eventMatch = eventMatch.replace("{{indi_displayname}}", self.rvalue(event['indi_name']) )

						# pas de lien pour l'indiv actuel
						if (cbPrivate is True or event['mother_private'] == 0) and event['person_2_id'] != '' and event['person_2_id'] is not None and event['person_2_id'] != indi['id'] :
							eventMatch = eventMatch.replace("{{person_2_id}}", self.rvalue(event['person_2_id']) )
							eventMatch = eventMatch.replace("{{mother_displayname}}", self.rvalue(event['mother_displayname']) )
							eventMatch = eventMatch.replace("{{i-2}}", "mariage-w" )

						if (cbPrivate is True or event['father_private'] == 0) and event['person_3_id'] != '' and event['person_3_id'] is not None:
							eventMatch = eventMatch.replace("{{person_3_id}}", self.rvalue(event['person_3_id']) )
							eventMatch = eventMatch.replace("{{father_displayname}}", self.rvalue(event['father_displayname']) )
							eventMatch = eventMatch.replace("{{i-3}}", "i-3" )

						eventMatch = eventMatch.replace("{{i.id}}", "_NONE_" )
						eventMatch = eventMatch.replace("{{person_2_id}}", "_NONE_" )
						eventMatch = eventMatch.replace("{{person_3_id}}", "_NONE_" )

					# BIRT
					if event['et.category'] == 'FAM2' :

						if event['i.id'] != indi['id'] :
							eventMatch = eventMatch.replace("{{i.id}}", self.rvalue(event['i.id']) )
							eventMatch = eventMatch.replace("{{indi_displayname}}", self.rvalue(event['indi_name']) )

							defIconName = "mariage-i"
							if event['i.sex'] == 'M' : defIconName = "mariage-m"
							if event['i.sex'] == 'F' : defIconName = "mariage-w"
							eventMatch = eventMatch.replace("{{i-1}}", defIconName )

					eventMatch = eventMatch.replace("{{i.id}}", "_NONE_"  )
					eventMatch = eventMatch.replace("{{person_2_id}}", "_NONE_" )
					eventMatch = eventMatch.replace("{{person_3_id}}", "_NONE_" )

					eventMatch = eventMatch.replace("{{p.gpslat}}", self.rvalue(event['p.gpslat']) )
					eventMatch = eventMatch.replace("{{p.gpslon}}", self.rvalue(event['p.gpslon']) )

					qEventIcon = self.renderIcon( rootMedia, event['icon'], outDirMediaName, matchImageDir, indi['id'], '_NONE_', matchImageMaxWidth, matchImageMaxHeight)
					eventMatch = eventMatch.replace ('{{iconevent}}', qEventIcon)

					# Url for "localmap" display
					#int(event['p.gpslat']) != 0 and int(event['p.gpslon']) != 0 :
					if  event['p.gpslat'] is not None and event['p.gpslon'] is not None and \
						event['p.gpslat'] != '' and event['p.gpslon'] != '' :

						fieldSep = '£'
						localmapquery = str(event['id']) + fieldSep +\
										str(indi['id']) + fieldSep +\
										str(event['name']) + fieldSep +\
										str(event['p.gpslat']) + fieldSep +\
										str(event['p.gpslon'])+ fieldSep +\
										str(event['et.code']) + fieldSep +\
										str(event['et.name']) + fieldSep +\
										str(rDateYear) + fieldSep +\
										str(rDateDay) + " " + str(rDateMonthLib) + " " + str(rDateYear) + fieldSep +\
										str(rDateHour) + str(rDateMin) + fieldSep +\
										str(dateprec) + fieldSep +\
										str(event['p.zipcode']) + fieldSep +\
										str(event['p.name']) + fieldSep +\
										str(self.place_accurate[ self.renderValue(event['p.accurate']) ]) + fieldSep +\
										str(event['indi_lastname']) + fieldSep +\
										str(event['indi_firstname']) + fieldSep +\
										str(self.renderValue(event['father_displayname'], '')) + fieldSep +\
										str(self.renderValue(event['mother_displayname'], '')) + fieldSep +\
										str(personIconUrl) + fieldSep +\
										str(qEventIcon)

						eventMatch = eventMatch.replace("{{localmapquery}}",quote(localmapquery))

						try :
							zoom = int(event['p.zoom'])
						except Exception as e:
							zoom = 12
						eventMatch = eventMatch.replace("{{mapzoom}}",quote(str(zoom)))

					else :
						
						eventMatch = eventMatch.replace("{{localmapquery}}", "_NONE_")

					personDataSections = personDataSections +  eventMatch
					eventMatch = eventMatchR

					# Sources ------------------------------------------------------------------------
					sourceList = myRapport.selectSourcesByEvent( event['id'] )
					if sourceList :
						for source in sourceList :

							sourceMatch = sourceMatchR

							self.printDebugData('    SOURCE # ', str(source['id']), ' name=', str(source['name']))
							sourcecount[source['id']] = sourcecount.get(source['id'], 0) + 1

							sourceMatch = sourceMatch.replace("{{id}}", self.rvalue(source['id']) )
							sourceMatch = sourceMatch.replace("{{name}}", self.rvalue(source['name']) )
							sourceMatch = sourceMatch.replace("{{page}}", self.rvalue(source['page']) )
							sourceMatch = sourceMatch.replace("{{citation}}", self.rvalue(source['citation']) )
							sourceMatch = sourceMatch.replace("{{traduction}}", self.rvalue(source['textdraduction']) )
							sourceMatch = sourceMatch.replace("{{description}}", self.rvalue(source['description']) )

							qualitylib = self.source_quality[ self.renderValue(source['quality']) ]
							sourceMatch = sourceMatch.replace("{{qualitylib}}", qualitylib )
							sourceMatch = sourceMatch.replace("{{quality}}", self.rvalue(source['quality']) )

							# Create source subdir
							sourceIndiSubdir = self.makePath(outDirMediaName,  "sources/" + str(indi['id']) )
							AppLog.log( LOG_INFO, "mkdir sourceIndiSubdir : ", sourceIndiSubdir)

							try:
								os.makedirs( sourceIndiSubdir, exist_ok=True ) # -p equivalent
							except Exception as e:
								AppLog.log( LOG_ERROR , str(e))

							# 1/3 Source type text
							sourceMatch = sourceMatch.replace("{{sourcetext}}", self.rvalue(source['sourcetext']) )

							# 2/3 Source type url+file
							if source['sourceurl'] is not None and source['sourceurl'] != "" :
								# copy media file
								try:
									mediaUrlSrcDir = self.makePath(rootMedia, source['sourceurl'])
									mediaUrlDstDir = self.makePath(sourceIndiSubdir, os.path.basename(source['sourceurl']))
									mediaUrlDstDirLink = self.makePath( "..", "sources/" + str(indi['id']) + '/' + str(os.path.basename(source['sourceurl'])))
									self.printDebugData("Copy media file : ",mediaUrlSrcDir ,"  to :",mediaUrlDstDir)
									try:
										shutil.copyfile( mediaUrlSrcDir , mediaUrlDstDir)
									except Exception as e:
										AppLog.log( LOG_ERROR, str(e) )


								except Exception as e:
									AppLog.log( LOG_ERROR , str(e))

								sourceMatch = sourceMatch.replace("{{sourceurl}}", self.rvalue(mediaUrlDstDirLink) )
								sourceMatch = sourceMatch.replace("{{sourceicon}}", self.getFileTypeIcon( source['sourceurl'] ) )
								sourceMatch = sourceMatch.replace("{{sourcetitle}}", os.path.basename( self.rvalue(source['sourceurl']) ) )

							# 3/3 Source type images local
							if source['localurl'] is not None and source['localurl'] != "" :

								localUrlList = str(source['localurl']).split(',')
								sourceMatchImageR = self.getMatchByPattern(tplFileContentR, "££u(.*?)££")
								sourceMatchImageFinal = ""

								# Lien vers le premier média de la source
								mediaUrl = "NONE"
								for localUrlelem in localUrlList :
									localUrlelem = self.makePath( rootMedia,localUrlelem)
									if localUrlelem != "None":
										mediaUrlBase, mediaUrlBaseExt = os.path.splitext(os.path.basename(str(localUrlList[0])))
										mediaUrl = os.path.basename(mediaUrlBase)
									break

								sourceIndiFirstUrl = "../sources/" + str(indi['id']) + '/' + str(mediaUrl + ".html")
								sourceMatch = sourceMatch.replace("{{localurl}}", sourceIndiFirstUrl )

								sourceMatch = sourceMatch.replace("{{localurlname}}", "Src.")

								sourceIconUrl = self.renderIcon( rootMedia, source['icon'], outDirMediaName, matchImageDir, indi['id'], "_NONE_", matchImageMaxWidth, matchImageMaxHeight)
								sourceMatch = sourceMatch.replace ('{{iconsource}}', sourceIconUrl)

								mediaCount = 0
								mediaMax = len(localUrlList)
								mediaIndex = 1

								# Pour chaque image de la source
								fileMatchAnnotation=""
								for localUrlelem in localUrlList :
									localUrlelem = self.makePath( rootMedia,localUrlelem)
									self.printDebugData('      photo = ', os.path.basename(localUrlelem))

									mediaUrlDstDir = self.makePath(outDirMediaName, os.path.basename(localUrlelem))

									# Page *principale* : zone des annotations
									fileMatchAnnotationR = self.getMatchByPattern(tplFileContent, "@@(.*?)@@")
									fileMatchAnnotation = fileMatchAnnotationR

									# Annotations -------------------------------------------------------------------
									if os.path.isfile(localUrlelem + ".shp"):

										AppLog.log( LOG_INFO, "Open shp : ", localUrlelem + ".shp")

										img = Image.open(localUrlelem)

										# Loop annotations sections
										config = configparser.ConfigParser()
										config.read(localUrlelem + ".shp")
										templateFileName, templateFileExt = os.path.splitext(mediaUrlDstDir)
										templateFileName = os.path.basename(templateFileName)

										for each_section in config.sections():

											self.printDebugData("        annotation, titre = " + str(config[each_section]['title']).replace("'","\\x27"))
											# Dans la page rapport
											fileMatchAnnotation = fileMatchAnnotation.replace("{{annoid}}", str(config[each_section]['id']))
											fileMatchAnnotation = fileMatchAnnotation.replace("{{annotitle}}", self.renderValue(config[each_section]['title'], config[each_section]['id']) )
											fileMatchAnnotation = fileMatchAnnotation.replace("{{annotext}}", str(config[each_section]['text']))

											fileMatchAnnotation = fileMatchAnnotation + fileMatchAnnotationR

									fileMatchAnnotation = fileMatchAnnotation.replace(fileMatchAnnotationR, "")
									#print("Append photo anno : ", fileMatchAnnotation)
									sourceMatch = sourceMatch + fileMatchAnnotation

							# Pas de source locale
							else :
								sourceMatch = sourceMatch.replace("{{localurl}}", "_NONE_")
								sourceMatch = sourceMatch.replace("{{localurlname}}", "_NONE_")
								sourceMatch = sourceMatch.replace ('{{iconsource}}', "_NONE_")

							personDataSections = personDataSections +  sourceMatch

							# End source type image local

							# Documents
							documentList = myRapport.selectDocumementsBySource( event['id'], source['id'] )
							documentMatch = documentMatchR
							if documentList :
								for document in documentList :
									self.printDebugData('       DOC #', document['id'], ' name = ' , str(document['name']))
									doccount[document['id']] = doccount.get(document['id'], 0) + 1

									documentMatch = documentMatch.replace("{{id}}", self.rvalue(document['id']) )
									documentMatch = documentMatch.replace("{{name}}", self.rvalue(document['name']) )
									documentMatch = documentMatch.replace("{{typename}}", self.rvalue(document['typename']) )
									documentMatch = documentMatch.replace("{{depot}}", self.rvalue(document['depot']) )
									documentMatch = documentMatch.replace("{{description}}", self.rvalue(document['description']) )
									documentMatch = documentMatch.replace("{{cote}}", self.rvalue(document['cote']) )
									documentMatch = documentMatch.replace("{{url}}", self.rvalue(document['url']) )

									sourceIconUrl = self.renderIcon( rootMedia, document['icon'], outDirMediaName, matchImageDir, indi['id'], "_NONE_", matchImageMaxWidth, matchImageMaxHeight)
									documentMatch = documentMatch.replace ('{{icon}}', sourceIconUrl)

									sourceIconUrl = self.renderIcon( rootMedia, document['t.icon'], outDirMediaName, matchImageDir, indi['id'], "icons/link-document.png", matchImageMaxWidth, matchImageMaxHeight)
									documentMatch = documentMatch.replace ('{{t.icon}}', sourceIconUrl)

									personDataSections = personDataSections +  documentMatch
									documentMatch = documentMatchR

								# print('documentMatchFinal')

						# print('sourceMatchFinal')

				# print('eventMatchFinal')

			# End each person
			self.printDebugData( "--------------- end person")

			# Hack pour résoudre le problèmes des url relatives (images) dans le fichier svg
			svgdata = open(svgurl, encoding='utf8').read()
			svgdata = svgdata.replace('<image xlink:href="annuaire' , '<image xlink:href="..')

			tplFileContent = tplFileContent.replace('{{svgdata}}' , svgdata )

			# Remplacement final pour un individu
			tplFileContent = tplFileContent.replace( personMatchR, personMatch )
			tplFileContent = tplFileContent.replace( blocPersonR, personDataSections )

			tplFileContent = tplFileContent.replace( ';;I', '')
			tplFileContent = tplFileContent.replace( ';;', '')
			tplFileContent = tplFileContent.replace( '%%', '')
			tplFileContent = tplFileContent.replace( 'µµ', '')

			# Compteurs d'entités
			tplFileContent = tplFileContent.replace('{{indicount}}', str(len(indicount)))
			tplFileContent = tplFileContent.replace('{{eventcount}}', str(len(eventcount)))
			tplFileContent = tplFileContent.replace('{{sourcecount}}', str(len(sourcecount)))
			tplFileContent = tplFileContent.replace('{{doccount}}', str(len(doccount)))

			tplFileContent = self.databaseMetaInfoFiller( tplFileContent )

			AppLog.log(LOG_INFO, "Insert menu : ", "annuaire")
			tplFileContent = self.insertMenu( tplFileContent, "annuaire" )

			# Fiche individuelle
			fichesDir = self.makePath(outDirMediaName, "fiches")
			try:
				os.mkdir( fichesDir )
			except Exception as e:
				pass

			outFicheFile = self.makePath ( fichesDir ,  str(indi['id'])+".html")
			AppLog.log( LOG_INFO , "Création fiche individu: " + str(outFicheFile) )

			tplFileContent = self.cleanFinalData( tplFileContent )
			fpFiche = open( outFicheFile,  "w", encoding='utf8')
			fpFiche.write(tplFileContent)
			fpFiche.close()


class ChooseTemplate(QDialog):

	cancel = False
	tplFileSelected = None

	cbPrivate = False
	cbOpenresult = False
	cbTraceDatas = False

	log = None

	def __init__(self):
		AppLog.log( LOG_INFO , "ChooseTemplate init" )

	def initDialog (self):

		QDialog.__init__(self)

		layout = QGridLayout()
		self.setLayout(layout)

		path = './templates/'
		list_of_files = []
		for path in Path(path).iterdir():
			metafile = os.path.join(path, "meta.ini")
			AppLog.log( LOG_INFO , "Template metafile = " + str(metafile) )
			list_of_files.append( metafile)

		self.listwidget = QListWidget()

		itemIndex = 0
		defaultItem = None
		for tplFile in list_of_files :

			try :
				fpi = open( tplFile, 'r', encoding='utf8')
				fileContentR = fpi.read()
				fpi.close()

				matchTitle = re.search("#TITRE=(.*)", fileContentR)
				if matchTitle is not None :
					matchTitle = matchTitle[1]
				else:
					matchTitle = ""

				AppLog.log( LOG_INFO , "Template titre = " + str(matchTitle) )

				matchDes = re.search("#DESCRIPTION=(.*)", fileContentR)
				if matchDes is not None :
					matchDes = matchDes[1]
				else:
					matchDes = ""

				matchMain = re.search("#MAIN=(.*)", fileContentR)
				if matchMain is not None :
					matchMain = matchMain[1]
				else:
					matchMain = ""

				matchPageAdd = re.search("#PAGEADD=(.*)", fileContentR)
				if matchPageAdd is not None :
					matchPageAdd = matchPageAdd[1]
				else:
					matchPageAdd = ""

				item = QListWidgetItem(matchTitle)

				item.setToolTip( matchDes + " ("+ tplFile +")" )

				item.setData(Qt.UserRole, os.path.join( os.path.dirname(tplFile), matchMain))
				item.setData(Qt.UserRole+1, matchPageAdd )

				self.listwidget.insertItem(itemIndex, item)

				# Select by default
				if os.path.join( os.path.dirname(tplFile), matchMain) == self.tplPathRoot :
					self.listwidget.setCurrentRow(itemIndex)
					self.listwidget.item(itemIndex).setSelected(True)

				itemIndex+=1

			except Exception as e:
				AppLog.log( LOG_ERROR , str(e) )


		self.listwidget.sortItems() # ascending by default

		for i in range(self.listwidget.count()):
			 AppLog.log( LOG_INFO, "Combo option : ", self.listwidget.item(i).data(Qt.UserRole), " ==>" , self.listwidget.item(i) )


		self.listwidget.clicked.connect(self.onListClicked)

		# Line 1
		layoutLine1 = QHBoxLayout()

		layoutLine1.addWidget(self.listwidget)
		self.listwidget.setMaximumWidth(300)
		self.listwidget.setMinimumWidth(300)

		self.tplLabelIcon = QLabel()
		self.tplLabelIcon.setPixmap(QPixmap(""))
		layoutLine1.addWidget( self.tplLabelIcon )

		layout.addLayout(layoutLine1,0,0 )

		# Line 2
		layoutLine2 = QVBoxLayout()

		self.cbPrivate = QCheckBox("Inclure les données privées", self)
		self.cbPrivate.stateChanged.connect(self.onCbPrivate)
		layoutLine2.addWidget(self.cbPrivate)

		self.cbOpenresult = QCheckBox("Ouvrir l'index après exportation", self)
		layoutLine2.addWidget(self.cbOpenresult)

		self.cbTraceDatas = QCheckBox("Trace des données", self)
		layoutLine2.addWidget(self.cbTraceDatas)

		layout.addLayout(layoutLine2, 1,0 )

		layoutLine3 = QHBoxLayout()
		self.labelPageAdd = QLabel("Page supplementaire")
		self.labelPageAdd.setToolTip( "Ajouter cette page Html statique au rapport" )
		self.lineEditPageAdd = QLineEdit("Page statique", self)
		self.lineEditPageAdd.textChanged.connect(self.onLineEditPageAdd)
		self.buttonPageAdd = QPushButton("...", self)
		self.buttonPageAdd.clicked.connect(self.onButtonPageAdd)
		self.lineEditPageAdd.setText( self.tplPageAdd )

		layoutLine3.addWidget(self.labelPageAdd)
		layoutLine3.addWidget(self.lineEditPageAdd)
		layoutLine3.addWidget(self.buttonPageAdd)

		layout.addLayout(layoutLine3, 2,0 )


		# OK and Cancel buttons
		buttons = QDialogButtonBox(
			QDialogButtonBox.Ok | QDialogButtonBox.Cancel,
			Qt.Horizontal, self)
		buttons.accepted.connect(self.onAccept)
		buttons.rejected.connect(self.onCancel)

		layout.addWidget(buttons)

		self.setWindowTitle("Selectionner le modèle")

		self.refreshIconPreview( self.tplFileSelected )

		self.listwidget.setFocus()
		self.resize(450, 400)

	def onButtonPageAdd( self ):

		selectFile , check = QFileDialog.getOpenFileName(None, "Sélectionner un fichier à inclure",
                                               self.tplPageAdd, "Html (*.html);;Tous (*)")
		if selectFile :
			self.tplPageAdd = selectFile
			self.lineEditPageAdd.setText( selectFile )

	def onLineEditPageAdd( self, newText ) :
		self.tplPageAdd = newText
		if os.access(newText, os.W_OK) is False :
			self.lineEditPageAdd.setToolTip("Le ficher base de données n'existe pas" )
			self.lineEditPageAdd.setStyleSheet("color: red;")
		else :
			self.lineEditPageAdd.setStyleSheet("color: black;")

	def refreshIconPreview( self, tplPath ) :
		pixmap = QPixmap( tplPath + "preview.png")
		pixmap_resized = pixmap.scaled(460, 340)
		self.tplLabelIcon.setPixmap(pixmap_resized)

	# Cross close
	def closeEvent ( self, event) :
		self.cancel = True
		self.reject()

	def onAccept( self) :
		self.accept()

	def onCancel( self) :
		self.cancel = True
		self.reject()

	def onListClicked(self, qmodelindex):
		item = self.listwidget.currentItem()
		self.tplFileSelected = item.data(Qt.UserRole)
		self.tplPageAdd = self.lineEditPageAdd.text()
		self.refreshIconPreview( self.tplFileSelected )

	def setCbPrivate( self, cbstate = False) :
		self.cbPrivate.setChecked( cbstate )

	def setCbOpenresult( self, cbstate = False) :
		self.cbOpenresult.setChecked( cbstate )

	def setCbTraceDatas( self, cbstate = False) :
		self.cbTraceDatas.setChecked( cbstate )

	def setDefaultTemplate (self, tplPathRoot, tplPageAdd  ) :
		self.tplPathRoot = tplPathRoot
		self.tplFileSelected = tplPathRoot
		self.tplPageAdd = tplPageAdd

	def onCbPrivate(self, state):

		if state == Qt.Checked:
			pass
		else:
			pass


class Rapportexport( QWidget ) :

	monthread = None
	prog = None

	db = None
	parent = None
	log = None

	# Paramètres utilisateur pour l'exportation
	dirExport = None
	privateExport = False
	tplPathRoot = None
	openResult = False
	traceDatas = False
	tplPageAdd = None

	def __init__(self , parent, db ):
		super().__init__()
		self.db = db
		self.parent = parent
		AppLog.log( LOG_INFO , "Rapportexport init" )

	# Point d'entrée depuis le menu principal
	def main( self , selectParams = True ) :

		projSetting = Project()
		projSetting.load((AppSetting.get('app/lastProject', '')))

		# xxx
		'''
		dirExport = Metadatas( self.db ).selectMetaByCode("EXPORTDIR")
		if dirExport :
			self.dirExport = Metadatas( self.db ).selectMetaByCode("EXPORTDIR")['value']
		
		if os.path.isdir (self.dirExport) is False : self.dirExport = None
		'''
		self.dirExport = projSetting.rootExportPath
		if os.path.isdir (self.dirExport) is False : self.dirExport = None
		

		privateExport = Metadatas( self.db ).selectMetaByCode("EXPORTPRIVATE")
		if privateExport :
			if Metadatas( self.db ).selectMetaByCode("EXPORTPRIVATE")['value'] == "1" :
				self.privateExport = True
			else :
				self.privateExport = False

		tplPathRoot = Metadatas( self.db ).selectMetaByCode("EXPORTPATHROOT")
		if tplPathRoot :
			self.tplPathRoot = Metadatas( self.db ).selectMetaByCode("EXPORTPATHROOT")['value']

		openResult = Metadatas( self.db ).selectMetaByCode("EXPORTOPENRESULT")
		if openResult :
			if Metadatas( self.db ).selectMetaByCode("EXPORTOPENRESULT")['value']  == "1" :
				self.openResult = True
			else :
				self.openResult = False

		traceDatas = Metadatas( self.db ).selectMetaByCode("EXPORTTRACEDATAS")
		if traceDatas :
			if Metadatas( self.db ).selectMetaByCode("EXPORTTRACEDATAS")['value']  == "1" :
				self.traceDatas = True
			else :
				self.traceDatas = False
				
		self.tplPageAdd = projSetting.pageAdd

		AppLog.log( LOG_INFO , "Option selected = "+ str(self.tplPathRoot) )
		AppLog.log( LOG_INFO , "Option pageAdd = "+ str(self.tplPageAdd) )
		AppLog.log( LOG_INFO , "Option cbPrivate = "+ str(self.privateExport) )
		AppLog.log( LOG_INFO , "Option cbOpenresult = "+ str(self.openResult) )
		AppLog.log( LOG_INFO , "Option cbTraceDatas = "+ str(self.traceDatas) )
		AppLog.log( LOG_INFO , "Option dirExport = "+ str(self.dirExport) )

		if selectParams is True or self.dirExport is None or self.privateExport is None or self.tplPathRoot is None :

			templateChoiceDlg = ChooseTemplate()

			templateChoiceDlg.setDefaultTemplate( str(self.tplPathRoot), str(self.tplPageAdd) )

			templateChoiceDlg.initDialog()

			templateChoiceDlg.setCbPrivate( self.privateExport )
			templateChoiceDlg.setCbOpenresult( self.openResult )
			templateChoiceDlg.setCbTraceDatas( self.traceDatas )
			templateChoiceDlg.exec()
			if templateChoiceDlg.cancel : return

			AppLog.log( LOG_INFO , "Option selected = "+ str(templateChoiceDlg.tplFileSelected ) )
			AppLog.log( LOG_INFO , "Option tplPageAdd = "+ str(templateChoiceDlg.tplPageAdd ) )
			AppLog.log( LOG_INFO , "Option cbPrivate = "+ str(templateChoiceDlg.cbPrivate.isChecked()) )
			AppLog.log( LOG_INFO , "Option cbOpenresult = "+ str(templateChoiceDlg.cbOpenresult.isChecked()) )
			AppLog.log( LOG_INFO , "Option cbTraceDatas = "+ str(templateChoiceDlg.cbTraceDatas.isChecked()) )
			AppLog.log( LOG_INFO , "Option dirExport = "+ str(self.dirExport) )

			if self.dirExport is None or not os.path.isdir( self.dirExport ):
				defaultDir = ''
				#defaultDir = self.makePath( defaultDir, os.path.basename(templateChoiceDlg.tplFileSelected))
			else :
				defaultDir = self.dirExport

			exportDir = str(QFileDialog.getExistingDirectory(self,
															"Choisir le dossier d'exportation",
															defaultDir,
															QFileDialog.ShowDirsOnly)
															)

			if not exportDir:
				AppLog.log( LOG_INFO , "Export annulé par l'utilisateur")
				return

			self.dirExport = exportDir
			self.privateExport = templateChoiceDlg.cbPrivate.isChecked()
			self.openResult = templateChoiceDlg.cbOpenresult.isChecked()
			self.traceDatas = templateChoiceDlg.cbTraceDatas.isChecked()
			self.tplPathRoot = templateChoiceDlg.tplFileSelected
			self.tplPageAdd = templateChoiceDlg.tplPageAdd

		# Mise à jour du dernier dossier d'export
		AppLog.log( LOG_INFO , "Enregistrement des paramètres :")
		AppLog.log( LOG_INFO , "Option selected = "+ str(self.tplPathRoot) )
		# xxx
		#AppLog.log( LOG_INFO , "Option tplPageAdd = "+ str(self.tplPageAdd) )
		
		AppLog.log( LOG_INFO , "Option cbPrivate = "+ str(self.privateExport) )
		AppLog.log( LOG_INFO , "Option cbOpenresult = "+ str(self.openResult) )
		AppLog.log( LOG_INFO , "Option cbTraceDatas = "+ str(self.traceDatas) )
		# xxx
		#AppLog.log( LOG_INFO , "Option dirExport = "+ str(self.dirExport) )

		# xxx
		# Metadatas( self.db ).updateMetaValueByCode("EXPORTDIR", self.dirExport, "Dossier destination de l'exportation")
		
		Metadatas( self.db ).updateMetaValueByCode("EXPORTPRIVATE", self.privateExport, "Exportation des données privées")
		Metadatas( self.db ).updateMetaValueByCode("EXPORTOPENRESULT", self.openResult, "Ouvrir l'index après l'exportation")
		Metadatas( self.db ).updateMetaValueByCode("EXPORTTRACEDATAS", self.traceDatas, "Tracer les données dans le log")		
		Metadatas( self.db ).updateMetaValueByCode("EXPORTPATHROOT", self.tplPathRoot, "Modèle utilisé pour l'exportation")		
		
		# xxx
		#Metadatas( self.db ).updateMetaValueByCode("EXPORTPAGEADD", self.tplPageAdd, "Page supplementaire à inclure dans le rapport")

		# Exporter et ouvrir le fichier index obtenu
		self.doExport( )

	def doExport( self ) :

		AppLog.log( LOG_INFO , "Début d'exportation ...")

		projSetting = Project()
		projSetting.load((AppSetting.get('app/lastProject', '')))
		self.db = Sqlitedb( projSetting.dbFile )

		# Compter le nombre de sous-dossiers du template selectionné
		maxTplSubdir = 0
		import os
		for dirSrc in Path(self.tplPathRoot).iterdir():
			if os.path.isdir(dirSrc) :
				maxTplSubdir = maxTplSubdir + 1

		# Pourcentage d'avancement à chaque sous-dossier traité
		AppLog.log( LOG_INFO , "Nombre de sous-dossiers du template : " + str(maxTplSubdir))
		progressInc = 100 / maxTplSubdir

		# Execution de l'export
		if self.monthread == None or not self.monthread.isRunning():
			# => démarrage de l'action dans le thread

			# création du QProgressDialog
			self.prog = QtWidgets.QProgressDialog("En cours...", "Annuler", 0, 100, self.parent)
			self.prog.setWindowTitle("Création du rapport")
			# branchement du bouton "annuler" à la méthode 'stopaction'
			self.prog.canceled.connect(self.stopaction)
			# affichage de la fenêtre de progression
			self.prog.show()

			# lancement du thread avec son opération
			self.monthread = GenRapportThread( progressInc, self.tplPathRoot, self.dirExport, self.tplPageAdd, projSetting, self.privateExport, self.traceDatas, self.tplPathRoot)

			# se préparer à recevoir le % d'avancement
			self.monthread.progduthread.connect(self.progression)
			# se préparer à recevoir le signal de fin du thread
			self.monthread.finduthread.connect(self.finaction)

			# démarrer le thread
			self.monthread.start()
			# surtout pas de "self.monthread.join()" ici!!!

			# changement du texte du bouton
			#self.bouton.setText("Stop")

		else:
			# => arrêt du thread avant la fin, demandé par le bouton "Stop"
			self.stopaction()

	#========================================================================
	def progression(self, pourc):
		"""mise à jour de la barre de progression
		"""

		self.prog.setValue(pourc * 100)
		if pourc >= 100:
			self.prog.reset()

	#========================================================================
	def stopaction(self):
		"""méthode lancée par le bouton "annuler" de la fenêtre de progression
		"""
		AppLog.log( LOG_INFO , "Annulation : fin du thread." )

		# débranchement des liens
		self.monthread.progduthread.disconnect()
		self.monthread.finduthread.disconnect()
		# arrêt brutal du thread
		self.monthread.terminate()
		# arrêt de la fenêtre de progression
		self.prog.reset()
		# retour au texte initial du bouton
		#self.bouton.setText("Début")

	#========================================================================
	def finaction(self, message, errorMessage):
		"""méthode lancée à la fin normale du thread
		"""
		AppLog.log( LOG_INFO , "Fin normale du thread." )

		# arrêt de la fenêtre de progression
		self.prog.reset()

		# retour au texte initial du bouton
		#self.bouton.setText("Début")

		from PyQt5.QtWidgets import QMessageBox
		msg = QMessageBox()

		if errorMessage == "" :
			# affichage du message de fin
			'''
			QtWidgets.QMessageBox.information(self,
				"Information",
				msg )
			'''
			msg.setIcon(QMessageBox.Information)
			msg.setText(message)
			#msg.setInformativeText('More information')
			msg.setWindowTitle("Fin du traitement")
		else :
			msg.setIcon(QMessageBox.Critical)
			msg.setText(errorMessage)
			#msg.setInformativeText('More information')
			msg.setWindowTitle("Erreur")

		msg.exec_()

		# Ouvrir le fichier index obtenu
		if self.openResult is True and errorMessage == "" :
			import os
			indexExported = os.path.join( self.dirExport, "index.html")
			AppLog.log( LOG_INFO , "Ouverture du fichier exporté : " + str(indexExported))
			import subprocess, os, platform
			if platform.system() == 'Darwin':       # macOS
				subprocess.call(('open', indexExported ))
			elif platform.system() == 'Windows':    # Windows
				os.startfile(indexExported )
			else:                                   # linux variants
				AppLog.log( LOG_INFO , "Ouverture...")
				subprocess.call(('xdg-open', indexExported ))


class GenRapportThread( QtCore.QThread ) :

	# création d'un nouveau signal pour info du % d'avancement
	progduthread = QtCore.pyqtSignal(float)
	# création d'un nouveau signal pour info de la fin du thread
	finduthread = QtCore.pyqtSignal(str, str)

	log = None

	def __init__(self , progressInc, pathRoot, dirExport, tplPageAdd, projSetting, privateExport, traceDatas, tplPathSelect ):
		super().__init__()

		#self.log = LogManager( AppSetting.get('app/logfile', "defaultlog.txt") )
		AppLog.log( LOG_INFO , "Rapportexport init" )

		self.pathRoot = pathRoot
		self.dirExport = dirExport
		self.projSetting = projSetting
		self.privateExport = privateExport
		self.traceDatas = traceDatas
		self.tplPathSelect = tplPathSelect
		self.progressInc = progressInc
		self.tplPageAdd = tplPageAdd

		self.errorMessage = ""

	def run( self ) :

		AppLog.log( LOG_INFO , "QThread run")

		# Hack : la génération, trop rapide, de "petits" modèles ne permet pas la fermeture de la fenêtre pop-up ...
		from time import sleep
		sleep(1)

		# Trier, afin de traiter "index" en dernier
		listTplDir = []
		tplDirIndex= ''
		for dirSrc in Path(self.pathRoot).iterdir():
			if os.path.isdir(dirSrc) :

				if os.path.basename(dirSrc) != 'index' :
					listTplDir.append(dirSrc)
				else :
					tplDirIndex = dirSrc


		self.progduthread.emit(1/100)

		# Ajouter l' "index" en fin de liste
		listTplDir.append(tplDirIndex)

		i = 0

		tplManagerObj = templateManager()
		tplManagerObj.databaseMetaInfoInit( self.projSetting, self.privateExport )

		self.progduthread.emit(2/100)
		AppLog.log( LOG_INFO , "Progress inc = " + str(i) )

		AppLog.log( LOG_INFO , "rmTree : " + str(self.dirExport) )
		shutil.rmtree( str(self.dirExport))

		tplManagerObj.pageAddUrl = self.tplPageAdd
		AppLog.log( LOG_INFO , "Page Add : " + str(tplManagerObj.pageAddUrl) )

		for dirSrc in listTplDir:

			if os.path.isdir(dirSrc) :

				AppLog.log( LOG_INFO , "Template dir = " + str(dirSrc) )
				metafile = os.path.join(dirSrc, "meta.ini")

				fpi = open( metafile, 'r', encoding='utf8')
				fileContentR = fpi.read()
				fpi.close()

				matchTitle = re.search("#TITRE=(.*)", fileContentR)
				if matchTitle is not None :
					matchTitle = matchTitle[1]
				else:
					matchTitle = ""

				matchDes = re.search("#DESCRIPTION=(.*)", fileContentR)
				if matchDes is not None :
					matchDes = matchDes[1]
				else:
					matchDes = ""

				matchMain = re.search("#MAIN=(.*)", fileContentR)
				if matchMain is not None :
					matchMain = matchMain[1]
				else:
					matchMain = ""

				#src =  templates/arbre/arbre.html
				#dst =  /home/.../Bureau/OUTTMP/arbre.htm
				srcMainFile = os.path.join(dirSrc, matchMain)

				AppLog.log( LOG_INFO , "srcMainFile = " + str(srcMainFile) )

				outMainFile = os.path.join(self.dirExport, matchMain)

				AppLog.log( LOG_INFO , "outMainFile = " + str(outMainFile) )

				try:
					tplManagerObj.genRapport(
												self.projSetting.dbFile,
												self.projSetting.rootMedia,
												self.tplPathSelect, # path to template selected (template/<name>)
												srcMainFile ,
												outMainFile,
												self.privateExport,
												self.traceDatas,
												self.projSetting
											)

				except Exception as e:
					self.errorMessage = str(e)
					AppLog.log( LOG_ERROR, str( f"{type(e).__name__} at line {e.__traceback__.tb_lineno} of {__file__}: {e}" ) )
					import traceback
					traceback.print_exc()
					i = 100 - self.progressInc

				i = i + self.progressInc
				self.progduthread.emit(i/100)
				AppLog.log( LOG_INFO , "Progress inc = " + str(i) )


		tplManagerObj.genRapportPageAdd( self.dirExport )

		# Signale que le thread est terminé
		self.finduthread.emit("Création du rapport terminé :\n" + str(self.dirExport), self.errorMessage)



